/* Generated from core.scm by the CHICKEN compiler
   http://www.call-cc.org
   Version 5.3.0rc4 ((HEAD detached at 5.3.0rc4)) (rev b6cbb1ba)
   linux-unix-gnu-x86-64 [ 64bit dload ptables ]
   command line: core.scm -optimize-level 2 -include-path . -include-path ./ -inline -ignore-repository -feature chicken-bootstrap -no-warnings -specialize -consult-types-file ./types.db -no-lambda-info -no-trace -emit-import-library chicken.compiler.core -output-file core.c
   unit: compiler
   uses: expand data-structures pathname internal extras file eval support scrutinizer library
*/
#include "chicken.h"

static C_word code_817() { C_clear_trace_buffer();
; return C_SCHEME_UNDEFINED; }


static C_PTABLE_ENTRY *create_ptable(void);
C_noret_decl(C_expand_toplevel)
C_externimport void C_ccall C_expand_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_data_2dstructures_toplevel)
C_externimport void C_ccall C_data_2dstructures_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_pathname_toplevel)
C_externimport void C_ccall C_pathname_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_internal_toplevel)
C_externimport void C_ccall C_internal_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_extras_toplevel)
C_externimport void C_ccall C_extras_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_file_toplevel)
C_externimport void C_ccall C_file_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_eval_toplevel)
C_externimport void C_ccall C_eval_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_support_toplevel)
C_externimport void C_ccall C_support_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_scrutinizer_toplevel)
C_externimport void C_ccall C_scrutinizer_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(C_library_toplevel)
C_externimport void C_ccall C_library_toplevel(C_word c,C_word *av) C_noret;

static C_TLS C_word lf[712];
static double C_possibly_force_alignment;


C_noret_decl(f25485)
static void C_ccall f25485(C_word c,C_word *av) C_noret;
C_noret_decl(f_10014)
static void C_ccall f_10014(C_word c,C_word *av) C_noret;
C_noret_decl(f_10018)
static void C_ccall f_10018(C_word c,C_word *av) C_noret;
C_noret_decl(f_10033)
static void C_ccall f_10033(C_word c,C_word *av) C_noret;
C_noret_decl(f_10036)
static void C_ccall f_10036(C_word c,C_word *av) C_noret;
C_noret_decl(f_10039)
static void C_ccall f_10039(C_word c,C_word *av) C_noret;
C_noret_decl(f_10043)
static C_word C_fcall f_10043(C_word t0,C_word t1);
C_noret_decl(f_10062)
static void C_ccall f_10062(C_word c,C_word *av) C_noret;
C_noret_decl(f_10066)
static void C_ccall f_10066(C_word c,C_word *av) C_noret;
C_noret_decl(f_10070)
static void C_ccall f_10070(C_word c,C_word *av) C_noret;
C_noret_decl(f_10073)
static void C_ccall f_10073(C_word c,C_word *av) C_noret;
C_noret_decl(f_10079)
static void C_ccall f_10079(C_word c,C_word *av) C_noret;
C_noret_decl(f_10087)
static void C_ccall f_10087(C_word c,C_word *av) C_noret;
C_noret_decl(f_10093)
static void C_ccall f_10093(C_word c,C_word *av) C_noret;
C_noret_decl(f_10097)
static void C_ccall f_10097(C_word c,C_word *av) C_noret;
C_noret_decl(f_10100)
static void C_ccall f_10100(C_word c,C_word *av) C_noret;
C_noret_decl(f_10103)
static C_word C_fcall f_10103(C_word t0,C_word t1);
C_noret_decl(f_10128)
static void C_ccall f_10128(C_word c,C_word *av) C_noret;
C_noret_decl(f_10130)
static void C_fcall f_10130(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10171)
static void C_ccall f_10171(C_word c,C_word *av) C_noret;
C_noret_decl(f_10174)
static void C_ccall f_10174(C_word c,C_word *av) C_noret;
C_noret_decl(f_10177)
static void C_fcall f_10177(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10180)
static void C_ccall f_10180(C_word c,C_word *av) C_noret;
C_noret_decl(f_10183)
static void C_ccall f_10183(C_word c,C_word *av) C_noret;
C_noret_decl(f_10186)
static void C_ccall f_10186(C_word c,C_word *av) C_noret;
C_noret_decl(f_10208)
static void C_ccall f_10208(C_word c,C_word *av) C_noret;
C_noret_decl(f_10212)
static void C_ccall f_10212(C_word c,C_word *av) C_noret;
C_noret_decl(f_10216)
static void C_ccall f_10216(C_word c,C_word *av) C_noret;
C_noret_decl(f_10230)
static void C_ccall f_10230(C_word c,C_word *av) C_noret;
C_noret_decl(f_10261)
static void C_ccall f_10261(C_word c,C_word *av) C_noret;
C_noret_decl(f_10283)
static void C_ccall f_10283(C_word c,C_word *av) C_noret;
C_noret_decl(f_10286)
static void C_fcall f_10286(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10289)
static void C_ccall f_10289(C_word c,C_word *av) C_noret;
C_noret_decl(f_10337)
static void C_ccall f_10337(C_word c,C_word *av) C_noret;
C_noret_decl(f_10341)
static void C_ccall f_10341(C_word c,C_word *av) C_noret;
C_noret_decl(f_10345)
static void C_ccall f_10345(C_word c,C_word *av) C_noret;
C_noret_decl(f_10353)
static void C_ccall f_10353(C_word c,C_word *av) C_noret;
C_noret_decl(f_10363)
static void C_fcall f_10363(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10367)
static void C_ccall f_10367(C_word c,C_word *av) C_noret;
C_noret_decl(f_10370)
static void C_ccall f_10370(C_word c,C_word *av) C_noret;
C_noret_decl(f_10377)
static void C_fcall f_10377(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10381)
static void C_ccall f_10381(C_word c,C_word *av) C_noret;
C_noret_decl(f_10391)
static void C_ccall f_10391(C_word c,C_word *av) C_noret;
C_noret_decl(f_10395)
static void C_ccall f_10395(C_word c,C_word *av) C_noret;
C_noret_decl(f_10399)
static void C_ccall f_10399(C_word c,C_word *av) C_noret;
C_noret_decl(f_10420)
static void C_ccall f_10420(C_word c,C_word *av) C_noret;
C_noret_decl(f_10423)
static void C_ccall f_10423(C_word c,C_word *av) C_noret;
C_noret_decl(f_10431)
static void C_ccall f_10431(C_word c,C_word *av) C_noret;
C_noret_decl(f_10453)
static void C_fcall f_10453(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10463)
static void C_ccall f_10463(C_word c,C_word *av) C_noret;
C_noret_decl(f_10476)
static void C_ccall f_10476(C_word c,C_word *av) C_noret;
C_noret_decl(f_10484)
static void C_ccall f_10484(C_word c,C_word *av) C_noret;
C_noret_decl(f_10490)
static void C_ccall f_10490(C_word c,C_word *av) C_noret;
C_noret_decl(f_10512)
static void C_fcall f_10512(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10522)
static void C_ccall f_10522(C_word c,C_word *av) C_noret;
C_noret_decl(f_10535)
static void C_fcall f_10535(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10560)
static void C_ccall f_10560(C_word c,C_word *av) C_noret;
C_noret_decl(f_10578)
static void C_ccall f_10578(C_word c,C_word *av) C_noret;
C_noret_decl(f_10583)
static void C_ccall f_10583(C_word c,C_word *av) C_noret;
C_noret_decl(f_10597)
static void C_ccall f_10597(C_word c,C_word *av) C_noret;
C_noret_decl(f_10605)
static void C_ccall f_10605(C_word c,C_word *av) C_noret;
C_noret_decl(f_10615)
static void C_ccall f_10615(C_word c,C_word *av) C_noret;
C_noret_decl(f_10628)
static void C_ccall f_10628(C_word c,C_word *av) C_noret;
C_noret_decl(f_10649)
static void C_ccall f_10649(C_word c,C_word *av) C_noret;
C_noret_decl(f_10661)
static void C_ccall f_10661(C_word c,C_word *av) C_noret;
C_noret_decl(f_10663)
static void C_ccall f_10663(C_word c,C_word *av) C_noret;
C_noret_decl(f_10677)
static void C_fcall f_10677(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10702)
static void C_ccall f_10702(C_word c,C_word *av) C_noret;
C_noret_decl(f_10718)
static void C_ccall f_10718(C_word c,C_word *av) C_noret;
C_noret_decl(f_10727)
static void C_fcall f_10727(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10730)
static void C_ccall f_10730(C_word c,C_word *av) C_noret;
C_noret_decl(f_10733)
static void C_ccall f_10733(C_word c,C_word *av) C_noret;
C_noret_decl(f_10736)
static void C_ccall f_10736(C_word c,C_word *av) C_noret;
C_noret_decl(f_10739)
static void C_ccall f_10739(C_word c,C_word *av) C_noret;
C_noret_decl(f_10742)
static void C_ccall f_10742(C_word c,C_word *av) C_noret;
C_noret_decl(f_10745)
static void C_fcall f_10745(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10752)
static void C_ccall f_10752(C_word c,C_word *av) C_noret;
C_noret_decl(f_10760)
static void C_ccall f_10760(C_word c,C_word *av) C_noret;
C_noret_decl(f_10766)
static void C_fcall f_10766(C_word t0,C_word t1) C_noret;
C_noret_decl(f_10769)
static void C_ccall f_10769(C_word c,C_word *av) C_noret;
C_noret_decl(f_10772)
static void C_ccall f_10772(C_word c,C_word *av) C_noret;
C_noret_decl(f_10777)
static void C_ccall f_10777(C_word c,C_word *av) C_noret;
C_noret_decl(f_10781)
static void C_ccall f_10781(C_word c,C_word *av) C_noret;
C_noret_decl(f_10784)
static void C_ccall f_10784(C_word c,C_word *av) C_noret;
C_noret_decl(f_10788)
static void C_ccall f_10788(C_word c,C_word *av) C_noret;
C_noret_decl(f_10792)
static void C_ccall f_10792(C_word c,C_word *av) C_noret;
C_noret_decl(f_10795)
static void C_ccall f_10795(C_word c,C_word *av) C_noret;
C_noret_decl(f_10798)
static void C_ccall f_10798(C_word c,C_word *av) C_noret;
C_noret_decl(f_10807)
static void C_ccall f_10807(C_word c,C_word *av) C_noret;
C_noret_decl(f_10812)
static void C_fcall f_10812(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10819)
static void C_ccall f_10819(C_word c,C_word *av) C_noret;
C_noret_decl(f_10827)
static void C_fcall f_10827(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10852)
static void C_ccall f_10852(C_word c,C_word *av) C_noret;
C_noret_decl(f_10865)
static void C_ccall f_10865(C_word c,C_word *av) C_noret;
C_noret_decl(f_10869)
static void C_ccall f_10869(C_word c,C_word *av) C_noret;
C_noret_decl(f_10872)
static void C_ccall f_10872(C_word c,C_word *av) C_noret;
C_noret_decl(f_10875)
static void C_ccall f_10875(C_word c,C_word *av) C_noret;
C_noret_decl(f_10878)
static void C_ccall f_10878(C_word c,C_word *av) C_noret;
C_noret_decl(f_10897)
static void C_ccall f_10897(C_word c,C_word *av) C_noret;
C_noret_decl(f_10900)
static void C_ccall f_10900(C_word c,C_word *av) C_noret;
C_noret_decl(f_10910)
static void C_ccall f_10910(C_word c,C_word *av) C_noret;
C_noret_decl(f_10912)
static void C_fcall f_10912(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_10923)
static void C_ccall f_10923(C_word c,C_word *av) C_noret;
C_noret_decl(f_10937)
static void C_ccall f_10937(C_word c,C_word *av) C_noret;
C_noret_decl(f_10941)
static void C_ccall f_10941(C_word c,C_word *av) C_noret;
C_noret_decl(f_10944)
static void C_ccall f_10944(C_word c,C_word *av) C_noret;
C_noret_decl(f_10947)
static void C_ccall f_10947(C_word c,C_word *av) C_noret;
C_noret_decl(f_10950)
static void C_ccall f_10950(C_word c,C_word *av) C_noret;
C_noret_decl(f_10954)
static void C_ccall f_10954(C_word c,C_word *av) C_noret;
C_noret_decl(f_10958)
static void C_ccall f_10958(C_word c,C_word *av) C_noret;
C_noret_decl(f_10962)
static void C_ccall f_10962(C_word c,C_word *av) C_noret;
C_noret_decl(f_10966)
static void C_ccall f_10966(C_word c,C_word *av) C_noret;
C_noret_decl(f_10969)
static void C_ccall f_10969(C_word c,C_word *av) C_noret;
C_noret_decl(f_10972)
static void C_ccall f_10972(C_word c,C_word *av) C_noret;
C_noret_decl(f_10975)
static void C_ccall f_10975(C_word c,C_word *av) C_noret;
C_noret_decl(f_10978)
static void C_ccall f_10978(C_word c,C_word *av) C_noret;
C_noret_decl(f_10993)
static void C_ccall f_10993(C_word c,C_word *av) C_noret;
C_noret_decl(f_10999)
static void C_ccall f_10999(C_word c,C_word *av) C_noret;
C_noret_decl(f_11009)
static void C_fcall f_11009(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_11019)
static void C_ccall f_11019(C_word c,C_word *av) C_noret;
C_noret_decl(f_11024)
static void C_ccall f_11024(C_word c,C_word *av) C_noret;
C_noret_decl(f_11030)
static void C_ccall f_11030(C_word c,C_word *av) C_noret;
C_noret_decl(f_11036)
static void C_ccall f_11036(C_word c,C_word *av) C_noret;
C_noret_decl(f_11040)
static void C_ccall f_11040(C_word c,C_word *av) C_noret;
C_noret_decl(f_11045)
static void C_ccall f_11045(C_word c,C_word *av) C_noret;
C_noret_decl(f_11047)
static void C_fcall f_11047(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11051)
static void C_ccall f_11051(C_word c,C_word *av) C_noret;
C_noret_decl(f_11056)
static void C_ccall f_11056(C_word c,C_word *av) C_noret;
C_noret_decl(f_11069)
static void C_ccall f_11069(C_word c,C_word *av) C_noret;
C_noret_decl(f_11075)
static void C_ccall f_11075(C_word c,C_word *av) C_noret;
C_noret_decl(f_11081)
static void C_ccall f_11081(C_word c,C_word *av) C_noret;
C_noret_decl(f_11082)
static void C_fcall f_11082(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11088)
static void C_ccall f_11088(C_word c,C_word *av) C_noret;
C_noret_decl(f_11099)
static void C_ccall f_11099(C_word c,C_word *av) C_noret;
C_noret_decl(f_11114)
static void C_ccall f_11114(C_word c,C_word *av) C_noret;
C_noret_decl(f_11118)
static void C_ccall f_11118(C_word c,C_word *av) C_noret;
C_noret_decl(f_11122)
static void C_ccall f_11122(C_word c,C_word *av) C_noret;
C_noret_decl(f_11125)
static void C_ccall f_11125(C_word c,C_word *av) C_noret;
C_noret_decl(f_11128)
static void C_ccall f_11128(C_word c,C_word *av) C_noret;
C_noret_decl(f_11131)
static void C_ccall f_11131(C_word c,C_word *av) C_noret;
C_noret_decl(f_11134)
static void C_ccall f_11134(C_word c,C_word *av) C_noret;
C_noret_decl(f_11137)
static void C_ccall f_11137(C_word c,C_word *av) C_noret;
C_noret_decl(f_11140)
static void C_ccall f_11140(C_word c,C_word *av) C_noret;
C_noret_decl(f_11143)
static void C_ccall f_11143(C_word c,C_word *av) C_noret;
C_noret_decl(f_11150)
static void C_ccall f_11150(C_word c,C_word *av) C_noret;
C_noret_decl(f_11160)
static void C_ccall f_11160(C_word c,C_word *av) C_noret;
C_noret_decl(f_11164)
static void C_fcall f_11164(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11177)
static void C_fcall f_11177(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11188)
static C_word C_fcall f_11188(C_word t0);
C_noret_decl(f_11213)
static void C_ccall f_11213(C_word c,C_word *av) C_noret;
C_noret_decl(f_11221)
static void C_fcall f_11221(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11246)
static void C_ccall f_11246(C_word c,C_word *av) C_noret;
C_noret_decl(f_11294)
static void C_ccall f_11294(C_word c,C_word *av) C_noret;
C_noret_decl(f_11297)
static void C_ccall f_11297(C_word c,C_word *av) C_noret;
C_noret_decl(f_11300)
static void C_ccall f_11300(C_word c,C_word *av) C_noret;
C_noret_decl(f_11306)
static void C_ccall f_11306(C_word c,C_word *av) C_noret;
C_noret_decl(f_11309)
static void C_ccall f_11309(C_word c,C_word *av) C_noret;
C_noret_decl(f_11314)
static void C_ccall f_11314(C_word c,C_word *av) C_noret;
C_noret_decl(f_11318)
static void C_ccall f_11318(C_word c,C_word *av) C_noret;
C_noret_decl(f_11322)
static void C_ccall f_11322(C_word c,C_word *av) C_noret;
C_noret_decl(f_11325)
static void C_ccall f_11325(C_word c,C_word *av) C_noret;
C_noret_decl(f_11331)
static void C_ccall f_11331(C_word c,C_word *av) C_noret;
C_noret_decl(f_11339)
static void C_ccall f_11339(C_word c,C_word *av) C_noret;
C_noret_decl(f_11343)
static void C_ccall f_11343(C_word c,C_word *av) C_noret;
C_noret_decl(f_11345)
static void C_ccall f_11345(C_word c,C_word *av) C_noret;
C_noret_decl(f_11349)
static void C_ccall f_11349(C_word c,C_word *av) C_noret;
C_noret_decl(f_11352)
static void C_ccall f_11352(C_word c,C_word *av) C_noret;
C_noret_decl(f_11357)
static void C_ccall f_11357(C_word c,C_word *av) C_noret;
C_noret_decl(f_11359)
static void C_fcall f_11359(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11384)
static void C_ccall f_11384(C_word c,C_word *av) C_noret;
C_noret_decl(f_11400)
static void C_ccall f_11400(C_word c,C_word *av) C_noret;
C_noret_decl(f_11406)
static void C_ccall f_11406(C_word c,C_word *av) C_noret;
C_noret_decl(f_11409)
static void C_ccall f_11409(C_word c,C_word *av) C_noret;
C_noret_decl(f_11416)
static void C_ccall f_11416(C_word c,C_word *av) C_noret;
C_noret_decl(f_11419)
static void C_ccall f_11419(C_word c,C_word *av) C_noret;
C_noret_decl(f_11425)
static void C_ccall f_11425(C_word c,C_word *av) C_noret;
C_noret_decl(f_11428)
static void C_ccall f_11428(C_word c,C_word *av) C_noret;
C_noret_decl(f_11431)
static void C_ccall f_11431(C_word c,C_word *av) C_noret;
C_noret_decl(f_11446)
static void C_ccall f_11446(C_word c,C_word *av) C_noret;
C_noret_decl(f_11449)
static void C_ccall f_11449(C_word c,C_word *av) C_noret;
C_noret_decl(f_11455)
static void C_ccall f_11455(C_word c,C_word *av) C_noret;
C_noret_decl(f_11462)
static void C_fcall f_11462(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11469)
static void C_ccall f_11469(C_word c,C_word *av) C_noret;
C_noret_decl(f_11492)
static void C_ccall f_11492(C_word c,C_word *av) C_noret;
C_noret_decl(f_11504)
static void C_ccall f_11504(C_word c,C_word *av) C_noret;
C_noret_decl(f_11514)
static void C_fcall f_11514(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11521)
static void C_ccall f_11521(C_word c,C_word *av) C_noret;
C_noret_decl(f_11548)
static void C_ccall f_11548(C_word c,C_word *av) C_noret;
C_noret_decl(f_11556)
static void C_ccall f_11556(C_word c,C_word *av) C_noret;
C_noret_decl(f_11562)
static void C_ccall f_11562(C_word c,C_word *av) C_noret;
C_noret_decl(f_11569)
static void C_ccall f_11569(C_word c,C_word *av) C_noret;
C_noret_decl(f_11580)
static void C_ccall f_11580(C_word c,C_word *av) C_noret;
C_noret_decl(f_11583)
static void C_ccall f_11583(C_word c,C_word *av) C_noret;
C_noret_decl(f_11586)
static void C_fcall f_11586(C_word t0,C_word t1) C_noret;
C_noret_decl(f_11592)
static void C_ccall f_11592(C_word c,C_word *av) C_noret;
C_noret_decl(f_11595)
static void C_ccall f_11595(C_word c,C_word *av) C_noret;
C_noret_decl(f_11601)
static void C_ccall f_11601(C_word c,C_word *av) C_noret;
C_noret_decl(f_11608)
static void C_ccall f_11608(C_word c,C_word *av) C_noret;
C_noret_decl(f_11612)
static void C_ccall f_11612(C_word c,C_word *av) C_noret;
C_noret_decl(f_11618)
static void C_ccall f_11618(C_word c,C_word *av) C_noret;
C_noret_decl(f_11621)
static void C_ccall f_11621(C_word c,C_word *av) C_noret;
C_noret_decl(f_11624)
static void C_ccall f_11624(C_word c,C_word *av) C_noret;
C_noret_decl(f_11627)
static void C_ccall f_11627(C_word c,C_word *av) C_noret;
C_noret_decl(f_11630)
static void C_ccall f_11630(C_word c,C_word *av) C_noret;
C_noret_decl(f_11634)
static void C_ccall f_11634(C_word c,C_word *av) C_noret;
C_noret_decl(f_11637)
static void C_ccall f_11637(C_word c,C_word *av) C_noret;
C_noret_decl(f_11643)
static void C_ccall f_11643(C_word c,C_word *av) C_noret;
C_noret_decl(f_11646)
static void C_ccall f_11646(C_word c,C_word *av) C_noret;
C_noret_decl(f_11649)
static void C_ccall f_11649(C_word c,C_word *av) C_noret;
C_noret_decl(f_11665)
static void C_ccall f_11665(C_word c,C_word *av) C_noret;
C_noret_decl(f_11671)
static void C_ccall f_11671(C_word c,C_word *av) C_noret;
C_noret_decl(f_11674)
static void C_ccall f_11674(C_word c,C_word *av) C_noret;
C_noret_decl(f_11677)
static void C_ccall f_11677(C_word c,C_word *av) C_noret;
C_noret_decl(f_11680)
static void C_ccall f_11680(C_word c,C_word *av) C_noret;
C_noret_decl(f_11683)
static void C_ccall f_11683(C_word c,C_word *av) C_noret;
C_noret_decl(f_11687)
static void C_ccall f_11687(C_word c,C_word *av) C_noret;
C_noret_decl(f_11690)
static void C_ccall f_11690(C_word c,C_word *av) C_noret;
C_noret_decl(f_11696)
static void C_ccall f_11696(C_word c,C_word *av) C_noret;
C_noret_decl(f_11699)
static void C_ccall f_11699(C_word c,C_word *av) C_noret;
C_noret_decl(f_11702)
static void C_ccall f_11702(C_word c,C_word *av) C_noret;
C_noret_decl(f_11709)
static void C_ccall f_11709(C_word c,C_word *av) C_noret;
C_noret_decl(f_11713)
static void C_ccall f_11713(C_word c,C_word *av) C_noret;
C_noret_decl(f_11736)
static void C_ccall f_11736(C_word c,C_word *av) C_noret;
C_noret_decl(f_11749)
static void C_ccall f_11749(C_word c,C_word *av) C_noret;
C_noret_decl(f_11755)
static void C_ccall f_11755(C_word c,C_word *av) C_noret;
C_noret_decl(f_11758)
static void C_ccall f_11758(C_word c,C_word *av) C_noret;
C_noret_decl(f_11761)
static void C_ccall f_11761(C_word c,C_word *av) C_noret;
C_noret_decl(f_11764)
static void C_ccall f_11764(C_word c,C_word *av) C_noret;
C_noret_decl(f_11767)
static void C_ccall f_11767(C_word c,C_word *av) C_noret;
C_noret_decl(f_11771)
static void C_ccall f_11771(C_word c,C_word *av) C_noret;
C_noret_decl(f_11774)
static void C_ccall f_11774(C_word c,C_word *av) C_noret;
C_noret_decl(f_11780)
static void C_ccall f_11780(C_word c,C_word *av) C_noret;
C_noret_decl(f_11783)
static void C_ccall f_11783(C_word c,C_word *av) C_noret;
C_noret_decl(f_11786)
static void C_ccall f_11786(C_word c,C_word *av) C_noret;
C_noret_decl(f_11815)
static void C_fcall f_11815(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11823)
static void C_ccall f_11823(C_word c,C_word *av) C_noret;
C_noret_decl(f_11831)
static void C_ccall f_11831(C_word c,C_word *av) C_noret;
C_noret_decl(f_11833)
static void C_fcall f_11833(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11858)
static void C_ccall f_11858(C_word c,C_word *av) C_noret;
C_noret_decl(f_11882)
static void C_ccall f_11882(C_word c,C_word *av) C_noret;
C_noret_decl(f_11886)
static void C_ccall f_11886(C_word c,C_word *av) C_noret;
C_noret_decl(f_11920)
static void C_ccall f_11920(C_word c,C_word *av) C_noret;
C_noret_decl(f_11924)
static void C_ccall f_11924(C_word c,C_word *av) C_noret;
C_noret_decl(f_11930)
static void C_fcall f_11930(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_11955)
static void C_ccall f_11955(C_word c,C_word *av) C_noret;
C_noret_decl(f_11979)
static void C_ccall f_11979(C_word c,C_word *av) C_noret;
C_noret_decl(f_12000)
static void C_ccall f_12000(C_word c,C_word *av) C_noret;
C_noret_decl(f_12004)
static void C_ccall f_12004(C_word c,C_word *av) C_noret;
C_noret_decl(f_12027)
static void C_ccall f_12027(C_word c,C_word *av) C_noret;
C_noret_decl(f_12042)
static void C_ccall f_12042(C_word c,C_word *av) C_noret;
C_noret_decl(f_12065)
static void C_ccall f_12065(C_word c,C_word *av) C_noret;
C_noret_decl(f_12069)
static void C_fcall f_12069(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12087)
static void C_ccall f_12087(C_word c,C_word *av) C_noret;
C_noret_decl(f_12094)
static void C_ccall f_12094(C_word c,C_word *av) C_noret;
C_noret_decl(f_12098)
static void C_ccall f_12098(C_word c,C_word *av) C_noret;
C_noret_decl(f_12113)
static void C_ccall f_12113(C_word c,C_word *av) C_noret;
C_noret_decl(f_12126)
static void C_ccall f_12126(C_word c,C_word *av) C_noret;
C_noret_decl(f_12139)
static void C_ccall f_12139(C_word c,C_word *av) C_noret;
C_noret_decl(f_12152)
static void C_ccall f_12152(C_word c,C_word *av) C_noret;
C_noret_decl(f_12165)
static void C_ccall f_12165(C_word c,C_word *av) C_noret;
C_noret_decl(f_12174)
static void C_ccall f_12174(C_word c,C_word *av) C_noret;
C_noret_decl(f_12177)
static void C_ccall f_12177(C_word c,C_word *av) C_noret;
C_noret_decl(f_12180)
static void C_ccall f_12180(C_word c,C_word *av) C_noret;
C_noret_decl(f_12221)
static void C_ccall f_12221(C_word c,C_word *av) C_noret;
C_noret_decl(f_12227)
static void C_ccall f_12227(C_word c,C_word *av) C_noret;
C_noret_decl(f_12236)
static void C_ccall f_12236(C_word c,C_word *av) C_noret;
C_noret_decl(f_12239)
static void C_ccall f_12239(C_word c,C_word *av) C_noret;
C_noret_decl(f_12242)
static void C_ccall f_12242(C_word c,C_word *av) C_noret;
C_noret_decl(f_12245)
static void C_ccall f_12245(C_word c,C_word *av) C_noret;
C_noret_decl(f_12248)
static void C_ccall f_12248(C_word c,C_word *av) C_noret;
C_noret_decl(f_12251)
static void C_ccall f_12251(C_word c,C_word *av) C_noret;
C_noret_decl(f_12254)
static void C_ccall f_12254(C_word c,C_word *av) C_noret;
C_noret_decl(f_12257)
static void C_ccall f_12257(C_word c,C_word *av) C_noret;
C_noret_decl(f_12260)
static void C_ccall f_12260(C_word c,C_word *av) C_noret;
C_noret_decl(f_12275)
static void C_fcall f_12275(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12295)
static void C_ccall f_12295(C_word c,C_word *av) C_noret;
C_noret_decl(f_12299)
static void C_ccall f_12299(C_word c,C_word *av) C_noret;
C_noret_decl(f_12302)
static void C_ccall f_12302(C_word c,C_word *av) C_noret;
C_noret_decl(f_12309)
static void C_ccall f_12309(C_word c,C_word *av) C_noret;
C_noret_decl(f_12312)
static void C_ccall f_12312(C_word c,C_word *av) C_noret;
C_noret_decl(f_12318)
static void C_ccall f_12318(C_word c,C_word *av) C_noret;
C_noret_decl(f_12321)
static void C_ccall f_12321(C_word c,C_word *av) C_noret;
C_noret_decl(f_12324)
static void C_ccall f_12324(C_word c,C_word *av) C_noret;
C_noret_decl(f_12343)
static void C_ccall f_12343(C_word c,C_word *av) C_noret;
C_noret_decl(f_12346)
static void C_ccall f_12346(C_word c,C_word *av) C_noret;
C_noret_decl(f_12355)
static void C_ccall f_12355(C_word c,C_word *av) C_noret;
C_noret_decl(f_12358)
static void C_ccall f_12358(C_word c,C_word *av) C_noret;
C_noret_decl(f_12361)
static void C_fcall f_12361(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12381)
static void C_ccall f_12381(C_word c,C_word *av) C_noret;
C_noret_decl(f_12389)
static void C_ccall f_12389(C_word c,C_word *av) C_noret;
C_noret_decl(f_12400)
static void C_ccall f_12400(C_word c,C_word *av) C_noret;
C_noret_decl(f_12403)
static void C_ccall f_12403(C_word c,C_word *av) C_noret;
C_noret_decl(f_12409)
static void C_ccall f_12409(C_word c,C_word *av) C_noret;
C_noret_decl(f_12412)
static void C_ccall f_12412(C_word c,C_word *av) C_noret;
C_noret_decl(f_12415)
static void C_ccall f_12415(C_word c,C_word *av) C_noret;
C_noret_decl(f_12430)
static void C_ccall f_12430(C_word c,C_word *av) C_noret;
C_noret_decl(f_12433)
static void C_ccall f_12433(C_word c,C_word *av) C_noret;
C_noret_decl(f_12436)
static void C_ccall f_12436(C_word c,C_word *av) C_noret;
C_noret_decl(f_12442)
static void C_ccall f_12442(C_word c,C_word *av) C_noret;
C_noret_decl(f_12454)
static void C_ccall f_12454(C_word c,C_word *av) C_noret;
C_noret_decl(f_12458)
static void C_ccall f_12458(C_word c,C_word *av) C_noret;
C_noret_decl(f_12462)
static void C_ccall f_12462(C_word c,C_word *av) C_noret;
C_noret_decl(f_12465)
static void C_ccall f_12465(C_word c,C_word *av) C_noret;
C_noret_decl(f_12471)
static void C_ccall f_12471(C_word c,C_word *av) C_noret;
C_noret_decl(f_12491)
static void C_ccall f_12491(C_word c,C_word *av) C_noret;
C_noret_decl(f_12495)
static void C_fcall f_12495(C_word t0,C_word t1) C_noret;
C_noret_decl(f_12520)
static void C_ccall f_12520(C_word c,C_word *av) C_noret;
C_noret_decl(f_12543)
static void C_ccall f_12543(C_word c,C_word *av) C_noret;
C_noret_decl(f_12545)
static void C_ccall f_12545(C_word c,C_word *av) C_noret;
C_noret_decl(f_12549)
static void C_ccall f_12549(C_word c,C_word *av) C_noret;
C_noret_decl(f_12552)
static void C_ccall f_12552(C_word c,C_word *av) C_noret;
C_noret_decl(f_12557)
static void C_ccall f_12557(C_word c,C_word *av) C_noret;
C_noret_decl(f_12590)
static void C_ccall f_12590(C_word c,C_word *av) C_noret;
C_noret_decl(f_12593)
static void C_ccall f_12593(C_word c,C_word *av) C_noret;
C_noret_decl(f_12596)
static void C_ccall f_12596(C_word c,C_word *av) C_noret;
C_noret_decl(f_12603)
static void C_ccall f_12603(C_word c,C_word *av) C_noret;
C_noret_decl(f_12606)
static void C_ccall f_12606(C_word c,C_word *av) C_noret;
C_noret_decl(f_12612)
static void C_ccall f_12612(C_word c,C_word *av) C_noret;
C_noret_decl(f_12615)
static void C_ccall f_12615(C_word c,C_word *av) C_noret;
C_noret_decl(f_12618)
static void C_ccall f_12618(C_word c,C_word *av) C_noret;
C_noret_decl(f_12625)
static void C_ccall f_12625(C_word c,C_word *av) C_noret;
C_noret_decl(f_12637)
static void C_ccall f_12637(C_word c,C_word *av) C_noret;
C_noret_decl(f_12643)
static void C_ccall f_12643(C_word c,C_word *av) C_noret;
C_noret_decl(f_12646)
static void C_ccall f_12646(C_word c,C_word *av) C_noret;
C_noret_decl(f_12649)
static void C_ccall f_12649(C_word c,C_word *av) C_noret;
C_noret_decl(f_12659)
static void C_ccall f_12659(C_word c,C_word *av) C_noret;
C_noret_decl(f_12662)
static void C_ccall f_12662(C_word c,C_word *av) C_noret;
C_noret_decl(f_12672)
static void C_ccall f_12672(C_word c,C_word *av) C_noret;
C_noret_decl(f_12675)
static void C_ccall f_12675(C_word c,C_word *av) C_noret;
C_noret_decl(f_12678)
static void C_ccall f_12678(C_word c,C_word *av) C_noret;
C_noret_decl(f_12681)
static void C_ccall f_12681(C_word c,C_word *av) C_noret;
C_noret_decl(f_12684)
static void C_ccall f_12684(C_word c,C_word *av) C_noret;
C_noret_decl(f_12687)
static void C_ccall f_12687(C_word c,C_word *av) C_noret;
C_noret_decl(f_12705)
static void C_ccall f_12705(C_word c,C_word *av) C_noret;
C_noret_decl(f_12708)
static void C_ccall f_12708(C_word c,C_word *av) C_noret;
C_noret_decl(f_12714)
static void C_ccall f_12714(C_word c,C_word *av) C_noret;
C_noret_decl(f_12717)
static void C_ccall f_12717(C_word c,C_word *av) C_noret;
C_noret_decl(f_12720)
static void C_ccall f_12720(C_word c,C_word *av) C_noret;
C_noret_decl(f_12738)
static void C_ccall f_12738(C_word c,C_word *av) C_noret;
C_noret_decl(f_12741)
static void C_ccall f_12741(C_word c,C_word *av) C_noret;
C_noret_decl(f_12747)
static void C_ccall f_12747(C_word c,C_word *av) C_noret;
C_noret_decl(f_12750)
static void C_ccall f_12750(C_word c,C_word *av) C_noret;
C_noret_decl(f_12753)
static void C_ccall f_12753(C_word c,C_word *av) C_noret;
C_noret_decl(f_12758)
static void C_ccall f_12758(C_word c,C_word *av) C_noret;
C_noret_decl(f_12764)
static void C_ccall f_12764(C_word c,C_word *av) C_noret;
C_noret_decl(f_12770)
static void C_ccall f_12770(C_word c,C_word *av) C_noret;
C_noret_decl(f_12776)
static void C_ccall f_12776(C_word c,C_word *av) C_noret;
C_noret_decl(f_12782)
static void C_ccall f_12782(C_word c,C_word *av) C_noret;
C_noret_decl(f_12789)
static void C_ccall f_12789(C_word c,C_word *av) C_noret;
C_noret_decl(f_12808)
static void C_ccall f_12808(C_word c,C_word *av) C_noret;
C_noret_decl(f_12814)
static void C_ccall f_12814(C_word c,C_word *av) C_noret;
C_noret_decl(f_12836)
static void C_fcall f_12836(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12842)
static void C_fcall f_12842(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12850)
static void C_ccall f_12850(C_word c,C_word *av) C_noret;
C_noret_decl(f_12857)
static void C_ccall f_12857(C_word c,C_word *av) C_noret;
C_noret_decl(f_12859)
static void C_fcall f_12859(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_12884)
static void C_ccall f_12884(C_word c,C_word *av) C_noret;
C_noret_decl(f_12902)
static void C_ccall f_12902(C_word c,C_word *av) C_noret;
C_noret_decl(f_12912)
static void C_ccall f_12912(C_word c,C_word *av) C_noret;
C_noret_decl(f_12922)
static void C_ccall f_12922(C_word c,C_word *av) C_noret;
C_noret_decl(f_12934)
static void C_ccall f_12934(C_word c,C_word *av) C_noret;
C_noret_decl(f_12937)
static void C_ccall f_12937(C_word c,C_word *av) C_noret;
C_noret_decl(f_12944)
static void C_ccall f_12944(C_word c,C_word *av) C_noret;
C_noret_decl(f_12948)
static void C_ccall f_12948(C_word c,C_word *av) C_noret;
C_noret_decl(f_12956)
static void C_ccall f_12956(C_word c,C_word *av) C_noret;
C_noret_decl(f_12968)
static void C_ccall f_12968(C_word c,C_word *av) C_noret;
C_noret_decl(f_12972)
static void C_ccall f_12972(C_word c,C_word *av) C_noret;
C_noret_decl(f_12984)
static void C_ccall f_12984(C_word c,C_word *av) C_noret;
C_noret_decl(f_13044)
static void C_ccall f_13044(C_word c,C_word *av) C_noret;
C_noret_decl(f_13073)
static void C_fcall f_13073(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13097)
static void C_ccall f_13097(C_word c,C_word *av) C_noret;
C_noret_decl(f_13105)
static void C_ccall f_13105(C_word c,C_word *av) C_noret;
C_noret_decl(f_13109)
static void C_ccall f_13109(C_word c,C_word *av) C_noret;
C_noret_decl(f_13113)
static void C_ccall f_13113(C_word c,C_word *av) C_noret;
C_noret_decl(f_13154)
static void C_ccall f_13154(C_word c,C_word *av) C_noret;
C_noret_decl(f_13165)
static void C_ccall f_13165(C_word c,C_word *av) C_noret;
C_noret_decl(f_13172)
static void C_ccall f_13172(C_word c,C_word *av) C_noret;
C_noret_decl(f_13175)
static void C_ccall f_13175(C_word c,C_word *av) C_noret;
C_noret_decl(f_13181)
static void C_ccall f_13181(C_word c,C_word *av) C_noret;
C_noret_decl(f_13184)
static void C_ccall f_13184(C_word c,C_word *av) C_noret;
C_noret_decl(f_13187)
static void C_ccall f_13187(C_word c,C_word *av) C_noret;
C_noret_decl(f_13224)
static void C_fcall f_13224(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13250)
static void C_fcall f_13250(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13278)
static void C_ccall f_13278(C_word c,C_word *av) C_noret;
C_noret_decl(f_13293)
static void C_ccall f_13293(C_word c,C_word *av) C_noret;
C_noret_decl(f_13310)
static void C_ccall f_13310(C_word c,C_word *av) C_noret;
C_noret_decl(f_13316)
static void C_ccall f_13316(C_word c,C_word *av) C_noret;
C_noret_decl(f_13319)
static void C_ccall f_13319(C_word c,C_word *av) C_noret;
C_noret_decl(f_13339)
static void C_ccall f_13339(C_word c,C_word *av) C_noret;
C_noret_decl(f_13344)
static void C_ccall f_13344(C_word c,C_word *av) C_noret;
C_noret_decl(f_13350)
static void C_ccall f_13350(C_word c,C_word *av) C_noret;
C_noret_decl(f_13489)
static void C_ccall f_13489(C_word c,C_word *av) C_noret;
C_noret_decl(f_13494)
static void C_ccall f_13494(C_word c,C_word *av) C_noret;
C_noret_decl(f_13502)
static void C_ccall f_13502(C_word c,C_word *av) C_noret;
C_noret_decl(f_13504)
static void C_ccall f_13504(C_word c,C_word *av) C_noret;
C_noret_decl(f_13522)
static void C_ccall f_13522(C_word c,C_word *av) C_noret;
C_noret_decl(f_13528)
static void C_ccall f_13528(C_word c,C_word *av) C_noret;
C_noret_decl(f_13531)
static void C_ccall f_13531(C_word c,C_word *av) C_noret;
C_noret_decl(f_13534)
static void C_ccall f_13534(C_word c,C_word *av) C_noret;
C_noret_decl(f_13537)
static void C_ccall f_13537(C_word c,C_word *av) C_noret;
C_noret_decl(f_13555)
static void C_ccall f_13555(C_word c,C_word *av) C_noret;
C_noret_decl(f_13558)
static void C_ccall f_13558(C_word c,C_word *av) C_noret;
C_noret_decl(f_13561)
static void C_ccall f_13561(C_word c,C_word *av) C_noret;
C_noret_decl(f_13567)
static void C_ccall f_13567(C_word c,C_word *av) C_noret;
C_noret_decl(f_13570)
static void C_ccall f_13570(C_word c,C_word *av) C_noret;
C_noret_decl(f_13609)
static void C_fcall f_13609(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_13614)
static void C_fcall f_13614(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13626)
static void C_fcall f_13626(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13651)
static void C_ccall f_13651(C_word c,C_word *av) C_noret;
C_noret_decl(f_13661)
static void C_ccall f_13661(C_word c,C_word *av) C_noret;
C_noret_decl(f_13672)
static void C_ccall f_13672(C_word c,C_word *av) C_noret;
C_noret_decl(f_13676)
static void C_ccall f_13676(C_word c,C_word *av) C_noret;
C_noret_decl(f_13686)
static void C_ccall f_13686(C_word c,C_word *av) C_noret;
C_noret_decl(f_13692)
static void C_ccall f_13692(C_word c,C_word *av) C_noret;
C_noret_decl(f_13700)
static void C_fcall f_13700(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_13735)
static void C_fcall f_13735(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13742)
static void C_ccall f_13742(C_word c,C_word *av) C_noret;
C_noret_decl(f_13745)
static void C_ccall f_13745(C_word c,C_word *av) C_noret;
C_noret_decl(f_13752)
static void C_ccall f_13752(C_word c,C_word *av) C_noret;
C_noret_decl(f_13754)
static void C_fcall f_13754(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_13760)
static void C_fcall f_13760(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13768)
static void C_ccall f_13768(C_word c,C_word *av) C_noret;
C_noret_decl(f_13774)
static void C_ccall f_13774(C_word c,C_word *av) C_noret;
C_noret_decl(f_13777)
static void C_ccall f_13777(C_word c,C_word *av) C_noret;
C_noret_decl(f_13780)
static void C_ccall f_13780(C_word c,C_word *av) C_noret;
C_noret_decl(f_13783)
static void C_ccall f_13783(C_word c,C_word *av) C_noret;
C_noret_decl(f_13788)
static void C_fcall f_13788(C_word t0,C_word t1) C_noret;
C_noret_decl(f_13792)
static void C_ccall f_13792(C_word c,C_word *av) C_noret;
C_noret_decl(f_13795)
static void C_ccall f_13795(C_word c,C_word *av) C_noret;
C_noret_decl(f_13798)
static void C_ccall f_13798(C_word c,C_word *av) C_noret;
C_noret_decl(f_13804)
static void C_ccall f_13804(C_word c,C_word *av) C_noret;
C_noret_decl(f_13808)
static void C_ccall f_13808(C_word c,C_word *av) C_noret;
C_noret_decl(f_13812)
static void C_ccall f_13812(C_word c,C_word *av) C_noret;
C_noret_decl(f_13823)
static void C_ccall f_13823(C_word c,C_word *av) C_noret;
C_noret_decl(f_13826)
static void C_ccall f_13826(C_word c,C_word *av) C_noret;
C_noret_decl(f_13829)
static void C_ccall f_13829(C_word c,C_word *av) C_noret;
C_noret_decl(f_13865)
static void C_ccall f_13865(C_word c,C_word *av) C_noret;
C_noret_decl(f_13869)
static void C_ccall f_13869(C_word c,C_word *av) C_noret;
C_noret_decl(f_13890)
static void C_ccall f_13890(C_word c,C_word *av) C_noret;
C_noret_decl(f_13894)
static void C_ccall f_13894(C_word c,C_word *av) C_noret;
C_noret_decl(f_13915)
static void C_ccall f_13915(C_word c,C_word *av) C_noret;
C_noret_decl(f_13919)
static void C_ccall f_13919(C_word c,C_word *av) C_noret;
C_noret_decl(f_13923)
static void C_ccall f_13923(C_word c,C_word *av) C_noret;
C_noret_decl(f_13936)
static void C_ccall f_13936(C_word c,C_word *av) C_noret;
C_noret_decl(f_13940)
static void C_ccall f_13940(C_word c,C_word *av) C_noret;
C_noret_decl(f_14023)
static void C_fcall f_14023(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14033)
static void C_ccall f_14033(C_word c,C_word *av) C_noret;
C_noret_decl(f_14065)
static void C_ccall f_14065(C_word c,C_word *av) C_noret;
C_noret_decl(f_14071)
static void C_ccall f_14071(C_word c,C_word *av) C_noret;
C_noret_decl(f_14079)
static void C_ccall f_14079(C_word c,C_word *av) C_noret;
C_noret_decl(f_14087)
static void C_fcall f_14087(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14097)
static void C_ccall f_14097(C_word c,C_word *av) C_noret;
C_noret_decl(f_14110)
static void C_fcall f_14110(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14120)
static void C_ccall f_14120(C_word c,C_word *av) C_noret;
C_noret_decl(f_14147)
static void C_ccall f_14147(C_word c,C_word *av) C_noret;
C_noret_decl(f_14158)
static void C_ccall f_14158(C_word c,C_word *av) C_noret;
C_noret_decl(f_14168)
static void C_fcall f_14168(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14178)
static void C_ccall f_14178(C_word c,C_word *av) C_noret;
C_noret_decl(f_14191)
static void C_fcall f_14191(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14201)
static void C_ccall f_14201(C_word c,C_word *av) C_noret;
C_noret_decl(f_14227)
static void C_ccall f_14227(C_word c,C_word *av) C_noret;
C_noret_decl(f_14231)
static void C_ccall f_14231(C_word c,C_word *av) C_noret;
C_noret_decl(f_14269)
static void C_ccall f_14269(C_word c,C_word *av) C_noret;
C_noret_decl(f_14277)
static void C_fcall f_14277(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14287)
static void C_ccall f_14287(C_word c,C_word *av) C_noret;
C_noret_decl(f_14314)
static void C_ccall f_14314(C_word c,C_word *av) C_noret;
C_noret_decl(f_14322)
static void C_fcall f_14322(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14332)
static void C_ccall f_14332(C_word c,C_word *av) C_noret;
C_noret_decl(f_14354)
static void C_ccall f_14354(C_word c,C_word *av) C_noret;
C_noret_decl(f_14357)
static void C_ccall f_14357(C_word c,C_word *av) C_noret;
C_noret_decl(f_14374)
static void C_ccall f_14374(C_word c,C_word *av) C_noret;
C_noret_decl(f_14378)
static void C_ccall f_14378(C_word c,C_word *av) C_noret;
C_noret_decl(f_14402)
static void C_ccall f_14402(C_word c,C_word *av) C_noret;
C_noret_decl(f_14406)
static void C_ccall f_14406(C_word c,C_word *av) C_noret;
C_noret_decl(f_14434)
static void C_ccall f_14434(C_word c,C_word *av) C_noret;
C_noret_decl(f_14442)
static void C_fcall f_14442(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14452)
static void C_ccall f_14452(C_word c,C_word *av) C_noret;
C_noret_decl(f_14487)
static void C_ccall f_14487(C_word c,C_word *av) C_noret;
C_noret_decl(f_14491)
static void C_ccall f_14491(C_word c,C_word *av) C_noret;
C_noret_decl(f_14495)
static void C_ccall f_14495(C_word c,C_word *av) C_noret;
C_noret_decl(f_14523)
static void C_ccall f_14523(C_word c,C_word *av) C_noret;
C_noret_decl(f_14531)
static void C_fcall f_14531(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14541)
static void C_ccall f_14541(C_word c,C_word *av) C_noret;
C_noret_decl(f_14562)
static void C_ccall f_14562(C_word c,C_word *av) C_noret;
C_noret_decl(f_14565)
static void C_ccall f_14565(C_word c,C_word *av) C_noret;
C_noret_decl(f_14614)
static void C_ccall f_14614(C_word c,C_word *av) C_noret;
C_noret_decl(f_14629)
static void C_fcall f_14629(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14639)
static void C_ccall f_14639(C_word c,C_word *av) C_noret;
C_noret_decl(f_14662)
static void C_ccall f_14662(C_word c,C_word *av) C_noret;
C_noret_decl(f_14670)
static void C_fcall f_14670(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14680)
static void C_ccall f_14680(C_word c,C_word *av) C_noret;
C_noret_decl(f_14721)
static void C_ccall f_14721(C_word c,C_word *av) C_noret;
C_noret_decl(f_14729)
static void C_fcall f_14729(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14739)
static void C_ccall f_14739(C_word c,C_word *av) C_noret;
C_noret_decl(f_14763)
static void C_ccall f_14763(C_word c,C_word *av) C_noret;
C_noret_decl(f_14785)
static void C_ccall f_14785(C_word c,C_word *av) C_noret;
C_noret_decl(f_14807)
static void C_ccall f_14807(C_word c,C_word *av) C_noret;
C_noret_decl(f_14813)
static void C_ccall f_14813(C_word c,C_word *av) C_noret;
C_noret_decl(f_14821)
static void C_ccall f_14821(C_word c,C_word *av) C_noret;
C_noret_decl(f_14829)
static void C_fcall f_14829(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14839)
static void C_ccall f_14839(C_word c,C_word *av) C_noret;
C_noret_decl(f_14865)
static void C_ccall f_14865(C_word c,C_word *av) C_noret;
C_noret_decl(f_14884)
static void C_ccall f_14884(C_word c,C_word *av) C_noret;
C_noret_decl(f_14888)
static void C_ccall f_14888(C_word c,C_word *av) C_noret;
C_noret_decl(f_14895)
static void C_fcall f_14895(C_word t0,C_word t1) C_noret;
C_noret_decl(f_14941)
static void C_ccall f_14941(C_word c,C_word *av) C_noret;
C_noret_decl(f_14947)
static void C_ccall f_14947(C_word c,C_word *av) C_noret;
C_noret_decl(f_14949)
static void C_fcall f_14949(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_14974)
static void C_ccall f_14974(C_word c,C_word *av) C_noret;
C_noret_decl(f_14999)
static void C_ccall f_14999(C_word c,C_word *av) C_noret;
C_noret_decl(f_15002)
static void C_fcall f_15002(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15051)
static void C_ccall f_15051(C_word c,C_word *av) C_noret;
C_noret_decl(f_15059)
static void C_fcall f_15059(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15069)
static void C_ccall f_15069(C_word c,C_word *av) C_noret;
C_noret_decl(f_15105)
static void C_ccall f_15105(C_word c,C_word *av) C_noret;
C_noret_decl(f_15113)
static void C_fcall f_15113(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15123)
static void C_ccall f_15123(C_word c,C_word *av) C_noret;
C_noret_decl(f_15160)
static void C_ccall f_15160(C_word c,C_word *av) C_noret;
C_noret_decl(f_15168)
static void C_fcall f_15168(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15178)
static void C_ccall f_15178(C_word c,C_word *av) C_noret;
C_noret_decl(f_15200)
static void C_fcall f_15200(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15207)
static void C_fcall f_15207(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15214)
static void C_ccall f_15214(C_word c,C_word *av) C_noret;
C_noret_decl(f_15217)
static void C_ccall f_15217(C_word c,C_word *av) C_noret;
C_noret_decl(f_15220)
static void C_ccall f_15220(C_word c,C_word *av) C_noret;
C_noret_decl(f_15226)
static void C_ccall f_15226(C_word c,C_word *av) C_noret;
C_noret_decl(f_15236)
static void C_ccall f_15236(C_word c,C_word *av) C_noret;
C_noret_decl(f_15242)
static void C_ccall f_15242(C_word c,C_word *av) C_noret;
C_noret_decl(f_15252)
static void C_ccall f_15252(C_word c,C_word *av) C_noret;
C_noret_decl(f_15255)
static void C_ccall f_15255(C_word c,C_word *av) C_noret;
C_noret_decl(f_15258)
static void C_ccall f_15258(C_word c,C_word *av) C_noret;
C_noret_decl(f_15261)
static void C_ccall f_15261(C_word c,C_word *av) C_noret;
C_noret_decl(f_15274)
static void C_ccall f_15274(C_word c,C_word *av) C_noret;
C_noret_decl(f_15290)
static void C_fcall f_15290(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15322)
static void C_ccall f_15322(C_word c,C_word *av) C_noret;
C_noret_decl(f_15364)
static void C_fcall f_15364(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15374)
static void C_ccall f_15374(C_word c,C_word *av) C_noret;
C_noret_decl(f_15392)
static void C_fcall f_15392(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15399)
static void C_fcall f_15399(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15402)
static void C_ccall f_15402(C_word c,C_word *av) C_noret;
C_noret_decl(f_15405)
static void C_ccall f_15405(C_word c,C_word *av) C_noret;
C_noret_decl(f_15411)
static void C_ccall f_15411(C_word c,C_word *av) C_noret;
C_noret_decl(f_15421)
static void C_ccall f_15421(C_word c,C_word *av) C_noret;
C_noret_decl(f_15427)
static void C_ccall f_15427(C_word c,C_word *av) C_noret;
C_noret_decl(f_15486)
static void C_fcall f_15486(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15496)
static void C_ccall f_15496(C_word c,C_word *av) C_noret;
C_noret_decl(f_15630)
static void C_ccall f_15630(C_word c,C_word *av) C_noret;
C_noret_decl(f_15653)
static void C_ccall f_15653(C_word c,C_word *av) C_noret;
C_noret_decl(f_15662)
static void C_ccall f_15662(C_word c,C_word *av) C_noret;
C_noret_decl(f_15671)
static void C_ccall f_15671(C_word c,C_word *av) C_noret;
C_noret_decl(f_15680)
static void C_ccall f_15680(C_word c,C_word *av) C_noret;
C_noret_decl(f_15689)
static void C_ccall f_15689(C_word c,C_word *av) C_noret;
C_noret_decl(f_15698)
static void C_ccall f_15698(C_word c,C_word *av) C_noret;
C_noret_decl(f_15707)
static void C_ccall f_15707(C_word c,C_word *av) C_noret;
C_noret_decl(f_15716)
static void C_ccall f_15716(C_word c,C_word *av) C_noret;
C_noret_decl(f_15725)
static void C_fcall f_15725(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7) C_noret;
C_noret_decl(f_15734)
static void C_fcall f_15734(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_15763)
static void C_ccall f_15763(C_word c,C_word *av) C_noret;
C_noret_decl(f_15767)
static void C_ccall f_15767(C_word c,C_word *av) C_noret;
C_noret_decl(f_15812)
static void C_ccall f_15812(C_word c,C_word *av) C_noret;
C_noret_decl(f_15815)
static void C_ccall f_15815(C_word c,C_word *av) C_noret;
C_noret_decl(f_15818)
static void C_ccall f_15818(C_word c,C_word *av) C_noret;
C_noret_decl(f_15821)
static void C_ccall f_15821(C_word c,C_word *av) C_noret;
C_noret_decl(f_15824)
static void C_ccall f_15824(C_word c,C_word *av) C_noret;
C_noret_decl(f_15827)
static void C_ccall f_15827(C_word c,C_word *av) C_noret;
C_noret_decl(f_15830)
static void C_ccall f_15830(C_word c,C_word *av) C_noret;
C_noret_decl(f_15840)
static void C_fcall f_15840(C_word t0,C_word t1) C_noret;
C_noret_decl(f_15852)
static void C_ccall f_15852(C_word c,C_word *av) C_noret;
C_noret_decl(f_15861)
static void C_ccall f_15861(C_word c,C_word *av) C_noret;
C_noret_decl(f_15872)
static void C_ccall f_15872(C_word c,C_word *av) C_noret;
C_noret_decl(f_15884)
static void C_ccall f_15884(C_word c,C_word *av) C_noret;
C_noret_decl(f_15892)
static void C_ccall f_15892(C_word c,C_word *av) C_noret;
C_noret_decl(f_15899)
static void C_ccall f_15899(C_word c,C_word *av) C_noret;
C_noret_decl(f_15902)
static void C_ccall f_15902(C_word c,C_word *av) C_noret;
C_noret_decl(f_15913)
static void C_ccall f_15913(C_word c,C_word *av) C_noret;
C_noret_decl(f_15917)
static void C_ccall f_15917(C_word c,C_word *av) C_noret;
C_noret_decl(f_15921)
static void C_ccall f_15921(C_word c,C_word *av) C_noret;
C_noret_decl(f_15947)
static void C_fcall f_15947(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_15976)
static void C_ccall f_15976(C_word c,C_word *av) C_noret;
C_noret_decl(f_16017)
static void C_ccall f_16017(C_word c,C_word *av) C_noret;
C_noret_decl(f_16028)
static void C_fcall f_16028(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16053)
static void C_ccall f_16053(C_word c,C_word *av) C_noret;
C_noret_decl(f_16072)
static void C_ccall f_16072(C_word c,C_word *av) C_noret;
C_noret_decl(f_16080)
static void C_fcall f_16080(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16105)
static void C_ccall f_16105(C_word c,C_word *av) C_noret;
C_noret_decl(f_16114)
static void C_fcall f_16114(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16121)
static void C_ccall f_16121(C_word c,C_word *av) C_noret;
C_noret_decl(f_16140)
static void C_ccall f_16140(C_word c,C_word *av) C_noret;
C_noret_decl(f_16151)
static void C_fcall f_16151(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16161)
static void C_ccall f_16161(C_word c,C_word *av) C_noret;
C_noret_decl(f_16175)
static void C_ccall f_16175(C_word c,C_word *av) C_noret;
C_noret_decl(f_16181)
static void C_ccall f_16181(C_word c,C_word *av) C_noret;
C_noret_decl(f_16187)
static void C_ccall f_16187(C_word c,C_word *av) C_noret;
C_noret_decl(f_16192)
static void C_fcall f_16192(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16226)
static void C_fcall f_16226(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16268)
static void C_fcall f_16268(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16274)
static void C_ccall f_16274(C_word c,C_word *av) C_noret;
C_noret_decl(f_16277)
static void C_ccall f_16277(C_word c,C_word *av) C_noret;
C_noret_decl(f_16291)
static void C_ccall f_16291(C_word c,C_word *av) C_noret;
C_noret_decl(f_16297)
static void C_ccall f_16297(C_word c,C_word *av) C_noret;
C_noret_decl(f_16303)
static void C_ccall f_16303(C_word c,C_word *av) C_noret;
C_noret_decl(f_16308)
static void C_fcall f_16308(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16342)
static void C_fcall f_16342(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16420)
static void C_fcall f_16420(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16426)
static void C_fcall f_16426(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16436)
static void C_ccall f_16436(C_word c,C_word *av) C_noret;
C_noret_decl(f_16445)
static void C_fcall f_16445(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16462)
static void C_ccall f_16462(C_word c,C_word *av) C_noret;
C_noret_decl(f_16468)
static void C_ccall f_16468(C_word c,C_word *av) C_noret;
C_noret_decl(f_16499)
static void C_ccall f_16499(C_word c,C_word *av) C_noret;
C_noret_decl(f_16502)
static void C_fcall f_16502(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_16506)
static void C_ccall f_16506(C_word c,C_word *av) C_noret;
C_noret_decl(f_16513)
static void C_ccall f_16513(C_word c,C_word *av) C_noret;
C_noret_decl(f_16525)
static void C_ccall f_16525(C_word c,C_word *av) C_noret;
C_noret_decl(f_16531)
static void C_ccall f_16531(C_word c,C_word *av) C_noret;
C_noret_decl(f_16547)
static void C_ccall f_16547(C_word c,C_word *av) C_noret;
C_noret_decl(f_16553)
static void C_fcall f_16553(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_16571)
static void C_ccall f_16571(C_word c,C_word *av) C_noret;
C_noret_decl(f_16575)
static void C_ccall f_16575(C_word c,C_word *av) C_noret;
C_noret_decl(f_16577)
static void C_fcall f_16577(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_16581)
static void C_ccall f_16581(C_word c,C_word *av) C_noret;
C_noret_decl(f_16584)
static void C_ccall f_16584(C_word c,C_word *av) C_noret;
C_noret_decl(f_16587)
static void C_ccall f_16587(C_word c,C_word *av) C_noret;
C_noret_decl(f_16590)
static void C_ccall f_16590(C_word c,C_word *av) C_noret;
C_noret_decl(f_16599)
static void C_fcall f_16599(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16611)
static void C_ccall f_16611(C_word c,C_word *av) C_noret;
C_noret_decl(f_16614)
static void C_ccall f_16614(C_word c,C_word *av) C_noret;
C_noret_decl(f_16615)
static void C_ccall f_16615(C_word c,C_word *av) C_noret;
C_noret_decl(f_16631)
static void C_ccall f_16631(C_word c,C_word *av) C_noret;
C_noret_decl(f_16646)
static void C_ccall f_16646(C_word c,C_word *av) C_noret;
C_noret_decl(f_16650)
static void C_ccall f_16650(C_word c,C_word *av) C_noret;
C_noret_decl(f_16656)
static void C_ccall f_16656(C_word c,C_word *av) C_noret;
C_noret_decl(f_16668)
static void C_ccall f_16668(C_word c,C_word *av) C_noret;
C_noret_decl(f_16672)
static void C_ccall f_16672(C_word c,C_word *av) C_noret;
C_noret_decl(f_16692)
static void C_ccall f_16692(C_word c,C_word *av) C_noret;
C_noret_decl(f_16696)
static void C_ccall f_16696(C_word c,C_word *av) C_noret;
C_noret_decl(f_16705)
static void C_ccall f_16705(C_word c,C_word *av) C_noret;
C_noret_decl(f_16720)
static void C_fcall f_16720(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_16743)
static void C_ccall f_16743(C_word c,C_word *av) C_noret;
C_noret_decl(f_16750)
static void C_ccall f_16750(C_word c,C_word *av) C_noret;
C_noret_decl(f_16774)
static void C_ccall f_16774(C_word c,C_word *av) C_noret;
C_noret_decl(f_16807)
static void C_ccall f_16807(C_word c,C_word *av) C_noret;
C_noret_decl(f_16820)
static void C_ccall f_16820(C_word c,C_word *av) C_noret;
C_noret_decl(f_16835)
static void C_ccall f_16835(C_word c,C_word *av) C_noret;
C_noret_decl(f_16851)
static void C_ccall f_16851(C_word c,C_word *av) C_noret;
C_noret_decl(f_16855)
static void C_ccall f_16855(C_word c,C_word *av) C_noret;
C_noret_decl(f_16859)
static void C_ccall f_16859(C_word c,C_word *av) C_noret;
C_noret_decl(f_16891)
static void C_ccall f_16891(C_word c,C_word *av) C_noret;
C_noret_decl(f_16897)
static void C_ccall f_16897(C_word c,C_word *av) C_noret;
C_noret_decl(f_16908)
static void C_ccall f_16908(C_word c,C_word *av) C_noret;
C_noret_decl(f_16912)
static void C_ccall f_16912(C_word c,C_word *av) C_noret;
C_noret_decl(f_16921)
static void C_fcall f_16921(C_word t0,C_word t1) C_noret;
C_noret_decl(f_16961)
static void C_ccall f_16961(C_word c,C_word *av) C_noret;
C_noret_decl(f_16969)
static void C_ccall f_16969(C_word c,C_word *av) C_noret;
C_noret_decl(f_16974)
static void C_ccall f_16974(C_word c,C_word *av) C_noret;
C_noret_decl(f_16980)
static void C_ccall f_16980(C_word c,C_word *av) C_noret;
C_noret_decl(f_17123)
static void C_fcall f_17123(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_17127)
static void C_ccall f_17127(C_word c,C_word *av) C_noret;
C_noret_decl(f_17130)
static void C_ccall f_17130(C_word c,C_word *av) C_noret;
C_noret_decl(f_17145)
static void C_ccall f_17145(C_word c,C_word *av) C_noret;
C_noret_decl(f_17149)
static void C_ccall f_17149(C_word c,C_word *av) C_noret;
C_noret_decl(f_17151)
static void C_ccall f_17151(C_word c,C_word *av) C_noret;
C_noret_decl(f_17157)
static void C_ccall f_17157(C_word c,C_word *av) C_noret;
C_noret_decl(f_17165)
static void C_ccall f_17165(C_word c,C_word *av) C_noret;
C_noret_decl(f_17169)
static void C_ccall f_17169(C_word c,C_word *av) C_noret;
C_noret_decl(f_17181)
static void C_ccall f_17181(C_word c,C_word *av) C_noret;
C_noret_decl(f_17185)
static void C_ccall f_17185(C_word c,C_word *av) C_noret;
C_noret_decl(f_17194)
static void C_ccall f_17194(C_word c,C_word *av) C_noret;
C_noret_decl(f_17204)
static void C_ccall f_17204(C_word c,C_word *av) C_noret;
C_noret_decl(f_17207)
static void C_ccall f_17207(C_word c,C_word *av) C_noret;
C_noret_decl(f_17222)
static void C_ccall f_17222(C_word c,C_word *av) C_noret;
C_noret_decl(f_17226)
static void C_ccall f_17226(C_word c,C_word *av) C_noret;
C_noret_decl(f_17238)
static void C_ccall f_17238(C_word c,C_word *av) C_noret;
C_noret_decl(f_17250)
static void C_ccall f_17250(C_word c,C_word *av) C_noret;
C_noret_decl(f_17254)
static void C_ccall f_17254(C_word c,C_word *av) C_noret;
C_noret_decl(f_17263)
static void C_ccall f_17263(C_word c,C_word *av) C_noret;
C_noret_decl(f_17269)
static void C_fcall f_17269(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_17275)
static void C_ccall f_17275(C_word c,C_word *av) C_noret;
C_noret_decl(f_17283)
static void C_ccall f_17283(C_word c,C_word *av) C_noret;
C_noret_decl(f_17285)
static void C_fcall f_17285(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_17291)
static void C_fcall f_17291(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_17305)
static void C_ccall f_17305(C_word c,C_word *av) C_noret;
C_noret_decl(f_17311)
static void C_ccall f_17311(C_word c,C_word *av) C_noret;
C_noret_decl(f_17325)
static void C_ccall f_17325(C_word c,C_word *av) C_noret;
C_noret_decl(f_17332)
static void C_ccall f_17332(C_word c,C_word *av) C_noret;
C_noret_decl(f_17339)
static void C_ccall f_17339(C_word c,C_word *av) C_noret;
C_noret_decl(f_17354)
static void C_ccall f_17354(C_word c,C_word *av) C_noret;
C_noret_decl(f_17369)
static void C_ccall f_17369(C_word c,C_word *av) C_noret;
C_noret_decl(f_17381)
static void C_ccall f_17381(C_word c,C_word *av) C_noret;
C_noret_decl(f_17387)
static void C_ccall f_17387(C_word c,C_word *av) C_noret;
C_noret_decl(f_17391)
static void C_ccall f_17391(C_word c,C_word *av) C_noret;
C_noret_decl(f_17408)
static void C_ccall f_17408(C_word c,C_word *av) C_noret;
C_noret_decl(f_17413)
static void C_ccall f_17413(C_word c,C_word *av) C_noret;
C_noret_decl(f_17420)
static void C_ccall f_17420(C_word c,C_word *av) C_noret;
C_noret_decl(f_17422)
static C_word C_fcall f_17422(C_word *a,C_word t0);
C_noret_decl(f_17429)
static void C_fcall f_17429(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_17433)
static void C_ccall f_17433(C_word c,C_word *av) C_noret;
C_noret_decl(f_17436)
static void C_ccall f_17436(C_word c,C_word *av) C_noret;
C_noret_decl(f_17439)
static void C_ccall f_17439(C_word c,C_word *av) C_noret;
C_noret_decl(f_17451)
static void C_fcall f_17451(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17466)
static void C_ccall f_17466(C_word c,C_word *av) C_noret;
C_noret_decl(f_17497)
static void C_ccall f_17497(C_word c,C_word *av) C_noret;
C_noret_decl(f_17527)
static void C_ccall f_17527(C_word c,C_word *av) C_noret;
C_noret_decl(f_17530)
static void C_ccall f_17530(C_word c,C_word *av) C_noret;
C_noret_decl(f_17548)
static void C_ccall f_17548(C_word c,C_word *av) C_noret;
C_noret_decl(f_17551)
static void C_ccall f_17551(C_word c,C_word *av) C_noret;
C_noret_decl(f_17561)
static void C_fcall f_17561(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17568)
static void C_ccall f_17568(C_word c,C_word *av) C_noret;
C_noret_decl(f_17596)
static void C_ccall f_17596(C_word c,C_word *av) C_noret;
C_noret_decl(f_17606)
static void C_ccall f_17606(C_word c,C_word *av) C_noret;
C_noret_decl(f_17610)
static void C_ccall f_17610(C_word c,C_word *av) C_noret;
C_noret_decl(f_17614)
static void C_ccall f_17614(C_word c,C_word *av) C_noret;
C_noret_decl(f_17618)
static void C_ccall f_17618(C_word c,C_word *av) C_noret;
C_noret_decl(f_17622)
static void C_ccall f_17622(C_word c,C_word *av) C_noret;
C_noret_decl(f_17626)
static void C_ccall f_17626(C_word c,C_word *av) C_noret;
C_noret_decl(f_17638)
static void C_ccall f_17638(C_word c,C_word *av) C_noret;
C_noret_decl(f_17643)
static void C_fcall f_17643(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_17661)
static void C_ccall f_17661(C_word c,C_word *av) C_noret;
C_noret_decl(f_17670)
static void C_ccall f_17670(C_word c,C_word *av) C_noret;
C_noret_decl(f_17673)
static void C_ccall f_17673(C_word c,C_word *av) C_noret;
C_noret_decl(f_17676)
static void C_ccall f_17676(C_word c,C_word *av) C_noret;
C_noret_decl(f_17701)
static void C_ccall f_17701(C_word c,C_word *av) C_noret;
C_noret_decl(f_17703)
static void C_fcall f_17703(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17713)
static void C_ccall f_17713(C_word c,C_word *av) C_noret;
C_noret_decl(f_17717)
static void C_ccall f_17717(C_word c,C_word *av) C_noret;
C_noret_decl(f_17726)
static void C_ccall f_17726(C_word c,C_word *av) C_noret;
C_noret_decl(f_17728)
static void C_fcall f_17728(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17738)
static void C_ccall f_17738(C_word c,C_word *av) C_noret;
C_noret_decl(f_17770)
static void C_ccall f_17770(C_word c,C_word *av) C_noret;
C_noret_decl(f_17777)
static void C_ccall f_17777(C_word c,C_word *av) C_noret;
C_noret_decl(f_17778)
static void C_fcall f_17778(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17782)
static void C_ccall f_17782(C_word c,C_word *av) C_noret;
C_noret_decl(f_17791)
static void C_ccall f_17791(C_word c,C_word *av) C_noret;
C_noret_decl(f_17794)
static void C_ccall f_17794(C_word c,C_word *av) C_noret;
C_noret_decl(f_17797)
static void C_ccall f_17797(C_word c,C_word *av) C_noret;
C_noret_decl(f_17800)
static void C_fcall f_17800(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17803)
static void C_fcall f_17803(C_word t0,C_word t1) C_noret;
C_noret_decl(f_17806)
static void C_ccall f_17806(C_word c,C_word *av) C_noret;
C_noret_decl(f_17822)
static void C_ccall f_17822(C_word c,C_word *av) C_noret;
C_noret_decl(f_17830)
static void C_ccall f_17830(C_word c,C_word *av) C_noret;
C_noret_decl(f_17848)
static void C_ccall f_17848(C_word c,C_word *av) C_noret;
C_noret_decl(f_17854)
static void C_ccall f_17854(C_word c,C_word *av) C_noret;
C_noret_decl(f_17863)
static void C_fcall f_17863(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_17873)
static void C_ccall f_17873(C_word c,C_word *av) C_noret;
C_noret_decl(f_17887)
static void C_ccall f_17887(C_word c,C_word *av) C_noret;
C_noret_decl(f_17908)
static void C_ccall f_17908(C_word c,C_word *av) C_noret;
C_noret_decl(f_17911)
static void C_ccall f_17911(C_word c,C_word *av) C_noret;
C_noret_decl(f_17914)
static void C_ccall f_17914(C_word c,C_word *av) C_noret;
C_noret_decl(f_17917)
static void C_ccall f_17917(C_word c,C_word *av) C_noret;
C_noret_decl(f_17920)
static void C_ccall f_17920(C_word c,C_word *av) C_noret;
C_noret_decl(f_17923)
static void C_ccall f_17923(C_word c,C_word *av) C_noret;
C_noret_decl(f_17959)
static void C_ccall f_17959(C_word c,C_word *av) C_noret;
C_noret_decl(f_17965)
static void C_ccall f_17965(C_word c,C_word *av) C_noret;
C_noret_decl(f_18005)
static void C_ccall f_18005(C_word c,C_word *av) C_noret;
C_noret_decl(f_18011)
static void C_ccall f_18011(C_word c,C_word *av) C_noret;
C_noret_decl(f_18100)
static void C_fcall f_18100(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_18102)
static void C_fcall f_18102(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18114)
static void C_fcall f_18114(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18124)
static void C_ccall f_18124(C_word c,C_word *av) C_noret;
C_noret_decl(f_18137)
static void C_fcall f_18137(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_18141)
static void C_ccall f_18141(C_word c,C_word *av) C_noret;
C_noret_decl(f_18150)
static void C_fcall f_18150(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_18154)
static void C_ccall f_18154(C_word c,C_word *av) C_noret;
C_noret_decl(f_18163)
static void C_fcall f_18163(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_18170)
static void C_ccall f_18170(C_word c,C_word *av) C_noret;
C_noret_decl(f_18176)
static void C_ccall f_18176(C_word c,C_word *av) C_noret;
C_noret_decl(f_18188)
static void C_fcall f_18188(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18212)
static void C_ccall f_18212(C_word c,C_word *av) C_noret;
C_noret_decl(f_18216)
static void C_ccall f_18216(C_word c,C_word *av) C_noret;
C_noret_decl(f_18235)
static void C_ccall f_18235(C_word c,C_word *av) C_noret;
C_noret_decl(f_18239)
static void C_ccall f_18239(C_word c,C_word *av) C_noret;
C_noret_decl(f_18242)
static void C_ccall f_18242(C_word c,C_word *av) C_noret;
C_noret_decl(f_18245)
static void C_fcall f_18245(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18249)
static void C_fcall f_18249(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_18253)
static void C_ccall f_18253(C_word c,C_word *av) C_noret;
C_noret_decl(f_18256)
static void C_ccall f_18256(C_word c,C_word *av) C_noret;
C_noret_decl(f_18273)
static void C_fcall f_18273(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18276)
static void C_ccall f_18276(C_word c,C_word *av) C_noret;
C_noret_decl(f_18279)
static void C_ccall f_18279(C_word c,C_word *av) C_noret;
C_noret_decl(f_18318)
static void C_fcall f_18318(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18321)
static void C_ccall f_18321(C_word c,C_word *av) C_noret;
C_noret_decl(f_18327)
static void C_ccall f_18327(C_word c,C_word *av) C_noret;
C_noret_decl(f_18354)
static void C_ccall f_18354(C_word c,C_word *av) C_noret;
C_noret_decl(f_18361)
static void C_ccall f_18361(C_word c,C_word *av) C_noret;
C_noret_decl(f_18367)
static void C_ccall f_18367(C_word c,C_word *av) C_noret;
C_noret_decl(f_18377)
static void C_ccall f_18377(C_word c,C_word *av) C_noret;
C_noret_decl(f_18380)
static void C_ccall f_18380(C_word c,C_word *av) C_noret;
C_noret_decl(f_18387)
static void C_ccall f_18387(C_word c,C_word *av) C_noret;
C_noret_decl(f_18391)
static void C_ccall f_18391(C_word c,C_word *av) C_noret;
C_noret_decl(f_18395)
static void C_ccall f_18395(C_word c,C_word *av) C_noret;
C_noret_decl(f_18409)
static void C_ccall f_18409(C_word c,C_word *av) C_noret;
C_noret_decl(f_18413)
static void C_ccall f_18413(C_word c,C_word *av) C_noret;
C_noret_decl(f_18417)
static void C_ccall f_18417(C_word c,C_word *av) C_noret;
C_noret_decl(f_18420)
static void C_ccall f_18420(C_word c,C_word *av) C_noret;
C_noret_decl(f_18440)
static void C_ccall f_18440(C_word c,C_word *av) C_noret;
C_noret_decl(f_18444)
static void C_ccall f_18444(C_word c,C_word *av) C_noret;
C_noret_decl(f_18447)
static void C_ccall f_18447(C_word c,C_word *av) C_noret;
C_noret_decl(f_18467)
static void C_ccall f_18467(C_word c,C_word *av) C_noret;
C_noret_decl(f_18471)
static void C_ccall f_18471(C_word c,C_word *av) C_noret;
C_noret_decl(f_18473)
static void C_fcall f_18473(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_18494)
static void C_ccall f_18494(C_word c,C_word *av) C_noret;
C_noret_decl(f_18498)
static void C_ccall f_18498(C_word c,C_word *av) C_noret;
C_noret_decl(f_18501)
static void C_ccall f_18501(C_word c,C_word *av) C_noret;
C_noret_decl(f_18505)
static void C_ccall f_18505(C_word c,C_word *av) C_noret;
C_noret_decl(f_18511)
static void C_ccall f_18511(C_word c,C_word *av) C_noret;
C_noret_decl(f_18517)
static C_word C_fcall f_18517(C_word t0,C_word t1);
C_noret_decl(f_18660)
static void C_ccall f_18660(C_word c,C_word *av) C_noret;
C_noret_decl(f_18663)
static void C_ccall f_18663(C_word c,C_word *av) C_noret;
C_noret_decl(f_18666)
static void C_fcall f_18666(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18669)
static void C_ccall f_18669(C_word c,C_word *av) C_noret;
C_noret_decl(f_18672)
static void C_fcall f_18672(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18675)
static void C_ccall f_18675(C_word c,C_word *av) C_noret;
C_noret_decl(f_18678)
static void C_fcall f_18678(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18681)
static void C_ccall f_18681(C_word c,C_word *av) C_noret;
C_noret_decl(f_18687)
static void C_fcall f_18687(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18690)
static void C_ccall f_18690(C_word c,C_word *av) C_noret;
C_noret_decl(f_18708)
static void C_fcall f_18708(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18711)
static void C_ccall f_18711(C_word c,C_word *av) C_noret;
C_noret_decl(f_18729)
static void C_fcall f_18729(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18742)
static void C_ccall f_18742(C_word c,C_word *av) C_noret;
C_noret_decl(f_18766)
static void C_ccall f_18766(C_word c,C_word *av) C_noret;
C_noret_decl(f_18770)
static void C_ccall f_18770(C_word c,C_word *av) C_noret;
C_noret_decl(f_18774)
static void C_ccall f_18774(C_word c,C_word *av) C_noret;
C_noret_decl(f_18795)
static void C_ccall f_18795(C_word c,C_word *av) C_noret;
C_noret_decl(f_18801)
static void C_ccall f_18801(C_word c,C_word *av) C_noret;
C_noret_decl(f_18812)
static void C_ccall f_18812(C_word c,C_word *av) C_noret;
C_noret_decl(f_18830)
static void C_fcall f_18830(C_word t0,C_word t1) C_noret;
C_noret_decl(f_18842)
static void C_ccall f_18842(C_word c,C_word *av) C_noret;
C_noret_decl(f_18864)
static void C_ccall f_18864(C_word c,C_word *av) C_noret;
C_noret_decl(f_18868)
static void C_ccall f_18868(C_word c,C_word *av) C_noret;
C_noret_decl(f_18872)
static void C_ccall f_18872(C_word c,C_word *av) C_noret;
C_noret_decl(f_18882)
static void C_ccall f_18882(C_word c,C_word *av) C_noret;
C_noret_decl(f_18886)
static void C_ccall f_18886(C_word c,C_word *av) C_noret;
C_noret_decl(f_18890)
static void C_ccall f_18890(C_word c,C_word *av) C_noret;
C_noret_decl(f_18911)
static void C_ccall f_18911(C_word c,C_word *av) C_noret;
C_noret_decl(f_18935)
static void C_ccall f_18935(C_word c,C_word *av) C_noret;
C_noret_decl(f_18945)
static void C_ccall f_18945(C_word c,C_word *av) C_noret;
C_noret_decl(f_18949)
static void C_ccall f_18949(C_word c,C_word *av) C_noret;
C_noret_decl(f_18956)
static void C_ccall f_18956(C_word c,C_word *av) C_noret;
C_noret_decl(f_18960)
static void C_ccall f_18960(C_word c,C_word *av) C_noret;
C_noret_decl(f_18969)
static void C_ccall f_18969(C_word c,C_word *av) C_noret;
C_noret_decl(f_18984)
static void C_ccall f_18984(C_word c,C_word *av) C_noret;
C_noret_decl(f_18988)
static void C_ccall f_18988(C_word c,C_word *av) C_noret;
C_noret_decl(f_18994)
static void C_fcall f_18994(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19007)
static void C_ccall f_19007(C_word c,C_word *av) C_noret;
C_noret_decl(f_19021)
static void C_ccall f_19021(C_word c,C_word *av) C_noret;
C_noret_decl(f_19022)
static void C_fcall f_19022(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19029)
static void C_fcall f_19029(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19032)
static void C_ccall f_19032(C_word c,C_word *av) C_noret;
C_noret_decl(f_19043)
static void C_ccall f_19043(C_word c,C_word *av) C_noret;
C_noret_decl(f_19047)
static void C_ccall f_19047(C_word c,C_word *av) C_noret;
C_noret_decl(f_19055)
static void C_fcall f_19055(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19065)
static void C_ccall f_19065(C_word c,C_word *av) C_noret;
C_noret_decl(f_19086)
static void C_ccall f_19086(C_word c,C_word *av) C_noret;
C_noret_decl(f_19089)
static void C_fcall f_19089(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19098)
static void C_ccall f_19098(C_word c,C_word *av) C_noret;
C_noret_decl(f_19111)
static void C_ccall f_19111(C_word c,C_word *av) C_noret;
C_noret_decl(f_19118)
static void C_ccall f_19118(C_word c,C_word *av) C_noret;
C_noret_decl(f_19121)
static void C_ccall f_19121(C_word c,C_word *av) C_noret;
C_noret_decl(f_19127)
static void C_ccall f_19127(C_word c,C_word *av) C_noret;
C_noret_decl(f_19159)
static void C_ccall f_19159(C_word c,C_word *av) C_noret;
C_noret_decl(f_19167)
static void C_ccall f_19167(C_word c,C_word *av) C_noret;
C_noret_decl(f_19169)
static void C_ccall f_19169(C_word c,C_word *av) C_noret;
C_noret_decl(f_19175)
static void C_ccall f_19175(C_word c,C_word *av) C_noret;
C_noret_decl(f_19187)
static void C_ccall f_19187(C_word c,C_word *av) C_noret;
C_noret_decl(f_19193)
static void C_ccall f_19193(C_word c,C_word *av) C_noret;
C_noret_decl(f_19204)
static void C_ccall f_19204(C_word c,C_word *av) C_noret;
C_noret_decl(f_19210)
static void C_ccall f_19210(C_word c,C_word *av) C_noret;
C_noret_decl(f_19214)
static void C_fcall f_19214(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19223)
static void C_ccall f_19223(C_word c,C_word *av) C_noret;
C_noret_decl(f_19231)
static void C_ccall f_19231(C_word c,C_word *av) C_noret;
C_noret_decl(f_19248)
static void C_ccall f_19248(C_word c,C_word *av) C_noret;
C_noret_decl(f_19255)
static void C_ccall f_19255(C_word c,C_word *av) C_noret;
C_noret_decl(f_19258)
static void C_ccall f_19258(C_word c,C_word *av) C_noret;
C_noret_decl(f_19262)
static void C_fcall f_19262(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19269)
static void C_ccall f_19269(C_word c,C_word *av) C_noret;
C_noret_decl(f_19275)
static void C_ccall f_19275(C_word c,C_word *av) C_noret;
C_noret_decl(f_19281)
static void C_ccall f_19281(C_word c,C_word *av) C_noret;
C_noret_decl(f_19299)
static void C_fcall f_19299(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19306)
static void C_ccall f_19306(C_word c,C_word *av) C_noret;
C_noret_decl(f_19312)
static void C_ccall f_19312(C_word c,C_word *av) C_noret;
C_noret_decl(f_19315)
static void C_ccall f_19315(C_word c,C_word *av) C_noret;
C_noret_decl(f_19318)
static void C_ccall f_19318(C_word c,C_word *av) C_noret;
C_noret_decl(f_19321)
static void C_ccall f_19321(C_word c,C_word *av) C_noret;
C_noret_decl(f_19355)
static void C_ccall f_19355(C_word c,C_word *av) C_noret;
C_noret_decl(f_19359)
static void C_ccall f_19359(C_word c,C_word *av) C_noret;
C_noret_decl(f_19363)
static void C_ccall f_19363(C_word c,C_word *av) C_noret;
C_noret_decl(f_19367)
static void C_ccall f_19367(C_word c,C_word *av) C_noret;
C_noret_decl(f_19373)
static void C_fcall f_19373(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19377)
static void C_fcall f_19377(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19395)
static void C_ccall f_19395(C_word c,C_word *av) C_noret;
C_noret_decl(f_19399)
static void C_ccall f_19399(C_word c,C_word *av) C_noret;
C_noret_decl(f_19407)
static void C_fcall f_19407(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19417)
static void C_ccall f_19417(C_word c,C_word *av) C_noret;
C_noret_decl(f_19442)
static C_word C_fcall f_19442(C_word t0,C_word t1);
C_noret_decl(f_19469)
static void C_ccall f_19469(C_word c,C_word *av) C_noret;
C_noret_decl(f_19472)
static void C_fcall f_19472(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19478)
static void C_fcall f_19478(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_19483)
static void C_ccall f_19483(C_word c,C_word *av) C_noret;
C_noret_decl(f_19488)
static void C_fcall f_19488(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19495)
static void C_ccall f_19495(C_word c,C_word *av) C_noret;
C_noret_decl(f_19497)
static void C_fcall f_19497(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_19501)
static void C_ccall f_19501(C_word c,C_word *av) C_noret;
C_noret_decl(f_19504)
static void C_ccall f_19504(C_word c,C_word *av) C_noret;
C_noret_decl(f_19507)
static void C_ccall f_19507(C_word c,C_word *av) C_noret;
C_noret_decl(f_19537)
static void C_fcall f_19537(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19546)
static void C_ccall f_19546(C_word c,C_word *av) C_noret;
C_noret_decl(f_19549)
static void C_ccall f_19549(C_word c,C_word *av) C_noret;
C_noret_decl(f_19559)
static void C_ccall f_19559(C_word c,C_word *av) C_noret;
C_noret_decl(f_19563)
static void C_ccall f_19563(C_word c,C_word *av) C_noret;
C_noret_decl(f_19597)
static void C_ccall f_19597(C_word c,C_word *av) C_noret;
C_noret_decl(f_19631)
static void C_ccall f_19631(C_word c,C_word *av) C_noret;
C_noret_decl(f_19639)
static void C_fcall f_19639(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19649)
static void C_ccall f_19649(C_word c,C_word *av) C_noret;
C_noret_decl(f_19651)
static void C_fcall f_19651(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19676)
static void C_ccall f_19676(C_word c,C_word *av) C_noret;
C_noret_decl(f_19707)
static void C_ccall f_19707(C_word c,C_word *av) C_noret;
C_noret_decl(f_19713)
static void C_fcall f_19713(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19716)
static void C_ccall f_19716(C_word c,C_word *av) C_noret;
C_noret_decl(f_19725)
static void C_ccall f_19725(C_word c,C_word *av) C_noret;
C_noret_decl(f_19728)
static void C_ccall f_19728(C_word c,C_word *av) C_noret;
C_noret_decl(f_19731)
static void C_fcall f_19731(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19737)
static void C_fcall f_19737(C_word t0,C_word t1) C_noret;
C_noret_decl(f_19740)
static void C_ccall f_19740(C_word c,C_word *av) C_noret;
C_noret_decl(f_19743)
static void C_ccall f_19743(C_word c,C_word *av) C_noret;
C_noret_decl(f_19756)
static void C_ccall f_19756(C_word c,C_word *av) C_noret;
C_noret_decl(f_19775)
static void C_ccall f_19775(C_word c,C_word *av) C_noret;
C_noret_decl(f_19793)
static void C_ccall f_19793(C_word c,C_word *av) C_noret;
C_noret_decl(f_19811)
static void C_ccall f_19811(C_word c,C_word *av) C_noret;
C_noret_decl(f_19821)
static void C_ccall f_19821(C_word c,C_word *av) C_noret;
C_noret_decl(f_19825)
static void C_ccall f_19825(C_word c,C_word *av) C_noret;
C_noret_decl(f_19833)
static void C_ccall f_19833(C_word c,C_word *av) C_noret;
C_noret_decl(f_19837)
static void C_ccall f_19837(C_word c,C_word *av) C_noret;
C_noret_decl(f_19841)
static void C_ccall f_19841(C_word c,C_word *av) C_noret;
C_noret_decl(f_19845)
static void C_ccall f_19845(C_word c,C_word *av) C_noret;
C_noret_decl(f_19873)
static void C_ccall f_19873(C_word c,C_word *av) C_noret;
C_noret_decl(f_19880)
static void C_ccall f_19880(C_word c,C_word *av) C_noret;
C_noret_decl(f_19885)
static void C_ccall f_19885(C_word c,C_word *av) C_noret;
C_noret_decl(f_19890)
static void C_ccall f_19890(C_word c,C_word *av) C_noret;
C_noret_decl(f_19894)
static void C_ccall f_19894(C_word c,C_word *av) C_noret;
C_noret_decl(f_19897)
static void C_ccall f_19897(C_word c,C_word *av) C_noret;
C_noret_decl(f_19900)
static void C_ccall f_19900(C_word c,C_word *av) C_noret;
C_noret_decl(f_19911)
static void C_ccall f_19911(C_word c,C_word *av) C_noret;
C_noret_decl(f_19917)
static void C_ccall f_19917(C_word c,C_word *av) C_noret;
C_noret_decl(f_19931)
static void C_fcall f_19931(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19941)
static void C_ccall f_19941(C_word c,C_word *av) C_noret;
C_noret_decl(f_19943)
static void C_fcall f_19943(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_19968)
static void C_ccall f_19968(C_word c,C_word *av) C_noret;
C_noret_decl(f_20028)
static void C_fcall f_20028(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_20032)
static void C_ccall f_20032(C_word c,C_word *av) C_noret;
C_noret_decl(f_20035)
static void C_ccall f_20035(C_word c,C_word *av) C_noret;
C_noret_decl(f_20038)
static void C_ccall f_20038(C_word c,C_word *av) C_noret;
C_noret_decl(f_20047)
static void C_fcall f_20047(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20059)
static void C_ccall f_20059(C_word c,C_word *av) C_noret;
C_noret_decl(f_20065)
static void C_ccall f_20065(C_word c,C_word *av) C_noret;
C_noret_decl(f_20081)
static void C_fcall f_20081(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20084)
static void C_ccall f_20084(C_word c,C_word *av) C_noret;
C_noret_decl(f_20087)
static void C_ccall f_20087(C_word c,C_word *av) C_noret;
C_noret_decl(f_20090)
static void C_ccall f_20090(C_word c,C_word *av) C_noret;
C_noret_decl(f_20103)
static void C_ccall f_20103(C_word c,C_word *av) C_noret;
C_noret_decl(f_20116)
static void C_ccall f_20116(C_word c,C_word *av) C_noret;
C_noret_decl(f_20126)
static void C_ccall f_20126(C_word c,C_word *av) C_noret;
C_noret_decl(f_20167)
static void C_fcall f_20167(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20174)
static void C_ccall f_20174(C_word c,C_word *av) C_noret;
C_noret_decl(f_20178)
static void C_ccall f_20178(C_word c,C_word *av) C_noret;
C_noret_decl(f_20190)
static void C_ccall f_20190(C_word c,C_word *av) C_noret;
C_noret_decl(f_20193)
static void C_ccall f_20193(C_word c,C_word *av) C_noret;
C_noret_decl(f_20211)
static void C_ccall f_20211(C_word c,C_word *av) C_noret;
C_noret_decl(f_20215)
static void C_ccall f_20215(C_word c,C_word *av) C_noret;
C_noret_decl(f_20227)
static void C_ccall f_20227(C_word c,C_word *av) C_noret;
C_noret_decl(f_20231)
static void C_ccall f_20231(C_word c,C_word *av) C_noret;
C_noret_decl(f_20243)
static void C_ccall f_20243(C_word c,C_word *av) C_noret;
C_noret_decl(f_20254)
static void C_ccall f_20254(C_word c,C_word *av) C_noret;
C_noret_decl(f_20271)
static void C_ccall f_20271(C_word c,C_word *av) C_noret;
C_noret_decl(f_20275)
static void C_ccall f_20275(C_word c,C_word *av) C_noret;
C_noret_decl(f_20287)
static void C_ccall f_20287(C_word c,C_word *av) C_noret;
C_noret_decl(f_20296)
static void C_ccall f_20296(C_word c,C_word *av) C_noret;
C_noret_decl(f_20299)
static void C_ccall f_20299(C_word c,C_word *av) C_noret;
C_noret_decl(f_20305)
static void C_ccall f_20305(C_word c,C_word *av) C_noret;
C_noret_decl(f_20311)
static void C_ccall f_20311(C_word c,C_word *av) C_noret;
C_noret_decl(f_20317)
static void C_fcall f_20317(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20320)
static void C_ccall f_20320(C_word c,C_word *av) C_noret;
C_noret_decl(f_20335)
static void C_ccall f_20335(C_word c,C_word *av) C_noret;
C_noret_decl(f_20340)
static void C_fcall f_20340(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20348)
static void C_ccall f_20348(C_word c,C_word *av) C_noret;
C_noret_decl(f_20354)
static void C_ccall f_20354(C_word c,C_word *av) C_noret;
C_noret_decl(f_20357)
static void C_ccall f_20357(C_word c,C_word *av) C_noret;
C_noret_decl(f_20368)
static void C_ccall f_20368(C_word c,C_word *av) C_noret;
C_noret_decl(f_20372)
static void C_ccall f_20372(C_word c,C_word *av) C_noret;
C_noret_decl(f_20376)
static void C_ccall f_20376(C_word c,C_word *av) C_noret;
C_noret_decl(f_20384)
static void C_ccall f_20384(C_word c,C_word *av) C_noret;
C_noret_decl(f_20391)
static void C_ccall f_20391(C_word c,C_word *av) C_noret;
C_noret_decl(f_20397)
static void C_fcall f_20397(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20422)
static void C_ccall f_20422(C_word c,C_word *av) C_noret;
C_noret_decl(f_20441)
static void C_ccall f_20441(C_word c,C_word *av) C_noret;
C_noret_decl(f_20444)
static void C_ccall f_20444(C_word c,C_word *av) C_noret;
C_noret_decl(f_20454)
static void C_ccall f_20454(C_word c,C_word *av) C_noret;
C_noret_decl(f_20471)
static void C_ccall f_20471(C_word c,C_word *av) C_noret;
C_noret_decl(f_20481)
static void C_ccall f_20481(C_word c,C_word *av) C_noret;
C_noret_decl(f_20483)
static void C_fcall f_20483(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20509)
static void C_ccall f_20509(C_word c,C_word *av) C_noret;
C_noret_decl(f_20519)
static void C_fcall f_20519(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20544)
static void C_ccall f_20544(C_word c,C_word *av) C_noret;
C_noret_decl(f_20571)
static void C_ccall f_20571(C_word c,C_word *av) C_noret;
C_noret_decl(f_20576)
static C_word C_fcall f_20576(C_word t0,C_word t1);
C_noret_decl(f_20592)
static void C_ccall f_20592(C_word c,C_word *av) C_noret;
C_noret_decl(f_20607)
static void C_fcall f_20607(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20650)
static void C_ccall f_20650(C_word c,C_word *av) C_noret;
C_noret_decl(f_20656)
static void C_ccall f_20656(C_word c,C_word *av) C_noret;
C_noret_decl(f_20684)
static void C_fcall f_20684(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_20732)
static void C_fcall f_20732(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20757)
static void C_ccall f_20757(C_word c,C_word *av) C_noret;
C_noret_decl(f_20766)
static void C_ccall f_20766(C_word c,C_word *av) C_noret;
C_noret_decl(f_20785)
static void C_ccall f_20785(C_word c,C_word *av) C_noret;
C_noret_decl(f_20788)
static void C_ccall f_20788(C_word c,C_word *av) C_noret;
C_noret_decl(f_20791)
static void C_fcall f_20791(C_word t0,C_word t1) C_noret;
C_noret_decl(f_20794)
static void C_ccall f_20794(C_word c,C_word *av) C_noret;
C_noret_decl(f_20798)
static void C_fcall f_20798(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_20805)
static void C_ccall f_20805(C_word c,C_word *av) C_noret;
C_noret_decl(f_20820)
static void C_ccall f_20820(C_word c,C_word *av) C_noret;
C_noret_decl(f_20824)
static void C_ccall f_20824(C_word c,C_word *av) C_noret;
C_noret_decl(f_20836)
static void C_ccall f_20836(C_word c,C_word *av) C_noret;
C_noret_decl(f_20857)
static void C_ccall f_20857(C_word c,C_word *av) C_noret;
C_noret_decl(f_20861)
static void C_ccall f_20861(C_word c,C_word *av) C_noret;
C_noret_decl(f_20872)
static void C_ccall f_20872(C_word c,C_word *av) C_noret;
C_noret_decl(f_20887)
static void C_ccall f_20887(C_word c,C_word *av) C_noret;
C_noret_decl(f_20891)
static void C_ccall f_20891(C_word c,C_word *av) C_noret;
C_noret_decl(f_20906)
static void C_ccall f_20906(C_word c,C_word *av) C_noret;
C_noret_decl(f_20939)
static void C_ccall f_20939(C_word c,C_word *av) C_noret;
C_noret_decl(f_20960)
static void C_ccall f_20960(C_word c,C_word *av) C_noret;
C_noret_decl(f_20971)
static void C_ccall f_20971(C_word c,C_word *av) C_noret;
C_noret_decl(f_20975)
static void C_ccall f_20975(C_word c,C_word *av) C_noret;
C_noret_decl(f_21214)
static void C_fcall f_21214(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_21219)
static void C_fcall f_21219(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21231)
static void C_fcall f_21231(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21256)
static void C_ccall f_21256(C_word c,C_word *av) C_noret;
C_noret_decl(f_21265)
static void C_fcall f_21265(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21272)
static void C_ccall f_21272(C_word c,C_word *av) C_noret;
C_noret_decl(f_21276)
static void C_fcall f_21276(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21292)
static void C_ccall f_21292(C_word c,C_word *av) C_noret;
C_noret_decl(f_21303)
static void C_ccall f_21303(C_word c,C_word *av) C_noret;
C_noret_decl(f_21306)
static void C_ccall f_21306(C_word c,C_word *av) C_noret;
C_noret_decl(f_21309)
static void C_ccall f_21309(C_word c,C_word *av) C_noret;
C_noret_decl(f_21312)
static void C_ccall f_21312(C_word c,C_word *av) C_noret;
C_noret_decl(f_21315)
static void C_ccall f_21315(C_word c,C_word *av) C_noret;
C_noret_decl(f_21318)
static void C_ccall f_21318(C_word c,C_word *av) C_noret;
C_noret_decl(f_21321)
static void C_ccall f_21321(C_word c,C_word *av) C_noret;
C_noret_decl(f_21346)
static void C_ccall f_21346(C_word c,C_word *av) C_noret;
C_noret_decl(f_21355)
static void C_ccall f_21355(C_word c,C_word *av) C_noret;
C_noret_decl(f_21373)
static void C_ccall f_21373(C_word c,C_word *av) C_noret;
C_noret_decl(f_21382)
static void C_ccall f_21382(C_word c,C_word *av) C_noret;
C_noret_decl(f_21391)
static void C_ccall f_21391(C_word c,C_word *av) C_noret;
C_noret_decl(f_21400)
static void C_ccall f_21400(C_word c,C_word *av) C_noret;
C_noret_decl(f_21409)
static void C_ccall f_21409(C_word c,C_word *av) C_noret;
C_noret_decl(f_21418)
static void C_ccall f_21418(C_word c,C_word *av) C_noret;
C_noret_decl(f_21436)
static void C_ccall f_21436(C_word c,C_word *av) C_noret;
C_noret_decl(f_21445)
static void C_ccall f_21445(C_word c,C_word *av) C_noret;
C_noret_decl(f_21454)
static void C_ccall f_21454(C_word c,C_word *av) C_noret;
C_noret_decl(f_21463)
static void C_ccall f_21463(C_word c,C_word *av) C_noret;
C_noret_decl(f_21472)
static void C_ccall f_21472(C_word c,C_word *av) C_noret;
C_noret_decl(f_21481)
static void C_ccall f_21481(C_word c,C_word *av) C_noret;
C_noret_decl(f_21490)
static void C_ccall f_21490(C_word c,C_word *av) C_noret;
C_noret_decl(f_21494)
static void C_ccall f_21494(C_word c,C_word *av) C_noret;
C_noret_decl(f_21496)
static void C_fcall f_21496(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5) C_noret;
C_noret_decl(f_21500)
static void C_ccall f_21500(C_word c,C_word *av) C_noret;
C_noret_decl(f_21504)
static void C_fcall f_21504(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_21529)
static void C_ccall f_21529(C_word c,C_word *av) C_noret;
C_noret_decl(f_21540)
static void C_ccall f_21540(C_word c,C_word *av) C_noret;
C_noret_decl(f_21545)
static void C_fcall f_21545(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_21549)
static void C_ccall f_21549(C_word c,C_word *av) C_noret;
C_noret_decl(f_21552)
static void C_fcall f_21552(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21555)
static void C_fcall f_21555(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21566)
static void C_ccall f_21566(C_word c,C_word *av) C_noret;
C_noret_decl(f_21577)
static void C_ccall f_21577(C_word c,C_word *av) C_noret;
C_noret_decl(f_21584)
static void C_ccall f_21584(C_word c,C_word *av) C_noret;
C_noret_decl(f_21593)
static void C_ccall f_21593(C_word c,C_word *av) C_noret;
C_noret_decl(f_21601)
static void C_fcall f_21601(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_21605)
static void C_ccall f_21605(C_word c,C_word *av) C_noret;
C_noret_decl(f_21608)
static void C_ccall f_21608(C_word c,C_word *av) C_noret;
C_noret_decl(f_21611)
static void C_ccall f_21611(C_word c,C_word *av) C_noret;
C_noret_decl(f_21620)
static void C_fcall f_21620(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21648)
static void C_fcall f_21648(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21659)
static void C_ccall f_21659(C_word c,C_word *av) C_noret;
C_noret_decl(f_21679)
static void C_ccall f_21679(C_word c,C_word *av) C_noret;
C_noret_decl(f_21683)
static void C_ccall f_21683(C_word c,C_word *av) C_noret;
C_noret_decl(f_21704)
static void C_ccall f_21704(C_word c,C_word *av) C_noret;
C_noret_decl(f_21725)
static void C_ccall f_21725(C_word c,C_word *av) C_noret;
C_noret_decl(f_21742)
static void C_ccall f_21742(C_word c,C_word *av) C_noret;
C_noret_decl(f_21746)
static void C_ccall f_21746(C_word c,C_word *av) C_noret;
C_noret_decl(f_21767)
static void C_ccall f_21767(C_word c,C_word *av) C_noret;
C_noret_decl(f_21771)
static void C_ccall f_21771(C_word c,C_word *av) C_noret;
C_noret_decl(f_21775)
static void C_ccall f_21775(C_word c,C_word *av) C_noret;
C_noret_decl(f_21796)
static void C_ccall f_21796(C_word c,C_word *av) C_noret;
C_noret_decl(f_21821)
static void C_ccall f_21821(C_word c,C_word *av) C_noret;
C_noret_decl(f_21837)
static void C_ccall f_21837(C_word c,C_word *av) C_noret;
C_noret_decl(f_21847)
static void C_fcall f_21847(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21863)
static void C_ccall f_21863(C_word c,C_word *av) C_noret;
C_noret_decl(f_21867)
static void C_ccall f_21867(C_word c,C_word *av) C_noret;
C_noret_decl(f_21876)
static void C_ccall f_21876(C_word c,C_word *av) C_noret;
C_noret_decl(f_21917)
static void C_ccall f_21917(C_word c,C_word *av) C_noret;
C_noret_decl(f_21924)
static void C_ccall f_21924(C_word c,C_word *av) C_noret;
C_noret_decl(f_21927)
static void C_ccall f_21927(C_word c,C_word *av) C_noret;
C_noret_decl(f_21930)
static void C_ccall f_21930(C_word c,C_word *av) C_noret;
C_noret_decl(f_21933)
static void C_ccall f_21933(C_word c,C_word *av) C_noret;
C_noret_decl(f_21936)
static void C_ccall f_21936(C_word c,C_word *av) C_noret;
C_noret_decl(f_21944)
static void C_ccall f_21944(C_word c,C_word *av) C_noret;
C_noret_decl(f_21973)
static void C_ccall f_21973(C_word c,C_word *av) C_noret;
C_noret_decl(f_21980)
static void C_fcall f_21980(C_word t0,C_word t1) C_noret;
C_noret_decl(f_21984)
static void C_ccall f_21984(C_word c,C_word *av) C_noret;
C_noret_decl(f_21999)
static void C_ccall f_21999(C_word c,C_word *av) C_noret;
C_noret_decl(f_22022)
static void C_ccall f_22022(C_word c,C_word *av) C_noret;
C_noret_decl(f_22043)
static void C_ccall f_22043(C_word c,C_word *av) C_noret;
C_noret_decl(f_22052)
static void C_ccall f_22052(C_word c,C_word *av) C_noret;
C_noret_decl(f_22058)
static void C_ccall f_22058(C_word c,C_word *av) C_noret;
C_noret_decl(f_22083)
static void C_ccall f_22083(C_word c,C_word *av) C_noret;
C_noret_decl(f_22115)
static void C_ccall f_22115(C_word c,C_word *av) C_noret;
C_noret_decl(f_22119)
static void C_ccall f_22119(C_word c,C_word *av) C_noret;
C_noret_decl(f_22127)
static void C_ccall f_22127(C_word c,C_word *av) C_noret;
C_noret_decl(f_22135)
static void C_ccall f_22135(C_word c,C_word *av) C_noret;
C_noret_decl(f_22139)
static void C_ccall f_22139(C_word c,C_word *av) C_noret;
C_noret_decl(f_22149)
static void C_ccall f_22149(C_word c,C_word *av) C_noret;
C_noret_decl(f_22176)
static void C_ccall f_22176(C_word c,C_word *av) C_noret;
C_noret_decl(f_22180)
static void C_ccall f_22180(C_word c,C_word *av) C_noret;
C_noret_decl(f_22199)
static void C_ccall f_22199(C_word c,C_word *av) C_noret;
C_noret_decl(f_22203)
static void C_fcall f_22203(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22219)
static void C_ccall f_22219(C_word c,C_word *av) C_noret;
C_noret_decl(f_22233)
static void C_ccall f_22233(C_word c,C_word *av) C_noret;
C_noret_decl(f_22239)
static void C_ccall f_22239(C_word c,C_word *av) C_noret;
C_noret_decl(f_22242)
static void C_fcall f_22242(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22245)
static void C_fcall f_22245(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22264)
static void C_ccall f_22264(C_word c,C_word *av) C_noret;
C_noret_decl(f_22272)
static void C_ccall f_22272(C_word c,C_word *av) C_noret;
C_noret_decl(f_22313)
static void C_ccall f_22313(C_word c,C_word *av) C_noret;
C_noret_decl(f_22317)
static void C_ccall f_22317(C_word c,C_word *av) C_noret;
C_noret_decl(f_22332)
static void C_fcall f_22332(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22336)
static void C_ccall f_22336(C_word c,C_word *av) C_noret;
C_noret_decl(f_22339)
static void C_fcall f_22339(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22342)
static void C_fcall f_22342(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22349)
static void C_ccall f_22349(C_word c,C_word *av) C_noret;
C_noret_decl(f_22369)
static void C_ccall f_22369(C_word c,C_word *av) C_noret;
C_noret_decl(f_22373)
static void C_ccall f_22373(C_word c,C_word *av) C_noret;
C_noret_decl(f_22380)
static void C_fcall f_22380(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22417)
static void C_fcall f_22417(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22424)
static void C_ccall f_22424(C_word c,C_word *av) C_noret;
C_noret_decl(f_22444)
static void C_fcall f_22444(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22465)
static void C_fcall f_22465(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22468)
static void C_ccall f_22468(C_word c,C_word *av) C_noret;
C_noret_decl(f_22475)
static void C_ccall f_22475(C_word c,C_word *av) C_noret;
C_noret_decl(f_22479)
static void C_ccall f_22479(C_word c,C_word *av) C_noret;
C_noret_decl(f_22485)
static void C_ccall f_22485(C_word c,C_word *av) C_noret;
C_noret_decl(f_22488)
static void C_ccall f_22488(C_word c,C_word *av) C_noret;
C_noret_decl(f_22491)
static void C_ccall f_22491(C_word c,C_word *av) C_noret;
C_noret_decl(f_22494)
static void C_ccall f_22494(C_word c,C_word *av) C_noret;
C_noret_decl(f_22497)
static void C_ccall f_22497(C_word c,C_word *av) C_noret;
C_noret_decl(f_22501)
static void C_ccall f_22501(C_word c,C_word *av) C_noret;
C_noret_decl(f_22512)
static void C_ccall f_22512(C_word c,C_word *av) C_noret;
C_noret_decl(f_22523)
static void C_ccall f_22523(C_word c,C_word *av) C_noret;
C_noret_decl(f_22529)
static void C_ccall f_22529(C_word c,C_word *av) C_noret;
C_noret_decl(f_22543)
static void C_ccall f_22543(C_word c,C_word *av) C_noret;
C_noret_decl(f_22553)
static void C_ccall f_22553(C_word c,C_word *av) C_noret;
C_noret_decl(f_22565)
static void C_ccall f_22565(C_word c,C_word *av) C_noret;
C_noret_decl(f_22568)
static void C_ccall f_22568(C_word c,C_word *av) C_noret;
C_noret_decl(f_22571)
static void C_ccall f_22571(C_word c,C_word *av) C_noret;
C_noret_decl(f_22590)
static void C_ccall f_22590(C_word c,C_word *av) C_noret;
C_noret_decl(f_22602)
static void C_ccall f_22602(C_word c,C_word *av) C_noret;
C_noret_decl(f_22631)
static void C_ccall f_22631(C_word c,C_word *av) C_noret;
C_noret_decl(f_22642)
static void C_ccall f_22642(C_word c,C_word *av) C_noret;
C_noret_decl(f_22652)
static void C_fcall f_22652(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_22661)
static void C_ccall f_22661(C_word c,C_word *av) C_noret;
C_noret_decl(f_22672)
static void C_ccall f_22672(C_word c,C_word *av) C_noret;
C_noret_decl(f_22683)
static void C_ccall f_22683(C_word c,C_word *av) C_noret;
C_noret_decl(f_22686)
static void C_ccall f_22686(C_word c,C_word *av) C_noret;
C_noret_decl(f_22693)
static void C_ccall f_22693(C_word c,C_word *av) C_noret;
C_noret_decl(f_22703)
static void C_ccall f_22703(C_word c,C_word *av) C_noret;
C_noret_decl(f_22744)
static void C_ccall f_22744(C_word c,C_word *av) C_noret;
C_noret_decl(f_22811)
static void C_fcall f_22811(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6) C_noret;
C_noret_decl(f_22816)
static void C_fcall f_22816(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22828)
static void C_fcall f_22828(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22853)
static void C_ccall f_22853(C_word c,C_word *av) C_noret;
C_noret_decl(f_22862)
static void C_fcall f_22862(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22869)
static void C_ccall f_22869(C_word c,C_word *av) C_noret;
C_noret_decl(f_22894)
static C_word C_fcall f_22894(C_word t0,C_word t1);
C_noret_decl(f_22910)
static C_word C_fcall f_22910(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_22919)
static void C_fcall f_22919(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22923)
static void C_ccall f_22923(C_word c,C_word *av) C_noret;
C_noret_decl(f_22927)
static C_word C_fcall f_22927(C_word t0,C_word t1);
C_noret_decl(f_22945)
static void C_ccall f_22945(C_word c,C_word *av) C_noret;
C_noret_decl(f_22947)
static void C_fcall f_22947(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_22954)
static void C_ccall f_22954(C_word c,C_word *av) C_noret;
C_noret_decl(f_22961)
static void C_ccall f_22961(C_word c,C_word *av) C_noret;
C_noret_decl(f_22963)
static void C_fcall f_22963(C_word t0,C_word t1) C_noret;
C_noret_decl(f_22980)
static void C_ccall f_22980(C_word c,C_word *av) C_noret;
C_noret_decl(f_23026)
static void C_ccall f_23026(C_word c,C_word *av) C_noret;
C_noret_decl(f_23029)
static void C_ccall f_23029(C_word c,C_word *av) C_noret;
C_noret_decl(f_23032)
static void C_ccall f_23032(C_word c,C_word *av) C_noret;
C_noret_decl(f_23035)
static void C_ccall f_23035(C_word c,C_word *av) C_noret;
C_noret_decl(f_23038)
static void C_ccall f_23038(C_word c,C_word *av) C_noret;
C_noret_decl(f_23045)
static void C_ccall f_23045(C_word c,C_word *av) C_noret;
C_noret_decl(f_23049)
static void C_ccall f_23049(C_word c,C_word *av) C_noret;
C_noret_decl(f_23053)
static void C_ccall f_23053(C_word c,C_word *av) C_noret;
C_noret_decl(f_6259)
static void C_ccall f_6259(C_word c,C_word *av) C_noret;
C_noret_decl(f_6262)
static void C_ccall f_6262(C_word c,C_word *av) C_noret;
C_noret_decl(f_6265)
static void C_ccall f_6265(C_word c,C_word *av) C_noret;
C_noret_decl(f_6268)
static void C_ccall f_6268(C_word c,C_word *av) C_noret;
C_noret_decl(f_6271)
static void C_ccall f_6271(C_word c,C_word *av) C_noret;
C_noret_decl(f_6274)
static void C_ccall f_6274(C_word c,C_word *av) C_noret;
C_noret_decl(f_6277)
static void C_ccall f_6277(C_word c,C_word *av) C_noret;
C_noret_decl(f_6280)
static void C_ccall f_6280(C_word c,C_word *av) C_noret;
C_noret_decl(f_6283)
static void C_ccall f_6283(C_word c,C_word *av) C_noret;
C_noret_decl(f_6286)
static void C_ccall f_6286(C_word c,C_word *av) C_noret;
C_noret_decl(f_6472)
static void C_fcall f_6472(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_6486)
static void C_ccall f_6486(C_word c,C_word *av) C_noret;
C_noret_decl(f_6675)
static void C_fcall f_6675(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6681)
static void C_fcall f_6681(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6703)
static void C_ccall f_6703(C_word c,C_word *av) C_noret;
C_noret_decl(f_6715)
static C_word C_fcall f_6715(C_word t0,C_word t1);
C_noret_decl(f_6739)
static void C_fcall f_6739(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6745)
static void C_fcall f_6745(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6759)
static void C_ccall f_6759(C_word c,C_word *av) C_noret;
C_noret_decl(f_6767)
static void C_fcall f_6767(C_word t0,C_word t1) C_noret;
C_noret_decl(f_6773)
static void C_fcall f_6773(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6791)
static void C_ccall f_6791(C_word c,C_word *av) C_noret;
C_noret_decl(f_6795)
static void C_fcall f_6795(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6801)
static void C_fcall f_6801(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6814)
static void C_ccall f_6814(C_word c,C_word *av) C_noret;
C_noret_decl(f_6828)
static void C_ccall f_6828(C_word c,C_word *av) C_noret;
C_noret_decl(f_6876)
static void C_fcall f_6876(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6890)
static void C_ccall f_6890(C_word c,C_word *av) C_noret;
C_noret_decl(f_6903)
static void C_ccall f_6903(C_word c,C_word *av) C_noret;
C_noret_decl(f_6915)
static void C_fcall f_6915(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6924)
static void C_fcall f_6924(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6932)
static void C_fcall f_6932(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6939)
static void C_ccall f_6939(C_word c,C_word *av) C_noret;
C_noret_decl(f_6953)
static void C_ccall f_6953(C_word c,C_word *av) C_noret;
C_noret_decl(f_6968)
static void C_fcall f_6968(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_6976)
static void C_fcall f_6976(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_6980)
static void C_ccall f_6980(C_word c,C_word *av) C_noret;
C_noret_decl(f_6984)
static C_word C_fcall f_6984(C_word *a,C_word t0,C_word t1);
C_noret_decl(f_7002)
static void C_ccall f_7002(C_word c,C_word *av) C_noret;
C_noret_decl(f_7024)
static void C_fcall f_7024(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7041)
static void C_fcall f_7041(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7081)
static C_word C_fcall f_7081(C_word t0);
C_noret_decl(f_7106)
static void C_fcall f_7106(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7119)
static void C_ccall f_7119(C_word c,C_word *av) C_noret;
C_noret_decl(f_7134)
static void C_fcall f_7134(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7140)
static void C_fcall f_7140(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7175)
static void C_fcall f_7175(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7181)
static void C_fcall f_7181(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7203)
static void C_ccall f_7203(C_word c,C_word *av) C_noret;
C_noret_decl(f_7214)
static void C_ccall f_7214(C_word c,C_word *av) C_noret;
C_noret_decl(f_7220)
static void C_fcall f_7220(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7226)
static void C_fcall f_7226(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7251)
static void C_fcall f_7251(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7279)
static C_word C_fcall f_7279(C_word t0,C_word t1);
C_noret_decl(f_7293)
static void C_ccall f_7293(C_word c,C_word *av) C_noret;
C_noret_decl(f_7299)
static void C_fcall f_7299(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7305)
static void C_ccall f_7305(C_word c,C_word *av) C_noret;
C_noret_decl(f_7311)
static void C_ccall f_7311(C_word c,C_word *av) C_noret;
C_noret_decl(f_7518)
static void C_fcall f_7518(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7524)
static C_word C_fcall f_7524(C_word t0,C_word t1,C_word t2);
C_noret_decl(f_7558)
static C_word C_fcall f_7558(C_word t0,C_word t1,C_word t2);
C_noret_decl(f_7654)
static void C_ccall f_7654(C_word c,C_word *av) C_noret;
C_noret_decl(f_7658)
static void C_ccall f_7658(C_word c,C_word *av) C_noret;
C_noret_decl(f_7661)
static void C_ccall f_7661(C_word c,C_word *av) C_noret;
C_noret_decl(f_7664)
static void C_ccall f_7664(C_word c,C_word *av) C_noret;
C_noret_decl(f_7667)
static void C_ccall f_7667(C_word c,C_word *av) C_noret;
C_noret_decl(f_7670)
static void C_ccall f_7670(C_word c,C_word *av) C_noret;
C_noret_decl(f_7680)
static void C_ccall f_7680(C_word c,C_word *av) C_noret;
C_noret_decl(f_7687)
static void C_ccall f_7687(C_word c,C_word *av) C_noret;
C_noret_decl(f_7694)
static void C_ccall f_7694(C_word c,C_word *av) C_noret;
C_noret_decl(f_7696)
static void C_ccall f_7696(C_word c,C_word *av) C_noret;
C_noret_decl(f_7700)
static void C_ccall f_7700(C_word c,C_word *av) C_noret;
C_noret_decl(f_7705)
static void C_ccall f_7705(C_word c,C_word *av) C_noret;
C_noret_decl(f_7707)
static void C_fcall f_7707(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7749)
static void C_ccall f_7749(C_word c,C_word *av) C_noret;
C_noret_decl(f_7781)
static void C_fcall f_7781(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7791)
static void C_ccall f_7791(C_word c,C_word *av) C_noret;
C_noret_decl(f_7804)
static void C_ccall f_7804(C_word c,C_word *av) C_noret;
C_noret_decl(f_7807)
static void C_fcall f_7807(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7820)
static void C_fcall f_7820(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7842)
static void C_fcall f_7842(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7846)
static void C_ccall f_7846(C_word c,C_word *av) C_noret;
C_noret_decl(f_7854)
static void C_ccall f_7854(C_word c,C_word *av) C_noret;
C_noret_decl(f_7860)
static void C_ccall f_7860(C_word c,C_word *av) C_noret;
C_noret_decl(f_7867)
static void C_ccall f_7867(C_word c,C_word *av) C_noret;
C_noret_decl(f_7874)
static void C_ccall f_7874(C_word c,C_word *av) C_noret;
C_noret_decl(f_7876)
static void C_fcall f_7876(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7880)
static void C_ccall f_7880(C_word c,C_word *av) C_noret;
C_noret_decl(f_7883)
static void C_ccall f_7883(C_word c,C_word *av) C_noret;
C_noret_decl(f_7887)
static void C_ccall f_7887(C_word c,C_word *av) C_noret;
C_noret_decl(f_7889)
static void C_fcall f_7889(C_word t0,C_word t1) C_noret;
C_noret_decl(f_7891)
static void C_ccall f_7891(C_word c,C_word *av) C_noret;
C_noret_decl(f_7898)
static void C_ccall f_7898(C_word c,C_word *av) C_noret;
C_noret_decl(f_7901)
static void C_ccall f_7901(C_word c,C_word *av) C_noret;
C_noret_decl(f_7916)
static void C_fcall f_7916(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4) C_noret;
C_noret_decl(f_7920)
static void C_ccall f_7920(C_word c,C_word *av) C_noret;
C_noret_decl(f_7925)
static void C_ccall f_7925(C_word c,C_word *av) C_noret;
C_noret_decl(f_7930)
static void C_ccall f_7930(C_word c,C_word *av) C_noret;
C_noret_decl(f_7938)
static void C_ccall f_7938(C_word c,C_word *av) C_noret;
C_noret_decl(f_7940)
static void C_ccall f_7940(C_word c,C_word *av) C_noret;
C_noret_decl(f_7945)
static void C_fcall f_7945(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_7962)
static void C_fcall f_7962(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_7972)
static void C_ccall f_7972(C_word c,C_word *av) C_noret;
C_noret_decl(f_8011)
static void C_fcall f_8011(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8043)
static void C_ccall f_8043(C_word c,C_word *av) C_noret;
C_noret_decl(f_8054)
static void C_fcall f_8054(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7) C_noret;
C_noret_decl(f_8058)
static void C_ccall f_8058(C_word c,C_word *av) C_noret;
C_noret_decl(f_8061)
static void C_ccall f_8061(C_word c,C_word *av) C_noret;
C_noret_decl(f_8073)
static void C_ccall f_8073(C_word c,C_word *av) C_noret;
C_noret_decl(f_8077)
static void C_fcall f_8077(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8087)
static void C_ccall f_8087(C_word c,C_word *av) C_noret;
C_noret_decl(f_8091)
static void C_fcall f_8091(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8105)
static void C_fcall f_8105(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8112)
static void C_ccall f_8112(C_word c,C_word *av) C_noret;
C_noret_decl(f_8122)
static void C_ccall f_8122(C_word c,C_word *av) C_noret;
C_noret_decl(f_8126)
static void C_ccall f_8126(C_word c,C_word *av) C_noret;
C_noret_decl(f_8144)
static void C_fcall f_8144(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8151)
static void C_ccall f_8151(C_word c,C_word *av) C_noret;
C_noret_decl(f_8161)
static void C_ccall f_8161(C_word c,C_word *av) C_noret;
C_noret_decl(f_8165)
static void C_ccall f_8165(C_word c,C_word *av) C_noret;
C_noret_decl(f_8196)
static void C_fcall f_8196(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8207)
static void C_ccall f_8207(C_word c,C_word *av) C_noret;
C_noret_decl(f_8211)
static void C_ccall f_8211(C_word c,C_word *av) C_noret;
C_noret_decl(f_8214)
static void C_ccall f_8214(C_word c,C_word *av) C_noret;
C_noret_decl(f_8220)
static void C_ccall f_8220(C_word c,C_word *av) C_noret;
C_noret_decl(f_8223)
static void C_ccall f_8223(C_word c,C_word *av) C_noret;
C_noret_decl(f_8226)
static void C_ccall f_8226(C_word c,C_word *av) C_noret;
C_noret_decl(f_8250)
static void C_ccall f_8250(C_word c,C_word *av) C_noret;
C_noret_decl(f_8256)
static void C_ccall f_8256(C_word c,C_word *av) C_noret;
C_noret_decl(f_8259)
static void C_ccall f_8259(C_word c,C_word *av) C_noret;
C_noret_decl(f_8262)
static void C_ccall f_8262(C_word c,C_word *av) C_noret;
C_noret_decl(f_8265)
static void C_ccall f_8265(C_word c,C_word *av) C_noret;
C_noret_decl(f_8271)
static void C_ccall f_8271(C_word c,C_word *av) C_noret;
C_noret_decl(f_8274)
static void C_ccall f_8274(C_word c,C_word *av) C_noret;
C_noret_decl(f_8277)
static void C_ccall f_8277(C_word c,C_word *av) C_noret;
C_noret_decl(f_8292)
static void C_ccall f_8292(C_word c,C_word *av) C_noret;
C_noret_decl(f_8297)
static void C_ccall f_8297(C_word c,C_word *av) C_noret;
C_noret_decl(f_8301)
static void C_ccall f_8301(C_word c,C_word *av) C_noret;
C_noret_decl(f_8307)
static void C_ccall f_8307(C_word c,C_word *av) C_noret;
C_noret_decl(f_8312)
static void C_fcall f_8312(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8322)
static void C_ccall f_8322(C_word c,C_word *av) C_noret;
C_noret_decl(f_8337)
static void C_ccall f_8337(C_word c,C_word *av) C_noret;
C_noret_decl(f_8346)
static void C_ccall f_8346(C_word c,C_word *av) C_noret;
C_noret_decl(f_8356)
static void C_ccall f_8356(C_word c,C_word *av) C_noret;
C_noret_decl(f_8362)
static void C_fcall f_8362(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7,C_word t8) C_noret;
C_noret_decl(f_8369)
static void C_ccall f_8369(C_word c,C_word *av) C_noret;
C_noret_decl(f_8393)
static void C_ccall f_8393(C_word c,C_word *av) C_noret;
C_noret_decl(f_8408)
static void C_ccall f_8408(C_word c,C_word *av) C_noret;
C_noret_decl(f_8414)
static void C_ccall f_8414(C_word c,C_word *av) C_noret;
C_noret_decl(f_8417)
static void C_ccall f_8417(C_word c,C_word *av) C_noret;
C_noret_decl(f_8421)
static void C_ccall f_8421(C_word c,C_word *av) C_noret;
C_noret_decl(f_8424)
static void C_ccall f_8424(C_word c,C_word *av) C_noret;
C_noret_decl(f_8427)
static void C_ccall f_8427(C_word c,C_word *av) C_noret;
C_noret_decl(f_8439)
static void C_ccall f_8439(C_word c,C_word *av) C_noret;
C_noret_decl(f_8443)
static void C_fcall f_8443(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8471)
static void C_ccall f_8471(C_word c,C_word *av) C_noret;
C_noret_decl(f_8475)
static void C_ccall f_8475(C_word c,C_word *av) C_noret;
C_noret_decl(f_8479)
static void C_ccall f_8479(C_word c,C_word *av) C_noret;
C_noret_decl(f_8517)
static void C_ccall f_8517(C_word c,C_word *av) C_noret;
C_noret_decl(f_8550)
static void C_ccall f_8550(C_word c,C_word *av) C_noret;
C_noret_decl(f_8558)
static void C_ccall f_8558(C_word c,C_word *av) C_noret;
C_noret_decl(f_8575)
static void C_ccall f_8575(C_word c,C_word *av) C_noret;
C_noret_decl(f_8581)
static void C_ccall f_8581(C_word c,C_word *av) C_noret;
C_noret_decl(f_8584)
static void C_ccall f_8584(C_word c,C_word *av) C_noret;
C_noret_decl(f_8585)
static void C_ccall f_8585(C_word c,C_word *av) C_noret;
C_noret_decl(f_8599)
static void C_ccall f_8599(C_word c,C_word *av) C_noret;
C_noret_decl(f_8603)
static void C_ccall f_8603(C_word c,C_word *av) C_noret;
C_noret_decl(f_8617)
static void C_ccall f_8617(C_word c,C_word *av) C_noret;
C_noret_decl(f_8621)
static void C_ccall f_8621(C_word c,C_word *av) C_noret;
C_noret_decl(f_8625)
static void C_ccall f_8625(C_word c,C_word *av) C_noret;
C_noret_decl(f_8658)
static void C_ccall f_8658(C_word c,C_word *av) C_noret;
C_noret_decl(f_8663)
static void C_fcall f_8663(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8671)
static void C_ccall f_8671(C_word c,C_word *av) C_noret;
C_noret_decl(f_8675)
static void C_ccall f_8675(C_word c,C_word *av) C_noret;
C_noret_decl(f_8692)
static void C_ccall f_8692(C_word c,C_word *av) C_noret;
C_noret_decl(f_8694)
static void C_fcall f_8694(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_8719)
static void C_ccall f_8719(C_word c,C_word *av) C_noret;
C_noret_decl(f_8742)
static void C_ccall f_8742(C_word c,C_word *av) C_noret;
C_noret_decl(f_8754)
static void C_ccall f_8754(C_word c,C_word *av) C_noret;
C_noret_decl(f_8761)
static void C_ccall f_8761(C_word c,C_word *av) C_noret;
C_noret_decl(f_8764)
static void C_ccall f_8764(C_word c,C_word *av) C_noret;
C_noret_decl(f_8773)
static void C_fcall f_8773(C_word t0,C_word t1) C_noret;
C_noret_decl(f_8786)
static void C_ccall f_8786(C_word c,C_word *av) C_noret;
C_noret_decl(f_8806)
static void C_ccall f_8806(C_word c,C_word *av) C_noret;
C_noret_decl(f_8810)
static void C_ccall f_8810(C_word c,C_word *av) C_noret;
C_noret_decl(f_8814)
static void C_ccall f_8814(C_word c,C_word *av) C_noret;
C_noret_decl(f_8827)
static void C_ccall f_8827(C_word c,C_word *av) C_noret;
C_noret_decl(f_8831)
static void C_ccall f_8831(C_word c,C_word *av) C_noret;
C_noret_decl(f_8848)
static void C_ccall f_8848(C_word c,C_word *av) C_noret;
C_noret_decl(f_8869)
static void C_ccall f_8869(C_word c,C_word *av) C_noret;
C_noret_decl(f_8885)
static void C_ccall f_8885(C_word c,C_word *av) C_noret;
C_noret_decl(f_8896)
static void C_ccall f_8896(C_word c,C_word *av) C_noret;
C_noret_decl(f_8917)
static void C_ccall f_8917(C_word c,C_word *av) C_noret;
C_noret_decl(f_8926)
static void C_ccall f_8926(C_word c,C_word *av) C_noret;
C_noret_decl(f_8929)
static void C_ccall f_8929(C_word c,C_word *av) C_noret;
C_noret_decl(f_8932)
static void C_ccall f_8932(C_word c,C_word *av) C_noret;
C_noret_decl(f_8938)
static void C_ccall f_8938(C_word c,C_word *av) C_noret;
C_noret_decl(f_8946)
static void C_fcall f_8946(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_8954)
static void C_ccall f_8954(C_word c,C_word *av) C_noret;
C_noret_decl(f_8969)
static void C_ccall f_8969(C_word c,C_word *av) C_noret;
C_noret_decl(f_8973)
static void C_ccall f_8973(C_word c,C_word *av) C_noret;
C_noret_decl(f_8975)
static void C_ccall f_8975(C_word c,C_word *av) C_noret;
C_noret_decl(f_8979)
static void C_ccall f_8979(C_word c,C_word *av) C_noret;
C_noret_decl(f_8983)
static void C_ccall f_8983(C_word c,C_word *av) C_noret;
C_noret_decl(f_8986)
static void C_ccall f_8986(C_word c,C_word *av) C_noret;
C_noret_decl(f_8992)
static void C_ccall f_8992(C_word c,C_word *av) C_noret;
C_noret_decl(f_9000)
static void C_ccall f_9000(C_word c,C_word *av) C_noret;
C_noret_decl(f_9004)
static void C_ccall f_9004(C_word c,C_word *av) C_noret;
C_noret_decl(f_9010)
static void C_ccall f_9010(C_word c,C_word *av) C_noret;
C_noret_decl(f_9014)
static void C_ccall f_9014(C_word c,C_word *av) C_noret;
C_noret_decl(f_9017)
static void C_ccall f_9017(C_word c,C_word *av) C_noret;
C_noret_decl(f_9020)
static void C_fcall f_9020(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9049)
static void C_ccall f_9049(C_word c,C_word *av) C_noret;
C_noret_decl(f_9070)
static void C_ccall f_9070(C_word c,C_word *av) C_noret;
C_noret_decl(f_9072)
static void C_fcall f_9072(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9097)
static void C_ccall f_9097(C_word c,C_word *av) C_noret;
C_noret_decl(f_9145)
static void C_ccall f_9145(C_word c,C_word *av) C_noret;
C_noret_decl(f_9149)
static void C_ccall f_9149(C_word c,C_word *av) C_noret;
C_noret_decl(f_9169)
static void C_ccall f_9169(C_word c,C_word *av) C_noret;
C_noret_decl(f_9183)
static void C_fcall f_9183(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9217)
static void C_fcall f_9217(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9261)
static void C_ccall f_9261(C_word c,C_word *av) C_noret;
C_noret_decl(f_9270)
static void C_ccall f_9270(C_word c,C_word *av) C_noret;
C_noret_decl(f_9299)
static void C_ccall f_9299(C_word c,C_word *av) C_noret;
C_noret_decl(f_9312)
static C_word C_fcall f_9312(C_word *a,C_word t0,C_word t1,C_word t2);
C_noret_decl(f_9330)
static void C_ccall f_9330(C_word c,C_word *av) C_noret;
C_noret_decl(f_9334)
static void C_ccall f_9334(C_word c,C_word *av) C_noret;
C_noret_decl(f_9346)
static void C_ccall f_9346(C_word c,C_word *av) C_noret;
C_noret_decl(f_9360)
static void C_fcall f_9360(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9408)
static void C_fcall f_9408(C_word t0,C_word t1,C_word t2,C_word t3) C_noret;
C_noret_decl(f_9456)
static void C_fcall f_9456(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9490)
static void C_fcall f_9490(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9515)
static void C_ccall f_9515(C_word c,C_word *av) C_noret;
C_noret_decl(f_9535)
static void C_fcall f_9535(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9543)
static void C_ccall f_9543(C_word c,C_word *av) C_noret;
C_noret_decl(f_9551)
static void C_ccall f_9551(C_word c,C_word *av) C_noret;
C_noret_decl(f_9554)
static void C_ccall f_9554(C_word c,C_word *av) C_noret;
C_noret_decl(f_9559)
static void C_ccall f_9559(C_word c,C_word *av) C_noret;
C_noret_decl(f_9564)
static void C_ccall f_9564(C_word c,C_word *av) C_noret;
C_noret_decl(f_9574)
static void C_ccall f_9574(C_word c,C_word *av) C_noret;
C_noret_decl(f_9579)
static void C_fcall f_9579(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9604)
static void C_ccall f_9604(C_word c,C_word *av) C_noret;
C_noret_decl(f_9625)
static void C_ccall f_9625(C_word c,C_word *av) C_noret;
C_noret_decl(f_9630)
static void C_ccall f_9630(C_word c,C_word *av) C_noret;
C_noret_decl(f_9640)
static void C_ccall f_9640(C_word c,C_word *av) C_noret;
C_noret_decl(f_9643)
static void C_ccall f_9643(C_word c,C_word *av) C_noret;
C_noret_decl(f_9649)
static void C_ccall f_9649(C_word c,C_word *av) C_noret;
C_noret_decl(f_9652)
static void C_ccall f_9652(C_word c,C_word *av) C_noret;
C_noret_decl(f_9655)
static void C_ccall f_9655(C_word c,C_word *av) C_noret;
C_noret_decl(f_9661)
static void C_ccall f_9661(C_word c,C_word *av) C_noret;
C_noret_decl(f_9670)
static void C_fcall f_9670(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9676)
static void C_ccall f_9676(C_word c,C_word *av) C_noret;
C_noret_decl(f_9683)
static void C_ccall f_9683(C_word c,C_word *av) C_noret;
C_noret_decl(f_9714)
static void C_ccall f_9714(C_word c,C_word *av) C_noret;
C_noret_decl(f_9725)
static void C_ccall f_9725(C_word c,C_word *av) C_noret;
C_noret_decl(f_9727)
static void C_ccall f_9727(C_word c,C_word *av) C_noret;
C_noret_decl(f_9731)
static void C_ccall f_9731(C_word c,C_word *av) C_noret;
C_noret_decl(f_9735)
static void C_ccall f_9735(C_word c,C_word *av) C_noret;
C_noret_decl(f_9738)
static void C_ccall f_9738(C_word c,C_word *av) C_noret;
C_noret_decl(f_9744)
static void C_ccall f_9744(C_word c,C_word *av) C_noret;
C_noret_decl(f_9748)
static void C_ccall f_9748(C_word c,C_word *av) C_noret;
C_noret_decl(f_9753)
static void C_ccall f_9753(C_word c,C_word *av) C_noret;
C_noret_decl(f_9758)
static void C_ccall f_9758(C_word c,C_word *av) C_noret;
C_noret_decl(f_9766)
static void C_fcall f_9766(C_word t0,C_word t1) C_noret;
C_noret_decl(f_9770)
static void C_ccall f_9770(C_word c,C_word *av) C_noret;
C_noret_decl(f_9783)
static void C_ccall f_9783(C_word c,C_word *av) C_noret;
C_noret_decl(f_9788)
static void C_ccall f_9788(C_word c,C_word *av) C_noret;
C_noret_decl(f_9792)
static void C_ccall f_9792(C_word c,C_word *av) C_noret;
C_noret_decl(f_9795)
static void C_ccall f_9795(C_word c,C_word *av) C_noret;
C_noret_decl(f_9800)
static void C_ccall f_9800(C_word c,C_word *av) C_noret;
C_noret_decl(f_9802)
static void C_fcall f_9802(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(f_9827)
static void C_ccall f_9827(C_word c,C_word *av) C_noret;
C_noret_decl(f_9837)
static void C_ccall f_9837(C_word c,C_word *av) C_noret;
C_noret_decl(f_9842)
static void C_ccall f_9842(C_word c,C_word *av) C_noret;
C_noret_decl(f_9850)
static void C_ccall f_9850(C_word c,C_word *av) C_noret;
C_noret_decl(f_9852)
static void C_ccall f_9852(C_word c,C_word *av) C_noret;
C_noret_decl(f_9864)
static void C_ccall f_9864(C_word c,C_word *av) C_noret;
C_noret_decl(f_9869)
static void C_ccall f_9869(C_word c,C_word *av) C_noret;
C_noret_decl(f_9873)
static void C_ccall f_9873(C_word c,C_word *av) C_noret;
C_noret_decl(f_9877)
static void C_ccall f_9877(C_word c,C_word *av) C_noret;
C_noret_decl(f_9880)
static void C_ccall f_9880(C_word c,C_word *av) C_noret;
C_noret_decl(f_9886)
static void C_ccall f_9886(C_word c,C_word *av) C_noret;
C_noret_decl(f_9890)
static void C_ccall f_9890(C_word c,C_word *av) C_noret;
C_noret_decl(f_9900)
static void C_ccall f_9900(C_word c,C_word *av) C_noret;
C_noret_decl(f_9906)
static void C_ccall f_9906(C_word c,C_word *av) C_noret;
C_noret_decl(f_9910)
static void C_ccall f_9910(C_word c,C_word *av) C_noret;
C_noret_decl(f_9913)
static void C_ccall f_9913(C_word c,C_word *av) C_noret;
C_noret_decl(f_9931)
static void C_ccall f_9931(C_word c,C_word *av) C_noret;
C_noret_decl(f_9935)
static void C_ccall f_9935(C_word c,C_word *av) C_noret;
C_noret_decl(f_9939)
static void C_ccall f_9939(C_word c,C_word *av) C_noret;
C_noret_decl(f_9954)
static void C_ccall f_9954(C_word c,C_word *av) C_noret;
C_noret_decl(f_9958)
static void C_ccall f_9958(C_word c,C_word *av) C_noret;
C_noret_decl(f_9960)
static void C_fcall f_9960(C_word t0,C_word t1,C_word t2) C_noret;
C_noret_decl(C_compiler_toplevel)
C_externexport void C_ccall C_compiler_toplevel(C_word c,C_word *av) C_noret;
C_noret_decl(va26382)
static void C_fcall va26382(C_word t0,C_word t1) C_noret;

C_noret_decl(trf_10130)
static void C_ccall trf_10130(C_word c,C_word *av) C_noret;
static void C_ccall trf_10130(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10130(t0,t1,t2);}

C_noret_decl(trf_10177)
static void C_ccall trf_10177(C_word c,C_word *av) C_noret;
static void C_ccall trf_10177(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10177(t0,t1);}

C_noret_decl(trf_10286)
static void C_ccall trf_10286(C_word c,C_word *av) C_noret;
static void C_ccall trf_10286(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10286(t0,t1);}

C_noret_decl(trf_10363)
static void C_ccall trf_10363(C_word c,C_word *av) C_noret;
static void C_ccall trf_10363(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10363(t0,t1,t2);}

C_noret_decl(trf_10377)
static void C_ccall trf_10377(C_word c,C_word *av) C_noret;
static void C_ccall trf_10377(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10377(t0,t1);}

C_noret_decl(trf_10453)
static void C_ccall trf_10453(C_word c,C_word *av) C_noret;
static void C_ccall trf_10453(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10453(t0,t1,t2);}

C_noret_decl(trf_10512)
static void C_ccall trf_10512(C_word c,C_word *av) C_noret;
static void C_ccall trf_10512(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10512(t0,t1,t2);}

C_noret_decl(trf_10535)
static void C_ccall trf_10535(C_word c,C_word *av) C_noret;
static void C_ccall trf_10535(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10535(t0,t1,t2);}

C_noret_decl(trf_10677)
static void C_ccall trf_10677(C_word c,C_word *av) C_noret;
static void C_ccall trf_10677(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10677(t0,t1,t2);}

C_noret_decl(trf_10727)
static void C_ccall trf_10727(C_word c,C_word *av) C_noret;
static void C_ccall trf_10727(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10727(t0,t1);}

C_noret_decl(trf_10745)
static void C_ccall trf_10745(C_word c,C_word *av) C_noret;
static void C_ccall trf_10745(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10745(t0,t1);}

C_noret_decl(trf_10766)
static void C_ccall trf_10766(C_word c,C_word *av) C_noret;
static void C_ccall trf_10766(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_10766(t0,t1);}

C_noret_decl(trf_10812)
static void C_ccall trf_10812(C_word c,C_word *av) C_noret;
static void C_ccall trf_10812(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10812(t0,t1,t2);}

C_noret_decl(trf_10827)
static void C_ccall trf_10827(C_word c,C_word *av) C_noret;
static void C_ccall trf_10827(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10827(t0,t1,t2);}

C_noret_decl(trf_10912)
static void C_ccall trf_10912(C_word c,C_word *av) C_noret;
static void C_ccall trf_10912(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_10912(t0,t1,t2);}

C_noret_decl(trf_11009)
static void C_ccall trf_11009(C_word c,C_word *av) C_noret;
static void C_ccall trf_11009(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_11009(t0,t1,t2,t3);}

C_noret_decl(trf_11047)
static void C_ccall trf_11047(C_word c,C_word *av) C_noret;
static void C_ccall trf_11047(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11047(t0,t1);}

C_noret_decl(trf_11082)
static void C_ccall trf_11082(C_word c,C_word *av) C_noret;
static void C_ccall trf_11082(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11082(t0,t1,t2);}

C_noret_decl(trf_11164)
static void C_ccall trf_11164(C_word c,C_word *av) C_noret;
static void C_ccall trf_11164(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11164(t0,t1,t2);}

C_noret_decl(trf_11177)
static void C_ccall trf_11177(C_word c,C_word *av) C_noret;
static void C_ccall trf_11177(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11177(t0,t1);}

C_noret_decl(trf_11221)
static void C_ccall trf_11221(C_word c,C_word *av) C_noret;
static void C_ccall trf_11221(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11221(t0,t1,t2);}

C_noret_decl(trf_11359)
static void C_ccall trf_11359(C_word c,C_word *av) C_noret;
static void C_ccall trf_11359(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11359(t0,t1,t2);}

C_noret_decl(trf_11462)
static void C_ccall trf_11462(C_word c,C_word *av) C_noret;
static void C_ccall trf_11462(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11462(t0,t1,t2);}

C_noret_decl(trf_11514)
static void C_ccall trf_11514(C_word c,C_word *av) C_noret;
static void C_ccall trf_11514(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11514(t0,t1,t2);}

C_noret_decl(trf_11586)
static void C_ccall trf_11586(C_word c,C_word *av) C_noret;
static void C_ccall trf_11586(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_11586(t0,t1);}

C_noret_decl(trf_11815)
static void C_ccall trf_11815(C_word c,C_word *av) C_noret;
static void C_ccall trf_11815(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11815(t0,t1,t2);}

C_noret_decl(trf_11833)
static void C_ccall trf_11833(C_word c,C_word *av) C_noret;
static void C_ccall trf_11833(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11833(t0,t1,t2);}

C_noret_decl(trf_11930)
static void C_ccall trf_11930(C_word c,C_word *av) C_noret;
static void C_ccall trf_11930(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_11930(t0,t1,t2);}

C_noret_decl(trf_12069)
static void C_ccall trf_12069(C_word c,C_word *av) C_noret;
static void C_ccall trf_12069(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12069(t0,t1,t2);}

C_noret_decl(trf_12275)
static void C_ccall trf_12275(C_word c,C_word *av) C_noret;
static void C_ccall trf_12275(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12275(t0,t1);}

C_noret_decl(trf_12361)
static void C_ccall trf_12361(C_word c,C_word *av) C_noret;
static void C_ccall trf_12361(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12361(t0,t1);}

C_noret_decl(trf_12495)
static void C_ccall trf_12495(C_word c,C_word *av) C_noret;
static void C_ccall trf_12495(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_12495(t0,t1);}

C_noret_decl(trf_12836)
static void C_ccall trf_12836(C_word c,C_word *av) C_noret;
static void C_ccall trf_12836(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12836(t0,t1,t2);}

C_noret_decl(trf_12842)
static void C_ccall trf_12842(C_word c,C_word *av) C_noret;
static void C_ccall trf_12842(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12842(t0,t1,t2);}

C_noret_decl(trf_12859)
static void C_ccall trf_12859(C_word c,C_word *av) C_noret;
static void C_ccall trf_12859(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_12859(t0,t1,t2);}

C_noret_decl(trf_13073)
static void C_ccall trf_13073(C_word c,C_word *av) C_noret;
static void C_ccall trf_13073(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13073(t0,t1,t2,t3);}

C_noret_decl(trf_13224)
static void C_ccall trf_13224(C_word c,C_word *av) C_noret;
static void C_ccall trf_13224(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13224(t0,t1,t2);}

C_noret_decl(trf_13250)
static void C_ccall trf_13250(C_word c,C_word *av) C_noret;
static void C_ccall trf_13250(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13250(t0,t1,t2);}

C_noret_decl(trf_13609)
static void C_ccall trf_13609(C_word c,C_word *av) C_noret;
static void C_ccall trf_13609(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_13609(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_13614)
static void C_ccall trf_13614(C_word c,C_word *av) C_noret;
static void C_ccall trf_13614(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13614(t0,t1,t2);}

C_noret_decl(trf_13626)
static void C_ccall trf_13626(C_word c,C_word *av) C_noret;
static void C_ccall trf_13626(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13626(t0,t1,t2);}

C_noret_decl(trf_13700)
static void C_ccall trf_13700(C_word c,C_word *av) C_noret;
static void C_ccall trf_13700(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_13700(t0,t1,t2,t3);}

C_noret_decl(trf_13735)
static void C_ccall trf_13735(C_word c,C_word *av) C_noret;
static void C_ccall trf_13735(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13735(t0,t1,t2);}

C_noret_decl(trf_13754)
static void C_ccall trf_13754(C_word c,C_word *av) C_noret;
static void C_ccall trf_13754(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_13754(t0,t1,t2);}

C_noret_decl(trf_13760)
static void C_ccall trf_13760(C_word c,C_word *av) C_noret;
static void C_ccall trf_13760(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13760(t0,t1);}

C_noret_decl(trf_13788)
static void C_ccall trf_13788(C_word c,C_word *av) C_noret;
static void C_ccall trf_13788(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_13788(t0,t1);}

C_noret_decl(trf_14023)
static void C_ccall trf_14023(C_word c,C_word *av) C_noret;
static void C_ccall trf_14023(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14023(t0,t1,t2);}

C_noret_decl(trf_14087)
static void C_ccall trf_14087(C_word c,C_word *av) C_noret;
static void C_ccall trf_14087(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14087(t0,t1,t2);}

C_noret_decl(trf_14110)
static void C_ccall trf_14110(C_word c,C_word *av) C_noret;
static void C_ccall trf_14110(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14110(t0,t1,t2);}

C_noret_decl(trf_14168)
static void C_ccall trf_14168(C_word c,C_word *av) C_noret;
static void C_ccall trf_14168(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14168(t0,t1,t2);}

C_noret_decl(trf_14191)
static void C_ccall trf_14191(C_word c,C_word *av) C_noret;
static void C_ccall trf_14191(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14191(t0,t1,t2);}

C_noret_decl(trf_14277)
static void C_ccall trf_14277(C_word c,C_word *av) C_noret;
static void C_ccall trf_14277(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14277(t0,t1,t2);}

C_noret_decl(trf_14322)
static void C_ccall trf_14322(C_word c,C_word *av) C_noret;
static void C_ccall trf_14322(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14322(t0,t1,t2);}

C_noret_decl(trf_14442)
static void C_ccall trf_14442(C_word c,C_word *av) C_noret;
static void C_ccall trf_14442(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14442(t0,t1,t2);}

C_noret_decl(trf_14531)
static void C_ccall trf_14531(C_word c,C_word *av) C_noret;
static void C_ccall trf_14531(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14531(t0,t1,t2);}

C_noret_decl(trf_14629)
static void C_ccall trf_14629(C_word c,C_word *av) C_noret;
static void C_ccall trf_14629(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14629(t0,t1,t2);}

C_noret_decl(trf_14670)
static void C_ccall trf_14670(C_word c,C_word *av) C_noret;
static void C_ccall trf_14670(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14670(t0,t1,t2);}

C_noret_decl(trf_14729)
static void C_ccall trf_14729(C_word c,C_word *av) C_noret;
static void C_ccall trf_14729(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14729(t0,t1,t2);}

C_noret_decl(trf_14829)
static void C_ccall trf_14829(C_word c,C_word *av) C_noret;
static void C_ccall trf_14829(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14829(t0,t1,t2);}

C_noret_decl(trf_14895)
static void C_ccall trf_14895(C_word c,C_word *av) C_noret;
static void C_ccall trf_14895(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_14895(t0,t1);}

C_noret_decl(trf_14949)
static void C_ccall trf_14949(C_word c,C_word *av) C_noret;
static void C_ccall trf_14949(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_14949(t0,t1,t2);}

C_noret_decl(trf_15002)
static void C_ccall trf_15002(C_word c,C_word *av) C_noret;
static void C_ccall trf_15002(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15002(t0,t1);}

C_noret_decl(trf_15059)
static void C_ccall trf_15059(C_word c,C_word *av) C_noret;
static void C_ccall trf_15059(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15059(t0,t1,t2);}

C_noret_decl(trf_15113)
static void C_ccall trf_15113(C_word c,C_word *av) C_noret;
static void C_ccall trf_15113(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15113(t0,t1,t2);}

C_noret_decl(trf_15168)
static void C_ccall trf_15168(C_word c,C_word *av) C_noret;
static void C_ccall trf_15168(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15168(t0,t1,t2);}

C_noret_decl(trf_15200)
static void C_ccall trf_15200(C_word c,C_word *av) C_noret;
static void C_ccall trf_15200(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15200(t0,t1,t2);}

C_noret_decl(trf_15207)
static void C_ccall trf_15207(C_word c,C_word *av) C_noret;
static void C_ccall trf_15207(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15207(t0,t1);}

C_noret_decl(trf_15290)
static void C_ccall trf_15290(C_word c,C_word *av) C_noret;
static void C_ccall trf_15290(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15290(t0,t1);}

C_noret_decl(trf_15364)
static void C_ccall trf_15364(C_word c,C_word *av) C_noret;
static void C_ccall trf_15364(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15364(t0,t1,t2);}

C_noret_decl(trf_15392)
static void C_ccall trf_15392(C_word c,C_word *av) C_noret;
static void C_ccall trf_15392(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15392(t0,t1,t2);}

C_noret_decl(trf_15399)
static void C_ccall trf_15399(C_word c,C_word *av) C_noret;
static void C_ccall trf_15399(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15399(t0,t1);}

C_noret_decl(trf_15486)
static void C_ccall trf_15486(C_word c,C_word *av) C_noret;
static void C_ccall trf_15486(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15486(t0,t1,t2);}

C_noret_decl(trf_15725)
static void C_ccall trf_15725(C_word c,C_word *av) C_noret;
static void C_ccall trf_15725(C_word c,C_word *av){
C_word t0=av[7];
C_word t1=av[6];
C_word t2=av[5];
C_word t3=av[4];
C_word t4=av[3];
C_word t5=av[2];
C_word t6=av[1];
C_word t7=av[0];
f_15725(t0,t1,t2,t3,t4,t5,t6,t7);}

C_noret_decl(trf_15734)
static void C_ccall trf_15734(C_word c,C_word *av) C_noret;
static void C_ccall trf_15734(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_15734(t0,t1,t2);}

C_noret_decl(trf_15840)
static void C_ccall trf_15840(C_word c,C_word *av) C_noret;
static void C_ccall trf_15840(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_15840(t0,t1);}

C_noret_decl(trf_15947)
static void C_ccall trf_15947(C_word c,C_word *av) C_noret;
static void C_ccall trf_15947(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_15947(t0,t1,t2,t3);}

C_noret_decl(trf_16028)
static void C_ccall trf_16028(C_word c,C_word *av) C_noret;
static void C_ccall trf_16028(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16028(t0,t1,t2);}

C_noret_decl(trf_16080)
static void C_ccall trf_16080(C_word c,C_word *av) C_noret;
static void C_ccall trf_16080(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16080(t0,t1,t2);}

C_noret_decl(trf_16114)
static void C_ccall trf_16114(C_word c,C_word *av) C_noret;
static void C_ccall trf_16114(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16114(t0,t1,t2);}

C_noret_decl(trf_16151)
static void C_ccall trf_16151(C_word c,C_word *av) C_noret;
static void C_ccall trf_16151(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16151(t0,t1,t2);}

C_noret_decl(trf_16192)
static void C_ccall trf_16192(C_word c,C_word *av) C_noret;
static void C_ccall trf_16192(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16192(t0,t1,t2);}

C_noret_decl(trf_16226)
static void C_ccall trf_16226(C_word c,C_word *av) C_noret;
static void C_ccall trf_16226(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16226(t0,t1,t2);}

C_noret_decl(trf_16268)
static void C_ccall trf_16268(C_word c,C_word *av) C_noret;
static void C_ccall trf_16268(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16268(t0,t1);}

C_noret_decl(trf_16308)
static void C_ccall trf_16308(C_word c,C_word *av) C_noret;
static void C_ccall trf_16308(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16308(t0,t1,t2);}

C_noret_decl(trf_16342)
static void C_ccall trf_16342(C_word c,C_word *av) C_noret;
static void C_ccall trf_16342(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16342(t0,t1,t2);}

C_noret_decl(trf_16420)
static void C_ccall trf_16420(C_word c,C_word *av) C_noret;
static void C_ccall trf_16420(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16420(t0,t1,t2);}

C_noret_decl(trf_16426)
static void C_ccall trf_16426(C_word c,C_word *av) C_noret;
static void C_ccall trf_16426(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16426(t0,t1,t2);}

C_noret_decl(trf_16445)
static void C_ccall trf_16445(C_word c,C_word *av) C_noret;
static void C_ccall trf_16445(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16445(t0,t1,t2);}

C_noret_decl(trf_16502)
static void C_ccall trf_16502(C_word c,C_word *av) C_noret;
static void C_ccall trf_16502(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_16502(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_16553)
static void C_ccall trf_16553(C_word c,C_word *av) C_noret;
static void C_ccall trf_16553(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_16553(t0,t1,t2);}

C_noret_decl(trf_16577)
static void C_ccall trf_16577(C_word c,C_word *av) C_noret;
static void C_ccall trf_16577(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_16577(t0,t1,t2,t3);}

C_noret_decl(trf_16599)
static void C_ccall trf_16599(C_word c,C_word *av) C_noret;
static void C_ccall trf_16599(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16599(t0,t1);}

C_noret_decl(trf_16720)
static void C_ccall trf_16720(C_word c,C_word *av) C_noret;
static void C_ccall trf_16720(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_16720(t0,t1,t2,t3);}

C_noret_decl(trf_16921)
static void C_ccall trf_16921(C_word c,C_word *av) C_noret;
static void C_ccall trf_16921(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_16921(t0,t1);}

C_noret_decl(trf_17123)
static void C_ccall trf_17123(C_word c,C_word *av) C_noret;
static void C_ccall trf_17123(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_17123(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_17269)
static void C_ccall trf_17269(C_word c,C_word *av) C_noret;
static void C_ccall trf_17269(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_17269(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_17285)
static void C_ccall trf_17285(C_word c,C_word *av) C_noret;
static void C_ccall trf_17285(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_17285(t0,t1,t2,t3);}

C_noret_decl(trf_17291)
static void C_ccall trf_17291(C_word c,C_word *av) C_noret;
static void C_ccall trf_17291(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_17291(t0,t1,t2,t3);}

C_noret_decl(trf_17429)
static void C_ccall trf_17429(C_word c,C_word *av) C_noret;
static void C_ccall trf_17429(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_17429(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_17451)
static void C_ccall trf_17451(C_word c,C_word *av) C_noret;
static void C_ccall trf_17451(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17451(t0,t1);}

C_noret_decl(trf_17561)
static void C_ccall trf_17561(C_word c,C_word *av) C_noret;
static void C_ccall trf_17561(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17561(t0,t1);}

C_noret_decl(trf_17643)
static void C_ccall trf_17643(C_word c,C_word *av) C_noret;
static void C_ccall trf_17643(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_17643(t0,t1,t2,t3);}

C_noret_decl(trf_17703)
static void C_ccall trf_17703(C_word c,C_word *av) C_noret;
static void C_ccall trf_17703(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17703(t0,t1,t2);}

C_noret_decl(trf_17728)
static void C_ccall trf_17728(C_word c,C_word *av) C_noret;
static void C_ccall trf_17728(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17728(t0,t1,t2);}

C_noret_decl(trf_17778)
static void C_ccall trf_17778(C_word c,C_word *av) C_noret;
static void C_ccall trf_17778(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17778(t0,t1,t2);}

C_noret_decl(trf_17800)
static void C_ccall trf_17800(C_word c,C_word *av) C_noret;
static void C_ccall trf_17800(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17800(t0,t1);}

C_noret_decl(trf_17803)
static void C_ccall trf_17803(C_word c,C_word *av) C_noret;
static void C_ccall trf_17803(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_17803(t0,t1);}

C_noret_decl(trf_17863)
static void C_ccall trf_17863(C_word c,C_word *av) C_noret;
static void C_ccall trf_17863(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_17863(t0,t1,t2);}

C_noret_decl(trf_18100)
static void C_ccall trf_18100(C_word c,C_word *av) C_noret;
static void C_ccall trf_18100(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_18100(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_18102)
static void C_ccall trf_18102(C_word c,C_word *av) C_noret;
static void C_ccall trf_18102(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18102(t0,t1,t2);}

C_noret_decl(trf_18114)
static void C_ccall trf_18114(C_word c,C_word *av) C_noret;
static void C_ccall trf_18114(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18114(t0,t1,t2);}

C_noret_decl(trf_18137)
static void C_ccall trf_18137(C_word c,C_word *av) C_noret;
static void C_ccall trf_18137(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_18137(t0,t1,t2,t3,t4);}

C_noret_decl(trf_18150)
static void C_ccall trf_18150(C_word c,C_word *av) C_noret;
static void C_ccall trf_18150(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_18150(t0,t1,t2,t3,t4);}

C_noret_decl(trf_18163)
static void C_ccall trf_18163(C_word c,C_word *av) C_noret;
static void C_ccall trf_18163(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_18163(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_18188)
static void C_ccall trf_18188(C_word c,C_word *av) C_noret;
static void C_ccall trf_18188(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18188(t0,t1);}

C_noret_decl(trf_18245)
static void C_ccall trf_18245(C_word c,C_word *av) C_noret;
static void C_ccall trf_18245(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18245(t0,t1);}

C_noret_decl(trf_18249)
static void C_ccall trf_18249(C_word c,C_word *av) C_noret;
static void C_ccall trf_18249(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_18249(t0,t1,t2);}

C_noret_decl(trf_18273)
static void C_ccall trf_18273(C_word c,C_word *av) C_noret;
static void C_ccall trf_18273(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18273(t0,t1);}

C_noret_decl(trf_18318)
static void C_ccall trf_18318(C_word c,C_word *av) C_noret;
static void C_ccall trf_18318(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18318(t0,t1);}

C_noret_decl(trf_18473)
static void C_ccall trf_18473(C_word c,C_word *av) C_noret;
static void C_ccall trf_18473(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_18473(t0,t1,t2,t3);}

C_noret_decl(trf_18666)
static void C_ccall trf_18666(C_word c,C_word *av) C_noret;
static void C_ccall trf_18666(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18666(t0,t1);}

C_noret_decl(trf_18672)
static void C_ccall trf_18672(C_word c,C_word *av) C_noret;
static void C_ccall trf_18672(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18672(t0,t1);}

C_noret_decl(trf_18678)
static void C_ccall trf_18678(C_word c,C_word *av) C_noret;
static void C_ccall trf_18678(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18678(t0,t1);}

C_noret_decl(trf_18687)
static void C_ccall trf_18687(C_word c,C_word *av) C_noret;
static void C_ccall trf_18687(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18687(t0,t1);}

C_noret_decl(trf_18708)
static void C_ccall trf_18708(C_word c,C_word *av) C_noret;
static void C_ccall trf_18708(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18708(t0,t1);}

C_noret_decl(trf_18729)
static void C_ccall trf_18729(C_word c,C_word *av) C_noret;
static void C_ccall trf_18729(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18729(t0,t1);}

C_noret_decl(trf_18830)
static void C_ccall trf_18830(C_word c,C_word *av) C_noret;
static void C_ccall trf_18830(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18830(t0,t1);}

C_noret_decl(trf_18994)
static void C_ccall trf_18994(C_word c,C_word *av) C_noret;
static void C_ccall trf_18994(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_18994(t0,t1);}

C_noret_decl(trf_19022)
static void C_ccall trf_19022(C_word c,C_word *av) C_noret;
static void C_ccall trf_19022(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19022(t0,t1,t2);}

C_noret_decl(trf_19029)
static void C_ccall trf_19029(C_word c,C_word *av) C_noret;
static void C_ccall trf_19029(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19029(t0,t1);}

C_noret_decl(trf_19055)
static void C_ccall trf_19055(C_word c,C_word *av) C_noret;
static void C_ccall trf_19055(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19055(t0,t1,t2);}

C_noret_decl(trf_19089)
static void C_ccall trf_19089(C_word c,C_word *av) C_noret;
static void C_ccall trf_19089(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19089(t0,t1);}

C_noret_decl(trf_19214)
static void C_ccall trf_19214(C_word c,C_word *av) C_noret;
static void C_ccall trf_19214(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19214(t0,t1);}

C_noret_decl(trf_19262)
static void C_ccall trf_19262(C_word c,C_word *av) C_noret;
static void C_ccall trf_19262(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19262(t0,t1,t2);}

C_noret_decl(trf_19299)
static void C_ccall trf_19299(C_word c,C_word *av) C_noret;
static void C_ccall trf_19299(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19299(t0,t1);}

C_noret_decl(trf_19373)
static void C_ccall trf_19373(C_word c,C_word *av) C_noret;
static void C_ccall trf_19373(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19373(t0,t1);}

C_noret_decl(trf_19377)
static void C_ccall trf_19377(C_word c,C_word *av) C_noret;
static void C_ccall trf_19377(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19377(t0,t1,t2);}

C_noret_decl(trf_19407)
static void C_ccall trf_19407(C_word c,C_word *av) C_noret;
static void C_ccall trf_19407(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19407(t0,t1,t2);}

C_noret_decl(trf_19472)
static void C_ccall trf_19472(C_word c,C_word *av) C_noret;
static void C_ccall trf_19472(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19472(t0,t1,t2,t3);}

C_noret_decl(trf_19478)
static void C_ccall trf_19478(C_word c,C_word *av) C_noret;
static void C_ccall trf_19478(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_19478(t0,t1,t2,t3);}

C_noret_decl(trf_19488)
static void C_ccall trf_19488(C_word c,C_word *av) C_noret;
static void C_ccall trf_19488(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19488(t0,t1,t2);}

C_noret_decl(trf_19497)
static void C_ccall trf_19497(C_word c,C_word *av) C_noret;
static void C_ccall trf_19497(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_19497(t0,t1,t2,t3,t4);}

C_noret_decl(trf_19537)
static void C_ccall trf_19537(C_word c,C_word *av) C_noret;
static void C_ccall trf_19537(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19537(t0,t1);}

C_noret_decl(trf_19639)
static void C_ccall trf_19639(C_word c,C_word *av) C_noret;
static void C_ccall trf_19639(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19639(t0,t1,t2);}

C_noret_decl(trf_19651)
static void C_ccall trf_19651(C_word c,C_word *av) C_noret;
static void C_ccall trf_19651(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19651(t0,t1,t2);}

C_noret_decl(trf_19713)
static void C_ccall trf_19713(C_word c,C_word *av) C_noret;
static void C_ccall trf_19713(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19713(t0,t1);}

C_noret_decl(trf_19731)
static void C_ccall trf_19731(C_word c,C_word *av) C_noret;
static void C_ccall trf_19731(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19731(t0,t1);}

C_noret_decl(trf_19737)
static void C_ccall trf_19737(C_word c,C_word *av) C_noret;
static void C_ccall trf_19737(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_19737(t0,t1);}

C_noret_decl(trf_19931)
static void C_ccall trf_19931(C_word c,C_word *av) C_noret;
static void C_ccall trf_19931(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19931(t0,t1,t2);}

C_noret_decl(trf_19943)
static void C_ccall trf_19943(C_word c,C_word *av) C_noret;
static void C_ccall trf_19943(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_19943(t0,t1,t2);}

C_noret_decl(trf_20028)
static void C_ccall trf_20028(C_word c,C_word *av) C_noret;
static void C_ccall trf_20028(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_20028(t0,t1,t2,t3,t4);}

C_noret_decl(trf_20047)
static void C_ccall trf_20047(C_word c,C_word *av) C_noret;
static void C_ccall trf_20047(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20047(t0,t1);}

C_noret_decl(trf_20081)
static void C_ccall trf_20081(C_word c,C_word *av) C_noret;
static void C_ccall trf_20081(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20081(t0,t1);}

C_noret_decl(trf_20167)
static void C_ccall trf_20167(C_word c,C_word *av) C_noret;
static void C_ccall trf_20167(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20167(t0,t1);}

C_noret_decl(trf_20317)
static void C_ccall trf_20317(C_word c,C_word *av) C_noret;
static void C_ccall trf_20317(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20317(t0,t1);}

C_noret_decl(trf_20340)
static void C_ccall trf_20340(C_word c,C_word *av) C_noret;
static void C_ccall trf_20340(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20340(t0,t1,t2);}

C_noret_decl(trf_20397)
static void C_ccall trf_20397(C_word c,C_word *av) C_noret;
static void C_ccall trf_20397(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20397(t0,t1,t2);}

C_noret_decl(trf_20483)
static void C_ccall trf_20483(C_word c,C_word *av) C_noret;
static void C_ccall trf_20483(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20483(t0,t1,t2,t3);}

C_noret_decl(trf_20519)
static void C_ccall trf_20519(C_word c,C_word *av) C_noret;
static void C_ccall trf_20519(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20519(t0,t1,t2);}

C_noret_decl(trf_20607)
static void C_ccall trf_20607(C_word c,C_word *av) C_noret;
static void C_ccall trf_20607(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20607(t0,t1,t2);}

C_noret_decl(trf_20684)
static void C_ccall trf_20684(C_word c,C_word *av) C_noret;
static void C_ccall trf_20684(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_20684(t0,t1,t2,t3);}

C_noret_decl(trf_20732)
static void C_ccall trf_20732(C_word c,C_word *av) C_noret;
static void C_ccall trf_20732(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20732(t0,t1,t2);}

C_noret_decl(trf_20791)
static void C_ccall trf_20791(C_word c,C_word *av) C_noret;
static void C_ccall trf_20791(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_20791(t0,t1);}

C_noret_decl(trf_20798)
static void C_ccall trf_20798(C_word c,C_word *av) C_noret;
static void C_ccall trf_20798(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_20798(t0,t1,t2);}

C_noret_decl(trf_21214)
static void C_ccall trf_21214(C_word c,C_word *av) C_noret;
static void C_ccall trf_21214(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_21214(t0,t1,t2,t3,t4);}

C_noret_decl(trf_21219)
static void C_ccall trf_21219(C_word c,C_word *av) C_noret;
static void C_ccall trf_21219(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21219(t0,t1,t2);}

C_noret_decl(trf_21231)
static void C_ccall trf_21231(C_word c,C_word *av) C_noret;
static void C_ccall trf_21231(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21231(t0,t1,t2);}

C_noret_decl(trf_21265)
static void C_ccall trf_21265(C_word c,C_word *av) C_noret;
static void C_ccall trf_21265(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21265(t0,t1,t2,t3);}

C_noret_decl(trf_21276)
static void C_ccall trf_21276(C_word c,C_word *av) C_noret;
static void C_ccall trf_21276(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21276(t0,t1,t2);}

C_noret_decl(trf_21496)
static void C_ccall trf_21496(C_word c,C_word *av) C_noret;
static void C_ccall trf_21496(C_word c,C_word *av){
C_word t0=av[5];
C_word t1=av[4];
C_word t2=av[3];
C_word t3=av[2];
C_word t4=av[1];
C_word t5=av[0];
f_21496(t0,t1,t2,t3,t4,t5);}

C_noret_decl(trf_21504)
static void C_ccall trf_21504(C_word c,C_word *av) C_noret;
static void C_ccall trf_21504(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_21504(t0,t1,t2);}

C_noret_decl(trf_21545)
static void C_ccall trf_21545(C_word c,C_word *av) C_noret;
static void C_ccall trf_21545(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_21545(t0,t1,t2,t3);}

C_noret_decl(trf_21552)
static void C_ccall trf_21552(C_word c,C_word *av) C_noret;
static void C_ccall trf_21552(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21552(t0,t1);}

C_noret_decl(trf_21555)
static void C_ccall trf_21555(C_word c,C_word *av) C_noret;
static void C_ccall trf_21555(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21555(t0,t1);}

C_noret_decl(trf_21601)
static void C_ccall trf_21601(C_word c,C_word *av) C_noret;
static void C_ccall trf_21601(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_21601(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_21620)
static void C_ccall trf_21620(C_word c,C_word *av) C_noret;
static void C_ccall trf_21620(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21620(t0,t1);}

C_noret_decl(trf_21648)
static void C_ccall trf_21648(C_word c,C_word *av) C_noret;
static void C_ccall trf_21648(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21648(t0,t1);}

C_noret_decl(trf_21847)
static void C_ccall trf_21847(C_word c,C_word *av) C_noret;
static void C_ccall trf_21847(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21847(t0,t1);}

C_noret_decl(trf_21980)
static void C_ccall trf_21980(C_word c,C_word *av) C_noret;
static void C_ccall trf_21980(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_21980(t0,t1);}

C_noret_decl(trf_22203)
static void C_ccall trf_22203(C_word c,C_word *av) C_noret;
static void C_ccall trf_22203(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22203(t0,t1,t2);}

C_noret_decl(trf_22242)
static void C_ccall trf_22242(C_word c,C_word *av) C_noret;
static void C_ccall trf_22242(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22242(t0,t1);}

C_noret_decl(trf_22245)
static void C_ccall trf_22245(C_word c,C_word *av) C_noret;
static void C_ccall trf_22245(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22245(t0,t1);}

C_noret_decl(trf_22332)
static void C_ccall trf_22332(C_word c,C_word *av) C_noret;
static void C_ccall trf_22332(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22332(t0,t1);}

C_noret_decl(trf_22339)
static void C_ccall trf_22339(C_word c,C_word *av) C_noret;
static void C_ccall trf_22339(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22339(t0,t1);}

C_noret_decl(trf_22342)
static void C_ccall trf_22342(C_word c,C_word *av) C_noret;
static void C_ccall trf_22342(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22342(t0,t1);}

C_noret_decl(trf_22380)
static void C_ccall trf_22380(C_word c,C_word *av) C_noret;
static void C_ccall trf_22380(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22380(t0,t1);}

C_noret_decl(trf_22417)
static void C_ccall trf_22417(C_word c,C_word *av) C_noret;
static void C_ccall trf_22417(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22417(t0,t1);}

C_noret_decl(trf_22444)
static void C_ccall trf_22444(C_word c,C_word *av) C_noret;
static void C_ccall trf_22444(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22444(t0,t1);}

C_noret_decl(trf_22465)
static void C_ccall trf_22465(C_word c,C_word *av) C_noret;
static void C_ccall trf_22465(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22465(t0,t1);}

C_noret_decl(trf_22652)
static void C_ccall trf_22652(C_word c,C_word *av) C_noret;
static void C_ccall trf_22652(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_22652(t0,t1,t2,t3,t4);}

C_noret_decl(trf_22811)
static void C_ccall trf_22811(C_word c,C_word *av) C_noret;
static void C_ccall trf_22811(C_word c,C_word *av){
C_word t0=av[6];
C_word t1=av[5];
C_word t2=av[4];
C_word t3=av[3];
C_word t4=av[2];
C_word t5=av[1];
C_word t6=av[0];
f_22811(t0,t1,t2,t3,t4,t5,t6);}

C_noret_decl(trf_22816)
static void C_ccall trf_22816(C_word c,C_word *av) C_noret;
static void C_ccall trf_22816(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22816(t0,t1,t2);}

C_noret_decl(trf_22828)
static void C_ccall trf_22828(C_word c,C_word *av) C_noret;
static void C_ccall trf_22828(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22828(t0,t1,t2);}

C_noret_decl(trf_22862)
static void C_ccall trf_22862(C_word c,C_word *av) C_noret;
static void C_ccall trf_22862(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22862(t0,t1,t2);}

C_noret_decl(trf_22919)
static void C_ccall trf_22919(C_word c,C_word *av) C_noret;
static void C_ccall trf_22919(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22919(t0,t1,t2);}

C_noret_decl(trf_22947)
static void C_ccall trf_22947(C_word c,C_word *av) C_noret;
static void C_ccall trf_22947(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_22947(t0,t1,t2);}

C_noret_decl(trf_22963)
static void C_ccall trf_22963(C_word c,C_word *av) C_noret;
static void C_ccall trf_22963(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_22963(t0,t1);}

C_noret_decl(trf_6472)
static void C_ccall trf_6472(C_word c,C_word *av) C_noret;
static void C_ccall trf_6472(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_6472(t0,t1,t2,t3,t4);}

C_noret_decl(trf_6675)
static void C_ccall trf_6675(C_word c,C_word *av) C_noret;
static void C_ccall trf_6675(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6675(t0,t1,t2);}

C_noret_decl(trf_6681)
static void C_ccall trf_6681(C_word c,C_word *av) C_noret;
static void C_ccall trf_6681(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6681(t0,t1,t2);}

C_noret_decl(trf_6739)
static void C_ccall trf_6739(C_word c,C_word *av) C_noret;
static void C_ccall trf_6739(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6739(t0,t1,t2);}

C_noret_decl(trf_6745)
static void C_ccall trf_6745(C_word c,C_word *av) C_noret;
static void C_ccall trf_6745(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6745(t0,t1,t2,t3);}

C_noret_decl(trf_6767)
static void C_ccall trf_6767(C_word c,C_word *av) C_noret;
static void C_ccall trf_6767(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_6767(t0,t1);}

C_noret_decl(trf_6773)
static void C_ccall trf_6773(C_word c,C_word *av) C_noret;
static void C_ccall trf_6773(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6773(t0,t1,t2);}

C_noret_decl(trf_6795)
static void C_ccall trf_6795(C_word c,C_word *av) C_noret;
static void C_ccall trf_6795(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6795(t0,t1,t2,t3);}

C_noret_decl(trf_6801)
static void C_ccall trf_6801(C_word c,C_word *av) C_noret;
static void C_ccall trf_6801(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6801(t0,t1,t2);}

C_noret_decl(trf_6876)
static void C_ccall trf_6876(C_word c,C_word *av) C_noret;
static void C_ccall trf_6876(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6876(t0,t1,t2);}

C_noret_decl(trf_6915)
static void C_ccall trf_6915(C_word c,C_word *av) C_noret;
static void C_ccall trf_6915(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6915(t0,t1,t2);}

C_noret_decl(trf_6924)
static void C_ccall trf_6924(C_word c,C_word *av) C_noret;
static void C_ccall trf_6924(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6924(t0,t1,t2);}

C_noret_decl(trf_6932)
static void C_ccall trf_6932(C_word c,C_word *av) C_noret;
static void C_ccall trf_6932(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6932(t0,t1,t2,t3);}

C_noret_decl(trf_6968)
static void C_ccall trf_6968(C_word c,C_word *av) C_noret;
static void C_ccall trf_6968(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_6968(t0,t1,t2);}

C_noret_decl(trf_6976)
static void C_ccall trf_6976(C_word c,C_word *av) C_noret;
static void C_ccall trf_6976(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_6976(t0,t1,t2,t3);}

C_noret_decl(trf_7024)
static void C_ccall trf_7024(C_word c,C_word *av) C_noret;
static void C_ccall trf_7024(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7024(t0,t1);}

C_noret_decl(trf_7041)
static void C_ccall trf_7041(C_word c,C_word *av) C_noret;
static void C_ccall trf_7041(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7041(t0,t1,t2);}

C_noret_decl(trf_7106)
static void C_ccall trf_7106(C_word c,C_word *av) C_noret;
static void C_ccall trf_7106(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7106(t0,t1,t2,t3);}

C_noret_decl(trf_7134)
static void C_ccall trf_7134(C_word c,C_word *av) C_noret;
static void C_ccall trf_7134(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7134(t0,t1,t2);}

C_noret_decl(trf_7140)
static void C_ccall trf_7140(C_word c,C_word *av) C_noret;
static void C_ccall trf_7140(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7140(t0,t1,t2,t3);}

C_noret_decl(trf_7175)
static void C_ccall trf_7175(C_word c,C_word *av) C_noret;
static void C_ccall trf_7175(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7175(t0,t1,t2);}

C_noret_decl(trf_7181)
static void C_ccall trf_7181(C_word c,C_word *av) C_noret;
static void C_ccall trf_7181(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7181(t0,t1,t2,t3);}

C_noret_decl(trf_7220)
static void C_ccall trf_7220(C_word c,C_word *av) C_noret;
static void C_ccall trf_7220(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7220(t0,t1,t2);}

C_noret_decl(trf_7226)
static void C_ccall trf_7226(C_word c,C_word *av) C_noret;
static void C_ccall trf_7226(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7226(t0,t1,t2,t3);}

C_noret_decl(trf_7251)
static void C_ccall trf_7251(C_word c,C_word *av) C_noret;
static void C_ccall trf_7251(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7251(t0,t1,t2,t3);}

C_noret_decl(trf_7299)
static void C_ccall trf_7299(C_word c,C_word *av) C_noret;
static void C_ccall trf_7299(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7299(t0,t1,t2);}

C_noret_decl(trf_7518)
static void C_ccall trf_7518(C_word c,C_word *av) C_noret;
static void C_ccall trf_7518(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7518(t0,t1,t2);}

C_noret_decl(trf_7707)
static void C_ccall trf_7707(C_word c,C_word *av) C_noret;
static void C_ccall trf_7707(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7707(t0,t1,t2);}

C_noret_decl(trf_7781)
static void C_ccall trf_7781(C_word c,C_word *av) C_noret;
static void C_ccall trf_7781(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7781(t0,t1,t2);}

C_noret_decl(trf_7807)
static void C_ccall trf_7807(C_word c,C_word *av) C_noret;
static void C_ccall trf_7807(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7807(t0,t1,t2,t3);}

C_noret_decl(trf_7820)
static void C_ccall trf_7820(C_word c,C_word *av) C_noret;
static void C_ccall trf_7820(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7820(t0,t1);}

C_noret_decl(trf_7842)
static void C_ccall trf_7842(C_word c,C_word *av) C_noret;
static void C_ccall trf_7842(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7842(t0,t1,t2);}

C_noret_decl(trf_7876)
static void C_ccall trf_7876(C_word c,C_word *av) C_noret;
static void C_ccall trf_7876(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7876(t0,t1,t2);}

C_noret_decl(trf_7889)
static void C_ccall trf_7889(C_word c,C_word *av) C_noret;
static void C_ccall trf_7889(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_7889(t0,t1);}

C_noret_decl(trf_7916)
static void C_ccall trf_7916(C_word c,C_word *av) C_noret;
static void C_ccall trf_7916(C_word c,C_word *av){
C_word t0=av[4];
C_word t1=av[3];
C_word t2=av[2];
C_word t3=av[1];
C_word t4=av[0];
f_7916(t0,t1,t2,t3,t4);}

C_noret_decl(trf_7945)
static void C_ccall trf_7945(C_word c,C_word *av) C_noret;
static void C_ccall trf_7945(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_7945(t0,t1,t2);}

C_noret_decl(trf_7962)
static void C_ccall trf_7962(C_word c,C_word *av) C_noret;
static void C_ccall trf_7962(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_7962(t0,t1,t2,t3);}

C_noret_decl(trf_8011)
static void C_ccall trf_8011(C_word c,C_word *av) C_noret;
static void C_ccall trf_8011(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8011(t0,t1,t2);}

C_noret_decl(trf_8054)
static void C_ccall trf_8054(C_word c,C_word *av) C_noret;
static void C_ccall trf_8054(C_word c,C_word *av){
C_word t0=av[7];
C_word t1=av[6];
C_word t2=av[5];
C_word t3=av[4];
C_word t4=av[3];
C_word t5=av[2];
C_word t6=av[1];
C_word t7=av[0];
f_8054(t0,t1,t2,t3,t4,t5,t6,t7);}

C_noret_decl(trf_8077)
static void C_ccall trf_8077(C_word c,C_word *av) C_noret;
static void C_ccall trf_8077(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8077(t0,t1,t2);}

C_noret_decl(trf_8091)
static void C_ccall trf_8091(C_word c,C_word *av) C_noret;
static void C_ccall trf_8091(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8091(t0,t1,t2);}

C_noret_decl(trf_8105)
static void C_ccall trf_8105(C_word c,C_word *av) C_noret;
static void C_ccall trf_8105(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8105(t0,t1,t2);}

C_noret_decl(trf_8144)
static void C_ccall trf_8144(C_word c,C_word *av) C_noret;
static void C_ccall trf_8144(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8144(t0,t1,t2);}

C_noret_decl(trf_8196)
static void C_ccall trf_8196(C_word c,C_word *av) C_noret;
static void C_ccall trf_8196(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8196(t0,t1,t2);}

C_noret_decl(trf_8312)
static void C_ccall trf_8312(C_word c,C_word *av) C_noret;
static void C_ccall trf_8312(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8312(t0,t1,t2);}

C_noret_decl(trf_8362)
static void C_ccall trf_8362(C_word c,C_word *av) C_noret;
static void C_ccall trf_8362(C_word c,C_word *av){
C_word t0=av[8];
C_word t1=av[7];
C_word t2=av[6];
C_word t3=av[5];
C_word t4=av[4];
C_word t5=av[3];
C_word t6=av[2];
C_word t7=av[1];
C_word t8=av[0];
f_8362(t0,t1,t2,t3,t4,t5,t6,t7,t8);}

C_noret_decl(trf_8443)
static void C_ccall trf_8443(C_word c,C_word *av) C_noret;
static void C_ccall trf_8443(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8443(t0,t1,t2);}

C_noret_decl(trf_8663)
static void C_ccall trf_8663(C_word c,C_word *av) C_noret;
static void C_ccall trf_8663(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8663(t0,t1,t2);}

C_noret_decl(trf_8694)
static void C_ccall trf_8694(C_word c,C_word *av) C_noret;
static void C_ccall trf_8694(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_8694(t0,t1,t2);}

C_noret_decl(trf_8773)
static void C_ccall trf_8773(C_word c,C_word *av) C_noret;
static void C_ccall trf_8773(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_8773(t0,t1);}

C_noret_decl(trf_8946)
static void C_ccall trf_8946(C_word c,C_word *av) C_noret;
static void C_ccall trf_8946(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_8946(t0,t1,t2,t3);}

C_noret_decl(trf_9020)
static void C_ccall trf_9020(C_word c,C_word *av) C_noret;
static void C_ccall trf_9020(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9020(t0,t1,t2,t3);}

C_noret_decl(trf_9072)
static void C_ccall trf_9072(C_word c,C_word *av) C_noret;
static void C_ccall trf_9072(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9072(t0,t1,t2);}

C_noret_decl(trf_9183)
static void C_ccall trf_9183(C_word c,C_word *av) C_noret;
static void C_ccall trf_9183(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9183(t0,t1,t2);}

C_noret_decl(trf_9217)
static void C_ccall trf_9217(C_word c,C_word *av) C_noret;
static void C_ccall trf_9217(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9217(t0,t1,t2);}

C_noret_decl(trf_9360)
static void C_ccall trf_9360(C_word c,C_word *av) C_noret;
static void C_ccall trf_9360(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9360(t0,t1,t2,t3);}

C_noret_decl(trf_9408)
static void C_ccall trf_9408(C_word c,C_word *av) C_noret;
static void C_ccall trf_9408(C_word c,C_word *av){
C_word t0=av[3];
C_word t1=av[2];
C_word t2=av[1];
C_word t3=av[0];
f_9408(t0,t1,t2,t3);}

C_noret_decl(trf_9456)
static void C_ccall trf_9456(C_word c,C_word *av) C_noret;
static void C_ccall trf_9456(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9456(t0,t1,t2);}

C_noret_decl(trf_9490)
static void C_ccall trf_9490(C_word c,C_word *av) C_noret;
static void C_ccall trf_9490(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9490(t0,t1,t2);}

C_noret_decl(trf_9535)
static void C_ccall trf_9535(C_word c,C_word *av) C_noret;
static void C_ccall trf_9535(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9535(t0,t1,t2);}

C_noret_decl(trf_9579)
static void C_ccall trf_9579(C_word c,C_word *av) C_noret;
static void C_ccall trf_9579(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9579(t0,t1,t2);}

C_noret_decl(trf_9670)
static void C_ccall trf_9670(C_word c,C_word *av) C_noret;
static void C_ccall trf_9670(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9670(t0,t1);}

C_noret_decl(trf_9766)
static void C_ccall trf_9766(C_word c,C_word *av) C_noret;
static void C_ccall trf_9766(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
f_9766(t0,t1);}

C_noret_decl(trf_9802)
static void C_ccall trf_9802(C_word c,C_word *av) C_noret;
static void C_ccall trf_9802(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9802(t0,t1,t2);}

C_noret_decl(trf_9960)
static void C_ccall trf_9960(C_word c,C_word *av) C_noret;
static void C_ccall trf_9960(C_word c,C_word *av){
C_word t0=av[2];
C_word t1=av[1];
C_word t2=av[0];
f_9960(t0,t1,t2);}

C_noret_decl(trva26382)
static void C_ccall trva26382(C_word c,C_word *av) C_noret;
static void C_ccall trva26382(C_word c,C_word *av){
C_word t0=av[1];
C_word t1=av[0];
va26382(t0,t1);}

/* f25485 in k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f25485(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f25485,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[343];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10012 in map-loop1627 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10014(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_10014,c,av);}
a=C_alloc(12);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],C_SCHEME_FALSE,t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[3])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,t3);
t6=((C_word*)((C_word*)t0)[4])[1];
f_10130(t6,((C_word*)t0)[5],C_slot(((C_word*)t0)[6],C_fix(1)));}

/* k10016 in map-loop1627 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10018(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10018,c,av);}
/* core.scm:887: ##sys#ensure-transformer */
t2=*((C_word*)lf[196]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10033(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_10033,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10036,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10128,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:891: ##sys#current-environment */
t4=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_10036,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_10039,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* core.scm:892: chicken.compiler.support#get-line */
t3=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10039(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_10039,c,av);}
a=C_alloc(29);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10043,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t4=C_i_check_list_2(((C_word*)t0)[4],lf[100]);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10103,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t6=(
  f_10103(t5,((C_word*)t0)[4])
);
t7=*((C_word*)lf[107]+1);
t8=((C_word*)t0)[3];
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_TRUE;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10062,a[2]=t9,a[3]=t11,a[4]=t7,tmp=(C_word)a,a+=5,tmp);
t13=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10079,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=t2,a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],tmp=(C_word)a,a+=10,tmp);
t14=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10093,a[2]=t9,a[3]=t7,tmp=(C_word)a,a+=4,tmp);
/* core.scm:897: ##sys#dynamic-wind */
t15=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t15;
av2[1]=((C_word*)t0)[12];
av2[2]=t12;
av2[3]=t13;
av2[4]=t14;
((C_proc)(void*)(*((C_word*)t15+1)))(5,av2);}}

/* g1660 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static C_word C_fcall f_10043(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_i_set_car(C_i_cdr(t1),((C_word*)t0)[2]));}

/* a10061 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10062(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10062,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10066,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[3])[1])){
/* core.scm:897: ##sys#current-environment1668 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_10066(2,av2);}}}

/* k10064 in a10061 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10066(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10066,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10070,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* core.scm:897: ##sys#current-environment1668 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10068 in k10064 in a10061 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_10070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_10070,c,av);}
a=C_alloc(4);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10073,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:897: ##sys#current-environment1668 */
t4=((C_word*)t0)[5];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k10071 in k10068 in k10064 in a10061 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_10073(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10073,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a10078 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10079(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_10079,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10087,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* core.scm:899: canonicalize-body/ln */
t3=((C_word*)((C_word*)t0)[8])[1];
f_7916(t3,t2,((C_word*)t0)[7],C_i_cddr(((C_word*)t0)[9]),*((C_word*)lf[52]+1));}

/* k10085 in a10078 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_10087,c,av);}
/* core.scm:898: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* a10092 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10093(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10093,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10097,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:897: ##sys#current-environment1668 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10095 in a10092 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_10097,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10100,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:897: ##sys#current-environment1668 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10098 in k10095 in a10092 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_10100(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10100,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* for-each-loop1659 in k10037 in k10034 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static C_word C_fcall f_10103(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=(
/* core.scm:893: g1660 */
  f_10043(((C_word*)t0)[2],C_slot(t1,C_fix(0)))
);
t4=C_slot(t1,C_fix(1));
t1=t4;
goto loop;}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* k10126 in k10031 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10128(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10128,c,av);}
/* core.scm:891: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop1627 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_10130(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_10130,3,t0,t1,t2);}
a=C_alloc(11);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10014,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t2,tmp=(C_word)a,a+=7,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10018,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* core.scm:888: ##sys#eval/meta */
t7=*((C_word*)lf[197]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_cadr(t3);
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10171(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_10171,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10174,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t3=C_i_cadr(((C_word*)t0)[10]);
if(C_truep(C_i_pairp(t3))){
/* core.scm:910: scheme#caadr */
t4=*((C_word*)lf[165]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_u_i_car(C_u_i_cdr(((C_word*)t0)[10]));
f_10174(2,av2);}}}

/* k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10174(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_10174,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10177,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t3=C_u_i_cdr(((C_word*)t0)[10]);
if(C_truep(C_i_pairp(C_u_i_car(t3)))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10230,a[2]=((C_word*)t0)[10],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:912: scheme#cdadr */
t5=*((C_word*)lf[205]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=t2;
f_10177(t4,C_i_caddr(((C_word*)t0)[10]));}}

/* k10175 in k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_10177(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_10177,2,t0,t1);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10180,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* core.scm:914: lookup */
t3=((C_word*)((C_word*)t0)[10])[1];
f_7842(t3,t2,((C_word*)t0)[2]);}

/* k10178 in k10175 in k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10180(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_10180,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10183,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,tmp=(C_word)a,a+=12,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10216,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:915: ##sys#current-module */
t4=*((C_word*)lf[204]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10181 in k10178 in k10175 in k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10183(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_10183,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10186,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10208,a[2]=t2,a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:918: ##sys#current-environment */
t4=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10184 in k10181 in k10178 in k10175 in k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_10186(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,8)))){
C_save_and_reclaim((void *)f_10186,c,av);}
a=C_alloc(21);
if(C_truep(*((C_word*)lf[200]+1))){
t2=C_a_i_list(&a,2,lf[146],((C_word*)t0)[2]);
t3=C_a_i_list(&a,1,lf[107]);
t4=C_a_i_list(&a,4,lf[201],t2,t3,((C_word*)t0)[3]);
/* core.scm:920: walk */
t5=((C_word*)((C_word*)t0)[4])[1];
f_8362(t5,((C_word*)t0)[5],t4,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],C_SCHEME_FALSE);}
else{
/* core.scm:920: walk */
t2=((C_word*)((C_word*)t0)[4])[1];
f_8362(t2,((C_word*)t0)[5],lf[202],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],C_SCHEME_FALSE);}}

/* k10206 in k10181 in k10178 in k10175 in k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_10208(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10208,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10212,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:919: ##sys#eval/meta */
t3=*((C_word*)lf[197]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10210 in k10206 in k10181 in k10178 in k10175 in k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_10212(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10212,c,av);}
/* core.scm:916: ##sys#extend-macro-environment */
t2=*((C_word*)lf[201]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k10214 in k10178 in k10175 in k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10216(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10216,c,av);}
/* core.scm:915: ##sys#register-syntax-export */
t2=*((C_word*)lf[203]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k10228 in k10172 in k10169 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10230(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_10230,c,av);}
a=C_alloc(6);
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_u_i_cdr(t2);
t4=C_a_i_cons(&a,2,t1,t3);
t5=((C_word*)t0)[3];
f_10177(t5,C_a_i_cons(&a,2,lf[97],t4));}

/* k10259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10261(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_10261,c,av);}
/* core.scm:904: ##sys#check-syntax */
t2=*((C_word*)lf[208]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=C_SCHEME_FALSE;
av2[6]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* k10281 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_10283,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10286,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[2])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10353,a[2]=((C_word*)t0)[11],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:936: ##sys#get */
t4=*((C_word*)lf[103]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
av2[3]=lf[210];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=t2;
f_10286(t3,C_SCHEME_UNDEFINED);}}

/* k10284 in k10281 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_10286(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,4)))){
C_save_and_reclaim_args((void *)trf_10286,2,t0,t1);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10289,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[3])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10337,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10345,a[2]=t3,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:943: ##sys#eval/meta */
t5=*((C_word*)lf[197]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
/* core.scm:938: ##sys#put! */
t3=*((C_word*)lf[108]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[210];
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k10287 in k10284 in k10281 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10289(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(51,c,8)))){
C_save_and_reclaim((void *)f_10289,c,av);}
a=C_alloc(51);
if(C_truep(*((C_word*)lf[200]+1))){
t2=C_a_i_list(&a,2,lf[145],((C_word*)t0)[2]);
t3=C_a_i_list(&a,2,lf[118],lf[210]);
if(C_truep(((C_word*)t0)[3])){
t4=C_a_i_list(&a,2,lf[146],((C_word*)t0)[4]);
t5=C_a_i_list(&a,3,lf[196],((C_word*)t0)[3],t4);
t6=C_a_i_list(&a,1,lf[107]);
t7=C_a_i_list(&a,3,lf[211],t5,t6);
t8=C_a_i_list(&a,4,lf[108],t2,t3,t7);
/* core.scm:946: walk */
t9=((C_word*)((C_word*)t0)[5])[1];
f_8362(t9,((C_word*)t0)[6],t8,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],C_SCHEME_FALSE);}
else{
t4=C_a_i_list(&a,4,lf[108],t2,t3,C_SCHEME_FALSE);
/* core.scm:946: walk */
t5=((C_word*)((C_word*)t0)[5])[1];
f_8362(t5,((C_word*)t0)[6],t4,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],C_SCHEME_FALSE);}}
else{
/* core.scm:946: walk */
t2=((C_word*)((C_word*)t0)[5])[1];
f_8362(t2,((C_word*)t0)[6],lf[212],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],C_SCHEME_FALSE);}}

/* k10335 in k10284 in k10281 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10337(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_10337,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10341,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:945: ##sys#current-environment */
t3=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10339 in k10335 in k10284 in k10281 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10341(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_10341,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* core.scm:938: ##sys#put! */
t3=*((C_word*)lf[108]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=lf[210];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10343 in k10284 in k10281 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10345,c,av);}
/* core.scm:942: ##sys#ensure-transformer */
t2=*((C_word*)lf[196]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k10351 in k10281 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10353(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_10353,c,av);}
a=C_alloc(6);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,t4);
t6=((C_word*)t0)[4];
f_10286(t6,t5);}

/* g1726 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_10363(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_10363,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10367,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* core.scm:963: ##sys#check-syntax */
t4=*((C_word*)lf[208]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[214];
av2[3]=t2;
av2[4]=lf[215];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k10365 in g1726 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10367(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_10367,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10370,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:965: lookup */
t3=((C_word*)((C_word*)t0)[4])[1];
f_7842(t3,t2,C_i_car(((C_word*)t0)[3]));}

/* k10368 in k10365 in g1726 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10370(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_10370,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10377,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[3])))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10391,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10399,a[2]=t3,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:970: ##sys#eval/meta */
t5=*((C_word*)lf[197]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=t2;
f_10377(t3,C_SCHEME_FALSE);}}

/* k10375 in k10368 in k10365 in g1726 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_10377(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_10377,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10381,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:973: ##sys#get */
t3=*((C_word*)lf[103]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[210];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k10379 in k10375 in k10368 in k10365 in g1726 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_10381,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list3(&a,3,((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k10389 in k10368 in k10365 in g1726 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_10391,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10395,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:972: ##sys#current-environment */
t3=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10393 in k10389 in k10368 in k10365 in g1726 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10395(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_10395,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];
f_10377(t2,C_a_i_cons(&a,2,((C_word*)t0)[3],t1));}

/* k10397 in k10368 in k10365 in g1726 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_10399,c,av);}
/* core.scm:969: ##sys#ensure-transformer */
t2=*((C_word*)lf[196]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_10420,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_10423,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* core.scm:975: chicken.compiler.support#get-line */
t3=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10423(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,4)))){
C_save_and_reclaim((void *)f_10423,c,av);}
a=C_alloc(17);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10431,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10476,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=t2,a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],tmp=(C_word)a,a+=11,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10490,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* core.scm:976: scheme#dynamic-wind */
t6=*((C_word*)lf[216]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[12];
av2[2]=t3;
av2[3]=t4;
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* a10430 in k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_10431,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10453,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_10453(t6,t1,((C_word*)t0)[2]);}

/* for-each-loop1753 in a10430 in k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_10453(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_10453,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10463,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
/* core.scm:980: ##sys#put! */
t5=*((C_word*)lf[108]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=C_i_car(t4);
av2[3]=lf[210];
av2[4]=C_i_cadr(t4);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k10461 in for-each-loop1753 in a10430 in k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10463(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10463,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_10453(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* a10475 in k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10476(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_10476,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10484,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:984: canonicalize-body/ln */
t3=((C_word*)((C_word*)t0)[9])[1];
f_7916(t3,t2,((C_word*)t0)[7],C_i_cddr(((C_word*)t0)[10]),*((C_word*)lf[52]+1));}

/* k10482 in a10475 in k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10484(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_10484,c,av);}
/* core.scm:983: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* a10489 in k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_10490,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10512,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_10512(t6,t1,((C_word*)t0)[2]);}

/* for-each-loop1771 in a10489 in k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_10512(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_10512,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10522,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
/* core.scm:990: ##sys#put! */
t5=*((C_word*)lf[108]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t3;
av2[2]=C_i_car(t4);
av2[3]=lf[210];
av2[4]=C_i_caddr(t4);
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k10520 in for-each-loop1771 in a10489 in k10421 in k10418 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10522(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10522,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_10512(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* map-loop1720 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_10535(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10535,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10560,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:961: g1726 */
t4=((C_word*)t0)[4];
f_10363(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10558 in map-loop1720 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10560(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10560,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10535(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a10577 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10578(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10578,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[219]+1));
t3=C_mutate((C_word*)lf[219]+1 /* (set! ##sys#default-read-info-hook ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a10582 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10583(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_10583,c,av);}
a=C_alloc(11);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=C_i_caddr(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10597,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[2],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:997: ##sys#include-forms-from-file */
t5=*((C_word*)lf[220]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* a10596 in a10582 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,8)))){
C_save_and_reclaim((void *)f_10597,c,av);}
a=C_alloc(15);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10605,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t4=C_i_cdddr(((C_word*)t0)[9]);
if(C_truep(C_i_pairp(t4))){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10615,a[2]=((C_word*)t0)[10],a[3]=t3,a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1004: scheme#append */
t6=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
av2[3]=C_i_cadddr(((C_word*)t0)[9]);
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t5=C_a_i_cons(&a,2,lf[190],t2);
/* core.scm:1001: walk */
t6=((C_word*)((C_word*)t0)[2])[1];
f_8362(t6,t1,t5,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}}

/* k10603 in a10596 in a10582 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10605(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_10605,c,av);}
/* core.scm:1001: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k10613 in a10596 in a10582 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10615,c,av);}
/* core.scm:1002: canonicalize-body/ln */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7916(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,*((C_word*)lf[52]+1));}

/* a10627 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10628(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10628,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[219]+1));
t3=C_mutate((C_word*)lf[219]+1 /* (set! ##sys#default-read-info-hook ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10647 in map-loop1803 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10649,c,av);}
/* core.scm:1013: chicken.syntax#strip-syntax */
t2=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k10659 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_10661,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10663,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1010: ##sys#with-module-aliases */
t3=*((C_word*)lf[222]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[9];
av2[2]=t1;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a10662 in k10659 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10663(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,8)))){
C_save_and_reclaim((void *)f_10663,c,av);}
a=C_alloc(3);
t2=C_i_cddr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[190],t2);
/* core.scm:1016: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_8362(t4,t1,t3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_TRUE);}

/* map-loop1803 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_10677(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_10677,3,t0,t1,t2);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10702,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10649,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1012: ##sys#check-syntax */
t6=*((C_word*)lf[208]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[223];
av2[3]=t4;
av2[4]=lf[224];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10700 in map-loop1803 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_10702(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10702,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10677(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_10718(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_10718,c,av);}
a=C_alloc(12);
t2=C_i_assq(t1,*((C_word*)lf[44]+1));
t3=(C_truep(t2)?t2:*((C_word*)lf[45]+1));
t4=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10727,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t3,a[10]=t1,a[11]=((C_word*)t0)[9],tmp=(C_word)a,a+=12,tmp);
if(C_truep(C_i_not(*((C_word*)lf[48]+1)))){
if(C_truep(t3)){
t5=*((C_word*)lf[17]+1);
t6=t4;
f_10727(t6,(C_truep(*((C_word*)lf[17]+1))?*((C_word*)lf[17]+1):t1));}
else{
t5=t4;
f_10727(t5,C_SCHEME_FALSE);}}
else{
t5=t4;
f_10727(t5,C_SCHEME_FALSE);}}

/* k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_10727(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(31,0,4)))){
C_save_and_reclaim_args((void *)trf_10727,2,t0,t1);}
a=C_alloc(31);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10730,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
t3=C_i_caddr(((C_word*)t0)[11]);
t4=C_eqp(C_SCHEME_TRUE,t3);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11160,a[2]=t2,a[3]=((C_word*)t0)[10],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
if(C_truep(t4)){
/* core.scm:1022: ##sys#register-module */
t6=*((C_word*)lf[272]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t2;
av2[2]=((C_word*)t0)[10];
av2[3]=t1;
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11164,a[2]=((C_word*)t0)[10],tmp=(C_word)a,a+=3,tmp);
t11=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11213,a[2]=t8,a[3]=t10,a[4]=t9,a[5]=t5,tmp=(C_word)a,a+=6,tmp);
t12=C_u_i_cdr(((C_word*)t0)[11]);
t13=C_u_i_cdr(t12);
/* core.scm:1037: chicken.syntax#strip-syntax */
t14=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t14;
av2[1]=t11;
av2[2]=C_u_i_car(t13);
((C_proc)(void*)(*((C_word*)t14+1)))(3,av2);}}}

/* k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_10730(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_10730,c,av);}
a=C_alloc(18);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_10733,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t2,a[12]=((C_word*)t0)[2],a[13]=((C_word*)t0)[11],tmp=(C_word)a,a+=14,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11150,a[2]=t3,a[3]=((C_word*)t0)[10],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1039: ##sys#current-module */
t5=*((C_word*)lf[204]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_10733(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_10733,c,av);}
a=C_alloc(19);
t2=*((C_word*)lf[204]+1);
t3=*((C_word*)lf[107]+1);
t4=*((C_word*)lf[226]+1);
t5=*((C_word*)lf[227]+1);
t6=*((C_word*)lf[228]+1);
t7=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_10736,a[2]=((C_word*)t0)[2],a[3]=t6,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=t5,a[15]=t4,a[16]=t3,a[17]=t2,a[18]=((C_word*)t0)[13],tmp=(C_word)a,a+=19,tmp);
/* core.scm:1047: ##sys#module-alias-environment */
t8=*((C_word*)lf[227]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}

/* k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_10736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(52,c,4)))){
C_save_and_reclaim((void *)f_10736,c,av);}
a=C_alloc(52);
t2=((C_word*)t0)[2];
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_END_OF_LIST;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t0)[3];
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=t1;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_TRUE;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_10739,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
t13=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10937,a[2]=t3,a[3]=t5,a[4]=t7,a[5]=t9,a[6]=t11,a[7]=((C_word*)t0)[14],a[8]=((C_word*)t0)[15],a[9]=((C_word*)t0)[16],a[10]=((C_word*)t0)[17],tmp=(C_word)a,a+=11,tmp);
t14=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10993,a[2]=((C_word*)t0)[18],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
t15=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11118,a[2]=t3,a[3]=t5,a[4]=t7,a[5]=t9,a[6]=((C_word*)t0)[14],a[7]=((C_word*)t0)[15],a[8]=((C_word*)t0)[16],a[9]=((C_word*)t0)[17],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1042: ##sys#dynamic-wind */
t16=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t16;
av2[1]=t12;
av2[2]=t13;
av2[3]=t14;
av2[4]=t15;
((C_proc)(void*)(*((C_word*)t16+1)))(5,av2);}}

/* k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_10739(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_10739,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10742,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10912,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=t4,tmp=(C_word)a,a+=5,tmp));
t6=((C_word*)t4)[1];
f_10912(t6,t2,((C_word*)((C_word*)t0)[12])[1]);}

/* k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_10742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,2)))){
C_save_and_reclaim((void *)f_10742,c,av);}
a=C_alloc(24);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10745,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[10])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10900,a[2]=((C_word*)t0)[10],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8271,a[2]=t3,a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[9],tmp=(C_word)a,a+=5,tmp);
if(C_truep(*((C_word*)lf[45]+1))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8356,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* core.scm:614: scheme#symbol->string */
t6=*((C_word*)lf[255]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_cdr(((C_word*)t0)[10]);
f_8271(2,av2);}}}
else{
t3=t2;
f_10745(t3,C_SCHEME_UNDEFINED);}}

/* k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_fcall f_10745(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_10745,2,t0,t1);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10752,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10897,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:1087: chicken.internal#module-requirement */
t4=*((C_word*)lf[234]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k10750 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_10752(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10752,c,av);}
/* core.scm:1085: chicken.compiler.support#canonicalize-begin-body */
t2=*((C_word*)lf[229]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k10758 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_10760(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_10760,c,av);}
/* core.scm:1086: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_fcall f_10766(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_10766,2,t0,t1);}
a=C_alloc(11);
if(C_truep(t1)){
t2=*((C_word*)lf[226]+1);
t3=*((C_word*)lf[107]+1);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10769,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* core.scm:1092: ##sys#meta-macro-environment */
t5=*((C_word*)lf[233]+1);{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* core.scm:1086: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word av2[5];
av2[0]=t2;
av2[1]=((C_word*)t0)[9];
av2[2]=((C_word*)t0)[10];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_10769(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_10769,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10772,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* core.scm:1094: ##sys#current-meta-environment */
t3=*((C_word*)lf[232]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_10772(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,4)))){
C_save_and_reclaim((void *)f_10772,c,av);}
a=C_alloc(27);
t2=((C_word*)t0)[2];
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=t1;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_TRUE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10777,a[2]=t3,a[3]=t5,a[4]=t7,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
t9=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10807,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],tmp=(C_word)a,a+=8,tmp);
t10=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10865,a[2]=t3,a[3]=t5,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1091: ##sys#dynamic-wind */
t11=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t11;
av2[1]=((C_word*)t0)[11];
av2[2]=t8;
av2[3]=t9;
av2[4]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(5,av2);}}

/* a10776 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_10777(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_10777,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10781,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)((C_word*)t0)[4])[1])){
/* core.scm:1091: ##sys#macro-environment1975 */
t3=((C_word*)t0)[6];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_10781(2,av2);}}}

/* k10779 in a10776 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_10781(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_10781,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10784,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)((C_word*)t0)[4])[1])){
/* core.scm:1091: ##sys#current-environment1976 */
t3=((C_word*)t0)[6];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[3])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[3])[1];
f_10784(2,av2);}}}

/* k10782 in k10779 in a10776 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in ... */
static void C_ccall f_10784(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_10784,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10788,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1091: ##sys#macro-environment1975 */
t3=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10786 in k10782 in k10779 in a10776 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in ... */
static void C_ccall f_10788(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_10788,c,av);}
a=C_alloc(9);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_10792,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1091: ##sys#current-environment1976 */
t4=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10790 in k10786 in k10782 in k10779 in a10776 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in ... */
static void C_ccall f_10792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10792,c,av);}
a=C_alloc(6);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10795,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1091: ##sys#macro-environment1975 */
t4=((C_word*)t0)[7];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k10793 in k10790 in k10786 in k10782 in k10779 in a10776 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in ... */
static void C_ccall f_10795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_10795,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10798,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1091: ##sys#current-environment1976 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10796 in k10793 in k10790 in k10786 in k10782 in k10779 in a10776 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in ... */
static void C_ccall f_10798(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10798,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a10806 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_10807(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_10807,c,av);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10812,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10819,a[2]=t4,a[3]=t6,a[4]=t5,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
if(C_truep(*((C_word*)lf[56]+1))){
/* core.scm:1096: ##sys#compiled-module-registration */
t8=*((C_word*)lf[231]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[7];
av2[3]=lf[171];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
/* core.scm:1096: ##sys#compiled-module-registration */
t8=*((C_word*)lf[231]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=((C_word*)t0)[7];
av2[3]=lf[172];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}}

/* g2006 in a10806 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_fcall f_10812(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,8)))){
C_save_and_reclaim_args((void *)trf_10812,3,t0,t1,t2);}
/* core.scm:1095: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8362(t3,t1,t2,((C_word*)t0)[3],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k10817 in a10806 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_10819(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_10819,c,av);}
a=C_alloc(8);
t2=C_i_check_list_2(t1,lf[10]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10827,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_10827(t6,((C_word*)t0)[5],t1);}

/* map-loop2000 in k10817 in a10806 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in ... */
static void C_fcall f_10827(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_10827,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10852,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1095: g2006 */
t4=((C_word*)t0)[4];
f_10812(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k10850 in map-loop2000 in k10817 in a10806 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in ... */
static void C_ccall f_10852(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_10852,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_10827(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a10864 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_10865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_10865,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10869,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1091: ##sys#macro-environment1975 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10867 in a10864 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_10869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_10869,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10872,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:1091: ##sys#current-environment1976 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10870 in k10867 in a10864 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in ... */
static void C_ccall f_10872(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_10872,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10875,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:1091: ##sys#macro-environment1975 */
t3=((C_word*)t0)[7];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10873 in k10870 in k10867 in a10864 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in ... */
static void C_ccall f_10875(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_10875,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_10878,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1091: ##sys#current-environment1976 */
t3=((C_word*)t0)[7];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10876 in k10873 in k10870 in k10867 in a10864 in k10770 in k10767 in k10764 in k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in ... */
static void C_ccall f_10878(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10878,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=C_mutate(((C_word *)((C_word*)t0)[4])+1,((C_word*)t0)[5]);
t4=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10895 in k10743 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_10897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,2)))){
C_save_and_reclaim((void *)f_10897,c,av);}
a=C_alloc(26);
t2=C_a_i_list2(&a,2,lf[163],t1);
t3=C_a_i_list1(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10760,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t5=C_eqp(*((C_word*)lf[47]+1),lf[230]);
t6=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10766,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=t4,a[9]=((C_word*)t0)[2],a[10]=t3,a[11]=((C_word*)t0)[3],tmp=(C_word)a,a+=12,tmp);
if(C_truep(t5)){
t7=t6;
f_10766(t7,t5);}
else{
t7=C_i_not(((C_word*)t0)[10]);
t8=t6;
f_10766(t8,(C_truep(t7)?C_i_not(*((C_word*)lf[47]+1)):C_SCHEME_FALSE));}}

/* k10898 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_10900(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_10900,c,av);}
a=C_alloc(3);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10910,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1084: delete */
f_6795(t2,((C_word*)t0)[2],*((C_word*)lf[44]+1),*((C_word*)lf[235]+1));}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];
f_10745(t3,t2);}}

/* k10908 in k10898 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_10910(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10910,c,av);}
t2=C_mutate((C_word*)lf[44]+1 /* (set! chicken.compiler.core#import-libraries ...) */,t1);
t3=((C_word*)t0)[2];
f_10745(t3,t2);}

/* doloop1964 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_fcall f_10912(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_10912,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_eqp(t2,((C_word*)t0)[2]);
if(C_truep(t3)){
t4=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)t0)[2]);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10923,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=C_i_caar(t2);
t6=C_u_i_car(t2);
/* core.scm:1079: ##sys#put! */
t7=*((C_word*)lf[108]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t4;
av2[2]=t5;
av2[3]=lf[210];
av2[4]=C_u_i_cdr(t6);
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}}

/* k10921 in doloop1964 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_10923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_10923,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_10912(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_10937(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_10937,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10941,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)((C_word*)t0)[6])[1])){
/* core.scm:1042: ##sys#current-module1889 */
t3=((C_word*)t0)[10];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_10941(2,av2);}}}

/* k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_10941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_10941,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_10944,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,tmp=(C_word)a,a+=13,tmp);
if(C_truep(((C_word*)((C_word*)t0)[6])[1])){
/* core.scm:1042: ##sys#current-environment1890 */
t3=((C_word*)t0)[10];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[3])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[3])[1];
f_10944(2,av2);}}}

/* k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_10944(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_10944,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_10947,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
if(C_truep(((C_word*)((C_word*)t0)[6])[1])){
/* core.scm:1042: ##sys#macro-environment1891 */
t3=((C_word*)t0)[9];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[4])[1];
f_10947(2,av2);}}}

/* k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_10947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_10947,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_10950,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
if(C_truep(((C_word*)((C_word*)t0)[6])[1])){
/* core.scm:1042: ##sys#module-alias-environment1892 */
t3=((C_word*)t0)[8];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[5])[1];
f_10950(2,av2);}}}

/* k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_10950(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_10950,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_10954,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* core.scm:1042: ##sys#current-module1889 */
t3=((C_word*)t0)[13];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k10952 in k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_10954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_10954,c,av);}
a=C_alloc(15);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_10958,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[15],tmp=(C_word)a,a+=15,tmp);
/* core.scm:1042: ##sys#current-environment1890 */
t4=((C_word*)t0)[12];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10956 in k10952 in k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_10958(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_10958,c,av);}
a=C_alloc(14);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_10962,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],tmp=(C_word)a,a+=14,tmp);
/* core.scm:1042: ##sys#macro-environment1891 */
t4=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10960 in k10956 in k10952 in k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_10962(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_10962,c,av);}
a=C_alloc(13);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_10966,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1042: ##sys#module-alias-environment1892 */
t4=((C_word*)t0)[6];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k10964 in k10960 in k10956 in k10952 in k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_10966(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_10966,c,av);}
a=C_alloc(10);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10969,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1042: ##sys#current-module1889 */
t4=((C_word*)t0)[11];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[12];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k10967 in k10964 in k10960 in k10956 in k10952 in k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in ... */
static void C_ccall f_10969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_10969,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10972,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* core.scm:1042: ##sys#current-environment1890 */
t3=((C_word*)t0)[8];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10970 in k10967 in k10964 in k10960 in k10956 in k10952 in k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in ... */
static void C_ccall f_10972(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_10972,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10975,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1042: ##sys#macro-environment1891 */
t3=((C_word*)t0)[6];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10973 in k10970 in k10967 in k10964 in k10960 in k10956 in k10952 in k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in ... */
static void C_ccall f_10975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_10975,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10978,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1042: ##sys#module-alias-environment1892 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k10976 in k10973 in k10970 in k10967 in k10964 in k10960 in k10956 in k10952 in k10948 in k10945 in k10942 in k10939 in a10936 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in ... */
static void C_ccall f_10978(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_10978,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_10993(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_10993,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_10999,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* core.scm:1048: ##sys#with-property-restore */
t3=*((C_word*)lf[268]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_10999(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_10999,c,av);}
a=C_alloc(10);
t2=C_i_cdddr(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11009,a[2]=((C_word*)t0)[3],a[3]=t4,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp));
t6=((C_word*)t4)[1];
f_11009(t6,t1,t2,C_SCHEME_END_OF_LIST);}

/* loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_fcall f_11009(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,8)))){
C_save_and_reclaim_args((void *)trf_11009,4,t0,t1,t2,t3);}
a=C_alloc(7);
if(C_truep(C_i_nullp(t2))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11019,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11024,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1052: scheme#call-with-current-continuation */
t6=*((C_word*)lf[267]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t4;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=C_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11114,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1073: walk */
t6=((C_word*)((C_word*)t0)[4])[1];
f_8362(t6,t5,C_u_i_car(t2),((C_word*)t0)[5],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_TRUE);}}

/* k11017 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11019(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11019,c,av);}
/* core.scm:1052: g1940 */
t2=t1;{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11024(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_11024,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11030,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11045,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1052: chicken.condition#with-exception-handler */
t5=*((C_word*)lf[266]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* a11029 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11030(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11030,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11036,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1052: k1937 */
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a11035 in a11029 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_11036(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_11036,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11040,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1055: chicken.condition#print-error-message */
t3=*((C_word*)lf[257]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=*((C_word*)lf[258]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11038 in a11035 in a11029 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11040(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11040,c,av);}
/* core.scm:1056: chicken.base#exit */
t2=*((C_word*)lf[256]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_fix(1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11045(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_11045,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11047,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11082,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11099,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* tmp15796 */
t5=t2;
f_11047(t5,t4);}

/* tmp15796 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_fcall f_11047(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_11047,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11051,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11056,tmp=(C_word)a,a+=2,tmp);
/* core.scm:1057: ##sys#finalize-module */
t4=*((C_word*)lf[265]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k11049 in tmp15796 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11051(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11051,c,av);}
/* core.scm:1070: scheme#reverse */
t2=*((C_word*)lf[259]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a11055 in tmp15796 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11056(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11056,c,av);}
a=C_alloc(4);
if(C_truep(C_i_assq(t2,lf[70]))){
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[260];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11069,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1063: chicken.internal#hash-table-ref */
t4=*((C_word*)lf[112]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[65]+1);
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k11067 in a11055 in tmp15796 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_11069(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11069,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[261];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11075,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1065: chicken.internal#hash-table-ref */
t3=*((C_word*)lf[112]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[66]+1);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k11073 in k11067 in a11055 in tmp15796 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_11075(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_11075,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[262];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11081,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1067: ##sys#get */
t3=*((C_word*)lf[103]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[264];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k11079 in k11073 in k11067 in a11055 in tmp15796 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in ... */
static void C_ccall f_11081(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11081,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?lf[263]:C_SCHEME_FALSE);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* tmp25797 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_fcall f_11082(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_11082,3,t0,t1,t2);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11088,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1052: k1937 */
t4=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a11087 in tmp25797 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11088(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11088,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
C_apply_values(3,av2);}}

/* k11097 in a11044 in a11023 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_11099(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11099,c,av);}
a=C_alloc(3);
/* tmp25797 */
t2=((C_word*)t0)[2];
f_11082(t2,((C_word*)t0)[3],C_a_i_list(&a,1,t1));}

/* k11112 in loop in a10998 in a10992 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11114(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_11114,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* core.scm:1071: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_11009(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11118(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_11118,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11122,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* core.scm:1042: ##sys#current-module1889 */
t3=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11120 in a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_11122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_11122,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_11125,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* core.scm:1042: ##sys#current-environment1890 */
t3=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11123 in k11120 in a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_11125(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_11125,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11128,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1042: ##sys#macro-environment1891 */
t3=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11126 in k11123 in k11120 in a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11128(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_11128,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_11131,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* core.scm:1042: ##sys#module-alias-environment1892 */
t3=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11129 in k11126 in k11123 in k11120 in a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11131(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_11131,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_11134,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* core.scm:1042: ##sys#current-module1889 */
t3=((C_word*)t0)[13];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11132 in k11129 in k11126 in k11123 in k11120 in a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_11134(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_11134,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11137,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1042: ##sys#current-environment1890 */
t3=((C_word*)t0)[13];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11135 in k11132 in k11129 in k11126 in k11123 in k11120 in a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11137(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_11137,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_11140,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* core.scm:1042: ##sys#macro-environment1891 */
t3=((C_word*)t0)[12];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[6])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11138 in k11135 in k11132 in k11129 in k11126 in k11123 in k11120 in a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_11140(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_11140,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11143,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:1042: ##sys#module-alias-environment1892 */
t3=((C_word*)t0)[11];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[8])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11141 in k11138 in k11135 in k11132 in k11129 in k11126 in k11123 in k11120 in a11117 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_11143(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11143,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=C_mutate(((C_word *)((C_word*)t0)[4])+1,((C_word*)t0)[5]);
t4=C_mutate(((C_word *)((C_word*)t0)[6])+1,((C_word*)t0)[7]);
t5=C_mutate(((C_word *)((C_word*)t0)[8])+1,((C_word*)t0)[9]);
t6=((C_word*)t0)[10];{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k11148 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11150(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11150,c,av);}
if(C_truep(t1)){
/* core.scm:1040: ##sys#syntax-error-hook */
t2=*((C_word*)lf[269]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[270];
av2[3]=lf[271];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_10733(2,av2);}}}

/* k11158 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_11160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11160,c,av);}
/* core.scm:1022: ##sys#register-module */
t2=*((C_word*)lf[272]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* g1855 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_11164(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_11164,3,t0,t1,t2);}
a=C_alloc(7);
if(C_truep(C_i_symbolp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11177,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_pairp(t2))){
t4=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_11188,tmp=(C_word)a,a+=2,tmp);
t5=t3;
f_11177(t5,(
  f_11188(t2)
));}
else{
t4=t3;
f_11177(t4,C_SCHEME_FALSE);}}}

/* k11175 in g1855 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_11177(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_11177,2,t0,t1);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* core.scm:1034: ##sys#syntax-error-hook */
t2=*((C_word*)lf[269]+1);{
C_word av2[6];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[270];
av2[3]=lf[273];
av2[4]=((C_word*)t0)[3];
av2[5]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}

/* loop in g1855 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static C_word C_fcall f_11188(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;
loop:{}
t2=C_i_nullp(t1);
if(C_truep(t2)){
return(t2);}
else{
t3=C_i_car(t1);
if(C_truep(C_i_symbolp(t3))){
t5=C_u_i_cdr(t1);
t1=t5;
goto loop;}
else{
return(C_SCHEME_FALSE);}}}

/* k11211 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_11213(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_11213,c,av);}
a=C_alloc(8);
t2=C_i_check_list_2(t1,lf[10]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11221,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp));
t6=((C_word*)t4)[1];
f_11221(t6,((C_word*)t0)[5],t1);}

/* map-loop1849 in k11211 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_11221(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11221,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11246,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1025: g1855 */
t4=((C_word*)t0)[4];
f_11164(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11244 in map-loop1849 in k11211 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11246(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11246,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11221(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_11294(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_11294,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11297,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11357,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1106: ##sys#current-environment */
t4=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11297(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_11297,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_11300,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1107: chicken.compiler.support#get-line */
t3=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_11300(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,4)))){
C_save_and_reclaim((void *)f_11300,c,av);}
a=C_alloc(28);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=*((C_word*)lf[107]+1);
t4=((C_word*)t0)[3];
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_TRUE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11306,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11314,a[2]=t5,a[3]=t7,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t10=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11331,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[9],a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[12],tmp=(C_word)a,a+=9,tmp);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11345,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1109: ##sys#dynamic-wind */
t12=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t12;
av2[1]=t8;
av2[2]=t9;
av2[3]=t10;
av2[4]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(5,av2);}}

/* k11304 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11306(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11306,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11309,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1114: set-real-names! */
f_7945(t2,((C_word*)t0)[3],((C_word*)t0)[5]);}

/* k11307 in k11304 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_11309,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[97],((C_word*)t0)[3],((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a11313 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11314(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_11314,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11318,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[3])[1])){
/* core.scm:1109: ##sys#current-environment2069 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_11318(2,av2);}}}

/* k11316 in a11313 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_11318,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11322,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* core.scm:1109: ##sys#current-environment2069 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11320 in k11316 in a11313 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11322,c,av);}
a=C_alloc(4);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11325,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1109: ##sys#current-environment2069 */
t4=((C_word*)t0)[5];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k11323 in k11320 in k11316 in a11313 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_11325(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11325,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a11330 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11331(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_11331,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11339,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:1111: canonicalize-body/ln */
t3=((C_word*)((C_word*)t0)[7])[1];
f_7916(t3,t2,((C_word*)t0)[4],((C_word*)t0)[8],*((C_word*)lf[52]+1));}

/* k11337 in a11330 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_11339,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11343,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1112: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11341 in k11337 in a11330 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11343(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_11343,c,av);}
/* core.scm:1110: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_FALSE);}

/* a11344 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11345(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_11345,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11349,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1109: ##sys#current-environment2069 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11347 in a11344 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11349(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11349,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11352,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1109: ##sys#current-environment2069 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11350 in k11347 in a11344 in k11298 in k11295 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11352(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11352,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11355 in k11292 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11357(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11357,c,av);}
/* core.scm:1106: ##sys#extend-se */
t2=*((C_word*)lf[175]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop2038 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_11359(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11359,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11384,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1105: g2044 */
t4=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11382 in map-loop2038 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11384(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11384,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11359(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11398 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_11400(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_11400,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[276];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k11404 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_11406(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_11406,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11409,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1121: chicken.compiler.support#get-line */
t3=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11407 in k11404 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_11409,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11416,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11419,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1124: chicken.base#open-output-string */
t4=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:1122: chicken.compiler.support#quit-compiling */
t3=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[277];
av2[3]=lf[279];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k11414 in k11407 in k11404 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_11416(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11416,c,av);}
/* core.scm:1122: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[277];
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k11417 in k11407 in k11404 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_11419(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11419,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11425,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1124: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k11423 in k11417 in k11407 in k11404 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11425(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11425,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11428,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1124: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11426 in k11423 in k11417 in k11407 in k11404 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11428(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11428,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11431,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1124: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[278];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11429 in k11426 in k11423 in k11417 in k11407 in k11404 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11431(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11431,c,av);}
/* core.scm:1124: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_11446(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_11446,c,av);}
a=C_alloc(12);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11449,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1131: chicken.compiler.support#get-line */
t5=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11449(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_11449,c,av);}
a=C_alloc(18);
t2=C_i_caddr(((C_word*)t0)[2]);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_11455,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=t4,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_memq(((C_word*)((C_word*)t0)[3])[1],lf[83]))){
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11749,a[2]=t5,a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1135: chicken.base#open-output-string */
t7=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=t5;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_11455(2,av2);}}}

/* k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_11455(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,4)))){
C_save_and_reclaim((void *)f_11455,c,av);}
a=C_alloc(17);
t2=C_i_assq(((C_word*)((C_word*)t0)[2])[1],lf[70]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11462,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1138: g2138 */
t4=t3;
f_11462(t4,((C_word*)t0)[8],t2);}
else{
t3=C_i_assq(((C_word*)((C_word*)t0)[2])[1],lf[74]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11514,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1138: g2147 */
t5=t4;
f_11514(t5,((C_word*)t0)[8],t3);}
else{
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11562,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
if(C_truep(C_i_memq(((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[4]))){
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_11562(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11580,a[2]=((C_word*)t0)[2],a[3]=t4,a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1161: ##sys#alias-global-hook */
t6=*((C_word*)lf[124]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}}}

/* g2138 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_11462(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_11462,3,t0,t1,t2);}
a=C_alloc(10);
t3=C_i_cadr(t2);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11469,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1141: chicken.base#gensym */
t5=*((C_word*)lf[109]+1);{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k11467 in g2138 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11469(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_11469,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11504,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1143: chicken.compiler.support#foreign-type-convert-argument */
t3=*((C_word*)lf[282]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[9])[1];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11490 in k11502 in k11467 in g2138 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_11492(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,8)))){
C_save_and_reclaim((void *)f_11492,c,av);}
a=C_alloc(18);
t2=C_a_i_list(&a,3,lf[280],((C_word*)t0)[2],t1);
t3=C_a_i_list(&a,3,lf[173],((C_word*)t0)[3],t2);
/* core.scm:1142: walk */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8362(t4,((C_word*)t0)[5],t3,((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k11502 in k11467 in g2138 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11504(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_11504,c,av);}
a=C_alloc(24);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_list(&a,1,t2);
t4=C_i_caddr(((C_word*)t0)[3]);
t5=C_a_i_list(&a,2,t4,((C_word*)t0)[4]);
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11492,a[2]=t5,a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1146: chicken.compiler.support#foreign-type-check */
t7=*((C_word*)lf[281]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* g2147 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_11514(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_11514,3,t0,t1,t2);}
a=C_alloc(10);
t3=C_i_caddr(t2);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11521,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1151: chicken.base#gensym */
t5=*((C_word*)lf[109]+1);{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k11519 in g2147 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11521(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_11521,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11556,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1153: chicken.compiler.support#foreign-type-convert-argument */
t3=*((C_word*)lf[282]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[9])[1];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11546 in k11554 in k11519 in g2147 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_11548(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,8)))){
C_save_and_reclaim((void *)f_11548,c,av);}
a=C_alloc(21);
t2=C_a_i_list(&a,4,lf[283],((C_word*)t0)[2],((C_word*)t0)[3],t1);
t3=C_a_i_list(&a,3,lf[173],((C_word*)t0)[4],t2);
/* core.scm:1152: walk */
t4=((C_word*)((C_word*)t0)[5])[1];
f_8362(t4,((C_word*)t0)[6],t3,((C_word*)t0)[7],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_FALSE);}

/* k11554 in k11519 in g2147 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11556(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_11556,c,av);}
a=C_alloc(22);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_list(&a,1,t2);
t4=C_a_i_list(&a,1,((C_word*)t0)[3]);
t5=C_i_cadr(((C_word*)t0)[4]);
t6=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_11548,a[2]=t4,a[3]=t5,a[4]=t3,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1157: chicken.compiler.support#foreign-type-check */
t7=*((C_word*)lf[281]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k11560 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11562(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,8)))){
C_save_and_reclaim((void *)f_11562,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11569,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1183: walk */
t3=((C_word*)((C_word*)t0)[4])[1];
f_8362(t3,t2,((C_word*)((C_word*)t0)[5])[1],((C_word*)t0)[6],((C_word*)t0)[7],C_i_memq(((C_word*)((C_word*)t0)[3])[1],((C_word*)t0)[6]),((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_FALSE);}

/* k11567 in k11560 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11569(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_11569,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[284],((C_word*)((C_word*)t0)[3])[1],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11580(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_11580,c,av);}
a=C_alloc(11);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_11583,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
if(C_truep(lf[35])){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11736,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1163: chicken.compiler.support#mark-variable */
t5=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=lf[295];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_11583(2,av2);}}}

/* k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11583(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(41,c,2)))){
C_save_and_reclaim((void *)f_11583,c,av);}
a=C_alloc(41);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11586,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(*((C_word*)lf[24]+1))){
t3=C_a_i_list(&a,2,((C_word*)((C_word*)t0)[5])[1],((C_word*)((C_word*)t0)[6])[1]);
t4=C_a_i_list(&a,1,t3);
t5=C_a_i_list(&a,2,lf[146],((C_word*)((C_word*)t0)[5])[1]);
t6=C_a_i_list(&a,3,lf[188],lf[294],t5);
t7=C_a_i_list(&a,4,lf[173],t4,t6,((C_word*)((C_word*)t0)[5])[1]);
t8=C_mutate(((C_word *)((C_word*)t0)[6])+1,t7);
t9=t2;
f_11586(t9,t8);}
else{
t3=t2;
f_11586(t3,C_SCHEME_UNDEFINED);}}

/* k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_11586(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_11586,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11592,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11713,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1172: ##sys#current-environment */
t4=*((C_word*)lf[107]+1);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_11592(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_11592,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11595,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11612,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1174: chicken.base#open-output-string */
t4=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11709,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1179: ##sys#current-environment */
t3=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11593 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_11595(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_11595,c,av);}
a=C_alloc(4);
if(C_truep(*((C_word*)lf[42]+1))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11601,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1177: ##sys#undefine-macro! */
t3=*((C_word*)lf[286]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_11562(2,av2);}}}

/* k11599 in k11593 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11601(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_11601,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11608,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1178: ##sys#current-module */
t3=*((C_word*)lf[204]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11606 in k11599 in k11593 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11608(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11608,c,av);}
/* core.scm:1178: ##sys#unregister-syntax-export */
t2=*((C_word*)lf[285]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_11612(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_11612,c,av);}
a=C_alloc(13);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11618,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11634,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11637,a[2]=t4,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1175: chicken.base#open-output-string */
t6=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* core.scm:1174: ##sys#print */
t5=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[289];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k11616 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11618,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11621,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1174: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[287];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11619 in k11616 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11621,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11624,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1174: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11622 in k11619 in k11616 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_11624(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11624,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11627,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1174: ##sys#write-char-0 */
t3=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(39);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11625 in k11622 in k11619 in k11616 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11627(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11627,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11630,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1174: chicken.base#get-output-string */
t3=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11628 in k11625 in k11622 in k11619 in k11616 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_11630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11630,c,av);}
/* core.scm:1173: chicken.base#warning */
t2=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11632 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11634(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11634,c,av);}
/* core.scm:1174: ##sys#print */
t2=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k11635 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11637(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11637,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11643,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1175: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k11641 in k11635 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11643,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11646,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1175: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11644 in k11641 in k11635 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_11646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11646,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11649,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1175: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[288];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11647 in k11644 in k11641 in k11635 in k11610 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11649,c,av);}
/* core.scm:1175: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_11665(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_11665,c,av);}
a=C_alloc(13);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11671,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11687,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11690,a[2]=t4,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1182: chicken.base#open-output-string */
t6=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* core.scm:1181: ##sys#print */
t5=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[292];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k11669 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11671(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11671,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11674,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1181: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[290];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11672 in k11669 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_11674(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11674,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11677,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1181: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11675 in k11672 in k11669 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11677(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_11677,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11680,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1181: ##sys#write-char-0 */
t3=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(39);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k11678 in k11675 in k11672 in k11669 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_11680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11680,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11683,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1181: chicken.base#get-output-string */
t3=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11681 in k11678 in k11675 in k11672 in k11669 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_11683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11683,c,av);}
/* core.scm:1180: chicken.base#warning */
t2=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11685 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11687(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11687,c,av);}
/* core.scm:1181: ##sys#print */
t2=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k11688 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_11690(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11690,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11696,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1182: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k11694 in k11688 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_11696(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11696,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11699,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1182: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11697 in k11694 in k11688 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_11699(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11699,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11702,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1182: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[291];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11700 in k11697 in k11694 in k11688 in k11663 in k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_11702(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11702,c,av);}
/* core.scm:1182: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11707 in k11590 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_11709(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_11709,c,av);}
a=C_alloc(5);
if(C_truep(C_i_assq(((C_word*)t0)[2],t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11665,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1181: chicken.base#open-output-string */
t3=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_11562(2,av2);}}}

/* k11711 in k11584 in k11581 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_11713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11713,c,av);}
/* core.scm:1172: ##sys#macro? */
t2=*((C_word*)lf[293]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k11734 in k11578 in k11453 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11736(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_11736,c,av);}
/* core.scm:1164: chicken.compiler.support#mark-variable */
t2=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)((C_word*)t0)[3])[1];
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_11749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_11749,c,av);}
a=C_alloc(13);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11755,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11771,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11774,a[2]=t4,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1136: chicken.base#open-output-string */
t6=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* core.scm:1135: ##sys#print */
t5=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[299];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}

/* k11753 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11755(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_11755,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11758,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1135: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[297];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11756 in k11753 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11758,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11761,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1135: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11759 in k11756 in k11753 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11761(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11761,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11764,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1135: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[296];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11762 in k11759 in k11756 in k11753 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_11764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11764,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11767,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1135: chicken.base#get-output-string */
t3=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k11765 in k11762 in k11759 in k11756 in k11753 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_11767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11767,c,av);}
/* core.scm:1134: chicken.base#warning */
t2=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k11769 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_11771,c,av);}
/* core.scm:1135: ##sys#print */
t2=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k11772 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_11774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_11774,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11780,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1136: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k11778 in k11772 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_11780(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11780,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11783,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1136: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11781 in k11778 in k11772 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_11783(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_11783,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11786,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1136: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[298];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k11784 in k11781 in k11778 in k11772 in k11747 in k11447 in k11444 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_11786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11786,c,av);}
/* core.scm:1136: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* g2234 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_11815(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,8)))){
C_save_and_reclaim_args((void *)trf_11815,3,t0,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11823,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1190: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_8362(t4,t3,t2,((C_word*)t0)[4],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k11821 in g2234 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11823(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_11823,c,av);}
/* core.scm:1190: unquotify */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8011(t2,((C_word*)t0)[3],t1);}

/* k11829 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_11831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_11831,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t2);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,lf[188],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* map-loop2228 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_11833(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11833,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11858,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1189: g2234 */
t4=((C_word*)t0)[4];
f_11815(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11856 in map-loop2228 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11858(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11858,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11833(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11880 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_11882(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,6)))){
C_save_and_reclaim((void *)f_11882,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11886,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1195: mapwalk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_13609(t3,t2,C_i_cddr(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE);}

/* k11884 in k11880 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11886(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_11886,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[300],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k11918 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_11920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,6)))){
C_save_and_reclaim((void *)f_11920,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11924,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1200: mapwalk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_13609(t3,t2,C_i_cddr(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE);}

/* k11922 in k11918 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_11924,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[301],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop2261 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_11930(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_11930,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11955,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1199: g2267 */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8011(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k11953 in map-loop2261 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_11955(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_11955,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_11930(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k11977 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_11979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_11979,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[280],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k11998 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,8)))){
C_save_and_reclaim((void *)f_12000,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12004,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1209: walk */
t3=((C_word*)((C_word*)t0)[4])[1];
f_8362(t3,t2,C_i_cadddr(((C_word*)t0)[5]),((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k12002 in k11998 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12004(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_12004,c,av);}
a=C_alloc(12);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,4,lf[283],((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12025 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12027(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_12027,c,av);}
/* core.scm:1214: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_FALSE,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k12040 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12042(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12042,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[306];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12063 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12065(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12065,c,av);}
/* core.scm:1222: chicken.compiler.support#canonicalize-begin-body */
t2=*((C_word*)lf[229]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* fold in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_12069(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,8)))){
C_save_and_reclaim_args((void *)trf_12069,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_car(t2);
t4=C_u_i_cdr(t2);
if(C_truep(C_i_nullp(t4))){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12087,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1227: walk */
t6=((C_word*)((C_word*)t0)[2])[1];
f_8362(t6,t5,t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12094,a[2]=t1,a[3]=((C_word*)t0)[9],a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1228: walk */
t6=((C_word*)((C_word*)t0)[2])[1];
f_8362(t6,t5,t3,((C_word*)t0)[3],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}}

/* k12085 in fold in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_12087,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list1(&a,1,t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12092 in fold in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12094(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_12094,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12098,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1228: fold */
t3=((C_word*)((C_word*)t0)[3])[1];
f_12069(t3,t2,((C_word*)t0)[4]);}

/* k12096 in k12092 in fold in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12098(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_12098,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12111 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12113(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_12113,c,av);}
/* core.scm:1232: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k12124 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_12126,c,av);}
/* core.scm:1235: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k12137 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_12139,c,av);}
/* core.scm:1238: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k12150 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12152(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_12152,c,av);}
/* core.scm:1241: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k12163 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12165(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_12165,c,av);}
/* core.scm:1244: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k12172 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12174(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12174,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12177,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1248: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_caddr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12175 in k12172 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12177(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12177,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12180,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_i_cdddr(((C_word*)t0)[4]);
if(C_truep(C_i_pairp(t3))){
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_cadddr(((C_word*)t0)[4]);
f_12180(2,av2);}}
else{
/* core.scm:1251: scheme#symbol->string */
t4=*((C_word*)lf[255]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k12178 in k12175 in k12172 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12180(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_12180,c,av);}
a=C_alloc(12);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],t1);
t3=C_a_i_cons(&a,2,t2,lf[70]);
t4=C_mutate(&lf[70] /* (set! chicken.compiler.core#foreign-variables ...) */,t3);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=lf[320];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12221(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,4)))){
C_save_and_reclaim((void *)f_12221,c,av);}
a=C_alloc(21);
t2=C_i_cdddr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12227,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,tmp=(C_word)a,a+=13,tmp);
if(C_truep(((C_word*)t0)[10])){
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_12227(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12309,a[2]=t3,a[3]=((C_word*)t0)[11],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[9])){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12312,a[2]=t4,a[3]=((C_word*)t0)[9],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1264: chicken.base#open-output-string */
t6=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* core.scm:1262: chicken.compiler.support#quit-compiling */
t5=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[326];
av2[3]=lf[328];
av2[4]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}}

/* k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12227(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_12227,c,av);}
a=C_alloc(13);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12236,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1267: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12302,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1287: chicken.compiler.support#register-foreign-type! */
t3=*((C_word*)lf[324]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[11];
av2[3]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_12236,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_12239,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* core.scm:1268: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12239(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,5)))){
C_save_and_reclaim((void *)f_12239,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12242,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,tmp=(C_word)a,a+=13,tmp);
/* core.scm:1269: chicken.compiler.support#register-foreign-type! */
t3=*((C_word*)lf[324]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
av2[3]=((C_word*)t0)[13];
av2[4]=((C_word*)t0)[3];
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_12242,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12245,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1270: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12245(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_12245,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12248,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1271: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k12246 in k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12248(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_12248,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12251,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1272: chicken.compiler.support#hide-variable */
t3=*((C_word*)lf[158]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12249 in k12246 in k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_12251(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_12251,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12254,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1273: chicken.compiler.support#hide-variable */
t3=*((C_word*)lf[158]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12252 in k12249 in k12246 in k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_12254(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_12254,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12257,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12299,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1277: ##sys#current-module */
t4=*((C_word*)lf[204]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k12255 in k12252 in k12249 in k12246 in k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_12257(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,2)))){
C_save_and_reclaim((void *)f_12257,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12260,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12295,a[2]=t2,a[3]=((C_word*)t0)[12],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1278: ##sys#current-module */
t4=*((C_word*)lf[204]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k12258 in k12255 in k12252 in k12249 in k12246 in k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_12260(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_12260,c,av);}
a=C_alloc(29);
t2=C_i_car(((C_word*)t0)[2]);
t3=C_a_i_list(&a,3,lf[177],((C_word*)t0)[3],t2);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12275,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[2])))){
t5=C_i_cadr(((C_word*)t0)[2]);
t6=t4;
f_12275(t6,C_a_i_list(&a,3,lf[177],((C_word*)t0)[12],t5));}
else{
t5=t4;
f_12275(t5,C_a_i_list(&a,3,lf[177],((C_word*)t0)[12],lf[322]));}}

/* k12273 in k12258 in k12255 in k12252 in k12249 in k12246 in k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_fcall f_12275(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,8)))){
C_save_and_reclaim_args((void *)trf_12275,2,t0,t1);}
a=C_alloc(9);
t2=C_a_i_list(&a,3,lf[190],((C_word*)t0)[2],t1);
/* core.scm:1279: walk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8362(t3,((C_word*)t0)[4],t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10]);}

/* k12293 in k12255 in k12252 in k12249 in k12246 in k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_12295(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12295,c,av);}
/* core.scm:1278: ##sys#register-export */
t2=*((C_word*)lf[323]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k12297 in k12252 in k12249 in k12246 in k12243 in k12240 in k12237 in k12234 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_12299(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_12299,c,av);}
/* core.scm:1277: ##sys#register-export */
t2=*((C_word*)lf[323]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k12300 in k12225 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12302(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12302,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[325];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12307 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12309,c,av);}
/* core.scm:1262: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[326];
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k12310 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12312,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12318,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1264: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k12316 in k12310 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12318,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12321,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1264: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12319 in k12316 in k12310 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12321,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12324,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1264: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[327];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12322 in k12319 in k12316 in k12310 in k12219 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12324(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12324,c,av);}
/* core.scm:1264: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12343(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_12343,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12346,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* core.scm:1293: scheme#symbol->string */
t3=*((C_word*)lf[255]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_12346,c,av);}
a=C_alloc(12);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_i_caddr(((C_word*)t0)[2]);
t5=C_i_cadddr(((C_word*)t0)[2]);
t6=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12355,a[2]=t3,a[3]=t4,a[4]=t5,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1296: chicken.compiler.support#make-random-name */
t7=*((C_word*)lf[337]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12355(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,4)))){
C_save_and_reclaim((void *)f_12355,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12358,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)t0)[8])){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_12358(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12400,a[2]=t2,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[9])){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12403,a[2]=t3,a[3]=((C_word*)t0)[9],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1300: chicken.base#open-output-string */
t5=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* core.scm:1298: chicken.compiler.support#quit-compiling */
t4=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[334];
av2[3]=lf[336];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}}

/* k12356 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12358(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_12358,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_12361,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=t2;
f_12361(t3,C_SCHEME_UNDEFINED);}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12389,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=C_i_cddddr(((C_word*)t0)[8]);
/* core.scm:1302: scheme#symbol->string */
t5=*((C_word*)lf[255]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_i_car(t4);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k12359 in k12356 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_12361(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_12361,2,t0,t1);}
a=C_alloc(12);
t2=C_a_i_vector3(&a,3,((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=C_a_i_cons(&a,2,t2,*((C_word*)lf[72]+1));
t4=C_mutate((C_word*)lf[72]+1 /* (set! chicken.compiler.core#external-variables ...) */,t3);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12381,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1305: scheme#string-append */
t6=*((C_word*)lf[332]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[333];
av2[3]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k12379 in k12359 in k12356 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,1)))){
C_save_and_reclaim((void *)f_12381,c,av);}
a=C_alloc(18);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],lf[330],t1);
t3=C_a_i_cons(&a,2,t2,lf[70]);
t4=C_mutate(&lf[70] /* (set! chicken.compiler.core#foreign-variables ...) */,t3);
t5=lf[73];
t6=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[2]);
t7=C_a_i_cons(&a,2,t6,lf[73]);
t8=C_mutate(&lf[73] /* (set! chicken.compiler.core#external-to-pointer ...) */,t7);
t9=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t9;
av2[1]=lf[331];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}

/* k12387 in k12356 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12389(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12389,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)t0)[3];
f_12361(t3,t2);}

/* k12398 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12400(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12400,c,av);}
/* core.scm:1298: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[334];
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k12401 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12403(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12403,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12409,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1300: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k12407 in k12401 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12409,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12412,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1300: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12410 in k12407 in k12401 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12412(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12412,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12415,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1300: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[335];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12413 in k12410 in k12407 in k12401 in k12353 in k12344 in k12341 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12415(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12415,c,av);}
/* core.scm:1300: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12430(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_12430,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_12433,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* core.scm:1313: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12433(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_12433,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12436,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1314: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_12436,c,av);}
a=C_alloc(15);
t2=C_i_cddddr(((C_word*)t0)[2]);
t3=C_i_pairp(t2);
t4=(C_truep(t3)?C_i_cadddr(((C_word*)t0)[2]):C_SCHEME_FALSE);
t5=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_12442,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t4,a[13]=((C_word*)t0)[2],a[14]=((C_word*)t0)[12],tmp=(C_word)a,a+=15,tmp);
/* core.scm:1316: chicken.compiler.support#set-real-name! */
t6=*((C_word*)lf[117]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12442(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,2)))){
C_save_and_reclaim((void *)f_12442,c,av);}
a=C_alloc(28);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
t3=C_a_i_cons(&a,2,t2,lf[74]);
t4=C_mutate(&lf[74] /* (set! chicken.compiler.core#location-pointer-map ...) */,t3);
t5=*((C_word*)lf[107]+1);
t6=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_12557,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=t5,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[4],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* core.scm:1320: ##sys#current-environment */
t7=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}

/* a12453 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_12454,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12458,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[3])[1])){
/* core.scm:1319: ##sys#current-environment2372 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_12458(2,av2);}}}

/* k12456 in a12453 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12458(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_12458,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12462,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* core.scm:1319: ##sys#current-environment2372 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k12460 in k12456 in a12453 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_12462(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12462,c,av);}
a=C_alloc(4);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12465,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1319: ##sys#current-environment2372 */
t4=((C_word*)t0)[5];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k12463 in k12460 in k12456 in a12453 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_12465(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12465,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a12470 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_12471,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12520,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12543,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1322: chicken.compiler.support#estimate-foreign-result-location-size */
t4=*((C_word*)lf[341]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12489 in k12518 in a12470 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_12491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,8)))){
C_save_and_reclaim((void *)f_12491,c,av);}
a=C_alloc(12);
t2=C_a_i_cons(&a,2,lf[190],t1);
t3=C_a_i_list(&a,3,lf[173],((C_word*)t0)[2],t2);
/* core.scm:1321: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_8362(t4,((C_word*)t0)[4],t3,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_FALSE);}

/* k12493 in k12518 in a12470 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_fcall f_12495(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_12495,2,t0,t1);}
a=C_alloc(3);
t2=(C_truep(((C_word*)t0)[2])?C_i_car(C_i_cddddr(((C_word*)t0)[3])):C_i_cadddr(((C_word*)t0)[3]));
t3=C_a_i_list(&a,1,t2);
/* core.scm:1322: ##sys#append */
t4=*((C_word*)lf[155]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=t1;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k12518 in a12470 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12520(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(86,c,2)))){
C_save_and_reclaim((void *)f_12520,c,av);}
a=C_alloc(86);
t2=C_s_a_i_plus(&a,2,C_fix(2),t1);
t3=C_a_i_list(&a,2,lf[339],t2);
t4=C_a_i_list(&a,2,lf[118],t1);
t5=C_a_i_list(&a,3,lf[301],t3,t4);
t6=C_a_i_list2(&a,2,((C_word*)t0)[2],t5);
t7=C_a_i_list(&a,1,t6);
t8=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12491,a[2]=t7,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12495,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[11],a[4]=t8,tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[10])){
t10=C_a_i_list(&a,3,lf[177],((C_word*)t0)[12],((C_word*)t0)[10]);
t11=t9;
f_12495(t11,C_a_i_list(&a,1,t10));}
else{
t10=t9;
f_12495(t10,C_SCHEME_END_OF_LIST);}}

/* k12541 in a12470 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12543,c,av);}
/* core.scm:1322: chicken.compiler.support#bytes->words */
t2=*((C_word*)lf[340]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a12544 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12545(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_12545,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12549,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1319: ##sys#current-environment2372 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k12547 in a12544 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12549(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_12549,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12552,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1319: ##sys#current-environment2372 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12550 in k12547 in a12544 in k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_12552(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12552,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k12555 in k12440 in k12434 in k12431 in k12428 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12557(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,4)))){
C_save_and_reclaim((void *)f_12557,c,av);}
a=C_alloc(32);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,t2,t1);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_TRUE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12454,a[2]=t5,a[3]=t7,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12471,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[13],a[11]=((C_word*)t0)[3],a[12]=((C_word*)t0)[14],tmp=(C_word)a,a+=13,tmp);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12545,a[2]=t5,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1319: ##sys#dynamic-wind */
t11=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t11;
av2[1]=((C_word*)t0)[15];
av2[2]=t8;
av2[3]=t9;
av2[4]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(5,av2);}}

/* k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_12590,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12593,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[5])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f25485,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1347: chicken.internal#hash-table-set! */
t4=*((C_word*)lf[110]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[65]+1);
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12603,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t1)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12606,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1345: chicken.base#open-output-string */
t5=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* core.scm:1343: chicken.compiler.support#quit-compiling */
t4=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[344];
av2[3]=lf[346];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}}

/* k12591 in k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12593(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_12593,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12596,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1347: chicken.internal#hash-table-set! */
t3=*((C_word*)lf[110]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[65]+1);
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12594 in k12591 in k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12596,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[343];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12601 in k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12603(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12603,c,av);}
/* core.scm:1343: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[344];
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k12604 in k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12606(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12606,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12612,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1345: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k12610 in k12604 in k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12612(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12612,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12615,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1345: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12613 in k12610 in k12604 in k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12615,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12618,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1345: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[345];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12616 in k12613 in k12610 in k12604 in k12588 in k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12618,c,av);}
/* core.scm:1345: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12625(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_12625,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,lf[97],t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12590,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1341: chicken.compiler.support#get-line */
t4=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12637(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_12637,c,av);}
a=C_alloc(13);
t2=C_i_caddr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12643,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t1,a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12758,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1354: scheme#call-with-current-continuation */
t5=*((C_word*)lf[267]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_12643,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12646,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1354: g2425 */
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,4)))){
C_save_and_reclaim((void *)f_12646,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12649,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[8])){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_12649(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12738,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[7])){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12741,a[2]=t3,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1367: chicken.base#open-output-string */
t5=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
/* core.scm:1365: chicken.compiler.support#quit-compiling */
t4=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[358];
av2[3]=lf[360];
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}}

/* k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,2)))){
C_save_and_reclaim((void *)f_12649,c,av);}
a=C_alloc(25);
t2=C_a_i_list(&a,2,lf[146],((C_word*)t0)[2]);
t3=C_a_i_list2(&a,2,((C_word*)t0)[3],t2);
t4=C_a_i_cons(&a,2,t3,lf[76]);
t5=C_mutate(&lf[76] /* (set! chicken.compiler.core#defconstant-bindings ...) */,t4);
t6=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12659,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1371: chicken.compiler.support#collapsable-literal? */
t7=*((C_word*)lf[357]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_12659,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12662,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_a_i_list(&a,2,lf[146],((C_word*)t0)[3]);
/* core.scm:1372: chicken.internal#hash-table-set! */
t4=*((C_word*)lf[110]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=*((C_word*)lf[66]+1);
av2[3]=((C_word*)t0)[4];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12672,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[4],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1374: chicken.compiler.support#basic-literal? */
t3=*((C_word*)lf[356]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k12660 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12662,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[349];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12672(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_12672,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12675,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1375: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[352];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12705,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[9],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[7])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12708,a[2]=t2,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1384: chicken.base#open-output-string */
t4=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:1382: chicken.compiler.support#quit-compiling */
t3=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[353];
av2[3]=lf[355];
av2[4]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}}

/* k12673 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12675(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_12675,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12678,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1376: chicken.internal#hash-table-set! */
t3=*((C_word*)lf[110]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[66]+1);
av2[3]=((C_word*)t0)[9];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12676 in k12673 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_12678(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_12678,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12681,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1377: chicken.compiler.support#hide-variable */
t3=*((C_word*)lf[158]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k12679 in k12676 in k12673 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_12681(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_12681,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12684,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1378: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[351];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k12682 in k12679 in k12676 in k12673 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_12684(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_12684,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12687,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1379: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k12685 in k12682 in k12679 in k12676 in k12673 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_12687(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,8)))){
C_save_and_reclaim((void *)f_12687,c,av);}
a=C_alloc(15);
t2=C_a_i_list(&a,2,lf[146],((C_word*)t0)[2]);
t3=C_a_i_list(&a,3,lf[350],((C_word*)t0)[3],t2);
/* core.scm:1380: walk */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8362(t4,((C_word*)t0)[5],t3,((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k12703 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12705,c,av);}
/* core.scm:1382: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[353];
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k12706 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12708(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12708,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12714,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1384: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k12712 in k12706 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_12714(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12714,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12717,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1384: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12715 in k12712 in k12706 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_12717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12717,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12720,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1384: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[354];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12718 in k12715 in k12712 in k12706 in k12670 in k12657 in k12647 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_12720(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12720,c,av);}
/* core.scm:1384: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k12736 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12738,c,av);}
/* core.scm:1365: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[358];
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k12739 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12741(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_12741,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12747,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1367: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k12745 in k12739 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12747(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12747,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12750,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1367: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12748 in k12745 in k12739 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_12750,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12753,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1367: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[359];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k12751 in k12748 in k12745 in k12739 in k12644 in k12641 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12753(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12753,c,av);}
/* core.scm:1367: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a12757 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_12758,c,av);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_12764,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12776,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1354: chicken.condition#with-exception-handler */
t5=*((C_word*)lf[266]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t1;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* a12763 in a12757 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_12764,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12770,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1354: k2422 */
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a12769 in a12763 in a12757 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_12770,c,av);}
/* core.scm:1356: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=lf[361];
av2[3]=((C_word*)t0)[2];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a12775 in a12757 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12776(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_12776,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12782,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12808,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1354: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t1;
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}

/* a12781 in a12775 in a12757 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_12782,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12789,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=C_i_symbolp(((C_word*)t0)[2]);
if(C_truep(C_i_not(t3))){
/* core.scm:1359: chicken.compiler.support#collapsable-literal? */
t4=*((C_word*)lf[357]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_12789(2,av2);}}}

/* k12787 in a12781 in a12775 in a12757 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12789(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_12789,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_a_i_list(&a,3,lf[173],lf[76],((C_word*)t0)[3]);
/* core.scm:1361: scheme#eval */
t3=*((C_word*)lf[362]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* a12807 in a12775 in a12757 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12808(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +3,c,2)))){
C_save_and_reclaim((void*)f_12808,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+3);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12814,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1354: k2422 */
t4=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* a12813 in a12807 in a12775 in a12757 in k12635 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12814(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_12814,c,av);}{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=0;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
C_apply_values(3,av2);}}

/* g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_12836(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,0,5)))){
C_save_and_reclaim_args((void *)trf_12836,3,t0,t1,t2);}
a=C_alloc(32);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12842,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13700,tmp=(C_word)a,a+=2,tmp));
t13=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13735,a[2]=t11,a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp));
t14=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13754,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t15=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_13760,tmp=(C_word)a,a+=2,tmp));
t16=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13788,a[2]=t2,a[3]=t5,a[4]=t9,a[5]=t11,a[6]=t3,a[7]=t7,tmp=(C_word)a,a+=8,tmp);
t17=t16;
f_13788(t17,t1);}

/* a12841 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_12842(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_12842,3,t0,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12850,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1391: lookup */
t4=((C_word*)((C_word*)t0)[3])[1];
f_7842(t4,t3,t2);}

/* k12848 in a12841 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_12850,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_memq(t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12855 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12857(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,8)))){
C_save_and_reclaim((void *)f_12857,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[190],t1);
/* core.scm:1388: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8362(t3,((C_word*)t0)[3],t2,((C_word*)t0)[4],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_FALSE);}

/* map-loop2484 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_12859(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_12859,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12884,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1390: g2490 */
t4=((C_word*)t0)[4];
f_12836(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k12882 in map-loop2484 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12884(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_12884,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_12859(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a12901 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12902(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,5)))){
C_save_and_reclaim((void *)f_12902,c,av);}
a=C_alloc(5);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6472,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_6472(t6,t1,C_fix(4),C_SCHEME_END_OF_LIST,t2);}

/* a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_12912(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_12912,c,av);}
a=C_alloc(13);
t4=C_i_car(t3);
t5=C_i_car(t2);
t6=C_i_cadr(t5);
t7=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12922,a[2]=t2,a[3]=t4,a[4]=t1,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],a[11]=t6,a[12]=((C_word*)t0)[8],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1399: ##sys#alias-global-hook */
t8=*((C_word*)lf[124]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t6;
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t8+1)))(5,av2);}}

/* k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_12922(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,2)))){
C_save_and_reclaim((void *)f_12922,c,av);}
a=C_alloc(21);
t2=C_i_caddr(((C_word*)t0)[2]);
t3=C_i_cadr(t2);
t4=C_i_cadddr(((C_word*)t0)[2]);
t5=C_i_cadr(t4);
t6=C_i_cadr(((C_word*)t0)[3]);
t7=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_12934,a[2]=((C_word*)t0)[4],a[3]=t6,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t3,a[9]=((C_word*)t0)[3],a[10]=t1,a[11]=((C_word*)t0)[9],a[12]=t5,a[13]=((C_word*)t0)[10],a[14]=((C_word*)t0)[2],tmp=(C_word)a,a+=15,tmp);
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13154,a[2]=((C_word*)t0)[11],a[3]=t1,a[4]=t7,a[5]=((C_word*)t0)[12],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1403: chicken.compiler.support#valid-c-identifier? */
t9=*((C_word*)lf[469]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}

/* k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_12934(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_12934,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_12937,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],tmp=(C_word)a,a+=15,tmp);
t3=C_i_listp(((C_word*)t0)[3]);
t4=C_i_not(t3);
if(C_truep(t4)){
if(C_truep(t4)){
/* core.scm:1414: chicken.syntax#syntax-error */
t5=*((C_word*)lf[364]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[465];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t5=t2;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_12937(2,av2);}}}
else{
t5=C_i_listp(((C_word*)t0)[12]);
t6=C_i_not(t5);
if(C_truep(t6)){
if(C_truep(t6)){
/* core.scm:1414: chicken.syntax#syntax-error */
t7=*((C_word*)lf[364]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t2;
av2[2]=lf[465];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t7=t2;{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_SCHEME_UNDEFINED;
f_12937(2,av2);}}}
else{
t7=C_i_length(((C_word*)t0)[3]);
t8=C_i_length(((C_word*)t0)[12]);
t9=C_eqp(t7,t8);
if(C_truep(C_i_not(t9))){
/* core.scm:1414: chicken.syntax#syntax-error */
t10=*((C_word*)lf[364]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t10;
av2[1]=t2;
av2[2]=lf[465];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}
else{
t10=t2;{
C_word *av2=av;
av2[0]=t10;
av2[1]=C_SCHEME_UNDEFINED;
f_12937(2,av2);}}}}}

/* k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_12937(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,6)))){
C_save_and_reclaim((void *)f_12937,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12944,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_12948,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1418: mapwalk */
t4=((C_word*)((C_word*)t0)[13])[1];
f_13609(t4,t3,((C_word*)t0)[14],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE);}

/* k12942 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12944(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_12944,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,lf[446],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_12948(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,4)))){
C_save_and_reclaim((void *)f_12948,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12956,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_12968,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13073,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_13073(t7,t3,((C_word*)t0)[3],((C_word*)t0)[12]);}

/* k12954 in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_12956,c,av);}
a=C_alloc(3);
t2=C_a_i_list(&a,1,t1);
/* core.scm:1417: ##sys#append */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k12966 in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_12968(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(43,c,4)))){
C_save_and_reclaim((void *)f_12968,c,av);}
a=C_alloc(43);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12972,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12984,a[2]=t2,a[3]=((C_word*)t0)[8],tmp=(C_word)a,a+=4,tmp);
if(C_truep((C_truep(C_i_equalp(((C_word*)t0)[8],lf[447]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[448]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[449]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[450]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))))){
t4=C_i_cddr(((C_word*)t0)[9]);
t5=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,t4);
t6=C_a_i_cons(&a,2,lf[173],t5);
t7=C_a_i_list(&a,2,lf[146],((C_word*)t0)[10]);
t8=C_a_i_list(&a,3,lf[451],t6,t7);
t9=C_a_i_list(&a,1,t8);
t10=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,t9);
t11=C_a_i_cons(&a,2,lf[173],t10);
/* core.scm:1436: chicken.compiler.support#foreign-type-convert-argument */
t12=*((C_word*)lf[282]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t12;
av2[1]=t2;
av2[2]=t11;
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t12+1)))(4,av2);}}
else{
if(C_truep((C_truep(C_i_equalp(((C_word*)t0)[8],lf[452]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[453]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[454]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[455]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[456]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[457]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))))))){
/* core.scm:1458: chicken.syntax#syntax-error */
t4=*((C_word*)lf[364]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[458];
av2[3]=((C_word*)t0)[8];
av2[4]=((C_word*)t0)[10];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
if(C_truep((C_truep(C_i_equalp(((C_word*)t0)[8],lf[459]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[460]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[461]))?C_SCHEME_TRUE:(C_truep(C_i_equalp(((C_word*)t0)[8],lf[462]))?C_SCHEME_TRUE:C_SCHEME_FALSE)))))){
t4=C_i_cddr(((C_word*)t0)[9]);
t5=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,t4);
t6=C_a_i_cons(&a,2,lf[173],t5);
t7=C_a_i_list(&a,2,lf[463],t6);
t8=C_a_i_list(&a,1,t7);
t9=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13044,a[2]=((C_word*)t0)[10],a[3]=t8,a[4]=t2,a[5]=((C_word*)t0)[8],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1470: macro-alias */
t10=((C_word*)((C_word*)t0)[11])[1];
f_7876(t10,t9,lf[464]);}
else{
t4=C_i_cddr(((C_word*)t0)[9]);
t5=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,t4);
t6=C_a_i_cons(&a,2,lf[173],t5);
/* core.scm:1436: chicken.compiler.support#foreign-type-convert-argument */
t7=*((C_word*)lf[282]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t2;
av2[2]=t6;
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}}}

/* k12970 in k12966 in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12972(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,8)))){
C_save_and_reclaim((void *)f_12972,c,av);}
a=C_alloc(18);
t2=C_a_i_list(&a,3,lf[173],((C_word*)t0)[2],t1);
t3=C_a_i_list(&a,3,lf[97],((C_word*)t0)[3],t2);
/* core.scm:1419: walk */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8362(t4,((C_word*)t0)[5],t3,((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k12982 in k12966 in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_12984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_12984,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,t1);
t3=C_a_i_cons(&a,2,lf[173],t2);
/* core.scm:1436: chicken.compiler.support#foreign-type-convert-argument */
t4=*((C_word*)lf[282]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=t3;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k13042 in k12966 in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13044(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(42,c,3)))){
C_save_and_reclaim((void *)f_13044,c,av);}
a=C_alloc(42);
t2=C_a_i_list(&a,2,lf[146],((C_word*)t0)[2]);
t3=C_a_i_list(&a,3,lf[451],lf[463],t2);
t4=C_a_i_list(&a,3,t1,lf[463],t3);
t5=C_a_i_list(&a,3,lf[173],((C_word*)t0)[3],t4);
t6=C_a_i_list(&a,1,t5);
t7=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,t6);
t8=C_a_i_cons(&a,2,lf[173],t7);
/* core.scm:1436: chicken.compiler.support#foreign-type-convert-argument */
t9=*((C_word*)lf[282]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[4];
av2[2]=t8;
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}

/* loop in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_13073(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,2)))){
C_save_and_reclaim_args((void *)trf_13073,4,t0,t1,t2,t3);}
a=C_alloc(15);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=C_i_car(t3);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13105,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13109,a[2]=t6,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13113,a[2]=t7,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1432: chicken.compiler.support#final-foreign-type */
t9=*((C_word*)lf[122]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}}

/* k13095 in k13103 in loop in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_13097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_13097,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k13103 in loop in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_13105,c,av);}
a=C_alloc(10);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13097,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1435: loop */
t4=((C_word*)((C_word*)t0)[4])[1];
f_13073(t4,t3,C_u_i_cdr(((C_word*)t0)[5]),C_u_i_cdr(((C_word*)t0)[6]));}

/* k13107 in loop in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13109(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13109,c,av);}
/* core.scm:1430: chicken.compiler.support#foreign-type-convert-result */
t2=*((C_word*)lf[120]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13111 in loop in k12946 in k12935 in k12932 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13113(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13113,c,av);}
/* core.scm:1431: chicken.compiler.support#finish-foreign-result */
t2=*((C_word*)lf[121]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13152 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_13154(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_13154,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,t2,lf[77]);
t4=C_mutate(&lf[77] /* (set! chicken.compiler.core#callback-names ...) */,t3);
t5=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
f_12934(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13165,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1406: chicken.compiler.support#get-line */
t3=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k13163 in k13152 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_13165(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_13165,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13172,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13175,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1409: chicken.base#open-output-string */
t4=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:1407: chicken.compiler.support#quit-compiling */
t3=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[466];
av2[3]=lf[468];
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k13170 in k13163 in k13152 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13172(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13172,c,av);}
/* core.scm:1407: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[466];
av2[3]=t1;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k13173 in k13163 in k13152 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13175(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_13175,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13181,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1409: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k13179 in k13173 in k13163 in k13152 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_13181(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_13181,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13184,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1409: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13182 in k13179 in k13173 in k13163 in k13152 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13184(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_13184,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13187,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1409: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[467];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13185 in k13182 in k13179 in k13173 in k13163 in k13152 in k12920 in a12911 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_13187(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13187,c,av);}
/* core.scm:1409: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* g2577 in k13291 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_13224(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,8)))){
C_save_and_reclaim_args((void *)trf_13224,3,t0,t1,t2);}
a=C_alloc(21);
t3=C_i_cadr(t2);
t4=C_a_i_list(&a,2,lf[146],lf[471]);
t5=C_a_i_list(&a,5,lf[472],t3,C_fix(0),C_SCHEME_FALSE,t4);
/* core.scm:1482: walk */
t6=((C_word*)((C_word*)t0)[2])[1];
f_8362(t6,t1,t5,((C_word*)t0)[3],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[4],((C_word*)t0)[5],C_SCHEME_FALSE);}

/* g2584 in k13291 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_13250(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,8)))){
C_save_and_reclaim_args((void *)trf_13250,3,t0,t1,t2);}
/* core.scm:1486: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8362(t3,t1,C_i_cdr(t2),((C_word*)t0)[3],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[4],((C_word*)t0)[5],C_SCHEME_FALSE);}

/* k13276 in k13291 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_13278(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_13278,c,av);}
a=C_alloc(12);
t2=C_a_i_list(&a,2,t1,lf[330]);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,2,lf[119],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k13291 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_13293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,8)))){
C_save_and_reclaim((void *)f_13293,c,av);}
a=C_alloc(21);
t2=C_i_assq(t1,lf[74]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13224,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1480: g2577 */
t4=t3;
f_13224(t4,((C_word*)t0)[6],t2);}
else{
t3=C_i_assq(((C_word*)t0)[7],lf[73]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13250,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1480: g2584 */
t5=t4;
f_13250(t5,((C_word*)t0)[6],t3);}
else{
if(C_truep(C_i_assq(((C_word*)t0)[7],lf[77]))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13278,a[2]=((C_word*)t0)[6],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1488: scheme#symbol->string */
t5=*((C_word*)lf[255]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_a_i_list(&a,2,lf[146],lf[471]);
t5=C_a_i_list(&a,5,lf[472],((C_word*)t0)[7],C_fix(0),C_SCHEME_FALSE,t4);
/* core.scm:1490: walk */
t6=((C_word*)((C_word*)t0)[2])[1];
f_8362(t6,((C_word*)t0)[6],t5,((C_word*)t0)[3],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[4],((C_word*)t0)[5],C_SCHEME_FALSE);}}}}

/* k13308 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_13310(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13310,c,av);}
a=C_alloc(7);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13316,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* core.scm:1502: chicken.internal#hash-table-ref */
t4=*((C_word*)lf[112]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[63]+1);
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k13314 in k13308 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_13316(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_13316,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13319,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
if(C_truep(((C_word*)t0)[4])){
if(C_truep(t1)){
t3=C_i_cdr(t1);
t4=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t5=C_a_i_cons(&a,2,t4,t3);
t6=C_a_i_cons(&a,2,((C_word*)t0)[5],t5);
/* core.scm:1504: chicken.internal#hash-table-set! */
t7=*((C_word*)lf[110]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t7;
av2[1]=t2;
av2[2]=*((C_word*)lf[63]+1);
av2[3]=((C_word*)t0)[6];
av2[4]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]);
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_a_i_cons(&a,2,((C_word*)t0)[5],t4);
/* core.scm:1504: chicken.internal#hash-table-set! */
t6=*((C_word*)lf[110]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t2;
av2[2]=*((C_word*)lf[63]+1);
av2[3]=((C_word*)t0)[6];
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k13317 in k13314 in k13308 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_13319(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13319,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a13338 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_13339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13339,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[473]+1));
t3=C_mutate((C_word*)lf[473]+1 /* (set! ##sys#syntax-context ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a13343 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_13344(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_13344,c,av);}
/* core.scm:1500: mapwalk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13609(t2,t1,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* a13349 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_13350(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13350,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[473]+1));
t3=C_mutate((C_word*)lf[473]+1 /* (set! ##sys#syntax-context ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a13488 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13489(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13489,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[114]+1));
t3=C_mutate((C_word*)lf[114]+1 /* (set! chicken.syntax#expansion-result-hook ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a13493 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13494,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13502,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:654: ##sys#current-environment */
t3=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k13500 in a13493 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13502(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13502,c,av);}
/* core.scm:654: chicken.syntax#expand */
t2=*((C_word*)lf[476]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=*((C_word*)lf[52]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a13503 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13504(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13504,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[114]+1));
t3=C_mutate((C_word*)lf[114]+1 /* (set! chicken.syntax#expansion-result-hook ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k13520 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13522(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_13522,c,av);}
a=C_alloc(6);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13528,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:647: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k13526 in k13520 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13528(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13528,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13531,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:647: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13529 in k13526 in k13520 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13531(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13531,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13534,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:647: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[477];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13532 in k13529 in k13526 in k13520 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13534(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13534,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13537,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:647: chicken.base#get-output-string */
t3=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13535 in k13532 in k13529 in k13526 in k13520 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13537(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13537,c,av);}
/* core.scm:647: ##sys#syntax-error/context */
t2=*((C_word*)lf[138]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13553 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13555(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_13555,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13558,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1514: chicken.compiler.support#emit-syntax-trace-info */
t3=*((C_word*)lf[479]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13567,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1519: chicken.compiler.support#emit-syntax-trace-info */
t3=*((C_word*)lf[479]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k13556 in k13553 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13558(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_13558,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13561,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1515: chicken.base#warning */
t3=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[481];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k13559 in k13556 in k13553 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13561(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_13561,c,av);}
/* core.scm:1516: mapwalk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_13609(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k13565 in k13553 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13567(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_13567,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_13570,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1520: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k13568 in k13565 in k13553 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13570(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,8)))){
C_save_and_reclaim((void *)f_13570,c,av);}
a=C_alloc(21);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=C_a_i_list(&a,2,t1,t2);
t4=C_a_i_list(&a,1,t3);
t5=C_u_i_cdr(((C_word*)t0)[2]);
t6=C_a_i_cons(&a,2,t1,t5);
t7=C_a_i_list(&a,3,lf[173],t4,t6);
/* core.scm:1521: walk */
t8=((C_word*)((C_word*)t0)[3])[1];
f_8362(t8,((C_word*)t0)[4],t7,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_FALSE);}

/* mapwalk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_13609(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,3)))){
C_save_and_reclaim_args((void *)trf_13609,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(20);
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_13614,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t4,a[5]=t5,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
t12=C_i_check_list_2(t2,lf[10]);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13626,a[2]=t9,a[3]=t14,a[4]=t11,a[5]=t10,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_13626(t16,t1,t2);}

/* g2633 in mapwalk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_13614(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,8)))){
C_save_and_reclaim_args((void *)trf_13614,3,t0,t1,t2);}
/* core.scm:1528: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8362(t3,t1,t2,((C_word*)t0)[3],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);}

/* map-loop2627 in mapwalk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_13626(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_13626,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13651,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1528: g2633 */
t4=((C_word*)t0)[4];
f_13614(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k13649 in map-loop2627 in mapwalk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13651(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_13651,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_13626(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k13659 in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_13661,c,av);}
a=C_alloc(8);
t2=code_817();
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13672,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13676,a[2]=((C_word*)t0)[4],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1535: scheme#reverse */
t5=*((C_word*)lf[259]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[75];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k13670 in k13659 in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13672(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,8)))){
C_save_and_reclaim((void *)f_13672,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,lf[190],t1);
/* core.scm:1533: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8362(t3,((C_word*)t0)[3],t2,C_SCHEME_END_OF_LIST,C_SCHEME_FALSE,C_SCHEME_FALSE,C_SCHEME_FALSE,C_SCHEME_FALSE,C_SCHEME_TRUE);}

/* k13674 in k13659 in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13676(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_13676,c,av);}
a=C_alloc(5);
t2=lf[75] /* chicken.compiler.core#pending-canonicalizations */ =C_SCHEME_END_OF_LIST;;
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13686,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1539: scheme#append */
t4=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[87]+1);
av2[3]=*((C_word*)lf[19]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k13684 in k13674 in k13659 in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13686(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_13686,c,av);}
a=C_alloc(3);
t2=C_mutate((C_word*)lf[19]+1 /* (set! chicken.compiler.core#extended-bindings ...) */,t1);
t3=C_a_i_list(&a,1,((C_word*)t0)[2]);
/* core.scm:1534: ##sys#append */
t4=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k13690 in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_13692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13692,c,av);}
/* core.scm:1530: chicken.pretty-print#pretty-print */
t2=*((C_word*)lf[240]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* check-decl in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_13700(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_13700,4,t1,t2,t3,t4);}
t5=C_i_cdr(t2);
t6=C_i_length(t5);
t7=C_i_lessp(t6,t3);
if(C_truep(t7)){
if(C_truep(t7)){
/* core.scm:1548: chicken.syntax#syntax-error */
t8=*((C_word*)lf[364]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t1;
av2[2]=lf[365];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
t8=C_SCHEME_UNDEFINED;
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
if(C_truep(C_i_nullp(t4))){
if(C_truep(C_i_greaterp(t6,C_fix(99999)))){
/* core.scm:1548: chicken.syntax#syntax-error */
t8=*((C_word*)lf[364]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t1;
av2[2]=lf[365];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
t8=C_SCHEME_UNDEFINED;
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t8=C_i_car(t4);
if(C_truep(C_i_greaterp(t6,t8))){
/* core.scm:1548: chicken.syntax#syntax-error */
t9=*((C_word*)lf[364]+1);{
C_word av2[4];
av2[0]=t9;
av2[1]=t1;
av2[2]=lf[365];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
t9=C_SCHEME_UNDEFINED;
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}}}

/* globalize in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_13735(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_13735,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_13742,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1550: local? */
t4=((C_word*)t0)[4];
f_12842(t4,t3,t2);}

/* k13740 in globalize in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_13742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_13742,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13745,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1551: note-local */
f_13760(t2,((C_word*)t0)[4]);}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13752,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1553: lookup */
t3=((C_word*)((C_word*)t0)[5])[1];
f_7842(t3,t2,((C_word*)t0)[4]);}}

/* k13743 in k13740 in globalize in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_13745(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13745,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k13750 in k13740 in globalize in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_13752(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_13752,c,av);}
/* core.scm:1553: ##sys#alias-global-hook */
t2=*((C_word*)lf[124]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* globalize-all in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_13754(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_13754,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_i_check_list_2(t2,lf[8]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6968,a[2]=((C_word*)t0)[2],a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_6968(t7,t1,t2);}

/* note-local in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_13760(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_13760,2,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13768,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1558: chicken.base#open-output-string */
t4=*((C_word*)lf[133]+1);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k13766 in note-local in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_13768(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_13768,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13774,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1558: ##sys#print */
t4=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[367];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k13772 in k13766 in note-local in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_13774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_13774,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13777,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1558: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k13775 in k13772 in k13766 in note-local in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13777(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_13777,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13780,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1558: ##sys#write-char-0 */
t3=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_make_character(39);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k13778 in k13775 in k13772 in k13766 in note-local in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_13780(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_13780,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13783,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1558: chicken.base#get-output-string */
t3=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13781 in k13778 in k13775 in k13772 in k13766 in note-local in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13783(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_13783,c,av);}
/* core.scm:1557: ##sys#notice */
t2=*((C_word*)lf[366]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_13788(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_13788,2,t0,t1);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13792,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_13792(2,av2);}}
else{
/* core.scm:1562: chicken.syntax#syntax-error */
t3=*((C_word*)lf[364]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[445];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_13792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_13792,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_13795,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1563: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_13795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_13795,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13798,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_eqp(t1,lf[369]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13804,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1565: chicken.syntax#strip-syntax */
t5=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=C_u_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_eqp(t1,lf[370]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13823,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1569: check-decl */
f_13700(t5,((C_word*)t0)[3],C_fix(1),C_a_i_list(&a,1,C_fix(1)));}
else{
t5=C_eqp(t1,lf[372]);
if(C_truep(t5)){
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[3])))){
t6=C_mutate((C_word*)lf[18]+1 /* (set! chicken.compiler.core#standard-bindings ...) */,*((C_word*)lf[86]+1));
t7=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t7;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13865,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13869,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1578: globalize-all */
t8=((C_word*)((C_word*)t0)[5])[1];
f_13754(t8,t7,C_u_i_cdr(((C_word*)t0)[3]));}}
else{
t6=C_eqp(t1,lf[373]);
if(C_truep(t6)){
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[3])))){
t7=C_mutate((C_word*)lf[19]+1 /* (set! chicken.compiler.core#extended-bindings ...) */,*((C_word*)lf[85]+1));
t8=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t8;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13890,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13894,a[2]=t7,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1582: globalize-all */
t9=((C_word*)((C_word*)t0)[5])[1];
f_13754(t9,t8,C_u_i_cdr(((C_word*)t0)[3]));}}
else{
t7=C_eqp(t1,lf[374]);
if(C_truep(t7)){
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[3])))){
t8=C_mutate((C_word*)lf[18]+1 /* (set! chicken.compiler.core#standard-bindings ...) */,*((C_word*)lf[86]+1));
t9=C_mutate((C_word*)lf[19]+1 /* (set! chicken.compiler.core#extended-bindings ...) */,*((C_word*)lf[85]+1));
t10=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t10;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13915,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1588: globalize-all */
t9=((C_word*)((C_word*)t0)[5])[1];
f_13754(t9,t8,C_u_i_cdr(((C_word*)t0)[3]));}}
else{
t8=C_eqp(t1,lf[375]);
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13936,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1592: check-decl */
f_13700(t9,((C_word*)t0)[3],C_fix(1),C_a_i_list(&a,1,C_fix(1)));}
else{
t9=C_eqp(t1,lf[377]);
t10=(C_truep(t9)?t9:C_eqp(t1,lf[378]));
if(C_truep(t10)){
t11=C_mutate((C_word*)lf[376]+1 /* (set! chicken.compiler.support#number-type ...) */,lf[377]);
t12=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t12;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t11=C_eqp(t1,lf[379]);
if(C_truep(t11)){
t12=C_mutate((C_word*)lf[376]+1 /* (set! chicken.compiler.support#number-type ...) */,lf[379]);
t13=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t13;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t13+1)))(2,av2);}}
else{
t12=C_eqp(t1,lf[380]);
if(C_truep(t12)){
t13=C_set_block_item(lf[149] /* chicken.compiler.support#unsafe */,0,C_SCHEME_TRUE);
t14=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t14;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t14+1)))(2,av2);}}
else{
t13=C_eqp(t1,lf[381]);
if(C_truep(t13)){
t14=C_set_block_item(lf[149] /* chicken.compiler.support#unsafe */,0,C_SCHEME_FALSE);
t15=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t15;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t15+1)))(2,av2);}}
else{
t14=C_eqp(t1,lf[382]);
if(C_truep(t14)){
t15=C_set_block_item(lf[31] /* chicken.compiler.core#no-bound-checks */,0,C_SCHEME_TRUE);
t16=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t16;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t16+1)))(2,av2);}}
else{
t15=C_eqp(t1,lf[383]);
if(C_truep(t15)){
t16=C_set_block_item(lf[32] /* chicken.compiler.core#no-argc-checks */,0,C_SCHEME_TRUE);
t17=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t17;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t17+1)))(2,av2);}}
else{
t16=C_eqp(t1,lf[384]);
if(C_truep(t16)){
t17=C_set_block_item(lf[33] /* chicken.compiler.core#no-procedure-checks */,0,C_SCHEME_TRUE);
t18=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t18;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t18+1)))(2,av2);}}
else{
t17=C_eqp(t1,lf[385]);
if(C_truep(t17)){
t18=C_set_block_item(lf[20] /* chicken.compiler.core#insert-timer-checks */,0,C_SCHEME_FALSE);
t19=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t19;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t19+1)))(2,av2);}}
else{
t18=C_eqp(t1,lf[386]);
if(C_truep(t18)){
t19=C_u_i_cdr(((C_word*)t0)[3]);
t20=C_i_check_list_2(t19,lf[100]);
t21=C_SCHEME_UNDEFINED;
t22=(*a=C_VECTOR_TYPE|1,a[1]=t21,tmp=(C_word)a,a+=2,tmp);
t23=C_set_block_item(t22,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14023,a[2]=t22,tmp=(C_word)a,a+=3,tmp));
t24=((C_word*)t22)[1];
f_14023(t24,t2,t19);}
else{
t19=C_eqp(t1,lf[387]);
if(C_truep(t19)){
t20=lf[35] /* chicken.compiler.core#safe-globals-flag */ =C_SCHEME_TRUE;;
t21=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t21;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t21+1)))(2,av2);}}
else{
t20=C_eqp(t1,lf[388]);
if(C_truep(t20)){
t21=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14065,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1608: scheme#append */
t22=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t22;
av2[1]=t21;
av2[2]=*((C_word*)lf[86]+1);
av2[3]=*((C_word*)lf[85]+1);
((C_proc)(void*)(*((C_word*)t22+1)))(4,av2);}}
else{
t21=C_eqp(t1,lf[389]);
if(C_truep(t21)){
t22=C_set_block_item(lf[34] /* chicken.compiler.core#no-global-procedure-checks */,0,C_SCHEME_TRUE);
t23=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t23;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t23+1)))(2,av2);}}
else{
t22=C_eqp(t1,lf[390]);
if(C_truep(t22)){
t23=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14147,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1615: globalize-all */
t24=((C_word*)((C_word*)t0)[5])[1];
f_13754(t24,t23,C_u_i_cdr(((C_word*)t0)[3]));}
else{
t23=C_eqp(t1,lf[391]);
if(C_truep(t23)){
t24=C_u_i_cdr(((C_word*)t0)[3]);
t25=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14227,a[2]=((C_word*)t0)[2],a[3]=t24,a[4]=t2,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1620: every */
f_6675(t25,*((C_word*)lf[394]+1),t24);}
else{
t24=C_eqp(t1,lf[395]);
if(C_truep(t24)){
t25=C_set_block_item(lf[25] /* chicken.compiler.core#block-compilation */,0,C_SCHEME_TRUE);
t26=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t26;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t26+1)))(2,av2);}}
else{
t25=C_eqp(t1,lf[396]);
if(C_truep(t25)){
t26=C_set_block_item(lf[25] /* chicken.compiler.core#block-compilation */,0,C_SCHEME_FALSE);
t27=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t27;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t27+1)))(2,av2);}}
else{
t26=C_eqp(t1,lf[397]);
if(C_truep(t26)){
t27=C_set_block_item(lf[42] /* chicken.compiler.core#undefine-shadowed-macros */,0,C_SCHEME_FALSE);
t28=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t28;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t28+1)))(2,av2);}}
else{
t27=C_eqp(t1,lf[398]);
if(C_truep(t27)){
t28=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14269,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1627: globalize-all */
t29=((C_word*)((C_word*)t0)[5])[1];
f_13754(t29,t28,C_u_i_cdr(((C_word*)t0)[3]));}
else{
t28=C_eqp(t1,lf[400]);
if(C_truep(t28)){
t29=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14314,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1631: globalize-all */
t30=((C_word*)((C_word*)t0)[5])[1];
f_13754(t30,t29,C_u_i_cdr(((C_word*)t0)[3]));}
else{
t29=C_eqp(t1,lf[402]);
if(C_truep(t29)){
t30=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14354,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1633: check-decl */
f_13700(t30,((C_word*)t0)[3],C_fix(1),C_SCHEME_END_OF_LIST);}
else{
t30=C_eqp(t1,lf[409]);
if(C_truep(t30)){
t31=C_set_block_item(lf[200] /* ##sys#enable-runtime-macros */,0,C_SCHEME_TRUE);
t32=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t32;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t32+1)))(2,av2);}}
else{
t31=C_eqp(t1,lf[410]);
t32=(C_truep(t31)?t31:C_eqp(t1,lf[411]));
if(C_truep(t32)){
t33=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14614,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1676: globalize-all */
t34=((C_word*)((C_word*)t0)[5])[1];
f_13754(t34,t33,C_u_i_cdr(((C_word*)t0)[3]));}
else{
t33=C_eqp(t1,lf[412]);
if(C_truep(t33)){
t34=C_set_block_item(lf[25] /* chicken.compiler.core#block-compilation */,0,C_SCHEME_TRUE);
t35=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14662,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1682: globalize-all */
t36=((C_word*)((C_word*)t0)[5])[1];
f_13754(t36,t35,C_u_i_cdr(((C_word*)t0)[3]));}
else{
t34=C_eqp(t1,lf[414]);
if(C_truep(t34)){
t35=C_set_block_item(lf[38] /* chicken.compiler.core#external-protos-first */,0,C_SCHEME_TRUE);
t36=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t36;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t36+1)))(2,av2);}}
else{
t35=C_eqp(t1,lf[403]);
if(C_truep(t35)){
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[3])))){
t36=C_set_block_item(lf[50] /* chicken.compiler.core#inline-locally */,0,C_SCHEME_TRUE);
t37=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t37;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t37+1)))(2,av2);}}
else{
t36=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14721,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1691: globalize-all */
t37=((C_word*)((C_word*)t0)[5])[1];
f_13754(t37,t36,C_u_i_cdr(((C_word*)t0)[3]));}}
else{
t36=C_eqp(t1,lf[416]);
if(C_truep(t36)){
t37=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14763,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1693: check-decl */
f_13700(t37,((C_word*)t0)[3],C_fix(1),C_a_i_list(&a,1,C_fix(1)));}
else{
t37=C_eqp(t1,lf[418]);
if(C_truep(t37)){
t38=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14785,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1701: check-decl */
f_13700(t38,((C_word*)t0)[3],C_fix(1),C_a_i_list(&a,1,C_fix(1)));}
else{
t38=C_eqp(t1,lf[420]);
if(C_truep(t38)){
t39=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14807,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1709: globalize-all */
t40=((C_word*)((C_word*)t0)[5])[1];
f_13754(t40,t39,C_u_i_cdr(((C_word*)t0)[3]));}
else{
t39=C_eqp(t1,lf[423]);
if(C_truep(t39)){
t40=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14865,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t41=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t42=t41;
t43=(*a=C_VECTOR_TYPE|1,a[1]=t42,tmp=(C_word)a,a+=2,tmp);
t44=((C_word*)t43)[1];
t45=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14941,a[2]=t40,a[3]=t43,a[4]=t44,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1728: chicken.syntax#strip-syntax */
t46=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t46;
av2[1]=t45;
av2[2]=C_u_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t46+1)))(3,av2);}}
else{
t40=C_eqp(t1,lf[426]);
if(C_truep(t40)){
if(C_truep(*((C_word*)lf[58]+1))){
t41=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t41;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t41+1)))(2,av2);}}
else{
t41=C_i_nullp(C_u_i_cdr(((C_word*)t0)[3]));
t42=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14999,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(t41)){
t43=C_mutate((C_word*)lf[58]+1 /* (set! chicken.compiler.core#types-output-file ...) */,t41);
t44=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t44;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t44+1)))(2,av2);}}
else{
t43=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15002,a[2]=((C_word*)t0)[2],a[3]=t42,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t44=C_i_cadr(((C_word*)t0)[3]);
if(C_truep(C_i_stringp(t44))){
t45=C_u_i_cdr(((C_word*)t0)[3]);
t46=C_i_nullp(C_u_i_cdr(t45));
t47=t43;
f_15002(t47,(C_truep(t46)?C_u_i_car(C_u_i_cdr(((C_word*)t0)[3])):C_SCHEME_FALSE));}
else{
t45=t43;
f_15002(t45,C_SCHEME_FALSE);}}}}
else{
t41=C_eqp(t1,lf[428]);
if(C_truep(t41)){
t42=C_set_block_item(lf[30] /* chicken.compiler.core#emit-profile */,0,C_SCHEME_TRUE);
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[3])))){
t43=C_mutate((C_word*)lf[43]+1 /* (set! chicken.compiler.core#profiled-procedures ...) */,lf[183]);
t44=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t44;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t44+1)))(2,av2);}}
else{
t43=C_mutate((C_word*)lf[43]+1 /* (set! chicken.compiler.core#profiled-procedures ...) */,lf[184]);
t44=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15051,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1743: globalize-all */
t45=((C_word*)((C_word*)t0)[5])[1];
f_13754(t45,t44,C_u_i_cdr(((C_word*)t0)[3]));}}
else{
t42=C_eqp(t1,lf[429]);
if(C_truep(t42)){
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[3])))){
t43=C_set_block_item(lf[49] /* chicken.compiler.core#local-definitions */,0,C_SCHEME_TRUE);
t44=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t44;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t44+1)))(2,av2);}}
else{
t43=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15105,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1750: globalize-all */
t44=((C_word*)((C_word*)t0)[5])[1];
f_13754(t44,t43,C_u_i_cdr(((C_word*)t0)[3]));}}
else{
t43=C_eqp(t1,lf[406]);
if(C_truep(t43)){
t44=C_set_block_item(lf[51] /* chicken.compiler.core#enable-inline-files */,0,C_SCHEME_TRUE);
t45=C_set_block_item(lf[50] /* chicken.compiler.core#inline-locally */,0,C_SCHEME_TRUE);
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[3])))){
t46=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15160,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1757: globalize-all */
t47=((C_word*)((C_word*)t0)[5])[1];
f_13754(t47,t46,C_u_i_cdr(((C_word*)t0)[3]));}
else{
t46=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t46;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t46+1)))(2,av2);}}}
else{
t44=C_eqp(t1,lf[430]);
if(C_truep(t44)){
t45=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15200,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
t46=C_u_i_cdr(((C_word*)t0)[3]);
t47=C_i_check_list_2(t46,lf[100]);
t48=C_SCHEME_UNDEFINED;
t49=(*a=C_VECTOR_TYPE|1,a[1]=t48,tmp=(C_word)a,a+=2,tmp);
t50=C_set_block_item(t49,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15364,a[2]=t49,a[3]=t45,tmp=(C_word)a,a+=4,tmp));
t51=((C_word*)t49)[1];
f_15364(t51,t2,t46);}
else{
t45=C_eqp(t1,lf[439]);
if(C_truep(t45)){
t46=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15392,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
t47=C_u_i_cdr(((C_word*)t0)[3]);
t48=C_i_check_list_2(t47,lf[100]);
t49=C_SCHEME_UNDEFINED;
t50=(*a=C_VECTOR_TYPE|1,a[1]=t49,tmp=(C_word)a,a+=2,tmp);
t51=C_set_block_item(t50,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15486,a[2]=t50,a[3]=t46,tmp=(C_word)a,a+=4,tmp));
t52=((C_word*)t50)[1];
f_15486(t52,t2,t47);}
else{
t46=C_eqp(t1,lf[442]);
if(C_truep(t46)){
t47=C_set_block_item(lf[55] /* chicken.compiler.core#enable-specialization */,0,C_SCHEME_TRUE);
t48=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t48;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t48+1)))(2,av2);}}
else{
t47=C_eqp(t1,lf[443]);
if(C_truep(t47)){
t48=C_set_block_item(lf[54] /* chicken.compiler.core#strict-variable-types */,0,C_SCHEME_TRUE);
t49=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t49;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t49+1)))(2,av2);}}
else{
/* core.scm:1812: chicken.base#warning */
t48=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t48;
av2[1]=t2;
av2[2]=lf[444];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t48+1)))(4,av2);}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}

/* k13796 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13798(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13798,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k13802 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13804,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13808,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1566: lset-union/eq? */
f_7220(t2,*((C_word*)lf[82]+1),C_a_i_list(&a,1,t1));}

/* k13806 in k13802 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_13808(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_13808,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[82]+1 /* (set! chicken.compiler.core#used-libraries ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13812,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1567: lset-union/eq? */
f_7220(t3,*((C_word*)lf[81]+1),C_a_i_list(&a,1,((C_word*)t0)[3]));}

/* k13810 in k13806 in k13802 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13812(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13812,c,av);}
t2=C_mutate((C_word*)lf[81]+1 /* (set! chicken.compiler.core#linked-libraries ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k13821 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13823(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_13823,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13826,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1570: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13824 in k13821 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_13826(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_13826,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13829,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(C_truep(*((C_word*)lf[17]+1))?C_i_not(C_eqp(*((C_word*)lf[17]+1),t1)):C_SCHEME_FALSE);
if(C_truep(t3)){
/* core.scm:1572: chicken.base#warning */
t4=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[371];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=C_mutate((C_word*)lf[17]+1 /* (set! chicken.compiler.core#unit-name ...) */,t1);
t5=C_set_block_item(lf[48] /* chicken.compiler.core#standalone-executable */,0,C_SCHEME_FALSE);
t6=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t6;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k13827 in k13824 in k13821 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13829(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13829,c,av);}
t2=C_mutate((C_word*)lf[17]+1 /* (set! chicken.compiler.core#unit-name ...) */,((C_word*)t0)[2]);
t3=C_set_block_item(lf[48] /* chicken.compiler.core#standalone-executable */,0,C_SCHEME_FALSE);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k13863 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13865,c,av);}
t2=C_mutate((C_word*)lf[18]+1 /* (set! chicken.compiler.core#standard-bindings ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k13867 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13869,c,av);}
/* core.scm:1578: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=*((C_word*)lf[18]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13888 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13890(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13890,c,av);}
t2=C_mutate((C_word*)lf[19]+1 /* (set! chicken.compiler.core#extended-bindings ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k13892 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13894(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_13894,c,av);}
/* core.scm:1582: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=*((C_word*)lf[19]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k13913 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13915(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_13915,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13919,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1589: lset-intersection/eq? */
f_7299(t2,t1,C_a_i_list(&a,1,*((C_word*)lf[86]+1)));}

/* k13917 in k13913 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_13919(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_13919,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[18]+1 /* (set! chicken.compiler.core#standard-bindings ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13923,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1590: lset-intersection/eq? */
f_7299(t3,((C_word*)t0)[3],C_a_i_list(&a,1,*((C_word*)lf[85]+1)));}

/* k13921 in k13917 in k13913 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_13923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13923,c,av);}
t2=C_mutate((C_word*)lf[19]+1 /* (set! chicken.compiler.core#extended-bindings ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k13934 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_13936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_13936,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13940,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1593: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k13938 in k13934 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_13940(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_13940,c,av);}
t2=C_mutate((C_word*)lf[376]+1 /* (set! chicken.compiler.support#number-type ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* for-each-loop2739 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_14023(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_14023,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14033,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1603: g2755 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14031 in for-each-loop2739 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_14033(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14033,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14023(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14063 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14065(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_14065,c,av);}
a=C_alloc(8);
t2=C_i_check_list_2(t1,lf[100]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14071,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14110,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_14110(t7,t3,t1);}

/* k14069 in k14063 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_14071(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_14071,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14079,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1611: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[86]+1);
av2[3]=*((C_word*)lf[85]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k14077 in k14069 in k14063 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14079(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14079,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14087,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_14087(t6,((C_word*)t0)[2],t1);}

/* for-each-loop2789 in k14077 in k14069 in k14063 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_fcall f_14087(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_14087,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14097,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1610: g2805 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14095 in for-each-loop2789 in k14077 in k14069 in k14063 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_14097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14097,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14087(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop2764 in k14063 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_14110(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_14110,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14120,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1607: g2780 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[295];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14118 in for-each-loop2764 in k14063 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14120(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14120,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14110(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14145 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14147(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_14147,c,av);}
a=C_alloc(9);
t2=C_i_check_list_2(t1,lf[100]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14158,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14191,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_14191(t7,t3,t1);}

/* k14156 in k14145 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_14158(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14158,c,av);}
a=C_alloc(5);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14168,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t5=((C_word*)t3)[1];
f_14168(t5,((C_word*)t0)[2],((C_word*)t0)[3]);}

/* for-each-loop2826 in k14156 in k14145 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_fcall f_14168(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_14168,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14178,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1617: g2857 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14176 in for-each-loop2826 in k14156 in k14145 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_14178(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14178,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14168(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop2816 in k14145 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_14191(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_14191,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14201,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1616: g2842 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[295];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14199 in for-each-loop2816 in k14145 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14201(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14201,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14191(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14225 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14227(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_14227,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14231,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1621: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[22]+1);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
/* core.scm:1622: chicken.syntax#syntax-error */
t2=*((C_word*)lf[364]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[392];
av2[3]=lf[393];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* k14229 in k14225 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_14231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14231,c,av);}
t2=C_mutate((C_word*)lf[22]+1 /* (set! chicken.compiler.core#foreign-declarations ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14267 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14269(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14269,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14277,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_14277(t6,((C_word*)t0)[2],t1);}

/* for-each-loop2868 in k14267 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_14277(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_14277,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14287,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1627: g2884 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[399];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14285 in for-each-loop2868 in k14267 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14287(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14287,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14277(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14312 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14314(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14314,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14322,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_14322(t6,((C_word*)t0)[2],t1);}

/* for-each-loop2893 in k14312 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_14322(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_14322,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14332,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1630: g2909 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[401];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14330 in for-each-loop2893 in k14312 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14332(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14332,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14322(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_14354,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_14357,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1634: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_14357(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_14357,c,av);}
a=C_alloc(8);
t2=C_eqp(t1,lf[372]);
if(C_truep(t2)){
t3=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_nullp(t3))){
t4=C_set_block_item(lf[18] /* chicken.compiler.core#standard-bindings */,0,C_SCHEME_END_OF_LIST);
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14374,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14378,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=C_u_i_cdr(((C_word*)t0)[2]);
/* core.scm:1640: globalize-all */
t7=((C_word*)((C_word*)t0)[4])[1];
f_13754(t7,t5,C_u_i_cdr(t6));}}
else{
t3=C_eqp(t1,lf[373]);
if(C_truep(t3)){
t4=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_nullp(t4))){
t5=C_set_block_item(lf[19] /* chicken.compiler.core#extended-bindings */,0,C_SCHEME_END_OF_LIST);
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14402,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14406,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t7=C_u_i_cdr(((C_word*)t0)[2]);
/* core.scm:1646: globalize-all */
t8=((C_word*)((C_word*)t0)[4])[1];
f_13754(t8,t6,C_u_i_cdr(t7));}}
else{
t4=C_eqp(t1,lf[403]);
if(C_truep(t4)){
t5=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_nullp(t5))){
t6=C_set_block_item(lf[50] /* chicken.compiler.core#inline-locally */,0,C_SCHEME_FALSE);
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14434,a[2]=((C_word*)t0)[5],tmp=(C_word)a,a+=3,tmp);
t7=C_u_i_cdr(((C_word*)t0)[2]);
/* core.scm:1652: globalize-all */
t8=((C_word*)((C_word*)t0)[4])[1];
f_13754(t8,t6,C_u_i_cdr(t7));}}
else{
t5=C_eqp(t1,lf[374]);
if(C_truep(t5)){
t6=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_nullp(t6))){
t7=C_set_block_item(lf[18] /* chicken.compiler.core#standard-bindings */,0,C_SCHEME_END_OF_LIST);
t8=C_set_block_item(lf[19] /* chicken.compiler.core#extended-bindings */,0,C_SCHEME_END_OF_LIST);
t9=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t9;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14487,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t8=C_u_i_cdr(((C_word*)t0)[2]);
/* core.scm:1658: globalize-all */
t9=((C_word*)((C_word*)t0)[4])[1];
f_13754(t9,t7,C_u_i_cdr(t8));}}
else{
t6=C_eqp(t1,lf[406]);
if(C_truep(t6)){
t7=C_set_block_item(lf[51] /* chicken.compiler.core#enable-inline-files */,0,C_SCHEME_TRUE);
t8=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_pairp(t8))){
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14523,a[2]=((C_word*)t0)[5],tmp=(C_word)a,a+=3,tmp);
t10=C_u_i_cdr(((C_word*)t0)[2]);
/* core.scm:1666: globalize-all */
t11=((C_word*)((C_word*)t0)[4])[1];
f_13754(t11,t9,C_u_i_cdr(t10));}
else{
t9=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t9;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}}
else{
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14562,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1668: check-decl */
f_13700(t7,((C_word*)t0)[2],C_fix(1),C_a_i_list(&a,1,C_fix(1)));}}}}}}

/* k14372 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14374(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14374,c,av);}
t2=C_mutate((C_word*)lf[18]+1 /* (set! chicken.compiler.core#standard-bindings ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14376 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14378(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_14378,c,av);}
a=C_alloc(3);
/* core.scm:1639: lset-difference/eq? */
f_7175(((C_word*)t0)[2],*((C_word*)lf[86]+1),C_a_i_list(&a,1,t1));}

/* k14400 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14402,c,av);}
t2=C_mutate((C_word*)lf[19]+1 /* (set! chicken.compiler.core#extended-bindings ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14404 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14406(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_14406,c,av);}
a=C_alloc(3);
/* core.scm:1645: lset-difference/eq? */
f_7175(((C_word*)t0)[2],*((C_word*)lf[85]+1),C_a_i_list(&a,1,t1));}

/* k14432 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14434(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14434,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14442,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_14442(t6,((C_word*)t0)[2],t1);}

/* for-each-loop2924 in k14432 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_fcall f_14442(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_14442,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14452,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1651: g2940 */
t6=*((C_word*)lf[159]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[404];
av2[4]=lf[405];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14450 in for-each-loop2924 in k14432 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_14452(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14452,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14442(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14485 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14487(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_14487,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14491,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1659: lset-difference/eq? */
f_7175(t2,*((C_word*)lf[86]+1),C_a_i_list(&a,1,t1));}

/* k14489 in k14485 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_14491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_14491,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[18]+1 /* (set! chicken.compiler.core#standard-bindings ...) */,t1);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14495,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1660: lset-difference/eq? */
f_7175(t3,*((C_word*)lf[85]+1),C_a_i_list(&a,1,((C_word*)t0)[3]));}

/* k14493 in k14489 in k14485 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_14495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14495,c,av);}
t2=C_mutate((C_word*)lf[19]+1 /* (set! chicken.compiler.core#extended-bindings ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14521 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14523(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14523,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14531,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_14531(t6,((C_word*)t0)[2],t1);}

/* for-each-loop2956 in k14521 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_fcall f_14531(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_14531,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14541,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1665: g2972 */
t6=*((C_word*)lf[159]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[407];
av2[4]=lf[405];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14539 in for-each-loop2956 in k14521 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_14541(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14541,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14531(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14560 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14562(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_14562,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14565,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1669: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k14563 in k14560 in k14355 in k14352 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_14565(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14565,c,av);}
t2=C_eqp(t1,lf[381]);
if(C_truep(t2)){
t3=C_set_block_item(lf[149] /* chicken.compiler.support#unsafe */,0,C_SCHEME_TRUE);
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:1672: chicken.base#warning */
t3=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[408];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k14612 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14614(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_14614,c,av);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t1))){
t2=C_set_block_item(lf[25] /* chicken.compiler.core#block-compilation */,0,C_SCHEME_TRUE);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=*((C_word*)lf[158]+1);
t3=C_i_check_list_2(t1,lf[100]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14629,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_14629(t7,((C_word*)t0)[3],t1);}}

/* for-each-loop2995 in k14612 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_14629(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_14629,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14639,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1679: g2996 */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14637 in for-each-loop2995 in k14612 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14639(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14639,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14629(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14660 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_14662,c,av);}
a=C_alloc(6);
t2=*((C_word*)lf[413]+1);
t3=C_i_check_list_2(t1,lf[100]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14670,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_14670(t7,((C_word*)t0)[2],t1);}

/* for-each-loop3013 in k14660 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_14670(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_14670,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14680,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1683: g3014 */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14678 in for-each-loop3013 in k14660 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14680,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14670(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14719 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14721(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14721,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14729,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_14729(t6,((C_word*)t0)[2],t1);}

/* for-each-loop3031 in k14719 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_14729(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_14729,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14739,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1690: g3047 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[415];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14737 in for-each-loop3031 in k14719 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14739(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14739,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14729(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14761 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14763(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14763,c,av);}
t2=C_i_cadr(((C_word*)t0)[2]);
if(C_truep(C_i_numberp(t2))){
t3=C_mutate((C_word*)lf[39]+1 /* (set! chicken.compiler.core#inline-max-size ...) */,t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:1697: chicken.base#warning */
t3=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[417];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k14783 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14785,c,av);}
t2=C_i_cadr(((C_word*)t0)[2]);
if(C_truep(C_i_numberp(t2))){
t3=C_mutate((C_word*)lf[40]+1 /* (set! chicken.compiler.core#unroll-limit ...) */,t2);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:1705: chicken.base#warning */
t3=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[419];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k14805 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14807(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_14807,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14813,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1710: every */
f_6675(t2,*((C_word*)lf[106]+1),t1);}

/* k14811 in k14805 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_14813(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_14813,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14821,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1713: globalize-all */
t3=((C_word*)((C_word*)t0)[3])[1];
f_13754(t3,t2,((C_word*)t0)[4]);}
else{
/* core.scm:1714: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[422];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k14819 in k14811 in k14805 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_14821,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14829,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_14829(t6,((C_word*)t0)[2],t1);}

/* for-each-loop3061 in k14819 in k14811 in k14805 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_fcall f_14829(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_14829,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14839,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1712: g3077 */
t6=*((C_word*)lf[159]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[421];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k14837 in for-each-loop3061 in k14819 in k14811 in k14805 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_14839(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_14839,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_14829(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k14863 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14865(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14865,c,av);}
t2=C_mutate((C_word*)lf[44]+1 /* (set! chicken.compiler.core#import-libraries ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k14882 in map-loop3086 in k14939 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14884(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_14884,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
f_14974(2,av2);}}

/* k14886 in map-loop3086 in k14939 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14888(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14888,c,av);}
/* ##sys#string-append */
t2=*((C_word*)lf[253]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[424];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k14893 in map-loop3086 in k14939 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_fcall f_14895(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_14895,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_i_car(((C_word*)t0)[2]);
t3=C_i_cadr(((C_word*)t0)[2]);
t4=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,t2,t3);
f_14974(2,av2);}}
else{
/* core.scm:1727: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[425];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k14939 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_14941,c,av);}
a=C_alloc(10);
t2=C_i_check_list_2(t1,lf[10]);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14947,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_14949,a[2]=((C_word*)t0)[3],a[3]=t5,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_14949(t7,t3,t1);}

/* k14945 in k14939 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_14947(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_14947,c,av);}
/* core.scm:1718: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[44]+1);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop3086 in k14939 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_14949(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,2)))){
C_save_and_reclaim_args((void *)trf_14949,3,t0,t1,t2);}
a=C_alloc(13);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_14974,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
if(C_truep(C_i_symbolp(t4))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14884,a[2]=t3,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_14888,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1722: scheme#symbol->string */
t7=*((C_word*)lf[255]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_14895,a[2]=t4,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_listp(t4))){
t6=C_eqp(C_fix(2),C_u_i_length(t4));
if(C_truep(t6)){
t7=C_i_car(t4);
t8=C_i_symbolp(t7);
t9=t5;
f_14895(t9,(C_truep(t8)?C_i_stringp(C_i_cadr(t4)):C_SCHEME_FALSE));}
else{
t7=t5;
f_14895(t7,C_SCHEME_FALSE);}}
else{
t6=t5;
f_14895(t6,C_SCHEME_FALSE);}}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k14972 in map-loop3086 in k14939 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_14974(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_14974,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_14949(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k14997 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_14999(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_14999,c,av);}
t2=C_mutate((C_word*)lf[58]+1 /* (set! chicken.compiler.core#types-output-file ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k15000 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_15002(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_15002,2,t0,t1);}
if(C_truep(t1)){
t2=C_mutate((C_word*)lf[58]+1 /* (set! chicken.compiler.core#types-output-file ...) */,t1);
t3=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[368];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
/* core.scm:1734: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[427];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k15049 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_15051(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_15051,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15059,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_15059(t6,((C_word*)t0)[2],t1);}

/* for-each-loop3132 in k15049 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_15059(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_15059,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15069,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1742: g3148 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[186];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k15067 in for-each-loop3132 in k15049 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_15069(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15069,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_15059(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k15103 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_15105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_15105,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15113,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_15113(t6,((C_word*)t0)[2],t1);}

/* for-each-loop3163 in k15103 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_15113(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_15113,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15123,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1749: g3179 */
t6=*((C_word*)lf[159]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[415];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k15121 in for-each-loop3163 in k15103 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_15123(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15123,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_15113(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k15158 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_15160(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_15160,c,av);}
a=C_alloc(5);
t2=C_i_check_list_2(t1,lf[100]);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15168,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_15168(t6,((C_word*)t0)[2],t1);}

/* for-each-loop3188 in k15158 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_15168(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_15168,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15178,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_slot(t2,C_fix(0));
t5=*((C_word*)lf[159]+1);
/* core.scm:1756: g3204 */
t6=*((C_word*)lf[159]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t3;
av2[2]=t4;
av2[3]=lf[407];
av2[4]=lf[230];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k15176 in for-each-loop3188 in k15158 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_15178(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15178,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_15168(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_15200(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_15200,3,t0,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15207,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_listp(t2))){
if(C_truep(C_fixnum_greater_or_equal_p(C_u_i_length(t2),C_fix(2)))){
t4=C_i_car(t2);
t5=t3;
f_15207(t5,C_i_not(C_i_symbolp(t4)));}
else{
t4=t3;
f_15207(t4,C_SCHEME_TRUE);}}
else{
t4=t3;
f_15207(t4,C_SCHEME_TRUE);}}

/* k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_15207(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15207,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15214,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1764: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15217,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1765: globalize */
t3=((C_word*)((C_word*)t0)[6])[1];
f_13735(t3,t2,C_i_car(((C_word*)t0)[3]));}}

/* k15212 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_15214(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15214,c,av);}
/* core.scm:1764: chicken.base#warning */
t2=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[431];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_15217(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_15217,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15220,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1766: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_15220(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_15220,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15226,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1767: local? */
t3=((C_word*)t0)[6];
f_12842(t3,t2,C_u_i_car(((C_word*)t0)[4]));}

/* k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_15226(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_15226,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
/* core.scm:1768: note-local */
f_13760(((C_word*)t0)[3],C_u_i_car(((C_word*)t0)[4]));}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15236,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15242,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1769: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}}

/* a15235 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_15236(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15236,c,av);}
/* core.scm:1769: chicken.compiler.scrutinizer#validate-type */
t2=*((C_word*)lf[432]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a15241 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_15242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_15242,c,av);}
a=C_alloc(8);
if(C_truep(t2)){
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_15290,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t3,a[7]=t4,tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_pairp(t2))){
t6=C_eqp(lf[437],C_u_i_car(t2));
t7=t5;
f_15290(t7,(C_truep(t6)?C_i_symbolp(C_i_cadr(t2)):C_SCHEME_FALSE));}
else{
t6=t5;
f_15290(t6,C_SCHEME_FALSE);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15322,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1791: chicken.syntax#strip-syntax */
t6=*((C_word*)lf[147]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k15250 in k15288 in a15241 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_15252(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_15252,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15255,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1779: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[435];
av2[4]=lf[429];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k15253 in k15250 in k15288 in a15241 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_15255(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_15255,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15258,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[6])){
/* core.scm:1781: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[421];
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_15258(2,av2);}}}

/* k15256 in k15253 in k15250 in k15288 in a15241 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_15258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_15258,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15261,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[5])){
/* core.scm:1783: chicken.compiler.support#mark-variable */
t3=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[434];
av2[4]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_15261(2,av2);}}}

/* k15259 in k15256 in k15253 in k15250 in k15288 in a15241 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in ... */
static void C_ccall f_15261(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_15261,c,av);}
a=C_alloc(4);
t2=C_i_cddr(((C_word*)t0)[2]);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15274,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
t4=C_u_i_cdr(((C_word*)t0)[2]);
/* core.scm:1787: chicken.syntax#strip-syntax */
t5=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=C_u_i_cdr(t4);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k15272 in k15259 in k15256 in k15253 in k15250 in k15288 in a15241 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in ... */
static void C_ccall f_15274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15274,c,av);}
/* core.scm:1785: chicken.compiler.scrutinizer#install-specializations */
t2=*((C_word*)lf[433]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k15288 in a15241 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_fcall f_15290(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,4)))){
C_save_and_reclaim_args((void *)trf_15290,2,t0,t1);}
a=C_alloc(7);
t2=(C_truep(t1)?C_i_set_car(C_i_cdr(((C_word*)t0)[2]),((C_word*)t0)[3]):C_SCHEME_UNDEFINED);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15252,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1778: chicken.compiler.support#mark-variable */
t4=*((C_word*)lf[159]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[436];
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k15320 in a15241 in k15224 in k15218 in k15215 in k15205 in g3216 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_15322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15322,c,av);}
/* core.scm:1789: chicken.base#warning */
t2=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[438];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* for-each-loop3215 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_15364(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_15364,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15374,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1759: g3216 */
t4=((C_word*)t0)[3];
f_15200(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k15372 in for-each-loop3215 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_15374(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15374,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_15364(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* g3260 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_15392(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_15392,3,t0,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15399,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_listp(t2))){
t4=C_i_car(t2);
t5=C_i_symbolp(t4);
t6=t3;
f_15399(t6,(C_truep(t5)?C_eqp(C_fix(2),C_i_length(t2)):C_SCHEME_FALSE));}
else{
t4=t3;
f_15399(t4,C_SCHEME_FALSE);}}

/* k15397 in g3260 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_15399(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_15399,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_15402,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1797: globalize */
t3=((C_word*)((C_word*)t0)[6])[1];
f_13735(t3,t2,C_i_car(((C_word*)t0)[4]));}
else{
/* core.scm:1806: chicken.base#warning */
t2=*((C_word*)lf[135]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[441];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k15400 in k15397 in g3260 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_15402(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_15402,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15405,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1798: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_i_cadr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15403 in k15400 in k15397 in g3260 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_15405(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_15405,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15411,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1799: local? */
t3=((C_word*)t0)[6];
f_12842(t3,t2,C_u_i_car(((C_word*)t0)[4]));}

/* k15409 in k15403 in k15400 in k15397 in g3260 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_15411(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_15411,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
/* core.scm:1800: note-local */
f_13760(((C_word*)t0)[3],C_u_i_car(((C_word*)t0)[4]));}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15421,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15427,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1801: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}}

/* a15420 in k15409 in k15403 in k15400 in k15397 in g3260 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_15421(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15421,c,av);}
/* core.scm:1801: chicken.compiler.scrutinizer#validate-type */
t2=*((C_word*)lf[432]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a15426 in k15409 in k15403 in k15400 in k15397 in g3260 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_15427(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_15427,c,av);}
t5=(C_truep(t2)?C_i_not(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
/* core.scm:1803: chicken.compiler.support#mark-variable */
t6=*((C_word*)lf[159]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[434];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
/* core.scm:1804: chicken.base#warning */
t6=*((C_word*)lf[135]+1);{
C_word *av2=av;
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[440];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* for-each-loop3259 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_15486(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_15486,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15496,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1794: g3260 */
t4=((C_word*)t0)[3];
f_15392(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k15494 in for-each-loop3259 in k13793 in k13790 in a13787 in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_15496(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15496,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_15486(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.core#build-toplevel-procedure in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_15630,c,av);}
a=C_alloc(3);
t3=C_a_i_list1(&a,1,t2);
/* core.scm:1819: chicken.compiler.support#make-node */
t4=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=lf[487];
av2[3]=lf[488];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* chicken.compiler.core#foreign-stub-id in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15653(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15653,c,av);}
t3=C_i_check_structure_2(t2,lf[490],lf[491]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#foreign-stub-return-type in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15662(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15662,c,av);}
t3=C_i_check_structure_2(t2,lf[490],lf[493]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#foreign-stub-name in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15671(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15671,c,av);}
t3=C_i_check_structure_2(t2,lf[490],lf[495]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(3));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#foreign-stub-argument-types in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15680,c,av);}
t3=C_i_check_structure_2(t2,lf[490],lf[497]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(4));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#foreign-stub-argument-names in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15689(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15689,c,av);}
t3=C_i_check_structure_2(t2,lf[490],lf[499]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(5));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#foreign-stub-body in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15698(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15698,c,av);}
t3=C_i_check_structure_2(t2,lf[490],lf[501]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(6));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#foreign-stub-cps in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15707,c,av);}
t3=C_i_check_structure_2(t2,lf[490],lf[503]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(7));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#foreign-stub-callback in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15716(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_15716,c,av);}
t3=C_i_check_structure_2(t2,lf[490],lf[505]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(8));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_15725(C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7,C_word t8){
C_word tmp;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_15725,8,t1,t2,t3,t4,t5,t6,t7,t8);}
a=C_alloc(9);
t9=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_15812,a[2]=t3,a[3]=t5,a[4]=t6,a[5]=t8,a[6]=t7,a[7]=t1,a[8]=t4,tmp=(C_word)a,a+=9,tmp);
/* core.scm:1850: chicken.syntax#strip-syntax */
t10=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}

/* loop in map-loop3413 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_15734(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_15734,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=lf[512];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_listp(t2))){
t3=C_i_car(t2);
t4=C_eqp(t3,lf[330]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15763,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15767,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1845: loop */
t8=t6;
t9=C_u_i_cdr(t2);
t1=t8;
t2=t9;
goto loop;}
else{
t5=C_eqp(t3,lf[515]);
t6=(C_truep(t5)?t5:C_eqp(t3,lf[516]));
if(C_truep(t6)){
/* core.scm:1846: loop */
t8=t1;
t9=C_u_i_cdr(t2);
t1=t8;
t2=t9;
goto loop;}
else{
/* core.scm:1847: loop */
t8=t1;
t9=C_u_i_car(t2);
t1=t8;
t2=t9;
goto loop;}}}
else{
t3=C_i_symbolp(t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=(C_truep(t3)?t2:lf[512]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_stringp(t2);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(C_truep(t4)?t2:lf[512]);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}}}

/* k15761 in loop in map-loop3413 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15763(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_15763,c,av);}
/* core.scm:1845: scheme#string->symbol */
t2=*((C_word*)lf[509]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k15765 in loop in map-loop3413 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15767,c,av);}
/* core.scm:1845: chicken.string#conc */
t2=*((C_word*)lf[513]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[514];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15812(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_15812,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_15815,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1851: chicken.syntax#strip-syntax */
t3=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15815(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_15815,c,av);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_15818,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[4],lf[10]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16028,a[2]=t5,a[3]=t9,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_16028(t11,t2,((C_word*)t0)[4]);}
else{
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(t1,lf[10]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16080,a[2]=t5,a[3]=t9,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_16080(t11,t2,t1);}}

/* k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15818(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_15818,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_15821,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* core.scm:1855: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[511];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_15821,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_15824,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* core.scm:1856: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_15824,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_15827,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,tmp=(C_word)a,a+=13,tmp);
/* core.scm:1857: chicken.compiler.support#estimate-foreign-result-size */
t3=*((C_word*)lf[510]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15827(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_15827,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_15830,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
if(C_truep(((C_word*)t0)[4])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16017,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1858: scheme#string->symbol */
t4=*((C_word*)lf[509]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_15830(2,av2);}}}

/* k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(60,c,2)))){
C_save_and_reclaim((void *)f_15830,c,av);}
a=C_alloc(60);
t2=C_a_i_record(&a,9,lf[490],((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);
t3=C_a_i_cons(&a,2,t2,*((C_word*)lf[71]+1));
t4=C_mutate((C_word*)lf[71]+1 /* (set! chicken.compiler.core#foreign-lambda-stubs ...) */,t3);
t5=(C_truep(((C_word*)t0)[9])?C_s_a_i_plus(&a,2,((C_word*)t0)[10],C_fix(24)):((C_word*)t0)[10]);
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_15840,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[9],a[6]=t5,a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[13],tmp=(C_word)a,a+=9,tmp);
if(C_truep(((C_word*)t0)[8])){
t7=C_a_i_list(&a,2,lf[474],((C_word*)t0)[2]);
t8=t6;
f_15840(t8,C_a_i_list(&a,1,t7));}
else{
t7=t6;
f_15840(t7,C_a_i_list(&a,2,lf[300],((C_word*)t0)[2]));}}

/* k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_15840(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,4)))){
C_save_and_reclaim_args((void *)trf_15840,2,t0,t1);}
a=C_alloc(21);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(((C_word*)t0)[2],lf[10]);
t7=C_i_check_list_2(((C_word*)t0)[3],lf[10]);
t8=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_15861,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t1,a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15947,a[2]=t4,a[3]=t10,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_15947(t12,t8,((C_word*)t0)[2],((C_word*)t0)[3]);}

/* k15850 in map-loop3454 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15852(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15852,c,av);}
/* core.scm:1866: chicken.compiler.support#foreign-type-check */
t2=*((C_word*)lf[281]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15861(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_15861,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15872,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(C_truep(((C_word*)t0)[4])?lf[506]:C_SCHEME_END_OF_LIST);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15884,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
if(C_truep(C_i_zerop(((C_word*)t0)[5]))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15892,a[2]=t4,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t6=C_a_i_cons(&a,2,lf[507],t1);
/* core.scm:1871: scheme#append */
t7=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=((C_word*)t0)[7];
av2[3]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_15899,a[2]=((C_word*)t0)[8],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[6],a[6]=t1,a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1872: chicken.compiler.support#final-foreign-type */
t6=*((C_word*)lf[122]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k15870 in k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15872(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_15872,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_cons(&a,2,lf[97],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k15882 in k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15884(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_15884,c,av);}
a=C_alloc(3);
t2=C_a_i_list(&a,1,t1);
/* core.scm:1867: ##sys#append */
t3=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k15890 in k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15892(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15892,c,av);}
/* core.scm:1871: chicken.compiler.support#foreign-type-convert-result */
t2=*((C_word*)lf[120]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k15897 in k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15899(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_15899,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_15902,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1873: chicken.compiler.support#bytes->words */
t3=*((C_word*)lf[340]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k15900 in k15897 in k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_15902(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(75,c,3)))){
C_save_and_reclaim((void *)f_15902,c,av);}
a=C_alloc(75);
t2=C_s_a_i_plus(&a,2,C_fix(2),t1);
t3=C_a_i_list(&a,2,lf[508],t2);
t4=C_a_i_list(&a,2,lf[146],t1);
t5=C_a_i_list(&a,3,lf[301],t3,t4);
t6=C_a_i_list(&a,2,((C_word*)t0)[2],t5);
t7=C_a_i_list(&a,1,t6);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_15913,a[2]=t7,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15917,a[2]=t8,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15921,a[2]=t9,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
t11=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[7]);
/* core.scm:1876: scheme#append */
t12=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t12;
av2[1]=t10;
av2[2]=((C_word*)t0)[8];
av2[3]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(4,av2);}}

/* k15911 in k15900 in k15897 in k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_15913(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_15913,c,av);}
a=C_alloc(12);
t2=C_a_i_list(&a,3,lf[173],((C_word*)t0)[2],t1);
t3=C_a_i_list(&a,1,t2);
/* core.scm:1867: ##sys#append */
t4=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k15915 in k15900 in k15897 in k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_15917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15917,c,av);}
/* core.scm:1875: chicken.compiler.support#foreign-type-convert-result */
t2=*((C_word*)lf[120]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k15919 in k15900 in k15897 in k15859 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_15921(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_15921,c,av);}
/* core.scm:1876: chicken.compiler.support#finish-foreign-result */
t2=*((C_word*)lf[121]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop3454 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_15947(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_15947,4,t0,t1,t2,t3);}
a=C_alloc(11);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_15976,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
t7=C_slot(t2,C_fix(0));
t8=C_slot(t3,C_fix(0));
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_15852,a[2]=t6,a[3]=t8,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1866: chicken.compiler.support#foreign-type-convert-argument */
t10=*((C_word*)lf[282]+1);{
C_word av2[4];
av2[0]=t10;
av2[1]=t9;
av2[2]=t7;
av2[3]=t8;
((C_proc)(void*)(*((C_word*)t10+1)))(4,av2);}}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k15974 in map-loop3454 in k15838 in k15828 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_15976(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_15976,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_15947(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)),C_slot(((C_word*)t0)[6],C_fix(1)));}

/* k16015 in k15825 in k15822 in k15819 in k15816 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16017,c,av);}
/* core.scm:1858: chicken.compiler.support#set-real-name! */
t2=*((C_word*)lf[117]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop3387 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16028(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_16028,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16053,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1853: g3393 */
t4=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16051 in map-loop3387 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16053(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_16053,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_16028(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k16070 in map-loop3413 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16072(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16072,c,av);}
/* core.scm:1854: chicken.base#gensym */
t2=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* map-loop3413 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16080(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,3)))){
C_save_and_reclaim_args((void *)trf_16080,3,t0,t1,t2);}
a=C_alloc(14);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16105,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16072,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_15734,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t9=((C_word*)t7)[1];
f_15734(t9,t5,t4);}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16103 in map-loop3413 in k15813 in k15810 in chicken.compiler.core#create-foreign-stub in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16105(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_16105,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_16080(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* chicken.compiler.core#expand-foreign-lambda in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16114(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,8)))){
C_save_and_reclaim_args((void *)trf_16114,3,t1,t2,t3);}
a=C_alloc(8);
t4=C_i_caddr(t2);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16121,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_symbolp(t4))){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16140,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1881: chicken.syntax#strip-syntax */
t7=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
if(C_truep(C_i_stringp(t4))){
/* core.scm:1888: create-foreign-stub */
f_15725(t1,C_i_cadr(t2),t4,C_i_cdddr(t2),C_SCHEME_FALSE,C_SCHEME_FALSE,t3,t3);}
else{
/* core.scm:1883: chicken.compiler.support#quit-compiling */
t6=*((C_word*)lf[125]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[517];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}}

/* k16119 in chicken.compiler.core#expand-foreign-lambda in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16121(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_16121,c,av);}
/* core.scm:1888: create-foreign-stub */
f_15725(((C_word*)t0)[2],C_i_cadr(((C_word*)t0)[3]),t1,C_i_cdddr(((C_word*)t0)[3]),C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[4],((C_word*)t0)[4]);}

/* k16138 in chicken.compiler.core#expand-foreign-lambda in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16140(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16140,c,av);}
/* core.scm:1881: scheme#symbol->string */
t2=*((C_word*)lf[255]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* chicken.compiler.core#expand-foreign-lambda* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16151(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_16151,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_cadr(t2);
t5=C_i_caddr(t2);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16161,a[2]=t5,a[3]=t1,a[4]=t4,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* core.scm:1893: chicken.string#string-intersperse */
t7=*((C_word*)lf[317]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_cdddr(t2);
av2[3]=lf[518];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k16159 in chicken.compiler.core#expand-foreign-lambda* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16161(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_16161,c,av);}
a=C_alloc(19);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(((C_word*)t0)[2],lf[10]);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16175,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16226,a[2]=t4,a[3]=t9,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_16226(t11,t7,((C_word*)t0)[2]);}

/* k16173 in k16159 in chicken.compiler.core#expand-foreign-lambda* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16175(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_16175,c,av);}
a=C_alloc(14);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_16181,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t4,a[8]=t5,tmp=(C_word)a,a+=9,tmp);
/* core.scm:1896: chicken.syntax#strip-syntax */
t7=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k16179 in k16173 in k16159 in chicken.compiler.core#expand-foreign-lambda* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16181(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_16181,c,av);}
a=C_alloc(14);
t2=C_i_check_list_2(t1,lf[10]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16187,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16192,a[2]=((C_word*)t0)[7],a[3]=t5,a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_16192(t7,t3,t1);}

/* k16185 in k16179 in k16173 in k16159 in chicken.compiler.core#expand-foreign-lambda* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16187(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_16187,c,av);}
/* core.scm:1897: create-foreign-stub */
f_15725(((C_word*)t0)[2],((C_word*)t0)[3],C_SCHEME_FALSE,((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[6]);}

/* map-loop3544 in k16179 in k16173 in k16159 in chicken.compiler.core#expand-foreign-lambda* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16192(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_16192,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_cadr(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* map-loop3516 in k16159 in chicken.compiler.core#expand-foreign-lambda* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16226(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_16226,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k16266 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_16268(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_16268,2,t0,t1);}
a=C_alloc(6);
t2=(C_truep(t1)?C_i_cadr(((C_word*)t0)[2]):lf[315]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16274,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=t1,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
t4=(C_truep(t1)?C_i_caddr(((C_word*)t0)[2]):C_i_cadr(((C_word*)t0)[2]));
/* core.scm:1903: chicken.syntax#strip-syntax */
t5=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k16272 in k16266 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_16274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_16274,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16277,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(C_truep(((C_word*)t0)[4])?C_i_cdddr(((C_word*)t0)[5]):C_i_cddr(((C_word*)t0)[5]));
/* core.scm:1904: chicken.string#string-intersperse */
t4=*((C_word*)lf[317]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=t3;
av2[3]=lf[318];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k16275 in k16272 in k16266 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_16277(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_16277,c,av);}
a=C_alloc(18);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(((C_word*)t0)[2],lf[10]);
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16291,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16342,a[2]=t4,a[3]=t9,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_16342(t11,t7,((C_word*)t0)[2]);}

/* k16289 in k16275 in k16272 in k16266 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_16291(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_16291,c,av);}
a=C_alloc(13);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16297,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=t4,a[7]=t5,tmp=(C_word)a,a+=8,tmp);
/* core.scm:1907: chicken.syntax#strip-syntax */
t7=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k16295 in k16289 in k16275 in k16272 in k16266 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_16297(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_16297,c,av);}
a=C_alloc(13);
t2=C_i_check_list_2(t1,lf[10]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16303,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16308,a[2]=((C_word*)t0)[6],a[3]=t5,a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp));
t7=((C_word*)t5)[1];
f_16308(t7,t3,t1);}

/* k16301 in k16295 in k16289 in k16275 in k16272 in k16266 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_16303(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_16303,c,av);}
/* core.scm:1908: create-foreign-stub */
f_15725(((C_word*)t0)[2],((C_word*)t0)[3],C_SCHEME_FALSE,((C_word*)t0)[4],t1,((C_word*)t0)[5],C_SCHEME_FALSE,C_SCHEME_TRUE);}

/* map-loop3606 in k16295 in k16289 in k16275 in k16272 in k16266 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_16308(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_16308,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_cadr(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* map-loop3578 in k16275 in k16272 in k16266 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_16342(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_16342,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* mapupdate in k7899 */
static void C_fcall f_16420(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_16420,3,t0,t1,t2);}
a=C_alloc(6);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16426,a[2]=t4,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_16426(t6,t1,t2);}

/* loop in mapupdate in k7899 */
static void C_fcall f_16426(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16426,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16436,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1917: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_16445(t4,t3,C_u_i_car(t2));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k16434 in loop in mapupdate in k7899 */
static void C_ccall f_16436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16436,c,av);}
/* core.scm:1918: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16426(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* walk in k7899 */
static void C_fcall f_16445(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_16445,3,t0,t1,t2);}
a=C_alloc(7);
t3=C_i_pairp(t2);
t4=C_i_not(t3);
if(C_truep(t4)){
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t5=C_i_car(t2);
if(C_truep(C_i_symbolp(t5))){
t6=C_u_i_car(t2);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16462,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=t6,tmp=(C_word)a,a+=7,tmp);
/* core.scm:1923: chicken.internal#hash-table-ref */
t8=*((C_word*)lf[112]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t7;
av2[2]=*((C_word*)lf[111]+1);
av2[3]=t6;
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}
else{
/* core.scm:1927: mapupdate */
t6=((C_word*)((C_word*)t0)[2])[1];
f_16420(t6,t1,t2);}}}

/* k16460 in walk in k7899 */
static void C_ccall f_16462(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_16462,c,av);}
a=C_alloc(11);
t2=(C_truep(t1)?t1:C_SCHEME_END_OF_LIST);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16468,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_assq(((C_word*)t0)[4],t2))){
/* core.scm:1926: mapupdate */
t4=((C_word*)((C_word*)t0)[2])[1];
f_16420(t4,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t4=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]);
t5=C_a_i_cons(&a,2,t4,t2);
/* core.scm:1925: chicken.internal#hash-table-set! */
t6=*((C_word*)lf[110]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t3;
av2[2]=*((C_word*)lf[111]+1);
av2[3]=((C_word*)t0)[6];
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}

/* k16466 in k16460 in walk in k7899 */
static void C_ccall f_16468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16468,c,av);}
/* core.scm:1926: mapupdate */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16420(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}

/* chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16499(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(44,c,6)))){
C_save_and_reclaim((void *)f_16499,c,av);}
a=C_alloc(44);
t3=C_SCHEME_END_OF_LIST;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_SCHEME_UNDEFINED;
t18=(*a=C_VECTOR_TYPE|1,a[1]=t17,tmp=(C_word)a,a+=2,tmp);
t19=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16502,a[2]=t10,tmp=(C_word)a,a+=3,tmp));
t20=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16553,tmp=(C_word)a,a+=2,tmp));
t21=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16577,a[2]=t10,a[3]=t8,a[4]=t6,a[5]=t14,a[6]=t12,a[7]=t4,tmp=(C_word)a,a+=8,tmp));
t22=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17123,a[2]=t10,a[3]=t16,tmp=(C_word)a,a+=4,tmp));
t23=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17269,a[2]=t16,tmp=(C_word)a,a+=3,tmp));
t24=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17285,a[2]=t8,a[3]=t10,a[4]=t18,tmp=(C_word)a,a+=5,tmp));
t25=C_set_block_item(t18,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17387,a[2]=t18,tmp=(C_word)a,a+=3,tmp));
/* core.scm:2066: walk */
t26=((C_word*)t10)[1];
f_16577(t26,t1,t2,*((C_word*)lf[105]+1));}

/* cps-lambda in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16502(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_16502,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(8);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_16506,a[2]=t5,a[3]=t1,a[4]=t3,a[5]=t2,a[6]=t4,a[7]=((C_word*)t0)[2],tmp=(C_word)a,a+=8,tmp);
/* core.scm:1937: chicken.base#gensym */
t7=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[522];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k16504 in cps-lambda in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16506(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_16506,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16513,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_a_i_cons(&a,2,t1,((C_word*)t0)[4]);
t4=C_a_i_list4(&a,4,((C_word*)t0)[5],C_SCHEME_TRUE,t3,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16525,a[2]=t2,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
t6=C_i_car(((C_word*)t0)[6]);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16531,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* core.scm:1940: walk */
t8=((C_word*)((C_word*)t0)[7])[1];
f_16577(t8,t5,t6,t7);}

/* k16511 in k16504 in cps-lambda in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16513(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16513,c,av);}
/* core.scm:1938: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k16523 in k16504 in cps-lambda in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16525(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_16525,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:1938: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[97];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* a16530 in k16504 in cps-lambda in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16531(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_16531,c,av);}
a=C_alloc(8);
t3=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16547,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1942: chicken.compiler.support#varnode */
t5=*((C_word*)lf[521]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k16545 in a16530 in k16504 in cps-lambda in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16547(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_16547,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
/* core.scm:1942: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[520];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* node-for-var? in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16553(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_16553,3,t1,t2,t3);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16575,a[2]=t1,a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1945: chicken.compiler.support#node-class */
t5=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k16569 in k16573 in node-for-var? in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16571(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16571,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_eqp(C_i_car(t1),((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k16573 in node-for-var? in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16575(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_16575,c,av);}
a=C_alloc(4);
t2=C_eqp(t1,lf[523]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16571,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1946: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16577(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_16577,4,t0,t1,t2,t3);}
a=C_alloc(11);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_16581,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],tmp=(C_word)a,a+=11,tmp);
/* core.scm:1949: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[527]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_16581,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_16584,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* core.scm:1950: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16584(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_16584,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_16587,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* core.scm:1951: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16587(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_16587,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_16590,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* core.scm:1952: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_16590,c,av);}
a=C_alloc(15);
t2=C_eqp(t1,lf[523]);
t3=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_16599,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
if(C_truep(t2)){
t4=t3;
f_16599(t4,t2);}
else{
t4=C_eqp(t1,lf[118]);
if(C_truep(t4)){
t5=t3;
f_16599(t5,t4);}
else{
t5=C_eqp(t1,lf[475]);
if(C_truep(t5)){
t6=t3;
f_16599(t6,t5);}
else{
t6=C_eqp(t1,lf[474]);
t7=t3;
f_16599(t7,(C_truep(t6)?t6:C_eqp(t1,lf[163])));}}}}

/* k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16599(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,4)))){
C_save_and_reclaim_args((void *)trf_16599,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
/* core.scm:1954: k */
t2=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[5],lf[143]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16611,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1955: chicken.base#gensym */
t4=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[522];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_eqp(((C_word*)t0)[5],lf[174]);
if(C_truep(t3)){
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16720,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[2],a[4]=t5,a[5]=((C_word*)t0)[8],tmp=(C_word)a,a+=6,tmp));
t7=((C_word*)t5)[1];
f_16720(t7,((C_word*)t0)[3],((C_word*)t0)[9],((C_word*)t0)[6]);}
else{
t4=C_eqp(((C_word*)t0)[5],lf[487]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[5],lf[97]));
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16807,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],tmp=(C_word)a,a+=7,tmp);
/* core.scm:373: chicken.base#gensym */
t7=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[525];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t6=C_eqp(((C_word*)t0)[5],lf[284]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16820,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[3],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1981: chicken.base#gensym */
t8=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=lf[526];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t7=C_eqp(((C_word*)t0)[5],lf[446]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16891,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[9],tmp=(C_word)a,a+=7,tmp);
/* core.scm:373: chicken.base#gensym */
t9=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=lf[525];
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t8=C_eqp(((C_word*)t0)[5],lf[300]);
t9=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_16921,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[13],a[10]=((C_word*)t0)[14],a[11]=((C_word*)t0)[7],tmp=(C_word)a,a+=12,tmp);
if(C_truep(t8)){
t10=t9;
f_16921(t10,t8);}
else{
t10=C_eqp(((C_word*)t0)[5],lf[301]);
if(C_truep(t10)){
t11=t9;
f_16921(t11,t10);}
else{
t11=C_eqp(((C_word*)t0)[5],lf[119]);
if(C_truep(t11)){
t12=t9;
f_16921(t12,t11);}
else{
t12=C_eqp(((C_word*)t0)[5],lf[280]);
if(C_truep(t12)){
t13=t9;
f_16921(t13,t12);}
else{
t13=C_eqp(((C_word*)t0)[5],lf[123]);
if(C_truep(t13)){
t14=t9;
f_16921(t14,t13);}
else{
t14=C_eqp(((C_word*)t0)[5],lf[283]);
t15=t9;
f_16921(t15,(C_truep(t14)?t14:C_eqp(((C_word*)t0)[5],lf[188])));}}}}}}}}}}}}

/* k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16611(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_16611,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16614,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1956: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[463];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16614(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_16614,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16615,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16646,a[2]=((C_word*)t0)[3],a[3]=t3,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16705,a[2]=t1,a[3]=t4,a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* core.scm:373: chicken.base#gensym */
t6=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[525];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k1 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16615(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_16615,c,av);}
a=C_alloc(8);
t3=C_a_i_list1(&a,1,C_SCHEME_TRUE);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16631,a[2]=t2,a[3]=t1,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1957: chicken.compiler.support#varnode */
t5=*((C_word*)lf[521]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k16629 in k1 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16631(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_16631,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,t1,((C_word*)t0)[2]);
/* core.scm:1957: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[520];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16644 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16646(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_16646,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16650,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_i_car(((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16656,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1963: walk */
t5=((C_word*)((C_word*)t0)[5])[1];
f_16577(t5,t2,t3,t4);}

/* k16648 in k16644 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16650(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_16650,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:1958: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* a16655 in k16644 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16656(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_16656,c,av);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16668,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1967: walk */
t4=((C_word*)((C_word*)t0)[2])[1];
f_16577(t4,t3,C_i_cadr(((C_word*)t0)[3]),((C_word*)t0)[4]);}

/* k16666 in a16655 in k16644 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_16668(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_16668,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16672,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:1968: walk */
t3=((C_word*)((C_word*)t0)[4])[1];
f_16577(t3,t2,C_i_caddr(((C_word*)t0)[5]),((C_word*)t0)[6]);}

/* k16670 in k16666 in a16655 in k16644 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_16672(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_16672,c,av);}
a=C_alloc(9);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],t1);
/* core.scm:1965: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[143];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16690 in k16703 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_16692,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:1961: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[97];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16694 in k16703 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16696(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16696,c,av);}
/* core.scm:1962: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k16703 in k16612 in k16609 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,2)))){
C_save_and_reclaim((void *)f_16705,c,av);}
a=C_alloc(23);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=C_a_i_list4(&a,4,t1,C_SCHEME_FALSE,t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16692,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16696,a[2]=((C_word*)t0)[4],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1962: chicken.compiler.support#varnode */
t6=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* loop in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16720(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_16720,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t2))){
/* core.scm:1972: walk */
t4=((C_word*)((C_word*)t0)[2])[1];
f_16577(t4,t1,C_i_car(t3),((C_word*)t0)[3]);}
else{
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16743,a[2]=((C_word*)t0)[4],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1973: walk */
t6=((C_word*)((C_word*)t0)[2])[1];
f_16577(t6,t1,t4,t5);}}

/* a16742 in loop in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_16743,c,av);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16750,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* core.scm:1975: node-for-var? */
f_16553(t3,t2,C_i_car(((C_word*)t0)[3]));}

/* k16748 in a16742 in loop in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16750(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_16750,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
/* core.scm:1976: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16720(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_i_cdr(((C_word*)t0)[5]));}
else{
t2=C_u_i_car(((C_word*)t0)[4]);
t3=C_a_i_list1(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16774,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1979: loop */
t5=((C_word*)((C_word*)t0)[2])[1];
f_16720(t5,t4,C_u_i_cdr(((C_word*)t0)[4]),C_i_cdr(((C_word*)t0)[5]));}}

/* k16772 in k16748 in a16742 in loop in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_16774,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:1977: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16805 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16807(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_16807,c,av);}
/* core.scm:1980: cps-lambda */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16502(t2,((C_word*)t0)[3],t1,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6]);}

/* k16818 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_16820,c,av);}
a=C_alloc(11);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_i_pairp(t2);
t4=(C_truep(t3)?C_i_cadr(((C_word*)t0)[2]):C_SCHEME_FALSE);
t5=C_i_car(((C_word*)t0)[2]);
t6=C_a_i_list2(&a,2,t5,t4);
t7=C_i_car(((C_word*)t0)[3]);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16835,a[2]=t1,a[3]=((C_word*)t0)[4],a[4]=t6,tmp=(C_word)a,a+=5,tmp);
/* core.scm:1984: walk */
t9=((C_word*)((C_word*)t0)[5])[1];
f_16577(t9,((C_word*)t0)[6],t7,t8);}

/* a16834 in k16818 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16835(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_16835,c,av);}
a=C_alloc(12);
t3=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_16851,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
t5=C_a_i_list1(&a,1,t2);
/* core.scm:1987: chicken.compiler.support#make-node */
t6=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t4;
av2[2]=lf[284];
av2[3]=((C_word*)t0)[4];
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k16849 in a16834 in k16818 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16851(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_16851,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16855,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16859,a[2]=((C_word*)t0)[4],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1988: chicken.compiler.support#varnode */
t4=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k16853 in k16849 in a16834 in k16818 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16855(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_16855,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:1986: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k16857 in k16849 in a16834 in k16818 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16859(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_16859,c,av);}
/* core.scm:1988: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k16889 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_16891,c,av);}
a=C_alloc(7);
t2=C_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16897,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,a[5]=((C_word*)t0)[5],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* core.scm:1992: chicken.compiler.support#register-foreign-callback-stub! */
t4=*((C_word*)lf[528]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
av2[3]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k16895 in k16889 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_16897,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16912,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1993: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k16906 in k16910 in k16895 in k16889 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16908(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_16908,c,av);}
/* core.scm:1993: cps-lambda */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16502(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1,((C_word*)t0)[6]);}

/* k16910 in k16895 in k16889 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16912(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_16912,c,av);}
a=C_alloc(7);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_16908,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:1993: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[527]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_16921(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,5)))){
C_save_and_reclaim_args((void *)trf_16921,2,t0,t1);}
a=C_alloc(21);
if(C_truep(t1)){
/* core.scm:1996: walk-inline-call */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17269(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}
else{
t2=C_eqp(((C_word*)t0)[8],lf[520]);
if(C_truep(t2)){
/* core.scm:1997: walk-call */
t3=((C_word*)((C_word*)t0)[9])[1];
f_17123(t3,((C_word*)t0)[3],C_i_car(((C_word*)t0)[6]),C_u_i_cdr(((C_word*)t0)[6]),((C_word*)t0)[5],((C_word*)t0)[7]);}
else{
t3=C_eqp(((C_word*)t0)[8],lf[164]);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[5]);
if(C_truep(C_i_memq(t4,((C_word*)((C_word*)t0)[10])[1]))){
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16961,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[7],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2001: chicken.compiler.support#make-node */
t6=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[475];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=C_a_i_cons(&a,2,t4,((C_word*)((C_word*)t0)[10])[1]);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_FALSE;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16969,a[2]=t9,a[3]=((C_word*)t0)[10],a[4]=t7,tmp=(C_word)a,a+=5,tmp);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16974,a[2]=t4,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_16980,a[2]=t7,a[3]=((C_word*)t0)[10],a[4]=t9,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2002: ##sys#dynamic-wind */
t13=*((C_word*)lf[116]+1);{
C_word av2[5];
av2[0]=t13;
av2[1]=((C_word*)t0)[3];
av2[2]=t10;
av2[3]=t11;
av2[4]=t12;
((C_proc)(void*)(*((C_word*)t13+1)))(5,av2);}}}
else{
t4=C_eqp(((C_word*)t0)[8],lf[151]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[8],lf[529]));
if(C_truep(t5)){
/* core.scm:2006: walk */
t6=((C_word*)((C_word*)t0)[11])[1];
f_16577(t6,((C_word*)t0)[3],C_i_car(((C_word*)t0)[6]),((C_word*)t0)[7]);}
else{
t6=C_eqp(((C_word*)t0)[8],lf[156]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7081,tmp=(C_word)a,a+=2,tmp);
t8=(
  f_7081(((C_word*)t0)[6])
);
/* core.scm:2009: walk */
t9=((C_word*)((C_word*)t0)[11])[1];
f_16577(t9,((C_word*)t0)[3],t8,((C_word*)t0)[7]);}
else{
/* core.scm:2010: chicken.compiler.support#bomb */
t7=*((C_word*)lf[530]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[531];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}}}}}

/* k16959 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16961(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_16961,c,av);}
/* core.scm:2001: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_16577(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4]);}

/* a16968 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16969,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16974(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_16974,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17204,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2027: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[522];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* a16979 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_16980(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_16980,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17123(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_17123,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(9);
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17127,a[2]=t1,a[3]=t4,a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=((C_word*)t0)[3],a[7]=t3,a[8]=t5,tmp=(C_word)a,a+=9,tmp);
/* core.scm:2013: chicken.base#gensym */
t7=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=lf[522];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17127(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_17127,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17130,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2014: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[463];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17130(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_17130,c,av);}
a=C_alloc(18);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17145,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17194,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[9],tmp=(C_word)a,a+=5,tmp);
/* core.scm:373: chicken.base#gensym */
t5=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[525];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k17143 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17145(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_17145,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17149,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17151,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2019: walk-arguments */
t4=((C_word*)((C_word*)t0)[8])[1];
f_17285(t4,t2,((C_word*)t0)[9],t3);}

/* k17147 in k17143 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_17149,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2015: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* a17150 in k17143 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17151(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_17151,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17157,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2022: walk */
t4=((C_word*)((C_word*)t0)[4])[1];
f_16577(t4,t1,((C_word*)t0)[5],t3);}

/* a17156 in a17150 in k17143 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17157(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_17157,c,av);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17165,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17169,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2024: chicken.compiler.support#varnode */
t5=*((C_word*)lf[521]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k17163 in a17156 in a17150 in k17143 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17165(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_17165,c,av);}
/* core.scm:2024: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[520];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k17167 in a17156 in a17150 in k17143 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17169(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_17169,c,av);}
a=C_alloc(6);
/* core.scm:2024: cons* */
f_6739(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,2,t1,((C_word*)t0)[4]));}

/* k17179 in k17192 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17181(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_17181,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2017: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[97];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17183 in k17192 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17185(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17185,c,av);}
/* core.scm:2018: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k17192 in k17128 in k17125 in walk-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17194(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,2)))){
C_save_and_reclaim((void *)f_17194,c,av);}
a=C_alloc(23);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=C_a_i_list4(&a,4,t1,C_SCHEME_FALSE,t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17181,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17185,a[2]=((C_word*)t0)[4],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2018: chicken.compiler.support#varnode */
t6=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k17202 in a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17204(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_17204,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17207,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2028: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[463];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17205 in k17202 in a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17207(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_17207,c,av);}
a=C_alloc(14);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17222,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17263,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* core.scm:373: chicken.base#gensym */
t5=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[525];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k17220 in k17205 in k17202 in a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_17222(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_17222,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17226,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_a_i_list1(&a,1,((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17238,a[2]=t2,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2034: chicken.compiler.support#varnode */
t5=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k17224 in k17220 in k17205 in k17202 in a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_17226(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_17226,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2029: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17236 in k17220 in k17205 in k17202 in a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_17238(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_17238,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2033: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[164];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17248 in k17261 in k17205 in k17202 in a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_17250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_17250,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2031: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[97];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17252 in k17261 in k17205 in k17202 in a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_17254(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17254,c,av);}
/* core.scm:2032: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k17261 in k17205 in k17202 in a16973 in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_17263(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,2)))){
C_save_and_reclaim((void *)f_17263,c,av);}
a=C_alloc(23);
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
t3=C_a_i_list4(&a,4,t1,C_SCHEME_FALSE,t2,C_fix(0));
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17250,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17254,a[2]=((C_word*)t0)[4],a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2032: chicken.compiler.support#varnode */
t6=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* walk-inline-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17269(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_17269,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(5);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17275,a[2]=t5,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2037: walk-arguments */
t7=((C_word*)((C_word*)t0)[2])[1];
f_17285(t7,t1,t4,t6);}

/* a17274 in walk-inline-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_17275,c,av);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17283,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2040: chicken.compiler.support#make-node */
t4=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k17281 in a17274 in walk-inline-call in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17283,c,av);}
/* core.scm:2040: k */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17285(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_17285,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17291,a[2]=t3,a[3]=t5,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],tmp=(C_word)a,a+=7,tmp));
t7=((C_word*)t5)[1];
f_17291(t7,t1,t2,C_SCHEME_END_OF_LIST);}

/* loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17291(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_17291,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17305,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2044: scheme#reverse */
t5=*((C_word*)lf[259]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17311,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2045: atomic? */
t5=((C_word*)((C_word*)t0)[6])[1];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(t2);
f_17387(3,av2);}}}

/* k17303 in loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17305,c,av);}
/* core.scm:2044: wk */
t2=((C_word*)t0)[2];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k17309 in loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17311,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=C_u_i_cdr(((C_word*)t0)[2]);
t3=C_u_i_car(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,t3,((C_word*)t0)[3]);
/* core.scm:2046: loop */
t5=((C_word*)((C_word*)t0)[4])[1];
f_17291(t5,((C_word*)t0)[5],t2,t4);}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17325,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2048: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[512];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k17323 in k17309 in loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17325(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_17325,c,av);}
a=C_alloc(7);
t2=C_u_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17332,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2049: walk */
t4=((C_word*)((C_word*)t0)[6])[1];
f_16577(t4,((C_word*)t0)[7],t2,t3);}

/* a17331 in k17323 in k17309 in loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17332(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_17332,c,av);}
a=C_alloc(8);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17339,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=t2,tmp=(C_word)a,a+=8,tmp);
/* core.scm:2051: node-for-var? */
f_16553(t3,t2,((C_word*)t0)[5]);}

/* k17337 in a17331 in k17323 in k17309 in loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17339(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_17339,c,av);}
a=C_alloc(14);
if(C_truep(t1)){
t2=C_i_cdr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17354,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2052: chicken.compiler.support#varnode */
t4=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=C_a_i_list1(&a,1,((C_word*)t0)[6]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17369,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[5],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t4=C_i_cdr(((C_word*)t0)[2]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17381,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2056: chicken.compiler.support#varnode */
t6=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k17352 in k17337 in a17331 in k17323 in k17309 in loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_17354,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* core.scm:2052: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_17291(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k17367 in k17337 in a17331 in k17323 in k17309 in loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_17369,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2053: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k17379 in k17337 in a17331 in k17323 in k17309 in loop in walk-arguments in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17381(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_17381,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,((C_word*)t0)[2]);
/* core.scm:2055: loop */
t3=((C_word*)((C_word*)t0)[3])[1];
f_17291(t3,((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* atomic? in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_17387,c,av);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17391,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2059: chicken.compiler.support#node-class */
t4=*((C_word*)lf[98]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k17389 in atomic? in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17391,c,av);}
a=C_alloc(4);
t2=C_u_i_memq(t1,lf[532]);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep((C_truep(C_eqp(t1,lf[301]))?C_SCHEME_TRUE:(C_truep(C_eqp(t1,lf[119]))?C_SCHEME_TRUE:(C_truep(C_eqp(t1,lf[280]))?C_SCHEME_TRUE:(C_truep(C_eqp(t1,lf[123]))?C_SCHEME_TRUE:(C_truep(C_eqp(t1,lf[283]))?C_SCHEME_TRUE:C_SCHEME_FALSE))))))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17408,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2064: chicken.compiler.support#node-subexpressions */
t4=*((C_word*)lf[527]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}}

/* k17406 in k17389 in atomic? in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17408(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17408,c,av);}
/* core.scm:2064: every */
f_6675(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1],t1);}

/* chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_17413,c,av);}
a=C_alloc(4);
t3=C_i_fixnum_max(lf[62],C_fix(1));
t4=C_fixnum_times(t3,C_fix(3));
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17420,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2074: scheme#make-vector */
t6=*((C_word*)lf[93]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=t4;
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}

/* k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(47,c,7)))){
C_save_and_reclaim((void *)f_17420,c,av);}
a=C_alloc(47);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17422,tmp=(C_word)a,a+=2,tmp));
t17=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17429,a[2]=t1,a[3]=t15,a[4]=t7,a[5]=t5,a[6]=t13,a[7]=t3,tmp=(C_word)a,a+=8,tmp));
t18=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18100,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t19=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18137,a[2]=t1,tmp=(C_word)a,a+=3,tmp));
t20=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18150,a[2]=t1,tmp=(C_word)a,a+=3,tmp));
t21=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18163,a[2]=t1,a[3]=t9,a[4]=t11,tmp=(C_word)a,a+=5,tmp));
t22=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18473,a[2]=t1,tmp=(C_word)a,a+=3,tmp));
t23=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18494,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t5,a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2285: chicken.compiler.support#debugging */
t24=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t24;
av2[1]=t23;
av2[2]=lf[601];
av2[3]=lf[603];
((C_proc)(void*)(*((C_word*)t24+1)))(4,av2);}}

/* grow in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_17422(C_word *a,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;{}
t2=C_s_a_i_plus(&a,2,lf[61],t1);
t3=C_mutate(&lf[61] /* (set! chicken.compiler.core#current-program-size ...) */,t2);
return(t3);}

/* walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17429(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,2)))){
C_save_and_reclaim_args((void *)trf_17429,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(14);
t7=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_17433,a[2]=t1,a[3]=t4,a[4]=t3,a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[3],a[7]=t2,a[8]=((C_word*)t0)[4],a[9]=t5,a[10]=t6,a[11]=((C_word*)t0)[5],a[12]=((C_word*)t0)[6],a[13]=((C_word*)t0)[7],tmp=(C_word)a,a+=14,tmp);
/* core.scm:2082: chicken.compiler.support#node-subexpressions */
t8=*((C_word*)lf[527]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}

/* k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17433(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_17433,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_17436,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
/* core.scm:2083: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_17436,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_17439,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* core.scm:2084: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17439(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(46,c,2)))){
C_save_and_reclaim((void *)f_17439,c,av);}
a=C_alloc(46);
t2=C_s_a_i_plus(&a,2,lf[61],C_fix(1));
t3=C_mutate(&lf[61] /* (set! chicken.compiler.core#current-program-size ...) */,t2);
t4=C_eqp(t1,lf[118]);
t5=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_17451,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],tmp=(C_word)a,a+=17,tmp);
if(C_truep(t4)){
t6=t5;
f_17451(t6,t4);}
else{
t6=C_eqp(t1,lf[475]);
if(C_truep(t6)){
t7=t5;
f_17451(t7,t6);}
else{
t7=C_eqp(t1,lf[163]);
t8=t5;
f_17451(t8,(C_truep(t7)?t7:C_eqp(t1,lf[561])));}}}

/* k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17451(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(47,0,6)))){
C_save_and_reclaim_args((void *)trf_17451,2,t0,t1);}
a=C_alloc(47);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[523]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[3],lf[534]));
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[4]);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17466,a[2]=t4,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2092: ref */
t6=((C_word*)((C_word*)t0)[8])[1];
f_18473(t6,t5,t4,((C_word*)t0)[9]);}
else{
t4=C_eqp(((C_word*)t0)[3],lf[164]);
t5=(C_truep(t4)?t4:C_eqp(((C_word*)t0)[3],lf[538]));
if(C_truep(t5)){
t6=C_s_a_i_plus(&a,2,lf[61],C_fix(1));
t7=C_mutate(&lf[61] /* (set! chicken.compiler.core#current-program-size ...) */,t6);
/* core.scm:2102: walkeach */
t8=((C_word*)((C_word*)t0)[10])[1];
f_18100(t8,((C_word*)t0)[2],((C_word*)t0)[11],((C_word*)t0)[6],((C_word*)t0)[5],((C_word*)t0)[12],((C_word*)t0)[13]);}
else{
t6=C_eqp(((C_word*)t0)[3],lf[520]);
if(C_truep(t6)){
t7=C_s_a_i_plus(&a,2,lf[61],C_fix(1));
t8=C_mutate(&lf[61] /* (set! chicken.compiler.core#current-program-size ...) */,t7);
t9=C_i_car(((C_word*)t0)[11]);
t10=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_17527,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[13],a[9]=((C_word*)t0)[14],tmp=(C_word)a,a+=10,tmp);
t11=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17626,a[2]=((C_word*)t0)[13],a[3]=((C_word*)t0)[9],a[4]=t10,a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[11],a[7]=t9,tmp=(C_word)a,a+=8,tmp);
/* core.scm:2107: chicken.compiler.support#node-class */
t12=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t12;
av2[1]=t11;
av2[2]=t9;
((C_proc)(void*)(*((C_word*)t12+1)))(3,av2);}}
else{
t7=C_eqp(((C_word*)t0)[3],lf[174]);
t8=(C_truep(t7)?t7:C_eqp(((C_word*)t0)[3],lf[173]));
if(C_truep(t8)){
t9=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_17638,a[2]=((C_word*)t0)[14],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[13],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[15],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[2],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* core.scm:2125: scheme#append */
t10=*((C_word*)lf[5]+1);{
C_word av2[4];
av2[0]=t10;
av2[1]=t9;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t10+1)))(4,av2);}}
else{
t9=C_eqp(((C_word*)t0)[3],lf[487]);
if(C_truep(t9)){
t10=(
/* core.scm:2137: grow */
  f_17422(C_a_i(&a,29),C_fix(1))
);
t11=C_i_car(((C_word*)t0)[4]);
t12=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17701,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[14],a[5]=((C_word*)t0)[12],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2138: ##sys#decompose-lambda-list */
t13=*((C_word*)lf[191]+1);{
C_word av2[4];
av2[0]=t13;
av2[1]=((C_word*)t0)[2];
av2[2]=t11;
av2[3]=t12;
((C_proc)(void*)(*((C_word*)t13+1)))(4,av2);}}
else{
t10=C_eqp(((C_word*)t0)[3],lf[97]);
t11=(C_truep(t10)?t10:C_eqp(((C_word*)t0)[3],lf[545]));
if(C_truep(t11)){
t12=(
/* core.scm:2150: grow */
  f_17422(C_a_i(&a,29),C_fix(1))
);
t13=C_i_caddr(((C_word*)t0)[4]);
t14=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17770,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[13],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[11],a[7]=((C_word*)t0)[14],a[8]=((C_word*)t0)[12],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2151: ##sys#decompose-lambda-list */
t15=*((C_word*)lf[191]+1);{
C_word av2[4];
av2[0]=t15;
av2[1]=((C_word*)t0)[2];
av2[2]=t13;
av2[3]=t14;
((C_proc)(void*)(*((C_word*)t15+1)))(4,av2);}}
else{
t12=C_eqp(((C_word*)t0)[3],lf[284]);
t13=(C_truep(t12)?t12:C_eqp(((C_word*)t0)[3],lf[177]));
if(C_truep(t13)){
t14=C_i_car(((C_word*)t0)[4]);
t15=C_i_car(((C_word*)t0)[11]);
t16=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_17908,a[2]=((C_word*)t0)[14],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[13],a[9]=((C_word*)t0)[7],a[10]=t14,a[11]=((C_word*)t0)[15],a[12]=t15,tmp=(C_word)a,a+=13,tmp);
t17=(C_truep(*((C_word*)lf[69]+1))?C_i_not(*((C_word*)lf[53]+1)):C_SCHEME_FALSE);
if(C_truep(t17)){
t18=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17965,a[2]=t16,a[3]=t14,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2181: chicken.compiler.support#variable-mark */
t19=*((C_word*)lf[185]+1);{
C_word av2[4];
av2[0]=t19;
av2[1]=t18;
av2[2]=t14;
av2[3]=lf[560];
((C_proc)(void*)(*((C_word*)t19+1)))(4,av2);}}
else{
t18=t16;{
C_word av2[2];
av2[0]=t18;
av2[1]=C_SCHEME_UNDEFINED;
f_17908(2,av2);}}}
else{
t14=C_eqp(((C_word*)t0)[3],lf[474]);
t15=(C_truep(t14)?t14:C_eqp(((C_word*)t0)[3],lf[300]));
if(C_truep(t15)){
t16=C_i_car(((C_word*)t0)[4]);
t17=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_18005,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[12],a[8]=((C_word*)t0)[13],tmp=(C_word)a,a+=9,tmp);
t18=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_18011,a[2]=t17,a[3]=t16,a[4]=((C_word*)t0)[13],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[12],tmp=(C_word)a,a+=11,tmp);
if(C_truep(*((C_word*)lf[69]+1))){
if(C_truep(((C_word*)t0)[13])){
if(C_truep(C_i_symbolp(t16))){
/* core.scm:2200: chicken.compiler.support#get-real-name */
t19=*((C_word*)lf[127]+1);{
C_word av2[3];
av2[0]=t19;
av2[1]=t18;
av2[2]=t16;
((C_proc)(void*)(*((C_word*)t19+1)))(3,av2);}}
else{
t19=t18;{
C_word av2[2];
av2[0]=t19;
av2[1]=C_SCHEME_FALSE;
f_18011(2,av2);}}}
else{
t19=t18;{
C_word av2[2];
av2[0]=t19;
av2[1]=C_SCHEME_FALSE;
f_18011(2,av2);}}}
else{
t19=t18;{
C_word av2[2];
av2[0]=t19;
av2[1]=C_SCHEME_FALSE;
f_18011(2,av2);}}}
else{
/* core.scm:2204: walkeach */
t16=((C_word*)((C_word*)t0)[10])[1];
f_18100(t16,((C_word*)t0)[2],((C_word*)t0)[11],((C_word*)t0)[6],((C_word*)t0)[5],((C_word*)t0)[12],((C_word*)t0)[13]);}}}}}}}}}}

/* k17464 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17466(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(34,c,5)))){
C_save_and_reclaim((void *)f_17466,c,av);}
a=C_alloc(34);
if(C_truep(C_i_memq(((C_word*)t0)[2],((C_word*)t0)[3]))){
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=C_s_a_i_plus(&a,2,lf[61],C_fix(1));
t3=C_mutate(&lf[61] /* (set! chicken.compiler.core#current-program-size ...) */,t2);
if(C_truep(C_i_memq(((C_word*)t0)[2],((C_word*)t0)[5]))){
/* core.scm:2096: chicken.compiler.support#db-put! */
t4=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[2];
av2[4]=lf[536];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17497,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2097: chicken.compiler.support#db-get */
t5=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[2];
av2[4]=lf[95];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}}}

/* k17495 in k17464 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17497(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17497,c,av);}
if(C_truep(C_i_not(t1))){
/* core.scm:2098: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[95];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k17525 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17527(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,6)))){
C_save_and_reclaim((void *)f_17527,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17530,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2121: walk */
t3=((C_word*)((C_word*)t0)[9])[1];
f_17429(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k17528 in k17525 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17530(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_17530,c,av);}
/* core.scm:2122: walkeach */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18100(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17548(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_17548,c,av);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17551,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17561,a[2]=t2,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
if(C_truep(*((C_word*)lf[69]+1))){
if(C_truep(t1)){
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17618,a[2]=t3,a[3]=((C_word*)t0)[7],a[4]=t1,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2112: chicken.compiler.support#db-get */
t5=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[95];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=t3;
f_17561(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_17561(t4,C_SCHEME_FALSE);}}

/* k17549 in k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17551(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_17551,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
/* core.scm:2120: chicken.compiler.support#collect! */
t3=*((C_word*)lf[539]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[99];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k17559 in k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17561(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,5)))){
C_save_and_reclaim_args((void *)trf_17561,2,t0,t1);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17568,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2119: chicken.compiler.support#real-name */
t3=*((C_word*)lf[541]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]);
/* core.scm:2120: chicken.compiler.support#collect! */
t3=*((C_word*)lf[539]+1);{
C_word av2[6];
av2[0]=t3;
av2[1]=((C_word*)t0)[6];
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[99];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}}

/* k17566 in k17559 in k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_17568(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17568,c,av);}
/* core.scm:2117: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[540];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17594 in k17608 in k17612 in k17616 in k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_17596(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17596,c,av);}
t2=((C_word*)t0)[2];
f_17561(t2,C_i_not(t1));}

/* k17604 in k17608 in k17612 in k17616 in k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_17606(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17606,c,av);}
/* core.scm:2115: chicken.compiler.support#llist-match? */
t2=*((C_word*)lf[542]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_caddr(t1);
av2[3]=C_u_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k17608 in k17612 in k17616 in k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_17610(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_17610,c,av);}
a=C_alloc(7);
t2=C_eqp(lf[97],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17596,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17606,a[2]=t3,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2115: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_17561(t3,C_SCHEME_FALSE);}}

/* k17612 in k17616 in k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_17614(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_17614,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17610,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2114: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];
f_17561(t2,C_SCHEME_FALSE);}}

/* k17616 in k17546 in k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17618(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_17618,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17614,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2113: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[543];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];
f_17561(t2,C_SCHEME_FALSE);}}

/* k17620 in k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17622(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_17622,c,av);}
a=C_alloc(8);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17548,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2109: chicken.compiler.support#db-get */
t4=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=t2;
av2[4]=lf[96];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k17624 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17626(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_17626,c,av);}
a=C_alloc(7);
t2=C_eqp(lf[523],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17622,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2108: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_17527(2,av2);}}}

/* k17636 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17638(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,4)))){
C_save_and_reclaim((void *)f_17638,c,av);}
a=C_alloc(14);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_17643,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=t3,a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[9],tmp=(C_word)a,a+=12,tmp));
t5=((C_word*)t3)[1];
f_17643(t5,((C_word*)t0)[10],((C_word*)t0)[5],((C_word*)t0)[11]);}

/* loop in k17636 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17643(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,5)))){
C_save_and_reclaim_args((void *)trf_17643,4,t0,t1,t2,t3);}
a=C_alloc(15);
if(C_truep(C_i_nullp(t2))){
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17661,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2128: scheme#append */
t6=*((C_word*)lf[5]+1);{
C_word av2[4];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}
else{
t4=C_i_car(t2);
t5=C_i_car(t3);
t6=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_17670,a[2]=((C_word*)t0)[8],a[3]=t1,a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[2],a[7]=t5,a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[5],a[12]=((C_word*)t0)[10],a[13]=t4,a[14]=((C_word*)t0)[4],tmp=(C_word)a,a+=15,tmp);
/* core.scm:2131: chicken.compiler.support#db-put! */
t7=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[11];
av2[3]=t4;
av2[4]=lf[544];
av2[5]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t7+1)))(6,av2);}}}

/* k17659 in loop in k17636 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_17661,c,av);}
/* core.scm:2128: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17429(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1,((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k17668 in loop in k17636 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17670(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,5)))){
C_save_and_reclaim((void *)f_17670,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_17673,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* core.scm:2132: assign */
t3=((C_word*)((C_word*)t0)[12])[1];
f_18163(t3,t2,((C_word*)t0)[13],((C_word*)t0)[7],((C_word*)t0)[14],((C_word*)t0)[11]);}

/* k17671 in k17668 in loop in k17636 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17673(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,6)))){
C_save_and_reclaim((void *)f_17673,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17676,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2133: walk */
t3=((C_word*)((C_word*)t0)[6])[1];
f_17429(t3,t2,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11]);}

/* k17674 in k17671 in k17668 in loop in k17636 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_17676(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17676,c,av);}
/* core.scm:2134: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17643(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]),C_u_i_cdr(((C_word*)t0)[5]));}

/* a17700 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17701(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_17701,c,av);}
a=C_alloc(16);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_17703,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t6=C_i_check_list_2(t2,lf[100]);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17713,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17728,a[2]=t9,a[3]=t5,tmp=(C_word)a,a+=4,tmp));
t11=((C_word*)t9)[1];
f_17728(t11,t7,t2);}

/* g3886 in a17700 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17703(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_17703,3,t0,t1,t2);}
/* core.scm:2142: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t3;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[543];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k17711 in a17700 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17713(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_17713,c,av);}
a=C_alloc(11);
t2=lf[78];
t3=lf[78] /* chicken.compiler.core#toplevel-scope */ =C_SCHEME_FALSE;;
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17717,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t5=C_i_car(((C_word*)t0)[3]);
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_17726,a[2]=((C_word*)t0)[4],a[3]=t4,a[4]=t5,a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2146: scheme#append */
t7=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k17715 in k17711 in a17700 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17717(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_17717,c,av);}
t2=C_mutate(&lf[78] /* (set! chicken.compiler.core#toplevel-scope ...) */,((C_word*)t0)[2]);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k17724 in k17711 in a17700 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17726(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_17726,c,av);}
/* core.scm:2146: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17429(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1,C_SCHEME_FALSE);}

/* for-each-loop3885 in a17700 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17728(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_17728,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17738,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2141: g3886 */
t4=((C_word*)t0)[3];
f_17703(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k17736 in for-each-loop3885 in a17700 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17738,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_17728(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,5)))){
C_save_and_reclaim((void *)f_17770,c,av);}
a=C_alloc(20);
t5=C_i_car(((C_word*)t0)[2]);
t6=lf[61];
t7=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_17777,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=t6,a[6]=t1,a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=t5,a[12]=((C_word*)t0)[2],a[13]=t4,tmp=(C_word)a,a+=14,tmp);
if(C_truep(((C_word*)t0)[4])){
t8=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17887,a[2]=t7,a[3]=((C_word*)t0)[3],a[4]=t5,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2157: chicken.compiler.support#collect! */
t9=*((C_word*)lf[539]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[552];
av2[5]=t5;
((C_proc)(void*)(*((C_word*)t9+1)))(6,av2);}}
else{
t8=t7;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_UNDEFINED;
f_17777(2,av2);}}}

/* k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17777(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,3)))){
C_save_and_reclaim((void *)f_17777,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17778,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=C_i_check_list_2(((C_word*)t0)[4],lf[100]);
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_17791,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[2],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17863,a[2]=t6,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_17863(t8,t4,((C_word*)t0)[4]);}

/* g3917 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17778(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,5)))){
C_save_and_reclaim_args((void *)trf_17778,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17782,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2161: chicken.compiler.support#db-put! */
t4=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[544];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k17780 in g3917 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17782(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17782,c,av);}
/* core.scm:2162: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[543];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,5)))){
C_save_and_reclaim((void *)f_17791,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_17794,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[12])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17854,a[2]=((C_word*)t0)[12],a[3]=t2,a[4]=((C_word*)t0)[11],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2165: chicken.compiler.support#db-put! */
t4=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[11];
av2[3]=((C_word*)t0)[12];
av2[4]=lf[549];
av2[5]=lf[550];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_17794(2,av2);}}}

/* k17792 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17794(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_17794,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17797,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17848,a[2]=t2,a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[9],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2167: chicken.compiler.support#simple-lambda-node? */
t4=*((C_word*)lf[547]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k17795 in k17792 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_17797(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_17797,c,av);}
a=C_alloc(12);
t2=lf[78];
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_17800,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
if(C_truep(lf[79])){
t4=t3;
f_17800(t4,C_SCHEME_UNDEFINED);}
else{
t4=C_mutate(&lf[79] /* (set! chicken.compiler.core#toplevel-lambda-id ...) */,((C_word*)t0)[9]);
t5=t3;
f_17800(t5,t4);}}

/* k17798 in k17795 in k17792 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_17800(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_17800,2,t0,t1);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17803,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(C_i_cadr(((C_word*)t0)[11]))){
t3=C_eqp(lf[79],((C_word*)t0)[10]);
if(C_truep(C_i_not(t3))){
t4=lf[78] /* chicken.compiler.core#toplevel-scope */ =C_SCHEME_FALSE;;
t5=t2;
f_17803(t5,t4);}
else{
t4=t2;
f_17803(t4,C_SCHEME_UNDEFINED);}}
else{
t3=t2;
f_17803(t3,C_SCHEME_UNDEFINED);}}

/* k17801 in k17798 in k17795 in k17792 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_17803(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,3)))){
C_save_and_reclaim_args((void *)trf_17803,2,t0,t1);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_17806,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=C_i_car(((C_word*)t0)[6]);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_17830,a[2]=((C_word*)t0)[7],a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2172: scheme#append */
t5=*((C_word*)lf[5]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k17804 in k17801 in k17798 in k17795 in k17792 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_17806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_17806,c,av);}
a=C_alloc(4);
t2=C_mutate(&lf[78] /* (set! chicken.compiler.core#toplevel-scope ...) */,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_17822,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2175: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k17820 in k17804 in k17801 in k17798 in k17795 in k17792 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_17822(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_17822,c,av);}
a=C_alloc(29);
t2=C_i_cdddr(t1);
t3=C_s_a_i_minus(&a,2,lf[61],((C_word*)t0)[2]);
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_set_car(t2,t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k17828 in k17801 in k17798 in k17795 in k17792 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_17830(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_17830,c,av);}
/* core.scm:2172: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17429(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],t1,((C_word*)t0)[7]);}

/* k17846 in k17792 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_17848(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17848,c,av);}
if(C_truep(t1)){
/* core.scm:2167: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[546];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_17797(2,av2);}}}

/* k17852 in k17789 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,5)))){
C_save_and_reclaim((void *)f_17854,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],C_fix(0));
/* core.scm:2166: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[2];
av2[4]=lf[548];
av2[5]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* for-each-loop3916 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_17863(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_17863,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17873,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2159: g3917 */
t4=((C_word*)t0)[3];
f_17778(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k17871 in for-each-loop3916 in k17775 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_17873,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_17863(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k17885 in a17769 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17887(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17887,c,av);}
/* core.scm:2158: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[551];
av2[5]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k17906 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17908(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,5)))){
C_save_and_reclaim((void *)f_17908,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_17911,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
/* core.scm:2186: chicken.compiler.support#collect! */
t3=*((C_word*)lf[539]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[10];
av2[4]=lf[555];
av2[5]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k17909 in k17906 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(47,c,5)))){
C_save_and_reclaim((void *)f_17911,c,av);}
a=C_alloc(47);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_17914,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],tmp=(C_word)a,a+=13,tmp);
if(C_truep(C_i_memq(((C_word*)t0)[10],((C_word*)t0)[6]))){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_17914(2,av2);}}
else{
t3=C_s_a_i_plus(&a,2,lf[61],C_fix(1));
t4=C_mutate(&lf[61] /* (set! chicken.compiler.core#current-program-size ...) */,t3);
if(C_truep(C_i_memq(((C_word*)t0)[10],((C_word*)t0)[5]))){
/* core.scm:2190: chicken.compiler.support#db-put! */
t5=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[10];
av2[4]=lf[536];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_17959,a[2]=t2,a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[10],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2191: chicken.compiler.support#db-get */
t6=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[10];
av2[4]=lf[95];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}}}

/* k17912 in k17909 in k17906 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17914(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_17914,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17917,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:2193: assign */
t3=((C_word*)((C_word*)t0)[11])[1];
f_18163(t3,t2,((C_word*)t0)[10],((C_word*)t0)[12],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k17915 in k17912 in k17909 in k17906 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,5)))){
C_save_and_reclaim((void *)f_17917,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_17920,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(lf[78])){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_17920(2,av2);}}
else{
/* core.scm:2194: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[10];
av2[4]=lf[554];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}}

/* k17918 in k17915 in k17912 in k17909 in k17906 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_17920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,5)))){
C_save_and_reclaim((void *)f_17920,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_17923,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2195: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
av2[3]=((C_word*)t0)[10];
av2[4]=lf[553];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k17921 in k17918 in k17915 in k17912 in k17909 in k17906 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_17923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_17923,c,av);}
/* core.scm:2196: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_17429(t2,((C_word*)t0)[3],C_u_i_car(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k17957 in k17909 in k17906 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17959(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_17959,c,av);}
if(C_truep(C_i_not(t1))){
/* core.scm:2192: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[95];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_17914(2,av2);}}}

/* k17963 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_17965(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_17965,c,av);}
t2=C_eqp(t1,lf[556]);
if(C_truep(t2)){
/* core.scm:2183: chicken.base#warning */
t3=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[557];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=C_eqp(t1,lf[558]);
if(C_truep(t3)){
/* core.scm:2185: chicken.base#warning */
t4=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[559];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_17908(2,av2);}}}}

/* k18003 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18005(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_18005,c,av);}
/* core.scm:2202: walkeach */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18100(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k18009 in k17449 in k17437 in k17434 in k17431 in walk in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18011(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_18011,c,av);}
if(C_truep(t1)){
/* core.scm:2201: chicken.compiler.support#set-real-name! */
t2=*((C_word*)lf[117]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
/* core.scm:2202: walkeach */
t2=((C_word*)((C_word*)t0)[5])[1];
f_18100(t2,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[4]);}}

/* walkeach in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18100(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,3)))){
C_save_and_reclaim_args((void *)trf_18100,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(13);
t7=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18102,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t4,a[5]=t5,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
t8=C_i_check_list_2(t2,lf[100]);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18114,a[2]=t10,a[3]=t7,tmp=(C_word)a,a+=4,tmp));
t12=((C_word*)t10)[1];
f_18114(t12,t1,t2);}

/* g3986 in walkeach in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18102(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,6)))){
C_save_and_reclaim_args((void *)trf_18102,3,t0,t1,t2);}
/* core.scm:2207: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_17429(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);}

/* for-each-loop3985 in walkeach in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18114(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_18114,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18124,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2207: g3986 */
t4=((C_word*)t0)[3];
f_18102(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k18122 in for-each-loop3985 in walkeach in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18124(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18124,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_18114(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* mark-rest-cdr in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18137(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_18137,5,t0,t1,t2,t3,t4);}
a=C_alloc(9);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18141,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t6=C_a_i_cons(&a,2,t3,t4);
/* core.scm:2210: chicken.compiler.support#db-put! */
t7=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t7;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[548];
av2[5]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(6,av2);}}

/* k18139 in mark-rest-cdr in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18141(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_18141,c,av);}
/* core.scm:2211: chicken.compiler.support#collect! */
t2=*((C_word*)lf[539]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[562];
av2[5]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* mark-rest-null? in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18150(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,5)))){
C_save_and_reclaim_args((void *)trf_18150,5,t0,t1,t2,t3,t4);}
a=C_alloc(9);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18154,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t3,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t6=C_a_i_cons(&a,2,t3,t4);
/* core.scm:2214: chicken.compiler.support#db-put! */
t7=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t7;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[563];
av2[5]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(6,av2);}}

/* k18152 in mark-rest-null? in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18154(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_18154,c,av);}
/* core.scm:2215: chicken.compiler.support#collect! */
t2=*((C_word*)lf[539]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[562];
av2[5]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18163(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_18163,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(10);
t6=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18471,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=t3,a[7]=((C_word*)t0)[4],a[8]=t4,a[9]=t5,tmp=(C_word)a,a+=10,tmp);
/* core.scm:2219: chicken.compiler.support#node-class */
t7=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t7;
av2[1]=t6;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}

/* k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18170(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_18170,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18444,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2223: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18176(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_18176,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18417,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2227: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18188(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_18188,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18395,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2233: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* k18210 in k18214 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_18212(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_18212,c,av);}
a=C_alloc(29);
t2=C_i_cadr(t1);
t3=C_s_a_i_plus(&a,2,t2,C_fix(1));
/* core.scm:2236: mark-rest-cdr */
t4=((C_word*)((C_word*)t0)[2])[1];
f_18137(t4,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t3);}

/* k18214 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18216(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18216,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18212,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2235: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k18233 in k18237 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_18235(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18235,c,av);}
/* core.scm:2241: mark-rest-null? */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18150(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],C_i_cadr(t1));}

/* k18237 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_18239(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18239,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18235,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2240: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_18242(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_18242,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_18245,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18380,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18387,a[2]=t3,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2249: chicken.base#alist-ref */
t5=*((C_word*)lf[566]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[572];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t3=t2;
f_18245(t3,C_SCHEME_FALSE);}}

/* k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_18245(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_18245,2,t0,t1);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18249,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2227: g4042 */
t3=t2;
f_18249(t3,((C_word*)t0)[4],t1);}
else{
t2=C_i_memq(((C_word*)t0)[3],((C_word*)t0)[5]);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18273,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],tmp=(C_word)a,a+=7,tmp);
if(C_truep(t2)){
t4=t3;
f_18273(t4,t2);}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18367,a[2]=t3,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2257: chicken.compiler.support#variable-mark */
t5=*((C_word*)lf[185]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[351];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}}

/* g4042 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_18249(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_18249,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18253,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2252: chicken.base#alist-ref */
t4=*((C_word*)lf[566]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[568];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k18251 in g4042 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_18253(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_18253,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18256,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2253: chicken.base#alist-ref */
t3=*((C_word*)lf[566]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[567];
av2[3]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18254 in k18251 in g4042 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_18256(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_18256,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,t1,C_fix(1));
/* core.scm:2254: mark-rest-cdr */
t3=((C_word*)((C_word*)t0)[2])[1];
f_18137(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t2);}

/* k18271 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_18273(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,5)))){
C_save_and_reclaim_args((void *)trf_18273,2,t0,t1);}
a=C_alloc(12);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18276,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2259: chicken.compiler.support#db-get-all */
t3=*((C_word*)lf[569]+1);{
C_word av2[6];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[543];
av2[5]=lf[96];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18318,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t3=*((C_word*)lf[49]+1);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18354,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(*((C_word*)lf[49]+1))){
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=*((C_word*)lf[49]+1);
f_18354(2,av2);}}
else{
/* core.scm:2268: chicken.compiler.support#variable-mark */
t5=*((C_word*)lf[185]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[415];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}}

/* k18274 in k18271 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_18276(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_18276,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18279,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2260: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[544];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k18277 in k18274 in k18271 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_18279(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_18279,c,av);}
if(C_truep(C_i_assq(lf[543],((C_word*)t0)[2]))){
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
if(C_truep(C_i_assq(lf[96],((C_word*)t0)[2]))){
/* core.scm:2263: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[543];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=C_i_not(t1);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[6],t1));
if(C_truep(t3)){
/* core.scm:2265: chicken.compiler.support#db-put! */
t4=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[96];
av2[5]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
/* core.scm:2266: chicken.compiler.support#db-put! */
t4=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[543];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}}}}

/* k18316 in k18271 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_18318(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,5)))){
C_save_and_reclaim_args((void *)trf_18318,2,t0,t1);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18321,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2270: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[544];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* core.scm:2276: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[543];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}}

/* k18319 in k18316 in k18271 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_18321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_18321,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18327,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2271: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[570];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k18325 in k18319 in k18316 in k18271 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_18327(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_18327,c,av);}
if(C_truep(t1)){
/* core.scm:2272: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[543];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
t2=C_i_not(((C_word*)t0)[5]);
t3=(C_truep(t2)?t2:C_eqp(((C_word*)t0)[6],((C_word*)t0)[5]));
if(C_truep(t3)){
/* core.scm:2274: chicken.compiler.support#db-put! */
t4=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[570];
av2[5]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
/* core.scm:2275: chicken.compiler.support#db-put! */
t4=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[543];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}}}

/* k18352 in k18271 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_18354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_18354,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18361,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2269: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[543];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];
f_18318(t2,C_SCHEME_FALSE);}}

/* k18359 in k18352 in k18271 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_18361(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18361,c,av);}
t2=((C_word*)t0)[2];
f_18318(t2,C_i_not(t1));}

/* k18365 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_18367(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_18367,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=((C_word*)t0)[2];
f_18273(t2,t1);}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18377,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2258: chicken.compiler.support#variable-visible? */
t3=*((C_word*)lf[571]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[25]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* k18375 in k18365 in k18243 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_18377(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18377,c,av);}
t2=((C_word*)t0)[2];
f_18273(t2,C_i_not(t1));}

/* k18378 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_18380(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18380,c,av);}
t2=((C_word*)t0)[2];
f_18245(t2,(C_truep(t1)?((C_word*)t0)[3]:C_SCHEME_FALSE));}

/* k18385 in k18240 in k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_18387(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18387,c,av);}
/* core.scm:2249: chicken.compiler.support#db-get */
t2=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=lf[563];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k18389 in k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_18391,c,av);}
a=C_alloc(9);
t2=C_eqp(lf[565],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18239,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2239: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_18242,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2244: chicken.compiler.support#match-node */
t4=*((C_word*)lf[573]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
av2[3]=lf[574];
av2[4]=lf[575];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k18393 in k18186 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18395(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_18395,c,av);}
a=C_alloc(10);
t2=C_eqp(lf[534],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18216,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2234: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18391,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2238: chicken.compiler.support#node-class */
t4=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k18407 in k18411 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18409,c,av);}
t2=((C_word*)t0)[2];
f_18188(t2,C_eqp(((C_word*)t0)[3],C_i_car(t1)));}

/* k18411 in k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18413(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_18413,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[523],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18409,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2230: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_18188(t3,C_SCHEME_FALSE);}}

/* k18415 in k18174 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,5)))){
C_save_and_reclaim((void *)f_18417,c,av);}
a=C_alloc(15);
t2=C_eqp(lf[475],t1);
if(C_truep(t2)){
/* core.scm:2228: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[564];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18188,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18413,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2229: chicken.compiler.support#node-class */
t5=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k18418 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18420(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18420,c,av);}
if(C_truep(t1)){
/* core.scm:2225: mark-rest-null? */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18150(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_i_car(t1),C_u_i_cdr(t1));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_18176(2,av2);}}}

/* k18438 in k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18440(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18440,c,av);}
/* core.scm:2224: chicken.compiler.support#db-get */
t2=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_i_car(t1);
av2[4]=lf[563];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k18442 in k18168 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_18444,c,av);}
a=C_alloc(19);
t2=C_eqp(lf[523],t1);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18176,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t2)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18420,a[2]=((C_word*)t0)[7],a[3]=t3,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18440,a[2]=t4,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2224: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_18176(2,av2);}}}

/* k18445 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18447(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18447,c,av);}
if(C_truep(t1)){
/* core.scm:2221: mark-rest-cdr */
t2=((C_word*)((C_word*)t0)[2])[1];
f_18137(t2,((C_word*)t0)[3],((C_word*)t0)[4],C_i_car(t1),C_u_i_cdr(t1));}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_18170(2,av2);}}}

/* k18465 in k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18467(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18467,c,av);}
/* core.scm:2220: chicken.compiler.support#db-get */
t2=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_i_car(t1);
av2[4]=lf[548];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k18469 in assign in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_18471,c,av);}
a=C_alloc(19);
t2=C_eqp(lf[523],t1);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_18170,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
if(C_truep(t2)){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18447,a[2]=((C_word*)t0)[5],a[3]=t3,a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18467,a[2]=t4,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2220: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_18170(2,av2);}}}

/* ref in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18473(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,5)))){
C_save_and_reclaim_args((void *)trf_18473,4,t0,t1,t2,t3);}
/* core.scm:2279: chicken.compiler.support#collect! */
t4=*((C_word*)lf[539]+1);{
C_word av2[6];
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[576];
av2[5]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}

/* k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,6)))){
C_save_and_reclaim((void *)f_18494,c,av);}
a=C_alloc(4);
t2=lf[61] /* chicken.compiler.core#current-program-size */ =C_fix(0);;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18498,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2287: walk */
t4=((C_word*)((C_word*)t0)[4])[1];
f_17429(t4,t3,((C_word*)t0)[5],C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST,C_SCHEME_END_OF_LIST,C_SCHEME_FALSE);}

/* k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18498(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_18498,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18501,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2290: chicken.compiler.support#debugging */
t3=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[601];
av2[3]=lf[602];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_18501,c,av);}
a=C_alloc(7);
t2=lf[62] /* chicken.compiler.core#current-analysis-database-size */ =C_fix(0);;
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18505,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18511,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2292: chicken.internal#hash-table-for-each */
t5=*((C_word*)lf[101]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t3;
av2[2]=t4;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k18503 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18505(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18505,c,av);}
if(C_truep(lf[60])){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_mutate(&lf[60] /* (set! chicken.compiler.core#original-program-size ...) */,lf[61]);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18511(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(67,c,2)))){
C_save_and_reclaim((void *)f_18511,c,av);}
a=C_alloc(67);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_FALSE;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_FALSE;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_FALSE;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_END_OF_LIST;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_FALSE;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_SCHEME_END_OF_LIST;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_SCHEME_FALSE;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_SCHEME_FALSE;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=C_SCHEME_FALSE;
t23=(*a=C_VECTOR_TYPE|1,a[1]=t22,tmp=(C_word)a,a+=2,tmp);
t24=C_fix(0);
t25=(*a=C_VECTOR_TYPE|1,a[1]=t24,tmp=(C_word)a,a+=2,tmp);
t26=C_SCHEME_FALSE;
t27=(*a=C_VECTOR_TYPE|1,a[1]=t26,tmp=(C_word)a,a+=2,tmp);
t28=C_fix(0);
t29=(*a=C_VECTOR_TYPE|1,a[1]=t28,tmp=(C_word)a,a+=2,tmp);
t30=C_fixnum_plus(lf[62],C_fix(1));
t31=lf[62] /* chicken.compiler.core#current-analysis-database-size */ =t30;;
t32=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_18517,a[2]=t5,a[3]=t13,a[4]=t25,a[5]=t15,a[6]=t11,a[7]=t17,a[8]=t29,a[9]=t19,a[10]=t21,a[11]=t23,a[12]=t7,a[13]=t9,a[14]=t27,tmp=(C_word)a,a+=15,tmp);
t33=C_i_check_list_2(t3,lf[100]);
t34=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19442,a[2]=t32,tmp=(C_word)a,a+=3,tmp);
t35=(
  f_19442(t34,t3)
);
t36=C_i_not(((C_word*)t5)[1]);
t37=(C_truep(t36)?((C_word*)t7)[1]:C_SCHEME_FALSE);
t38=C_set_block_item(t7,0,t37);
t39=C_i_not(((C_word*)t5)[1]);
t40=(C_truep(t39)?((C_word*)t9)[1]:C_SCHEME_FALSE);
t41=C_set_block_item(t9,0,t40);
t42=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_18660,a[2]=t3,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=t7,a[6]=t23,a[7]=t19,a[8]=t13,a[9]=t21,a[10]=t27,a[11]=t25,a[12]=t29,a[13]=t2,a[14]=t9,a[15]=t15,tmp=(C_word)a,a+=16,tmp);
if(C_truep(*((C_word*)lf[69]+1))){
t43=(C_truep(((C_word*)t7)[1])?C_a_i_list1(&a,1,((C_word*)t7)[1]):C_SCHEME_FALSE);
t44=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19373,a[2]=t2,a[3]=t42,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t43)){
t45=t44;
f_19373(t45,t43);}
else{
if(C_truep(((C_word*)t23)[1])){
t45=((C_word*)t11)[1];
t46=t44;
f_19373(t46,t45);}
else{
t45=t44;
f_19373(t45,C_SCHEME_FALSE);}}}
else{
t43=t42;{
C_word *av2=av;
av2[0]=t43;
av2[1]=C_SCHEME_UNDEFINED;
f_18660(2,av2);}}}

/* g4096 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_18517(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_stack_overflow_check;{}
t2=C_i_car(t1);
t3=C_eqp(t2,lf[543]);
if(C_truep(t3)){
t4=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_TRUE);
return(t4);}
else{
t4=C_eqp(t2,lf[576]);
if(C_truep(t4)){
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,C_u_i_cdr(t1));
t6=C_i_length(((C_word*)((C_word*)t0)[3])[1]);
t7=C_set_block_item(((C_word*)t0)[4],0,t6);
return(t7);}
else{
t5=C_eqp(t2,lf[536]);
if(C_truep(t5)){
t6=C_set_block_item(((C_word*)t0)[5],0,C_SCHEME_TRUE);
return(t6);}
else{
t6=C_eqp(t2,lf[555]);
if(C_truep(t6)){
t7=C_mutate(((C_word *)((C_word*)t0)[6])+1,C_u_i_cdr(t1));
return(t7);}
else{
t7=C_eqp(t2,lf[99]);
if(C_truep(t7)){
t8=C_mutate(((C_word *)((C_word*)t0)[7])+1,C_u_i_cdr(t1));
t9=C_i_length(((C_word*)((C_word*)t0)[7])[1]);
t10=C_set_block_item(((C_word*)t0)[8],0,t9);
return(t10);}
else{
t8=C_eqp(t2,lf[553]);
if(C_truep(t8)){
t9=C_set_block_item(((C_word*)t0)[9],0,C_SCHEME_TRUE);
return(t9);}
else{
t9=C_eqp(t2,lf[554]);
if(C_truep(t9)){
t10=C_SCHEME_UNDEFINED;
return(t10);}
else{
t10=C_eqp(t2,lf[564]);
if(C_truep(t10)){
t11=C_set_block_item(((C_word*)t0)[10],0,C_SCHEME_TRUE);
return(t11);}
else{
t11=C_eqp(t2,lf[95]);
if(C_truep(t11)){
t12=C_set_block_item(((C_word*)t0)[11],0,C_SCHEME_TRUE);
return(t12);}
else{
t12=C_eqp(t2,lf[96]);
if(C_truep(t12)){
t13=C_mutate(((C_word *)((C_word*)t0)[12])+1,C_u_i_cdr(t1));
return(t13);}
else{
t13=C_eqp(t2,lf[570]);
if(C_truep(t13)){
t14=C_mutate(((C_word *)((C_word*)t0)[13])+1,C_u_i_cdr(t1));
return(t14);}
else{
t14=C_eqp(t2,lf[548]);
if(C_truep(t14)){
t15=C_mutate(((C_word *)((C_word*)t0)[14])+1,C_u_i_cdr(t1));
return(t15);}
else{
return(C_SCHEME_UNDEFINED);}}}}}}}}}}}}}

/* k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18660(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_18660,c,av);}
a=C_alloc(24);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_18663,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],tmp=(C_word)a,a+=16,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19299,a[2]=t2,a[3]=((C_word*)t0)[13],tmp=(C_word)a,a+=4,tmp);
if(C_truep(*((C_word*)lf[69]+1))){
if(C_truep(((C_word*)((C_word*)t0)[6])[1])){
if(C_truep(C_i_nullp(((C_word*)((C_word*)t0)[8])[1]))){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19367,a[2]=t3,a[3]=((C_word*)t0)[13],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2352: chicken.compiler.support#variable-mark */
t5=*((C_word*)lf[185]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[13];
av2[3]=lf[399];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t4=t3;
f_19299(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_19299(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_19299(t4,C_SCHEME_FALSE);}}

/* k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18663(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,2)))){
C_save_and_reclaim((void *)f_18663,c,av);}
a=C_alloc(21);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_18666,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],tmp=(C_word)a,a+=15,tmp);
if(C_truep(((C_word*)((C_word*)t0)[7])[1])){
t3=((C_word*)((C_word*)t0)[15])[1];
if(C_truep(t3)){
t4=C_i_cdr(((C_word*)t0)[2]);
t5=C_a_i_cons(&a,2,lf[597],C_SCHEME_TRUE);
t6=C_a_i_cons(&a,2,t5,t4);
t7=t2;
f_18666(t7,C_i_set_cdr(((C_word*)t0)[2],t6));}
else{
t4=t2;
f_18666(t4,C_SCHEME_UNDEFINED);}}
else{
t3=t2;
f_18666(t3,C_SCHEME_UNDEFINED);}}

/* k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18666(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,0,3)))){
C_save_and_reclaim_args((void *)trf_18666,2,t0,t1);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_18669,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19121,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2369: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
if(C_truep(((C_word*)((C_word*)t0)[14])[1])){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19193,a[2]=((C_word*)t0)[14],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[13],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* core.scm:2381: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)((C_word*)t0)[14])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19258,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[13],a[4]=((C_word*)t0)[2],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2393: chicken.compiler.support#variable-mark */
t4=*((C_word*)lf[185]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[13];
av2[3]=lf[407];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}}

/* k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_18669(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,2)))){
C_save_and_reclaim((void *)f_18669,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_18672,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19089,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[11],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19118,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2409: chicken.compiler.support#node-class */
t5=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=t3;
f_19089(t4,C_SCHEME_FALSE);}}

/* k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_18672(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,0,2)))){
C_save_and_reclaim_args((void *)trf_18672,2,t0,t1);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_18675,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
t3=C_SCHEME_FALSE;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_19086,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[4],a[5]=t4,a[6]=((C_word*)t0)[13],a[7]=t2,a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2423: chicken.compiler.support#node-class */
t6=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_18675(2,av2);}}}

/* k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_18675(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_18675,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18678,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[7])[1]))){
if(C_truep(C_i_nullp(((C_word*)((C_word*)t0)[8])[1]))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18911,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[10],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18960,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2453: chicken.compiler.support#node-class */
t5=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=t3;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_18911(2,av2);}}}
else{
t3=t2;
f_18678(t3,C_SCHEME_UNDEFINED);}}
else{
t3=t2;
f_18678(t3,C_SCHEME_UNDEFINED);}}

/* k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_18678(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,0,2)))){
C_save_and_reclaim_args((void *)trf_18678,2,t0,t1);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18681,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(C_truep(((C_word*)((C_word*)t0)[5])[1])?C_i_not(((C_word*)((C_word*)t0)[6])[1]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18890,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2475: chicken.compiler.support#node-class */
t5=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=t2;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_18681(2,av2);}}}

/* k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_18681(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_18681,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18687,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[5])[1])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18812,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2489: chicken.compiler.support#node-class */
t4=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;
f_18687(t3,C_SCHEME_FALSE);}}

/* k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_18687(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_18687,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18690,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2490: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_18690(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18690,c,av);}
a=C_alloc(6);
t2=C_i_cadr(t1);
if(C_truep(C_i_not(t2))){
t3=C_i_caddr(t1);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18801,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2493: chicken.compiler.support#node-subexpressions */
t5=*((C_word*)lf[527]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18706 in k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_fcall f_18708(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_18708,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18711,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2497: chicken.compiler.support#node-subexpressions */
t3=*((C_word*)lf[527]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18709 in k18706 in k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_18711(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_18711,c,av);}
a=C_alloc(11);
t2=C_i_length(t1);
t3=C_eqp(C_fix(2),t2);
if(C_truep(t3)){
t4=C_i_car(t1);
t5=C_i_cadr(t1);
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18729,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18774,a[2]=((C_word*)t0)[5],a[3]=t6,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2501: chicken.compiler.support#node-class */
t8=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=t4;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k18727 in k18709 in k18706 in k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in ... */
static void C_fcall f_18729(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_18729,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18742,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2504: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k18740 in k18727 in k18709 in k18706 in k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in ... */
static void C_ccall f_18742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_18742,c,av);}
a=C_alloc(6);
t2=C_i_car(t1);
t3=C_i_cdr(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,lf[577],t2);
t5=C_a_i_cons(&a,2,t4,t3);
t6=C_i_set_cdr(((C_word*)t0)[2],t5);
/* core.scm:2506: chicken.compiler.support#db-put! */
t7=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t7;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
av2[4]=lf[578];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t7+1)))(6,av2);}}

/* k18764 in k18768 in k18772 in k18709 in k18706 in k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in ... */
static void C_ccall f_18766(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18766,c,av);}
t2=((C_word*)t0)[2];
f_18729(t2,C_eqp(((C_word*)t0)[3],C_i_car(t1)));}

/* k18768 in k18772 in k18709 in k18706 in k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in ... */
static void C_ccall f_18770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_18770,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[523],t1);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[2]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18766,a[2]=((C_word*)t0)[3],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2503: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];
f_18729(t3,C_SCHEME_FALSE);}}

/* k18772 in k18709 in k18706 in k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in ... */
static void C_ccall f_18774(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_18774,c,av);}
a=C_alloc(5);
t2=C_eqp(lf[523],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18770,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2502: chicken.compiler.support#node-class */
t4=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[3];
f_18729(t3,C_SCHEME_FALSE);}}

/* k18793 in k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_18795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18795,c,av);}
t2=((C_word*)t0)[2];
f_18708(t2,C_eqp(lf[520],t1));}

/* k18799 in k18688 in k18685 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_18801(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_18801,c,av);}
a=C_alloc(10);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_18708,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[5]))){
if(C_truep(C_i_nullp(C_u_i_cdr(((C_word*)t0)[5])))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18795,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2496: chicken.compiler.support#node-class */
t5=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=t3;
f_18708(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_18708(t4,C_SCHEME_FALSE);}}

/* k18810 in k18679 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_18812(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18812,c,av);}
t2=((C_word*)t0)[2];
f_18687(t2,C_eqp(lf[97],t1));}

/* k18828 in k18884 in k18888 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_18830(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,5)))){
C_save_and_reclaim_args((void *)trf_18830,2,t0,t1);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[577],((C_word*)t0)[3]);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_i_set_cdr(((C_word*)t0)[2],t4);
/* core.scm:2485: chicken.compiler.support#db-put! */
t6=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t6;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[578];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t6+1)))(6,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_18681(2,av2);}}}

/* k18840 in k18884 in k18888 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_18842(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_18842,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=((C_word*)t0)[2];
f_18830(t2,t1);}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18872,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2480: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[553];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k18862 in k18866 in k18870 in k18840 in k18884 in k18888 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_18864(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18864,c,av);}
t2=((C_word*)t0)[2];
f_18830(t2,C_i_not(t1));}

/* k18866 in k18870 in k18840 in k18884 in k18888 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_18868(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_18868,c,av);}
a=C_alloc(3);
t2=C_i_not(t1);
if(C_truep(t2)){
t3=((C_word*)t0)[2];
f_18830(t3,t2);}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18864,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2483: chicken.compiler.support#db-get */
t4=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[95];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k18870 in k18840 in k18884 in k18888 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_18872(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_18872,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18868,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2481: chicken.compiler.support#variable-visible? */
t3=*((C_word*)lf[571]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=*((C_word*)lf[25]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];
f_18830(t2,C_SCHEME_FALSE);}}

/* k18880 in k18884 in k18888 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_18882(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_18882,c,av);}
if(C_truep(C_i_not(t1))){
/* core.scm:2479: chicken.compiler.support#db-get */
t2=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[96];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_18842(2,av2);}}}

/* k18884 in k18888 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_18886(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_18886,c,av);}
a=C_alloc(16);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18830,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_not(((C_word*)((C_word*)t0)[5])[1]))){
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18842,a[2]=t3,a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_18882,a[2]=t4,a[3]=((C_word*)t0)[4],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2478: chicken.compiler.support#db-get */
t6=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
av2[3]=t2;
av2[4]=lf[543];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t4=t3;
f_18830(t4,C_SCHEME_FALSE);}}

/* k18888 in k18676 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_18890(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_18890,c,av);}
a=C_alloc(6);
t2=C_eqp(lf[523],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18886,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2476: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)((C_word*)t0)[6])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
f_18681(2,av2);}}}

/* k18909 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_18911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_18911,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
if(C_truep(t1)){
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[579],C_SCHEME_TRUE);
t4=C_a_i_cons(&a,2,t3,t2);
t5=((C_word*)t0)[3];
f_18678(t5,C_i_set_cdr(((C_word*)t0)[2],t4));}
else{
t2=((C_word*)t0)[3];
f_18678(t2,C_SCHEME_UNDEFINED);}}
else{
t2=((C_word*)((C_word*)t0)[4])[1];
if(C_truep(t2)){
if(C_truep(t2)){
t3=C_i_cdr(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,lf[579],C_SCHEME_TRUE);
t5=C_a_i_cons(&a,2,t4,t3);
t6=((C_word*)t0)[3];
f_18678(t6,C_i_set_cdr(((C_word*)t0)[2],t5));}
else{
t3=((C_word*)t0)[3];
f_18678(t3,C_SCHEME_UNDEFINED);}}
else{
t3=((C_word*)((C_word*)t0)[5])[1];
if(C_truep(t3)){
t4=C_i_cdr(((C_word*)t0)[2]);
t5=C_a_i_cons(&a,2,lf[579],C_SCHEME_TRUE);
t6=C_a_i_cons(&a,2,t5,t4);
t7=((C_word*)t0)[3];
f_18678(t7,C_i_set_cdr(((C_word*)t0)[2],t6));}
else{
t4=((C_word*)t0)[3];
f_18678(t4,C_SCHEME_UNDEFINED);}}}}

/* k18933 in k18943 in k18947 in k18958 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_18935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_18935,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
f_18911(2,av2);}}
else{
/* core.scm:2457: chicken.compiler.support#intrinsic? */
t2=*((C_word*)lf[580]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}

/* k18943 in k18947 in k18958 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_18945(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_18945,c,av);}
a=C_alloc(4);
t2=C_i_not(t1);
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_18911(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18935,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2456: chicken.compiler.support#variable-mark */
t4=*((C_word*)lf[185]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[581];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k18947 in k18958 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_18949(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_18949,c,av);}
a=C_alloc(4);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18945,a[2]=((C_word*)t0)[2],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2455: chicken.compiler.support#db-get */
t4=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
av2[4]=lf[95];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k18954 in k18958 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_18956(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_18956,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_i_not(t1);
f_18911(2,av2);}}

/* k18958 in k18673 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_18960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_18960,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[523],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_18949,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2454: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_18956,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2458: chicken.compiler.support#expression-has-side-effects? */
t4=*((C_word*)lf[582]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)((C_word*)t0)[4])[1];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_18969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_18969,c,av);}
a=C_alloc(6);
if(C_truep(C_i_cadr(t1))){
t2=C_i_caddr(t1);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18984,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2427: ##sys#decompose-lambda-list */
t4=*((C_word*)lf[191]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[5];
av2[2]=t2;
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_18675(2,av2);}}}

/* a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_18984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_18984,c,av);}
a=C_alloc(18);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_18988,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t4,a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t4)){
t6=t5;{
C_word *av2=av;
av2[0]=t6;
av2[1]=C_SCHEME_UNDEFINED;
f_18988(2,av2);}}
else{
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19022,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t7=C_i_check_list_2(t2,lf[100]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19055,a[2]=t9,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t11=((C_word*)t9)[1];
f_19055(t11,t5,t2);}}

/* k18986 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_18988(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_18988,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18994,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[6])[1])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19021,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2440: chicken.base#rassoc */
t4=*((C_word*)lf[586]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=lf[77];
av2[4]=*((C_word*)lf[587]+1);
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=t2;
f_18994(t3,C_SCHEME_FALSE);}}

/* k18992 in k18986 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_18994(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,5)))){
C_save_and_reclaim_args((void *)trf_18994,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
/* core.scm:2441: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=C_i_car(((C_word*)t0)[4]);
av2[4]=lf[583];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}
else{
if(C_truep(((C_word*)t0)[5])){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19007,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2443: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=C_i_car(((C_word*)t0)[4]);
av2[4]=lf[585];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}}

/* k19005 in k18992 in k18986 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_19007(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_19007,c,av);}
/* core.scm:2444: chicken.compiler.support#db-put! */
t2=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[584];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* k19019 in k18986 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_19021(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19021,c,av);}
t2=((C_word*)t0)[2];
f_18994(t2,C_i_not(t1));}

/* g4223 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_19022(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,4)))){
C_save_and_reclaim_args((void *)trf_19022,3,t0,t1,t2);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19029,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19047,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2433: chicken.compiler.support#db-get */
t5=*((C_word*)lf[537]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
av2[4]=lf[576];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k19027 in g4223 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_19029(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,5)))){
C_save_and_reclaim_args((void *)trf_19029,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19032,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2435: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word av2[6];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[398];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k19030 in k19027 in g4223 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_19032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19032,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_TRUE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k19041 in k19045 in g4223 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_19043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19043,c,av);}
t2=((C_word*)t0)[2];
f_19029(t2,C_i_not(t1));}

/* k19045 in g4223 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_19047(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_19047,c,av);}
a=C_alloc(3);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19043,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2434: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[553];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[2];
f_19029(t2,C_SCHEME_FALSE);}}

/* for-each-loop4222 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_19055(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_19055,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19065,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2431: g4223 */
t4=((C_word*)t0)[3];
f_19022(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k19063 in for-each-loop4222 in a18983 in k18967 in k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_19065(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19065,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_19055(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k19084 in k18670 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19086(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_19086,c,av);}
a=C_alloc(6);
t2=C_eqp(lf[97],t1);
t3=(C_truep(t2)?C_i_nequalp(((C_word*)((C_word*)t0)[2])[1],((C_word*)((C_word*)t0)[3])[1]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_18969,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2425: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)((C_word*)t0)[8])[1];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_SCHEME_UNDEFINED;
t5=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
f_18675(2,av2);}}}

/* k19087 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19089(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_19089,2,t0,t1);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19111,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2410: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[5])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[3];
f_18672(t2,C_SCHEME_UNDEFINED);}}

/* k19096 in k19109 in k19087 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_19098(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_19098,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
if(C_truep(t1)){
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[588],C_SCHEME_TRUE);
t4=C_a_i_cons(&a,2,t3,t2);
t5=((C_word*)t0)[3];
f_18672(t5,C_i_set_cdr(((C_word*)t0)[2],t4));}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];
f_18672(t3,t2);}}
else{
if(C_truep(C_i_nequalp(C_fix(1),((C_word*)((C_word*)t0)[4])[1]))){
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[588],C_SCHEME_TRUE);
t4=C_a_i_cons(&a,2,t3,t2);
t5=((C_word*)t0)[3];
f_18672(t5,C_i_set_cdr(((C_word*)t0)[2],t4));}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];
f_18672(t3,t2);}}}

/* k19109 in k19087 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19111(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_19111,c,av);}
a=C_alloc(5);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19098,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2411: chicken.compiler.support#collapsable-literal? */
t4=*((C_word*)lf[357]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19116 in k18667 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19118(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19118,c,av);}
t2=((C_word*)t0)[2];
f_19089(t2,C_eqp(lf[118],t1));}

/* k19119 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19121(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_19121,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19127,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19187,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2370: chicken.compiler.support#node-class */
t4=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)((C_word*)t0)[7])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19125 in k19119 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19127(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_19127,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_nequalp(C_fix(1),((C_word*)((C_word*)t0)[2])[1]);
t3=(C_truep(t2)?C_i_nequalp(C_fix(1),((C_word*)((C_word*)t0)[3])[1]):C_SCHEME_FALSE);
if(C_truep(t3)){
t4=C_i_cdr(((C_word*)t0)[4]);
t5=C_a_i_cons(&a,2,lf[589],C_SCHEME_TRUE);
t6=C_a_i_cons(&a,2,t5,t4);
t7=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_i_set_cdr(((C_word*)t0)[4],t6);
f_18669(2,av2);}}
else{
t4=C_i_cdr(((C_word*)t0)[4]);
t5=C_a_i_cons(&a,2,lf[590],C_SCHEME_TRUE);
t6=C_a_i_cons(&a,2,t5,t4);
t7=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t7;
av2[1]=C_i_set_cdr(((C_word*)t0)[4],t6);
f_18669(2,av2);}}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_18669(2,av2);}}}

/* a19158 in k19185 in k19119 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19159(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19159,c,av);}
/* core.scm:2373: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[95];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k19165 in k19185 in k19119 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19167(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19167,c,av);}
/* core.scm:2372: every */
f_6675(((C_word*)t0)[2],((C_word*)t0)[3],t1);}

/* a19168 in k19185 in k19119 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19169(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19169,c,av);}
/* core.scm:2374: chicken.compiler.support#scan-free-variables */
t2=*((C_word*)lf[591]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=*((C_word*)lf[25]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a19174 in k19185 in k19119 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19175(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +0,c,1)))){
C_save_and_reclaim((void*)f_19175,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+0);
t2=C_build_rest(&a,c,2,av);
C_word t3;
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_list_ref(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k19185 in k19119 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19187(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_19187,c,av);}
a=C_alloc(12);
t2=C_eqp(lf[97],t1);
if(C_truep(t2)){
t3=C_i_cadr(((C_word*)t0)[2]);
t4=C_i_not(t3);
if(C_truep(t4)){
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
f_19127(2,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19159,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19167,a[2]=((C_word*)t0)[3],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19169,a[2]=((C_word*)t0)[5],tmp=(C_word)a,a+=3,tmp);
t8=(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19175,tmp=(C_word)a,a+=2,tmp);
/* core.scm:2374: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t6;
av2[2]=t7;
av2[3]=t8;
C_call_with_values(4,av2);}}}
else{
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_19127(2,av2);}}}

/* k19191 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19193(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_19193,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19255,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2382: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* a19203 in k19253 in k19191 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19204(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19204,c,av);}
/* core.scm:2383: chicken.compiler.support#scan-free-variables */
t2=*((C_word*)lf[591]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=*((C_word*)lf[25]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a19209 in k19253 in k19191 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19210(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_19210,c,av);}
a=C_alloc(12);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19214,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19248,a[2]=t3,a[3]=((C_word*)t0)[3],a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2385: chicken.compiler.support#db-get */
t6=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[95];
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k19212 in a19209 in k19253 in k19191 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_19214(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_19214,2,t0,t1);}
a=C_alloc(10);
t2=C_i_cadr(((C_word*)t0)[2]);
t3=C_i_not(t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19223,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t3)){
if(C_truep(t3)){
t5=C_i_cdr(((C_word*)t0)[3]);
t6=C_a_i_cons(&a,2,lf[590],C_SCHEME_TRUE);
t7=C_a_i_cons(&a,2,t6,t5);
t8=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t8;
av2[1]=C_i_set_cdr(((C_word*)t0)[3],t7);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t5=C_SCHEME_UNDEFINED;
t6=((C_word*)t0)[4];{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19231,a[2]=((C_word*)t0)[5],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2389: every */
f_6675(t4,t5,((C_word*)t0)[6]);}}

/* k19221 in k19212 in a19209 in k19253 in k19191 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_19223(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_19223,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[590],C_SCHEME_TRUE);
t4=C_a_i_cons(&a,2,t3,t2);
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_i_set_cdr(((C_word*)t0)[2],t4);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* a19230 in k19212 in a19209 in k19253 in k19191 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_19231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19231,c,av);}
/* core.scm:2390: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=lf[95];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k19246 in a19209 in k19253 in k19191 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_19248(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_19248,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
if(C_truep(C_i_pairp(((C_word*)t0)[2]))){
t2=C_i_cdr(((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,lf[592],C_SCHEME_TRUE);
t4=C_a_i_cons(&a,2,t3,t2);
t5=((C_word*)t0)[4];
f_19214(t5,C_i_set_cdr(((C_word*)t0)[3],t4));}
else{
t2=((C_word*)t0)[4];
f_19214(t2,C_SCHEME_UNDEFINED);}}
else{
t2=((C_word*)t0)[4];
f_19214(t2,C_SCHEME_UNDEFINED);}}

/* k19253 in k19191 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19255(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_19255,c,av);}
a=C_alloc(9);
t2=C_eqp(lf[97],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19204,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19210,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2383: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[7];
av2[2]=t3;
av2[3]=t4;
C_call_with_values(4,av2);}}
else{
t3=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_18669(2,av2);}}}

/* k19256 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19258(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_19258,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19262,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2368: g4196 */
t3=t2;
f_19262(t3,((C_word*)t0)[5],t1);}
else{
t2=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_18669(2,av2);}}}

/* g4196 in k19256 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19262(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,2)))){
C_save_and_reclaim_args((void *)trf_19262,3,t0,t1,t2);}
a=C_alloc(7);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19269,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=t2,tmp=(C_word)a,a+=7,tmp);
/* core.scm:2395: chicken.compiler.support#node? */
t4=*((C_word*)lf[596]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19267 in g4196 in k19256 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19269(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_19269,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
if(C_truep(((C_word*)((C_word*)t0)[2])[1])){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19275,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2397: chicken.compiler.support#debugging */
t3=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[594];
av2[3]=lf[595];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19281,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2403: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}
else{
t2=C_SCHEME_UNDEFINED;
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19273 in k19267 in g4196 in k19256 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_19275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19275,c,av);}
/* core.scm:2401: chicken.compiler.support#mark-variable */
t2=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[407];
av2[4]=lf[405];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k19279 in k19267 in g4196 in k19256 in k18664 in k18661 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_19281(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_19281,c,av);}
a=C_alloc(12);
t2=C_i_cdr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[590],C_SCHEME_TRUE);
t4=C_a_i_cons(&a,2,t3,t2);
t5=C_i_set_cdr(((C_word*)t0)[2],t4);
t6=C_i_cdr(((C_word*)t0)[2]);
t7=C_a_i_cons(&a,2,lf[570],((C_word*)t0)[3]);
t8=C_a_i_cons(&a,2,t7,t6);
t9=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t9;
av2[1]=C_i_set_cdr(((C_word*)t0)[2],t8);
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}

/* k19297 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19299(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_19299,2,t0,t1);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19306,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2357: chicken.base#open-output-string */
t3=*((C_word*)lf[133]+1);{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_18663(2,av2);}}}

/* k19304 in k19297 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19306(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_19306,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19312,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2357: ##sys#print */
t4=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[599];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k19310 in k19304 in k19297 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_19312,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19315,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2357: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k19313 in k19310 in k19304 in k19297 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_19315,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19318,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2357: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[598];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k19316 in k19313 in k19310 in k19304 in k19297 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19318,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19321,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2357: chicken.base#get-output-string */
t3=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k19319 in k19316 in k19313 in k19310 in k19304 in k19297 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_19321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19321,c,av);}
/* core.scm:2356: ##sys#notice */
t2=*((C_word*)lf[366]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k19353 in k19357 in k19361 in k19365 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19355(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19355,c,av);}
t2=((C_word*)t0)[2];
f_19299(t2,C_i_not(t1));}

/* k19357 in k19361 in k19365 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19359(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_19359,c,av);}
a=C_alloc(3);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19355,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2355: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[185]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=lf[351];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];
f_19299(t2,C_SCHEME_FALSE);}}

/* k19361 in k19365 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19363(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_19363,c,av);}
a=C_alloc(4);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19359,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2354: chicken.compiler.support#variable-visible? */
t3=*((C_word*)lf[571]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[25]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];
f_19299(t2,C_SCHEME_FALSE);}}

/* k19365 in k18658 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19367(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_19367,c,av);}
a=C_alloc(4);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19363,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2353: chicken.compiler.support#variable-hidden? */
t3=*((C_word*)lf[600]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];
f_19299(t2,C_SCHEME_FALSE);}}

/* k19371 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19373(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_19373,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19377,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t3=C_i_check_list_2(t1,lf[100]);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19407,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_19407(t7,((C_word*)t0)[3],t1);}
else{
t2=((C_word*)t0)[3];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_18660(2,av2);}}}

/* g4135 in k19371 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19377(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_19377,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19399,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2343: chicken.compiler.support#node-class */
t4=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k19393 in k19397 in g4135 in k19371 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19395(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19395,c,av);}
/* core.scm:2344: chicken.compiler.support#set-real-name! */
t2=*((C_word*)lf[117]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_car(t1);
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k19397 in g4135 in k19371 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19399(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_19399,c,av);}
a=C_alloc(4);
t2=C_eqp(t1,lf[97]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19395,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2344: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* for-each-loop4134 in k19371 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19407(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_19407,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19417,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2341: g4135 */
t4=((C_word*)t0)[3];
f_19377(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k19415 in for-each-loop4134 in k19371 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19417,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_19407(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* for-each-loop4095 in a18510 in k18499 in k18496 in k18492 in k17418 in chicken.compiler.core#analyze-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_19442(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_pairp(t1))){
t2=(
/* core.scm:2311: g4096 */
  f_18517(((C_word*)t0)[2],C_slot(t1,C_fix(0)))
);
t4=C_slot(t1,C_fix(1));
t1=t4;
goto loop;}
else{
t2=C_SCHEME_UNDEFINED;
return(t2);}}

/* chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19469(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(58,c,5)))){
C_save_and_reclaim((void *)f_19469,c,av);}
a=C_alloc(58);
t4=C_fix(0);
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_SCHEME_END_OF_LIST;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_END_OF_LIST;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_SCHEME_UNDEFINED;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_SCHEME_UNDEFINED;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_SCHEME_UNDEFINED;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=C_SCHEME_UNDEFINED;
t23=(*a=C_VECTOR_TYPE|1,a[1]=t22,tmp=(C_word)a,a+=2,tmp);
t24=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19472,a[2]=t3,tmp=(C_word)a,a+=3,tmp));
t25=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19478,a[2]=t7,a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t26=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19488,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t27=C_set_block_item(t17,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19497,a[2]=t9,a[3]=t17,a[4]=t13,a[5]=t15,a[6]=t11,a[7]=t3,tmp=(C_word)a,a+=8,tmp));
t28=C_set_block_item(t19,0,(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20028,a[2]=t11,a[3]=t23,a[4]=t19,a[5]=t21,a[6]=t3,tmp=(C_word)a,a+=7,tmp));
t29=C_set_block_item(t21,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21214,a[2]=t19,tmp=(C_word)a,a+=3,tmp));
t30=C_set_block_item(t23,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21265,tmp=(C_word)a,a+=2,tmp));
t31=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21306,a[2]=t1,a[3]=t5,a[4]=t19,a[5]=t2,a[6]=t7,a[7]=t17,tmp=(C_word)a,a+=8,tmp);
/* core.scm:2806: chicken.compiler.support#debugging */
t32=*((C_word*)lf[593]+1);{
C_word *av2=av;
av2[0]=t32;
av2[1]=t31;
av2[2]=lf[601];
av2[3]=lf[644];
((C_proc)(void*)(*((C_word*)t32+1)))(4,av2);}}

/* test in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19472(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_19472,4,t0,t1,t2,t3);}
/* core.scm:2528: chicken.compiler.support#db-get */
t4=*((C_word*)lf[537]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=t2;
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* register-customizable! in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19478(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_19478,4,t0,t1,t2,t3);}
a=C_alloc(9);
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19483,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2531: lset-adjoin/eq? */
f_7134(t4,((C_word*)((C_word*)t0)[2])[1],C_a_i_list(&a,1,t2));}

/* k19481 in register-customizable! in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19483(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_19483,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
/* core.scm:2532: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[605];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* register-direct-call! in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19488(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(35,0,3)))){
C_save_and_reclaim_args((void *)trf_19488,3,t0,t1,t2);}
a=C_alloc(35);
t3=((C_word*)((C_word*)t0)[2])[1];
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t3,C_fix(1)));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19495,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2536: lset-adjoin/eq? */
f_7134(t5,lf[68],C_a_i_list(&a,1,t2));}

/* k19493 in register-direct-call! in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19495(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19495,c,av);}
t2=C_mutate(&lf[68] /* (set! chicken.compiler.core#direct-call-ids ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19497(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_19497,5,t0,t1,t2,t3,t4);}
a=C_alloc(12);
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_19501,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,a[5]=((C_word*)t0)[3],a[6]=t3,a[7]=t2,a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[6],a[11]=((C_word*)t0)[7],tmp=(C_word)a,a+=12,tmp);
/* core.scm:2542: chicken.compiler.support#node-subexpressions */
t6=*((C_word*)lf[527]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_19501,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_19504,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* core.scm:2543: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19504(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_19504,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_19507,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* core.scm:2544: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19507(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_19507,c,av);}
a=C_alloc(15);
t2=C_eqp(t1,lf[523]);
t3=(C_truep(t2)?t2:C_eqp(t1,lf[534]));
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[2]);
t5=C_i_memq(t4,((C_word*)((C_word*)t0)[3])[1]);
t6=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t6;
av2[1]=(C_truep(t5)?C_a_i_list1(&a,1,t4):C_SCHEME_END_OF_LIST);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t4=C_eqp(t1,lf[118]);
t5=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_19537,a[2]=((C_word*)t0)[4],a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[3],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
if(C_truep(t4)){
t6=t5;
f_19537(t6,t4);}
else{
t6=C_eqp(t1,lf[475]);
if(C_truep(t6)){
t7=t5;
f_19537(t7,t6);}
else{
t7=C_eqp(t1,lf[163]);
if(C_truep(t7)){
t8=t5;
f_19537(t8,t7);}
else{
t8=C_eqp(t1,lf[561]);
t9=t5;
f_19537(t9,(C_truep(t8)?t8:C_eqp(t1,lf[474])));}}}}}

/* k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19537(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,5)))){
C_save_and_reclaim_args((void *)trf_19537,2,t0,t1);}
a=C_alloc(21);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[3],lf[174]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19546,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
t4=C_i_cdr(((C_word*)t0)[4]);
if(C_truep(C_i_pairp(t4))){
/* core.scm:2557: chicken.compiler.support#bomb */
t5=*((C_word*)lf[530]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t3;
av2[2]=lf[606];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=t3;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_19546(2,av2);}}}
else{
t3=C_eqp(((C_word*)t0)[3],lf[284]);
if(C_truep(t3)){
t4=C_i_car(((C_word*)t0)[4]);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19597,a[2]=t4,a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2564: gather */
t6=((C_word*)((C_word*)t0)[7])[1];
f_19497(t6,t5,C_i_car(((C_word*)t0)[5]),((C_word*)t0)[8],((C_word*)t0)[6]);}
else{
t4=C_eqp(((C_word*)t0)[3],lf[520]);
if(C_truep(t4)){
t5=C_i_car(((C_word*)t0)[5]);
t6=C_i_car(((C_word*)t0)[4]);
t7=C_i_cdr(((C_word*)t0)[4]);
t8=C_i_pairp(t7);
t9=(C_truep(t8)?C_i_cadr(((C_word*)t0)[4]):C_SCHEME_FALSE);
t10=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_19845,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=t9,a[8]=t6,a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=t5,tmp=(C_word)a,a+=14,tmp);
/* core.scm:2573: chicken.compiler.support#node-class */
t11=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t5=C_eqp(((C_word*)t0)[3],lf[97]);
t6=(C_truep(t5)?t5:C_eqp(((C_word*)t0)[3],lf[545]));
if(C_truep(t6)){
t7=C_i_caddr(((C_word*)t0)[4]);
t8=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_19873,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[14],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[5],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2614: ##sys#decompose-lambda-list */
t9=*((C_word*)lf[191]+1);{
C_word av2[4];
av2[0]=t9;
av2[1]=((C_word*)t0)[2];
av2[2]=t7;
av2[3]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(4,av2);}}
else{
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19931,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
t12=C_i_check_list_2(((C_word*)t0)[5],lf[10]);
t13=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19941,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19943,a[2]=t9,a[3]=t15,a[4]=t11,a[5]=t10,tmp=(C_word)a,a+=6,tmp));
t17=((C_word*)t15)[1];
f_19943(t17,t13,((C_word*)t0)[5]);}}}}}}

/* k19544 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19546(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_19546,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19549,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2558: gather */
t3=((C_word*)((C_word*)t0)[6])[1];
f_19497(t3,t2,C_i_car(((C_word*)t0)[4]),((C_word*)t0)[7],((C_word*)t0)[5]);}

/* k19547 in k19544 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19549(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,4)))){
C_save_and_reclaim((void *)f_19549,c,av);}
a=C_alloc(11);
t2=C_i_car(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19559,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19563,a[2]=t3,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t5=C_i_cadr(((C_word*)t0)[4]);
t6=C_a_i_cons(&a,2,t2,((C_word*)t0)[5]);
/* core.scm:2560: gather */
t7=((C_word*)((C_word*)t0)[6])[1];
f_19497(t7,t4,t5,((C_word*)t0)[7],t6);}

/* k19557 in k19547 in k19544 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19559,c,av);}
/* core.scm:2560: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k19561 in k19547 in k19544 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19563(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_19563,c,av);}
/* core.scm:2560: delete */
f_6795(((C_word*)t0)[2],((C_word*)t0)[3],t1,*((C_word*)lf[587]+1));}

/* k19595 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19597(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_19597,c,av);}
a=C_alloc(3);
t2=C_i_memq(((C_word*)t0)[2],((C_word*)((C_word*)t0)[3])[1]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_a_i_cons(&a,2,((C_word*)t0)[2],t1):t1);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k19629 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19631(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_19631,c,av);}
a=C_alloc(21);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19639,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t7=C_i_check_list_2(((C_word*)t0)[5],lf[10]);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19649,a[2]=((C_word*)t0)[6],tmp=(C_word)a,a+=3,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19651,a[2]=t4,a[3]=t10,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_19651(t12,t8,((C_word*)t0)[5]);}

/* g4410 in k19629 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19639(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_19639,3,t0,t1,t2);}
/* core.scm:2611: gather */
t3=((C_word*)((C_word*)t0)[2])[1];
f_19497(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k19647 in k19629 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19649,c,av);}
/* core.scm:2611: concatenate */
f_6767(((C_word*)t0)[2],t1);}

/* map-loop4404 in k19629 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19651(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_19651,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19676,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2611: g4410 */
t4=((C_word*)t0)[4];
f_19639(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19674 in map-loop4404 in k19629 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19676(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19676,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_19651(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19707(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,2)))){
C_save_and_reclaim((void *)f_19707,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_19713,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,tmp=(C_word)a,a+=12,tmp);
if(C_truep(t1)){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19793,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2590: chicken.compiler.support#node-class */
t4=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t2;
f_19713(t3,C_SCHEME_FALSE);}}

/* k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19713(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,3)))){
C_save_and_reclaim_args((void *)trf_19713,2,t0,t1);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_19716,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:2591: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],C_SCHEME_END_OF_LIST);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t2);
/* core.scm:2574: chicken.compiler.support#node-parameters-set! */
t4=*((C_word*)lf[607]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19716(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_19716,c,av);}
a=C_alloc(13);
t2=C_i_caddr(t1);
t3=C_i_car(t1);
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_19725,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t3,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=t2,a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],tmp=(C_word)a,a+=13,tmp);
/* core.scm:2594: test */
t5=((C_word*)((C_word*)t0)[10])[1];
f_19472(t5,t4,((C_word*)t0)[7],lf[576]);}

/* k19723 in k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_19725,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_19728,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
/* core.scm:2595: test */
t3=((C_word*)((C_word*)t0)[12])[1];
f_19472(t3,t2,((C_word*)t0)[8],lf[99]);}

/* k19726 in k19723 in k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_19728(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_19728,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_19731,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[12])){
if(C_truep(t1)){
t3=C_i_length(((C_word*)t0)[12]);
t4=C_i_length(t1);
t5=C_eqp(t3,t4);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_19775,a[2]=t2,a[3]=((C_word*)t0)[10],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2599: test */
t7=((C_word*)((C_word*)t0)[13])[1];
f_19472(t7,t6,((C_word*)t0)[8],lf[96]);}
else{
t6=t2;
f_19731(t6,C_SCHEME_FALSE);}}
else{
t3=t2;
f_19731(t3,C_SCHEME_FALSE);}}
else{
t3=t2;
f_19731(t3,C_SCHEME_FALSE);}}

/* k19729 in k19726 in k19723 in k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_19731(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,3)))){
C_save_and_reclaim_args((void *)trf_19731,2,t0,t1);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_19737,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
if(C_truep(((C_word*)t0)[2])){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19756,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2602: chicken.compiler.support#llist-match? */
t4=*((C_word*)lf[542]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[10];
av2[3]=C_i_cdr(((C_word*)t0)[11]);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t3=t2;
f_19737(t3,C_SCHEME_FALSE);}}

/* k19735 in k19729 in k19726 in k19723 in k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_19737(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_19737,2,t0,t1);}
a=C_alloc(10);
if(C_truep(t1)){
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],C_SCHEME_END_OF_LIST);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t2);
/* core.scm:2574: chicken.compiler.support#node-parameters-set! */
t4=*((C_word*)lf[607]+1);{
C_word av2[4];
av2[0]=t4;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_19740,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2605: register-direct-call! */
t3=((C_word*)((C_word*)t0)[10])[1];
f_19488(t3,t2,((C_word*)t0)[6]);}}

/* k19738 in k19735 in k19729 in k19726 in k19723 in k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_19740(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_19740,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19743,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(((C_word*)t0)[3])){
/* core.scm:2606: register-customizable! */
t3=((C_word*)((C_word*)t0)[8])[1];
f_19478(t3,t2,((C_word*)t0)[9],((C_word*)t0)[2]);}
else{
t3=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t4=C_a_i_cons(&a,2,((C_word*)t0)[4],t3);
t5=C_a_i_cons(&a,2,((C_word*)t0)[5],t4);
/* core.scm:2574: chicken.compiler.support#node-parameters-set! */
t6=*((C_word*)lf[607]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[6];
av2[2]=((C_word*)t0)[7];
av2[3]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* k19741 in k19738 in k19735 in k19729 in k19726 in k19723 in k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_19743(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,3)))){
C_save_and_reclaim((void *)f_19743,c,av);}
a=C_alloc(12);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=C_a_i_cons(&a,2,((C_word*)t0)[4],t2);
t4=C_a_i_cons(&a,2,((C_word*)t0)[5],t3);
/* core.scm:2574: chicken.compiler.support#node-parameters-set! */
t5=*((C_word*)lf[607]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[6];
av2[2]=((C_word*)t0)[7];
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k19754 in k19729 in k19726 in k19723 in k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_19756(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19756,c,av);}
t2=((C_word*)t0)[2];
f_19737(t2,C_i_not(t1));}

/* k19773 in k19726 in k19723 in k19714 in k19711 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_19775(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19775,c,av);}
t2=((C_word*)t0)[2];
f_19731(t2,(C_truep(t1)?C_i_listp(((C_word*)t0)[3]):C_SCHEME_FALSE));}

/* k19791 in k19705 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19793(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19793,c,av);}
t2=((C_word*)t0)[2];
f_19713(t2,C_eqp(lf[97],t1));}

/* k19809 in k19819 in k19831 in k19835 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19811(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_19811,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
f_19707(2,av2);}}
else{
/* core.scm:2589: test */
t2=((C_word*)((C_word*)t0)[3])[1];
f_19472(t2,((C_word*)t0)[2],((C_word*)t0)[4],lf[570]);}}

/* k19819 in k19831 in k19835 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_19821,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19811,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2588: test */
t3=((C_word*)((C_word*)t0)[3])[1];
f_19472(t3,t2,((C_word*)t0)[4],lf[96]);}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_19707(2,av2);}}}

/* k19823 in k19831 in k19835 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19825(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19825,c,av);}
/* core.scm:2587: chicken.compiler.support#node? */
t2=*((C_word*)lf[596]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k19831 in k19835 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19833(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_19833,c,av);}
a=C_alloc(8);
t2=C_eqp(lf[405],t1);
if(C_truep(C_i_not(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19821,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19825,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2587: chicken.compiler.support#variable-mark */
t5=*((C_word*)lf[185]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[407];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_19707(2,av2);}}}

/* k19835 in k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_19837,c,av);}
a=C_alloc(5);
if(C_truep(C_i_not(t1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19833,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2584: chicken.compiler.support#variable-mark */
t3=*((C_word*)lf[185]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[404];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_19707(2,av2);}}}

/* k19839 in k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19841(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_19841,c,av);}
a=C_alloc(16);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_19707,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t2,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19837,a[2]=t3,a[3]=((C_word*)t0)[9],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2581: test */
t5=((C_word*)((C_word*)t0)[9])[1];
f_19472(t5,t4,t2,lf[543]);}

/* k19843 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19845(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_19845,c,av);}
a=C_alloc(17);
t2=C_eqp(lf[523],t1);
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19631,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t4=(C_truep(((C_word*)t0)[7])?((C_word*)t0)[7]:t2);
if(C_truep(t4)){
if(C_truep(t2)){
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_19841,a[2]=((C_word*)t0)[7],a[3]=((C_word*)t0)[8],a[4]=t3,a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[12],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2580: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[13];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_a_i_cons(&a,2,((C_word*)t0)[7],C_SCHEME_END_OF_LIST);
t6=C_a_i_cons(&a,2,((C_word*)t0)[8],t5);
/* core.scm:2574: chicken.compiler.support#node-parameters-set! */
t7=*((C_word*)lf[607]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
av2[3]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}}
else{
t5=C_a_i_cons(&a,2,((C_word*)t0)[8],C_SCHEME_END_OF_LIST);
/* core.scm:2574: chicken.compiler.support#node-parameters-set! */
t6=*((C_word*)lf[607]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t6;
av2[1]=t3;
av2[2]=((C_word*)t0)[9];
av2[3]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(4,av2);}}}

/* a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_19873,c,av);}
a=C_alloc(10);
t5=(C_truep(((C_word*)t0)[2])?C_i_car(((C_word*)t0)[3]):lf[608]);
t6=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_19880,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t2,a[5]=((C_word*)t0)[6],a[6]=t5,a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,tmp=(C_word)a,a+=10,tmp);
/* core.scm:2618: scheme#append */
t7=*((C_word*)lf[5]+1);{
C_word *av2=av;
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_19880,c,av);}
a=C_alloc(22);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19885,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19890,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_19917,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2618: ##sys#dynamic-wind */
t9=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[9];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a19884 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19885,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a19889 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19890(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_19890,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_19894,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_19911,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2619: gather */
t4=((C_word*)((C_word*)t0)[6])[1];
f_19497(t4,t3,C_i_car(((C_word*)t0)[7]),((C_word*)t0)[5],((C_word*)t0)[3]);}

/* k19892 in a19889 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19894(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,5)))){
C_save_and_reclaim((void *)f_19894,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_19897,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2620: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=((C_word*)t0)[6];
av2[4]=lf[610];
av2[5]=C_i_length(t1);
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k19895 in k19892 in a19889 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19897(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,5)))){
C_save_and_reclaim((void *)f_19897,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19900,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2621: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
av2[4]=lf[609];
av2[5]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}

/* k19898 in k19895 in k19892 in a19889 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_19900(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_19900,c,av);}
a=C_alloc(6);
/* core.scm:2622: lset-difference/eq? */
f_7175(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,2,((C_word*)t0)[4],((C_word*)t0)[5]));}

/* k19909 in a19889 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19911(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_19911,c,av);}
a=C_alloc(6);
t2=*((C_word*)lf[587]+1);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6876,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t6=((C_word*)t4)[1];
f_6876(t6,((C_word*)t0)[2],t1);}

/* a19916 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_19917,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* g4458 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19931(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_19931,3,t0,t1,t2);}
/* core.scm:2624: gather */
t3=((C_word*)((C_word*)t0)[2])[1];
f_19497(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* k19939 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19941(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_19941,c,av);}
/* core.scm:2624: concatenate */
f_6767(((C_word*)t0)[2],t1);}

/* map-loop4452 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_19943(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_19943,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_19968,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2624: g4458 */
t4=((C_word*)t0)[4];
f_19931(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k19966 in map-loop4452 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_19968(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_19968,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_19943(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_20028(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_20028,5,t0,t1,t2,t3,t4);}
a=C_alloc(11);
t5=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_20032,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=t3,a[7]=t4,a[8]=((C_word*)t0)[4],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[6],tmp=(C_word)a,a+=11,tmp);
/* core.scm:2628: chicken.compiler.support#node-subexpressions */
t6=*((C_word*)lf[527]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_20032,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_20035,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
/* core.scm:2629: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20035(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,2)))){
C_save_and_reclaim((void *)f_20035,c,av);}
a=C_alloc(13);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_20038,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
/* core.scm:2630: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20038(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_20038,c,av);}
a=C_alloc(14);
t2=C_eqp(t1,lf[118]);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_20047,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
if(C_truep(t2)){
t4=t3;
f_20047(t4,t2);}
else{
t4=C_eqp(t1,lf[475]);
if(C_truep(t4)){
t5=t3;
f_20047(t5,t4);}
else{
t5=C_eqp(t1,lf[163]);
if(C_truep(t5)){
t6=t3;
f_20047(t6,t5);}
else{
t6=C_eqp(t1,lf[561]);
if(C_truep(t6)){
t7=t3;
f_20047(t7,t6);}
else{
t7=C_eqp(t1,lf[638]);
t8=t3;
f_20047(t8,(C_truep(t7)?t7:C_eqp(t1,lf[639])));}}}}}

/* k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_20047(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,4)))){
C_save_and_reclaim_args((void *)trf_20047,2,t0,t1);}
a=C_alloc(14);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[523]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[5]);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20059,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[6],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2639: ref-var */
f_21265(t4,((C_word*)t0)[3],((C_word*)t0)[8],((C_word*)t0)[9]);}
else{
t3=C_eqp(((C_word*)t0)[4],lf[534]);
t4=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_20081,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[6],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
if(C_truep(t3)){
t5=t4;
f_20081(t5,t3);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[636]);
if(C_truep(t5)){
t6=t4;
f_20081(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[565]);
t7=t4;
f_20081(t7,(C_truep(t6)?t6:C_eqp(((C_word*)t0)[4],lf[637])));}}}}}

/* k20057 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20059(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_20059,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20065,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2640: test */
t3=((C_word*)((C_word*)t0)[3])[1];
f_19472(t3,t2,((C_word*)t0)[4],lf[597]);}

/* k20063 in k20057 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20065(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_20065,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=C_a_i_list1(&a,1,((C_word*)t0)[2]);
/* core.scm:2641: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[611];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_20081(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,4)))){
C_save_and_reclaim_args((void *)trf_20081,2,t0,t1);}
a=C_alloc(14);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20084,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2645: ref-var */
f_21265(t2,((C_word*)t0)[8],((C_word*)t0)[5],((C_word*)t0)[6]);}
else{
t2=C_eqp(((C_word*)t0)[2],lf[143]);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_20167,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[2],a[10]=((C_word*)t0)[3],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[9],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
if(C_truep(t2)){
t4=t3;
f_20167(t4,t2);}
else{
t4=C_eqp(((C_word*)t0)[2],lf[520]);
if(C_truep(t4)){
t5=t3;
f_20167(t5,t4);}
else{
t5=C_eqp(((C_word*)t0)[2],lf[300]);
if(C_truep(t5)){
t6=t3;
f_20167(t6,t5);}
else{
t6=C_eqp(((C_word*)t0)[2],lf[301]);
if(C_truep(t6)){
t7=t3;
f_20167(t7,t6);}
else{
t7=C_eqp(((C_word*)t0)[2],lf[164]);
if(C_truep(t7)){
t8=t3;
f_20167(t8,t7);}
else{
t8=C_eqp(((C_word*)t0)[2],lf[119]);
if(C_truep(t8)){
t9=t3;
f_20167(t9,t8);}
else{
t9=C_eqp(((C_word*)t0)[2],lf[280]);
if(C_truep(t9)){
t10=t3;
f_20167(t10,t9);}
else{
t10=C_eqp(((C_word*)t0)[2],lf[188]);
if(C_truep(t10)){
t11=t3;
f_20167(t11,t10);}
else{
t11=C_eqp(((C_word*)t0)[2],lf[629]);
if(C_truep(t11)){
t12=t3;
f_20167(t12,t11);}
else{
t12=C_eqp(((C_word*)t0)[2],lf[630]);
if(C_truep(t12)){
t13=t3;
f_20167(t13,t12);}
else{
t13=C_eqp(((C_word*)t0)[2],lf[631]);
if(C_truep(t13)){
t14=t3;
f_20167(t14,t13);}
else{
t14=C_eqp(((C_word*)t0)[2],lf[538]);
if(C_truep(t14)){
t15=t3;
f_20167(t15,t14);}
else{
t15=C_eqp(((C_word*)t0)[2],lf[632]);
if(C_truep(t15)){
t16=t3;
f_20167(t16,t15);}
else{
t16=C_eqp(((C_word*)t0)[2],lf[633]);
if(C_truep(t16)){
t17=t3;
f_20167(t17,t16);}
else{
t17=C_eqp(((C_word*)t0)[2],lf[634]);
if(C_truep(t17)){
t18=t3;
f_20167(t18,t17);}
else{
t18=C_eqp(((C_word*)t0)[2],lf[635]);
if(C_truep(t18)){
t19=t3;
f_20167(t19,t18);}
else{
t19=C_eqp(((C_word*)t0)[2],lf[123]);
t20=t3;
f_20167(t20,(C_truep(t19)?t19:C_eqp(((C_word*)t0)[2],lf[283])));}}}}}}}}}}}}}}}}}}

/* k20082 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20084(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_20084,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_20087,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t3=C_eqp(t1,((C_word*)t0)[8]);
if(C_truep(t3)){
/* core.scm:2646: chicken.compiler.support#varnode */
t4=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=C_i_car(((C_word*)t0)[7]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
f_20087(2,av2);}}}

/* k20085 in k20082 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,4)))){
C_save_and_reclaim((void *)f_20087,c,av);}
a=C_alloc(36);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_20090,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t3=C_eqp(((C_word*)t0)[8],((C_word*)t0)[9]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20126,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t3)){
if(C_truep(t3)){
t5=t2;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_20090(2,av2);}}
else{
/* core.scm:2650: chicken.compiler.support#bomb */
t5=*((C_word*)lf[530]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=lf[613];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}}
else{
t5=C_a_i_list(&a,1,lf[594]);
t6=C_a_i_list(&a,1,((C_word*)t0)[5]);
t7=C_a_i_list(&a,2,lf[523],t6);
t8=C_a_i_list(&a,3,lf[614],t5,t7);
/* core.scm:2648: chicken.compiler.support#match-node */
t9=*((C_word*)lf[573]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t4;
av2[2]=((C_word*)t0)[8];
av2[3]=t8;
av2[4]=lf[615];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}}

/* k20088 in k20085 in k20082 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20090(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20090,c,av);}
a=C_alloc(6);
t2=C_eqp(((C_word*)t0)[2],lf[534]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20103,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2656: chicken.compiler.support#replace-rest-op-with-list-ops */
t4=*((C_word*)lf[612]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[7];
av2[4]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t3=C_eqp(((C_word*)t0)[9],((C_word*)t0)[10]);
if(C_truep(C_i_not(t3))){
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20116,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2661: chicken.compiler.support#replace-rest-op-with-list-ops */
t5=*((C_word*)lf[612]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[7];
av2[4]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}}

/* k20101 in k20088 in k20085 in k20082 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20103(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20103,c,av);}
/* core.scm:2656: transform */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20028(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k20114 in k20088 in k20085 in k20082 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20116(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20116,c,av);}
/* core.scm:2661: transform */
t2=((C_word*)((C_word*)t0)[2])[1];
f_20028(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k20124 in k20085 in k20082 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20126,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_20090(2,av2);}}
else{
/* core.scm:2650: chicken.compiler.support#bomb */
t2=*((C_word*)lf[530]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[613];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_20167(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,5)))){
C_save_and_reclaim_args((void *)trf_20167,2,t0,t1);}
a=C_alloc(14);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20174,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2671: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=C_eqp(((C_word*)t0)[9],lf[174]);
if(C_truep(t2)){
t3=C_i_car(((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_20190,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[4],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2675: test */
t5=((C_word*)((C_word*)t0)[11])[1];
f_19472(t5,t4,t3,lf[597]);}
else{
t3=C_eqp(((C_word*)t0)[9],lf[97]);
t4=(C_truep(t3)?t3:C_eqp(((C_word*)t0)[9],lf[545]));
if(C_truep(t4)){
t5=C_i_caddr(((C_word*)t0)[3]);
t6=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_20271,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[7],a[6]=t5,a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[5],a[10]=((C_word*)t0)[13],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* core.scm:2691: ##sys#decompose-lambda-list */
t7=*((C_word*)lf[191]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=((C_word*)t0)[2];
av2[2]=t5;
av2[3]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t5=C_eqp(((C_word*)t0)[9],lf[284]);
if(C_truep(t5)){
t6=C_i_car(((C_word*)t0)[3]);
t7=C_i_car(((C_word*)t0)[5]);
t8=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_20785,a[2]=((C_word*)t0)[10],a[3]=t7,a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[11],a[7]=t6,a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[3],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2755: chicken.compiler.support#node-class */
t9=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t9;
av2[1]=t8;
av2[2]=t7;
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}
else{
t6=C_eqp(((C_word*)t0)[9],lf[474]);
if(C_truep(t6)){
t7=(C_truep(*((C_word*)lf[41]+1))?C_a_i_list1(&a,1,C_fix(2)):C_a_i_list1(&a,1,C_fix(1)));
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20960,a[2]=((C_word*)t0)[2],a[3]=t7,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t9=C_i_car(((C_word*)t0)[3]);
t10=C_a_i_list2(&a,2,t9,C_SCHEME_TRUE);
/* core.scm:2786: chicken.compiler.support#make-node */
t11=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t11;
av2[1]=t8;
av2[2]=lf[561];
av2[3]=t10;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t11+1)))(5,av2);}}
else{
t7=C_eqp(((C_word*)t0)[9],lf[614]);
if(C_truep(t7)){
t8=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t8;
av2[1]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
/* core.scm:2793: chicken.compiler.support#bomb */
t8=*((C_word*)lf[530]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[628];
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}}}}}}}

/* k20172 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20174(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_20174,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20178,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2671: maptransform */
t3=((C_word*)((C_word*)t0)[4])[1];
f_21214(t3,t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k20176 in k20172 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20178(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20178,c,av);}
/* core.scm:2671: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k20188 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20190(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_20190,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_20193,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* core.scm:2676: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k20191 in k20188 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20193(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,4)))){
C_save_and_reclaim((void *)f_20193,c,av);}
a=C_alloc(13);
if(C_truep(((C_word*)t0)[2])){
t2=C_a_i_list1(&a,1,t1);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_20211,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,tmp=(C_word)a,a+=10,tmp);
/* core.scm:2680: transform */
t4=((C_word*)((C_word*)t0)[5])[1];
f_20028(t4,t3,C_i_car(((C_word*)t0)[6]),((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20254,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[9],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2687: maptransform */
t3=((C_word*)((C_word*)t0)[10])[1];
f_21214(t3,t2,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}}

/* k20209 in k20191 in k20188 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20211(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_20211,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20215,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_a_i_list1(&a,1,((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20227,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20243,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2683: chicken.compiler.support#varnode */
t6=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k20213 in k20209 in k20191 in k20188 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_20215(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20215,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2678: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20225 in k20209 in k20191 in k20188 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_20227(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_20227,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20231,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2684: transform */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20028(t3,t2,C_i_cadr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k20229 in k20225 in k20209 in k20191 in k20188 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_20231(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20231,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2681: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20241 in k20209 in k20191 in k20188 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_20243(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_20243,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2683: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[616];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20252 in k20191 in k20188 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20254(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20254,c,av);}
/* core.scm:2685: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20271(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,3)))){
C_save_and_reclaim((void *)f_20271,c,av);}
a=C_alloc(19);
t5=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_20275,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=t2,a[12]=t4,a[13]=t3,a[14]=((C_word*)t0)[10],a[15]=((C_word*)t0)[11],tmp=(C_word)a,a+=16,tmp);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20766,a[2]=((C_word*)t0)[11],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2694: filter */
f_6915(t5,t6,t2);}

/* k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20275(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,c,3)))){
C_save_and_reclaim((void *)f_20275,c,av);}
a=C_alloc(36);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t7=t6;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=((C_word*)t8)[1];
t10=C_i_check_list_2(t1,lf[10]);
t11=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_20287,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=t4,a[18]=t5,tmp=(C_word)a,a+=19,tmp);
t12=C_SCHEME_UNDEFINED;
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_set_block_item(t13,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20732,a[2]=t8,a[3]=t13,a[4]=t9,tmp=(C_word)a,a+=5,tmp));
t15=((C_word*)t13)[1];
f_20732(t15,t11,t1);}

/* k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20287(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,4)))){
C_save_and_reclaim((void *)f_20287,c,av);}
a=C_alloc(24);
t2=C_i_check_list_2(((C_word*)t0)[2],lf[10]);
t3=C_i_check_list_2(t1,lf[10]);
t4=(*a=C_CLOSURE_TYPE|16,a[1]=(C_word)f_20296,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[2],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],tmp=(C_word)a,a+=17,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20684,a[2]=((C_word*)t0)[17],a[3]=t6,a[4]=((C_word*)t0)[18],tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_20684(t8,t4,((C_word*)t0)[2],t1);}

/* k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_20296(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_20296,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_20299,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],tmp=(C_word)a,a+=18,tmp);
/* core.scm:2696: chicken.base#gensym */
t3=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[161];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_20299(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,3)))){
C_save_and_reclaim((void *)f_20299,c,av);}
a=C_alloc(20);
t2=(C_truep(((C_word*)t0)[2])?C_i_car(((C_word*)t0)[3]):lf[608]);
t3=(*a=C_CLOSURE_TYPE|19,a[1]=(C_word)f_20305,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t2,a[8]=((C_word*)t0)[3],a[9]=t1,a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[9],a[12]=((C_word*)t0)[10],a[13]=((C_word*)t0)[11],a[14]=((C_word*)t0)[12],a[15]=((C_word*)t0)[13],a[16]=((C_word*)t0)[14],a[17]=((C_word*)t0)[15],a[18]=((C_word*)t0)[16],a[19]=((C_word*)t0)[17],tmp=(C_word)a,a+=20,tmp);
/* core.scm:2698: test */
t4=((C_word*)((C_word*)t0)[17])[1];
f_19472(t4,t3,t2,lf[609]);}

/* k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_20305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,3)))){
C_save_and_reclaim((void *)f_20305,c,av);}
a=C_alloc(21);
t2=(C_truep(t1)?t1:C_SCHEME_END_OF_LIST);
t3=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_20311,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t2,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],tmp=(C_word)a,a+=21,tmp);
/* core.scm:2699: test */
t4=((C_word*)((C_word*)t0)[19])[1];
f_19472(t4,t3,((C_word*)t0)[7],lf[610]);}

/* k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_20311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,2)))){
C_save_and_reclaim((void *)f_20311,c,av);}
a=C_alloc(22);
t2=(C_truep(t1)?t1:C_fix(0));
t3=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_20317,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],tmp=(C_word)a,a+=22,tmp);
if(C_truep(*((C_word*)lf[41]+1))){
t4=C_i_cadr(((C_word*)t0)[9]);
t5=t3;
f_20317(t5,(C_truep(t4)?C_i_pairp(((C_word*)t0)[7]):C_SCHEME_FALSE));}
else{
t4=t3;
f_20317(t4,C_SCHEME_FALSE);}}

/* k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_20317(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,0,3)))){
C_save_and_reclaim_args((void *)trf_20317,2,t0,t1);}
a=C_alloc(28);
t2=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_20320,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],tmp=(C_word)a,a+=21,tmp);
if(C_truep(((C_word*)t0)[18])){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20650,a[2]=((C_word*)t0)[18],a[3]=((C_word*)t0)[14],a[4]=t2,a[5]=((C_word*)t0)[20],a[6]=((C_word*)t0)[21],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2705: test */
t4=((C_word*)((C_word*)t0)[21])[1];
f_19472(t4,t3,((C_word*)t0)[18],lf[597]);}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_20320(2,av2);}}}

/* k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_20320(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(78,c,3)))){
C_save_and_reclaim((void *)f_20320,c,av);}
a=C_alloc(78);
t2=(C_truep(((C_word*)t0)[2])?C_s_a_i_plus(&a,2,((C_word*)t0)[3],C_fix(2)):C_s_a_i_plus(&a,2,((C_word*)t0)[3],C_fix(1)));
t3=C_a_i_list1(&a,1,t2);
t4=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_20335,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=t3,a[8]=((C_word*)t0)[2],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t5=C_i_cadr(((C_word*)t0)[11]);
t6=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_20571,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[10],a[5]=t5,a[6]=t4,a[7]=((C_word*)t0)[13],a[8]=((C_word*)t0)[14],a[9]=((C_word*)t0)[15],a[10]=((C_word*)t0)[16],a[11]=((C_word*)t0)[17],a[12]=((C_word*)t0)[7],tmp=(C_word)a,a+=13,tmp);
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20576,a[2]=((C_word*)t0)[15],tmp=(C_word)a,a+=3,tmp);
t12=C_i_check_list_2(((C_word*)t0)[18],lf[10]);
t13=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20592,a[2]=((C_word*)t0)[19],a[3]=((C_word*)t0)[15],a[4]=t6,a[5]=((C_word*)t0)[20],tmp=(C_word)a,a+=6,tmp);
t14=C_SCHEME_UNDEFINED;
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_set_block_item(t15,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20607,a[2]=t11,a[3]=t9,a[4]=t15,a[5]=t10,tmp=(C_word)a,a+=6,tmp));
t17=((C_word*)t15)[1];
f_20607(t17,t13,((C_word*)t0)[18]);}

/* k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_20335(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_20335,c,av);}
a=C_alloc(26);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20340,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t7=C_i_check_list_2(((C_word*)t0)[5],lf[10]);
t8=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20354,a[2]=t1,a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],tmp=(C_word)a,a+=8,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20397,a[2]=t4,a[3]=t10,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_20397(t12,t8,((C_word*)t0)[5]);}

/* g4751 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in ... */
static void C_fcall f_20340(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20340,3,t0,t1,t2);}
a=C_alloc(6);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20348,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2739: chicken.compiler.support#varnode */
t4=*((C_word*)lf[521]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k20346 in g4751 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20348(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20348,c,av);}
/* core.scm:2739: ref-var */
f_21265(((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5]);}

/* k20352 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in ... */
static void C_ccall f_20354(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,4)))){
C_save_and_reclaim((void *)f_20354,c,av);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20357,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(((C_word*)t0)[5])){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20368,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20372,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20376,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20384,a[2]=t5,a[3]=((C_word*)t0)[6],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2747: chicken.compiler.support#real-name */
t7=*((C_word*)lf[541]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2708: chicken.compiler.support#make-node */
t4=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[617];
av2[3]=((C_word*)t0)[4];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k20355 in k20352 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20357(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_20357,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2708: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[617];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20366 in k20352 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20368(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_20368,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2742: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k20370 in k20352 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20372(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20372,c,av);}
/* core.scm:2745: chicken.compiler.support#qnode */
t2=*((C_word*)lf[618]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k20374 in k20352 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20376(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20376,c,av);}
/* core.scm:2746: ##sys#make-lambda-info */
t2=*((C_word*)lf[619]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k20382 in k20352 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20384(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_20384,c,av);}
a=C_alloc(4);
t2=(C_truep(t1)?t1:lf[620]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20391,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2749: chicken.syntax#strip-syntax */
t4=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_cdr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k20389 in k20382 in k20352 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in ... */
static void C_ccall f_20391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20391,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2747: chicken.string#->string */
t3=*((C_word*)lf[621]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* map-loop4745 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in ... */
static void C_fcall f_20397(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20397,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20422,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2739: g4751 */
t4=((C_word*)t0)[4];
f_20340(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20420 in map-loop4745 in k20333 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20422(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20422,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20397(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in ... */
static void C_ccall f_20441(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_20441,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20444,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_pairp(((C_word*)t0)[5]))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20454,a[2]=((C_word*)t0)[6],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2728: unzip1 */
f_7024(t3,((C_word*)t0)[6]);}
else{
t3=C_a_i_list1(&a,1,t1);
/* core.scm:2711: chicken.compiler.support#make-node */
t4=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k20442 in k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20444(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_20444,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2711: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20452 in k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in ... */
static void C_ccall f_20454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_20454,c,av);}
a=C_alloc(17);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(((C_word*)t0)[2],lf[10]);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20481,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20519,a[2]=t4,a[3]=t9,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_20519(t11,t7,((C_word*)t0)[2]);}

/* k20469 in map-loop4717 in k20452 in k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in ... */
static void C_ccall f_20471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_20471,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2731: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[616];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20479 in k20452 in k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in ... */
static void C_ccall f_20481(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20481,c,av);}
a=C_alloc(6);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_set_block_item(t3,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20483,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp));
t5=((C_word*)t3)[1];
f_20483(t5,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* loop in k20479 in k20452 in k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in ... */
static void C_fcall f_20483(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_20483,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
t5=C_a_i_list1(&a,1,t4);
t6=C_i_car(t3);
t7=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20509,a[2]=t6,a[3]=t1,a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2737: loop */
t9=t7;
t10=C_u_i_cdr(t2);
t11=C_u_i_cdr(t3);
t1=t9;
t2=t10;
t3=t11;
goto loop;}}

/* k20507 in loop in k20479 in k20452 in k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in ... */
static void C_ccall f_20509(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20509,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2735: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[174];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* map-loop4717 in k20452 in k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in ... */
static void C_fcall f_20519(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_20519,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20544,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20471,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2731: chicken.compiler.support#varnode */
t6=*((C_word*)lf[521]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_i_cdr(t4);
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20542 in map-loop4717 in k20452 in k20439 in k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in ... */
static void C_ccall f_20544(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20544,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20519(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k20569 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_20571(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_20571,c,av);}
a=C_alloc(22);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_i_cadddr(((C_word*)t0)[3]);
t4=C_a_i_list4(&a,4,((C_word*)t0)[4],((C_word*)t0)[5],t2,t3);
t5=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_20441,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[7],a[4]=t4,a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2726: transform */
t6=((C_word*)((C_word*)t0)[10])[1];
f_20028(t6,t5,C_i_car(((C_word*)t0)[11]),((C_word*)t0)[2],((C_word*)t0)[12]);}

/* g4678 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in ... */
static C_word C_fcall f_20576(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_stack_overflow_check;{}
t2=C_i_assq(t1,((C_word*)t0)[2]);
return((C_truep(t2)?C_i_cdr(t2):t1));}

/* k20590 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_20592(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_20592,c,av);}
t2=(C_truep(((C_word*)t0)[2])?C_i_assq(((C_word*)t0)[2],((C_word*)t0)[3]):C_SCHEME_FALSE);
if(C_truep(t2)){
/* core.scm:2717: chicken.compiler.support#build-lambda-list */
t3=*((C_word*)lf[187]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=t1;
av2[3]=((C_word*)t0)[5];
av2[4]=C_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* core.scm:2717: chicken.compiler.support#build-lambda-list */
t3=*((C_word*)lf[187]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=t1;
av2[3]=((C_word*)t0)[5];
av2[4]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* map-loop4672 in k20318 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in ... */
static void C_fcall f_20607(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_20607,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=(
/* core.scm:2718: g4678 */
  f_20576(((C_word*)t0)[2],C_slot(t2,C_fix(0)))
);
t4=C_a_i_cons(&a,2,t3,C_SCHEME_END_OF_LIST);
t5=C_i_setslot(((C_word*)((C_word*)t0)[3])[1],C_fix(1),t4);
t6=C_mutate(((C_word *)((C_word*)t0)[3])+1,t4);
t8=t1;
t9=C_slot(t2,C_fix(1));
t1=t8;
t2=t9;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20648 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_20650(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_20650,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20656,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2706: test */
t3=((C_word*)((C_word*)t0)[6])[1];
f_19472(t3,t2,((C_word*)t0)[2],lf[549]);}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_20320(2,av2);}}}

/* k20654 in k20648 in k20315 in k20309 in k20303 in k20297 in k20294 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_20656(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_20656,c,av);}
if(C_truep(t1)){
t2=C_i_assq(((C_word*)t0)[2],((C_word*)t0)[3]);
/* core.scm:2707: chicken.compiler.support#db-put! */
t3=*((C_word*)lf[535]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[5];
av2[3]=C_i_cdr(t2);
av2[4]=lf[622];
av2[5]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(6,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_20320(2,av2);}}}

/* map-loop4594 in k20285 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_20684(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_20684,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_slot(t2,C_fix(0));
t7=C_slot(t3,C_fix(0));
t8=C_a_i_cons(&a,2,t6,t7);
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
t10=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t9);
t11=C_mutate(((C_word *)((C_word*)t0)[2])+1,t9);
t13=t1;
t14=C_slot(t2,C_fix(1));
t15=C_slot(t3,C_fix(1));
t1=t13;
t2=t14;
t3=t15;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* map-loop4615 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_20732(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_20732,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20757,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2695: g4621 */
t4=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k20755 in map-loop4615 in k20273 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_20757(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_20757,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_20732(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* a20765 in a20270 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20766(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_20766,c,av);}
/* core.scm:2694: test */
t3=((C_word*)((C_word*)t0)[2])[1];
f_19472(t3,t1,t2,lf[597]);}

/* k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20785(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_20785,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_20788,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=t1,tmp=(C_word)a,a+=11,tmp);
t3=C_eqp(lf[118],t1);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20939,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2756: chicken.compiler.support#node-parameters */
t5=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
f_20788(2,av2);}}}

/* k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20788(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,2)))){
C_save_and_reclaim((void *)f_20788,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20791,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
if(C_truep(t1)){
t3=t2;
f_20791(t3,t1);}
else{
t3=C_i_cdr(((C_word*)t0)[9]);
if(C_truep(C_i_pairp(t3))){
t4=C_i_cadr(((C_word*)t0)[9]);
t5=t2;
f_20791(t5,(C_truep(t4)?t4:C_eqp(lf[475],((C_word*)t0)[10])));}
else{
t4=t2;
f_20791(t4,C_eqp(lf[475],((C_word*)t0)[10]));}}}

/* k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_20791(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,3)))){
C_save_and_reclaim_args((void *)trf_20791,2,t0,t1);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_20794,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:2759: posq */
f_7518(t2,((C_word*)t0)[7],((C_word*)t0)[5]);}

/* k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_20794(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_20794,c,av);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20798,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2759: g4792 */
t3=t2;
f_20798(t3,((C_word*)t0)[9],t1);}
else{
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20872,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2773: test */
t3=((C_word*)((C_word*)t0)[7])[1];
f_19472(t3,t2,((C_word*)t0)[8],lf[597]);}}

/* g4792 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_20798(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_20798,3,t0,t1,t2);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20805,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=t2,tmp=(C_word)a,a+=9,tmp);
/* core.scm:2761: test */
t4=((C_word*)((C_word*)t0)[7])[1];
f_19472(t4,t3,((C_word*)t0)[8],lf[597]);}

/* k20803 in g4792 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_20805(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(44,c,2)))){
C_save_and_reclaim((void *)f_20805,c,av);}
a=C_alloc(44);
if(C_truep(t1)){
t2=(C_truep(((C_word*)t0)[2])?lf[623]:lf[624]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20820,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t4=C_s_a_i_plus(&a,2,((C_word*)t0)[8],C_fix(1));
t5=C_a_i_list1(&a,1,t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20836,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2765: chicken.compiler.support#varnode */
t7=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
t2=(C_truep(((C_word*)t0)[2])?lf[625]:lf[626]);
t3=C_s_a_i_plus(&a,2,((C_word*)t0)[8],C_fix(1));
t4=C_a_i_list1(&a,1,t3);
t5=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_20857,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=t4,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* core.scm:2771: chicken.compiler.support#varnode */
t6=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}

/* k20818 in k20803 in g4792 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_20820(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_20820,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20824,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2766: transform */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20028(t3,t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k20822 in k20818 in k20803 in g4792 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_20824(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20824,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2762: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20834 in k20803 in g4792 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_20836(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_20836,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2765: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[614];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20855 in k20803 in g4792 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_20857(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20857,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_20861,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2772: transform */
t3=((C_word*)((C_word*)t0)[5])[1];
f_20028(t3,t2,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k20859 in k20855 in k20803 in g4792 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_20861(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20861,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2768: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20870 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_20872(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_20872,c,av);}
a=C_alloc(10);
if(C_truep(t1)){
t2=(C_truep(((C_word*)t0)[2])?lf[623]:lf[624]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_20887,a[2]=((C_word*)t0)[3],a[3]=t2,a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* core.scm:2777: chicken.compiler.support#varnode */
t4=*((C_word*)lf[521]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=C_a_i_list2(&a,2,((C_word*)t0)[8],((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_20906,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2781: transform */
t4=((C_word*)((C_word*)t0)[4])[1];
f_20028(t4,t3,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}}

/* k20885 in k20870 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_20887(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_20887,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20891,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2778: transform */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20028(t3,t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k20889 in k20885 in k20870 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_20891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20891,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:2774: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20904 in k20870 in k20792 in k20789 in k20786 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_20906(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_20906,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2779: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[284];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k20937 in k20783 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20939,c,av);}
/* core.scm:2756: chicken.compiler.support#immediate? */
t2=*((C_word*)lf[627]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_car(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k20958 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20960(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_20960,c,av);}
a=C_alloc(8);
if(C_truep(*((C_word*)lf[41]+1))){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_20971,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_20975,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:2788: ##sys#make-lambda-info */
t4=*((C_word*)lf[619]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_u_i_car(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
/* core.scm:2784: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[617];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k20969 in k20958 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20971(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_20971,c,av);}
a=C_alloc(6);
t2=C_a_i_list1(&a,1,t1);
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],t2);
/* core.scm:2784: chicken.compiler.support#make-node */
t4=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[617];
av2[3]=((C_word*)t0)[4];
av2[4]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k20973 in k20958 in k20165 in k20079 in k20045 in k20036 in k20033 in k20030 in transform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_20975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_20975,c,av);}
/* core.scm:2788: chicken.compiler.support#qnode */
t2=*((C_word*)lf[618]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* maptransform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21214(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,3)))){
C_save_and_reclaim_args((void *)trf_21214,5,t0,t1,t2,t3,t4);}
a=C_alloc(18);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21219,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t10=C_i_check_list_2(t2,lf[10]);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21231,a[2]=t7,a[3]=t12,a[4]=t9,a[5]=t8,tmp=(C_word)a,a+=6,tmp));
t14=((C_word*)t12)[1];
f_21231(t14,t1,t2);}

/* g4806 in maptransform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21219(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,4)))){
C_save_and_reclaim_args((void *)trf_21219,3,t0,t1,t2);}
/* core.scm:2796: transform */
t3=((C_word*)((C_word*)t0)[2])[1];
f_20028(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4]);}

/* map-loop4800 in maptransform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21231(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21231,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21256,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2796: g4806 */
t4=((C_word*)t0)[4];
f_21219(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k21254 in map-loop4800 in maptransform in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21256(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_21256,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_21231(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* ref-var in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21265(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21265,4,t1,t2,t3,t4);}
a=C_alloc(6);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21303,a[2]=t3,a[3]=t1,a[4]=t2,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* core.scm:2799: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[524]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k21270 in k21301 in ref-var in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21272(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21272,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21276,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2800: g4835 */
t3=t2;
f_21276(t3,((C_word*)t0)[3],t1);}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g4835 in k21270 in k21301 in ref-var in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21276(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(36,0,2)))){
C_save_and_reclaim_args((void *)trf_21276,3,t0,t1,t2);}
a=C_alloc(36);
t3=C_s_a_i_plus(&a,2,t2,C_fix(1));
t4=C_a_i_list1(&a,1,t3);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21292,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2803: chicken.compiler.support#varnode */
t6=*((C_word*)lf[521]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k21290 in g4835 in k21270 in k21301 in ref-var in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_21292,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2802: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[614];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21301 in ref-var in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21303(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_21303,c,av);}
a=C_alloc(5);
t2=C_i_car(t1);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21272,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2800: posq */
f_7518(t3,t2,((C_word*)t0)[5]);}

/* k21304 in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21306(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_21306,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21309,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2807: gather */
t3=((C_word*)((C_word*)t0)[7])[1];
f_19497(t3,t2,((C_word*)t0)[5],C_SCHEME_FALSE,C_SCHEME_END_OF_LIST);}

/* k21307 in k21304 in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21309(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_21309,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21312,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_i_pairp(((C_word*)((C_word*)t0)[6])[1]))){
/* core.scm:2809: chicken.compiler.support#debugging */
t3=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[640];
av2[3]=lf[643];
av2[4]=((C_word*)((C_word*)t0)[6])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_21312(2,av2);}}}

/* k21310 in k21307 in k21304 in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21312(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_21312,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21315,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2810: chicken.compiler.support#debugging */
t3=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[601];
av2[3]=lf[642];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k21313 in k21310 in k21307 in k21304 in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21315(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21315,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21318,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2811: transform */
t3=((C_word*)((C_word*)t0)[4])[1];
f_20028(t3,t2,((C_word*)t0)[5],C_SCHEME_FALSE,C_SCHEME_FALSE);}

/* k21316 in k21313 in k21310 in k21307 in k21304 in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21318(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_21318,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21321,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=((C_word*)((C_word*)t0)[3])[1];
if(C_truep(C_i_zerop(t3))){
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:2813: chicken.compiler.support#debugging */
t4=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[640];
av2[3]=lf[641];
av2[4]=((C_word*)((C_word*)t0)[3])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* k21319 in k21316 in k21313 in k21310 in k21307 in k21304 in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21321(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21321,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-id in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21346,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[647]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(1));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-external in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21355(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21355,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[649]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(2));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-argument-count in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21373(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21373,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[651]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(4));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-rest-argument in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21382(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21382,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[653]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(5));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-temporaries in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21391(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21391,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[655]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(6));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-float-temporaries in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21400(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21400,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[657]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(7));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-callee-signatures in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21409(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21409,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[659]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(8));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-allocated in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21418(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21418,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[661]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(9));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-closure-size in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21436(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21436,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[663]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(11));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-looping in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21445(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21445,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[665]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(12));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-customizable in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21454(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21454,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[667]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(13));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-rest-argument-mode in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21463(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21463,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[669]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(14));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-body in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21472(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21472,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[671]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(15));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#lambda-literal-direct in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21481(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21481,c,av);}
t3=C_i_check_structure_2(t2,lf[646],lf[673]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_i_block_ref(t2,C_fix(16));
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21490(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,3)))){
C_save_and_reclaim((void *)f_21490,c,av);}
a=C_alloc(17);
t4=C_SCHEME_END_OF_LIST;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_fix(0);
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_SCHEME_END_OF_LIST;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_fix(0);
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_21494,a[2]=t3,a[3]=t11,a[4]=t9,a[5]=t5,a[6]=t7,a[7]=t1,a[8]=t2,tmp=(C_word)a,a+=9,tmp);
t13=C_i_fixnum_max(lf[62],C_fix(1));
/* core.scm:2849: scheme#make-vector */
t14=*((C_word*)lf[93]+1);{
C_word *av2=av;
av2[0]=t14;
av2[1]=t12;
av2[2]=C_fixnum_times(t13,C_fix(3));
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t14+1)))(4,av2);}}

/* k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(99,c,7)))){
C_save_and_reclaim((void *)f_21494,c,av);}
a=C_alloc(99);
t2=C_fix(0);
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_END_OF_LIST;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_fix(0);
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_fix(0);
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_SCHEME_END_OF_LIST;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_fix(0);
t13=(*a=C_VECTOR_TYPE|1,a[1]=t12,tmp=(C_word)a,a+=2,tmp);
t14=C_fix(0);
t15=(*a=C_VECTOR_TYPE|1,a[1]=t14,tmp=(C_word)a,a+=2,tmp);
t16=C_fix(0);
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=C_fix(0);
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_SCHEME_END_OF_LIST;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=C_SCHEME_UNDEFINED;
t23=(*a=C_VECTOR_TYPE|1,a[1]=t22,tmp=(C_word)a,a+=2,tmp);
t24=C_SCHEME_UNDEFINED;
t25=(*a=C_VECTOR_TYPE|1,a[1]=t24,tmp=(C_word)a,a+=2,tmp);
t26=C_SCHEME_UNDEFINED;
t27=(*a=C_VECTOR_TYPE|1,a[1]=t26,tmp=(C_word)a,a+=2,tmp);
t28=C_SCHEME_UNDEFINED;
t29=(*a=C_VECTOR_TYPE|1,a[1]=t28,tmp=(C_word)a,a+=2,tmp);
t30=C_SCHEME_UNDEFINED;
t31=(*a=C_VECTOR_TYPE|1,a[1]=t30,tmp=(C_word)a,a+=2,tmp);
t32=C_SCHEME_UNDEFINED;
t33=(*a=C_VECTOR_TYPE|1,a[1]=t32,tmp=(C_word)a,a+=2,tmp);
t34=C_SCHEME_UNDEFINED;
t35=(*a=C_VECTOR_TYPE|1,a[1]=t34,tmp=(C_word)a,a+=2,tmp);
t36=C_SCHEME_UNDEFINED;
t37=(*a=C_VECTOR_TYPE|1,a[1]=t36,tmp=(C_word)a,a+=2,tmp);
t38=C_set_block_item(t23,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21496,a[2]=t31,a[3]=t25,tmp=(C_word)a,a+=4,tmp));
t39=C_set_block_item(t25,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21545,a[2]=t35,a[3]=t31,a[4]=t15,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp));
t40=C_set_block_item(t27,0,(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_21601,a[2]=t23,a[3]=t7,a[4]=t29,a[5]=t19,a[6]=t21,a[7]=t27,a[8]=t13,a[9]=t31,a[10]=t3,a[11]=t5,a[12]=t11,a[13]=t9,a[14]=t1,a[15]=((C_word*)t0)[2],a[16]=t35,a[17]=t17,a[18]=t37,tmp=(C_word)a,a+=19,tmp));
t41=C_set_block_item(t29,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22811,a[2]=t27,tmp=(C_word)a,a+=3,tmp));
t42=C_set_block_item(t31,0,(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22862,a[2]=t37,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=t33,tmp=(C_word)a,a+=8,tmp));
t43=C_set_block_item(t33,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22910,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp));
t44=C_set_block_item(t35,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22919,a[2]=((C_word*)t0)[6],a[3]=t33,a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp));
t45=C_set_block_item(t37,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_22963,tmp=(C_word)a,a+=2,tmp));
t46=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_23026,a[2]=((C_word*)t0)[7],a[3]=t1,a[4]=t21,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=t17,a[8]=t15,a[9]=t13,a[10]=t27,a[11]=((C_word*)t0)[8],tmp=(C_word)a,a+=12,tmp);
/* core.scm:3189: chicken.compiler.support#debugging */
t47=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t47;
av2[1]=t46;
av2[2]=lf[601];
av2[3]=lf[711];
((C_proc)(void*)(*((C_word*)t47+1)))(4,av2);}}

/* walk-var in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21496(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5){
C_word tmp;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_21496,6,t0,t1,t2,t3,t4,t5);}
a=C_alloc(8);
t6=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21500,a[2]=t4,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=t2,a[6]=((C_word*)t0)[3],a[7]=t5,tmp=(C_word)a,a+=8,tmp);
/* core.scm:2862: posq */
f_7518(t6,t2,t3);}

/* k21498 in walk-var in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21500(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,3)))){
C_save_and_reclaim((void *)f_21500,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21504,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2862: g4976 */
t3=t2;
f_21504(t3,((C_word*)t0)[3],t1);}
else{
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21529,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2865: chicken.keyword#keyword? */
t3=*((C_word*)lf[482]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* g4976 in k21498 in walk-var in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21504(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_21504,3,t0,t1,t2);}
a=C_alloc(3);
t3=C_fixnum_plus(t2,C_fix(1));
t4=C_fixnum_difference(((C_word*)t0)[2],t3);
t5=C_a_i_list1(&a,1,t4);
/* core.scm:2864: chicken.compiler.support#make-node */
t6=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[675];
av2[3]=t5;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k21527 in k21498 in walk-var in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21529,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21540,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2865: literal */
t3=((C_word*)((C_word*)t0)[3])[1];
f_22862(t3,t2,((C_word*)t0)[4]);}
else{
/* core.scm:2866: walk-global */
t2=((C_word*)((C_word*)t0)[5])[1];
f_21545(t2,((C_word*)t0)[2],((C_word*)t0)[4],((C_word*)t0)[6]);}}

/* k21538 in k21527 in k21498 in walk-var in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21540(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_21540,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2865: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[676];
av2[3]=t2;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* walk-global in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21545(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_21545,4,t0,t1,t2,t3);}
a=C_alloc(12);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21549,a[2]=t2,a[3]=t1,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t3)){
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=t3;
f_21549(2,av2);}}
else{
t5=*((C_word*)lf[31]+1);
if(C_truep(*((C_word*)lf[31]+1))){
t6=*((C_word*)lf[31]+1);
t7=t4;{
C_word av2[2];
av2[0]=t7;
av2[1]=*((C_word*)lf[31]+1);
f_21549(2,av2);}}
else{
t6=*((C_word*)lf[149]+1);
if(C_truep(*((C_word*)lf[149]+1))){
t7=t4;{
C_word av2[2];
av2[0]=t7;
av2[1]=*((C_word*)lf[149]+1);
f_21549(2,av2);}}
else{
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21593,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:2872: chicken.compiler.support#variable-mark */
t8=*((C_word*)lf[185]+1);{
C_word av2[4];
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t8+1)))(4,av2);}}}}}

/* k21547 in walk-global in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21549(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_21549,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21552,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21577,a[2]=t2,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2874: chicken.compiler.support#db-get */
t4=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[2];
av2[4]=lf[553];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k21550 in k21547 in walk-global in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21552(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(37,0,2)))){
C_save_and_reclaim_args((void *)trf_21552,2,t0,t1);}
a=C_alloc(37);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_21555,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t1)){
t3=((C_word*)((C_word*)t0)[7])[1];
t4=C_mutate(((C_word *)((C_word*)t0)[7])+1,C_s_a_i_plus(&a,2,t3,C_fix(1)));
t5=t2;
f_21555(t5,t4);}
else{
t3=t2;
f_21555(t3,C_SCHEME_UNDEFINED);}}

/* k21553 in k21550 in k21547 in walk-global in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21555(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_21555,2,t0,t1);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21566,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)t0)[3])){
/* core.scm:2880: blockvar-literal */
t3=((C_word*)((C_word*)t0)[6])[1];
f_22919(t3,t2,((C_word*)t0)[4]);}
else{
/* core.scm:2881: literal */
t3=((C_word*)((C_word*)t0)[7])[1];
f_22862(t3,t2,((C_word*)t0)[4]);}}

/* k21564 in k21553 in k21550 in k21547 in walk-global in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21566(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,4)))){
C_save_and_reclaim((void *)f_21566,c,av);}
a=C_alloc(12);
t2=C_a_i_list4(&a,4,t1,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[4]);
/* core.scm:2877: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[5];
av2[2]=lf[677];
av2[3]=t2;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21575 in k21547 in walk-global in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21577(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_21577,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21584,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:2875: chicken.compiler.support#variable-visible? */
t3=*((C_word*)lf[571]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[25]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];
f_21552(t2,C_SCHEME_FALSE);}}

/* k21582 in k21575 in k21547 in walk-global in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21584(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21584,c,av);}
t2=((C_word*)t0)[2];
f_21552(t2,C_i_not(t1));}

/* k21591 in walk-global in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21593(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21593,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
f_21549(2,av2);}}
else{
/* core.scm:2873: chicken.compiler.support#intrinsic? */
t2=*((C_word*)lf[580]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}}

/* walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21601(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,0,2)))){
C_save_and_reclaim_args((void *)trf_21601,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(25);
t7=(*a=C_CLOSURE_TYPE|24,a[1]=(C_word)f_21605,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t3,a[6]=t4,a[7]=((C_word*)t0)[3],a[8]=((C_word*)t0)[4],a[9]=t5,a[10]=t6,a[11]=((C_word*)t0)[5],a[12]=((C_word*)t0)[6],a[13]=((C_word*)t0)[7],a[14]=((C_word*)t0)[8],a[15]=((C_word*)t0)[9],a[16]=((C_word*)t0)[10],a[17]=((C_word*)t0)[11],a[18]=((C_word*)t0)[12],a[19]=((C_word*)t0)[13],a[20]=((C_word*)t0)[14],a[21]=((C_word*)t0)[15],a[22]=((C_word*)t0)[16],a[23]=((C_word*)t0)[17],a[24]=((C_word*)t0)[18],tmp=(C_word)a,a+=25,tmp);
/* core.scm:2888: chicken.compiler.support#node-subexpressions */
t8=*((C_word*)lf[527]+1);{
C_word av2[3];
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}

/* k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21605(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,2)))){
C_save_and_reclaim((void *)f_21605,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|25,a[1]=(C_word)f_21608,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=t1,a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],a[24]=((C_word*)t0)[23],a[25]=((C_word*)t0)[24],tmp=(C_word)a,a+=26,tmp);
/* core.scm:2889: chicken.compiler.support#node-parameters */
t3=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21608(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,2)))){
C_save_and_reclaim((void *)f_21608,c,av);}
a=C_alloc(29);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|26,a[1]=(C_word)f_21611,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],a[24]=((C_word*)t0)[23],a[25]=((C_word*)t0)[24],a[26]=((C_word*)t0)[25],tmp=(C_word)a,a+=27,tmp);
/* core.scm:2890: chicken.compiler.support#node-class */
t5=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21611(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,2)))){
C_save_and_reclaim((void *)f_21611,c,av);}
a=C_alloc(28);
t2=C_eqp(t1,lf[475]);
t3=(*a=C_CLOSURE_TYPE|27,a[1]=(C_word)f_21620,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],a[23]=((C_word*)t0)[22],a[24]=((C_word*)t0)[23],a[25]=((C_word*)t0)[24],a[26]=((C_word*)t0)[25],a[27]=((C_word*)t0)[26],tmp=(C_word)a,a+=28,tmp);
if(C_truep(t2)){
t4=t3;
f_21620(t4,t2);}
else{
t4=C_eqp(t1,lf[561]);
t5=t3;
f_21620(t5,(C_truep(t4)?t4:C_eqp(t1,lf[638])));}}

/* k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21620(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(40,0,6)))){
C_save_and_reclaim_args((void *)trf_21620,2,t0,t1);}
a=C_alloc(40);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[523]);
if(C_truep(t2)){
t3=((C_word*)((C_word*)t0)[5])[1];
/* core.scm:2896: walk-var */
t4=((C_word*)((C_word*)t0)[6])[1];
f_21496(t4,((C_word*)t0)[2],C_i_car(t3),((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}
else{
t3=C_eqp(((C_word*)t0)[4],lf[631]);
if(C_truep(t3)){
t4=((C_word*)((C_word*)t0)[5])[1];
t5=C_i_cadr(t4);
t6=((C_word*)((C_word*)t0)[5])[1];
t7=C_i_cadddr(t6);
t8=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_21648,a[2]=((C_word*)t0)[9],a[3]=t7,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
t9=(C_truep(*((C_word*)lf[24]+1))?t5:C_SCHEME_FALSE);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21679,a[2]=((C_word*)t0)[14],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[15],a[5]=t8,a[6]=t5,tmp=(C_word)a,a+=7,tmp);
/* core.scm:2903: chicken.compiler.support#source-info->line */
t11=*((C_word*)lf[680]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=t5;
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=C_a_i_cons(&a,2,C_SCHEME_FALSE,((C_word*)((C_word*)t0)[5])[1]);
t11=C_mutate(((C_word *)((C_word*)t0)[5])+1,t10);
t12=t8;
f_21648(t12,t11);}}
else{
t4=C_eqp(((C_word*)t0)[4],lf[301]);
if(C_truep(t4)){
t5=((C_word*)((C_word*)t0)[5])[1];
t6=C_i_cadr(t5);
t7=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[9])[1],t6);
t8=C_mutate(((C_word *)((C_word*)t0)[9])+1,t7);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21704,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2914: mapwalk */
t10=((C_word*)((C_word*)t0)[10])[1];
f_22811(t10,t9,((C_word*)t0)[11],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[12],((C_word*)t0)[13]);}
else{
t5=C_eqp(((C_word*)t0)[4],lf[634]);
if(C_truep(t5)){
t6=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[9])[1],C_fix(4));
t7=C_mutate(((C_word *)((C_word*)t0)[9])+1,t6);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21725,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2918: mapwalk */
t9=((C_word*)((C_word*)t0)[10])[1];
f_22811(t9,t8,((C_word*)t0)[11],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[12],((C_word*)t0)[13]);}
else{
t6=C_eqp(((C_word*)t0)[4],lf[119]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21742,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21746,a[2]=t7,tmp=(C_word)a,a+=3,tmp);
t9=((C_word*)((C_word*)t0)[5])[1];
/* core.scm:2921: chicken.compiler.support#estimate-foreign-result-size */
t10=*((C_word*)lf[510]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t8;
av2[2]=C_i_cadr(t9);
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}
else{
t7=C_eqp(((C_word*)t0)[4],lf[123]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_21771,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[13],tmp=(C_word)a,a+=12,tmp);
t9=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21775,a[2]=t8,tmp=(C_word)a,a+=3,tmp);
t10=((C_word*)((C_word*)t0)[5])[1];
/* core.scm:2925: chicken.compiler.support#estimate-foreign-result-size */
t11=*((C_word*)lf[510]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t9;
av2[2]=C_i_car(t10);
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t8=C_eqp(((C_word*)t0)[4],lf[617]);
if(C_truep(t8)){
t9=((C_word*)((C_word*)t0)[5])[1];
t10=C_i_car(t9);
t11=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)va26382,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[11],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[13],tmp=(C_word)a,a+=11,tmp);
t12=t11;
va26382(t12,C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[9])[1],t10));}
else{
t9=C_eqp(((C_word*)t0)[4],lf[616]);
if(C_truep(t9)){
t10=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[9])[1],C_fix(2));
t11=C_mutate(((C_word *)((C_word*)t0)[9])+1,t10);
t12=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21821,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2934: walk */
t13=((C_word*)((C_word*)t0)[16])[1];
f_21601(t13,t12,C_i_car(((C_word*)t0)[11]),((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[12],((C_word*)t0)[13]);}
else{
t10=C_eqp(((C_word*)t0)[4],lf[624]);
if(C_truep(t10)){
t11=C_i_car(((C_word*)t0)[11]);
t12=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21837,a[2]=((C_word*)t0)[17],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[13],a[6]=t11,tmp=(C_word)a,a+=7,tmp);
/* core.scm:2938: mapwalk */
t13=((C_word*)((C_word*)t0)[10])[1];
f_22811(t13,t12,((C_word*)t0)[11],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[12],((C_word*)t0)[13]);}
else{
t11=C_eqp(((C_word*)t0)[4],lf[163]);
if(C_truep(t11)){
t12=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21876,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t13=((C_word*)((C_word*)t0)[5])[1];
/* core.scm:2950: literal */
t14=((C_word*)((C_word*)t0)[18])[1];
f_22862(t14,t12,C_i_car(t13));}
else{
t12=C_eqp(((C_word*)t0)[4],lf[97]);
t13=(C_truep(t12)?t12:C_eqp(((C_word*)t0)[4],lf[545]));
if(C_truep(t13)){
t14=((C_word*)((C_word*)t0)[19])[1];
t15=((C_word*)((C_word*)t0)[20])[1];
t16=((C_word*)((C_word*)t0)[21])[1];
t17=((C_word*)((C_word*)t0)[22])[1];
t18=((C_word*)((C_word*)t0)[9])[1];
t19=C_eqp(((C_word*)t0)[4],lf[545]);
t20=C_set_block_item(((C_word*)t0)[19],0,C_fix(0));
t21=C_set_block_item(((C_word*)t0)[20],0,C_SCHEME_END_OF_LIST);
t22=C_set_block_item(((C_word*)t0)[9],0,C_fix(0));
t23=C_set_block_item(((C_word*)t0)[21],0,C_SCHEME_END_OF_LIST);
t24=C_set_block_item(((C_word*)t0)[22],0,C_fix(0));
t25=((C_word*)((C_word*)t0)[5])[1];
t26=C_i_caddr(t25);
t27=(*a=C_CLOSURE_TYPE|17,a[1]=(C_word)f_21917,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[22],a[4]=t17,a[5]=((C_word*)t0)[19],a[6]=t14,a[7]=((C_word*)t0)[20],a[8]=t15,a[9]=((C_word*)t0)[9],a[10]=t18,a[11]=((C_word*)t0)[21],a[12]=t16,a[13]=t19,a[14]=((C_word*)t0)[23],a[15]=((C_word*)t0)[24],a[16]=((C_word*)t0)[11],a[17]=((C_word*)t0)[16],tmp=(C_word)a,a+=18,tmp);
/* core.scm:2966: ##sys#decompose-lambda-list */
t28=*((C_word*)lf[191]+1);{
C_word av2[4];
av2[0]=t28;
av2[1]=((C_word*)t0)[2];
av2[2]=t26;
av2[3]=t27;
((C_proc)(void*)(*((C_word*)t28+1)))(4,av2);}}
else{
t14=C_eqp(((C_word*)t0)[4],lf[174]);
if(C_truep(t14)){
t15=((C_word*)((C_word*)t0)[5])[1];
t16=C_i_car(t15);
t17=C_i_car(((C_word*)t0)[11]);
t18=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_22149,a[2]=t16,a[3]=((C_word*)t0)[19],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[16],a[8]=((C_word*)t0)[12],a[9]=((C_word*)t0)[13],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[5],a[12]=t17,tmp=(C_word)a,a+=13,tmp);
/* core.scm:3025: chicken.compiler.support#node-class */
t19=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t19;
av2[1]=t18;
av2[2]=t17;
((C_proc)(void*)(*((C_word*)t19+1)))(3,av2);}}
else{
t15=C_eqp(((C_word*)t0)[4],lf[633]);
if(C_truep(t15)){
t16=((C_word*)((C_word*)t0)[5])[1];
t17=C_i_car(t16);
t18=C_i_car(((C_word*)t0)[11]);
t19=C_a_i_cons(&a,2,t17,((C_word*)((C_word*)t0)[20])[1]);
t20=C_mutate(((C_word *)((C_word*)t0)[20])+1,t19);
t21=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22176,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[16],a[5]=((C_word*)t0)[11],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[12],a[9]=((C_word*)t0)[13],tmp=(C_word)a,a+=10,tmp);
/* core.scm:3040: walk */
t22=((C_word*)((C_word*)t0)[16])[1];
f_21601(t22,t21,t18,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[12],((C_word*)t0)[13]);}
else{
t16=C_eqp(((C_word*)t0)[4],lf[284]);
if(C_truep(t16)){
t17=((C_word*)((C_word*)t0)[5])[1];
t18=C_i_car(t17);
t19=C_i_car(((C_word*)t0)[11]);
t20=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_22199,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[16],a[4]=t19,a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[12],a[7]=((C_word*)t0)[13],a[8]=((C_word*)t0)[2],a[9]=t18,a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[25],a[12]=((C_word*)t0)[18],a[13]=((C_word*)t0)[26],a[14]=((C_word*)t0)[5],tmp=(C_word)a,a+=15,tmp);
/* core.scm:3046: posq */
f_7518(t20,t18,((C_word*)t0)[7]);}
else{
t17=C_eqp(((C_word*)t0)[4],lf[520]);
if(C_truep(t17)){
t18=C_i_cdr(((C_word*)t0)[11]);
t19=C_i_length(t18);
t20=C_i_cdr(((C_word*)((C_word*)t0)[5])[1]);
t21=C_i_pairp(t20);
t22=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_22332,a[2]=((C_word*)t0)[21],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[11],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[14],a[13]=((C_word*)t0)[15],a[14]=((C_word*)t0)[22],a[15]=t19,tmp=(C_word)a,a+=16,tmp);
if(C_truep(t21)){
t23=((C_word*)((C_word*)t0)[5])[1];
t24=t22;
f_22332(t24,C_i_cadr(t23));}
else{
t23=t22;
f_22332(t23,C_SCHEME_FALSE);}}
else{
t18=C_eqp(((C_word*)t0)[4],lf[538]);
if(C_truep(t18)){
t19=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22417,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[11],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[13],tmp=(C_word)a,a+=11,tmp);
t20=((C_word*)((C_word*)t0)[5])[1];
if(C_truep(C_i_car(t20))){
t21=((C_word*)((C_word*)t0)[22])[1];
t22=C_mutate(((C_word *)((C_word*)t0)[22])+1,C_s_a_i_plus(&a,2,t21,C_fix(1)));
t23=t19;
f_22417(t23,t22);}
else{
t21=t19;
f_22417(t21,C_SCHEME_UNDEFINED);}}
else{
t19=C_eqp(((C_word*)t0)[4],lf[118]);
if(C_truep(t19)){
t20=((C_word*)((C_word*)t0)[5])[1];
t21=C_i_car(t20);
t22=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22444,a[2]=((C_word*)t0)[27],a[3]=((C_word*)t0)[2],a[4]=t21,a[5]=((C_word*)t0)[18],tmp=(C_word)a,a+=6,tmp);
if(C_truep(C_fixnump(t21))){
t23=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22553,a[2]=t22,tmp=(C_word)a,a+=3,tmp);
/* core.scm:3091: chicken.compiler.support#big-fixnum? */
t24=*((C_word*)lf[695]+1);{
C_word av2[3];
av2[0]=t24;
av2[1]=t23;
av2[2]=t21;
((C_proc)(void*)(*((C_word*)t24+1)))(3,av2);}}
else{
t23=t22;
f_22444(t23,C_SCHEME_FALSE);}}
else{
t20=C_eqp(((C_word*)t0)[4],lf[143]);
t21=(C_truep(t20)?t20:C_eqp(((C_word*)t0)[4],lf[630]));
if(C_truep(t21)){
t22=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_22565,a[2]=((C_word*)t0)[19],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[16],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
/* core.scm:3107: walk */
t23=((C_word*)((C_word*)t0)[16])[1];
f_21601(t23,t22,C_i_car(((C_word*)t0)[11]),((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[12],((C_word*)t0)[13]);}
else{
t22=C_eqp(((C_word*)t0)[4],lf[629]);
if(C_truep(t22)){
t23=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_22631,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[11],a[7]=((C_word*)t0)[16],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[13],tmp=(C_word)a,a+=12,tmp);
/* core.scm:3119: walk */
t24=((C_word*)((C_word*)t0)[16])[1];
f_21601(t24,t23,C_i_car(((C_word*)t0)[11]),((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[12],((C_word*)t0)[13]);}
else{
t23=C_eqp(((C_word*)t0)[4],lf[188]);
if(C_truep(t23)){
t24=((C_word*)((C_word*)t0)[14])[1];
t25=C_a_i_cons(&a,2,t24,((C_word*)((C_word*)t0)[5])[1]);
t26=C_a_i_cons(&a,2,t25,((C_word*)((C_word*)t0)[15])[1]);
t27=C_mutate(((C_word *)((C_word*)t0)[15])+1,t26);
t28=((C_word*)((C_word*)t0)[14])[1];
t29=C_mutate(((C_word *)((C_word*)t0)[14])+1,C_s_a_i_plus(&a,2,t28,C_fix(1)));
/* core.scm:3143: chicken.compiler.support#make-node */
t30=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t30;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[4];
av2[3]=t25;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t30+1)))(5,av2);}}
else{
t24=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22744,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* core.scm:3145: mapwalk */
t25=((C_word*)((C_word*)t0)[10])[1];
f_22811(t25,t24,((C_word*)t0)[11],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[12],((C_word*)t0)[13]);}}}}}}}}}}}}}}}}}}}}}}

/* k21646 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21648(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(34,0,6)))){
C_save_and_reclaim_args((void *)trf_21648,2,t0,t1);}
a=C_alloc(34);
t2=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[2])[1],((C_word*)t0)[3]);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21659,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2910: mapwalk */
t5=((C_word*)((C_word*)t0)[7])[1];
f_22811(t5,t4,((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],((C_word*)t0)[12]);}

/* k21657 in k21646 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21659(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21659,c,av);}
/* core.scm:2910: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k21677 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21679(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_21679,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_21683,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:2904: chicken.compiler.support#source-info->name */
t3=*((C_word*)lf[679]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k21681 in k21677 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(47,c,1)))){
C_save_and_reclaim((void *)f_21683,c,av);}
a=C_alloc(47);
t2=C_a_i_list4(&a,4,((C_word*)((C_word*)t0)[2])[1],lf[678],((C_word*)t0)[3],t1);
t3=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[2])[1],((C_word*)((C_word*)t0)[4])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[4])+1,t3);
t5=C_a_i_cons(&a,2,t2,((C_word*)((C_word*)t0)[5])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[5])+1,t5);
t7=((C_word*)((C_word*)t0)[2])[1];
t8=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t7,C_fix(1)));
t9=((C_word*)t0)[6];
f_21648(t9,t8);}

/* k21702 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21704(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21704,c,av);}
/* core.scm:2914: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k21723 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21725,c,av);}
/* core.scm:2918: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k21740 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,4)))){
C_save_and_reclaim((void *)f_21742,c,av);}
a=C_alloc(29);
t2=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[2])[1],t1);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
/* core.scm:2922: chicken.compiler.support#make-node */
t4=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)((C_word*)t0)[5])[1];
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k21744 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21746(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21746,c,av);}
/* core.scm:2921: chicken.compiler.support#bytes->words */
t2=*((C_word*)lf[340]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k21765 in k21769 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21767(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21767,c,av);}
/* core.scm:2926: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k21769 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21771(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(34,c,6)))){
C_save_and_reclaim((void *)f_21771,c,av);}
a=C_alloc(34);
t2=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[2])[1],t1);
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21767,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2926: mapwalk */
t5=((C_word*)((C_word*)t0)[6])[1];
f_22811(t5,t4,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11]);}

/* k21773 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21775(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_21775,c,av);}
/* core.scm:2925: chicken.compiler.support#bytes->words */
t2=*((C_word*)lf[340]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k21794 */
static void C_ccall f_21796(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_21796,c,av);}
/* core.scm:2930: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[617];
av2[3]=((C_word*)((C_word*)t0)[3])[1];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k21819 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21821(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_21821,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:2934: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[616];
av2[3]=((C_word*)((C_word*)t0)[3])[1];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k21835 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_21837,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_21847,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21867,a[2]=t2,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],tmp=(C_word)a,a+=5,tmp);
/* core.scm:2940: chicken.compiler.support#node-class */
t4=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k21845 in k21835 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_21847(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,4)))){
C_save_and_reclaim_args((void *)trf_21847,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t2,C_fix(1)));
/* core.scm:2939: chicken.compiler.support#make-node */
t4=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[623];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
/* core.scm:2939: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* k21861 in k21865 in k21835 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21863(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21863,c,av);}
t2=((C_word*)t0)[2];
f_21847(t2,C_i_memq(C_i_car(t1),((C_word*)t0)[3]));}

/* k21865 in k21835 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21867(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_21867,c,av);}
a=C_alloc(4);
t2=C_eqp(lf[523],t1);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21863,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2941: chicken.compiler.support#node-parameters */
t4=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=((C_word*)t0)[2];
f_21847(t3,C_SCHEME_FALSE);}}

/* k21874 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21876(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,4)))){
C_save_and_reclaim((void *)f_21876,c,av);}
a=C_alloc(32);
t2=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[2])[1],C_fix(8));
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t4=C_a_i_list1(&a,1,t1);
/* core.scm:2952: chicken.compiler.support#make-node */
t5=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=t4;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,4)))){
C_save_and_reclaim((void *)f_21917,c,av);}
a=C_alloc(28);
t5=((C_word*)((C_word*)t0)[2])[1];
t6=C_i_car(t5);
t7=(*a=C_CLOSURE_TYPE|22,a[1]=(C_word)f_21924,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[2],a[12]=t1,a[13]=((C_word*)t0)[12],a[14]=t3,a[15]=((C_word*)t0)[13],a[16]=t6,a[17]=t2,a[18]=t4,a[19]=((C_word*)t0)[14],a[20]=((C_word*)t0)[15],a[21]=((C_word*)t0)[16],a[22]=((C_word*)t0)[17],tmp=(C_word)a,a+=23,tmp);
if(C_truep(t4)){
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22052,a[2]=t7,a[3]=((C_word*)t0)[15],a[4]=t4,tmp=(C_word)a,a+=5,tmp);
/* core.scm:2972: chicken.compiler.support#db-get */
t9=*((C_word*)lf[537]+1);{
C_word *av2=av;
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[15];
av2[3]=t4;
av2[4]=lf[576];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}
else{
t8=t7;{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_SCHEME_FALSE;
f_21924(2,av2);}}}

/* k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21924(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,c,2)))){
C_save_and_reclaim((void *)f_21924,c,av);}
a=C_alloc(33);
t2=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_21927,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=t1,a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],tmp=(C_word)a,a+=22,tmp);
t3=C_i_car(((C_word*)t0)[21]);
t4=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22022,a[2]=t1,a[3]=((C_word*)t0)[17],a[4]=((C_word*)t0)[22],a[5]=t2,a[6]=t3,a[7]=((C_word*)t0)[16],tmp=(C_word)a,a+=8,tmp);
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22043,a[2]=t4,tmp=(C_word)a,a+=3,tmp);
t6=C_eqp(lf[683],t1);
if(C_truep(t6)){
/* core.scm:2980: chicken.base#butlast */
t7=*((C_word*)lf[686]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=((C_word*)t0)[17];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}
else{
/* core.scm:2979: ##sys#fast-reverse */
t7=*((C_word*)lf[685]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t7;
av2[1]=t4;
av2[2]=((C_word*)t0)[17];
((C_proc)(void*)(*((C_word*)t7+1)))(3,av2);}}}

/* k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21927(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,5)))){
C_save_and_reclaim((void *)f_21927,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|22,a[1]=(C_word)f_21930,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=t1,a[21]=((C_word*)t0)[20],a[22]=((C_word*)t0)[21],tmp=(C_word)a,a+=23,tmp);
t3=C_eqp(((C_word*)t0)[19],lf[683]);
if(C_truep(t3)){
/* core.scm:2988: chicken.compiler.support#debugging */
t4=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[640];
av2[3]=lf[684];
av2[4]=((C_word*)t0)[18];
av2[5]=((C_word*)t0)[16];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_21930(2,av2);}}}

/* k21928 in k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_21930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,5)))){
C_save_and_reclaim((void *)f_21930,c,av);}
a=C_alloc(23);
t2=(*a=C_CLOSURE_TYPE|22,a[1]=(C_word)f_21933,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],a[22]=((C_word*)t0)[22],tmp=(C_word)a,a+=23,tmp);
t3=(C_truep(((C_word*)t0)[15])?((C_word*)t0)[18]:C_SCHEME_FALSE);
if(C_truep(t3)){
/* core.scm:2990: chicken.compiler.support#bomb */
t4=*((C_word*)lf[530]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[682];
av2[3]=((C_word*)t0)[16];
av2[4]=((C_word*)((C_word*)t0)[8])[1];
av2[5]=((C_word*)t0)[18];
((C_proc)(void*)(*((C_word*)t4+1)))(6,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_21933(2,av2);}}}

/* k21931 in k21928 in k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_21933(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(63,c,4)))){
C_save_and_reclaim((void *)f_21933,c,av);}
a=C_alloc(63);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_21936,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],tmp=(C_word)a,a+=15,tmp);
t3=((C_word*)((C_word*)t0)[11])[1];
t4=C_i_cadr(t3);
t5=((C_word*)((C_word*)t0)[4])[1];
t6=C_s_a_i_plus(&a,2,t5,C_fix(1));
t7=(C_truep(((C_word*)t0)[15])?((C_word*)t0)[15]:C_i_memq(((C_word*)t0)[16],lf[68]));
t8=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_21973,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[8],a[5]=((C_word*)t0)[16],a[6]=t4,a[7]=((C_word*)t0)[17],a[8]=((C_word*)t0)[14],a[9]=((C_word*)t0)[18],a[10]=t6,a[11]=t7,a[12]=((C_word*)t0)[19],a[13]=((C_word*)t0)[20],a[14]=((C_word*)t0)[15],a[15]=t2,a[16]=((C_word*)t0)[21],a[17]=((C_word*)t0)[22],a[18]=((C_word*)t0)[2],tmp=(C_word)a,a+=19,tmp);
/* core.scm:3005: chicken.compiler.support#db-get */
t9=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=((C_word*)t0)[22];
av2[3]=((C_word*)t0)[16];
av2[4]=lf[610];
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* k21934 in k21931 in k21928 in k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_21936(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_21936,c,av);}
a=C_alloc(8);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=C_mutate(((C_word *)((C_word*)t0)[4])+1,((C_word*)t0)[5]);
t4=C_mutate(((C_word *)((C_word*)t0)[6])+1,((C_word*)t0)[7]);
t5=C_mutate(((C_word *)((C_word*)t0)[8])+1,((C_word*)t0)[9]);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_21944,a[2]=((C_word*)t0)[10],a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[12],tmp=(C_word)a,a+=5,tmp);
/* core.scm:3019: lset-adjoin/eq? */
f_7134(t6,((C_word*)t0)[13],C_a_i_list(&a,1,((C_word*)t0)[14]));}

/* k21942 in k21934 in k21931 in k21928 in k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_21944(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_21944,c,av);}
a=C_alloc(3);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=((C_word*)((C_word*)t0)[3])[1];
t4=C_i_car(t3);
t5=C_a_i_list1(&a,1,t4);
/* core.scm:3020: chicken.compiler.support#make-node */
t6=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[4];
av2[2]=lf[561];
av2[3]=t5;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k21971 in k21931 in k21928 in k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_21973(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,5)))){
C_save_and_reclaim((void *)f_21973,c,av);}
a=C_alloc(22);
t2=(C_truep(t1)?t1:C_fix(0));
t3=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_21980,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t2,a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
if(C_truep(C_i_not(((C_word*)t0)[9]))){
if(C_truep(C_i_greaterp(((C_word*)((C_word*)t0)[18])[1],C_fix(0)))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_21999,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* core.scm:3009: chicken.compiler.support#debugging */
t5=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[640];
av2[3]=lf[681];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)((C_word*)t0)[18])[1];
((C_proc)(void*)(*((C_word*)t5+1)))(6,av2);}}
else{
t4=t3;
f_21980(t4,C_SCHEME_FALSE);}}
else{
t4=t3;
f_21980(t4,C_SCHEME_FALSE);}}

/* k21978 in k21971 in k21931 in k21928 in k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_21980(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,4)))){
C_save_and_reclaim_args((void *)trf_21980,2,t0,t1);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|18,a[1]=(C_word)f_21984,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=t1,a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],tmp=(C_word)a,a+=19,tmp);
if(C_truep(((C_word*)t0)[15])){
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=((C_word*)t0)[15];
f_21984(2,av2);}}
else{
/* core.scm:3011: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word av2[5];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[18];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[605];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k21982 in k21978 in k21971 in k21931 in k21928 in k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_21984(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,4)))){
C_save_and_reclaim((void *)f_21984,c,av);}
a=C_alloc(18);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=((C_word*)((C_word*)t0)[3])[1];
t4=((C_word*)((C_word*)t0)[4])[1];
t5=C_a_i_record(&a,17,lf[646],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],t2,t3,t4,((C_word*)t0)[11],((C_word*)t0)[12],((C_word*)t0)[13],t1,((C_word*)t0)[14],((C_word*)t0)[15],((C_word*)t0)[16]);
/* core.scm:2991: chicken.internal#hash-table-set! */
t6=*((C_word*)lf[110]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[17];
av2[2]=((C_word*)t0)[18];
av2[3]=((C_word*)t0)[5];
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}

/* k21997 in k21971 in k21931 in k21928 in k21925 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_21999(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_21999,c,av);}
t2=((C_word*)t0)[2];
f_21980(t2,C_SCHEME_TRUE);}

/* k22020 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22022(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_22022,c,av);}
t2=C_eqp(lf[683],((C_word*)t0)[2]);
if(C_truep(t2)){
t3=C_i_length(((C_word*)t0)[3]);
/* core.scm:2977: walk */
t4=((C_word*)((C_word*)t0)[4])[1];
f_21601(t4,((C_word*)t0)[5],((C_word*)t0)[6],t1,C_fixnum_difference(t3,C_fix(1)),((C_word*)t0)[7],C_SCHEME_END_OF_LIST);}
else{
/* core.scm:2977: walk */
t3=((C_word*)((C_word*)t0)[4])[1];
f_21601(t3,((C_word*)t0)[5],((C_word*)t0)[6],t1,C_i_length(((C_word*)t0)[3]),((C_word*)t0)[7],C_SCHEME_END_OF_LIST);}}

/* k22041 in k21922 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22043,c,av);}
/* core.scm:2979: ##sys#fast-reverse */
t2=*((C_word*)lf[685]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22050 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22052(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22052,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22058,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2973: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[553];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22056 in k22050 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22058(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22058,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[550];
f_21924(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22083,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:2974: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[622];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k22081 in k22056 in k22050 in a21916 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22083(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22083,c,av);}
if(C_truep(C_i_not(t1))){
t2=C_i_not(((C_word*)t0)[2]);
if(C_truep(t2)){
if(C_truep(t2)){
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[683];
f_21924(2,av2);}}
else{
/* core.scm:2976: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[549];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}
else{
if(C_truep(C_i_nullp(((C_word*)t0)[2]))){
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=lf[683];
f_21924(2,av2);}}
else{
/* core.scm:2976: chicken.compiler.support#db-get */
t3=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[549];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}}
else{
/* core.scm:2976: chicken.compiler.support#db-get */
t2=*((C_word*)lf[537]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)t0)[5];
av2[4]=lf[549];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}}

/* k22113 in k22147 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22115(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_22115,c,av);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22119,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t3=C_i_cadr(((C_word*)t0)[4]);
t4=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_22127,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=t2,a[5]=t3,a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp);
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22139,a[2]=t4,a[3]=((C_word*)t0)[10],tmp=(C_word)a,a+=4,tmp);
/* core.scm:3031: ##sys#fast-reverse */
t6=*((C_word*)lf[685]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)((C_word*)t0)[11])[1];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}

/* k22117 in k22113 in k22147 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22119(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22119,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:3027: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[687];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22125 in k22113 in k22147 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22127(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,3)))){
C_save_and_reclaim((void *)f_22127,c,av);}
a=C_alloc(8);
t2=C_fixnum_plus(((C_word*)t0)[2],C_fix(1));
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22135,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t1,a[6]=t2,a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
/* core.scm:3032: scheme#append */
t4=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k22133 in k22125 in k22113 in k22147 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22135(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_22135,c,av);}
/* core.scm:3030: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_21601(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],t1);}

/* k22137 in k22113 in k22147 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22139(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_22139,c,av);}
/* core.scm:3031: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k22147 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(47,c,6)))){
C_save_and_reclaim((void *)f_22149,c,av);}
a=C_alloc(47);
t2=C_eqp(lf[616],t1);
t3=(C_truep(t2)?C_a_i_list1(&a,1,((C_word*)t0)[2]):C_SCHEME_END_OF_LIST);
t4=((C_word*)((C_word*)t0)[3])[1];
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,C_s_a_i_plus(&a,2,t4,C_fix(1)));
t6=C_a_i_list1(&a,1,C_fix(1));
t7=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_22115,a[2]=((C_word*)t0)[4],a[3]=t6,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t3,a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* core.scm:3029: walk */
t8=((C_word*)((C_word*)t0)[7])[1];
f_21601(t8,t7,((C_word*)t0)[12],((C_word*)t0)[10],((C_word*)t0)[6],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k22174 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22176(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,6)))){
C_save_and_reclaim((void *)f_22176,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22180,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:3041: walk */
t3=((C_word*)((C_word*)t0)[4])[1];
f_21601(t3,t2,C_i_cadr(((C_word*)t0)[5]),((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k22178 in k22174 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22180(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_22180,c,av);}
a=C_alloc(6);
t2=C_a_i_list2(&a,2,((C_word*)t0)[2],t1);
/* core.scm:3038: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[633];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22199(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,3)))){
C_save_and_reclaim((void *)f_22199,c,av);}
a=C_alloc(15);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_22203,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
/* core.scm:3046: g5107 */
t3=t2;
f_22203(t3,((C_word*)t0)[8],t1);}
else{
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_22233,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[8],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],a[14]=((C_word*)t0)[4],tmp=(C_word)a,a+=15,tmp);
/* core.scm:3052: chicken.compiler.support#node-class */
t3=*((C_word*)lf[98]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* g5107 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22203(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,6)))){
C_save_and_reclaim_args((void *)trf_22203,3,t0,t1,t2);}
a=C_alloc(7);
t3=C_fixnum_plus(t2,C_fix(1));
t4=C_fixnum_difference(((C_word*)t0)[2],t3);
t5=C_a_i_list1(&a,1,t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22219,a[2]=t1,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* core.scm:3050: walk */
t7=((C_word*)((C_word*)t0)[3])[1];
f_21601(t7,t6,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[2],((C_word*)t0)[6],((C_word*)t0)[7]);}

/* k22217 in g5107 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22219(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_22219,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:3048: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[688];
av2[3]=((C_word*)t0)[3];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22231 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22233(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,3)))){
C_save_and_reclaim((void *)f_22233,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_22317,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=t1,a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* core.scm:3053: chicken.compiler.support#variable-visible? */
t3=*((C_word*)lf[571]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[2];
av2[3]=*((C_word*)lf[25]+1);
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k22237 in k22315 in k22231 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22239(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,2)))){
C_save_and_reclaim((void *)f_22239,c,av);}
a=C_alloc(14);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_22242,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
if(C_truep(t1)){
t3=t2;
f_22242(t3,t1);}
else{
t3=C_i_cdr(((C_word*)((C_word*)t0)[14])[1]);
if(C_truep(C_i_pairp(t3))){
t4=((C_word*)((C_word*)t0)[14])[1];
t5=C_i_cadr(t4);
t6=t2;
f_22242(t6,(C_truep(t5)?t5:C_eqp(lf[475],((C_word*)t0)[15])));}
else{
t4=t2;
f_22242(t4,C_eqp(lf[475],((C_word*)t0)[15]));}}}

/* k22240 in k22237 in k22315 in k22231 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_22242(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(43,0,2)))){
C_save_and_reclaim_args((void *)trf_22242,2,t0,t1);}
a=C_alloc(43);
t2=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_22245,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],tmp=(C_word)a,a+=14,tmp);
if(C_truep(((C_word*)t0)[2])){
t3=((C_word*)((C_word*)t0)[13])[1];
t4=C_mutate(((C_word *)((C_word*)t0)[13])+1,C_s_a_i_plus(&a,2,t3,C_fix(1)));
t5=t2;
f_22245(t5,t4);}
else{
t3=t2;
f_22245(t3,C_SCHEME_UNDEFINED);}}

/* k22243 in k22240 in k22237 in k22315 in k22231 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_22245(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_22245,2,t0,t1);}
a=C_alloc(12);
t2=(C_truep(((C_word*)t0)[2])?lf[689]:lf[690]);
t3=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_22272,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
if(C_truep(((C_word*)t0)[3])){
/* core.scm:3062: blockvar-literal */
t4=((C_word*)((C_word*)t0)[12])[1];
f_22919(t4,t3,((C_word*)t0)[4]);}
else{
/* core.scm:3063: literal */
t4=((C_word*)((C_word*)t0)[13])[1];
f_22862(t4,t3,((C_word*)t0)[4]);}}

/* k22262 in k22270 in k22243 in k22240 in k22237 in k22315 in k22231 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_22264(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_22264,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:3059: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22270 in k22243 in k22240 in k22237 in k22315 in k22231 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_22272(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,6)))){
C_save_and_reclaim((void *)f_22272,c,av);}
a=C_alloc(14);
t2=C_a_i_list3(&a,3,t1,((C_word*)t0)[2],((C_word*)t0)[3]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22264,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:3066: walk */
t4=((C_word*)((C_word*)t0)[6])[1];
f_21601(t4,t3,C_i_car(((C_word*)t0)[7]),((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11]);}

/* k22311 in k22315 in k22231 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22313(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22313,c,av);}
/* core.scm:3055: chicken.compiler.support#immediate? */
t2=*((C_word*)lf[627]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=C_i_car(t1);
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22315 in k22231 in k22197 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22317(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_22317,c,av);}
a=C_alloc(19);
t2=C_i_not(t1);
t3=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_22239,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
t4=C_eqp(((C_word*)t0)[14],lf[118]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22313,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* core.scm:3055: chicken.compiler.support#node-parameters */
t6=*((C_word*)lf[524]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=((C_word*)t0)[15];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=t3;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_22239(2,av2);}}}

/* k22330 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22332(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,0,3)))){
C_save_and_reclaim_args((void *)trf_22332,2,t0,t1);}
a=C_alloc(19);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_22336,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=t1,a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* core.scm:3072: lset-adjoin/eq? */
f_7134(t2,((C_word*)((C_word*)t0)[2])[1],C_a_i_list(&a,1,((C_word*)t0)[15]));}

/* k22334 in k22330 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22336(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,2)))){
C_save_and_reclaim((void *)f_22336,c,av);}
a=C_alloc(18);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_22339,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=((C_word*)t0)[14],tmp=(C_word)a,a+=14,tmp);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22380,a[2]=((C_word*)t0)[15],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t5=C_i_length(((C_word*)((C_word*)t0)[5])[1]);
if(C_truep(C_fixnum_greater_or_equal_p(t5,C_fix(3)))){
t6=((C_word*)((C_word*)t0)[5])[1];
t7=t4;
f_22380(t7,C_eqp(((C_word*)t0)[10],C_i_caddr(t6)));}
else{
t6=t4;
f_22380(t6,C_SCHEME_FALSE);}}

/* k22337 in k22334 in k22330 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22339(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,0,2)))){
C_save_and_reclaim_args((void *)trf_22339,2,t0,t1);}
a=C_alloc(18);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_22342,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t3=(C_truep(*((C_word*)lf[24]+1))?((C_word*)t0)[11]:C_SCHEME_FALSE);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22369,a[2]=((C_word*)t0)[12],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[13],a[5]=t2,a[6]=((C_word*)t0)[11],tmp=(C_word)a,a+=7,tmp);
/* core.scm:3077: chicken.compiler.support#source-info->line */
t5=*((C_word*)lf[680]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_a_i_cons(&a,2,C_SCHEME_FALSE,((C_word*)((C_word*)t0)[4])[1]);
t5=C_mutate(((C_word *)((C_word*)t0)[4])+1,t4);
t6=t2;
f_22342(t6,t5);}}

/* k22340 in k22337 in k22334 in k22330 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22342(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,6)))){
C_save_and_reclaim_args((void *)trf_22342,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22349,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:3083: mapwalk */
t3=((C_word*)((C_word*)t0)[5])[1];
f_22811(t3,t2,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10]);}

/* k22347 in k22340 in k22337 in k22334 in k22330 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_22349(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22349,c,av);}
/* core.scm:3083: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22367 in k22337 in k22334 in k22330 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_22369,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22373,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:3078: chicken.compiler.support#source-info->name */
t3=*((C_word*)lf[679]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22371 in k22367 in k22337 in k22334 in k22330 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_22373(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(47,c,1)))){
C_save_and_reclaim((void *)f_22373,c,av);}
a=C_alloc(47);
t2=C_a_i_list4(&a,4,((C_word*)((C_word*)t0)[2])[1],lf[678],((C_word*)t0)[3],t1);
t3=C_a_i_cons(&a,2,((C_word*)((C_word*)t0)[2])[1],((C_word*)((C_word*)t0)[4])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[4])+1,t3);
t5=C_a_i_cons(&a,2,t2,((C_word*)((C_word*)t0)[5])[1]);
t6=C_mutate(((C_word *)((C_word*)t0)[5])+1,t5);
t7=((C_word*)((C_word*)t0)[2])[1];
t8=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t7,C_fix(1)));
t9=((C_word*)t0)[6];
f_22342(t9,t8);}

/* k22378 in k22334 in k22330 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22380(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,0,1)))){
C_save_and_reclaim_args((void *)trf_22380,2,t0,t1);}
a=C_alloc(29);
if(C_truep(t1)){
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t2,C_fix(1)));
t4=((C_word*)t0)[3];
f_22339(t4,t3);}
else{
t2=((C_word*)t0)[3];
f_22339(t2,C_SCHEME_UNDEFINED);}}

/* k22415 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22417(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,6)))){
C_save_and_reclaim_args((void *)trf_22417,2,t0,t1);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22424,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:3087: mapwalk */
t3=((C_word*)((C_word*)t0)[5])[1];
f_22811(t3,t2,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10]);}

/* k22422 in k22415 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22424,c,av);}
/* core.scm:3087: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22444(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_22444,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
/* core.scm:3092: immediate-literal */
f_22963(((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
if(C_truep(C_i_numberp(((C_word*)t0)[4]))){
t2=C_eqp(lf[377],*((C_word*)lf[376]+1));
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22465,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_integerp(((C_word*)t0)[4]))){
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22512,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* core.scm:3095: chicken.compiler.support#big-fixnum? */
t5=*((C_word*)lf[695]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=t3;
f_22465(t4,C_SCHEME_FALSE);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22523,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* core.scm:3102: literal */
t4=((C_word*)((C_word*)t0)[5])[1];
f_22862(t4,t3,((C_word*)t0)[4]);}}
else{
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22529,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
/* core.scm:3103: chicken.compiler.support#immediate? */
t3=*((C_word*)lf[627]+1);{
C_word av2[3];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}}

/* k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22465(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_22465,2,t0,t1);}
a=C_alloc(9);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22468,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22479,a[2]=t2,a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:3097: chicken.base#open-output-string */
t4=*((C_word*)lf[133]+1);{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:3101: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word av2[4];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[694];
av2[3]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k22466 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22468(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22468,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22475,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:3100: scheme#inexact->exact */
t3=*((C_word*)lf[691]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22473 in k22466 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22475,c,av);}
/* core.scm:3100: immediate-literal */
f_22963(((C_word*)t0)[3],t1);}

/* k22477 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22479,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22485,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:3097: ##sys#print */
t4=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[693];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k22483 in k22477 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22485(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22485,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22488,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:3097: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22486 in k22483 in k22477 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_22488(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_22488,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22491,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
/* core.scm:3097: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[692];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22489 in k22486 in k22483 in k22477 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_22491(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_22491,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22494,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22501,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:3099: scheme#inexact->exact */
t4=*((C_word*)lf[691]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k22492 in k22489 in k22486 in k22483 in k22477 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_22494(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_22494,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22497,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:3097: chicken.base#get-output-string */
t3=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k22495 in k22492 in k22489 in k22486 in k22483 in k22477 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_22497(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_22497,c,av);}
/* core.scm:3096: chicken.base#warning */
t2=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k22499 in k22489 in k22486 in k22483 in k22477 in k22463 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_22501(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22501,c,av);}
/* core.scm:3097: ##sys#print */
t2=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k22510 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22512(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22512,c,av);}
t2=((C_word*)t0)[2];
f_22465(t2,C_i_not(t1));}

/* k22521 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22523(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_22523,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:3102: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[676];
av2[3]=t2;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22527 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22529(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_22529,c,av);}
a=C_alloc(3);
if(C_truep(t1)){
/* core.scm:3103: immediate-literal */
f_22963(((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22543,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* core.scm:3104: literal */
t3=((C_word*)((C_word*)t0)[5])[1];
f_22862(t3,t2,((C_word*)t0)[4]);}}

/* k22541 in k22527 in k22442 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_22543,c,av);}
a=C_alloc(3);
t2=C_a_i_list1(&a,1,t1);
/* core.scm:3104: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[676];
av2[3]=t2;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k22551 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22553(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22553,c,av);}
t2=((C_word*)t0)[2];
f_22444(t2,C_i_not(t1));}

/* k22563 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22565(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,6)))){
C_save_and_reclaim((void *)f_22565,c,av);}
a=C_alloc(16);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=((C_word*)((C_word*)t0)[3])[1];
t4=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_22568,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t3,a[5]=t2,a[6]=t1,a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[7],a[11]=((C_word*)t0)[8],a[12]=((C_word*)t0)[9],a[13]=((C_word*)t0)[10],a[14]=((C_word*)t0)[11],a[15]=((C_word*)t0)[12],tmp=(C_word)a,a+=16,tmp);
/* core.scm:3110: walk */
t5=((C_word*)((C_word*)t0)[7])[1];
f_21601(t5,t4,C_i_cadr(((C_word*)t0)[8]),((C_word*)t0)[9],((C_word*)t0)[10],((C_word*)t0)[11],((C_word*)t0)[12]);}

/* k22566 in k22563 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22568(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,6)))){
C_save_and_reclaim((void *)f_22568,c,av);}
a=C_alloc(13);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=((C_word*)((C_word*)t0)[3])[1];
t4=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_22571,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],a[7]=t1,a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=t2,a[12]=t3,tmp=(C_word)a,a+=13,tmp);
/* core.scm:3113: walk */
t5=((C_word*)((C_word*)t0)[10])[1];
f_21601(t5,t4,C_i_caddr(((C_word*)t0)[11]),((C_word*)t0)[12],((C_word*)t0)[13],((C_word*)t0)[14],((C_word*)t0)[15]);}

/* k22569 in k22566 in k22563 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22571(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(71,c,3)))){
C_save_and_reclaim((void *)f_22571,c,av);}
a=C_alloc(71);
t2=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_22602,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],tmp=(C_word)a,a+=13,tmp);
t3=C_s_a_i_minus(&a,2,((C_word*)((C_word*)t0)[3])[1],((C_word*)t0)[12]);
t4=C_s_a_i_minus(&a,2,((C_word*)t0)[12],((C_word*)t0)[2]);
/* core.scm:3114: scheme#max */
t5=*((C_word*)lf[696]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=t2;
av2[2]=t3;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* k22588 in k22600 in k22569 in k22566 in k22563 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_22590(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(38,c,4)))){
C_save_and_reclaim((void *)f_22590,c,av);}
a=C_alloc(38);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],t1);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,t2);
t4=C_a_i_list3(&a,3,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);
/* core.scm:3116: chicken.compiler.support#make-node */
t5=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[7];
av2[2]=((C_word*)t0)[8];
av2[3]=((C_word*)((C_word*)t0)[9])[1];
av2[4]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}

/* k22600 in k22569 in k22566 in k22563 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22602(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(97,c,3)))){
C_save_and_reclaim((void *)f_22602,c,av);}
a=C_alloc(97);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],t1);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,t2);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22590,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],tmp=(C_word)a,a+=10,tmp);
t5=C_s_a_i_minus(&a,2,((C_word*)((C_word*)t0)[5])[1],((C_word*)t0)[12]);
t6=C_s_a_i_minus(&a,2,((C_word*)t0)[12],((C_word*)t0)[4]);
/* core.scm:3115: scheme#max */
t7=*((C_word*)lf[696]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t4;
av2[2]=t5;
av2[3]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22631(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,5)))){
C_save_and_reclaim((void *)f_22631,c,av);}
a=C_alloc(18);
t2=((C_word*)((C_word*)t0)[2])[1];
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22642,a[2]=t1,a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t4=((C_word*)((C_word*)t0)[5])[1];
t5=C_i_car(t4);
t6=C_i_cdr(((C_word*)t0)[6]);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22652,a[2]=((C_word*)t0)[2],a[3]=t2,a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[11],a[9]=t8,tmp=(C_word)a,a+=10,tmp));
t10=((C_word*)t8)[1];
f_22652(t10,t3,t5,t6,C_fix(0));}

/* k22640 in k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22642(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,4)))){
C_save_and_reclaim((void *)f_22642,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
/* core.scm:3121: chicken.compiler.support#make-node */
t3=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
av2[2]=((C_word*)t0)[4];
av2[3]=((C_word*)((C_word*)t0)[5])[1];
av2[4]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* loop in k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22652(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,0,6)))){
C_save_and_reclaim_args((void *)trf_22652,5,t0,t1,t2,t3,t4);}
a=C_alloc(14);
t5=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
if(C_truep(C_i_zerop(t2))){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22661,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[2],a[4]=t1,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
/* core.scm:3129: walk */
t7=((C_word*)((C_word*)t0)[4])[1];
f_21601(t7,t6,C_i_car(t3),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}
else{
t6=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_22683,a[2]=t1,a[3]=t2,a[4]=t3,a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[3],a[8]=t4,a[9]=((C_word*)t0)[4],a[10]=((C_word*)t0)[5],a[11]=((C_word*)t0)[6],a[12]=((C_word*)t0)[7],a[13]=((C_word*)t0)[8],tmp=(C_word)a,a+=14,tmp);
/* core.scm:3132: walk */
t7=((C_word*)((C_word*)t0)[4])[1];
f_21601(t7,t6,C_i_car(t3),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}}

/* k22659 in loop in k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(35,c,3)))){
C_save_and_reclaim((void *)f_22661,c,av);}
a=C_alloc(35);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22672,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,tmp=(C_word)a,a+=6,tmp);
t3=C_s_a_i_minus(&a,2,((C_word*)((C_word*)t0)[3])[1],((C_word*)t0)[2]);
/* core.scm:3130: scheme#max */
t4=*((C_word*)lf[696]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k22670 in k22659 in loop in k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22672(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,1)))){
C_save_and_reclaim((void *)f_22672,c,av);}
a=C_alloc(32);
t2=C_s_a_i_plus(&a,2,((C_word*)t0)[2],t1);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,t2);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list1(&a,1,((C_word*)t0)[5]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k22681 in loop in k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,6)))){
C_save_and_reclaim((void *)f_22683,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22686,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:3133: walk */
t3=((C_word*)((C_word*)t0)[9])[1];
f_21601(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[10],((C_word*)t0)[11],((C_word*)t0)[12],((C_word*)t0)[13]);}

/* k22684 in k22681 in loop in k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22686(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(69,c,3)))){
C_save_and_reclaim((void *)f_22686,c,av);}
a=C_alloc(69);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22693,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t3=C_s_a_i_minus(&a,2,((C_word*)t0)[4],C_fix(1));
t4=C_i_cddr(((C_word*)t0)[5]);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22703,a[2]=((C_word*)t0)[6],a[3]=t2,a[4]=t3,a[5]=t4,tmp=(C_word)a,a+=6,tmp);
t6=C_s_a_i_minus(&a,2,((C_word*)((C_word*)t0)[7])[1],((C_word*)t0)[8]);
/* core.scm:3136: scheme#max */
t7=*((C_word*)lf[696]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t5;
av2[2]=t6;
av2[3]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k22691 in k22684 in k22681 in loop in k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_22693(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_22693,c,av);}
a=C_alloc(6);
/* core.scm:3134: cons* */
f_6739(((C_word*)t0)[2],((C_word*)t0)[3],C_a_i_list(&a,2,((C_word*)t0)[4],t1));}

/* k22701 in k22684 in k22681 in loop in k22629 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_22703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22703,c,av);}
/* core.scm:3136: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_22652(t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],t1);}

/* k22742 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22744,c,av);}
/* core.scm:3145: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* mapwalk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22811(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6){
C_word tmp;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,0,3)))){
C_save_and_reclaim_args((void *)trf_22811,7,t0,t1,t2,t3,t4,t5,t6);}
a=C_alloc(20);
t7=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t8=t7;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=((C_word*)t9)[1];
t11=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_22816,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t4,a[5]=t5,a[6]=t6,tmp=(C_word)a,a+=7,tmp);
t12=C_i_check_list_2(t2,lf[10]);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22828,a[2]=t9,a[3]=t14,a[4]=t11,a[5]=t10,tmp=(C_word)a,a+=6,tmp));
t16=((C_word*)t14)[1];
f_22828(t16,t1,t2);}

/* g5205 in mapwalk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22816(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,6)))){
C_save_and_reclaim_args((void *)trf_22816,3,t0,t1,t2);}
/* core.scm:3148: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_21601(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6]);}

/* map-loop5199 in mapwalk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22828(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_22828,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22853,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:3148: g5205 */
t4=((C_word*)t0)[4];
f_22816(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k22851 in map-loop5199 in mapwalk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22853(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_22853,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_22828(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22862(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_22862,3,t0,t1,t2);}
a=C_alloc(10);
t3=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_22869,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* core.scm:3151: chicken.compiler.support#immediate? */
t4=*((C_word*)lf[627]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k22867 in literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(35,c,3)))){
C_save_and_reclaim((void *)f_22869,c,av);}
a=C_alloc(35);
if(C_truep(t1)){
/* core.scm:3151: immediate-literal */
f_22963(((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
t2=(C_truep(C_blockp(((C_word*)t0)[4]))?C_lambdainfop(((C_word*)t0)[4]):C_SCHEME_FALSE);
if(C_truep(t2)){
t3=((C_word*)((C_word*)t0)[5])[1];
t4=C_a_i_cons(&a,2,((C_word*)t0)[4],((C_word*)((C_word*)t0)[6])[1]);
t5=C_mutate(((C_word *)((C_word*)t0)[6])+1,t4);
t6=((C_word*)((C_word*)t0)[5])[1];
t7=C_mutate(((C_word *)((C_word*)t0)[5])+1,C_s_a_i_plus(&a,2,t6,C_fix(1)));
t8=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t8;
av2[1]=C_a_i_vector1(&a,1,t3);
((C_proc)(void*)(*((C_word*)t8+1)))(2,av2);}}
else{
t3=((C_word*)((C_word*)t0)[7])[1];
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7558,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t5=(
  f_7558(t4,C_fix(0),t3)
);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22894,a[2]=((C_word*)t0)[8],tmp=(C_word)a,a+=3,tmp);
/* core.scm:3151: g5235 */
t7=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t7;
av2[1]=(
/* core.scm:3151: g5235 */
  f_22894(t6,t5)
);
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}
else{
/* core.scm:3160: new-literal */
t6=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t6;
av2[1]=(
/* core.scm:3160: new-literal */
  f_22910(C_a_i(&a,32),((C_word*)((C_word*)t0)[9])[1],((C_word*)t0)[4])
);
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}

/* g5235 in k22867 in literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_22894(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_fixnum_difference(((C_word*)((C_word*)t0)[2])[1],C_fixnum_plus(t1,C_fix(1))));}

/* new-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_22910(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_stack_overflow_check;{}
t2=((C_word*)((C_word*)t0)[2])[1];
t3=C_a_i_cons(&a,2,t1,((C_word*)((C_word*)t0)[3])[1]);
t4=C_mutate(((C_word *)((C_word*)t0)[3])+1,t3);
t5=((C_word*)((C_word*)t0)[2])[1];
t6=C_mutate(((C_word *)((C_word*)t0)[2])+1,C_s_a_i_plus(&a,2,t5,C_fix(1)));
return(t2);}

/* blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22919(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,0,4)))){
C_save_and_reclaim_args((void *)trf_22919,3,t0,t1,t2);}
a=C_alloc(15);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_22923,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22947,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t5=((C_word*)((C_word*)t0)[4])[1];
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7106,a[2]=t7,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_7106(t9,t3,C_fix(0),t5);}

/* k22921 in blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22923(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22923,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22927,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:3169: g5251 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
/* core.scm:3169: g5251 */
  f_22927(t2,t1)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22945,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:3175: chicken.compiler.support#make-block-variable-literal */
t3=*((C_word*)lf[697]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}}

/* g5251 in k22921 in blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_22927(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_fixnum_difference(((C_word*)((C_word*)t0)[2])[1],C_fixnum_plus(t1,C_fix(1))));}

/* k22943 in k22921 in blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22945(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,1)))){
C_save_and_reclaim((void *)f_22945,c,av);}
a=C_alloc(32);
/* core.scm:3175: new-literal */
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(
/* core.scm:3175: new-literal */
  f_22910(C_a_i(&a,32),((C_word*)((C_word*)t0)[3])[1],t1)
);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a22946 in blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22947(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_22947,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_22954,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:3171: chicken.compiler.support#block-variable-literal? */
t4=*((C_word*)lf[699]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k22952 in a22946 in blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_22954,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_22961,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:3172: chicken.compiler.support#block-variable-literal-name */
t3=*((C_word*)lf[698]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k22959 in k22952 in a22946 in blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22961(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_22961,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_eqp(((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* immediate-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_22963(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,4)))){
C_save_and_reclaim_args((void *)trf_22963,2,t1,t2);}
a=C_alloc(9);
t3=*((C_word*)lf[700]+1);
t4=C_eqp(*((C_word*)lf[700]+1),t2);
if(C_truep(t4)){
/* core.scm:3179: chicken.compiler.support#make-node */
t5=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t5;
av2[1]=t1;
av2[2]=lf[475];
av2[3]=C_SCHEME_END_OF_LIST;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t5+1)))(5,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_22980,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
if(C_truep(C_fixnump(t2))){
t6=C_a_i_list(&a,2,lf[702],t2);
/* core.scm:3180: chicken.compiler.support#make-node */
t7=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[701];
av2[3]=t6;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
if(C_truep(C_booleanp(t2))){
t6=C_a_i_list(&a,2,lf[703],t2);
/* core.scm:3180: chicken.compiler.support#make-node */
t7=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t1;
av2[2]=lf[701];
av2[3]=t6;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
if(C_truep(C_charp(t2))){
t6=t5;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_a_i_list(&a,2,lf[704],t2);
f_22980(2,av2);}}
else{
if(C_truep(C_i_nullp(t2))){
/* core.scm:3180: chicken.compiler.support#make-node */
t6=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[701];
av2[3]=lf[705];
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
if(C_truep(C_eofp(t2))){
/* core.scm:3180: chicken.compiler.support#make-node */
t6=*((C_word*)lf[486]+1);{
C_word av2[5];
av2[0]=t6;
av2[1]=t1;
av2[2]=lf[701];
av2[3]=lf[706];
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
/* core.scm:3186: chicken.compiler.support#bomb */
t6=*((C_word*)lf[530]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=lf[707];
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}}}}}}

/* k22978 in immediate-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_22980(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_22980,c,av);}
/* core.scm:3180: chicken.compiler.support#make-node */
t2=*((C_word*)lf[486]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[701];
av2[3]=t1;
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k23024 in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_23026(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,6)))){
C_save_and_reclaim((void *)f_23026,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_23029,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],tmp=(C_word)a,a+=10,tmp);
/* core.scm:3190: walk */
t3=((C_word*)((C_word*)t0)[10])[1];
f_21601(t3,t2,((C_word*)t0)[11],C_SCHEME_END_OF_LIST,C_fix(0),C_SCHEME_FALSE,C_SCHEME_END_OF_LIST);}

/* k23027 in k23024 in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_23029(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_23029,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_23032,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t3=((C_word*)((C_word*)t0)[9])[1];
if(C_truep(C_i_positivep(t3))){
/* core.scm:3192: chicken.compiler.support#debugging */
t4=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[640];
av2[3]=lf[710];
av2[4]=((C_word*)((C_word*)t0)[9])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_23032(2,av2);}}}

/* k23030 in k23027 in k23024 in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_23032(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_23032,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_23035,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t3=((C_word*)((C_word*)t0)[9])[1];
if(C_truep(C_i_positivep(t3))){
/* core.scm:3194: chicken.compiler.support#debugging */
t4=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[640];
av2[3]=lf[709];
av2[4]=((C_word*)((C_word*)t0)[9])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_23035(2,av2);}}}

/* k23033 in k23030 in k23027 in k23024 in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_23035(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_23035,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_23038,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
t3=((C_word*)((C_word*)t0)[8])[1];
if(C_truep(C_i_positivep(t3))){
/* core.scm:3196: chicken.compiler.support#debugging */
t4=*((C_word*)lf[593]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t2;
av2[2]=lf[640];
av2[3]=lf[708];
av2[4]=((C_word*)((C_word*)t0)[8])[1];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
t4=t2;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_23038(2,av2);}}}

/* k23036 in k23033 in k23030 in k23027 in k23024 in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_23038(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_23038,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23045,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:3198: ##sys#fast-reverse */
t3=*((C_word*)lf[685]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[7])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k23043 in k23036 in k23033 in k23030 in k23027 in k23024 in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_23045(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_23045,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23049,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:3199: ##sys#fast-reverse */
t3=*((C_word*)lf[685]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[6])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k23047 in k23043 in k23036 in k23033 in k23030 in k23027 in k23024 in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_23049(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_23049,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_23053,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=t1,a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:3201: scheme#reverse */
t3=*((C_word*)lf[259]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[6])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k23051 in k23047 in k23043 in k23036 in k23033 in k23030 in k23027 in k23024 in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_23053(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_23053,c,av);}
/* core.scm:3197: scheme#values */{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=((C_word*)t0)[5];
av2[5]=((C_word*)t0)[6];
av2[6]=t1;
C_values(7,av2);}}

/* k6257 */
static void C_ccall f_6259(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6259,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6262,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_eval_toplevel(2,av2);}}

/* k6260 in k6257 */
static void C_ccall f_6262(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6262,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6265,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_extras_toplevel(2,av2);}}

/* k6263 in k6260 in k6257 */
static void C_ccall f_6265(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6265,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6268,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_data_2dstructures_toplevel(2,av2);}}

/* k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6268(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6268,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6271,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_scrutinizer_toplevel(2,av2);}}

/* k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6271(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6271,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6274,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_support_toplevel(2,av2);}}

/* k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(19,c,2)))){
C_save_and_reclaim((void *)f_6274,c,av);}
a=C_alloc(19);
t2=C_a_i_provide(&a,1,lf[0]);
t3=C_a_i_provide(&a,1,lf[1]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6277,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t4;
C_file_toplevel(2,av2);}}

/* k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6277(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6277,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6280,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_internal_toplevel(2,av2);}}

/* k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6280(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6280,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6283,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_pathname_toplevel(2,av2);}}

/* k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6283(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_6283,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6286,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_expand_toplevel(2,av2);}}

/* k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6286(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word t57;
C_word t58;
C_word t59;
C_word t60;
C_word t61;
C_word t62;
C_word t63;
C_word t64;
C_word t65;
C_word t66;
C_word t67;
C_word t68;
C_word t69;
C_word t70;
C_word t71;
C_word t72;
C_word t73;
C_word t74;
C_word t75;
C_word t76;
C_word t77;
C_word t78;
C_word t79;
C_word t80;
C_word t81;
C_word t82;
C_word t83;
C_word t84;
C_word t85;
C_word t86;
C_word t87;
C_word t88;
C_word t89;
C_word t90;
C_word t91;
C_word t92;
C_word t93;
C_word t94;
C_word t95;
C_word t96;
C_word t97;
C_word t98;
C_word t99;
C_word t100;
C_word t101;
C_word t102;
C_word t103;
C_word t104;
C_word t105;
C_word t106;
C_word t107;
C_word t108;
C_word t109;
C_word t110;
C_word t111;
C_word t112;
C_word t113;
C_word t114;
C_word t115;
C_word t116;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(88,c,9)))){
C_save_and_reclaim((void *)f_6286,c,av);}
a=C_alloc(88);
t2=C_mutate(&lf[2] /* (set! chicken.compiler.core#every ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6675,tmp=(C_word)a,a+=2,tmp));
t3=C_mutate(&lf[3] /* (set! chicken.compiler.core#cons* ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6739,tmp=(C_word)a,a+=2,tmp));
t4=C_mutate(&lf[4] /* (set! chicken.compiler.core#concatenate ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6767,tmp=(C_word)a,a+=2,tmp));
t5=C_mutate(&lf[6] /* (set! chicken.compiler.core#delete ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6795,tmp=(C_word)a,a+=2,tmp));
t6=C_mutate(&lf[7] /* (set! chicken.compiler.core#filter ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_6915,tmp=(C_word)a,a+=2,tmp));
t7=C_mutate(&lf[9] /* (set! chicken.compiler.core#unzip1 ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7024,tmp=(C_word)a,a+=2,tmp));
t8=C_mutate(&lf[11] /* (set! chicken.compiler.core#lset-adjoin/eq? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7134,tmp=(C_word)a,a+=2,tmp));
t9=C_mutate(&lf[12] /* (set! chicken.compiler.core#lset-difference/eq? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7175,tmp=(C_word)a,a+=2,tmp));
t10=C_mutate(&lf[13] /* (set! chicken.compiler.core#lset-union/eq? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7220,tmp=(C_word)a,a+=2,tmp));
t11=C_mutate(&lf[15] /* (set! chicken.compiler.core#lset-intersection/eq? ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7299,tmp=(C_word)a,a+=2,tmp));
t12=C_mutate(&lf[16] /* (set! chicken.compiler.core#posq ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7518,tmp=(C_word)a,a+=2,tmp));
t13=C_set_block_item(lf[17] /* chicken.compiler.core#unit-name */,0,C_SCHEME_FALSE);
t14=C_set_block_item(lf[18] /* chicken.compiler.core#standard-bindings */,0,C_SCHEME_END_OF_LIST);
t15=C_set_block_item(lf[19] /* chicken.compiler.core#extended-bindings */,0,C_SCHEME_END_OF_LIST);
t16=C_set_block_item(lf[20] /* chicken.compiler.core#insert-timer-checks */,0,C_SCHEME_TRUE);
t17=C_set_block_item(lf[21] /* chicken.compiler.core#used-units */,0,C_SCHEME_END_OF_LIST);
t18=C_set_block_item(lf[22] /* chicken.compiler.core#foreign-declarations */,0,C_SCHEME_END_OF_LIST);
t19=C_set_block_item(lf[23] /* chicken.compiler.core#emit-trace-info */,0,C_SCHEME_FALSE);
t20=C_set_block_item(lf[24] /* chicken.compiler.core#emit-debug-info */,0,C_SCHEME_FALSE);
t21=C_set_block_item(lf[25] /* chicken.compiler.core#block-compilation */,0,C_SCHEME_FALSE);
t22=C_set_block_item(lf[26] /* chicken.compiler.core#line-number-database-size */,0,C_fix(997));
t23=C_set_block_item(lf[27] /* chicken.compiler.core#target-heap-size */,0,C_SCHEME_FALSE);
t24=C_set_block_item(lf[28] /* chicken.compiler.core#target-stack-size */,0,C_SCHEME_FALSE);
t25=C_set_block_item(lf[29] /* chicken.compiler.core#optimize-leaf-routines */,0,C_SCHEME_FALSE);
t26=C_set_block_item(lf[30] /* chicken.compiler.core#emit-profile */,0,C_SCHEME_FALSE);
t27=C_set_block_item(lf[31] /* chicken.compiler.core#no-bound-checks */,0,C_SCHEME_FALSE);
t28=C_set_block_item(lf[32] /* chicken.compiler.core#no-argc-checks */,0,C_SCHEME_FALSE);
t29=C_set_block_item(lf[33] /* chicken.compiler.core#no-procedure-checks */,0,C_SCHEME_FALSE);
t30=C_set_block_item(lf[34] /* chicken.compiler.core#no-global-procedure-checks */,0,C_SCHEME_FALSE);
t31=lf[35] /* chicken.compiler.core#safe-globals-flag */ =C_SCHEME_FALSE;;
t32=C_set_block_item(lf[36] /* chicken.compiler.core#explicit-use-flag */,0,C_SCHEME_FALSE);
t33=C_set_block_item(lf[37] /* chicken.compiler.core#disable-stack-overflow-checking */,0,C_SCHEME_FALSE);
t34=C_set_block_item(lf[38] /* chicken.compiler.core#external-protos-first */,0,C_SCHEME_FALSE);
t35=C_set_block_item(lf[39] /* chicken.compiler.core#inline-max-size */,0,C_fix(20));
t36=C_set_block_item(lf[40] /* chicken.compiler.core#unroll-limit */,0,C_fix(1));
t37=C_set_block_item(lf[41] /* chicken.compiler.core#emit-closure-info */,0,C_SCHEME_TRUE);
t38=C_set_block_item(lf[42] /* chicken.compiler.core#undefine-shadowed-macros */,0,C_SCHEME_TRUE);
t39=C_set_block_item(lf[43] /* chicken.compiler.core#profiled-procedures */,0,C_SCHEME_FALSE);
t40=C_set_block_item(lf[44] /* chicken.compiler.core#import-libraries */,0,C_SCHEME_END_OF_LIST);
t41=C_set_block_item(lf[45] /* chicken.compiler.core#all-import-libraries */,0,C_SCHEME_FALSE);
t42=C_set_block_item(lf[46] /* chicken.compiler.core#preserve-unchanged-import-libraries */,0,C_SCHEME_TRUE);
t43=C_set_block_item(lf[47] /* chicken.compiler.core#compile-module-registration */,0,C_SCHEME_FALSE);
t44=C_set_block_item(lf[48] /* chicken.compiler.core#standalone-executable */,0,C_SCHEME_TRUE);
t45=C_set_block_item(lf[49] /* chicken.compiler.core#local-definitions */,0,C_SCHEME_FALSE);
t46=C_set_block_item(lf[50] /* chicken.compiler.core#inline-locally */,0,C_SCHEME_FALSE);
t47=C_set_block_item(lf[51] /* chicken.compiler.core#enable-inline-files */,0,C_SCHEME_FALSE);
t48=C_set_block_item(lf[52] /* chicken.compiler.core#compiler-syntax-enabled */,0,C_SCHEME_TRUE);
t49=C_set_block_item(lf[53] /* chicken.compiler.core#bootstrap-mode */,0,C_SCHEME_FALSE);
t50=C_set_block_item(lf[54] /* chicken.compiler.core#strict-variable-types */,0,C_SCHEME_FALSE);
t51=C_set_block_item(lf[55] /* chicken.compiler.core#enable-specialization */,0,C_SCHEME_FALSE);
t52=C_set_block_item(lf[56] /* chicken.compiler.core#static-extensions */,0,C_SCHEME_FALSE);
t53=C_set_block_item(lf[57] /* chicken.compiler.core#emit-link-file */,0,C_SCHEME_FALSE);
t54=C_set_block_item(lf[58] /* chicken.compiler.core#types-output-file */,0,C_SCHEME_FALSE);
t55=C_set_block_item(lf[59] /* chicken.compiler.core#verbose-mode */,0,C_SCHEME_FALSE);
t56=lf[60] /* chicken.compiler.core#original-program-size */ =C_SCHEME_FALSE;;
t57=lf[61] /* chicken.compiler.core#current-program-size */ =C_fix(0);;
t58=lf[62] /* chicken.compiler.core#current-analysis-database-size */ =C_fix(3001);;
t59=C_set_block_item(lf[63] /* chicken.compiler.core#line-number-database-2 */,0,C_SCHEME_FALSE);
t60=C_set_block_item(lf[64] /* chicken.compiler.core#immutable-constants */,0,C_SCHEME_END_OF_LIST);
t61=C_set_block_item(lf[65] /* chicken.compiler.core#inline-table */,0,C_SCHEME_FALSE);
t62=C_set_block_item(lf[66] /* chicken.compiler.core#constant-table */,0,C_SCHEME_FALSE);
t63=C_set_block_item(lf[67] /* chicken.compiler.core#inline-substitutions-enabled */,0,C_SCHEME_FALSE);
t64=lf[68] /* chicken.compiler.core#direct-call-ids */ =C_SCHEME_END_OF_LIST;;
t65=C_set_block_item(lf[69] /* chicken.compiler.core#first-analysis */,0,C_SCHEME_TRUE);
t66=lf[70] /* chicken.compiler.core#foreign-variables */ =C_SCHEME_END_OF_LIST;;
t67=C_set_block_item(lf[71] /* chicken.compiler.core#foreign-lambda-stubs */,0,C_SCHEME_END_OF_LIST);
t68=C_set_block_item(lf[72] /* chicken.compiler.core#external-variables */,0,C_SCHEME_END_OF_LIST);
t69=lf[73] /* chicken.compiler.core#external-to-pointer */ =C_SCHEME_END_OF_LIST;;
t70=lf[74] /* chicken.compiler.core#location-pointer-map */ =C_SCHEME_END_OF_LIST;;
t71=lf[75] /* chicken.compiler.core#pending-canonicalizations */ =C_SCHEME_END_OF_LIST;;
t72=lf[76] /* chicken.compiler.core#defconstant-bindings */ =C_SCHEME_END_OF_LIST;;
t73=lf[77] /* chicken.compiler.core#callback-names */ =C_SCHEME_END_OF_LIST;;
t74=lf[78] /* chicken.compiler.core#toplevel-scope */ =C_SCHEME_TRUE;;
t75=lf[79] /* chicken.compiler.core#toplevel-lambda-id */ =C_SCHEME_FALSE;;
t76=C_set_block_item(lf[80] /* chicken.compiler.core#required-libraries */,0,C_SCHEME_END_OF_LIST);
t77=C_set_block_item(lf[81] /* chicken.compiler.core#linked-libraries */,0,C_SCHEME_END_OF_LIST);
t78=C_set_block_item(lf[82] /* chicken.compiler.core#used-libraries */,0,C_SCHEME_END_OF_LIST);
t79=C_mutate(&lf[83] /* (set! chicken.compiler.core#unlikely-variables ...) */,lf[84]);
t80=C_set_block_item(lf[85] /* chicken.compiler.core#default-extended-bindings */,0,C_SCHEME_END_OF_LIST);
t81=C_set_block_item(lf[86] /* chicken.compiler.core#default-standard-bindings */,0,C_SCHEME_END_OF_LIST);
t82=C_set_block_item(lf[87] /* chicken.compiler.core#internal-bindings */,0,C_SCHEME_END_OF_LIST);
t83=C_mutate((C_word*)lf[88]+1 /* (set! chicken.compiler.core#initialize-compiler ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7654,tmp=(C_word)a,a+=2,tmp));
t84=C_mutate((C_word*)lf[94]+1 /* (set! chicken.compiler.core#compute-database-statistics ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7696,tmp=(C_word)a,a+=2,tmp));
t85=C_mutate((C_word*)lf[102]+1 /* (set! chicken.compiler.core#canonicalize-expression ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7804,tmp=(C_word)a,a+=2,tmp));
t86=C_mutate((C_word*)lf[485]+1 /* (set! chicken.compiler.core#build-toplevel-procedure ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15630,tmp=(C_word)a,a+=2,tmp));
t87=C_mutate((C_word*)lf[489]+1 /* (set! chicken.compiler.core#foreign-stub-id ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15653,tmp=(C_word)a,a+=2,tmp));
t88=C_mutate((C_word*)lf[492]+1 /* (set! chicken.compiler.core#foreign-stub-return-type ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15662,tmp=(C_word)a,a+=2,tmp));
t89=C_mutate((C_word*)lf[494]+1 /* (set! chicken.compiler.core#foreign-stub-name ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15671,tmp=(C_word)a,a+=2,tmp));
t90=C_mutate((C_word*)lf[496]+1 /* (set! chicken.compiler.core#foreign-stub-argument-types ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15680,tmp=(C_word)a,a+=2,tmp));
t91=C_mutate((C_word*)lf[498]+1 /* (set! chicken.compiler.core#foreign-stub-argument-names ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15689,tmp=(C_word)a,a+=2,tmp));
t92=C_mutate((C_word*)lf[500]+1 /* (set! chicken.compiler.core#foreign-stub-body ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15698,tmp=(C_word)a,a+=2,tmp));
t93=C_mutate((C_word*)lf[502]+1 /* (set! chicken.compiler.core#foreign-stub-cps ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15707,tmp=(C_word)a,a+=2,tmp));
t94=C_mutate((C_word*)lf[504]+1 /* (set! chicken.compiler.core#foreign-stub-callback ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15716,tmp=(C_word)a,a+=2,tmp));
t95=C_mutate(&lf[316] /* (set! chicken.compiler.core#create-foreign-stub ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_15725,tmp=(C_word)a,a+=2,tmp));
t96=C_mutate(&lf[309] /* (set! chicken.compiler.core#expand-foreign-lambda ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16114,tmp=(C_word)a,a+=2,tmp));
t97=C_mutate(&lf[312] /* (set! chicken.compiler.core#expand-foreign-lambda* ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16151,tmp=(C_word)a,a+=2,tmp));
t98=C_mutate((C_word*)lf[519]+1 /* (set! chicken.compiler.core#perform-cps-conversion ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_16499,tmp=(C_word)a,a+=2,tmp));
t99=C_mutate((C_word*)lf[533]+1 /* (set! chicken.compiler.core#analyze-expression ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_17413,tmp=(C_word)a,a+=2,tmp));
t100=C_mutate((C_word*)lf[604]+1 /* (set! chicken.compiler.core#perform-closure-conversion ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_19469,tmp=(C_word)a,a+=2,tmp));
t101=C_mutate((C_word*)lf[645]+1 /* (set! chicken.compiler.core#lambda-literal-id ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21346,tmp=(C_word)a,a+=2,tmp));
t102=C_mutate((C_word*)lf[648]+1 /* (set! chicken.compiler.core#lambda-literal-external ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21355,tmp=(C_word)a,a+=2,tmp));
t103=C_mutate((C_word*)lf[650]+1 /* (set! chicken.compiler.core#lambda-literal-argument-count ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21373,tmp=(C_word)a,a+=2,tmp));
t104=C_mutate((C_word*)lf[652]+1 /* (set! chicken.compiler.core#lambda-literal-rest-argument ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21382,tmp=(C_word)a,a+=2,tmp));
t105=C_mutate((C_word*)lf[654]+1 /* (set! chicken.compiler.core#lambda-literal-temporaries ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21391,tmp=(C_word)a,a+=2,tmp));
t106=C_mutate((C_word*)lf[656]+1 /* (set! chicken.compiler.core#lambda-literal-float-temporaries ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21400,tmp=(C_word)a,a+=2,tmp));
t107=C_mutate((C_word*)lf[658]+1 /* (set! chicken.compiler.core#lambda-literal-callee-signatures ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21409,tmp=(C_word)a,a+=2,tmp));
t108=C_mutate((C_word*)lf[660]+1 /* (set! chicken.compiler.core#lambda-literal-allocated ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21418,tmp=(C_word)a,a+=2,tmp));
t109=C_mutate((C_word*)lf[662]+1 /* (set! chicken.compiler.core#lambda-literal-closure-size ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21436,tmp=(C_word)a,a+=2,tmp));
t110=C_mutate((C_word*)lf[664]+1 /* (set! chicken.compiler.core#lambda-literal-looping ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21445,tmp=(C_word)a,a+=2,tmp));
t111=C_mutate((C_word*)lf[666]+1 /* (set! chicken.compiler.core#lambda-literal-customizable ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21454,tmp=(C_word)a,a+=2,tmp));
t112=C_mutate((C_word*)lf[668]+1 /* (set! chicken.compiler.core#lambda-literal-rest-argument-mode ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21463,tmp=(C_word)a,a+=2,tmp));
t113=C_mutate((C_word*)lf[670]+1 /* (set! chicken.compiler.core#lambda-literal-body ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21472,tmp=(C_word)a,a+=2,tmp));
t114=C_mutate((C_word*)lf[672]+1 /* (set! chicken.compiler.core#lambda-literal-direct ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21481,tmp=(C_word)a,a+=2,tmp));
t115=C_mutate((C_word*)lf[674]+1 /* (set! chicken.compiler.core#prepare-for-code-generation ...) */,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_21490,tmp=(C_word)a,a+=2,tmp));
t116=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t116;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t116+1)))(2,av2);}}

/* loop in a12901 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_6472(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,4)))){
C_save_and_reclaim_args((void *)trf_6472,5,t0,t1,t2,t3,t4);}
a=C_alloc(4);
if(C_truep(C_fixnum_less_or_equal_p(t2,C_fix(0)))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6486,a[2]=t1,a[3]=t4,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:67: scheme#reverse */
t6=*((C_word*)lf[259]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_fixnum_difference(t2,C_fix(1));
t6=C_i_car(t4);
t7=C_a_i_cons(&a,2,t6,t3);
/* mini-srfi-1.scm:68: loop */
t9=t1;
t10=t5;
t11=t7;
t12=C_u_i_cdr(t4);
t1=t9;
t2=t10;
t3=t11;
t4=t12;
goto loop;}}

/* k6484 in loop in a12901 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_6486(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6486,c,av);}
/* mini-srfi-1.scm:67: scheme#values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
C_values(4,av2);}}

/* chicken.compiler.core#every in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6675(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_6675,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6681,a[2]=t5,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t7=((C_word*)t5)[1];
f_6681(t7,t1,t3);}

/* loop in chicken.compiler.core#every in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6681(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_6681,3,t0,t1,t2);}
a=C_alloc(5);
t3=C_i_nullp(t2);
if(C_truep(t3)){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6703,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:82: pred */
t5=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}}

/* k6701 in loop in chicken.compiler.core#every in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6703(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6703,c,av);}
if(C_truep(C_i_not(t1))){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:83: loop */
t2=((C_word*)((C_word*)t0)[3])[1];
f_6681(t2,((C_word*)t0)[2],C_u_i_cdr(((C_word*)t0)[4]));}}

/* loop in foldl545 in foldl529 in chicken.compiler.core#lset-union/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_6715(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_nullp(t1))){
return(C_SCHEME_FALSE);}
else{
t2=(
/* mini-srfi-1.scm:88: pred */
  f_7279(((C_word*)t0)[2],C_i_car(t1))
);
if(C_truep(t2)){
return(t2);}
else{
t4=C_u_i_cdr(t1);
t1=t4;
goto loop;}}}

/* chicken.compiler.core#cons* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6739(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_6739,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6745,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_6745(t7,t1,t2,t3);}

/* loop in chicken.compiler.core#cons* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6745(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,3)))){
C_save_and_reclaim_args((void *)trf_6745,4,t0,t1,t2,t3);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6759,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:95: loop */
t6=t4;
t7=C_i_car(t3);
t8=C_u_i_cdr(t3);
t1=t6;
t2=t7;
t3=t8;
goto loop;}}

/* k6757 in loop in chicken.compiler.core#cons* in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6759(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_6759,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* chicken.compiler.core#concatenate in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6767(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_6767,2,t1,t2);}
a=C_alloc(5);
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6773,a[2]=t4,tmp=(C_word)a,a+=3,tmp));
t6=((C_word*)t4)[1];
f_6773(t6,t1,t2);}

/* loop in chicken.compiler.core#concatenate in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6773(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_6773,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6791,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:101: loop */
t6=t4;
t7=C_u_i_cdr(t2);
t1=t6;
t2=t7;
goto loop;}}

/* k6789 in loop in chicken.compiler.core#concatenate in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6791,c,av);}
/* mini-srfi-1.scm:101: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* chicken.compiler.core#delete in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6795(C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_6795,4,t1,t2,t3,t4);}
a=C_alloc(7);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6801,a[2]=t6,a[3]=t4,a[4]=t2,tmp=(C_word)a,a+=5,tmp));
t8=((C_word*)t6)[1];
f_6801(t8,t1,t3);}

/* loop in chicken.compiler.core#delete in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6801(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,3)))){
C_save_and_reclaim_args((void *)trf_6801,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6814,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:106: test */
t4=((C_word*)t0)[3];{
C_word av2[4];
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[4];
av2[3]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k6812 in loop in chicken.compiler.core#delete in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6814(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_6814,c,av);}
a=C_alloc(4);
if(C_truep(t1)){
/* mini-srfi-1.scm:107: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_6801(t2,((C_word*)t0)[3],C_u_i_cdr(((C_word*)t0)[4]));}
else{
t2=C_u_i_car(((C_word*)t0)[4]);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6828,a[2]=((C_word*)t0)[3],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:109: loop */
t4=((C_word*)((C_word*)t0)[2])[1];
f_6801(t4,t3,C_u_i_cdr(((C_word*)t0)[4]));}}

/* k6826 in k6812 in loop in chicken.compiler.core#delete in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6828(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_6828,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* loop in k19909 in a19889 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6876(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_6876,3,t0,t1,t2);}
a=C_alloc(10);
if(C_truep(C_i_nullp(t2))){
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=C_i_car(t2);
t4=C_u_i_cdr(t2);
t5=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_6890,a[2]=t4,a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6903,a[2]=((C_word*)t0)[2],a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:123: delete */
f_6795(t6,t3,t4,((C_word*)t0)[3]);}}

/* k6888 in loop in k19909 in a19889 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_6890(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_6890,c,av);}
a=C_alloc(3);
t2=C_i_equalp(((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?((C_word*)t0)[4]:C_a_i_cons(&a,2,((C_word*)t0)[5],t1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k6901 in loop in k19909 in a19889 in k19878 in a19872 in k19535 in k19505 in k19502 in k19499 in gather in chicken.compiler.core#perform-closure-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_6903(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_6903,c,av);}
/* mini-srfi-1.scm:123: loop */
t2=((C_word*)((C_word*)t0)[2])[1];
f_6876(t2,((C_word*)t0)[3],t1);}

/* chicken.compiler.core#filter in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6915(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_6915,3,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_check_list_2(t3,lf[8]);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6924,a[2]=t2,a[3]=t6,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_6924(t8,t1,t3);}

/* foldr401 in chicken.compiler.core#filter in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6924(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_6924,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6932,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6953,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g406 in foldr401 in chicken.compiler.core#filter in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_6932(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_6932,4,t0,t1,t2,t3);}
a=C_alloc(5);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_6939,a[2]=t1,a[3]=t2,a[4]=t3,tmp=(C_word)a,a+=5,tmp);
/* mini-srfi-1.scm:131: pred */
t5=((C_word*)t0)[2];{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k6937 in g406 in foldr401 in chicken.compiler.core#filter in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_6939,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=(C_truep(t1)?C_a_i_cons(&a,2,((C_word*)t0)[3],((C_word*)t0)[4]):((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k6951 in foldr401 in chicken.compiler.core#filter in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_6953(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_6953,c,av);}
/* mini-srfi-1.scm:131: g406 */
t2=((C_word*)t0)[2];
f_6932(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* foldr419 in globalize-all in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_6968(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(8,0,4)))){
C_save_and_reclaim_args((void *)trf_6968,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6976,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t4=C_slot(t2,C_fix(0));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7002,a[2]=t3,a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t7=t5;
t8=C_slot(t2,C_fix(1));
t1=t7;
t2=t8;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* g424 in foldr419 in globalize-all in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_6976(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_6976,4,t0,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_6980,a[2]=t3,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* mini-srfi-1.scm:135: pred */
t5=((C_word*)((C_word*)t0)[2])[1];
f_13735(t5,t4,t2);}

/* k6978 in g424 in foldr419 in globalize-all in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_6980(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,2)))){
C_save_and_reclaim((void *)f_6980,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6984,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:135: g434 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(
/* mini-srfi-1.scm:135: g434 */
  f_6984(C_a_i(&a,3),t2,t1)
);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t2=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* g434 in k6978 in g424 in foldr419 in globalize-all in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static C_word C_fcall f_6984(C_word *a,C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_a_i_cons(&a,2,t1,((C_word*)t0)[2]));}

/* k7000 in foldr419 in globalize-all in g2490 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_7002(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7002,c,av);}
/* mini-srfi-1.scm:134: g424 */
t2=((C_word*)t0)[2];
f_6976(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.compiler.core#unzip1 in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7024(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_7024,2,t1,t2);}
a=C_alloc(12);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(t2,lf[10]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7041,a[2]=t5,a[3]=t9,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_7041(t11,t1,t2);}

/* map-loop447 in chicken.compiler.core#unzip1 in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7041(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,2)))){
C_save_and_reclaim_args((void *)trf_7041,3,t0,t1,t2);}
a=C_alloc(3);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_cons(&a,2,t4,C_SCHEME_END_OF_LIST);
t6=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t5);
t7=C_mutate(((C_word *)((C_word*)t0)[2])+1,t5);
t9=t1;
t10=C_slot(t2,C_fix(1));
t1=t9;
t2=t10;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* loop in k16919 in k16597 in k16588 in k16585 in k16582 in k16579 in walk in chicken.compiler.core#perform-cps-conversion in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_7081(C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_stack_overflow_check;
loop:{}
t2=C_i_cdr(t1);
if(C_truep(C_i_nullp(t2))){
return(C_u_i_car(t1));}
else{
t4=C_u_i_cdr(t1);
t1=t4;
goto loop;}}

/* loop in blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7106(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_7106,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7119,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* mini-srfi-1.scm:154: pred */
t5=((C_word*)t0)[3];
f_22947(t5,t4,C_i_car(t3));}}

/* k7117 in loop in blockvar-literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7119(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7119,c,av);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* mini-srfi-1.scm:155: loop */
t2=((C_word*)((C_word*)t0)[4])[1];
f_7106(t2,((C_word*)t0)[2],C_fixnum_plus(((C_word*)t0)[3],C_fix(1)),C_u_i_cdr(((C_word*)t0)[5]));}}

/* chicken.compiler.core#lset-adjoin/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7134(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_7134,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7140,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_7140(t7,t1,t3,t2);}

/* loop in chicken.compiler.core#lset-adjoin/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7140(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_7140,4,t0,t1,t2,t3);}
a=C_alloc(3);
if(C_truep(C_i_nullp(t2))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=C_i_car(t2);
if(C_truep(C_i_memq(t4,t3))){
/* mini-srfi-1.scm:160: loop */
t9=t1;
t10=C_u_i_cdr(t2);
t11=t3;
t1=t9;
t2=t10;
t3=t11;
goto loop;}
else{
t5=C_u_i_cdr(t2);
t6=C_u_i_car(t2);
t7=C_a_i_cons(&a,2,t6,t3);
/* mini-srfi-1.scm:161: loop */
t9=t1;
t10=t5;
t11=t7;
t1=t9;
t2=t10;
t3=t11;
goto loop;}}}

/* chicken.compiler.core#lset-difference/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7175(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_7175,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7181,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_7181(t7,t1,t3,t2);}

/* foldl507 in chicken.compiler.core#lset-difference/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7181(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_7181,4,t0,t1,t2,t3);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7214,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=C_slot(t2,C_fix(0));
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7203,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:166: filter */
f_6915(t5,t7,t3);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* a7202 in foldl507 in chicken.compiler.core#lset-difference/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7203(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7203,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_not(C_i_memq(t2,((C_word*)t0)[2]));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7212 in foldl507 in chicken.compiler.core#lset-difference/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7214(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7214,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7181(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.compiler.core#lset-union/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7220(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_7220,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_SCHEME_UNDEFINED;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=C_set_block_item(t5,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7226,a[2]=t5,tmp=(C_word)a,a+=3,tmp));
t7=((C_word*)t5)[1];
f_7226(t7,t1,t3,t2);}

/* foldl529 in chicken.compiler.core#lset-union/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7226(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,4)))){
C_save_and_reclaim_args((void *)trf_7226,4,t0,t1,t2,t3);}
a=C_alloc(10);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7293,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t4,tmp=(C_word)a,a+=5,tmp);
t6=C_slot(t2,C_fix(0));
t7=C_i_check_list_2(t6,lf[14]);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7251,a[2]=t9,tmp=(C_word)a,a+=3,tmp));
t11=((C_word*)t9)[1];
f_7251(t11,t5,t6,t3);}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* foldl545 in foldl529 in chicken.compiler.core#lset-union/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7251(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(9,0,3)))){
C_save_and_reclaim_args((void *)trf_7251,4,t0,t1,t2,t3);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t4=C_slot(t2,C_fix(1));
t5=C_slot(t2,C_fix(0));
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7279,a[2]=t5,tmp=(C_word)a,a+=3,tmp);
t7=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6715,a[2]=t6,tmp=(C_word)a,a+=3,tmp);
t8=(
  f_6715(t7,t3)
);
if(C_truep(t8)){
t11=t1;
t12=t4;
t13=t3;
t1=t11;
t2=t12;
t3=t13;
goto loop;}
else{
t9=C_a_i_cons(&a,2,t5,t3);
t11=t1;
t12=t4;
t13=t9;
t1=t11;
t2=t12;
t3=t13;
goto loop;}}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* a7278 in foldl545 in foldl529 in chicken.compiler.core#lset-union/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_7279(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_stack_overflow_check;{}
return(C_eqp(t1,((C_word*)t0)[2]));}

/* k7291 in foldl529 in chicken.compiler.core#lset-union/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7293(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7293,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7226(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* chicken.compiler.core#lset-intersection/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7299(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_7299,3,t1,t2,t3);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7305,a[2]=t3,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:182: filter */
f_6915(t1,t4,t2);}

/* a7304 in chicken.compiler.core#lset-intersection/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7305(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_7305,c,av);}
a=C_alloc(3);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7311,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* mini-srfi-1.scm:183: every */
f_6675(t1,t3,((C_word*)t0)[2]);}

/* a7310 in a7304 in chicken.compiler.core#lset-intersection/eq? in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7311(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7311,c,av);}
t3=t1;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_memq(((C_word*)t0)[2],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* chicken.compiler.core#posq in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7518(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,3)))){
C_save_and_reclaim_args((void *)trf_7518,3,t1,t2,t3);}
a=C_alloc(3);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7524,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=(
  f_7524(t4,C_fix(0),t3)
);
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* loop in chicken.compiler.core#posq in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_7524(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_nullp(t2))){
return(C_SCHEME_FALSE);}
else{
t3=C_i_car(t2);
t4=C_eqp(t3,((C_word*)t0)[2]);
if(C_truep(t4)){
return(t1);}
else{
t6=C_fixnum_plus(t1,C_fix(1));
t7=C_u_i_cdr(t2);
t1=t6;
t2=t7;
goto loop;}}}

/* loop in k22867 in literal in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static C_word C_fcall f_7558(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_stack_overflow_check;
loop:{}
if(C_truep(C_i_nullp(t2))){
return(C_SCHEME_FALSE);}
else{
t3=C_i_car(t2);
if(C_truep(C_i_eqvp(t3,((C_word*)t0)[2]))){
return(t1);}
else{
t5=C_fixnum_plus(t1,C_fix(1));
t6=C_u_i_cdr(t2);
t1=t5;
t2=t6;
goto loop;}}}

/* chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7654(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7654,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7658,a[2]=t1,tmp=(C_word)a,a+=3,tmp);
if(C_truep(*((C_word*)lf[63]+1))){
/* core.scm:468: scheme#vector-fill! */
t3=*((C_word*)lf[92]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[63]+1);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7694,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:469: scheme#make-vector */
t4=*((C_word*)lf[93]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[26]+1);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k7656 in chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7658(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7658,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7661,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(*((C_word*)lf[65]+1))){
/* core.scm:471: scheme#vector-fill! */
t3=*((C_word*)lf[92]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[65]+1);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7687,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:472: scheme#make-vector */
t4=*((C_word*)lf[93]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_fix(301);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k7659 in k7656 in chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7661,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7664,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(*((C_word*)lf[66]+1))){
/* core.scm:474: scheme#vector-fill! */
t3=*((C_word*)lf[92]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[66]+1);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7680,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:475: scheme#make-vector */
t4=*((C_word*)lf[93]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_fix(301);
av2[3]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k7662 in k7659 in k7656 in chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7664(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7664,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7667,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:476: chicken.compiler.support#reset-profile-info-vector-name! */
t3=*((C_word*)lf[91]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7665 in k7662 in k7659 in k7656 in chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7667(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_7667,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7670,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:477: chicken.compiler.support#clear-real-name-table! */
t3=*((C_word*)lf[90]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7668 in k7665 in k7662 in k7659 in k7656 in chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7670(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7670,c,av);}
/* core.scm:478: chicken.compiler.support#clear-foreign-type-table! */
t2=*((C_word*)lf[89]+1);{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k7678 in k7659 in k7656 in chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7680(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7680,c,av);}
t2=C_mutate((C_word*)lf[66]+1 /* (set! chicken.compiler.core#constant-table ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_7664(2,av2);}}

/* k7685 in k7656 in chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7687(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7687,c,av);}
t2=C_mutate((C_word*)lf[65]+1 /* (set! chicken.compiler.core#inline-table ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_7661(2,av2);}}

/* k7692 in chicken.compiler.core#initialize-compiler in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7694(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7694,c,av);}
t2=C_mutate((C_word*)lf[63]+1 /* (set! chicken.compiler.core#line-number-database-2 ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
f_7658(2,av2);}}

/* chicken.compiler.core#compute-database-statistics in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7696(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,4)))){
C_save_and_reclaim((void *)f_7696,c,av);}
a=C_alloc(25);
t3=C_fix(0);
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_fix(0);
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_fix(0);
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_fix(0);
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_fix(0);
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_7700,a[2]=t1,a[3]=t6,a[4]=t4,a[5]=t8,a[6]=t12,a[7]=t10,tmp=(C_word)a,a+=8,tmp);
t14=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7705,a[2]=t10,a[3]=t8,a[4]=t6,a[5]=t4,a[6]=t12,tmp=(C_word)a,a+=7,tmp);
/* core.scm:500: chicken.internal#hash-table-for-each */
t15=*((C_word*)lf[101]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t15;
av2[1]=t13;
av2[2]=t14;
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t15+1)))(4,av2);}}

/* k7698 in chicken.compiler.core#compute-database-statistics in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7700(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_7700,c,av);}
/* core.scm:514: scheme#values */{
C_word *av2;
if(c >= 9) {
  av2=av;
} else {
  av2=C_alloc(9);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[61];
av2[3]=lf[60];
av2[4]=((C_word*)((C_word*)t0)[3])[1];
av2[5]=((C_word*)((C_word*)t0)[4])[1];
av2[6]=((C_word*)((C_word*)t0)[5])[1];
av2[7]=((C_word*)((C_word*)t0)[6])[1];
av2[8]=((C_word*)((C_word*)t0)[7])[1];
C_values(9,av2);}}

/* a7704 in chicken.compiler.core#compute-database-statistics in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7705(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,3)))){
C_save_and_reclaim((void *)f_7705,c,av);}
a=C_alloc(13);
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_7707,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
t5=C_i_check_list_2(t3,lf[100]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7781,a[2]=t7,a[3]=t4,tmp=(C_word)a,a+=4,tmp));
t9=((C_word*)t7)[1];
f_7781(t9,t1,t3);}

/* g777 in a7704 in chicken.compiler.core#compute-database-statistics in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7707(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(62,0,2)))){
C_save_and_reclaim_args((void *)trf_7707,3,t0,t1,t2);}
a=C_alloc(62);
t3=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[2])[1],C_fix(1));
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t5=C_i_car(t2);
t6=C_eqp(t5,lf[95]);
if(C_truep(t6)){
t7=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[3])[1],C_fix(1));
t8=C_mutate(((C_word *)((C_word*)t0)[3])+1,t7);
t9=t1;{
C_word av2[2];
av2[0]=t9;
av2[1]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(2,av2);}}
else{
t7=C_eqp(t5,lf[96]);
if(C_truep(t7)){
t8=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[4])[1],C_fix(1));
t9=C_mutate(((C_word *)((C_word*)t0)[4])+1,t8);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7749,a[2]=((C_word*)t0)[5],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:509: chicken.compiler.support#node-class */
t11=*((C_word*)lf[98]+1);{
C_word av2[3];
av2[0]=t11;
av2[1]=t10;
av2[2]=C_u_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t8=C_eqp(t5,lf[99]);
if(C_truep(t8)){
t9=C_i_length(C_u_i_cdr(t2));
t10=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[6])[1],t9);
t11=C_mutate(((C_word *)((C_word*)t0)[6])+1,t10);
t12=t1;{
C_word av2[2];
av2[0]=t12;
av2[1]=t11;
((C_proc)(void*)(*((C_word*)t12+1)))(2,av2);}}
else{
t9=C_SCHEME_UNDEFINED;
t10=t1;{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}}}}

/* k7747 in g777 in a7704 in chicken.compiler.core#compute-database-statistics in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7749(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(29,c,1)))){
C_save_and_reclaim((void *)f_7749,c,av);}
a=C_alloc(29);
t2=C_eqp(lf[97],t1);
if(C_truep(t2)){
t3=C_s_a_i_plus(&a,2,((C_word*)((C_word*)t0)[2])[1],C_fix(1));
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t3);
t5=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* for-each-loop776 in a7704 in chicken.compiler.core#compute-database-statistics in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7781(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7781,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7791,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:502: g777 */
t4=((C_word*)t0)[3];
f_7707(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k7789 in for-each-loop776 in a7704 in chicken.compiler.core#compute-database-statistics in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7791(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_7791,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7781(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7804(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(73,c,9)))){
C_save_and_reclaim((void *)f_7804,c,av);}
a=C_alloc(73);
t3=C_SCHEME_END_OF_LIST;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_END_OF_LIST;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_SCHEME_UNDEFINED;
t8=(*a=C_VECTOR_TYPE|1,a[1]=t7,tmp=(C_word)a,a+=2,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_SCHEME_UNDEFINED;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_SCHEME_UNDEFINED;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=C_SCHEME_UNDEFINED;
t18=(*a=C_VECTOR_TYPE|1,a[1]=t17,tmp=(C_word)a,a+=2,tmp);
t19=C_SCHEME_UNDEFINED;
t20=(*a=C_VECTOR_TYPE|1,a[1]=t19,tmp=(C_word)a,a+=2,tmp);
t21=C_SCHEME_UNDEFINED;
t22=(*a=C_VECTOR_TYPE|1,a[1]=t21,tmp=(C_word)a,a+=2,tmp);
t23=C_SCHEME_UNDEFINED;
t24=(*a=C_VECTOR_TYPE|1,a[1]=t23,tmp=(C_word)a,a+=2,tmp);
t25=C_SCHEME_UNDEFINED;
t26=(*a=C_VECTOR_TYPE|1,a[1]=t25,tmp=(C_word)a,a+=2,tmp);
t27=C_set_block_item(t8,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7807,a[2]=t8,tmp=(C_word)a,a+=3,tmp));
t28=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7842,a[2]=t8,tmp=(C_word)a,a+=3,tmp));
t29=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7876,a[2]=t10,tmp=(C_word)a,a+=3,tmp));
t30=C_set_block_item(t14,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7889,tmp=(C_word)a,a+=2,tmp));
t31=C_set_block_item(t16,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7916,a[2]=t14,tmp=(C_word)a,a+=3,tmp));
t32=C_set_block_item(t18,0,(*a=C_CLOSURE_TYPE|1,a[1]=(C_word)f_7945,tmp=(C_word)a,a+=2,tmp));
t33=C_set_block_item(t20,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8011,a[2]=t10,tmp=(C_word)a,a+=3,tmp));
t34=C_set_block_item(t22,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8054,a[2]=t24,a[3]=t6,a[4]=t10,tmp=(C_word)a,a+=5,tmp));
t35=C_set_block_item(t24,0,(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8362,a[2]=t22,a[3]=t24,a[4]=t16,a[5]=t18,a[6]=t6,a[7]=t10,a[8]=t4,a[9]=t20,a[10]=t26,a[11]=t12,a[12]=t14,tmp=(C_word)a,a+=13,tmp));
t36=C_set_block_item(t26,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13609,a[2]=t24,tmp=(C_word)a,a+=3,tmp));
t37=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13661,a[2]=t24,a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_memq(lf[161],*((C_word*)lf[483]+1)))){
t38=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13692,a[2]=t37,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1530: scheme#newline */
t39=*((C_word*)lf[484]+1);{
C_word *av2=av;
av2[0]=t39;
av2[1]=t38;
((C_proc)(void*)(*((C_word*)t39+1)))(2,av2);}}
else{
t38=t37;{
C_word *av2=av;
av2[0]=t38;
av2[1]=C_SCHEME_UNDEFINED;
f_13661(2,av2);}}}

/* find-id in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7807(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_7807,4,t0,t1,t2,t3);}
a=C_alloc(6);
if(C_truep(C_i_nullp(t3))){
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
t4=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7820,a[2]=t1,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t2,tmp=(C_word)a,a+=6,tmp);
t5=C_i_caar(t3);
t6=C_eqp(t2,t5);
if(C_truep(t6)){
t7=C_u_i_car(t3);
t8=t4;
f_7820(t8,C_i_symbolp(C_u_i_cdr(t7)));}
else{
t7=t4;
f_7820(t7,C_SCHEME_FALSE);}}}

/* k7818 in find-id in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7820(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,3)))){
C_save_and_reclaim_args((void *)trf_7820,2,t0,t1);}
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=C_u_i_cdr(C_u_i_car(((C_word*)t0)[3]));
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* core.scm:531: find-id */
t2=((C_word*)((C_word*)t0)[4])[1];
f_7807(t2,((C_word*)t0)[2],((C_word*)t0)[5],C_u_i_cdr(((C_word*)t0)[3]));}}

/* lookup in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7842(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_7842,3,t0,t1,t2);}
a=C_alloc(9);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7846,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7874,a[2]=((C_word*)t0)[2],a[3]=t3,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:534: ##sys#current-environment */
t5=*((C_word*)lf[107]+1);{
C_word av2[2];
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k7844 in lookup in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7846(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_7846,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7854,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7860,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
/* core.scm:534: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}}

/* a7853 in k7844 in lookup in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7854(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7854,c,av);}
/* core.scm:535: ##sys#get */
t2=*((C_word*)lf[103]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=lf[104];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* a7859 in k7844 in lookup in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7860(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand((c-2)*C_SIZEOF_PAIR +5,c,3)))){
C_save_and_reclaim((void*)f_7860,c,av);}
a=C_alloc((c-2)*C_SIZEOF_PAIR+5);
t2=C_build_rest(&a,c,2,av);
C_word t3;
C_word t4;
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7867,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=t3;
av2[2]=*((C_word*)lf[106]+1);
av2[3]=t2;
C_apply(4,av2);}}

/* k7865 in a7859 in k7844 in lookup in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7867(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7867,c,av);}
if(C_truep(t1)){{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=*((C_word*)lf[105]+1);
av2[3]=((C_word*)t0)[3];
C_apply(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k7872 in lookup in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7874(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7874,c,av);}
/* core.scm:534: find-id */
t2=((C_word*)((C_word*)t0)[2])[1];
f_7807(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1);}

/* macro-alias in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7876(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7876,3,t0,t1,t2);}
a=C_alloc(5);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7880,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:539: chicken.base#gensym */
t4=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k7878 in macro-alias in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_7880,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7883,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7887,a[2]=t2,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:540: lookup */
t4=((C_word*)((C_word*)t0)[3])[1];
f_7842(t4,t3,((C_word*)t0)[4]);}

/* k7881 in k7878 in macro-alias in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7883(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7883,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k7885 in k7878 in macro-alias in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7887(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_7887,c,av);}
/* core.scm:540: ##sys#put! */
t2=*((C_word*)lf[108]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[104];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* handle-expansion-result in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7889(C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,4)))){
C_save_and_reclaim_args((void *)trf_7889,2,t1,t2);}
a=C_alloc(3);
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7891,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* f_7891 in handle-expansion-result in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7891(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_7891,c,av);}
a=C_alloc(10);
t4=C_eqp(t2,t3);
t5=C_i_not(t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7898,a[2]=t1,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
if(C_truep(t5)){
t7=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7901,a[2]=((C_word*)t0)[2],a[3]=t6,a[4]=t3,a[5]=t1,tmp=(C_word)a,a+=6,tmp);
/* core.scm:546: chicken.compiler.support#get-line */
t8=*((C_word*)lf[113]+1);{
C_word *av2=av;
av2[0]=t8;
av2[1]=t7;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t7=t1;{
C_word *av2=av;
av2[0]=t7;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k7896 */
static void C_ccall f_7898(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7898,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k7899 */
static void C_ccall f_7901(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_7901,c,av);}
a=C_alloc(11);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
if(C_truep(t2)){
t3=C_SCHEME_UNDEFINED;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t4,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_16420,a[2]=t6,tmp=(C_word)a,a+=3,tmp));
t8=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16445,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
/* core.scm:1928: walk */
t9=((C_word*)t6)[1];
f_16445(t9,((C_word*)t0)[3],((C_word*)t0)[4]);}
else{
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* canonicalize-body/ln in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7916(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4){
C_word tmp;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_7916,5,t0,t1,t2,t3,t4);}
a=C_alloc(5);
t5=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7920,a[2]=t3,a[3]=t4,a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:552: handle-expansion-result */
f_7889(t5,t2);}

/* k7918 in canonicalize-body/ln in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7920(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,4)))){
C_save_and_reclaim((void *)f_7920,c,av);}
a=C_alloc(16);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7925,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7930,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_7940,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* core.scm:551: ##sys#dynamic-wind */
t9=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[4];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a7924 in k7918 in canonicalize-body/ln in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7925(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7925,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[114]+1));
t3=C_mutate((C_word*)lf[114]+1 /* (set! chicken.syntax#expansion-result-hook ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a7929 in k7918 in canonicalize-body/ln in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7930(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_7930,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_7938,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:553: ##sys#current-environment */
t3=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k7936 in a7929 in k7918 in canonicalize-body/ln in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_7938,c,av);}
/* core.scm:553: ##sys#canonicalize-body */
t2=*((C_word*)lf[115]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a7939 in k7918 in canonicalize-body/ln in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7940(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_7940,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,*((C_word*)lf[114]+1));
t3=C_mutate((C_word*)lf[114]+1 /* (set! chicken.syntax#expansion-result-hook ...) */,((C_word*)((C_word*)t0)[3])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* set-real-names! in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7945(C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,4)))){
C_save_and_reclaim_args((void *)trf_7945,3,t1,t2,t3);}
a=C_alloc(5);
t4=C_i_check_list_2(t2,lf[100]);
t5=C_i_check_list_2(t3,lf[100]);
t6=C_SCHEME_UNDEFINED;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=C_set_block_item(t7,0,(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_7962,a[2]=t7,tmp=(C_word)a,a+=3,tmp));
t9=((C_word*)t7)[1];
f_7962(t9,t1,t2,t3);}

/* for-each-loop868 in set-real-names! in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_7962(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,3)))){
C_save_and_reclaim_args((void *)trf_7962,4,t0,t1,t2,t3);}
a=C_alloc(6);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_7972,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=t3,tmp=(C_word)a,a+=6,tmp);
/* core.scm:556: chicken.compiler.support#set-real-name! */
t7=*((C_word*)lf[117]+1);{
C_word av2[4];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_slot(t2,C_fix(0));
av2[3]=C_slot(t3,C_fix(0));
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}
else{
t6=C_SCHEME_UNDEFINED;
t7=t1;{
C_word av2[2];
av2[0]=t7;
av2[1]=t6;
((C_proc)(void*)(*((C_word*)t7+1)))(2,av2);}}}

/* k7970 in for-each-loop868 in set-real-names! in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_7972(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_7972,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_7962(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)),C_slot(((C_word*)t0)[5],C_fix(1)));}

/* unquotify in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8011(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,2)))){
C_save_and_reclaim_args((void *)trf_8011,3,t0,t1,t2);}
a=C_alloc(4);
if(C_truep(C_i_listp(t2))){
t3=C_eqp(C_fix(2),C_u_i_length(t2));
if(C_truep(t3)){
t4=C_i_car(t2);
if(C_truep(C_i_symbolp(t4))){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8043,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:567: lookup */
t6=((C_word*)((C_word*)t0)[2])[1];
f_7842(t6,t5,C_u_i_car(t2));}
else{
t5=t1;{
C_word av2[2];
av2[0]=t5;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}
else{
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8041 in unquotify in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8043(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8043,c,av);}
t2=C_eqp(lf[118],t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=(C_truep(t2)?C_i_cadr(((C_word*)t0)[3]):((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8054(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7){
C_word tmp;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,0,2)))){
C_save_and_reclaim_args((void *)trf_8054,8,t0,t1,t2,t3,t4,t5,t6,t7);}
a=C_alloc(16);
t8=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_8058,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=((C_word*)t0)[3],a[11]=((C_word*)t0)[4],tmp=(C_word)a,a+=12,tmp);
if(C_truep(C_i_memq(t2,lf[83]))){
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8250,a[2]=t8,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:574: chicken.base#open-output-string */
t10=*((C_word*)lf[133]+1);{
C_word av2[2];
av2[0]=t10;
av2[1]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(2,av2);}}
else{
t9=t8;{
C_word av2[2];
av2[0]=t9;
av2[1]=C_SCHEME_UNDEFINED;
f_8058(2,av2);}}}

/* k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8058(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_8058,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8061,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:575: lookup */
t3=((C_word*)((C_word*)t0)[11])[1];
f_7842(t3,t2,((C_word*)t0)[3]);}

/* k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8061(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_8061,c,av);}
a=C_alloc(11);
t2=C_i_symbolp(t1);
if(C_truep(C_i_not(t2))){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8073,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[2],a[8]=t1,a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:578: chicken.internal#hash-table-ref */
t4=*((C_word*)lf[112]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[66]+1);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8073(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_8073,c,av);}
a=C_alloc(11);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8077,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:577: g923 */
t3=t2;
f_8077(t3,((C_word*)t0)[7],t1);}
else{
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8087,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:580: chicken.internal#hash-table-ref */
t3=*((C_word*)lf[112]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=*((C_word*)lf[65]+1);
av2[3]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}

/* g923 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8077(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,8)))){
C_save_and_reclaim_args((void *)trf_8077,3,t0,t1,t2);}
/* core.scm:579: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8362(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE);}

/* k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8087(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_8087,c,av);}
a=C_alloc(7);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8091,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:577: g927 */
t3=t2;
f_8091(t3,((C_word*)t0)[7],t1);}
else{
t2=C_i_assq(((C_word*)t0)[8],lf[70]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8105,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:577: g931 */
t4=t3;
f_8105(t4,((C_word*)t0)[7],t2);}
else{
t3=C_i_assq(((C_word*)t0)[8],lf[74]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8144,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],tmp=(C_word)a,a+=7,tmp);
/* core.scm:577: g941 */
t5=t4;
f_8144(t5,((C_word*)t0)[7],t3);}
else{
t4=C_i_memq(((C_word*)t0)[8],((C_word*)t0)[3]);
if(C_truep(C_i_not(t4))){
t5=C_a_i_cons(&a,2,((C_word*)t0)[6],((C_word*)t0)[9]);
/* core.scm:602: ##sys#alias-global-hook */
t6=*((C_word*)lf[124]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t6;
av2[1]=((C_word*)t0)[7];
av2[2]=((C_word*)t0)[8];
av2[3]=C_SCHEME_FALSE;
av2[4]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(5,av2);}}
else{
t5=C_i_assq(((C_word*)t0)[8],((C_word*)((C_word*)t0)[10])[1]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8196,a[2]=((C_word*)t0)[8],tmp=(C_word)a,a+=3,tmp);
/* core.scm:577: g951 */
t7=t6;
f_8196(t7,((C_word*)t0)[7],t5);}
else{
t6=((C_word*)t0)[7];{
C_word *av2=av;
av2[0]=t6;
av2[1]=((C_word*)t0)[8];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}}}}}

/* g927 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8091(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,0,8)))){
C_save_and_reclaim_args((void *)trf_8091,3,t0,t1,t2);}
/* core.scm:581: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8362(t3,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE);}

/* g931 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8105(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_8105,3,t0,t1,t2);}
a=C_alloc(10);
t3=C_i_cadr(t2);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8112,a[2]=t2,a[3]=t3,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
/* core.scm:585: chicken.compiler.support#final-foreign-type */
t5=*((C_word*)lf[122]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k8110 in g931 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8112(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_8112,c,av);}
a=C_alloc(24);
t2=C_i_caddr(((C_word*)t0)[2]);
t3=C_a_i_list(&a,2,t2,((C_word*)t0)[3]);
t4=C_a_i_list(&a,2,lf[119],t3);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_8122,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],tmp=(C_word)a,a+=8,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8126,a[2]=t5,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:589: chicken.compiler.support#finish-foreign-result */
t7=*((C_word*)lf[121]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=t1;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k8120 in k8110 in g931 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8122(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_8122,c,av);}
/* core.scm:587: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE,C_SCHEME_FALSE);}

/* k8124 in k8110 in g931 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8126(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8126,c,av);}
/* core.scm:588: chicken.compiler.support#foreign-type-convert-result */
t2=*((C_word*)lf[120]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* g941 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8144(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,0,2)))){
C_save_and_reclaim_args((void *)trf_8144,3,t0,t1,t2);}
a=C_alloc(10);
t3=C_i_caddr(t2);
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8151,a[2]=t3,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t1,a[6]=((C_word*)t0)[3],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],tmp=(C_word)a,a+=10,tmp);
/* core.scm:595: chicken.compiler.support#final-foreign-type */
t5=*((C_word*)lf[122]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=t3;
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}

/* k8149 in g941 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8151(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_8151,c,av);}
a=C_alloc(24);
t2=C_a_i_list(&a,1,((C_word*)t0)[2]);
t3=C_i_cadr(((C_word*)t0)[3]);
t4=C_a_i_list(&a,3,lf[123],t2,t3);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_8161,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],tmp=(C_word)a,a+=8,tmp);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8165,a[2]=t5,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:599: chicken.compiler.support#finish-foreign-result */
t7=*((C_word*)lf[121]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=t1;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k8159 in k8149 in g941 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8161(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_8161,c,av);}
/* core.scm:597: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE,C_SCHEME_FALSE);}

/* k8163 in k8149 in g941 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8165(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8165,c,av);}
/* core.scm:598: chicken.compiler.support#foreign-type-convert-result */
t2=*((C_word*)lf[120]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* g951 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8196(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_8196,3,t0,t1,t2);}
a=C_alloc(8);
t3=C_i_cdr(t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8207,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
if(C_truep(t3)){
t5=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8214,a[2]=t4,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* core.scm:608: chicken.base#open-output-string */
t6=*((C_word*)lf[133]+1);{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t5=t4;{
C_word av2[2];
av2[0]=t5;
av2[1]=lf[134];
f_8207(2,av2);}}}

/* k8205 in g951 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8207(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8207,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8211,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:609: chicken.compiler.support#get-real-name */
t3=*((C_word*)lf[127]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8209 in k8205 in g951 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8211(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8211,c,av);}
/* core.scm:606: chicken.compiler.support#quit-compiling */
t2=*((C_word*)lf[125]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[126];
av2[3]=((C_word*)t0)[3];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k8212 in g951 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8214(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_8214,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8220,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:608: ##sys#write-char-0 */
t4=*((C_word*)lf[132]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_make_character(40);
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k8218 in k8212 in g951 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8220(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_8220,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8223,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:608: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8221 in k8218 in k8212 in g951 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8223(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_8223,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8226,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:608: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[131];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8224 in k8221 in k8218 in k8212 in g951 in k8085 in k8071 in k8059 in k8056 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8226(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8226,c,av);}
/* core.scm:608: chicken.base#get-output-string */
t2=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k8248 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8250(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_8250,c,av);}
a=C_alloc(5);
t2=C_i_check_port_2(t1,C_fix(2),C_SCHEME_TRUE,lf[128]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8256,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:574: ##sys#print */
t4=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[137];
av2[3]=C_SCHEME_FALSE;
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k8254 in k8248 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8256(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_8256,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8259,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:574: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_TRUE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8257 in k8254 in k8248 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8259(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_8259,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8262,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:574: ##sys#print */
t3=*((C_word*)lf[130]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[136];
av2[3]=C_SCHEME_FALSE;
av2[4]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8260 in k8257 in k8254 in k8248 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8262(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8262,c,av);}
a=C_alloc(3);
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8265,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:574: chicken.base#get-output-string */
t3=*((C_word*)lf[129]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8263 in k8260 in k8257 in k8254 in k8248 in resolve-variable in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8265(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8265,c,av);}
/* core.scm:573: chicken.base#warning */
t2=*((C_word*)lf[135]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_8271(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,3)))){
C_save_and_reclaim((void *)f_8271,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8274,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:616: ##sys#compiled-module-registration */
t3=*((C_word*)lf[231]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_8274(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,2)))){
C_save_and_reclaim((void *)f_8274,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8277,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8346,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:618: chicken.file#file-exists? */
t4=*((C_word*)lf[252]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8275 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_8277(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,6)))){
C_save_and_reclaim((void *)f_8277,c,av);}
a=C_alloc(5);
t2=C_i_equalp(((C_word*)t0)[2],t1);
t3=(C_truep(t2)?*((C_word*)lf[46]+1):C_SCHEME_FALSE);
if(C_truep(t3)){
if(C_truep(*((C_word*)lf[59]+1))){
/* core.scm:622: chicken.base#print */
t4=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t4;
av2[1]=((C_word*)t0)[3];
av2[2]=lf[237];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[238];
av2[5]=((C_word*)t0)[5];
av2[6]=lf[239];
((C_proc)(void*)(*((C_word*)t4+1)))(7,av2);}}
else{
t4=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
f_10900(2,av2);}}}
else{
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8292,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
if(C_truep(*((C_word*)lf[59]+1))){
/* core.scm:626: chicken.base#print */
t5=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=lf[247];
av2[3]=((C_word*)t0)[4];
av2[4]=lf[248];
av2[5]=((C_word*)t0)[5];
av2[6]=lf[249];
((C_proc)(void*)(*((C_word*)t5+1)))(7,av2);}}
else{
t5=t4;{
C_word *av2=av;
av2[0]=t5;
av2[1]=C_SCHEME_UNDEFINED;
f_8292(2,av2);}}}}

/* k8290 in k8275 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_8292(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,3)))){
C_save_and_reclaim((void *)f_8292,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8297,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:628: scheme#with-output-to-file */
t3=*((C_word*)lf[246]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[4];
av2[2]=((C_word*)t0)[3];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a8296 in k8290 in k8275 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in ... */
static void C_ccall f_8297(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,2)))){
C_save_and_reclaim((void *)f_8297,c,av);}
a=C_alloc(8);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8301,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8337,a[2]=t2,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:631: chicken.platform#chicken-version */
t4=*((C_word*)lf[245]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k8299 in a8296 in k8290 in k8275 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_8301(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_8301,c,av);}
a=C_alloc(9);
t2=*((C_word*)lf[240]+1);
t3=C_i_check_list_2(((C_word*)t0)[2],lf[100]);
t4=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8307,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t5=C_SCHEME_UNDEFINED;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=C_set_block_item(t6,0,(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8312,a[2]=t6,a[3]=t2,tmp=(C_word)a,a+=4,tmp));
t8=((C_word*)t6)[1];
f_8312(t8,t4,((C_word*)t0)[2]);}

/* k8305 in k8299 in a8296 in k8290 in k8275 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_ccall f_8307(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8307,c,av);}
/* core.scm:633: chicken.base#print */
t2=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[241];
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* for-each-loop983 in k8299 in a8296 in k8290 in k8275 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in ... */
static void C_fcall f_8312(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,0,2)))){
C_save_and_reclaim_args((void *)trf_8312,3,t0,t1,t2);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8322,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:632: g984 */
t4=((C_word*)t0)[3];{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_SCHEME_UNDEFINED;
t4=t1;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k8320 in for-each-loop983 in k8299 in a8296 in k8290 in k8275 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in ... */
static void C_ccall f_8322(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8322,c,av);}
t2=((C_word*)((C_word*)t0)[2])[1];
f_8312(t2,((C_word*)t0)[3],C_slot(((C_word*)t0)[4],C_fix(1)));}

/* k8335 in a8296 in k8290 in k8275 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in ... */
static void C_ccall f_8337(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,6)))){
C_save_and_reclaim((void *)f_8337,c,av);}
/* core.scm:630: chicken.base#print */
t2=*((C_word*)lf[236]+1);{
C_word *av2;
if(c >= 7) {
  av2=av;
} else {
  av2=C_alloc(7);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[242];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[243];
av2[5]=t1;
av2[6]=lf[244];
((C_proc)(void*)(*((C_word*)t2+1)))(7,av2);}}

/* k8344 in k8272 in k8269 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_8346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8346,c,av);}
if(C_truep(t1)){
/* core.scm:619: scheme#call-with-input-file */
t2=*((C_word*)lf[250]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=*((C_word*)lf[251]+1);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_FALSE;
f_8277(2,av2);}}}

/* k8354 in k10740 in k10737 in k10734 in k10731 in k10728 in k10725 in k10716 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_8356(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8356,c,av);}
/* ##sys#string-append */
t2=*((C_word*)lf[253]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=lf[254];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8362(C_word t0,C_word t1,C_word t2,C_word t3,C_word t4,C_word t5,C_word t6,C_word t7,C_word t8){
C_word tmp;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,0,2)))){
C_save_and_reclaim_args((void *)trf_8362,9,t0,t1,t2,t3,t4,t5,t6,t7,t8);}
a=C_alloc(21);
t9=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_8369,a[2]=t1,a[3]=t2,a[4]=((C_word*)t0)[2],a[5]=t3,a[6]=t4,a[7]=t5,a[8]=t6,a[9]=t7,a[10]=((C_word*)t0)[3],a[11]=t8,a[12]=((C_word*)t0)[4],a[13]=((C_word*)t0)[5],a[14]=((C_word*)t0)[6],a[15]=((C_word*)t0)[7],a[16]=((C_word*)t0)[8],a[17]=((C_word*)t0)[9],a[18]=((C_word*)t0)[10],a[19]=((C_word*)t0)[11],a[20]=((C_word*)t0)[12],tmp=(C_word)a,a+=21,tmp);
/* core.scm:636: chicken.keyword#keyword? */
t10=*((C_word*)lf[482]+1);{
C_word av2[3];
av2[0]=t10;
av2[1]=t9;
av2[2]=t2;
((C_proc)(void*)(*((C_word*)t10+1)))(3,av2);}}

/* k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8369(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,7)))){
C_save_and_reclaim((void *)f_8369,c,av);}
a=C_alloc(21);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,2,lf[118],((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
if(C_truep(C_i_symbolp(((C_word*)t0)[3]))){
/* core.scm:637: resolve-variable */
t2=((C_word*)((C_word*)t0)[4])[1];
f_8054(t2,((C_word*)t0)[2],((C_word*)t0)[3],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}
else{
t2=C_i_pairp(((C_word*)t0)[3]);
if(C_truep(C_i_not(t2))){
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8393,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:639: chicken.compiler.support#constant? */
t4=*((C_word*)lf[140]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_i_car(((C_word*)t0)[3]);
if(C_truep(C_i_symbolp(t3))){
t4=(*a=C_CLOSURE_TYPE|20,a[1]=(C_word)f_8408,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[4],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],tmp=(C_word)a,a+=21,tmp);
/* core.scm:643: chicken.compiler.support#get-line */
t5=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_i_listp(((C_word*)t0)[3]);
if(C_truep(C_i_not(t4))){
/* core.scm:1511: ##sys#syntax-error/context */
t5=*((C_word*)lf[138]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[480];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_13555,a[2]=((C_word*)t0)[18],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[6],a[11]=((C_word*)t0)[7],tmp=(C_word)a,a+=12,tmp);
/* core.scm:1513: chicken.compiler.support#constant? */
t6=*((C_word*)lf[140]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}}}}}}

/* k8391 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8393(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,3)))){
C_save_and_reclaim((void *)f_8393,c,av);}
a=C_alloc(6);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,2,lf[118],((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
/* core.scm:641: ##sys#syntax-error/context */
t2=*((C_word*)lf[138]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=lf[139];
av2[3]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}}

/* k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8408(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,3)))){
C_save_and_reclaim((void *)f_8408,c,av);}
a=C_alloc(22);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_8414,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=t2,a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[2],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],tmp=(C_word)a,a+=22,tmp);
/* core.scm:644: chicken.compiler.support#emit-syntax-trace-info */
t4=*((C_word*)lf[479]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
av2[3]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}

/* k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8414(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,3)))){
C_save_and_reclaim((void *)f_8414,c,av);}
a=C_alloc(27);
t2=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_8417,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],tmp=(C_word)a,a+=22,tmp);
if(C_truep(C_i_listp(((C_word*)t0)[2]))){
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_SCHEME_UNDEFINED;
f_8417(2,av2);}}
else{
if(C_truep(((C_word*)t0)[9])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13522,a[2]=t2,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[9],tmp=(C_word)a,a+=5,tmp);
/* core.scm:647: chicken.base#open-output-string */
t4=*((C_word*)lf[133]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:648: ##sys#syntax-error/context */
t3=*((C_word*)lf[138]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=lf[478];
av2[3]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}}}

/* k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8417(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,2)))){
C_save_and_reclaim((void *)f_8417,c,av);}
a=C_alloc(22);
t2=C_mutate((C_word*)lf[141]+1 /* (set! ##sys#syntax-error-culprit ...) */,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_8421,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],tmp=(C_word)a,a+=22,tmp);
/* core.scm:650: lookup */
t4=((C_word*)((C_word*)t0)[16])[1];
f_7842(t4,t3,C_u_i_car(((C_word*)t0)[2]));}

/* k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8421(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,2)))){
C_save_and_reclaim((void *)f_8421,c,av);}
a=C_alloc(22);
t2=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_8424,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=t1,a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],a[16]=((C_word*)t0)[15],a[17]=((C_word*)t0)[16],a[18]=((C_word*)t0)[17],a[19]=((C_word*)t0)[18],a[20]=((C_word*)t0)[19],a[21]=((C_word*)t0)[20],tmp=(C_word)a,a+=22,tmp);
/* core.scm:653: handle-expansion-result */
f_7889(t2,((C_word*)t0)[9]);}

/* k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8424(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(37,c,4)))){
C_save_and_reclaim((void *)f_8424,c,av);}
a=C_alloc(37);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_8427,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],tmp=(C_word)a,a+=22,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13489,a[2]=t5,a[3]=t3,tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_13494,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t9=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13504,a[2]=t3,a[3]=t5,tmp=(C_word)a,a+=4,tmp);
/* core.scm:652: ##sys#dynamic-wind */
t10=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t10;
av2[1]=t6;
av2[2]=t7;
av2[3]=t8;
av2[4]=t9;
((C_proc)(void*)(*((C_word*)t10+1)))(5,av2);}}

/* k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8427(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,8)))){
C_save_and_reclaim((void *)f_8427,c,av);}
a=C_alloc(22);
t2=C_eqp(((C_word*)t0)[2],t1);
if(C_truep(C_i_not(t2))){
/* core.scm:656: walk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8362(t3,((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10]);}
else{
t3=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_8439,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[4],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],a[16]=((C_word*)t0)[16],a[17]=((C_word*)t0)[17],a[18]=((C_word*)t0)[18],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],tmp=(C_word)a,a+=22,tmp);
/* core.scm:658: chicken.internal#hash-table-ref */
t4=*((C_word*)lf[112]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=*((C_word*)lf[65]+1);
av2[3]=((C_word*)t0)[11];
((C_proc)(void*)(*((C_word*)t4+1)))(4,av2);}}}

/* k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8439(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,8)))){
C_save_and_reclaim((void *)f_8439,c,av);}
a=C_alloc(22);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8443,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
/* core.scm:655: g1052 */
t3=t2;
f_8443(t3,((C_word*)t0)[9],t1);}
else{
t2=C_eqp(((C_word*)t0)[10],lf[142]);
if(C_truep(t2)){
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_8471,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],tmp=(C_word)a,a+=8,tmp);
/* core.scm:667: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_8362(t4,t3,C_i_cadr(((C_word*)t0)[2]),((C_word*)t0)[4],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}
else{
t3=C_eqp(((C_word*)t0)[10],lf[145]);
t4=(C_truep(t3)?t3:C_eqp(((C_word*)t0)[10],lf[146]));
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8517,a[2]=((C_word*)t0)[9],tmp=(C_word)a,a+=3,tmp);
/* core.scm:674: chicken.syntax#strip-syntax */
t6=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t6;
av2[1]=t5;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_eqp(((C_word*)t0)[10],lf[148]);
if(C_truep(t5)){
if(C_truep(*((C_word*)lf[149]+1))){
t6=((C_word*)t0)[9];{
C_word *av2=av;
av2[0]=t6;
av2[1]=lf[150];
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
/* core.scm:679: walk */
t6=((C_word*)((C_word*)t0)[3])[1];
f_8362(t6,((C_word*)t0)[9],C_i_cadr(((C_word*)t0)[2]),((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[11]);}}
else{
t6=C_eqp(((C_word*)t0)[10],lf[151]);
if(C_truep(t6)){
t7=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8550,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[11],tmp=(C_word)a,a+=11,tmp);
/* core.scm:683: chicken.syntax#strip-syntax */
t8=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t8;
av2[1]=t7;
av2[2]=C_i_cadr(((C_word*)t0)[2]);
((C_proc)(void*)(*((C_word*)t8+1)))(3,av2);}}
else{
t7=C_eqp(((C_word*)t0)[10],lf[152]);
if(C_truep(t7)){
t8=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8575,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[12],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[13],tmp=(C_word)a,a+=10,tmp);
/* core.scm:688: resolve-variable */
t9=((C_word*)((C_word*)t0)[12])[1];
f_8054(t9,t8,C_i_cadr(((C_word*)t0)[2]),((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[13]);}
else{
t8=C_eqp(((C_word*)t0)[10],lf[156]);
if(C_truep(t8)){
t9=(C_truep(((C_word*)t0)[8])?((C_word*)t0)[8]:C_i_cadr(((C_word*)t0)[2]));
t10=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_8658,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[11],a[9]=((C_word*)t0)[2],a[10]=t9,a[11]=((C_word*)t0)[9],tmp=(C_word)a,a+=12,tmp);
/* core.scm:702: walk */
t11=((C_word*)((C_word*)t0)[3])[1];
f_8362(t11,t10,C_i_caddr(((C_word*)t0)[2]),((C_word*)t0)[4],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[11]);}
else{
t9=C_eqp(((C_word*)t0)[10],lf[157]);
if(C_truep(t9)){
t10=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8742,a[2]=((C_word*)t0)[9],tmp=(C_word)a,a+=3,tmp);
/* core.scm:709: scheme#cadadr */
t11=*((C_word*)lf[162]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t11;
av2[1]=t10;
av2[2]=((C_word*)t0)[2];
((C_proc)(void*)(*((C_word*)t11+1)))(3,av2);}}
else{
t10=C_eqp(((C_word*)t0)[10],lf[163]);
t11=(*a=C_CLOSURE_TYPE|21,a[1]=(C_word)f_8773,a[2]=((C_word*)t0)[9],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],a[11]=((C_word*)t0)[13],a[12]=((C_word*)t0)[14],a[13]=((C_word*)t0)[15],a[14]=((C_word*)t0)[12],a[15]=((C_word*)t0)[16],a[16]=((C_word*)t0)[17],a[17]=((C_word*)t0)[18],a[18]=((C_word*)t0)[11],a[19]=((C_word*)t0)[19],a[20]=((C_word*)t0)[20],a[21]=((C_word*)t0)[21],tmp=(C_word)a,a+=22,tmp);
if(C_truep(t10)){
t12=t11;
f_8773(t12,t10);}
else{
t12=C_eqp(((C_word*)t0)[10],lf[474]);
t13=t11;
f_8773(t13,(C_truep(t12)?t12:C_eqp(((C_word*)t0)[10],lf[475])));}}}}}}}}}}

/* g1052 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8443(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,0,8)))){
C_save_and_reclaim_args((void *)trf_8443,3,t0,t1,t2);}
a=C_alloc(3);
t3=C_i_cdr(((C_word*)t0)[2]);
t4=C_a_i_cons(&a,2,t2,t3);
/* core.scm:660: walk */
t5=((C_word*)((C_word*)t0)[3])[1];
f_8362(t5,t1,t4,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* k8469 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8471(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,8)))){
C_save_and_reclaim((void *)f_8471,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8475,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* core.scm:668: walk */
t3=((C_word*)((C_word*)t0)[4])[1];
f_8362(t3,t2,C_i_caddr(((C_word*)t0)[3]),((C_word*)t0)[5],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE);}

/* k8473 in k8469 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8475(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(17,c,8)))){
C_save_and_reclaim((void *)f_8475,c,av);}
a=C_alloc(17);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8479,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
t3=C_i_cdddr(((C_word*)t0)[4]);
if(C_truep(C_i_nullp(t3))){
t4=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_list(&a,4,lf[143],((C_word*)t0)[3],t1,lf[144]);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}
else{
/* core.scm:671: walk */
t4=((C_word*)((C_word*)t0)[5])[1];
f_8362(t4,t2,C_i_cadddr(((C_word*)t0)[4]),((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}}

/* k8477 in k8473 in k8469 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8479(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_8479,c,av);}
a=C_alloc(12);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,4,lf[143],((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8515 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8517(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_8517,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,2,lf[118],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8548 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8550(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,8)))){
C_save_and_reclaim((void *)f_8550,c,av);}
a=C_alloc(5);
t2=C_i_caddr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8558,a[2]=((C_word*)t0)[3],a[3]=t1,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:685: walk */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8362(t4,t3,C_i_cadddr(((C_word*)t0)[2]),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10]);}

/* k8556 in k8548 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8558(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_8558,c,av);}
a=C_alloc(12);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,4,lf[151],((C_word*)t0)[3],((C_word*)t0)[4],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8575(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,7)))){
C_save_and_reclaim((void *)f_8575,c,av);}
a=C_alloc(6);
t2=C_i_caddr(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8581,a[2]=t2,a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[2],tmp=(C_word)a,a+=6,tmp);
/* core.scm:690: resolve-variable */
t4=((C_word*)((C_word*)t0)[4])[1];
f_8054(t4,t3,t2,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k8579 in k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8581(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,4)))){
C_save_and_reclaim((void *)f_8581,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8584,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:691: ##sys#get */
t3=*((C_word*)lf[103]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[4];
av2[3]=lf[154];
av2[4]=C_SCHEME_END_OF_LIST;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k8582 in k8579 in k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8584(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_8584,c,av);}
a=C_alloc(18);
t2=C_SCHEME_UNDEFINED;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8585,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
t5=C_set_block_item(t3,0,t4);
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8617,a[2]=((C_word*)t0)[4],tmp=(C_word)a,a+=3,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8621,a[2]=t6,a[3]=((C_word*)t0)[5],tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8625,a[2]=t7,a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:696: resolve-alias */
t9=((C_word*)t3)[1];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t9;
av2[1]=t8;
av2[2]=C_i_cdddr(((C_word*)t0)[6]);
((C_proc)(void*)(*((C_word*)t9+1)))(3,av2);}}

/* resolve-alias1077 in k8582 in k8579 in k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_8585(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_8585,c,av);}
a=C_alloc(5);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8599,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,tmp=(C_word)a,a+=5,tmp);
/* core.scm:693: resolve-alias */
t4=((C_word*)((C_word*)t0)[2])[1];{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
av2[2]=C_u_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=C_eqp(t2,((C_word*)t0)[3]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=(C_truep(t3)?((C_word*)t0)[4]:t2);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}}

/* k8597 in resolve-alias1077 in k8582 in k8579 in k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_8599(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8599,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8603,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:693: resolve-alias */
t3=((C_word*)((C_word*)t0)[3])[1];{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=C_u_i_cdr(((C_word*)t0)[4]);
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8601 in k8597 in resolve-alias1077 in k8582 in k8579 in k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_8603(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_8603,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8615 in k8582 in k8579 in k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_8617(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8617,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[153];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8619 in k8582 in k8579 in k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_8621(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8621,c,av);}
/* core.scm:696: ##sys#put! */
t2=*((C_word*)lf[108]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=lf[154];
av2[4]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k8623 in k8582 in k8579 in k8573 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_8625(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_8625,c,av);}
/* core.scm:696: ##sys#append */
t2=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k8656 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8658(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,3)))){
C_save_and_reclaim((void *)f_8658,c,av);}
a=C_alloc(27);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8663,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],tmp=(C_word)a,a+=9,tmp);
t7=C_i_cdddr(((C_word*)t0)[9]);
t8=C_i_check_list_2(t7,lf[10]);
t9=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8692,a[2]=t1,a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[11],tmp=(C_word)a,a+=5,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8694,a[2]=t4,a[3]=t11,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_8694(t13,t9,t7);}

/* g1101 in k8656 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_8663(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,0,2)))){
C_save_and_reclaim_args((void *)trf_8663,3,t0,t1,t2);}
a=C_alloc(11);
t3=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8671,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],a[10]=((C_word*)t0)[8],tmp=(C_word)a,a+=11,tmp);
/* core.scm:704: chicken.syntax#strip-syntax */
t4=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_car(t2);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8669 in g1101 in k8656 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8671(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,8)))){
C_save_and_reclaim((void *)f_8671,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8675,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:705: walk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8362(t3,t2,C_i_cadr(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10]);}

/* k8673 in k8669 in g1101 in k8656 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_8675(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_8675,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8690 in k8656 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8692(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_8692,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,((C_word*)t0)[3],t2);
t4=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_a_i_cons(&a,2,lf[156],t3);
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* map-loop1095 in k8656 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_8694(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_8694,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8719,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:703: g1101 */
t4=((C_word*)t0)[4];
f_8663(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k8717 in map-loop1095 in k8656 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8719(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_8719,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_8694(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k8740 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_ccall f_8742(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8742,c,av);}
a=C_alloc(4);
t2=C_i_assoc(t1,*((C_word*)lf[64]+1));
if(C_truep(t2)){
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_i_cdr(t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}
else{
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8754,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:712: chicken.base#gensym */
t4=*((C_word*)lf[109]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=lf[161];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}}

/* k8752 in k8740 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8754(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,3)))){
C_save_and_reclaim((void *)f_8754,c,av);}
a=C_alloc(10);
t2=*((C_word*)lf[64]+1);
t3=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t4=C_a_i_cons(&a,2,t3,*((C_word*)lf[64]+1));
t5=C_mutate((C_word*)lf[64]+1 /* (set! chicken.compiler.core#immutable-constants ...) */,t4);
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8761,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:714: chicken.compiler.support#mark-variable */
t7=*((C_word*)lf[159]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t7;
av2[1]=t6;
av2[2]=t1;
av2[3]=lf[160];
((C_proc)(void*)(*((C_word*)t7+1)))(4,av2);}}

/* k8759 in k8752 in k8740 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8761(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_8761,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8764,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:715: chicken.compiler.support#hide-variable */
t3=*((C_word*)lf[158]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8762 in k8759 in k8752 in k8740 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_8764(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8764,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall f_8773(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word t16;
C_word t17;
C_word t18;
C_word t19;
C_word t20;
C_word t21;
C_word t22;
C_word t23;
C_word t24;
C_word t25;
C_word t26;
C_word t27;
C_word t28;
C_word t29;
C_word t30;
C_word t31;
C_word t32;
C_word t33;
C_word t34;
C_word t35;
C_word t36;
C_word t37;
C_word t38;
C_word t39;
C_word t40;
C_word t41;
C_word t42;
C_word t43;
C_word t44;
C_word t45;
C_word t46;
C_word t47;
C_word t48;
C_word t49;
C_word t50;
C_word t51;
C_word t52;
C_word t53;
C_word t54;
C_word t55;
C_word t56;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,0,8)))){
C_save_and_reclaim_args((void *)trf_8773,2,t0,t1);}
a=C_alloc(33);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=C_eqp(((C_word*)t0)[4],lf[164]);
if(C_truep(t2)){
t3=C_i_cadr(((C_word*)t0)[3]);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8786,a[2]=((C_word*)t0)[2],a[3]=t3,tmp=(C_word)a,a+=4,tmp);
/* core.scm:722: lset-adjoin/eq? */
f_7134(t4,*((C_word*)lf[21]+1),C_a_i_list(&a,1,t3));}
else{
t3=C_eqp(((C_word*)t0)[4],lf[119]);
if(C_truep(t3)){
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8806,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:727: scheme#caadr */
t5=*((C_word*)lf[165]+1);{
C_word av2[3];
av2[0]=t5;
av2[1]=t4;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t5+1)))(3,av2);}}
else{
t4=C_eqp(((C_word*)t0)[4],lf[123]);
if(C_truep(t4)){
t5=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8827,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
/* core.scm:731: chicken.syntax#strip-syntax */
t6=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t6;
av2[1]=t5;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t6+1)))(3,av2);}}
else{
t5=C_eqp(((C_word*)t0)[4],lf[166]);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8848,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:735: chicken.load#load-extension */
t7=*((C_word*)lf[168]+1);{
C_word av2[5];
av2[0]=t7;
av2[1]=t6;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t7+1)))(5,av2);}}
else{
t6=C_eqp(((C_word*)t0)[4],lf[169]);
if(C_truep(t6)){
t7=C_i_cadr(((C_word*)t0)[3]);
t8=C_u_i_cdr(((C_word*)t0)[3]);
t9=C_i_pairp(C_u_i_cdr(t8));
t10=(C_truep(t9)?C_i_caddr(((C_word*)t0)[3]):C_SCHEME_FALSE);
t11=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8869,a[2]=t7,a[3]=t10,tmp=(C_word)a,a+=4,tmp);
t12=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_8885,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=t7,tmp=(C_word)a,a+=9,tmp);
/* core.scm:741: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t11;
av2[3]=t12;
C_call_with_values(4,av2);}}
else{
t7=C_eqp(((C_word*)t0)[4],lf[173]);
if(C_truep(t7)){
t8=C_i_cadr(((C_word*)t0)[3]);
t9=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8917,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[9],a[6]=t8,a[7]=((C_word*)t0)[2],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[12],a[11]=((C_word*)t0)[3],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
/* core.scm:755: unzip1 */
f_7024(t9,t8);}
else{
t8=C_eqp(((C_word*)t0)[4],lf[176]);
if(C_truep(t8)){
t9=C_i_cadr(((C_word*)t0)[3]);
t10=C_u_i_cdr(((C_word*)t0)[3]);
t11=C_u_i_cdr(t10);
t12=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t13=t12;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=((C_word*)t14)[1];
t16=C_i_check_list_2(t9,lf[10]);
t17=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9145,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=t11,a[10]=t9,tmp=(C_word)a,a+=11,tmp);
t18=C_SCHEME_UNDEFINED;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_set_block_item(t19,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9217,a[2]=t14,a[3]=t19,a[4]=t15,tmp=(C_word)a,a+=5,tmp));
t21=((C_word*)t19)[1];
f_9217(t21,t17,t9);}
else{
t9=C_eqp(((C_word*)t0)[4],lf[179]);
if(C_truep(t9)){
t10=C_i_cadr(((C_word*)t0)[3]);
t11=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9261,a[2]=((C_word*)t0)[3],a[3]=t10,a[4]=((C_word*)t0)[10],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* core.scm:786: unzip1 */
f_7024(t11,t10);}
else{
t10=C_eqp(((C_word*)t0)[4],lf[180]);
if(C_truep(t10)){
t11=C_i_caddr(((C_word*)t0)[3]);
t12=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t13=t12;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=((C_word*)t14)[1];
t16=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9535,a[2]=t11,a[3]=((C_word*)t0)[14],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[11],tmp=(C_word)a,a+=9,tmp);
t17=C_u_i_cdr(((C_word*)t0)[3]);
t18=C_u_i_car(t17);
t19=C_i_check_list_2(t18,lf[10]);
t20=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9551,a[2]=((C_word*)t0)[15],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[2],tmp=(C_word)a,a+=11,tmp);
t21=C_SCHEME_UNDEFINED;
t22=(*a=C_VECTOR_TYPE|1,a[1]=t21,tmp=(C_word)a,a+=2,tmp);
t23=C_set_block_item(t22,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9579,a[2]=t14,a[3]=t22,a[4]=t16,a[5]=t15,tmp=(C_word)a,a+=6,tmp));
t24=((C_word*)t22)[1];
f_9579(t24,t20,t18);}
else{
t11=C_eqp(((C_word*)t0)[4],lf[97]);
if(C_truep(t11)){
t12=C_i_cadr(((C_word*)t0)[3]);
t13=t12;
t14=(*a=C_VECTOR_TYPE|1,a[1]=t13,tmp=(C_word)a,a+=2,tmp);
t15=C_u_i_cdr(((C_word*)t0)[3]);
t16=C_u_i_cdr(t15);
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_9625,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[13],a[7]=((C_word*)t0)[15],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[12],a[10]=t17,a[11]=((C_word*)t0)[3],a[12]=((C_word*)t0)[2],a[13]=t14,tmp=(C_word)a,a+=14,tmp);
t19=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9837,a[2]=t14,a[3]=t17,a[4]=t18,tmp=(C_word)a,a+=5,tmp);
/* core.scm:818: ##sys#extended-lambda-list? */
t20=*((C_word*)lf[194]+1);{
C_word av2[3];
av2[0]=t20;
av2[1]=t19;
av2[2]=((C_word*)t14)[1];
((C_proc)(void*)(*((C_word*)t20+1)))(3,av2);}}
else{
t12=C_eqp(((C_word*)t0)[4],lf[195]);
if(C_truep(t12)){
t13=*((C_word*)lf[107]+1);
t14=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9864,a[2]=t13,a[3]=((C_word*)t0)[11],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[3],a[11]=((C_word*)t0)[2],tmp=(C_word)a,a+=12,tmp);
t15=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t16=t15;
t17=(*a=C_VECTOR_TYPE|1,a[1]=t16,tmp=(C_word)a,a+=2,tmp);
t18=((C_word*)t17)[1];
t19=C_i_cadr(((C_word*)t0)[3]);
t20=C_i_check_list_2(t19,lf[10]);
t21=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_9954,a[2]=t14,tmp=(C_word)a,a+=3,tmp);
t22=C_SCHEME_UNDEFINED;
t23=(*a=C_VECTOR_TYPE|1,a[1]=t22,tmp=(C_word)a,a+=2,tmp);
t24=C_set_block_item(t23,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9960,a[2]=t17,a[3]=t23,a[4]=t18,tmp=(C_word)a,a+=5,tmp));
t25=((C_word*)t23)[1];
f_9960(t25,t21,t19);}
else{
t13=C_eqp(((C_word*)t0)[4],lf[198]);
if(C_truep(t13)){
t14=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t15=t14;
t16=(*a=C_VECTOR_TYPE|1,a[1]=t15,tmp=(C_word)a,a+=2,tmp);
t17=((C_word*)t16)[1];
t18=C_i_cadr(((C_word*)t0)[3]);
t19=C_i_check_list_2(t18,lf[10]);
t20=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10033,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[12],a[9]=((C_word*)t0)[3],a[10]=((C_word*)t0)[2],tmp=(C_word)a,a+=11,tmp);
t21=C_SCHEME_UNDEFINED;
t22=(*a=C_VECTOR_TYPE|1,a[1]=t21,tmp=(C_word)a,a+=2,tmp);
t23=C_set_block_item(t22,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10130,a[2]=t16,a[3]=t22,a[4]=t17,tmp=(C_word)a,a+=5,tmp));
t24=((C_word*)t22)[1];
f_10130(t24,t20,t18);}
else{
t14=C_eqp(((C_word*)t0)[4],lf[199]);
if(C_truep(t14)){
t15=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10171,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[16],a[10]=((C_word*)t0)[3],tmp=(C_word)a,a+=11,tmp);
t16=C_u_i_car(((C_word*)t0)[3]);
t17=C_i_cadr(((C_word*)t0)[3]);
t18=C_i_pairp(t17);
t19=(C_truep(t18)?lf[206]:lf[207]);
t20=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10261,a[2]=t15,a[3]=t16,a[4]=((C_word*)t0)[3],a[5]=t19,tmp=(C_word)a,a+=6,tmp);
/* core.scm:909: ##sys#current-environment */
t21=*((C_word*)lf[107]+1);{
C_word av2[2];
av2[0]=t21;
av2[1]=t20;
((C_proc)(void*)(*((C_word*)t21+1)))(2,av2);}}
else{
t15=C_eqp(((C_word*)t0)[4],lf[209]);
if(C_truep(t15)){
t16=C_i_cadr(((C_word*)t0)[3]);
t17=C_i_caddr(((C_word*)t0)[3]);
t18=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10283,a[2]=t17,a[3]=t16,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[2],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[17],tmp=(C_word)a,a+=12,tmp);
/* core.scm:931: lookup */
t19=((C_word*)((C_word*)t0)[16])[1];
f_7842(t19,t18,t16);}
else{
t16=C_eqp(((C_word*)t0)[4],lf[213]);
if(C_truep(t16)){
t17=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t18=t17;
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=((C_word*)t19)[1];
t21=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_10363,a[2]=((C_word*)t0)[16],tmp=(C_word)a,a+=3,tmp);
t22=C_i_cadr(((C_word*)t0)[3]);
t23=C_i_check_list_2(t22,lf[10]);
t24=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_10420,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[18],a[9]=((C_word*)t0)[12],a[10]=((C_word*)t0)[3],a[11]=((C_word*)t0)[2],tmp=(C_word)a,a+=12,tmp);
t25=C_SCHEME_UNDEFINED;
t26=(*a=C_VECTOR_TYPE|1,a[1]=t25,tmp=(C_word)a,a+=2,tmp);
t27=C_set_block_item(t26,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_10535,a[2]=t19,a[3]=t26,a[4]=t21,a[5]=t20,tmp=(C_word)a,a+=6,tmp));
t28=((C_word*)t26)[1];
f_10535(t28,t24,t22);}
else{
t17=C_eqp(((C_word*)t0)[4],lf[217]);
if(C_truep(t17)){
t18=*((C_word*)lf[218]+1);
t19=(*a=C_VECTOR_TYPE|1,a[1]=t18,tmp=(C_word)a,a+=2,tmp);
t20=C_SCHEME_FALSE;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10578,a[2]=t21,a[3]=t19,tmp=(C_word)a,a+=4,tmp);
t23=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_10583,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[18],a[10]=((C_word*)t0)[12],tmp=(C_word)a,a+=11,tmp);
t24=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_10628,a[2]=t19,a[3]=t21,tmp=(C_word)a,a+=4,tmp);
/* core.scm:996: ##sys#dynamic-wind */
t25=*((C_word*)lf[116]+1);{
C_word av2[5];
av2[0]=t25;
av2[1]=((C_word*)t0)[2];
av2[2]=t22;
av2[3]=t23;
av2[4]=t24;
((C_proc)(void*)(*((C_word*)t25+1)))(5,av2);}}
else{
t18=C_eqp(((C_word*)t0)[4],lf[221]);
if(C_truep(t18)){
t19=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t20=t19;
t21=(*a=C_VECTOR_TYPE|1,a[1]=t20,tmp=(C_word)a,a+=2,tmp);
t22=((C_word*)t21)[1];
t23=C_i_cadr(((C_word*)t0)[3]);
t24=C_i_check_list_2(t23,lf[10]);
t25=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10661,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[2],tmp=(C_word)a,a+=10,tmp);
t26=C_SCHEME_UNDEFINED;
t27=(*a=C_VECTOR_TYPE|1,a[1]=t26,tmp=(C_word)a,a+=2,tmp);
t28=C_set_block_item(t27,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_10677,a[2]=t21,a[3]=t27,a[4]=t22,tmp=(C_word)a,a+=5,tmp));
t29=((C_word*)t27)[1];
f_10677(t29,t25,t23);}
else{
t19=C_eqp(((C_word*)t0)[4],lf[225]);
if(C_truep(t19)){
t20=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_10718,a[2]=((C_word*)t0)[17],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[18],a[9]=((C_word*)t0)[3],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1019: chicken.syntax#strip-syntax */
t21=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t21;
av2[1]=t20;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t21+1)))(3,av2);}}
else{
t20=C_eqp(((C_word*)t0)[4],lf[274]);
if(C_truep(t20)){
t21=C_i_cadr(((C_word*)t0)[3]);
t22=C_u_i_cdr(((C_word*)t0)[3]);
t23=C_u_i_cdr(t22);
t24=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t25=t24;
t26=(*a=C_VECTOR_TYPE|1,a[1]=t25,tmp=(C_word)a,a+=2,tmp);
t27=((C_word*)t26)[1];
t28=C_i_check_list_2(t21,lf[10]);
t29=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_11294,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[13],a[5]=t21,a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[12],a[10]=t23,a[11]=((C_word*)t0)[3],tmp=(C_word)a,a+=12,tmp);
t30=C_SCHEME_UNDEFINED;
t31=(*a=C_VECTOR_TYPE|1,a[1]=t30,tmp=(C_word)a,a+=2,tmp);
t32=C_set_block_item(t31,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11359,a[2]=t26,a[3]=t31,a[4]=t27,tmp=(C_word)a,a+=5,tmp));
t33=((C_word*)t31)[1];
f_11359(t33,t29,t21);}
else{
t21=C_eqp(((C_word*)t0)[4],lf[275]);
if(C_truep(t21)){
t22=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_11400,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
if(C_truep(((C_word*)t0)[18])){
t23=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t23;
av2[1]=lf[276];
((C_proc)(void*)(*((C_word*)t23+1)))(2,av2);}}
else{
t23=C_i_cadr(((C_word*)t0)[3]);
t24=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11406,a[2]=t22,a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1120: lookup */
t25=((C_word*)((C_word*)t0)[16])[1];
f_7842(t25,t24,t23);}}
else{
t22=C_eqp(((C_word*)t0)[4],lf[177]);
if(C_truep(t22)){
t23=C_i_cadr(((C_word*)t0)[3]);
t24=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_11446,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[2],a[7]=t23,a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1130: lookup */
t25=((C_word*)((C_word*)t0)[16])[1];
f_7842(t25,t24,t23);}
else{
t23=C_eqp(((C_word*)t0)[4],lf[188]);
if(C_truep(t23)){
t24=C_i_cadr(((C_word*)t0)[3]);
t25=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t26=t25;
t27=(*a=C_VECTOR_TYPE|1,a[1]=t26,tmp=(C_word)a,a+=2,tmp);
t28=((C_word*)t27)[1];
t29=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11815,a[2]=((C_word*)t0)[19],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[18],tmp=(C_word)a,a+=8,tmp);
t30=C_u_i_cdr(((C_word*)t0)[3]);
t31=C_u_i_cdr(t30);
t32=C_i_check_list_2(t31,lf[10]);
t33=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_11831,a[2]=((C_word*)t0)[10],a[3]=t24,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
t34=C_SCHEME_UNDEFINED;
t35=(*a=C_VECTOR_TYPE|1,a[1]=t34,tmp=(C_word)a,a+=2,tmp);
t36=C_set_block_item(t35,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11833,a[2]=t27,a[3]=t35,a[4]=t29,a[5]=t28,tmp=(C_word)a,a+=6,tmp));
t37=((C_word*)t35)[1];
f_11833(t37,t33,t31);}
else{
t24=C_eqp(((C_word*)t0)[4],lf[300]);
if(C_truep(t24)){
t25=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11882,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[20],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],tmp=(C_word)a,a+=8,tmp);
/* core.scm:1195: unquotify */
t26=((C_word*)((C_word*)t0)[19])[1];
f_8011(t26,t25,C_i_cadr(((C_word*)t0)[3]));}
else{
t25=C_eqp(((C_word*)t0)[4],lf[301]);
if(C_truep(t25)){
t26=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t27=t26;
t28=(*a=C_VECTOR_TYPE|1,a[1]=t27,tmp=(C_word)a,a+=2,tmp);
t29=((C_word*)t28)[1];
t30=C_i_cadr(((C_word*)t0)[3]);
t31=C_i_check_list_2(t30,lf[10]);
t32=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_11920,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[20],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],tmp=(C_word)a,a+=8,tmp);
t33=C_SCHEME_UNDEFINED;
t34=(*a=C_VECTOR_TYPE|1,a[1]=t33,tmp=(C_word)a,a+=2,tmp);
t35=C_set_block_item(t34,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_11930,a[2]=t28,a[3]=t34,a[4]=((C_word*)t0)[19],a[5]=t29,tmp=(C_word)a,a+=6,tmp));
t36=((C_word*)t34)[1];
f_11930(t36,t32,t30);}
else{
t26=C_eqp(((C_word*)t0)[4],lf[280]);
if(C_truep(t26)){
t27=C_i_cadr(((C_word*)t0)[3]);
t28=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_11979,a[2]=((C_word*)t0)[2],a[3]=t27,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1203: walk */
t29=((C_word*)((C_word*)t0)[5])[1];
f_8362(t29,t28,C_i_caddr(((C_word*)t0)[3]),((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[9],((C_word*)t0)[10],C_SCHEME_FALSE);}
else{
t27=C_eqp(((C_word*)t0)[4],lf[283]);
if(C_truep(t27)){
t28=C_i_cadr(((C_word*)t0)[3]);
t29=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12000,a[2]=((C_word*)t0)[2],a[3]=t28,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1208: walk */
t30=((C_word*)((C_word*)t0)[5])[1];
f_8362(t30,t29,C_i_caddr(((C_word*)t0)[3]),((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[9],((C_word*)t0)[10],C_SCHEME_FALSE);}
else{
t28=C_eqp(((C_word*)t0)[4],lf[302]);
t29=(C_truep(t28)?t28:C_eqp(((C_word*)t0)[4],lf[303]));
if(C_truep(t29)){
t30=C_i_cadr(((C_word*)t0)[3]);
t31=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12027,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=t30,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[18],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1213: ##sys#eval/meta */
t32=*((C_word*)lf[197]+1);{
C_word av2[3];
av2[0]=t32;
av2[1]=t31;
av2[2]=t30;
((C_proc)(void*)(*((C_word*)t32+1)))(3,av2);}}
else{
t30=C_eqp(((C_word*)t0)[4],lf[304]);
t31=(C_truep(t30)?t30:C_eqp(((C_word*)t0)[4],lf[305]));
if(C_truep(t31)){
t32=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12042,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
/* core.scm:1217: ##sys#eval/meta */
t33=*((C_word*)lf[197]+1);{
C_word av2[3];
av2[0]=t33;
av2[1]=t32;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t33+1)))(3,av2);}}
else{
t32=C_eqp(((C_word*)t0)[4],lf[190]);
if(C_truep(t32)){
if(C_truep(C_i_pairp(C_u_i_cdr(((C_word*)t0)[3])))){
t33=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12065,a[2]=((C_word*)t0)[2],tmp=(C_word)a,a+=3,tmp);
t34=C_u_i_cdr(((C_word*)t0)[3]);
t35=C_SCHEME_UNDEFINED;
t36=(*a=C_VECTOR_TYPE|1,a[1]=t35,tmp=(C_word)a,a+=2,tmp);
t37=C_set_block_item(t36,0,(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12069,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[7],a[5]=((C_word*)t0)[8],a[6]=((C_word*)t0)[9],a[7]=((C_word*)t0)[10],a[8]=((C_word*)t0)[18],a[9]=t36,tmp=(C_word)a,a+=10,tmp));
t38=((C_word*)t36)[1];
f_12069(t38,t33,t34);}
else{
t33=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t33;
av2[1]=lf[307];
((C_proc)(void*)(*((C_word*)t33+1)))(2,av2);}}}
else{
t33=C_eqp(((C_word*)t0)[4],lf[308]);
if(C_truep(t33)){
t34=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12113,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1232: expand-foreign-lambda */
f_16114(t34,((C_word*)t0)[3],C_SCHEME_FALSE);}
else{
t34=C_eqp(((C_word*)t0)[4],lf[310]);
if(C_truep(t34)){
t35=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12126,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1235: expand-foreign-lambda */
f_16114(t35,((C_word*)t0)[3],C_SCHEME_TRUE);}
else{
t35=C_eqp(((C_word*)t0)[4],lf[311]);
if(C_truep(t35)){
t36=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12139,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1238: expand-foreign-lambda* */
f_16151(t36,((C_word*)t0)[3],C_SCHEME_FALSE);}
else{
t36=C_eqp(((C_word*)t0)[4],lf[313]);
if(C_truep(t36)){
t37=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12152,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1241: expand-foreign-lambda* */
f_16151(t37,((C_word*)t0)[3],C_SCHEME_TRUE);}
else{
t37=C_eqp(((C_word*)t0)[4],lf[314]);
if(C_truep(t37)){
t38=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12165,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],tmp=(C_word)a,a+=9,tmp);
t39=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_16268,a[2]=((C_word*)t0)[3],a[3]=t38,tmp=(C_word)a,a+=4,tmp);
t40=C_i_cddr(((C_word*)t0)[3]);
if(C_truep(C_i_pairp(t40))){
t41=C_i_caddr(((C_word*)t0)[3]);
t42=t39;
f_16268(t42,C_i_not(C_i_stringp(t41)));}
else{
t41=t39;
f_16268(t41,C_SCHEME_FALSE);}}
else{
t38=C_eqp(((C_word*)t0)[4],lf[319]);
if(C_truep(t38)){
t39=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12174,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:1247: chicken.syntax#strip-syntax */
t40=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t40;
av2[1]=t39;
av2[2]=C_i_cadr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t40+1)))(3,av2);}}
else{
t39=C_eqp(((C_word*)t0)[4],lf[321]);
if(C_truep(t39)){
t40=C_i_cadr(((C_word*)t0)[3]);
t41=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_12221,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[18],a[11]=t40,tmp=(C_word)a,a+=12,tmp);
/* core.scm:1259: chicken.syntax#strip-syntax */
t42=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t42;
av2[1]=t41;
av2[2]=C_i_caddr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t42+1)))(3,av2);}}
else{
t40=C_eqp(((C_word*)t0)[4],lf[329]);
if(C_truep(t40)){
t41=C_i_cadr(((C_word*)t0)[3]);
t42=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12343,a[2]=((C_word*)t0)[3],a[3]=t41,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[18],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1292: chicken.compiler.support#get-line */
t43=*((C_word*)lf[113]+1);{
C_word av2[3];
av2[0]=t43;
av2[1]=t42;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t43+1)))(3,av2);}}
else{
t41=C_eqp(((C_word*)t0)[4],lf[338]);
if(C_truep(t41)){
t42=C_i_cadr(((C_word*)t0)[3]);
t43=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_12430,a[2]=((C_word*)t0)[3],a[3]=t42,a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[2],tmp=(C_word)a,a+=11,tmp);
/* core.scm:1312: chicken.syntax#strip-syntax */
t44=*((C_word*)lf[147]+1);{
C_word av2[3];
av2[0]=t44;
av2[1]=t43;
av2[2]=C_i_caddr(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t44+1)))(3,av2);}}
else{
t42=C_eqp(((C_word*)t0)[4],lf[342]);
if(C_truep(t42)){
t43=C_i_cadr(((C_word*)t0)[3]);
t44=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12625,a[2]=((C_word*)t0)[2],a[3]=t43,a[4]=((C_word*)t0)[18],a[5]=((C_word*)t0)[3],tmp=(C_word)a,a+=6,tmp);
/* core.scm:1340: scheme#cdaddr */
t45=*((C_word*)lf[347]+1);{
C_word av2[3];
av2[0]=t45;
av2[1]=t44;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t45+1)))(3,av2);}}
else{
t43=C_eqp(((C_word*)t0)[4],lf[348]);
if(C_truep(t43)){
t44=C_i_cadr(((C_word*)t0)[3]);
t45=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_12637,a[2]=((C_word*)t0)[3],a[3]=t44,a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[18],tmp=(C_word)a,a+=9,tmp);
/* core.scm:1352: chicken.compiler.support#get-line */
t46=*((C_word*)lf[113]+1);{
C_word av2[3];
av2[0]=t46;
av2[1]=t45;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t46+1)))(3,av2);}}
else{
t44=C_eqp(((C_word*)t0)[4],lf[363]);
if(C_truep(t44)){
t45=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t46=t45;
t47=(*a=C_VECTOR_TYPE|1,a[1]=t46,tmp=(C_word)a,a+=2,tmp);
t48=((C_word*)t47)[1];
t49=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_12836,a[2]=((C_word*)t0)[6],a[3]=((C_word*)t0)[16],tmp=(C_word)a,a+=4,tmp);
t50=C_u_i_cdr(((C_word*)t0)[3]);
t51=C_i_check_list_2(t50,lf[10]);
t52=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_12857,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[10],tmp=(C_word)a,a+=7,tmp);
t53=C_SCHEME_UNDEFINED;
t54=(*a=C_VECTOR_TYPE|1,a[1]=t53,tmp=(C_word)a,a+=2,tmp);
t55=C_set_block_item(t54,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_12859,a[2]=t47,a[3]=t54,a[4]=t49,a[5]=t48,tmp=(C_word)a,a+=6,tmp));
t56=((C_word*)t54)[1];
f_12859(t56,t52,t50);}
else{
t45=C_eqp(((C_word*)t0)[4],lf[446]);
if(C_truep(t45)){
t46=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_12902,a[2]=((C_word*)t0)[3],tmp=(C_word)a,a+=3,tmp);
t47=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_12912,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[21],a[7]=((C_word*)t0)[20],a[8]=((C_word*)t0)[3],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* core.scm:1396: ##sys#call-with-values */{
C_word av2[4];
av2[0]=0;
av2[1]=((C_word*)t0)[2];
av2[2]=t46;
av2[3]=t47;
C_call_with_values(4,av2);}}
else{
t46=C_eqp(((C_word*)t0)[4],lf[470]);
if(C_truep(t46)){
t47=C_i_cadr(((C_word*)t0)[3]);
if(C_truep(C_i_symbolp(t47))){
t48=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13293,a[2]=((C_word*)t0)[5],a[3]=((C_word*)t0)[6],a[4]=((C_word*)t0)[9],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[2],a[7]=t47,tmp=(C_word)a,a+=8,tmp);
/* core.scm:1480: lookup */
t49=((C_word*)((C_word*)t0)[16])[1];
f_7842(t49,t48,t47);}
else{
t48=C_a_i_list(&a,2,lf[146],lf[471]);
t49=C_a_i_list(&a,5,lf[472],t47,C_fix(0),C_SCHEME_FALSE,t48);
/* core.scm:1493: walk */
t50=((C_word*)((C_word*)t0)[5])[1];
f_8362(t50,((C_word*)t0)[2],t49,((C_word*)t0)[6],C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[9],((C_word*)t0)[10],C_SCHEME_FALSE);}}
else{
t47=C_a_i_cons(&a,2,((C_word*)t0)[4],*((C_word*)lf[473]+1));
t48=t47;
t49=(*a=C_VECTOR_TYPE|1,a[1]=t48,tmp=(C_word)a,a+=2,tmp);
t50=C_SCHEME_FALSE;
t51=(*a=C_VECTOR_TYPE|1,a[1]=t50,tmp=(C_word)a,a+=2,tmp);
t52=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_13310,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[10],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
t53=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13339,a[2]=t51,a[3]=t49,tmp=(C_word)a,a+=4,tmp);
t54=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_13344,a[2]=((C_word*)t0)[20],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[9],a[6]=((C_word*)t0)[10],a[7]=((C_word*)t0)[18],tmp=(C_word)a,a+=8,tmp);
t55=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_13350,a[2]=t49,a[3]=t51,tmp=(C_word)a,a+=4,tmp);
/* core.scm:1498: ##sys#dynamic-wind */
t56=*((C_word*)lf[116]+1);{
C_word av2[5];
av2[0]=t56;
av2[1]=t52;
av2[2]=t53;
av2[3]=t54;
av2[4]=t55;
((C_proc)(void*)(*((C_word*)t56+1)))(5,av2);}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}

/* k8784 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8786(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_8786,c,av);}
a=C_alloc(6);
t2=C_mutate((C_word*)lf[21]+1 /* (set! chicken.compiler.core#used-units ...) */,t1);
t3=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,2,lf[164],((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k8804 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8806(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_8806,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8810,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_8814,a[2]=t2,tmp=(C_word)a,a+=3,tmp);
/* core.scm:727: scheme#cadadr */
t4=*((C_word*)lf[162]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k8808 in k8804 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8810(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,1)))){
C_save_and_reclaim((void *)f_8810,c,av);}
a=C_alloc(12);
t2=C_a_i_list(&a,2,((C_word*)t0)[2],t1);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=C_a_i_list(&a,2,lf[119],t2);
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k8812 in k8804 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8814(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,2)))){
C_save_and_reclaim((void *)f_8814,c,av);}
/* core.scm:727: chicken.syntax#strip-syntax */
t2=*((C_word*)lf[147]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(3,av2);}}

/* k8825 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8827(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,8)))){
C_save_and_reclaim((void *)f_8827,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8831,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:732: walk */
t3=((C_word*)((C_word*)t0)[3])[1];
f_8362(t3,t2,C_i_caddr(((C_word*)t0)[4]),((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_FALSE);}

/* k8829 in k8825 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8831(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_8831,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[123],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8846 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8848(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8848,c,av);}
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=lf[167];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a8868 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_8869,c,av);}
t2=C_i_memq(((C_word*)t0)[2],*((C_word*)lf[81]+1));
t3=(C_truep(t2)?t2:*((C_word*)lf[56]+1));
if(C_truep(t3)){
/* core.scm:742: ##sys#process-require */
t4=*((C_word*)lf[170]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[171];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}
else{
/* core.scm:742: ##sys#process-require */
t4=*((C_word*)lf[170]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t1;
av2[2]=((C_word*)t0)[2];
av2[3]=((C_word*)t0)[3];
av2[4]=lf[172];
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}}

/* a8884 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8885(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(13,c,8)))){
C_save_and_reclaim((void *)f_8885,c,av);}
a=C_alloc(13);
if(C_truep(t3)){
/* core.scm:751: walk */
t4=((C_word*)((C_word*)t0)[2])[1];
f_8362(t4,t1,t2,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],C_SCHEME_FALSE);}
else{
t4=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_8896,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=t2,a[5]=((C_word*)t0)[3],a[6]=((C_word*)t0)[4],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],a[9]=((C_word*)t0)[7],tmp=(C_word)a,a+=10,tmp);
/* core.scm:750: lset-adjoin/eq? */
f_7134(t4,*((C_word*)lf[80]+1),C_a_i_list(&a,1,((C_word*)t0)[8]));}}

/* k8894 in a8884 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8896(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_8896,c,av);}
t2=C_mutate((C_word*)lf[80]+1 /* (set! chicken.compiler.core#required-libraries ...) */,t1);
/* core.scm:751: walk */
t3=((C_word*)((C_word*)t0)[2])[1];
f_8362(t3,((C_word*)t0)[3],((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_FALSE);}

/* k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_8917(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_8917,c,av);}
a=C_alloc(26);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(t1,lf[10]);
t7=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_8926,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=t1,tmp=(C_word)a,a+=14,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9072,a[2]=t4,a[3]=t9,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_9072(t11,t7,t1);}

/* k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_8926(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(20,c,2)))){
C_save_and_reclaim((void *)f_8926,c,av);}
a=C_alloc(20);
t2=(*a=C_CLOSURE_TYPE|14,a[1]=(C_word)f_8929,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],tmp=(C_word)a,a+=15,tmp);
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9070,a[2]=t2,a[3]=((C_word*)t0)[13],a[4]=t1,tmp=(C_word)a,a+=5,tmp);
/* core.scm:757: ##sys#current-environment */
t4=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t4;
av2[1]=t3;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_8929(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_8929,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_8932,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* core.scm:758: chicken.compiler.support#get-line */
t3=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[12];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_8932(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(14,c,3)))){
C_save_and_reclaim((void *)f_8932,c,av);}
a=C_alloc(14);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|13,a[1]=(C_word)f_8938,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=t2,a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],a[12]=((C_word*)t0)[12],a[13]=((C_word*)t0)[13],tmp=(C_word)a,a+=14,tmp);
/* core.scm:759: set-real-names! */
f_7945(t3,((C_word*)t0)[6],((C_word*)t0)[15]);}

/* k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_8938(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,4)))){
C_save_and_reclaim((void *)f_8938,c,av);}
a=C_alloc(32);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8946,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],tmp=(C_word)a,a+=6,tmp);
t7=C_i_check_list_2(((C_word*)t0)[6],lf[10]);
t8=C_i_check_list_2(((C_word*)t0)[7],lf[10]);
t9=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_8969,a[2]=((C_word*)t0)[8],a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[10],a[6]=((C_word*)t0)[11],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[5],a[9]=((C_word*)t0)[6],a[10]=((C_word*)t0)[3],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],tmp=(C_word)a,a+=13,tmp);
t10=C_SCHEME_UNDEFINED;
t11=(*a=C_VECTOR_TYPE|1,a[1]=t10,tmp=(C_word)a,a+=2,tmp);
t12=C_set_block_item(t11,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9020,a[2]=t4,a[3]=t11,a[4]=t6,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t13=((C_word*)t11)[1];
f_9020(t13,t9,((C_word*)t0)[6],((C_word*)t0)[7]);}

/* g1209 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_8946(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,8)))){
C_save_and_reclaim_args((void *)trf_8946,4,t0,t1,t2,t3);}
a=C_alloc(4);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8954,a[2]=t1,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:762: walk */
t5=((C_word*)((C_word*)t0)[2])[1];
f_8362(t5,t4,C_i_cadr(t3),((C_word*)t0)[3],C_u_i_car(t3),C_SCHEME_TRUE,((C_word*)t0)[4],((C_word*)t0)[5],C_SCHEME_FALSE);}

/* k8952 in g1209 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_8954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,1)))){
C_save_and_reclaim((void *)f_8954,c,av);}
a=C_alloc(6);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list2(&a,2,((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_8969(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,4)))){
C_save_and_reclaim((void *)f_8969,c,av);}
a=C_alloc(28);
t2=*((C_word*)lf[107]+1);
t3=((C_word*)t0)[2];
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_TRUE;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8973,a[2]=((C_word*)t0)[3],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_8975,a[2]=t4,a[3]=t6,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_8992,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[11],a[10]=((C_word*)t0)[12],tmp=(C_word)a,a+=11,tmp);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9010,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:764: ##sys#dynamic-wind */
t11=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t11;
av2[1]=t7;
av2[2]=t8;
av2[3]=t9;
av2[4]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(5,av2);}}

/* k8971 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_8973(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,1)))){
C_save_and_reclaim((void *)f_8973,c,av);}
a=C_alloc(9);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_list(&a,3,lf[174],((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* a8974 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_8975(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_8975,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_8979,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[3])[1])){
/* core.scm:764: ##sys#current-environment1235 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_8979(2,av2);}}}

/* k8977 in a8974 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_8979(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_8979,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_8983,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* core.scm:764: ##sys#current-environment1235 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k8981 in k8977 in a8974 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_8983(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_8983,c,av);}
a=C_alloc(4);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_8986,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:764: ##sys#current-environment1235 */
t4=((C_word*)t0)[5];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k8984 in k8981 in k8977 in a8974 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_8986(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_8986,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a8991 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_8992(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(10,c,4)))){
C_save_and_reclaim((void *)f_8992,c,av);}
a=C_alloc(10);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9000,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
/* core.scm:765: canonicalize-body/ln */
t3=((C_word*)((C_word*)t0)[9])[1];
f_7916(t3,t2,((C_word*)t0)[6],C_i_cddr(((C_word*)t0)[10]),*((C_word*)lf[52]+1));}

/* k8998 in a8991 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_9000(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_9000,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9004,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],tmp=(C_word)a,a+=9,tmp);
/* core.scm:767: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[8];
av2[3]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k9002 in k8998 in a8991 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_9004(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_9004,c,av);}
/* core.scm:765: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* a9009 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_9010(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9010,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9014,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:764: ##sys#current-environment1235 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9012 in a9009 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_9014(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_9014,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9017,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:764: ##sys#current-environment1235 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9015 in k9012 in a9009 in k8967 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_9017(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9017,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* map-loop1203 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_fcall f_9020(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_9020,4,t0,t1,t2,t3);}
a=C_alloc(7);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9049,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,a[6]=t3,tmp=(C_word)a,a+=7,tmp);
/* core.scm:761: g1209 */
t7=((C_word*)t0)[4];
f_8946(t7,t6,C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)));}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* k9047 in map-loop1203 in k8936 in k8930 in k8927 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_9049(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,3)))){
C_save_and_reclaim((void *)f_9049,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9020(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)),C_slot(((C_word*)t0)[6],C_fix(1)));}

/* k9068 in k8924 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9070(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9070,c,av);}
/* core.scm:757: ##sys#extend-se */
t2=*((C_word*)lf[175]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop1169 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_9072(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9072,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9097,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:756: g1175 */
t4=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9095 in map-loop1169 in k8915 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9097(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9097,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9072(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9143 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_9145(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(26,c,3)))){
C_save_and_reclaim((void *)f_9145,c,av);}
a=C_alloc(26);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9149,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9169,a[2]=((C_word*)t0)[9],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9183,a[2]=t5,a[3]=t9,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_9183(t11,t7,((C_word*)t0)[10]);}

/* k9147 in k9143 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9149(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,8)))){
C_save_and_reclaim((void *)f_9149,c,av);}
a=C_alloc(6);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,lf[173],t2);
/* core.scm:773: walk */
t4=((C_word*)((C_word*)t0)[3])[1];
f_8362(t4,((C_word*)t0)[4],t3,((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],C_SCHEME_FALSE);}

/* k9167 in k9143 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9169(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_9169,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[173],t2);
t4=C_a_i_list(&a,1,t3);
/* core.scm:774: ##sys#append */
t5=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* map-loop1284 in k9143 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_9183(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(12,0,2)))){
C_save_and_reclaim_args((void *)trf_9183,3,t0,t1,t2);}
a=C_alloc(12);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_i_cadr(t3);
t6=C_a_i_list(&a,3,lf[177],t4,t5);
t7=C_a_i_cons(&a,2,t6,C_SCHEME_END_OF_LIST);
t8=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t7);
t9=C_mutate(((C_word *)((C_word*)t0)[2])+1,t7);
t11=t1;
t12=C_slot(t2,C_fix(1));
t1=t11;
t2=t12;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* map-loop1257 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_9217(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_9217,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_list2(&a,2,t4,lf[178]);
t6=C_a_i_cons(&a,2,t5,C_SCHEME_END_OF_LIST);
t7=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t6);
t8=C_mutate(((C_word *)((C_word*)t0)[2])+1,t6);
t10=t1;
t11=C_slot(t2,C_fix(1));
t1=t10;
t2=t11;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_9261(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(24,c,3)))){
C_save_and_reclaim((void *)f_9261,c,av);}
a=C_alloc(24);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=C_i_check_list_2(t1,lf[10]);
t7=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9270,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],tmp=(C_word)a,a+=12,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9490,a[2]=t4,a[3]=t9,a[4]=t5,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_9490(t11,t7,t1);}

/* k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9270(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(25,c,3)))){
C_save_and_reclaim((void *)f_9270,c,av);}
a=C_alloc(25);
t2=C_i_cddr(((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=C_i_check_list_2(((C_word*)t0)[3],lf[10]);
t8=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9299,a[2]=((C_word*)t0)[4],a[3]=((C_word*)t0)[5],a[4]=t1,a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=t2,a[12]=((C_word*)t0)[3],tmp=(C_word)a,a+=13,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9456,a[2]=t5,a[3]=t10,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t12=((C_word*)t10)[1];
f_9456(t12,t8,((C_word*)t0)[3]);}

/* k9297 in k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9299(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(30,c,4)))){
C_save_and_reclaim((void *)f_9299,c,av);}
a=C_alloc(30);
t2=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t3=t2;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=((C_word*)t4)[1];
t6=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9312,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t7=C_i_check_list_2(((C_word*)t0)[4],lf[10]);
t8=(*a=C_CLOSURE_TYPE|12,a[1]=(C_word)f_9330,a[2]=t1,a[3]=((C_word*)t0)[5],a[4]=((C_word*)t0)[6],a[5]=((C_word*)t0)[7],a[6]=((C_word*)t0)[8],a[7]=((C_word*)t0)[9],a[8]=((C_word*)t0)[10],a[9]=((C_word*)t0)[3],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[2],a[12]=((C_word*)t0)[4],tmp=(C_word)a,a+=13,tmp);
t9=C_SCHEME_UNDEFINED;
t10=(*a=C_VECTOR_TYPE|1,a[1]=t9,tmp=(C_word)a,a+=2,tmp);
t11=C_set_block_item(t10,0,(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9408,a[2]=t6,a[3]=t4,a[4]=t10,a[5]=t5,tmp=(C_word)a,a+=6,tmp));
t12=((C_word*)t10)[1];
f_9408(t12,t8,((C_word*)t0)[4],((C_word*)t0)[12]);}

/* g1380 in k9297 in k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static C_word C_fcall f_9312(C_word *a,C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_stack_overflow_check;{}
t3=C_i_cadr(t2);
t4=C_a_i_list(&a,4,lf[180],((C_word*)t0)[2],((C_word*)t0)[3],t3);
return(C_a_i_list2(&a,2,t1,t4));}

/* k9328 in k9297 in k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_9330(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(27,c,4)))){
C_save_and_reclaim((void *)f_9330,c,av);}
a=C_alloc(27);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9334,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
t3=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t4=t3;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=((C_word*)t5)[1];
t7=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9346,a[2]=((C_word*)t0)[10],a[3]=t2,tmp=(C_word)a,a+=4,tmp);
t8=C_SCHEME_UNDEFINED;
t9=(*a=C_VECTOR_TYPE|1,a[1]=t8,tmp=(C_word)a,a+=2,tmp);
t10=C_set_block_item(t9,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9360,a[2]=t5,a[3]=t9,a[4]=t6,tmp=(C_word)a,a+=5,tmp));
t11=((C_word*)t9)[1];
f_9360(t11,t7,((C_word*)t0)[11],((C_word*)t0)[12]);}

/* k9332 in k9328 in k9297 in k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_9334(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,8)))){
C_save_and_reclaim((void *)f_9334,c,av);}
a=C_alloc(15);
t2=C_a_i_cons(&a,2,((C_word*)t0)[2],t1);
t3=C_a_i_cons(&a,2,lf[173],t2);
t4=C_a_i_list(&a,3,lf[173],((C_word*)t0)[3],t3);
/* core.scm:789: walk */
t5=((C_word*)((C_word*)t0)[4])[1];
f_8362(t5,((C_word*)t0)[5],t4,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10],C_SCHEME_FALSE);}

/* k9344 in k9328 in k9297 in k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_9346(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,3)))){
C_save_and_reclaim((void *)f_9346,c,av);}
a=C_alloc(9);
t2=C_a_i_cons(&a,2,C_SCHEME_END_OF_LIST,((C_word*)t0)[2]);
t3=C_a_i_cons(&a,2,lf[173],t2);
t4=C_a_i_list(&a,1,t3);
/* core.scm:790: ##sys#append */
t5=*((C_word*)lf[155]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t5;
av2[1]=((C_word*)t0)[3];
av2[2]=t1;
av2[3]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}

/* map-loop1410 in k9328 in k9297 in k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_fcall f_9360(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word t15;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(12,0,3)))){
C_save_and_reclaim_args((void *)trf_9360,4,t0,t1,t2,t3);}
a=C_alloc(12);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=C_slot(t2,C_fix(0));
t7=C_slot(t3,C_fix(0));
t8=C_a_i_list(&a,3,lf[177],t6,t7);
t9=C_a_i_cons(&a,2,t8,C_SCHEME_END_OF_LIST);
t10=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t9);
t11=C_mutate(((C_word *)((C_word*)t0)[2])+1,t9);
t13=t1;
t14=C_slot(t2,C_fix(1));
t15=C_slot(t3,C_fix(1));
t1=t13;
t2=t14;
t3=t15;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* map-loop1374 in k9297 in k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_fcall f_9408(C_word t0,C_word t1,C_word t2,C_word t3){
C_word tmp;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(21,0,3)))){
C_save_and_reclaim_args((void *)trf_9408,4,t0,t1,t2,t3);}
a=C_alloc(21);
t4=C_i_pairp(t2);
t5=(C_truep(t4)?C_i_pairp(t3):C_SCHEME_FALSE);
if(C_truep(t5)){
t6=(
/* core.scm:795: g1380 */
  f_9312(C_a_i(&a,18),((C_word*)t0)[2],C_slot(t2,C_fix(0)),C_slot(t3,C_fix(0)))
);
t7=C_a_i_cons(&a,2,t6,C_SCHEME_END_OF_LIST);
t8=C_i_setslot(((C_word*)((C_word*)t0)[3])[1],C_fix(1),t7);
t9=C_mutate(((C_word *)((C_word*)t0)[3])+1,t7);
t11=t1;
t12=C_slot(t2,C_fix(1));
t13=C_slot(t3,C_fix(1));
t1=t11;
t2=t12;
t3=t13;
goto loop;}
else{
t6=t1;{
C_word av2[2];
av2[0]=t6;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}}

/* map-loop1347 in k9268 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_9456(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
loop:
if(C_unlikely(!C_demand(C_calculate_demand(9,0,2)))){
C_save_and_reclaim_args((void *)trf_9456,3,t0,t1,t2);}
a=C_alloc(9);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=C_a_i_list2(&a,2,t4,lf[181]);
t6=C_a_i_cons(&a,2,t5,C_SCHEME_END_OF_LIST);
t7=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t6);
t8=C_mutate(((C_word *)((C_word*)t0)[2])+1,t6);
t10=t1;
t11=C_slot(t2,C_fix(1));
t1=t10;
t2=t11;
goto loop;}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* map-loop1317 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_fcall f_9490(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9490,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9515,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:787: g1323 */
t4=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9513 in map-loop1317 in k9259 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9515(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9515,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9490(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* g1454 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_9535(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,0,7)))){
C_save_and_reclaim_args((void *)trf_9535,3,t0,t1,t2);}
a=C_alloc(4);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9543,a[2]=t1,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:808: resolve-variable */
t4=((C_word*)((C_word*)t0)[3])[1];
f_8054(t4,t3,t2,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8]);}

/* k9541 in g1454 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9543(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,1)))){
C_save_and_reclaim((void *)f_9543,c,av);}
a=C_alloc(3);
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_a_i_cons(&a,2,t1,((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}

/* k9549 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_9551(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,3)))){
C_save_and_reclaim((void *)f_9551,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9554,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
/* core.scm:812: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=t1;
av2[3]=((C_word*)((C_word*)t0)[2])[1];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k9552 in k9549 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9554(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,4)))){
C_save_and_reclaim((void *)f_9554,c,av);}
a=C_alloc(23);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9559,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t7=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9564,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],tmp=(C_word)a,a+=9,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9574,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* core.scm:811: ##sys#dynamic-wind */
t9=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[10];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a9558 in k9552 in k9549 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9559(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9559,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a9563 in k9552 in k9549 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9564(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_9564,c,av);}
/* core.scm:813: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,t1,C_i_cadddr(((C_word*)t0)[3]),((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* a9573 in k9552 in k9549 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9574(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9574,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* map-loop1448 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_9579(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9579,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9604,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:807: g1454 */
t4=((C_word*)t0)[4];
f_9535(t4,t3,C_slot(t2,C_fix(0)));}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[5],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9602 in map-loop1448 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9604(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9604,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9579(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_9625(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,5)))){
C_save_and_reclaim((void *)f_9625,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|11,a[1]=(C_word)f_9630,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],a[11]=((C_word*)t0)[11],tmp=(C_word)a,a+=12,tmp);
/* core.scm:823: ##sys#decompose-lambda-list */
t3=*((C_word*)lf[191]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[12];
av2[2]=((C_word*)((C_word*)t0)[13])[1];
av2[3]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9630(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4=av[4];
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word t12;
C_word t13;
C_word t14;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(28,c,3)))){
C_save_and_reclaim((void *)f_9630,c,av);}
a=C_alloc(28);
t5=C_a_i_cons(&a,2,C_SCHEME_UNDEFINED,C_SCHEME_END_OF_LIST);
t6=t5;
t7=(*a=C_VECTOR_TYPE|1,a[1]=t6,tmp=(C_word)a,a+=2,tmp);
t8=((C_word*)t7)[1];
t9=C_i_check_list_2(t2,lf[10]);
t10=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_9640,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=t2,a[9]=t4,a[10]=t3,a[11]=((C_word*)t0)[7],a[12]=((C_word*)t0)[8],a[13]=((C_word*)t0)[9],a[14]=((C_word*)t0)[10],a[15]=((C_word*)t0)[11],tmp=(C_word)a,a+=16,tmp);
t11=C_SCHEME_UNDEFINED;
t12=(*a=C_VECTOR_TYPE|1,a[1]=t11,tmp=(C_word)a,a+=2,tmp);
t13=C_set_block_item(t12,0,(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9802,a[2]=t7,a[3]=t12,a[4]=t8,tmp=(C_word)a,a+=5,tmp));
t14=((C_word*)t12)[1];
f_9802(t14,t10,t2);}

/* k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9640(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(16,c,2)))){
C_save_and_reclaim((void *)f_9640,c,av);}
a=C_alloc(16);
t2=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_9643,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],a[11]=((C_word*)t0)[10],a[12]=((C_word*)t0)[11],a[13]=((C_word*)t0)[12],a[14]=((C_word*)t0)[13],a[15]=((C_word*)t0)[14],tmp=(C_word)a,a+=16,tmp);
/* core.scm:827: chicken.compiler.support#get-line */
t3=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[15];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_9643(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(21,c,2)))){
C_save_and_reclaim((void *)f_9643,c,av);}
a=C_alloc(21);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|15,a[1]=(C_word)f_9649,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],a[10]=((C_word*)t0)[11],a[11]=((C_word*)t0)[12],a[12]=((C_word*)t0)[13],a[13]=t2,a[14]=((C_word*)t0)[14],a[15]=((C_word*)t0)[15],tmp=(C_word)a,a+=16,tmp);
t4=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9800,a[2]=t3,a[3]=((C_word*)t0)[9],a[4]=((C_word*)t0)[8],tmp=(C_word)a,a+=5,tmp);
/* core.scm:828: ##sys#current-environment */
t5=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}

/* k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_9649(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word t10;
C_word t11;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(34,c,4)))){
C_save_and_reclaim((void *)f_9649,c,av);}
a=C_alloc(34);
t2=*((C_word*)lf[107]+1);
t3=t1;
t4=(*a=C_VECTOR_TYPE|1,a[1]=t3,tmp=(C_word)a,a+=2,tmp);
t5=C_SCHEME_TRUE;
t6=(*a=C_VECTOR_TYPE|1,a[1]=t5,tmp=(C_word)a,a+=2,tmp);
t7=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9652,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=((C_word*)t0)[8],a[9]=((C_word*)t0)[9],a[10]=((C_word*)t0)[10],tmp=(C_word)a,a+=11,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9727,a[2]=t4,a[3]=t6,a[4]=t2,tmp=(C_word)a,a+=5,tmp);
t9=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9744,a[2]=((C_word*)t0)[11],a[3]=((C_word*)t0)[12],a[4]=((C_word*)t0)[2],a[5]=((C_word*)t0)[13],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[4],a[8]=((C_word*)t0)[14],a[9]=((C_word*)t0)[15],tmp=(C_word)a,a+=10,tmp);
t10=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9788,a[2]=t4,a[3]=t2,tmp=(C_word)a,a+=4,tmp);
/* core.scm:829: ##sys#dynamic-wind */
t11=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t11;
av2[1]=t7;
av2[2]=t8;
av2[3]=t9;
av2[4]=t10;
((C_proc)(void*)(*((C_word*)t11+1)))(5,av2);}}

/* k9650 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_9652(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(15,c,4)))){
C_save_and_reclaim((void *)f_9652,c,av);}
a=C_alloc(15);
t2=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9655,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],tmp=(C_word)a,a+=10,tmp);
if(C_truep(((C_word*)t0)[9])){
t3=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9725,a[2]=t2,a[3]=((C_word*)t0)[7],a[4]=((C_word*)t0)[10],tmp=(C_word)a,a+=5,tmp);
/* core.scm:844: posq */
f_7518(t3,((C_word*)t0)[9],((C_word*)t0)[8]);}
else{
/* core.scm:842: chicken.compiler.support#build-lambda-list */
t3=*((C_word*)lf[187]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[7];
av2[3]=((C_word*)t0)[10];
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}

/* k9653 in k9650 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_9655(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(18,c,3)))){
C_save_and_reclaim((void *)f_9655,c,av);}
a=C_alloc(18);
t2=C_a_i_list(&a,3,lf[97],t1,((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9661,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=t2,a[5]=t1,a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[5],a[8]=((C_word*)t0)[6],tmp=(C_word)a,a+=9,tmp);
/* core.scm:846: set-real-names! */
f_7945(t3,((C_word*)t0)[8],((C_word*)t0)[9]);}

/* k9659 in k9653 in k9650 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_9661(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_9661,c,av);}
a=C_alloc(12);
t2=C_i_not(((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9670,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[2],a[7]=((C_word*)t0)[7],tmp=(C_word)a,a+=8,tmp);
if(C_truep(t2)){
t4=t3;
f_9670(t4,t2);}
else{
if(C_truep(((C_word*)t0)[8])){
t4=t3;
f_9670(t4,((C_word*)t0)[8]);}
else{
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9714,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:849: ##sys#current-environment */
t5=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t5;
av2[1]=t4;
((C_proc)(void*)(*((C_word*)t5+1)))(2,av2);}}}}

/* k9668 in k9659 in k9653 in k9650 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_fcall f_9670(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,3)))){
C_save_and_reclaim_args((void *)trf_9670,2,t0,t1);}
a=C_alloc(8);
if(C_truep(t1)){
t2=((C_word*)t0)[2];{
C_word av2[2];
av2[0]=t2;
av2[1]=((C_word*)t0)[3];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}
else{
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9676,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[3],tmp=(C_word)a,a+=8,tmp);
if(C_truep(*((C_word*)lf[30]+1))){
t3=C_eqp(*((C_word*)lf[43]+1),lf[183]);
if(C_truep(t3)){
t4=t2;{
C_word av2[2];
av2[0]=t4;
av2[1]=t3;
f_9676(2,av2);}}
else{
t4=C_eqp(*((C_word*)lf[43]+1),lf[184]);
if(C_truep(t4)){
/* core.scm:855: chicken.compiler.support#variable-mark */
t5=*((C_word*)lf[185]+1);{
C_word av2[4];
av2[0]=t5;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=lf[186];
((C_proc)(void*)(*((C_word*)t5+1)))(4,av2);}}
else{
t5=t2;{
C_word av2[2];
av2[0]=t5;
av2[1]=C_SCHEME_FALSE;
f_9676(2,av2);}}}}
else{
t3=t2;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_SCHEME_FALSE;
f_9676(2,av2);}}}}

/* k9674 in k9668 in k9659 in k9653 in k9650 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_9676(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_9676,c,av);}
a=C_alloc(5);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9683,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],tmp=(C_word)a,a+=5,tmp);
if(C_truep(C_i_memq(((C_word*)t0)[5],((C_word*)t0)[6]))){
/* core.scm:856: chicken.compiler.support#expand-profile-lambda */
t3=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
/* core.scm:859: ##sys#alias-global-hook */
t3=*((C_word*)lf[124]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[5];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}}
else{
t2=((C_word*)t0)[2];{
C_word *av2=av;
av2[0]=t2;
av2[1]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t2+1)))(2,av2);}}}

/* k9681 in k9674 in k9668 in k9659 in k9653 in k9650 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in ... */
static void C_ccall f_9683(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9683,c,av);}
/* core.scm:856: chicken.compiler.support#expand-profile-lambda */
t2=*((C_word*)lf[182]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* k9712 in k9659 in k9653 in k9650 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_9714(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9714,c,av);}
t2=((C_word*)t0)[2];
f_9670(t2,C_i_assq(((C_word*)t0)[3],t1));}

/* k9723 in k9650 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_9725(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9725,c,av);}
/* core.scm:842: chicken.compiler.support#build-lambda-list */
t2=*((C_word*)lf[187]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=((C_word*)t0)[4];
av2[4]=C_i_list_ref(((C_word*)t0)[3],t1);
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* a9726 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_9727(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9727,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9731,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[3])[1])){
/* core.scm:829: ##sys#current-environment1527 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_9731(2,av2);}}}

/* k9729 in a9726 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_9731(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_9731,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9735,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* core.scm:829: ##sys#current-environment1527 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9733 in k9729 in a9726 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_9735(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_9735,c,av);}
a=C_alloc(4);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9738,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:829: ##sys#current-environment1527 */
t4=((C_word*)t0)[5];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k9736 in k9733 in k9729 in a9726 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_ccall f_9738(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9738,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a9743 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_9744(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_9744,c,av);}
a=C_alloc(9);
t2=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9748,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=((C_word*)t0)[6],a[7]=((C_word*)t0)[7],a[8]=t1,tmp=(C_word)a,a+=9,tmp);
/* core.scm:830: canonicalize-body/ln */
t3=((C_word*)((C_word*)t0)[8])[1];
f_7916(t3,t2,((C_word*)t0)[5],((C_word*)((C_word*)t0)[9])[1],*((C_word*)lf[52]+1));}

/* k9746 in a9743 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_9748(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(22,c,4)))){
C_save_and_reclaim((void *)f_9748,c,av);}
a=C_alloc(22);
t2=C_SCHEME_END_OF_LIST;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_FALSE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9753,a[2]=t5,a[3]=((C_word*)t0)[2],a[4]=t3,tmp=(C_word)a,a+=5,tmp);
t7=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9758,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=t1,tmp=(C_word)a,a+=8,tmp);
t8=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9783,a[2]=t3,a[3]=((C_word*)t0)[2],a[4]=t5,tmp=(C_word)a,a+=5,tmp);
/* core.scm:832: ##sys#dynamic-wind */
t9=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[8];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a9752 in k9746 in a9743 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_9753(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9753,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a9757 in k9746 in a9743 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_9758(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(32,c,2)))){
C_save_and_reclaim((void *)f_9758,c,av);}
a=C_alloc(32);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9766,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
if(C_truep(*((C_word*)lf[24]+1))){
t3=C_a_i_list(&a,2,lf[146],((C_word*)t0)[3]);
t4=C_a_i_list(&a,3,lf[188],lf[189],t3);
t5=t2;
f_9766(t5,C_a_i_list(&a,3,lf[190],t4,((C_word*)t0)[7]));}
else{
t3=t2;
f_9766(t3,((C_word*)t0)[7]);}}

/* k9764 in a9757 in k9746 in a9743 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in ... */
static void C_fcall f_9766(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,0,3)))){
C_save_and_reclaim_args((void *)trf_9766,2,t0,t1);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9770,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],tmp=(C_word)a,a+=7,tmp);
/* core.scm:839: scheme#append */
t3=*((C_word*)lf[5]+1);{
C_word av2[4];
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[6];
av2[3]=((C_word*)t0)[7];
((C_proc)(void*)(*((C_word*)t3+1)))(4,av2);}}

/* k9768 in k9764 in a9757 in k9746 in a9743 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in ... */
static void C_ccall f_9770(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_9770,c,av);}
/* core.scm:833: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],((C_word*)t0)[4],t1,C_SCHEME_FALSE,C_SCHEME_FALSE,((C_word*)t0)[5],((C_word*)t0)[6],C_SCHEME_FALSE);}

/* a9782 in k9746 in a9743 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_9783(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9783,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)((C_word*)t0)[3])[1]);
t3=C_mutate(((C_word *)((C_word*)t0)[3])+1,((C_word*)((C_word*)t0)[4])[1]);
t4=t1;{
C_word *av2=av;
av2[0]=t4;
av2[1]=C_SCHEME_UNDEFINED;
((C_proc)(void*)(*((C_word*)t4+1)))(2,av2);}}

/* a9787 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in ... */
static void C_ccall f_9788(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9788,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9792,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:829: ##sys#current-environment1527 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9790 in a9787 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in ... */
static void C_ccall f_9792(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_9792,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9795,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:829: ##sys#current-environment1527 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9793 in k9790 in a9787 in k9647 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in ... */
static void C_ccall f_9795(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9795,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9798 in k9641 in k9638 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_9800(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,4)))){
C_save_and_reclaim((void *)f_9800,c,av);}
/* core.scm:828: ##sys#extend-se */
t2=*((C_word*)lf[175]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=((C_word*)t0)[3];
av2[4]=((C_word*)t0)[4];
((C_proc)(void*)(*((C_word*)t2+1)))(5,av2);}}

/* map-loop1496 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_fcall f_9802(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,0,2)))){
C_save_and_reclaim_args((void *)trf_9802,3,t0,t1,t2);}
a=C_alloc(6);
if(C_truep(C_i_pairp(t2))){
t3=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9827,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=t2,tmp=(C_word)a,a+=6,tmp);
/* core.scm:826: g1502 */
t4=*((C_word*)lf[109]+1);{
C_word av2[3];
av2[0]=t4;
av2[1]=t3;
av2[2]=C_slot(t2,C_fix(0));
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* k9825 in map-loop1496 in a9629 in k9623 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_9827(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void *)f_9827,c,av);}
a=C_alloc(3);
t2=C_a_i_cons(&a,2,t1,C_SCHEME_END_OF_LIST);
t3=C_i_setslot(((C_word*)((C_word*)t0)[2])[1],C_fix(1),t2);
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=((C_word*)((C_word*)t0)[3])[1];
f_9802(t5,((C_word*)t0)[4],C_slot(((C_word*)t0)[5],C_fix(1)));}

/* k9835 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_9837(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,c,4)))){
C_save_and_reclaim((void *)f_9837,c,av);}
a=C_alloc(8);
if(C_truep(t1)){
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9842,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9852,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],tmp=(C_word)a,a+=4,tmp);
/* core.scm:819: ##sys#call-with-values */{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=0;
av2[1]=((C_word*)t0)[4];
av2[2]=t2;
av2[3]=t3;
C_call_with_values(4,av2);}}
else{
t2=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t2;
av2[1]=C_SCHEME_UNDEFINED;
f_9625(2,av2);}}}

/* a9841 in k9835 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9842(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9842,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9850,a[2]=t1,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:822: ##sys#current-environment */
t3=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9848 in a9841 in k9835 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9850(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,5)))){
C_save_and_reclaim((void *)f_9850,c,av);}
/* core.scm:821: ##sys#expand-extended-lambda-list */
t2=*((C_word*)lf[192]+1);{
C_word *av2;
if(c >= 6) {
  av2=av;
} else {
  av2=C_alloc(6);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)((C_word*)t0)[3])[1];
av2[3]=((C_word*)((C_word*)t0)[4])[1];
av2[4]=*((C_word*)lf[193]+1);
av2[5]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(6,av2);}}

/* a9851 in k9835 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9852(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2=av[2];
C_word t3=av[3];
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9852,c,av);}
t4=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t5=C_mutate(((C_word *)((C_word*)t0)[3])+1,t3);
t6=t1;{
C_word *av2=av;
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}

/* k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_9864(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word t7;
C_word t8;
C_word t9;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(23,c,4)))){
C_save_and_reclaim((void *)f_9864,c,av);}
a=C_alloc(23);
t2=t1;
t3=(*a=C_VECTOR_TYPE|1,a[1]=t2,tmp=(C_word)a,a+=2,tmp);
t4=C_SCHEME_TRUE;
t5=(*a=C_VECTOR_TYPE|1,a[1]=t4,tmp=(C_word)a,a+=2,tmp);
t6=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9869,a[2]=t3,a[3]=t5,a[4]=((C_word*)t0)[2],tmp=(C_word)a,a+=5,tmp);
t7=(*a=C_CLOSURE_TYPE|9,a[1]=(C_word)f_9886,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=((C_word*)t0)[9],a[9]=((C_word*)t0)[10],tmp=(C_word)a,a+=10,tmp);
t8=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9906,a[2]=t3,a[3]=((C_word*)t0)[2],tmp=(C_word)a,a+=4,tmp);
/* core.scm:864: ##sys#dynamic-wind */
t9=*((C_word*)lf[116]+1);{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t9;
av2[1]=((C_word*)t0)[11];
av2[2]=t6;
av2[3]=t7;
av2[4]=t8;
((C_proc)(void*)(*((C_word*)t9+1)))(5,av2);}}

/* a9868 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9869(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(6,c,4)))){
C_save_and_reclaim((void *)f_9869,c,av);}
a=C_alloc(6);
t2=(*a=C_CLOSURE_TYPE|5,a[1]=(C_word)f_9873,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],tmp=(C_word)a,a+=6,tmp);
if(C_truep(((C_word*)((C_word*)t0)[3])[1])){
/* core.scm:864: ##sys#current-environment1579 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_TRUE;
av2[4]=C_SCHEME_FALSE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}
else{
t3=t2;{
C_word *av2=av;
av2[0]=t3;
av2[1]=((C_word*)((C_word*)t0)[2])[1];
f_9873(2,av2);}}}

/* k9871 in a9868 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9873(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(7,c,2)))){
C_save_and_reclaim((void *)f_9873,c,av);}
a=C_alloc(7);
t2=(*a=C_CLOSURE_TYPE|6,a[1]=(C_word)f_9877,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=((C_word*)t0)[4],a[5]=((C_word*)t0)[5],a[6]=t1,tmp=(C_word)a,a+=7,tmp);
/* core.scm:864: ##sys#current-environment1579 */
t3=((C_word*)t0)[5];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9875 in k9871 in a9868 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_9877(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,4)))){
C_save_and_reclaim((void *)f_9877,c,av);}
a=C_alloc(4);
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,t1);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9880,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:864: ##sys#current-environment1579 */
t4=((C_word*)t0)[5];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=((C_word*)t0)[6];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t4+1)))(5,av2);}}

/* k9878 in k9875 in k9871 in a9868 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in ... */
static void C_ccall f_9880(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9880,c,av);}
t2=C_set_block_item(((C_word*)t0)[2],0,C_SCHEME_FALSE);
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* a9885 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9886(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(11,c,2)))){
C_save_and_reclaim((void *)f_9886,c,av);}
a=C_alloc(11);
t2=(*a=C_CLOSURE_TYPE|10,a[1]=(C_word)f_9890,a[2]=((C_word*)t0)[2],a[3]=((C_word*)t0)[3],a[4]=t1,a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],a[8]=((C_word*)t0)[7],a[9]=((C_word*)t0)[8],a[10]=((C_word*)t0)[9],tmp=(C_word)a,a+=11,tmp);
/* core.scm:876: chicken.compiler.support#get-line */
t3=*((C_word*)lf[113]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)t0)[9];
((C_proc)(void*)(*((C_word*)t3+1)))(3,av2);}}

/* k9888 in a9885 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9890(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(9,c,4)))){
C_save_and_reclaim((void *)f_9890,c,av);}
a=C_alloc(9);
t2=(C_truep(t1)?t1:((C_word*)t0)[2]);
t3=(*a=C_CLOSURE_TYPE|8,a[1]=(C_word)f_9900,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],a[4]=((C_word*)t0)[5],a[5]=((C_word*)t0)[6],a[6]=((C_word*)t0)[7],a[7]=((C_word*)t0)[8],a[8]=t2,tmp=(C_word)a,a+=9,tmp);
/* core.scm:878: canonicalize-body/ln */
t4=((C_word*)((C_word*)t0)[9])[1];
f_7916(t4,t3,t2,C_i_cddr(((C_word*)t0)[10]),*((C_word*)lf[52]+1));}

/* k9898 in k9888 in a9885 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_9900(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,8)))){
C_save_and_reclaim((void *)f_9900,c,av);}
/* core.scm:877: walk */
t2=((C_word*)((C_word*)t0)[2])[1];
f_8362(t2,((C_word*)t0)[3],t1,((C_word*)t0)[4],((C_word*)t0)[5],((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],C_SCHEME_FALSE);}

/* a9905 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9906(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,2)))){
C_save_and_reclaim((void *)f_9906,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9910,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:864: ##sys#current-environment1579 */
t3=((C_word*)t0)[3];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9908 in a9905 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9910(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(5,c,4)))){
C_save_and_reclaim((void *)f_9910,c,av);}
a=C_alloc(5);
t2=(*a=C_CLOSURE_TYPE|4,a[1]=(C_word)f_9913,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],tmp=(C_word)a,a+=5,tmp);
/* core.scm:864: ##sys#current-environment1579 */
t3=((C_word*)t0)[4];{
C_word *av2;
if(c >= 5) {
  av2=av;
} else {
  av2=C_alloc(5);
}
av2[0]=t3;
av2[1]=t2;
av2[2]=((C_word*)((C_word*)t0)[2])[1];
av2[3]=C_SCHEME_FALSE;
av2[4]=C_SCHEME_TRUE;
((C_proc)(void*)(*((C_word*)t3+1)))(5,av2);}}

/* k9911 in k9908 in a9905 in k9862 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in ... */
static void C_ccall f_9913(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,1)))){
C_save_and_reclaim((void *)f_9913,c,av);}
t2=C_mutate(((C_word *)((C_word*)t0)[2])+1,((C_word*)t0)[3]);
t3=((C_word*)t0)[4];{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9929 in map-loop1588 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9931(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_9931,c,av);}
a=C_alloc(12);
t2=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9935,a[2]=((C_word*)t0)[2],a[3]=t1,a[4]=((C_word*)t0)[3],a[5]=((C_word*)t0)[4],a[6]=((C_word*)t0)[5],a[7]=((C_word*)t0)[6],tmp=(C_word)a,a+=8,tmp);
t3=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9939,a[2]=t2,a[3]=((C_word*)t0)[7],tmp=(C_word)a,a+=4,tmp);
/* core.scm:872: ##sys#eval/meta */
t4=*((C_word*)lf[197]+1);{
C_word *av2;
if(c >= 3) {
  av2=av;
} else {
  av2=C_alloc(3);
}
av2[0]=t4;
av2[1]=t3;
av2[2]=C_i_cadr(((C_word*)t0)[7]);
((C_proc)(void*)(*((C_word*)t4+1)))(3,av2);}}

/* k9933 in k9929 in map-loop1588 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9935(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(12,c,2)))){
C_save_and_reclaim((void *)f_9935,c,av);}
a=C_alloc(12);
t2=C_a_i_list3(&a,3,((C_word*)t0)[2],((C_word*)t0)[3],t1);
t3=C_a_i_cons(&a,2,t2,C_SCHEME_END_OF_LIST);
t4=C_i_setslot(((C_word*)((C_word*)t0)[4])[1],C_fix(1),t3);
t5=C_mutate(((C_word *)((C_word*)t0)[4])+1,t3);
t6=((C_word*)((C_word*)t0)[5])[1];
f_9960(t6,((C_word*)t0)[6],C_slot(((C_word*)t0)[7],C_fix(1)));}

/* k9937 in k9929 in map-loop1588 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in ... */
static void C_ccall f_9939(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9939,c,av);}
/* core.scm:871: ##sys#ensure-transformer */
t2=*((C_word*)lf[196]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=t1;
av2[3]=C_u_i_car(((C_word*)t0)[3]);
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* k9952 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_ccall f_9954(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(4,c,2)))){
C_save_and_reclaim((void *)f_9954,c,av);}
a=C_alloc(4);
t2=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_9958,a[2]=((C_word*)t0)[2],a[3]=t1,tmp=(C_word)a,a+=4,tmp);
/* core.scm:875: ##sys#current-environment */
t3=*((C_word*)lf[107]+1);{
C_word *av2=av;
av2[0]=t3;
av2[1]=t2;
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}

/* k9956 in k9952 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in ... */
static void C_ccall f_9958(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(0,c,3)))){
C_save_and_reclaim((void *)f_9958,c,av);}
/* core.scm:866: scheme#append */
t2=*((C_word*)lf[5]+1);{
C_word *av2;
if(c >= 4) {
  av2=av;
} else {
  av2=C_alloc(4);
}
av2[0]=t2;
av2[1]=((C_word*)t0)[2];
av2[2]=((C_word*)t0)[3];
av2[3]=t1;
((C_proc)(void*)(*((C_word*)t2+1)))(4,av2);}}

/* map-loop1588 in k8771 in k8437 in k8425 in k8422 in k8419 in k8415 in k8412 in k8406 in k8367 in walk in chicken.compiler.core#canonicalize-expression in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 in ... */
static void C_fcall f_9960(C_word t0,C_word t1,C_word t2){
C_word tmp;
C_word t3;
C_word t4;
C_word t5;
C_word t6;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(8,0,2)))){
C_save_and_reclaim_args((void *)trf_9960,3,t0,t1,t2);}
a=C_alloc(8);
if(C_truep(C_i_pairp(t2))){
t3=C_slot(t2,C_fix(0));
t4=C_i_car(t3);
t5=(*a=C_CLOSURE_TYPE|7,a[1]=(C_word)f_9931,a[2]=t4,a[3]=((C_word*)t0)[2],a[4]=((C_word*)t0)[3],a[5]=t1,a[6]=t2,a[7]=t3,tmp=(C_word)a,a+=8,tmp);
/* core.scm:870: ##sys#current-environment */
t6=*((C_word*)lf[107]+1);{
C_word av2[2];
av2[0]=t6;
av2[1]=t5;
((C_proc)(void*)(*((C_word*)t6+1)))(2,av2);}}
else{
t3=t1;{
C_word av2[2];
av2[0]=t3;
av2[1]=C_slot(((C_word*)t0)[4],C_fix(1));
((C_proc)(void*)(*((C_word*)t3+1)))(2,av2);}}}

/* toplevel */
static C_TLS int toplevel_initialized=0;

void C_ccall C_compiler_toplevel(C_word c,C_word *av){
C_word tmp;
C_word t0=av[0];
C_word t1=av[1];
C_word t2;
C_word t3;
C_word *a;
if(toplevel_initialized) {C_kontinue(t1,C_SCHEME_UNDEFINED);}
else C_toplevel_entry(C_text("compiler"));
C_check_nursery_minimum(C_calculate_demand(3,c,2));
if(C_unlikely(!C_demand(C_calculate_demand(3,c,2)))){
C_save_and_reclaim((void*)C_compiler_toplevel,c,av);}
toplevel_initialized=1;
if(C_unlikely(!C_demand_2(4375))){
C_save(t1);
C_rereclaim2(4375*sizeof(C_word),1);
t1=C_restore;}
a=C_alloc(3);
C_initialize_lf(lf,712);
lf[0]=C_h_intern(&lf[0],8, C_text("compiler"));
lf[1]=C_h_intern(&lf[1],22, C_text("chicken.compiler.core#"));
lf[5]=C_h_intern(&lf[5],13, C_text("scheme#append"));
lf[8]=C_h_intern(&lf[8],5, C_text("foldr"));
lf[10]=C_h_intern(&lf[10],3, C_text("map"));
lf[14]=C_h_intern(&lf[14],5, C_text("foldl"));
lf[17]=C_h_intern(&lf[17],31, C_text("chicken.compiler.core#unit-name"));
lf[18]=C_h_intern(&lf[18],39, C_text("chicken.compiler.core#standard-bindings"));
lf[19]=C_h_intern(&lf[19],39, C_text("chicken.compiler.core#extended-bindings"));
lf[20]=C_h_intern(&lf[20],41, C_text("chicken.compiler.core#insert-timer-checks"));
lf[21]=C_h_intern(&lf[21],32, C_text("chicken.compiler.core#used-units"));
lf[22]=C_h_intern(&lf[22],42, C_text("chicken.compiler.core#foreign-declarations"));
lf[23]=C_h_intern(&lf[23],37, C_text("chicken.compiler.core#emit-trace-info"));
lf[24]=C_h_intern(&lf[24],37, C_text("chicken.compiler.core#emit-debug-info"));
lf[25]=C_h_intern(&lf[25],39, C_text("chicken.compiler.core#block-compilation"));
lf[26]=C_h_intern(&lf[26],47, C_text("chicken.compiler.core#line-number-database-size"));
lf[27]=C_h_intern(&lf[27],38, C_text("chicken.compiler.core#target-heap-size"));
lf[28]=C_h_intern(&lf[28],39, C_text("chicken.compiler.core#target-stack-size"));
lf[29]=C_h_intern(&lf[29],44, C_text("chicken.compiler.core#optimize-leaf-routines"));
lf[30]=C_h_intern(&lf[30],34, C_text("chicken.compiler.core#emit-profile"));
lf[31]=C_h_intern(&lf[31],37, C_text("chicken.compiler.core#no-bound-checks"));
lf[32]=C_h_intern(&lf[32],36, C_text("chicken.compiler.core#no-argc-checks"));
lf[33]=C_h_intern(&lf[33],41, C_text("chicken.compiler.core#no-procedure-checks"));
lf[34]=C_h_intern(&lf[34],48, C_text("chicken.compiler.core#no-global-procedure-checks"));
lf[36]=C_h_intern(&lf[36],39, C_text("chicken.compiler.core#explicit-use-flag"));
lf[37]=C_h_intern(&lf[37],53, C_text("chicken.compiler.core#disable-stack-overflow-checking"));
lf[38]=C_h_intern(&lf[38],43, C_text("chicken.compiler.core#external-protos-first"));
lf[39]=C_h_intern(&lf[39],37, C_text("chicken.compiler.core#inline-max-size"));
lf[40]=C_h_intern(&lf[40],34, C_text("chicken.compiler.core#unroll-limit"));
lf[41]=C_h_intern(&lf[41],39, C_text("chicken.compiler.core#emit-closure-info"));
lf[42]=C_h_intern(&lf[42],46, C_text("chicken.compiler.core#undefine-shadowed-macros"));
lf[43]=C_h_intern(&lf[43],41, C_text("chicken.compiler.core#profiled-procedures"));
lf[44]=C_h_intern(&lf[44],38, C_text("chicken.compiler.core#import-libraries"));
lf[45]=C_h_intern(&lf[45],42, C_text("chicken.compiler.core#all-import-libraries"));
lf[46]=C_h_intern(&lf[46],57, C_text("chicken.compiler.core#preserve-unchanged-import-libraries"));
lf[47]=C_h_intern(&lf[47],49, C_text("chicken.compiler.core#compile-module-registration"));
lf[48]=C_h_intern(&lf[48],43, C_text("chicken.compiler.core#standalone-executable"));
lf[49]=C_h_intern(&lf[49],39, C_text("chicken.compiler.core#local-definitions"));
lf[50]=C_h_intern(&lf[50],36, C_text("chicken.compiler.core#inline-locally"));
lf[51]=C_h_intern(&lf[51],41, C_text("chicken.compiler.core#enable-inline-files"));
lf[52]=C_h_intern(&lf[52],45, C_text("chicken.compiler.core#compiler-syntax-enabled"));
lf[53]=C_h_intern(&lf[53],36, C_text("chicken.compiler.core#bootstrap-mode"));
lf[54]=C_h_intern(&lf[54],43, C_text("chicken.compiler.core#strict-variable-types"));
lf[55]=C_h_intern(&lf[55],43, C_text("chicken.compiler.core#enable-specialization"));
lf[56]=C_h_intern(&lf[56],39, C_text("chicken.compiler.core#static-extensions"));
lf[57]=C_h_intern(&lf[57],36, C_text("chicken.compiler.core#emit-link-file"));
lf[58]=C_h_intern(&lf[58],39, C_text("chicken.compiler.core#types-output-file"));
lf[59]=C_h_intern(&lf[59],34, C_text("chicken.compiler.core#verbose-mode"));
lf[63]=C_h_intern(&lf[63],44, C_text("chicken.compiler.core#line-number-database-2"));
lf[64]=C_h_intern(&lf[64],41, C_text("chicken.compiler.core#immutable-constants"));
lf[65]=C_h_intern(&lf[65],34, C_text("chicken.compiler.core#inline-table"));
lf[66]=C_h_intern(&lf[66],36, C_text("chicken.compiler.core#constant-table"));
lf[67]=C_h_intern(&lf[67],50, C_text("chicken.compiler.core#inline-substitutions-enabled"));
lf[69]=C_h_intern(&lf[69],36, C_text("chicken.compiler.core#first-analysis"));
lf[71]=C_h_intern(&lf[71],42, C_text("chicken.compiler.core#foreign-lambda-stubs"));
lf[72]=C_h_intern(&lf[72],40, C_text("chicken.compiler.core#external-variables"));
lf[80]=C_h_intern(&lf[80],40, C_text("chicken.compiler.core#required-libraries"));
lf[81]=C_h_intern(&lf[81],38, C_text("chicken.compiler.core#linked-libraries"));
lf[82]=C_h_intern(&lf[82],36, C_text("chicken.compiler.core#used-libraries"));
lf[84]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\007\001unquote\376\003\000\000\002\376\001\000\000\020\001unquote-splicing\376\377\016"));
lf[85]=C_h_intern(&lf[85],47, C_text("chicken.compiler.core#default-extended-bindings"));
lf[86]=C_h_intern(&lf[86],47, C_text("chicken.compiler.core#default-standard-bindings"));
lf[87]=C_h_intern(&lf[87],39, C_text("chicken.compiler.core#internal-bindings"));
lf[88]=C_h_intern(&lf[88],41, C_text("chicken.compiler.core#initialize-compiler"));
lf[89]=C_h_intern(&lf[89],50, C_text("chicken.compiler.support#clear-foreign-type-table!"));
lf[90]=C_h_intern(&lf[90],47, C_text("chicken.compiler.support#clear-real-name-table!"));
lf[91]=C_h_intern(&lf[91],56, C_text("chicken.compiler.support#reset-profile-info-vector-name!"));
lf[92]=C_h_intern(&lf[92],19, C_text("scheme#vector-fill!"));
lf[93]=C_h_intern(&lf[93],18, C_text("scheme#make-vector"));
lf[94]=C_h_intern(&lf[94],49, C_text("chicken.compiler.core#compute-database-statistics"));
lf[95]=C_h_intern(&lf[95],6, C_text("global"));
lf[96]=C_h_intern(&lf[96],5, C_text("value"));
lf[97]=C_h_intern(&lf[97],13, C_text("##core#lambda"));
lf[98]=C_h_intern(&lf[98],35, C_text("chicken.compiler.support#node-class"));
lf[99]=C_h_intern(&lf[99],10, C_text("call-sites"));
lf[100]=C_h_intern(&lf[100],8, C_text("for-each"));
lf[101]=C_h_intern(&lf[101],36, C_text("chicken.internal#hash-table-for-each"));
lf[102]=C_h_intern(&lf[102],45, C_text("chicken.compiler.core#canonicalize-expression"));
lf[103]=C_h_intern(&lf[103],9, C_text("##sys#get"));
lf[104]=C_h_intern(&lf[104],18, C_text("##core#macro-alias"));
lf[105]=C_h_intern(&lf[105],13, C_text("scheme#values"));
lf[106]=C_h_intern(&lf[106],14, C_text("scheme#symbol\077"));
lf[107]=C_h_intern(&lf[107],25, C_text("##sys#current-environment"));
lf[108]=C_h_intern(&lf[108],10, C_text("##sys#put!"));
lf[109]=C_h_intern(&lf[109],19, C_text("chicken.base#gensym"));
lf[110]=C_h_intern(&lf[110],32, C_text("chicken.internal#hash-table-set!"));
lf[111]=C_h_intern(&lf[111],26, C_text("##sys#line-number-database"));
lf[112]=C_h_intern(&lf[112],31, C_text("chicken.internal#hash-table-ref"));
lf[113]=C_h_intern(&lf[113],33, C_text("chicken.compiler.support#get-line"));
lf[114]=C_h_intern(&lf[114],36, C_text("chicken.syntax#expansion-result-hook"));
lf[115]=C_h_intern(&lf[115],23, C_text("##sys#canonicalize-body"));
lf[116]=C_h_intern(&lf[116],18, C_text("##sys#dynamic-wind"));
lf[117]=C_h_intern(&lf[117],39, C_text("chicken.compiler.support#set-real-name!"));
lf[118]=C_h_intern(&lf[118],5, C_text("quote"));
lf[119]=C_h_intern(&lf[119],17, C_text("##core#inline_ref"));
lf[120]=C_h_intern(&lf[120],52, C_text("chicken.compiler.support#foreign-type-convert-result"));
lf[121]=C_h_intern(&lf[121],46, C_text("chicken.compiler.support#finish-foreign-result"));
lf[122]=C_h_intern(&lf[122],43, C_text("chicken.compiler.support#final-foreign-type"));
lf[123]=C_h_intern(&lf[123],21, C_text("##core#inline_loc_ref"));
lf[124]=C_h_intern(&lf[124],23, C_text("##sys#alias-global-hook"));
lf[125]=C_h_intern(&lf[125],39, C_text("chicken.compiler.support#quit-compiling"));
lf[126]=C_decode_literal(C_heaptop,C_text("\376B\000\0008~acyclical reference in LETREC binding for variable `~a\047"));
lf[127]=C_h_intern(&lf[127],38, C_text("chicken.compiler.support#get-real-name"));
lf[128]=C_h_intern(&lf[128],6, C_text("format"));
lf[129]=C_h_intern(&lf[129],30, C_text("chicken.base#get-output-string"));
lf[130]=C_h_intern(&lf[130],11, C_text("##sys#print"));
lf[131]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[132]=C_h_intern(&lf[132],18, C_text("##sys#write-char-0"));
lf[133]=C_h_intern(&lf[133],31, C_text("chicken.base#open-output-string"));
lf[134]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[135]=C_h_intern(&lf[135],20, C_text("chicken.base#warning"));
lf[136]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025\047 possibly unintended"));
lf[137]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027reference to variable `"));
lf[138]=C_h_intern(&lf[138],26, C_text("##sys#syntax-error/context"));
lf[139]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023illegal atomic form"));
lf[140]=C_h_intern(&lf[140],34, C_text("chicken.compiler.support#constant\077"));
lf[141]=C_h_intern(&lf[141],26, C_text("##sys#syntax-error-culprit"));
lf[142]=C_h_intern(&lf[142],9, C_text("##core#if"));
lf[143]=C_h_intern(&lf[143],2, C_text("if"));
lf[144]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[145]=C_h_intern(&lf[145],13, C_text("##core#syntax"));
lf[146]=C_h_intern(&lf[146],12, C_text("##core#quote"));
lf[147]=C_h_intern(&lf[147],27, C_text("chicken.syntax#strip-syntax"));
lf[148]=C_h_intern(&lf[148],12, C_text("##core#check"));
lf[149]=C_h_intern(&lf[149],31, C_text("chicken.compiler.support#unsafe"));
lf[150]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001quote\376\003\000\000\002\376\377\006\001\376\377\016"));
lf[151]=C_h_intern(&lf[151],10, C_text("##core#the"));
lf[152]=C_h_intern(&lf[152],27, C_text("##core#local-specialization"));
lf[153]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[154]=C_h_intern(&lf[154],32, C_text("##compiler#local-specializations"));
lf[155]=C_h_intern(&lf[155],12, C_text("##sys#append"));
lf[156]=C_h_intern(&lf[156],15, C_text("##core#typecase"));
lf[157]=C_h_intern(&lf[157],16, C_text("##core#immutable"));
lf[158]=C_h_intern(&lf[158],38, C_text("chicken.compiler.support#hide-variable"));
lf[159]=C_h_intern(&lf[159],38, C_text("chicken.compiler.support#mark-variable"));
lf[160]=C_h_intern(&lf[160],23, C_text("##compiler#always-bound"));
lf[161]=C_h_intern(&lf[161],1, C_text("c"));
lf[162]=C_h_intern(&lf[162],13, C_text("scheme#cadadr"));
lf[163]=C_h_intern(&lf[163],14, C_text("##core#provide"));
lf[164]=C_h_intern(&lf[164],15, C_text("##core#callunit"));
lf[165]=C_h_intern(&lf[165],12, C_text("scheme#caadr"));
lf[166]=C_h_intern(&lf[166],25, C_text("##core#require-for-syntax"));
lf[167]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[168]=C_h_intern(&lf[168],27, C_text("chicken.load#load-extension"));
lf[169]=C_h_intern(&lf[169],14, C_text("##core#require"));
lf[170]=C_h_intern(&lf[170],21, C_text("##sys#process-require"));
lf[171]=C_h_intern(&lf[171],6, C_text("static"));
lf[172]=C_h_intern(&lf[172],7, C_text("dynamic"));
lf[173]=C_h_intern(&lf[173],10, C_text("##core#let"));
lf[174]=C_h_intern(&lf[174],3, C_text("let"));
lf[175]=C_h_intern(&lf[175],15, C_text("##sys#extend-se"));
lf[176]=C_h_intern(&lf[176],14, C_text("##core#letrec\052"));
lf[177]=C_h_intern(&lf[177],11, C_text("##core#set!"));
lf[178]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[179]=C_h_intern(&lf[179],13, C_text("##core#letrec"));
lf[180]=C_h_intern(&lf[180],26, C_text("##core#with-forbidden-refs"));
lf[181]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[182]=C_h_intern(&lf[182],46, C_text("chicken.compiler.support#expand-profile-lambda"));
lf[183]=C_h_intern(&lf[183],3, C_text("all"));
lf[184]=C_h_intern(&lf[184],4, C_text("some"));
lf[185]=C_h_intern(&lf[185],38, C_text("chicken.compiler.support#variable-mark"));
lf[186]=C_h_intern(&lf[186],18, C_text("##compiler#profile"));
lf[187]=C_h_intern(&lf[187],42, C_text("chicken.compiler.support#build-lambda-list"));
lf[188]=C_h_intern(&lf[188],18, C_text("##core#debug-event"));
lf[189]=C_h_intern(&lf[189],13, C_text("C_DEBUG_ENTRY"));
lf[190]=C_h_intern(&lf[190],12, C_text("##core#begin"));
lf[191]=C_h_intern(&lf[191],27, C_text("##sys#decompose-lambda-list"));
lf[192]=C_h_intern(&lf[192],33, C_text("##sys#expand-extended-lambda-list"));
lf[193]=C_h_intern(&lf[193],11, C_text("##sys#error"));
lf[194]=C_h_intern(&lf[194],27, C_text("##sys#extended-lambda-list\077"));
lf[195]=C_h_intern(&lf[195],17, C_text("##core#let-syntax"));
lf[196]=C_h_intern(&lf[196],24, C_text("##sys#ensure-transformer"));
lf[197]=C_h_intern(&lf[197],15, C_text("##sys#eval/meta"));
lf[198]=C_h_intern(&lf[198],20, C_text("##core#letrec-syntax"));
lf[199]=C_h_intern(&lf[199],20, C_text("##core#define-syntax"));
lf[200]=C_h_intern(&lf[200],27, C_text("##sys#enable-runtime-macros"));
lf[201]=C_h_intern(&lf[201],30, C_text("##sys#extend-macro-environment"));
lf[202]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[203]=C_h_intern(&lf[203],28, C_text("##sys#register-syntax-export"));
lf[204]=C_h_intern(&lf[204],20, C_text("##sys#current-module"));
lf[205]=C_h_intern(&lf[205],12, C_text("scheme#cdadr"));
lf[206]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001_\376\003\000\000\002\376\003\000\000\002\376\001\000\000\010\001variable\376\001\000\000\013\001lambda-list\376\000\000\000\002\376\001\000\000\001\001_\376\377\001\000\000\000\001"));
lf[207]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001_\376\003\000\000\002\376\001\000\000\010\001variable\376\003\000\000\002\376\001\000\000\001\001_\376\377\016"));
lf[208]=C_h_intern(&lf[208],18, C_text("##sys#check-syntax"));
lf[209]=C_h_intern(&lf[209],29, C_text("##core#define-compiler-syntax"));
lf[210]=C_h_intern(&lf[210],26, C_text("##compiler#compiler-syntax"));
lf[211]=C_h_intern(&lf[211],10, C_text("##sys#cons"));
lf[212]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[213]=C_h_intern(&lf[213],26, C_text("##core#let-compiler-syntax"));
lf[214]=C_h_intern(&lf[214],19, C_text("let-compiler-syntax"));
lf[215]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001symbol\376\000\000\000\003\376\001\000\000\001\001_\376\377\001\000\000\000\000\376\377\001\000\000\000\001"));
lf[216]=C_h_intern(&lf[216],19, C_text("scheme#dynamic-wind"));
lf[217]=C_h_intern(&lf[217],14, C_text("##core#include"));
lf[218]=C_h_intern(&lf[218],39, C_text("chicken.compiler.support#read-info-hook"));
lf[219]=C_h_intern(&lf[219],28, C_text("##sys#default-read-info-hook"));
lf[220]=C_h_intern(&lf[220],29, C_text("##sys#include-forms-from-file"));
lf[221]=C_h_intern(&lf[221],23, C_text("##core#let-module-alias"));
lf[222]=C_h_intern(&lf[222],25, C_text("##sys#with-module-aliases"));
lf[223]=C_h_intern(&lf[223],7, C_text("functor"));
lf[224]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\006\001symbol\376\003\000\000\002\376\001\000\000\006\001symbol\376\377\016"));
lf[225]=C_h_intern(&lf[225],13, C_text("##core#module"));
lf[226]=C_h_intern(&lf[226],23, C_text("##sys#macro-environment"));
lf[227]=C_h_intern(&lf[227],30, C_text("##sys#module-alias-environment"));
lf[228]=C_h_intern(&lf[228],31, C_text("##sys#initial-macro-environment"));
lf[229]=C_h_intern(&lf[229],48, C_text("chicken.compiler.support#canonicalize-begin-body"));
lf[230]=C_h_intern(&lf[230],3, C_text("yes"));
lf[231]=C_h_intern(&lf[231],34, C_text("##sys#compiled-module-registration"));
lf[232]=C_h_intern(&lf[232],30, C_text("##sys#current-meta-environment"));
lf[233]=C_h_intern(&lf[233],28, C_text("##sys#meta-macro-environment"));
lf[234]=C_h_intern(&lf[234],35, C_text("chicken.internal#module-requirement"));
lf[235]=C_h_intern(&lf[235],13, C_text("scheme#equal\077"));
lf[236]=C_h_intern(&lf[236],18, C_text("chicken.base#print"));
lf[237]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037not generating import library `"));
lf[238]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016\047 for module `"));
lf[239]=C_decode_literal(C_heaptop,C_text("\376B\000\000 \047 because imports did not change"));
lf[240]=C_h_intern(&lf[240],33, C_text("chicken.pretty-print#pretty-print"));
lf[241]=C_decode_literal(C_heaptop,C_text("\376B\000\000\017\012;; END OF FILE"));
lf[242]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005;;;; "));
lf[243]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030 - GENERATED BY CHICKEN "));
lf[244]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020 -\052- Scheme -\052-\012"));
lf[245]=C_h_intern(&lf[245],32, C_text("chicken.platform#chicken-version"));
lf[246]=C_h_intern(&lf[246],26, C_text("scheme#with-output-to-file"));
lf[247]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033generating import library `"));
lf[248]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016\047 for module `"));
lf[249]=C_decode_literal(C_heaptop,C_text("\376B\000\000\005\047 ..."));
lf[250]=C_h_intern(&lf[250],27, C_text("scheme#call-with-input-file"));
lf[251]=C_h_intern(&lf[251],41, C_text("chicken.compiler.support#read-expressions"));
lf[252]=C_h_intern(&lf[252],25, C_text("chicken.file#file-exists\077"));
lf[253]=C_h_intern(&lf[253],19, C_text("##sys#string-append"));
lf[254]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013.import.scm"));
lf[255]=C_h_intern(&lf[255],21, C_text("scheme#symbol->string"));
lf[256]=C_h_intern(&lf[256],17, C_text("chicken.base#exit"));
lf[257]=C_h_intern(&lf[257],37, C_text("chicken.condition#print-error-message"));
lf[258]=C_h_intern(&lf[258],20, C_text("##sys#standard-error"));
lf[259]=C_h_intern(&lf[259],14, C_text("scheme#reverse"));
lf[260]=C_decode_literal(C_heaptop,C_text("\376B\000\000\022a foreign variable"));
lf[261]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023an inlined function"));
lf[262]=C_decode_literal(C_heaptop,C_text("\376B\000\000\012a constant"));
lf[263]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023a type abbreviation"));
lf[264]=C_h_intern(&lf[264],28, C_text("##compiler#type-abbreviation"));
lf[265]=C_h_intern(&lf[265],21, C_text("##sys#finalize-module"));
lf[266]=C_h_intern(&lf[266],40, C_text("chicken.condition#with-exception-handler"));
lf[267]=C_h_intern(&lf[267],37, C_text("scheme#call-with-current-continuation"));
lf[268]=C_h_intern(&lf[268],27, C_text("##sys#with-property-restore"));
lf[269]=C_h_intern(&lf[269],23, C_text("##sys#syntax-error-hook"));
lf[270]=C_h_intern(&lf[270],6, C_text("module"));
lf[271]=C_decode_literal(C_heaptop,C_text("\376B\000\000\031modules may not be nested"));
lf[272]=C_h_intern(&lf[272],21, C_text("##sys#register-module"));
lf[273]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025invalid export syntax"));
lf[274]=C_h_intern(&lf[274],18, C_text("##core#loop-lambda"));
lf[275]=C_h_intern(&lf[275],33, C_text("##core#ensure-toplevel-definition"));
lf[276]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[277]=C_decode_literal(C_heaptop,C_text("\376B\000\0005~atoplevel definition of `~s\047 in non-toplevel context"));
lf[278]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[279]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[280]=C_h_intern(&lf[280],20, C_text("##core#inline_update"));
lf[281]=C_h_intern(&lf[281],43, C_text("chicken.compiler.support#foreign-type-check"));
lf[282]=C_h_intern(&lf[282],54, C_text("chicken.compiler.support#foreign-type-convert-argument"));
lf[283]=C_h_intern(&lf[283],24, C_text("##core#inline_loc_update"));
lf[284]=C_h_intern(&lf[284],4, C_text("set!"));
lf[285]=C_h_intern(&lf[285],30, C_text("##sys#unregister-syntax-export"));
lf[286]=C_h_intern(&lf[286],21, C_text("##sys#undefine-macro!"));
lf[287]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026assignment to syntax `"));
lf[288]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[289]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[290]=C_decode_literal(C_heaptop,C_text("\376B\000\000&assignment to imported value binding `"));
lf[291]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[292]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[293]=C_h_intern(&lf[293],12, C_text("##sys#macro\077"));
lf[294]=C_h_intern(&lf[294],21, C_text("C_DEBUG_GLOBAL_ASSIGN"));
lf[295]=C_h_intern(&lf[295],36, C_text("##compiler#always-bound-to-procedure"));
lf[296]=C_decode_literal(C_heaptop,C_text("\376B\000\000\025\047 possibly unintended"));
lf[297]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030assignment to variable `"));
lf[298]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[299]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[300]=C_h_intern(&lf[300],13, C_text("##core#inline"));
lf[301]=C_h_intern(&lf[301],22, C_text("##core#inline_allocate"));
lf[302]=C_h_intern(&lf[302],21, C_text("##core#compiletimetoo"));
lf[303]=C_h_intern(&lf[303],25, C_text("##core#elaborationtimetoo"));
lf[304]=C_h_intern(&lf[304],22, C_text("##core#compiletimeonly"));
lf[305]=C_h_intern(&lf[305],26, C_text("##core#elaborationtimeonly"));
lf[306]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[307]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[308]=C_h_intern(&lf[308],21, C_text("##core#foreign-lambda"));
lf[310]=C_h_intern(&lf[310],26, C_text("##core#foreign-safe-lambda"));
lf[311]=C_h_intern(&lf[311],22, C_text("##core#foreign-lambda\052"));
lf[313]=C_h_intern(&lf[313],27, C_text("##core#foreign-safe-lambda\052"));
lf[314]=C_h_intern(&lf[314],24, C_text("##core#foreign-primitive"));
lf[315]=C_h_intern(&lf[315],4, C_text("void"));
lf[317]=C_h_intern(&lf[317],33, C_text("chicken.string#string-intersperse"));
lf[318]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[319]=C_h_intern(&lf[319],30, C_text("##core#define-foreign-variable"));
lf[320]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[321]=C_h_intern(&lf[321],26, C_text("##core#define-foreign-type"));
lf[322]=C_h_intern(&lf[322],12, C_text("##sys#values"));
lf[323]=C_h_intern(&lf[323],21, C_text("##sys#register-export"));
lf[324]=C_h_intern(&lf[324],47, C_text("chicken.compiler.support#register-foreign-type!"));
lf[325]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[326]=C_decode_literal(C_heaptop,C_text("\376B\000\0009~adefinition of foreign type `~s\047 in non-toplevel context"));
lf[327]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[328]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[329]=C_h_intern(&lf[329],31, C_text("##core#define-external-variable"));
lf[330]=C_h_intern(&lf[330],9, C_text("c-pointer"));
lf[331]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[332]=C_h_intern(&lf[332],20, C_text("scheme#string-append"));
lf[333]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001&"));
lf[334]=C_decode_literal(C_heaptop,C_text("\376B\000\000>~aexternal variable definition of `~s\047 in non-toplevel context"));
lf[335]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[336]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[337]=C_h_intern(&lf[337],41, C_text("chicken.compiler.support#make-random-name"));
lf[338]=C_h_intern(&lf[338],19, C_text("##core#let-location"));
lf[339]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020C_a_i_bytevector"));
lf[340]=C_h_intern(&lf[340],37, C_text("chicken.compiler.support#bytes->words"));
lf[341]=C_h_intern(&lf[341],62, C_text("chicken.compiler.support#estimate-foreign-result-location-size"));
lf[342]=C_h_intern(&lf[342],20, C_text("##core#define-inline"));
lf[343]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[344]=C_decode_literal(C_heaptop,C_text("\376B\000\0003~ainline definition of `~s\047 in non-toplevel context"));
lf[345]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[346]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[347]=C_h_intern(&lf[347],13, C_text("scheme#cdaddr"));
lf[348]=C_h_intern(&lf[348],22, C_text("##core#define-constant"));
lf[349]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[350]=C_h_intern(&lf[350],6, C_text("define"));
lf[351]=C_h_intern(&lf[351],19, C_text("##compiler#constant"));
lf[352]=C_decode_literal(C_heaptop,C_text("\376B\000\000\010constant"));
lf[353]=C_decode_literal(C_heaptop,C_text("\376B\000\0004~ainvalid compile-time value for named constant `~S\047"));
lf[354]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[355]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[356]=C_h_intern(&lf[356],39, C_text("chicken.compiler.support#basic-literal\077"));
lf[357]=C_h_intern(&lf[357],45, C_text("chicken.compiler.support#collapsable-literal\077"));
lf[358]=C_decode_literal(C_heaptop,C_text("\376B\000\0005~aconstant definition of `~s\047 in non-toplevel context"));
lf[359]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[360]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[361]=C_decode_literal(C_heaptop,C_text("\376B\000\000:error in constant evaluation of ~S for named constant `~S\047"));
lf[362]=C_h_intern(&lf[362],11, C_text("scheme#eval"));
lf[363]=C_h_intern(&lf[363],14, C_text("##core#declare"));
lf[364]=C_h_intern(&lf[364],27, C_text("chicken.syntax#syntax-error"));
lf[365]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023invalid declaration"));
lf[366]=C_h_intern(&lf[366],12, C_text("##sys#notice"));
lf[367]=C_decode_literal(C_heaptop,C_text("\376B\000\0001ignoring declaration for locally bound variable `"));
lf[368]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[369]=C_h_intern(&lf[369],4, C_text("uses"));
lf[370]=C_h_intern(&lf[370],4, C_text("unit"));
lf[371]=C_decode_literal(C_heaptop,C_text("\376B\000\0003unit was already given a name (new name is ignored)"));
lf[372]=C_h_intern(&lf[372],17, C_text("standard-bindings"));
lf[373]=C_h_intern(&lf[373],17, C_text("extended-bindings"));
lf[374]=C_h_intern(&lf[374],18, C_text("usual-integrations"));
lf[375]=C_h_intern(&lf[375],11, C_text("number-type"));
lf[376]=C_h_intern(&lf[376],36, C_text("chicken.compiler.support#number-type"));
lf[377]=C_h_intern(&lf[377],6, C_text("fixnum"));
lf[378]=C_h_intern(&lf[378],17, C_text("fixnum-arithmetic"));
lf[379]=C_h_intern(&lf[379],7, C_text("generic"));
lf[380]=C_h_intern(&lf[380],6, C_text("unsafe"));
lf[381]=C_h_intern(&lf[381],4, C_text("safe"));
lf[382]=C_h_intern(&lf[382],15, C_text("no-bound-checks"));
lf[383]=C_h_intern(&lf[383],14, C_text("no-argc-checks"));
lf[384]=C_h_intern(&lf[384],19, C_text("no-procedure-checks"));
lf[385]=C_h_intern(&lf[385],18, C_text("disable-interrupts"));
lf[386]=C_h_intern(&lf[386],12, C_text("always-bound"));
lf[387]=C_h_intern(&lf[387],12, C_text("safe-globals"));
lf[388]=C_h_intern(&lf[388],38, C_text("no-procedure-checks-for-usual-bindings"));
lf[389]=C_h_intern(&lf[389],41, C_text("no-procedure-checks-for-toplevel-bindings"));
lf[390]=C_h_intern(&lf[390],18, C_text("bound-to-procedure"));
lf[391]=C_h_intern(&lf[391],15, C_text("foreign-declare"));
lf[392]=C_h_intern(&lf[392],7, C_text("declare"));
lf[393]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023invalid declaration"));
lf[394]=C_h_intern(&lf[394],14, C_text("scheme#string\077"));
lf[395]=C_h_intern(&lf[395],5, C_text("block"));
lf[396]=C_h_intern(&lf[396],8, C_text("separate"));
lf[397]=C_h_intern(&lf[397],20, C_text("keep-shadowed-macros"));
lf[398]=C_h_intern(&lf[398],6, C_text("unused"));
lf[399]=C_h_intern(&lf[399],17, C_text("##compiler#unused"));
lf[400]=C_h_intern(&lf[400],22, C_text("enforce-argument-types"));
lf[401]=C_h_intern(&lf[401],18, C_text("##compiler#enforce"));
lf[402]=C_h_intern(&lf[402],3, C_text("not"));
lf[403]=C_h_intern(&lf[403],6, C_text("inline"));
lf[404]=C_h_intern(&lf[404],17, C_text("##compiler#inline"));
lf[405]=C_h_intern(&lf[405],2, C_text("no"));
lf[406]=C_h_intern(&lf[406],13, C_text("inline-global"));
lf[407]=C_h_intern(&lf[407],24, C_text("##compiler#inline-global"));
lf[408]=C_decode_literal(C_heaptop,C_text("\376B\000\000!unsupported declaration specifier"));
lf[409]=C_h_intern(&lf[409],14, C_text("compile-syntax"));
lf[410]=C_h_intern(&lf[410],12, C_text("block-global"));
lf[411]=C_h_intern(&lf[411],4, C_text("hide"));
lf[412]=C_h_intern(&lf[412],6, C_text("export"));
lf[413]=C_h_intern(&lf[413],40, C_text("chicken.compiler.support#export-variable"));
lf[414]=C_h_intern(&lf[414],30, C_text("emit-external-prototypes-first"));
lf[415]=C_h_intern(&lf[415],16, C_text("##compiler#local"));
lf[416]=C_h_intern(&lf[416],12, C_text("inline-limit"));
lf[417]=C_decode_literal(C_heaptop,C_text("\376B\000\000.invalid argument to `inline-limit\047 declaration"));
lf[418]=C_h_intern(&lf[418],12, C_text("unroll-limit"));
lf[419]=C_decode_literal(C_heaptop,C_text("\376B\000\000.invalid argument to `unroll-limit\047 declaration"));
lf[420]=C_h_intern(&lf[420],4, C_text("pure"));
lf[421]=C_h_intern(&lf[421],15, C_text("##compiler#pure"));
lf[422]=C_decode_literal(C_heaptop,C_text("\376B\000\000+invalid arguments to `pure\047 declaration: ~S"));
lf[423]=C_h_intern(&lf[423],19, C_text("emit-import-library"));
lf[424]=C_decode_literal(C_heaptop,C_text("\376B\000\000\013.import.scm"));
lf[425]=C_decode_literal(C_heaptop,C_text("\376B\000\000\052invalid `import-library\047 specification: ~S"));
lf[426]=C_h_intern(&lf[426],15, C_text("emit-types-file"));
lf[427]=C_decode_literal(C_heaptop,C_text("\376B\000\000)invalid `emit-types-file\047 declaration: ~S"));
lf[428]=C_h_intern(&lf[428],7, C_text("profile"));
lf[429]=C_h_intern(&lf[429],5, C_text("local"));
lf[430]=C_h_intern(&lf[430],4, C_text("type"));
lf[431]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030illegal type declaration"));
lf[432]=C_h_intern(&lf[432],42, C_text("chicken.compiler.scrutinizer#validate-type"));
lf[433]=C_h_intern(&lf[433],52, C_text("chicken.compiler.scrutinizer#install-specializations"));
lf[434]=C_h_intern(&lf[434],20, C_text("##compiler#predicate"));
lf[435]=C_h_intern(&lf[435],22, C_text("##compiler#type-source"));
lf[436]=C_h_intern(&lf[436],15, C_text("##compiler#type"));
lf[437]=C_h_intern(&lf[437],9, C_text("procedure"));
lf[438]=C_decode_literal(C_heaptop,C_text("\376B\000\000\032illegal `type\047 declaration"));
lf[439]=C_h_intern(&lf[439],9, C_text("predicate"));
lf[440]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037illegal `predicate\047 declaration"));
lf[441]=C_decode_literal(C_heaptop,C_text("\376B\000\000\037illegal `type\047 declaration item"));
lf[442]=C_h_intern(&lf[442],10, C_text("specialize"));
lf[443]=C_h_intern(&lf[443],12, C_text("strict-types"));
lf[444]=C_decode_literal(C_heaptop,C_text("\376B\000\000\035unknown declaration specifier"));
lf[445]=C_decode_literal(C_heaptop,C_text("\376B\000\000!invalid declaration specification"));
lf[446]=C_h_intern(&lf[446],31, C_text("##core#foreign-callback-wrapper"));
lf[447]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001const\376\003\000\000\002\376\001\000\000\020\001nonnull-c-string\376\377\016"));
lf[448]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001const\376\003\000\000\002\376\001\000\000\031\001nonnull-unsigned-c-string\376\377\016"));
lf[449]=C_h_intern(&lf[449],25, C_text("nonnull-unsigned-c-string"));
lf[450]=C_h_intern(&lf[450],16, C_text("nonnull-c-string"));
lf[451]=C_h_intern(&lf[451],19, C_text("##sys#make-c-string"));
lf[452]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001const\376\003\000\000\002\376\001\000\000\011\001c-string\052\376\377\016"));
lf[453]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001const\376\003\000\000\002\376\001\000\000\022\001unsigned-c-string\052\376\377\016"));
lf[454]=C_h_intern(&lf[454],18, C_text("unsigned-c-string\052"));
lf[455]=C_h_intern(&lf[455],9, C_text("c-string\052"));
lf[456]=C_h_intern(&lf[456],13, C_text("c-string-list"));
lf[457]=C_h_intern(&lf[457],14, C_text("c-string-list\052"));
lf[458]=C_decode_literal(C_heaptop,C_text("\376B\000\000/not a valid result type for callback procedures"));
lf[459]=C_h_intern(&lf[459],8, C_text("c-string"));
lf[460]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001const\376\003\000\000\002\376\001\000\000\021\001unsigned-c-string\376\377\016"));
lf[461]=C_h_intern(&lf[461],17, C_text("unsigned-c-string"));
lf[462]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001const\376\003\000\000\002\376\001\000\000\010\001c-string\376\377\016"));
lf[463]=C_h_intern(&lf[463],1, C_text("r"));
lf[464]=C_h_intern(&lf[464],3, C_text("and"));
lf[465]=C_decode_literal(C_heaptop,C_text("\376B\000\000Anon-matching or invalid argument list to foreign callback-wrapper"));
lf[466]=C_decode_literal(C_heaptop,C_text("\376B\000\000>~aname `~S\047 of external definition is not a valid C identifier"));
lf[467]=C_decode_literal(C_heaptop,C_text("\376B\000\000\004) - "));
lf[468]=C_decode_literal(C_heaptop,C_text("\376B\000\000\000"));
lf[469]=C_h_intern(&lf[469],44, C_text("chicken.compiler.support#valid-c-identifier\077"));
lf[470]=C_h_intern(&lf[470],15, C_text("##core#location"));
lf[471]=C_h_intern(&lf[471],8, C_text("location"));
lf[472]=C_h_intern(&lf[472],19, C_text("##sys#make-locative"));
lf[473]=C_h_intern(&lf[473],20, C_text("##sys#syntax-context"));
lf[474]=C_h_intern(&lf[474],16, C_text("##core#primitive"));
lf[475]=C_h_intern(&lf[475],16, C_text("##core#undefined"));
lf[476]=C_h_intern(&lf[476],21, C_text("chicken.syntax#expand"));
lf[477]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030) - malformed expression"));
lf[478]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024malformed expression"));
lf[479]=C_h_intern(&lf[479],47, C_text("chicken.compiler.support#emit-syntax-trace-info"));
lf[480]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024malformed expression"));
lf[481]=C_decode_literal(C_heaptop,C_text("\376B\000\000\034literal in operator position"));
lf[482]=C_h_intern(&lf[482],24, C_text("chicken.keyword#keyword\077"));
lf[483]=C_h_intern(&lf[483],42, C_text("chicken.compiler.support#debugging-chicken"));
lf[484]=C_h_intern(&lf[484],14, C_text("scheme#newline"));
lf[485]=C_h_intern(&lf[485],46, C_text("chicken.compiler.core#build-toplevel-procedure"));
lf[486]=C_h_intern(&lf[486],34, C_text("chicken.compiler.support#make-node"));
lf[487]=C_h_intern(&lf[487],6, C_text("lambda"));
lf[488]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\377\016\376\377\016"));
lf[489]=C_h_intern(&lf[489],37, C_text("chicken.compiler.core#foreign-stub-id"));
lf[490]=C_h_intern(&lf[490],34, C_text("chicken.compiler.core#foreign-stub"));
lf[491]=C_h_intern(&lf[491],15, C_text("foreign-stub-id"));
lf[492]=C_h_intern(&lf[492],46, C_text("chicken.compiler.core#foreign-stub-return-type"));
lf[493]=C_h_intern(&lf[493],24, C_text("foreign-stub-return-type"));
lf[494]=C_h_intern(&lf[494],39, C_text("chicken.compiler.core#foreign-stub-name"));
lf[495]=C_h_intern(&lf[495],17, C_text("foreign-stub-name"));
lf[496]=C_h_intern(&lf[496],49, C_text("chicken.compiler.core#foreign-stub-argument-types"));
lf[497]=C_h_intern(&lf[497],27, C_text("foreign-stub-argument-types"));
lf[498]=C_h_intern(&lf[498],49, C_text("chicken.compiler.core#foreign-stub-argument-names"));
lf[499]=C_h_intern(&lf[499],27, C_text("foreign-stub-argument-names"));
lf[500]=C_h_intern(&lf[500],39, C_text("chicken.compiler.core#foreign-stub-body"));
lf[501]=C_h_intern(&lf[501],17, C_text("foreign-stub-body"));
lf[502]=C_h_intern(&lf[502],38, C_text("chicken.compiler.core#foreign-stub-cps"));
lf[503]=C_h_intern(&lf[503],16, C_text("foreign-stub-cps"));
lf[504]=C_h_intern(&lf[504],43, C_text("chicken.compiler.core#foreign-stub-callback"));
lf[505]=C_h_intern(&lf[505],21, C_text("foreign-stub-callback"));
lf[506]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\003\000\000\002\376\001\000\000\010\001##sys#gc\376\003\000\000\002\376\377\006\000\376\377\016\376\377\016"));
lf[507]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[508]=C_decode_literal(C_heaptop,C_text("\376B\000\000\020C_a_i_bytevector"));
lf[509]=C_h_intern(&lf[509],21, C_text("scheme#string->symbol"));
lf[510]=C_h_intern(&lf[510],53, C_text("chicken.compiler.support#estimate-foreign-result-size"));
lf[511]=C_h_intern(&lf[511],4, C_text("stub"));
lf[512]=C_h_intern(&lf[512],1, C_text("a"));
lf[513]=C_h_intern(&lf[513],19, C_text("chicken.string#conc"));
lf[514]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\052"));
lf[515]=C_h_intern(&lf[515],5, C_text("const"));
lf[516]=C_h_intern(&lf[516],6, C_text("struct"));
lf[517]=C_decode_literal(C_heaptop,C_text("\376B\000\000-name `~s\047 of foreign procedure has wrong type"));
lf[518]=C_decode_literal(C_heaptop,C_text("\376B\000\000\001\012"));
lf[519]=C_h_intern(&lf[519],44, C_text("chicken.compiler.core#perform-cps-conversion"));
lf[520]=C_h_intern(&lf[520],11, C_text("##core#call"));
lf[521]=C_h_intern(&lf[521],32, C_text("chicken.compiler.support#varnode"));
lf[522]=C_h_intern(&lf[522],1, C_text("k"));
lf[523]=C_h_intern(&lf[523],15, C_text("##core#variable"));
lf[524]=C_h_intern(&lf[524],40, C_text("chicken.compiler.support#node-parameters"));
lf[525]=C_h_intern(&lf[525],2, C_text("f_"));
lf[526]=C_h_intern(&lf[526],1, C_text("t"));
lf[527]=C_h_intern(&lf[527],44, C_text("chicken.compiler.support#node-subexpressions"));
lf[528]=C_h_intern(&lf[528],56, C_text("chicken.compiler.support#register-foreign-callback-stub!"));
lf[529]=C_h_intern(&lf[529],17, C_text("##core#the/result"));
lf[530]=C_h_intern(&lf[530],29, C_text("chicken.compiler.support#bomb"));
lf[531]=C_decode_literal(C_heaptop,C_text("\376B\000\000\016bad node (cps)"));
lf[532]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\005\001quote\376\003\000\000\002\376\001\000\000\017\001##core#variable\376\003\000\000\002\376\001\000\000\020\001##core#undefined\376\377\016"));
lf[533]=C_h_intern(&lf[533],40, C_text("chicken.compiler.core#analyze-expression"));
lf[534]=C_h_intern(&lf[534],15, C_text("##core#rest-cdr"));
lf[535]=C_h_intern(&lf[535],32, C_text("chicken.compiler.support#db-put!"));
lf[536]=C_h_intern(&lf[536],8, C_text("captured"));
lf[537]=C_h_intern(&lf[537],31, C_text("chicken.compiler.support#db-get"));
lf[538]=C_h_intern(&lf[538],14, C_text("##core#recurse"));
lf[539]=C_h_intern(&lf[539],33, C_text("chicken.compiler.support#collect!"));
lf[540]=C_decode_literal(C_heaptop,C_text("\376B\000\000;known procedure called with wrong number of arguments: `~A\047"));
lf[541]=C_h_intern(&lf[541],34, C_text("chicken.compiler.support#real-name"));
lf[542]=C_h_intern(&lf[542],37, C_text("chicken.compiler.support#llist-match\077"));
lf[543]=C_h_intern(&lf[543],7, C_text("unknown"));
lf[544]=C_h_intern(&lf[544],4, C_text("home"));
lf[545]=C_h_intern(&lf[545],20, C_text("##core#direct_lambda"));
lf[546]=C_h_intern(&lf[546],6, C_text("simple"));
lf[547]=C_h_intern(&lf[547],44, C_text("chicken.compiler.support#simple-lambda-node\077"));
lf[548]=C_h_intern(&lf[548],8, C_text("rest-cdr"));
lf[549]=C_h_intern(&lf[549],14, C_text("rest-parameter"));
lf[550]=C_h_intern(&lf[550],4, C_text("list"));
lf[551]=C_h_intern(&lf[551],12, C_text("contained-in"));
lf[552]=C_h_intern(&lf[552],8, C_text("contains"));
lf[553]=C_h_intern(&lf[553],8, C_text("assigned"));
lf[554]=C_h_intern(&lf[554],16, C_text("assigned-locally"));
lf[555]=C_h_intern(&lf[555],16, C_text("potential-values"));
lf[556]=C_h_intern(&lf[556],8, C_text("standard"));
lf[557]=C_decode_literal(C_heaptop,C_text("\376B\000\000 redefinition of standard binding"));
lf[558]=C_h_intern(&lf[558],8, C_text("extended"));
lf[559]=C_decode_literal(C_heaptop,C_text("\376B\000\000 redefinition of extended binding"));
lf[560]=C_h_intern(&lf[560],20, C_text("##compiler#intrinsic"));
lf[561]=C_h_intern(&lf[561],11, C_text("##core#proc"));
lf[562]=C_h_intern(&lf[562],17, C_text("derived-rest-vars"));
lf[563]=C_h_intern(&lf[563],10, C_text("rest-null\077"));
lf[564]=C_h_intern(&lf[564],9, C_text("undefined"));
lf[565]=C_h_intern(&lf[565],17, C_text("##core#rest-null\077"));
lf[566]=C_h_intern(&lf[566],22, C_text("chicken.base#alist-ref"));
lf[567]=C_h_intern(&lf[567],5, C_text("depth"));
lf[568]=C_h_intern(&lf[568],4, C_text("rvar"));
lf[569]=C_h_intern(&lf[569],35, C_text("chicken.compiler.support#db-get-all"));
lf[570]=C_h_intern(&lf[570],11, C_text("local-value"));
lf[571]=C_h_intern(&lf[571],42, C_text("chicken.compiler.support#variable-visible\077"));
lf[572]=C_h_intern(&lf[572],8, C_text("test-var"));
lf[573]=C_h_intern(&lf[573],35, C_text("chicken.compiler.support#match-node"));
lf[574]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\013\001##core#cond\376\003\000\000\002\376\377\016\376\003\000\000\002\376\003\000\000\002\376\001\000\000\017\001##core#variable\376\003\000\000\002\376\003\000\000\002\376\001\000\000\010\001tes"
"t-var\376\377\016\376\377\016\376\003\000\000\002\376\003\000\000\002\376\001\000\000\005\001quote\376\003\000\000\002\376\003\000\000\002\376\377\016\376\377\016\376\377\016\376\003\000\000\002\376\003\000\000\002\376\001\000\000\017\001##core#rest-c"
"dr\376\003\000\000\002\376\003\000\000\002\376\001\000\000\004\001rvar\376\003\000\000\002\376\001\000\000\005\001depth\376\377\016\376\377\016\376\377\016"));
lf[575]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\010\001test-var\376\003\000\000\002\376\001\000\000\004\001rvar\376\003\000\000\002\376\001\000\000\005\001depth\376\377\016"));
lf[576]=C_h_intern(&lf[576],10, C_text("references"));
lf[577]=C_h_intern(&lf[577],10, C_text("replacable"));
lf[578]=C_h_intern(&lf[578],9, C_text("replacing"));
lf[579]=C_h_intern(&lf[579],9, C_text("removable"));
lf[580]=C_h_intern(&lf[580],35, C_text("chicken.compiler.support#intrinsic\077"));
lf[581]=C_h_intern(&lf[581],19, C_text("##core#always-bound"));
lf[582]=C_h_intern(&lf[582],53, C_text("chicken.compiler.support#expression-has-side-effects\077"));
lf[583]=C_h_intern(&lf[583],21, C_text("has-unused-parameters"));
lf[584]=C_h_intern(&lf[584],15, C_text("consed-rest-arg"));
lf[585]=C_h_intern(&lf[585],13, C_text("explicit-rest"));
lf[586]=C_h_intern(&lf[586],19, C_text("chicken.base#rassoc"));
lf[587]=C_h_intern(&lf[587],10, C_text("scheme#eq\077"));
lf[588]=C_h_intern(&lf[588],11, C_text("collapsable"));
lf[589]=C_h_intern(&lf[589],12, C_text("contractable"));
lf[590]=C_h_intern(&lf[590],9, C_text("inlinable"));
lf[591]=C_h_intern(&lf[591],44, C_text("chicken.compiler.support#scan-free-variables"));
lf[592]=C_h_intern(&lf[592],11, C_text("hidden-refs"));
lf[593]=C_h_intern(&lf[593],34, C_text("chicken.compiler.support#debugging"));
lf[594]=C_h_intern(&lf[594],1, C_text("i"));
lf[595]=C_decode_literal(C_heaptop,C_text("\376B\000\000>global inlining candidate was assigned and will not be inlined"));
lf[596]=C_h_intern(&lf[596],30, C_text("chicken.compiler.support#node\077"));
lf[597]=C_h_intern(&lf[597],5, C_text("boxed"));
lf[598]=C_decode_literal(C_heaptop,C_text("\376B\000\000(\047 is only locally visible and never used"));
lf[599]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021global variable `"));
lf[600]=C_h_intern(&lf[600],41, C_text("chicken.compiler.support#variable-hidden\077"));
lf[601]=C_h_intern(&lf[601],1, C_text("p"));
lf[602]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033analysis gathering phase..."));
lf[603]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033analysis traversal phase..."));
lf[604]=C_h_intern(&lf[604],48, C_text("chicken.compiler.core#perform-closure-conversion"));
lf[605]=C_h_intern(&lf[605],12, C_text("customizable"));
lf[606]=C_decode_literal(C_heaptop,C_text("\376B\000\000\033let-node has invalid format"));
lf[607]=C_h_intern(&lf[607],45, C_text("chicken.compiler.support#node-parameters-set!"));
lf[608]=C_h_intern(&lf[608],8, C_text("toplevel"));
lf[609]=C_h_intern(&lf[609],18, C_text("captured-variables"));
lf[610]=C_h_intern(&lf[610],12, C_text("closure-size"));
lf[611]=C_h_intern(&lf[611],12, C_text("##core#unbox"));
lf[612]=C_h_intern(&lf[612],54, C_text("chicken.compiler.support#replace-rest-op-with-list-ops"));
lf[613]=C_decode_literal(C_heaptop,C_text("\376B\000\000;Saw rest op for captured variable.  This should not happen!"));
lf[614]=C_h_intern(&lf[614],10, C_text("##core#ref"));
lf[615]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\001\001i\376\377\016"));
lf[616]=C_h_intern(&lf[616],10, C_text("##core#box"));
lf[617]=C_h_intern(&lf[617],14, C_text("##core#closure"));
lf[618]=C_h_intern(&lf[618],30, C_text("chicken.compiler.support#qnode"));
lf[619]=C_h_intern(&lf[619],22, C_text("##sys#make-lambda-info"));
lf[620]=C_h_intern(&lf[620],1, C_text("\077"));
lf[621]=C_h_intern(&lf[621],23, C_text("chicken.string#->string"));
lf[622]=C_h_intern(&lf[622],10, C_text("boxed-rest"));
lf[623]=C_h_intern(&lf[623],18, C_text("##core#updatebox_i"));
lf[624]=C_h_intern(&lf[624],16, C_text("##core#updatebox"));
lf[625]=C_h_intern(&lf[625],15, C_text("##core#update_i"));
lf[626]=C_h_intern(&lf[626],13, C_text("##core#update"));
lf[627]=C_h_intern(&lf[627],35, C_text("chicken.compiler.support#immediate\077"));
lf[628]=C_decode_literal(C_heaptop,C_text("\376B\000\000\023bad node (closure2)"));
lf[629]=C_h_intern(&lf[629],13, C_text("##core#switch"));
lf[630]=C_h_intern(&lf[630],11, C_text("##core#cond"));
lf[631]=C_h_intern(&lf[631],18, C_text("##core#direct_call"));
lf[632]=C_h_intern(&lf[632],13, C_text("##core#return"));
lf[633]=C_h_intern(&lf[633],16, C_text("##core#let_float"));
lf[634]=C_h_intern(&lf[634],16, C_text("##core#box_float"));
lf[635]=C_h_intern(&lf[635],18, C_text("##core#unbox_float"));
lf[636]=C_h_intern(&lf[636],15, C_text("##core#rest-car"));
lf[637]=C_h_intern(&lf[637],18, C_text("##core#rest-length"));
lf[638]=C_h_intern(&lf[638],12, C_text("##core#float"));
lf[639]=C_h_intern(&lf[639],21, C_text("##core#float-variable"));
lf[640]=C_h_intern(&lf[640],1, C_text("o"));
lf[641]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026calls to known targets"));
lf[642]=C_decode_literal(C_heaptop,C_text("\376B\000\000\052closure conversion transformation phase..."));
lf[643]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027customizable procedures"));
lf[644]=C_decode_literal(C_heaptop,C_text("\376B\000\000%closure conversion gathering phase..."));
lf[645]=C_h_intern(&lf[645],39, C_text("chicken.compiler.core#lambda-literal-id"));
lf[646]=C_h_intern(&lf[646],36, C_text("chicken.compiler.core#lambda-literal"));
lf[647]=C_h_intern(&lf[647],17, C_text("lambda-literal-id"));
lf[648]=C_h_intern(&lf[648],45, C_text("chicken.compiler.core#lambda-literal-external"));
lf[649]=C_h_intern(&lf[649],23, C_text("lambda-literal-external"));
lf[650]=C_h_intern(&lf[650],51, C_text("chicken.compiler.core#lambda-literal-argument-count"));
lf[651]=C_h_intern(&lf[651],29, C_text("lambda-literal-argument-count"));
lf[652]=C_h_intern(&lf[652],50, C_text("chicken.compiler.core#lambda-literal-rest-argument"));
lf[653]=C_h_intern(&lf[653],28, C_text("lambda-literal-rest-argument"));
lf[654]=C_h_intern(&lf[654],48, C_text("chicken.compiler.core#lambda-literal-temporaries"));
lf[655]=C_h_intern(&lf[655],26, C_text("lambda-literal-temporaries"));
lf[656]=C_h_intern(&lf[656],54, C_text("chicken.compiler.core#lambda-literal-float-temporaries"));
lf[657]=C_h_intern(&lf[657],32, C_text("lambda-literal-float-temporaries"));
lf[658]=C_h_intern(&lf[658],54, C_text("chicken.compiler.core#lambda-literal-callee-signatures"));
lf[659]=C_h_intern(&lf[659],32, C_text("lambda-literal-callee-signatures"));
lf[660]=C_h_intern(&lf[660],46, C_text("chicken.compiler.core#lambda-literal-allocated"));
lf[661]=C_h_intern(&lf[661],24, C_text("lambda-literal-allocated"));
lf[662]=C_h_intern(&lf[662],49, C_text("chicken.compiler.core#lambda-literal-closure-size"));
lf[663]=C_h_intern(&lf[663],27, C_text("lambda-literal-closure-size"));
lf[664]=C_h_intern(&lf[664],44, C_text("chicken.compiler.core#lambda-literal-looping"));
lf[665]=C_h_intern(&lf[665],22, C_text("lambda-literal-looping"));
lf[666]=C_h_intern(&lf[666],49, C_text("chicken.compiler.core#lambda-literal-customizable"));
lf[667]=C_h_intern(&lf[667],27, C_text("lambda-literal-customizable"));
lf[668]=C_h_intern(&lf[668],55, C_text("chicken.compiler.core#lambda-literal-rest-argument-mode"));
lf[669]=C_h_intern(&lf[669],33, C_text("lambda-literal-rest-argument-mode"));
lf[670]=C_h_intern(&lf[670],41, C_text("chicken.compiler.core#lambda-literal-body"));
lf[671]=C_h_intern(&lf[671],19, C_text("lambda-literal-body"));
lf[672]=C_h_intern(&lf[672],43, C_text("chicken.compiler.core#lambda-literal-direct"));
lf[673]=C_h_intern(&lf[673],21, C_text("lambda-literal-direct"));
lf[674]=C_h_intern(&lf[674],49, C_text("chicken.compiler.core#prepare-for-code-generation"));
lf[675]=C_h_intern(&lf[675],12, C_text("##core#local"));
lf[676]=C_h_intern(&lf[676],14, C_text("##core#literal"));
lf[677]=C_h_intern(&lf[677],13, C_text("##core#global"));
lf[678]=C_h_intern(&lf[678],12, C_text("C_DEBUG_CALL"));
lf[679]=C_h_intern(&lf[679],42, C_text("chicken.compiler.support#source-info->name"));
lf[680]=C_h_intern(&lf[680],42, C_text("chicken.compiler.support#source-info->line"));
lf[681]=C_decode_literal(C_heaptop,C_text("\376B\000\000!identified direct recursive calls"));
lf[682]=C_decode_literal(C_heaptop,C_text("\376B\000\000\021bad direct lambda"));
lf[683]=C_h_intern(&lf[683],4, C_text("none"));
lf[684]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024unused rest argument"));
lf[685]=C_h_intern(&lf[685],18, C_text("##sys#fast-reverse"));
lf[686]=C_h_intern(&lf[686],20, C_text("chicken.base#butlast"));
lf[687]=C_h_intern(&lf[687],11, C_text("##core#bind"));
lf[688]=C_h_intern(&lf[688],15, C_text("##core#setlocal"));
lf[689]=C_h_intern(&lf[689],18, C_text("##core#setglobal_i"));
lf[690]=C_h_intern(&lf[690],16, C_text("##core#setglobal"));
lf[691]=C_h_intern(&lf[691],21, C_text("scheme#inexact->exact"));
lf[692]=C_decode_literal(C_heaptop,C_text("\376B\000\000\014\047 to fixnum "));
lf[693]=C_decode_literal(C_heaptop,C_text("\376B\000\000 coerced inexact literal number `"));
lf[694]=C_decode_literal(C_heaptop,C_text("\376B\000\000,cannot coerce inexact literal `~S\047 to fixnum"));
lf[695]=C_h_intern(&lf[695],36, C_text("chicken.compiler.support#big-fixnum\077"));
lf[696]=C_h_intern(&lf[696],10, C_text("scheme#max"));
lf[697]=C_h_intern(&lf[697],52, C_text("chicken.compiler.support#make-block-variable-literal"));
lf[698]=C_h_intern(&lf[698],52, C_text("chicken.compiler.support#block-variable-literal-name"));
lf[699]=C_h_intern(&lf[699],48, C_text("chicken.compiler.support#block-variable-literal\077"));
lf[700]=C_h_intern(&lf[700],21, C_text("##sys#undefined-value"));
lf[701]=C_h_intern(&lf[701],16, C_text("##core#immediate"));
lf[702]=C_h_intern(&lf[702],3, C_text("fix"));
lf[703]=C_h_intern(&lf[703],4, C_text("bool"));
lf[704]=C_h_intern(&lf[704],4, C_text("char"));
lf[705]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001nil\376\377\016"));
lf[706]=C_decode_literal(C_heaptop,C_text("\376\003\000\000\002\376\001\000\000\003\001eof\376\377\016"));
lf[707]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027bad immediate (prepare)"));
lf[708]=C_decode_literal(C_heaptop,C_text("\376B\000\000\027fast global assignments"));
lf[709]=C_decode_literal(C_heaptop,C_text("\376B\000\000\026fast global references"));
lf[710]=C_decode_literal(C_heaptop,C_text("\376B\000\000\030fast box initializations"));
lf[711]=C_decode_literal(C_heaptop,C_text("\376B\000\000\024preparation phase..."));
C_register_lf2(lf,712,create_ptable());{}
t2=(*a=C_CLOSURE_TYPE|2,a[1]=(C_word)f_6259,a[2]=t1,tmp=(C_word)a,a+=3,tmp);{
C_word *av2=av;
av2[0]=C_SCHEME_UNDEFINED;
av2[1]=t2;
C_library_toplevel(2,av2);}}

/* va26382 in k21618 in k21609 in k21606 in k21603 in walk in k21492 in chicken.compiler.core#prepare-for-code-generation in k6284 in k6281 in k6278 in k6275 in k6272 in k6269 in k6266 in k6263 in k6260 in k6257 */
static void C_fcall va26382(C_word t0,C_word t1){
C_word tmp;
C_word t2;
C_word t3;
C_word t4;
C_word t5;
C_word *a;
if(C_unlikely(!C_demand(C_calculate_demand(33,0,6)))){
C_save_and_reclaim_args((void *)trva26382,2,t0,t1);}
a=C_alloc(33);
t2=C_s_a_i_plus(&a,2,t1,C_fix(1));
t3=C_mutate(((C_word *)((C_word*)t0)[2])+1,t2);
t4=(*a=C_CLOSURE_TYPE|3,a[1]=(C_word)f_21796,a[2]=((C_word*)t0)[3],a[3]=((C_word*)t0)[4],tmp=(C_word)a,a+=4,tmp);
/* core.scm:2930: mapwalk */
t5=((C_word*)((C_word*)t0)[5])[1];
f_22811(t5,t4,((C_word*)t0)[6],((C_word*)t0)[7],((C_word*)t0)[8],((C_word*)t0)[9],((C_word*)t0)[10]);}

#ifdef C_ENABLE_PTABLES
static C_PTABLE_ENTRY ptable[1496] = {
{C_text("f25485:core_2escm"),(void*)f25485},
{C_text("f_10014:core_2escm"),(void*)f_10014},
{C_text("f_10018:core_2escm"),(void*)f_10018},
{C_text("f_10033:core_2escm"),(void*)f_10033},
{C_text("f_10036:core_2escm"),(void*)f_10036},
{C_text("f_10039:core_2escm"),(void*)f_10039},
{C_text("f_10043:core_2escm"),(void*)f_10043},
{C_text("f_10062:core_2escm"),(void*)f_10062},
{C_text("f_10066:core_2escm"),(void*)f_10066},
{C_text("f_10070:core_2escm"),(void*)f_10070},
{C_text("f_10073:core_2escm"),(void*)f_10073},
{C_text("f_10079:core_2escm"),(void*)f_10079},
{C_text("f_10087:core_2escm"),(void*)f_10087},
{C_text("f_10093:core_2escm"),(void*)f_10093},
{C_text("f_10097:core_2escm"),(void*)f_10097},
{C_text("f_10100:core_2escm"),(void*)f_10100},
{C_text("f_10103:core_2escm"),(void*)f_10103},
{C_text("f_10128:core_2escm"),(void*)f_10128},
{C_text("f_10130:core_2escm"),(void*)f_10130},
{C_text("f_10171:core_2escm"),(void*)f_10171},
{C_text("f_10174:core_2escm"),(void*)f_10174},
{C_text("f_10177:core_2escm"),(void*)f_10177},
{C_text("f_10180:core_2escm"),(void*)f_10180},
{C_text("f_10183:core_2escm"),(void*)f_10183},
{C_text("f_10186:core_2escm"),(void*)f_10186},
{C_text("f_10208:core_2escm"),(void*)f_10208},
{C_text("f_10212:core_2escm"),(void*)f_10212},
{C_text("f_10216:core_2escm"),(void*)f_10216},
{C_text("f_10230:core_2escm"),(void*)f_10230},
{C_text("f_10261:core_2escm"),(void*)f_10261},
{C_text("f_10283:core_2escm"),(void*)f_10283},
{C_text("f_10286:core_2escm"),(void*)f_10286},
{C_text("f_10289:core_2escm"),(void*)f_10289},
{C_text("f_10337:core_2escm"),(void*)f_10337},
{C_text("f_10341:core_2escm"),(void*)f_10341},
{C_text("f_10345:core_2escm"),(void*)f_10345},
{C_text("f_10353:core_2escm"),(void*)f_10353},
{C_text("f_10363:core_2escm"),(void*)f_10363},
{C_text("f_10367:core_2escm"),(void*)f_10367},
{C_text("f_10370:core_2escm"),(void*)f_10370},
{C_text("f_10377:core_2escm"),(void*)f_10377},
{C_text("f_10381:core_2escm"),(void*)f_10381},
{C_text("f_10391:core_2escm"),(void*)f_10391},
{C_text("f_10395:core_2escm"),(void*)f_10395},
{C_text("f_10399:core_2escm"),(void*)f_10399},
{C_text("f_10420:core_2escm"),(void*)f_10420},
{C_text("f_10423:core_2escm"),(void*)f_10423},
{C_text("f_10431:core_2escm"),(void*)f_10431},
{C_text("f_10453:core_2escm"),(void*)f_10453},
{C_text("f_10463:core_2escm"),(void*)f_10463},
{C_text("f_10476:core_2escm"),(void*)f_10476},
{C_text("f_10484:core_2escm"),(void*)f_10484},
{C_text("f_10490:core_2escm"),(void*)f_10490},
{C_text("f_10512:core_2escm"),(void*)f_10512},
{C_text("f_10522:core_2escm"),(void*)f_10522},
{C_text("f_10535:core_2escm"),(void*)f_10535},
{C_text("f_10560:core_2escm"),(void*)f_10560},
{C_text("f_10578:core_2escm"),(void*)f_10578},
{C_text("f_10583:core_2escm"),(void*)f_10583},
{C_text("f_10597:core_2escm"),(void*)f_10597},
{C_text("f_10605:core_2escm"),(void*)f_10605},
{C_text("f_10615:core_2escm"),(void*)f_10615},
{C_text("f_10628:core_2escm"),(void*)f_10628},
{C_text("f_10649:core_2escm"),(void*)f_10649},
{C_text("f_10661:core_2escm"),(void*)f_10661},
{C_text("f_10663:core_2escm"),(void*)f_10663},
{C_text("f_10677:core_2escm"),(void*)f_10677},
{C_text("f_10702:core_2escm"),(void*)f_10702},
{C_text("f_10718:core_2escm"),(void*)f_10718},
{C_text("f_10727:core_2escm"),(void*)f_10727},
{C_text("f_10730:core_2escm"),(void*)f_10730},
{C_text("f_10733:core_2escm"),(void*)f_10733},
{C_text("f_10736:core_2escm"),(void*)f_10736},
{C_text("f_10739:core_2escm"),(void*)f_10739},
{C_text("f_10742:core_2escm"),(void*)f_10742},
{C_text("f_10745:core_2escm"),(void*)f_10745},
{C_text("f_10752:core_2escm"),(void*)f_10752},
{C_text("f_10760:core_2escm"),(void*)f_10760},
{C_text("f_10766:core_2escm"),(void*)f_10766},
{C_text("f_10769:core_2escm"),(void*)f_10769},
{C_text("f_10772:core_2escm"),(void*)f_10772},
{C_text("f_10777:core_2escm"),(void*)f_10777},
{C_text("f_10781:core_2escm"),(void*)f_10781},
{C_text("f_10784:core_2escm"),(void*)f_10784},
{C_text("f_10788:core_2escm"),(void*)f_10788},
{C_text("f_10792:core_2escm"),(void*)f_10792},
{C_text("f_10795:core_2escm"),(void*)f_10795},
{C_text("f_10798:core_2escm"),(void*)f_10798},
{C_text("f_10807:core_2escm"),(void*)f_10807},
{C_text("f_10812:core_2escm"),(void*)f_10812},
{C_text("f_10819:core_2escm"),(void*)f_10819},
{C_text("f_10827:core_2escm"),(void*)f_10827},
{C_text("f_10852:core_2escm"),(void*)f_10852},
{C_text("f_10865:core_2escm"),(void*)f_10865},
{C_text("f_10869:core_2escm"),(void*)f_10869},
{C_text("f_10872:core_2escm"),(void*)f_10872},
{C_text("f_10875:core_2escm"),(void*)f_10875},
{C_text("f_10878:core_2escm"),(void*)f_10878},
{C_text("f_10897:core_2escm"),(void*)f_10897},
{C_text("f_10900:core_2escm"),(void*)f_10900},
{C_text("f_10910:core_2escm"),(void*)f_10910},
{C_text("f_10912:core_2escm"),(void*)f_10912},
{C_text("f_10923:core_2escm"),(void*)f_10923},
{C_text("f_10937:core_2escm"),(void*)f_10937},
{C_text("f_10941:core_2escm"),(void*)f_10941},
{C_text("f_10944:core_2escm"),(void*)f_10944},
{C_text("f_10947:core_2escm"),(void*)f_10947},
{C_text("f_10950:core_2escm"),(void*)f_10950},
{C_text("f_10954:core_2escm"),(void*)f_10954},
{C_text("f_10958:core_2escm"),(void*)f_10958},
{C_text("f_10962:core_2escm"),(void*)f_10962},
{C_text("f_10966:core_2escm"),(void*)f_10966},
{C_text("f_10969:core_2escm"),(void*)f_10969},
{C_text("f_10972:core_2escm"),(void*)f_10972},
{C_text("f_10975:core_2escm"),(void*)f_10975},
{C_text("f_10978:core_2escm"),(void*)f_10978},
{C_text("f_10993:core_2escm"),(void*)f_10993},
{C_text("f_10999:core_2escm"),(void*)f_10999},
{C_text("f_11009:core_2escm"),(void*)f_11009},
{C_text("f_11019:core_2escm"),(void*)f_11019},
{C_text("f_11024:core_2escm"),(void*)f_11024},
{C_text("f_11030:core_2escm"),(void*)f_11030},
{C_text("f_11036:core_2escm"),(void*)f_11036},
{C_text("f_11040:core_2escm"),(void*)f_11040},
{C_text("f_11045:core_2escm"),(void*)f_11045},
{C_text("f_11047:core_2escm"),(void*)f_11047},
{C_text("f_11051:core_2escm"),(void*)f_11051},
{C_text("f_11056:core_2escm"),(void*)f_11056},
{C_text("f_11069:core_2escm"),(void*)f_11069},
{C_text("f_11075:core_2escm"),(void*)f_11075},
{C_text("f_11081:core_2escm"),(void*)f_11081},
{C_text("f_11082:core_2escm"),(void*)f_11082},
{C_text("f_11088:core_2escm"),(void*)f_11088},
{C_text("f_11099:core_2escm"),(void*)f_11099},
{C_text("f_11114:core_2escm"),(void*)f_11114},
{C_text("f_11118:core_2escm"),(void*)f_11118},
{C_text("f_11122:core_2escm"),(void*)f_11122},
{C_text("f_11125:core_2escm"),(void*)f_11125},
{C_text("f_11128:core_2escm"),(void*)f_11128},
{C_text("f_11131:core_2escm"),(void*)f_11131},
{C_text("f_11134:core_2escm"),(void*)f_11134},
{C_text("f_11137:core_2escm"),(void*)f_11137},
{C_text("f_11140:core_2escm"),(void*)f_11140},
{C_text("f_11143:core_2escm"),(void*)f_11143},
{C_text("f_11150:core_2escm"),(void*)f_11150},
{C_text("f_11160:core_2escm"),(void*)f_11160},
{C_text("f_11164:core_2escm"),(void*)f_11164},
{C_text("f_11177:core_2escm"),(void*)f_11177},
{C_text("f_11188:core_2escm"),(void*)f_11188},
{C_text("f_11213:core_2escm"),(void*)f_11213},
{C_text("f_11221:core_2escm"),(void*)f_11221},
{C_text("f_11246:core_2escm"),(void*)f_11246},
{C_text("f_11294:core_2escm"),(void*)f_11294},
{C_text("f_11297:core_2escm"),(void*)f_11297},
{C_text("f_11300:core_2escm"),(void*)f_11300},
{C_text("f_11306:core_2escm"),(void*)f_11306},
{C_text("f_11309:core_2escm"),(void*)f_11309},
{C_text("f_11314:core_2escm"),(void*)f_11314},
{C_text("f_11318:core_2escm"),(void*)f_11318},
{C_text("f_11322:core_2escm"),(void*)f_11322},
{C_text("f_11325:core_2escm"),(void*)f_11325},
{C_text("f_11331:core_2escm"),(void*)f_11331},
{C_text("f_11339:core_2escm"),(void*)f_11339},
{C_text("f_11343:core_2escm"),(void*)f_11343},
{C_text("f_11345:core_2escm"),(void*)f_11345},
{C_text("f_11349:core_2escm"),(void*)f_11349},
{C_text("f_11352:core_2escm"),(void*)f_11352},
{C_text("f_11357:core_2escm"),(void*)f_11357},
{C_text("f_11359:core_2escm"),(void*)f_11359},
{C_text("f_11384:core_2escm"),(void*)f_11384},
{C_text("f_11400:core_2escm"),(void*)f_11400},
{C_text("f_11406:core_2escm"),(void*)f_11406},
{C_text("f_11409:core_2escm"),(void*)f_11409},
{C_text("f_11416:core_2escm"),(void*)f_11416},
{C_text("f_11419:core_2escm"),(void*)f_11419},
{C_text("f_11425:core_2escm"),(void*)f_11425},
{C_text("f_11428:core_2escm"),(void*)f_11428},
{C_text("f_11431:core_2escm"),(void*)f_11431},
{C_text("f_11446:core_2escm"),(void*)f_11446},
{C_text("f_11449:core_2escm"),(void*)f_11449},
{C_text("f_11455:core_2escm"),(void*)f_11455},
{C_text("f_11462:core_2escm"),(void*)f_11462},
{C_text("f_11469:core_2escm"),(void*)f_11469},
{C_text("f_11492:core_2escm"),(void*)f_11492},
{C_text("f_11504:core_2escm"),(void*)f_11504},
{C_text("f_11514:core_2escm"),(void*)f_11514},
{C_text("f_11521:core_2escm"),(void*)f_11521},
{C_text("f_11548:core_2escm"),(void*)f_11548},
{C_text("f_11556:core_2escm"),(void*)f_11556},
{C_text("f_11562:core_2escm"),(void*)f_11562},
{C_text("f_11569:core_2escm"),(void*)f_11569},
{C_text("f_11580:core_2escm"),(void*)f_11580},
{C_text("f_11583:core_2escm"),(void*)f_11583},
{C_text("f_11586:core_2escm"),(void*)f_11586},
{C_text("f_11592:core_2escm"),(void*)f_11592},
{C_text("f_11595:core_2escm"),(void*)f_11595},
{C_text("f_11601:core_2escm"),(void*)f_11601},
{C_text("f_11608:core_2escm"),(void*)f_11608},
{C_text("f_11612:core_2escm"),(void*)f_11612},
{C_text("f_11618:core_2escm"),(void*)f_11618},
{C_text("f_11621:core_2escm"),(void*)f_11621},
{C_text("f_11624:core_2escm"),(void*)f_11624},
{C_text("f_11627:core_2escm"),(void*)f_11627},
{C_text("f_11630:core_2escm"),(void*)f_11630},
{C_text("f_11634:core_2escm"),(void*)f_11634},
{C_text("f_11637:core_2escm"),(void*)f_11637},
{C_text("f_11643:core_2escm"),(void*)f_11643},
{C_text("f_11646:core_2escm"),(void*)f_11646},
{C_text("f_11649:core_2escm"),(void*)f_11649},
{C_text("f_11665:core_2escm"),(void*)f_11665},
{C_text("f_11671:core_2escm"),(void*)f_11671},
{C_text("f_11674:core_2escm"),(void*)f_11674},
{C_text("f_11677:core_2escm"),(void*)f_11677},
{C_text("f_11680:core_2escm"),(void*)f_11680},
{C_text("f_11683:core_2escm"),(void*)f_11683},
{C_text("f_11687:core_2escm"),(void*)f_11687},
{C_text("f_11690:core_2escm"),(void*)f_11690},
{C_text("f_11696:core_2escm"),(void*)f_11696},
{C_text("f_11699:core_2escm"),(void*)f_11699},
{C_text("f_11702:core_2escm"),(void*)f_11702},
{C_text("f_11709:core_2escm"),(void*)f_11709},
{C_text("f_11713:core_2escm"),(void*)f_11713},
{C_text("f_11736:core_2escm"),(void*)f_11736},
{C_text("f_11749:core_2escm"),(void*)f_11749},
{C_text("f_11755:core_2escm"),(void*)f_11755},
{C_text("f_11758:core_2escm"),(void*)f_11758},
{C_text("f_11761:core_2escm"),(void*)f_11761},
{C_text("f_11764:core_2escm"),(void*)f_11764},
{C_text("f_11767:core_2escm"),(void*)f_11767},
{C_text("f_11771:core_2escm"),(void*)f_11771},
{C_text("f_11774:core_2escm"),(void*)f_11774},
{C_text("f_11780:core_2escm"),(void*)f_11780},
{C_text("f_11783:core_2escm"),(void*)f_11783},
{C_text("f_11786:core_2escm"),(void*)f_11786},
{C_text("f_11815:core_2escm"),(void*)f_11815},
{C_text("f_11823:core_2escm"),(void*)f_11823},
{C_text("f_11831:core_2escm"),(void*)f_11831},
{C_text("f_11833:core_2escm"),(void*)f_11833},
{C_text("f_11858:core_2escm"),(void*)f_11858},
{C_text("f_11882:core_2escm"),(void*)f_11882},
{C_text("f_11886:core_2escm"),(void*)f_11886},
{C_text("f_11920:core_2escm"),(void*)f_11920},
{C_text("f_11924:core_2escm"),(void*)f_11924},
{C_text("f_11930:core_2escm"),(void*)f_11930},
{C_text("f_11955:core_2escm"),(void*)f_11955},
{C_text("f_11979:core_2escm"),(void*)f_11979},
{C_text("f_12000:core_2escm"),(void*)f_12000},
{C_text("f_12004:core_2escm"),(void*)f_12004},
{C_text("f_12027:core_2escm"),(void*)f_12027},
{C_text("f_12042:core_2escm"),(void*)f_12042},
{C_text("f_12065:core_2escm"),(void*)f_12065},
{C_text("f_12069:core_2escm"),(void*)f_12069},
{C_text("f_12087:core_2escm"),(void*)f_12087},
{C_text("f_12094:core_2escm"),(void*)f_12094},
{C_text("f_12098:core_2escm"),(void*)f_12098},
{C_text("f_12113:core_2escm"),(void*)f_12113},
{C_text("f_12126:core_2escm"),(void*)f_12126},
{C_text("f_12139:core_2escm"),(void*)f_12139},
{C_text("f_12152:core_2escm"),(void*)f_12152},
{C_text("f_12165:core_2escm"),(void*)f_12165},
{C_text("f_12174:core_2escm"),(void*)f_12174},
{C_text("f_12177:core_2escm"),(void*)f_12177},
{C_text("f_12180:core_2escm"),(void*)f_12180},
{C_text("f_12221:core_2escm"),(void*)f_12221},
{C_text("f_12227:core_2escm"),(void*)f_12227},
{C_text("f_12236:core_2escm"),(void*)f_12236},
{C_text("f_12239:core_2escm"),(void*)f_12239},
{C_text("f_12242:core_2escm"),(void*)f_12242},
{C_text("f_12245:core_2escm"),(void*)f_12245},
{C_text("f_12248:core_2escm"),(void*)f_12248},
{C_text("f_12251:core_2escm"),(void*)f_12251},
{C_text("f_12254:core_2escm"),(void*)f_12254},
{C_text("f_12257:core_2escm"),(void*)f_12257},
{C_text("f_12260:core_2escm"),(void*)f_12260},
{C_text("f_12275:core_2escm"),(void*)f_12275},
{C_text("f_12295:core_2escm"),(void*)f_12295},
{C_text("f_12299:core_2escm"),(void*)f_12299},
{C_text("f_12302:core_2escm"),(void*)f_12302},
{C_text("f_12309:core_2escm"),(void*)f_12309},
{C_text("f_12312:core_2escm"),(void*)f_12312},
{C_text("f_12318:core_2escm"),(void*)f_12318},
{C_text("f_12321:core_2escm"),(void*)f_12321},
{C_text("f_12324:core_2escm"),(void*)f_12324},
{C_text("f_12343:core_2escm"),(void*)f_12343},
{C_text("f_12346:core_2escm"),(void*)f_12346},
{C_text("f_12355:core_2escm"),(void*)f_12355},
{C_text("f_12358:core_2escm"),(void*)f_12358},
{C_text("f_12361:core_2escm"),(void*)f_12361},
{C_text("f_12381:core_2escm"),(void*)f_12381},
{C_text("f_12389:core_2escm"),(void*)f_12389},
{C_text("f_12400:core_2escm"),(void*)f_12400},
{C_text("f_12403:core_2escm"),(void*)f_12403},
{C_text("f_12409:core_2escm"),(void*)f_12409},
{C_text("f_12412:core_2escm"),(void*)f_12412},
{C_text("f_12415:core_2escm"),(void*)f_12415},
{C_text("f_12430:core_2escm"),(void*)f_12430},
{C_text("f_12433:core_2escm"),(void*)f_12433},
{C_text("f_12436:core_2escm"),(void*)f_12436},
{C_text("f_12442:core_2escm"),(void*)f_12442},
{C_text("f_12454:core_2escm"),(void*)f_12454},
{C_text("f_12458:core_2escm"),(void*)f_12458},
{C_text("f_12462:core_2escm"),(void*)f_12462},
{C_text("f_12465:core_2escm"),(void*)f_12465},
{C_text("f_12471:core_2escm"),(void*)f_12471},
{C_text("f_12491:core_2escm"),(void*)f_12491},
{C_text("f_12495:core_2escm"),(void*)f_12495},
{C_text("f_12520:core_2escm"),(void*)f_12520},
{C_text("f_12543:core_2escm"),(void*)f_12543},
{C_text("f_12545:core_2escm"),(void*)f_12545},
{C_text("f_12549:core_2escm"),(void*)f_12549},
{C_text("f_12552:core_2escm"),(void*)f_12552},
{C_text("f_12557:core_2escm"),(void*)f_12557},
{C_text("f_12590:core_2escm"),(void*)f_12590},
{C_text("f_12593:core_2escm"),(void*)f_12593},
{C_text("f_12596:core_2escm"),(void*)f_12596},
{C_text("f_12603:core_2escm"),(void*)f_12603},
{C_text("f_12606:core_2escm"),(void*)f_12606},
{C_text("f_12612:core_2escm"),(void*)f_12612},
{C_text("f_12615:core_2escm"),(void*)f_12615},
{C_text("f_12618:core_2escm"),(void*)f_12618},
{C_text("f_12625:core_2escm"),(void*)f_12625},
{C_text("f_12637:core_2escm"),(void*)f_12637},
{C_text("f_12643:core_2escm"),(void*)f_12643},
{C_text("f_12646:core_2escm"),(void*)f_12646},
{C_text("f_12649:core_2escm"),(void*)f_12649},
{C_text("f_12659:core_2escm"),(void*)f_12659},
{C_text("f_12662:core_2escm"),(void*)f_12662},
{C_text("f_12672:core_2escm"),(void*)f_12672},
{C_text("f_12675:core_2escm"),(void*)f_12675},
{C_text("f_12678:core_2escm"),(void*)f_12678},
{C_text("f_12681:core_2escm"),(void*)f_12681},
{C_text("f_12684:core_2escm"),(void*)f_12684},
{C_text("f_12687:core_2escm"),(void*)f_12687},
{C_text("f_12705:core_2escm"),(void*)f_12705},
{C_text("f_12708:core_2escm"),(void*)f_12708},
{C_text("f_12714:core_2escm"),(void*)f_12714},
{C_text("f_12717:core_2escm"),(void*)f_12717},
{C_text("f_12720:core_2escm"),(void*)f_12720},
{C_text("f_12738:core_2escm"),(void*)f_12738},
{C_text("f_12741:core_2escm"),(void*)f_12741},
{C_text("f_12747:core_2escm"),(void*)f_12747},
{C_text("f_12750:core_2escm"),(void*)f_12750},
{C_text("f_12753:core_2escm"),(void*)f_12753},
{C_text("f_12758:core_2escm"),(void*)f_12758},
{C_text("f_12764:core_2escm"),(void*)f_12764},
{C_text("f_12770:core_2escm"),(void*)f_12770},
{C_text("f_12776:core_2escm"),(void*)f_12776},
{C_text("f_12782:core_2escm"),(void*)f_12782},
{C_text("f_12789:core_2escm"),(void*)f_12789},
{C_text("f_12808:core_2escm"),(void*)f_12808},
{C_text("f_12814:core_2escm"),(void*)f_12814},
{C_text("f_12836:core_2escm"),(void*)f_12836},
{C_text("f_12842:core_2escm"),(void*)f_12842},
{C_text("f_12850:core_2escm"),(void*)f_12850},
{C_text("f_12857:core_2escm"),(void*)f_12857},
{C_text("f_12859:core_2escm"),(void*)f_12859},
{C_text("f_12884:core_2escm"),(void*)f_12884},
{C_text("f_12902:core_2escm"),(void*)f_12902},
{C_text("f_12912:core_2escm"),(void*)f_12912},
{C_text("f_12922:core_2escm"),(void*)f_12922},
{C_text("f_12934:core_2escm"),(void*)f_12934},
{C_text("f_12937:core_2escm"),(void*)f_12937},
{C_text("f_12944:core_2escm"),(void*)f_12944},
{C_text("f_12948:core_2escm"),(void*)f_12948},
{C_text("f_12956:core_2escm"),(void*)f_12956},
{C_text("f_12968:core_2escm"),(void*)f_12968},
{C_text("f_12972:core_2escm"),(void*)f_12972},
{C_text("f_12984:core_2escm"),(void*)f_12984},
{C_text("f_13044:core_2escm"),(void*)f_13044},
{C_text("f_13073:core_2escm"),(void*)f_13073},
{C_text("f_13097:core_2escm"),(void*)f_13097},
{C_text("f_13105:core_2escm"),(void*)f_13105},
{C_text("f_13109:core_2escm"),(void*)f_13109},
{C_text("f_13113:core_2escm"),(void*)f_13113},
{C_text("f_13154:core_2escm"),(void*)f_13154},
{C_text("f_13165:core_2escm"),(void*)f_13165},
{C_text("f_13172:core_2escm"),(void*)f_13172},
{C_text("f_13175:core_2escm"),(void*)f_13175},
{C_text("f_13181:core_2escm"),(void*)f_13181},
{C_text("f_13184:core_2escm"),(void*)f_13184},
{C_text("f_13187:core_2escm"),(void*)f_13187},
{C_text("f_13224:core_2escm"),(void*)f_13224},
{C_text("f_13250:core_2escm"),(void*)f_13250},
{C_text("f_13278:core_2escm"),(void*)f_13278},
{C_text("f_13293:core_2escm"),(void*)f_13293},
{C_text("f_13310:core_2escm"),(void*)f_13310},
{C_text("f_13316:core_2escm"),(void*)f_13316},
{C_text("f_13319:core_2escm"),(void*)f_13319},
{C_text("f_13339:core_2escm"),(void*)f_13339},
{C_text("f_13344:core_2escm"),(void*)f_13344},
{C_text("f_13350:core_2escm"),(void*)f_13350},
{C_text("f_13489:core_2escm"),(void*)f_13489},
{C_text("f_13494:core_2escm"),(void*)f_13494},
{C_text("f_13502:core_2escm"),(void*)f_13502},
{C_text("f_13504:core_2escm"),(void*)f_13504},
{C_text("f_13522:core_2escm"),(void*)f_13522},
{C_text("f_13528:core_2escm"),(void*)f_13528},
{C_text("f_13531:core_2escm"),(void*)f_13531},
{C_text("f_13534:core_2escm"),(void*)f_13534},
{C_text("f_13537:core_2escm"),(void*)f_13537},
{C_text("f_13555:core_2escm"),(void*)f_13555},
{C_text("f_13558:core_2escm"),(void*)f_13558},
{C_text("f_13561:core_2escm"),(void*)f_13561},
{C_text("f_13567:core_2escm"),(void*)f_13567},
{C_text("f_13570:core_2escm"),(void*)f_13570},
{C_text("f_13609:core_2escm"),(void*)f_13609},
{C_text("f_13614:core_2escm"),(void*)f_13614},
{C_text("f_13626:core_2escm"),(void*)f_13626},
{C_text("f_13651:core_2escm"),(void*)f_13651},
{C_text("f_13661:core_2escm"),(void*)f_13661},
{C_text("f_13672:core_2escm"),(void*)f_13672},
{C_text("f_13676:core_2escm"),(void*)f_13676},
{C_text("f_13686:core_2escm"),(void*)f_13686},
{C_text("f_13692:core_2escm"),(void*)f_13692},
{C_text("f_13700:core_2escm"),(void*)f_13700},
{C_text("f_13735:core_2escm"),(void*)f_13735},
{C_text("f_13742:core_2escm"),(void*)f_13742},
{C_text("f_13745:core_2escm"),(void*)f_13745},
{C_text("f_13752:core_2escm"),(void*)f_13752},
{C_text("f_13754:core_2escm"),(void*)f_13754},
{C_text("f_13760:core_2escm"),(void*)f_13760},
{C_text("f_13768:core_2escm"),(void*)f_13768},
{C_text("f_13774:core_2escm"),(void*)f_13774},
{C_text("f_13777:core_2escm"),(void*)f_13777},
{C_text("f_13780:core_2escm"),(void*)f_13780},
{C_text("f_13783:core_2escm"),(void*)f_13783},
{C_text("f_13788:core_2escm"),(void*)f_13788},
{C_text("f_13792:core_2escm"),(void*)f_13792},
{C_text("f_13795:core_2escm"),(void*)f_13795},
{C_text("f_13798:core_2escm"),(void*)f_13798},
{C_text("f_13804:core_2escm"),(void*)f_13804},
{C_text("f_13808:core_2escm"),(void*)f_13808},
{C_text("f_13812:core_2escm"),(void*)f_13812},
{C_text("f_13823:core_2escm"),(void*)f_13823},
{C_text("f_13826:core_2escm"),(void*)f_13826},
{C_text("f_13829:core_2escm"),(void*)f_13829},
{C_text("f_13865:core_2escm"),(void*)f_13865},
{C_text("f_13869:core_2escm"),(void*)f_13869},
{C_text("f_13890:core_2escm"),(void*)f_13890},
{C_text("f_13894:core_2escm"),(void*)f_13894},
{C_text("f_13915:core_2escm"),(void*)f_13915},
{C_text("f_13919:core_2escm"),(void*)f_13919},
{C_text("f_13923:core_2escm"),(void*)f_13923},
{C_text("f_13936:core_2escm"),(void*)f_13936},
{C_text("f_13940:core_2escm"),(void*)f_13940},
{C_text("f_14023:core_2escm"),(void*)f_14023},
{C_text("f_14033:core_2escm"),(void*)f_14033},
{C_text("f_14065:core_2escm"),(void*)f_14065},
{C_text("f_14071:core_2escm"),(void*)f_14071},
{C_text("f_14079:core_2escm"),(void*)f_14079},
{C_text("f_14087:core_2escm"),(void*)f_14087},
{C_text("f_14097:core_2escm"),(void*)f_14097},
{C_text("f_14110:core_2escm"),(void*)f_14110},
{C_text("f_14120:core_2escm"),(void*)f_14120},
{C_text("f_14147:core_2escm"),(void*)f_14147},
{C_text("f_14158:core_2escm"),(void*)f_14158},
{C_text("f_14168:core_2escm"),(void*)f_14168},
{C_text("f_14178:core_2escm"),(void*)f_14178},
{C_text("f_14191:core_2escm"),(void*)f_14191},
{C_text("f_14201:core_2escm"),(void*)f_14201},
{C_text("f_14227:core_2escm"),(void*)f_14227},
{C_text("f_14231:core_2escm"),(void*)f_14231},
{C_text("f_14269:core_2escm"),(void*)f_14269},
{C_text("f_14277:core_2escm"),(void*)f_14277},
{C_text("f_14287:core_2escm"),(void*)f_14287},
{C_text("f_14314:core_2escm"),(void*)f_14314},
{C_text("f_14322:core_2escm"),(void*)f_14322},
{C_text("f_14332:core_2escm"),(void*)f_14332},
{C_text("f_14354:core_2escm"),(void*)f_14354},
{C_text("f_14357:core_2escm"),(void*)f_14357},
{C_text("f_14374:core_2escm"),(void*)f_14374},
{C_text("f_14378:core_2escm"),(void*)f_14378},
{C_text("f_14402:core_2escm"),(void*)f_14402},
{C_text("f_14406:core_2escm"),(void*)f_14406},
{C_text("f_14434:core_2escm"),(void*)f_14434},
{C_text("f_14442:core_2escm"),(void*)f_14442},
{C_text("f_14452:core_2escm"),(void*)f_14452},
{C_text("f_14487:core_2escm"),(void*)f_14487},
{C_text("f_14491:core_2escm"),(void*)f_14491},
{C_text("f_14495:core_2escm"),(void*)f_14495},
{C_text("f_14523:core_2escm"),(void*)f_14523},
{C_text("f_14531:core_2escm"),(void*)f_14531},
{C_text("f_14541:core_2escm"),(void*)f_14541},
{C_text("f_14562:core_2escm"),(void*)f_14562},
{C_text("f_14565:core_2escm"),(void*)f_14565},
{C_text("f_14614:core_2escm"),(void*)f_14614},
{C_text("f_14629:core_2escm"),(void*)f_14629},
{C_text("f_14639:core_2escm"),(void*)f_14639},
{C_text("f_14662:core_2escm"),(void*)f_14662},
{C_text("f_14670:core_2escm"),(void*)f_14670},
{C_text("f_14680:core_2escm"),(void*)f_14680},
{C_text("f_14721:core_2escm"),(void*)f_14721},
{C_text("f_14729:core_2escm"),(void*)f_14729},
{C_text("f_14739:core_2escm"),(void*)f_14739},
{C_text("f_14763:core_2escm"),(void*)f_14763},
{C_text("f_14785:core_2escm"),(void*)f_14785},
{C_text("f_14807:core_2escm"),(void*)f_14807},
{C_text("f_14813:core_2escm"),(void*)f_14813},
{C_text("f_14821:core_2escm"),(void*)f_14821},
{C_text("f_14829:core_2escm"),(void*)f_14829},
{C_text("f_14839:core_2escm"),(void*)f_14839},
{C_text("f_14865:core_2escm"),(void*)f_14865},
{C_text("f_14884:core_2escm"),(void*)f_14884},
{C_text("f_14888:core_2escm"),(void*)f_14888},
{C_text("f_14895:core_2escm"),(void*)f_14895},
{C_text("f_14941:core_2escm"),(void*)f_14941},
{C_text("f_14947:core_2escm"),(void*)f_14947},
{C_text("f_14949:core_2escm"),(void*)f_14949},
{C_text("f_14974:core_2escm"),(void*)f_14974},
{C_text("f_14999:core_2escm"),(void*)f_14999},
{C_text("f_15002:core_2escm"),(void*)f_15002},
{C_text("f_15051:core_2escm"),(void*)f_15051},
{C_text("f_15059:core_2escm"),(void*)f_15059},
{C_text("f_15069:core_2escm"),(void*)f_15069},
{C_text("f_15105:core_2escm"),(void*)f_15105},
{C_text("f_15113:core_2escm"),(void*)f_15113},
{C_text("f_15123:core_2escm"),(void*)f_15123},
{C_text("f_15160:core_2escm"),(void*)f_15160},
{C_text("f_15168:core_2escm"),(void*)f_15168},
{C_text("f_15178:core_2escm"),(void*)f_15178},
{C_text("f_15200:core_2escm"),(void*)f_15200},
{C_text("f_15207:core_2escm"),(void*)f_15207},
{C_text("f_15214:core_2escm"),(void*)f_15214},
{C_text("f_15217:core_2escm"),(void*)f_15217},
{C_text("f_15220:core_2escm"),(void*)f_15220},
{C_text("f_15226:core_2escm"),(void*)f_15226},
{C_text("f_15236:core_2escm"),(void*)f_15236},
{C_text("f_15242:core_2escm"),(void*)f_15242},
{C_text("f_15252:core_2escm"),(void*)f_15252},
{C_text("f_15255:core_2escm"),(void*)f_15255},
{C_text("f_15258:core_2escm"),(void*)f_15258},
{C_text("f_15261:core_2escm"),(void*)f_15261},
{C_text("f_15274:core_2escm"),(void*)f_15274},
{C_text("f_15290:core_2escm"),(void*)f_15290},
{C_text("f_15322:core_2escm"),(void*)f_15322},
{C_text("f_15364:core_2escm"),(void*)f_15364},
{C_text("f_15374:core_2escm"),(void*)f_15374},
{C_text("f_15392:core_2escm"),(void*)f_15392},
{C_text("f_15399:core_2escm"),(void*)f_15399},
{C_text("f_15402:core_2escm"),(void*)f_15402},
{C_text("f_15405:core_2escm"),(void*)f_15405},
{C_text("f_15411:core_2escm"),(void*)f_15411},
{C_text("f_15421:core_2escm"),(void*)f_15421},
{C_text("f_15427:core_2escm"),(void*)f_15427},
{C_text("f_15486:core_2escm"),(void*)f_15486},
{C_text("f_15496:core_2escm"),(void*)f_15496},
{C_text("f_15630:core_2escm"),(void*)f_15630},
{C_text("f_15653:core_2escm"),(void*)f_15653},
{C_text("f_15662:core_2escm"),(void*)f_15662},
{C_text("f_15671:core_2escm"),(void*)f_15671},
{C_text("f_15680:core_2escm"),(void*)f_15680},
{C_text("f_15689:core_2escm"),(void*)f_15689},
{C_text("f_15698:core_2escm"),(void*)f_15698},
{C_text("f_15707:core_2escm"),(void*)f_15707},
{C_text("f_15716:core_2escm"),(void*)f_15716},
{C_text("f_15725:core_2escm"),(void*)f_15725},
{C_text("f_15734:core_2escm"),(void*)f_15734},
{C_text("f_15763:core_2escm"),(void*)f_15763},
{C_text("f_15767:core_2escm"),(void*)f_15767},
{C_text("f_15812:core_2escm"),(void*)f_15812},
{C_text("f_15815:core_2escm"),(void*)f_15815},
{C_text("f_15818:core_2escm"),(void*)f_15818},
{C_text("f_15821:core_2escm"),(void*)f_15821},
{C_text("f_15824:core_2escm"),(void*)f_15824},
{C_text("f_15827:core_2escm"),(void*)f_15827},
{C_text("f_15830:core_2escm"),(void*)f_15830},
{C_text("f_15840:core_2escm"),(void*)f_15840},
{C_text("f_15852:core_2escm"),(void*)f_15852},
{C_text("f_15861:core_2escm"),(void*)f_15861},
{C_text("f_15872:core_2escm"),(void*)f_15872},
{C_text("f_15884:core_2escm"),(void*)f_15884},
{C_text("f_15892:core_2escm"),(void*)f_15892},
{C_text("f_15899:core_2escm"),(void*)f_15899},
{C_text("f_15902:core_2escm"),(void*)f_15902},
{C_text("f_15913:core_2escm"),(void*)f_15913},
{C_text("f_15917:core_2escm"),(void*)f_15917},
{C_text("f_15921:core_2escm"),(void*)f_15921},
{C_text("f_15947:core_2escm"),(void*)f_15947},
{C_text("f_15976:core_2escm"),(void*)f_15976},
{C_text("f_16017:core_2escm"),(void*)f_16017},
{C_text("f_16028:core_2escm"),(void*)f_16028},
{C_text("f_16053:core_2escm"),(void*)f_16053},
{C_text("f_16072:core_2escm"),(void*)f_16072},
{C_text("f_16080:core_2escm"),(void*)f_16080},
{C_text("f_16105:core_2escm"),(void*)f_16105},
{C_text("f_16114:core_2escm"),(void*)f_16114},
{C_text("f_16121:core_2escm"),(void*)f_16121},
{C_text("f_16140:core_2escm"),(void*)f_16140},
{C_text("f_16151:core_2escm"),(void*)f_16151},
{C_text("f_16161:core_2escm"),(void*)f_16161},
{C_text("f_16175:core_2escm"),(void*)f_16175},
{C_text("f_16181:core_2escm"),(void*)f_16181},
{C_text("f_16187:core_2escm"),(void*)f_16187},
{C_text("f_16192:core_2escm"),(void*)f_16192},
{C_text("f_16226:core_2escm"),(void*)f_16226},
{C_text("f_16268:core_2escm"),(void*)f_16268},
{C_text("f_16274:core_2escm"),(void*)f_16274},
{C_text("f_16277:core_2escm"),(void*)f_16277},
{C_text("f_16291:core_2escm"),(void*)f_16291},
{C_text("f_16297:core_2escm"),(void*)f_16297},
{C_text("f_16303:core_2escm"),(void*)f_16303},
{C_text("f_16308:core_2escm"),(void*)f_16308},
{C_text("f_16342:core_2escm"),(void*)f_16342},
{C_text("f_16420:core_2escm"),(void*)f_16420},
{C_text("f_16426:core_2escm"),(void*)f_16426},
{C_text("f_16436:core_2escm"),(void*)f_16436},
{C_text("f_16445:core_2escm"),(void*)f_16445},
{C_text("f_16462:core_2escm"),(void*)f_16462},
{C_text("f_16468:core_2escm"),(void*)f_16468},
{C_text("f_16499:core_2escm"),(void*)f_16499},
{C_text("f_16502:core_2escm"),(void*)f_16502},
{C_text("f_16506:core_2escm"),(void*)f_16506},
{C_text("f_16513:core_2escm"),(void*)f_16513},
{C_text("f_16525:core_2escm"),(void*)f_16525},
{C_text("f_16531:core_2escm"),(void*)f_16531},
{C_text("f_16547:core_2escm"),(void*)f_16547},
{C_text("f_16553:core_2escm"),(void*)f_16553},
{C_text("f_16571:core_2escm"),(void*)f_16571},
{C_text("f_16575:core_2escm"),(void*)f_16575},
{C_text("f_16577:core_2escm"),(void*)f_16577},
{C_text("f_16581:core_2escm"),(void*)f_16581},
{C_text("f_16584:core_2escm"),(void*)f_16584},
{C_text("f_16587:core_2escm"),(void*)f_16587},
{C_text("f_16590:core_2escm"),(void*)f_16590},
{C_text("f_16599:core_2escm"),(void*)f_16599},
{C_text("f_16611:core_2escm"),(void*)f_16611},
{C_text("f_16614:core_2escm"),(void*)f_16614},
{C_text("f_16615:core_2escm"),(void*)f_16615},
{C_text("f_16631:core_2escm"),(void*)f_16631},
{C_text("f_16646:core_2escm"),(void*)f_16646},
{C_text("f_16650:core_2escm"),(void*)f_16650},
{C_text("f_16656:core_2escm"),(void*)f_16656},
{C_text("f_16668:core_2escm"),(void*)f_16668},
{C_text("f_16672:core_2escm"),(void*)f_16672},
{C_text("f_16692:core_2escm"),(void*)f_16692},
{C_text("f_16696:core_2escm"),(void*)f_16696},
{C_text("f_16705:core_2escm"),(void*)f_16705},
{C_text("f_16720:core_2escm"),(void*)f_16720},
{C_text("f_16743:core_2escm"),(void*)f_16743},
{C_text("f_16750:core_2escm"),(void*)f_16750},
{C_text("f_16774:core_2escm"),(void*)f_16774},
{C_text("f_16807:core_2escm"),(void*)f_16807},
{C_text("f_16820:core_2escm"),(void*)f_16820},
{C_text("f_16835:core_2escm"),(void*)f_16835},
{C_text("f_16851:core_2escm"),(void*)f_16851},
{C_text("f_16855:core_2escm"),(void*)f_16855},
{C_text("f_16859:core_2escm"),(void*)f_16859},
{C_text("f_16891:core_2escm"),(void*)f_16891},
{C_text("f_16897:core_2escm"),(void*)f_16897},
{C_text("f_16908:core_2escm"),(void*)f_16908},
{C_text("f_16912:core_2escm"),(void*)f_16912},
{C_text("f_16921:core_2escm"),(void*)f_16921},
{C_text("f_16961:core_2escm"),(void*)f_16961},
{C_text("f_16969:core_2escm"),(void*)f_16969},
{C_text("f_16974:core_2escm"),(void*)f_16974},
{C_text("f_16980:core_2escm"),(void*)f_16980},
{C_text("f_17123:core_2escm"),(void*)f_17123},
{C_text("f_17127:core_2escm"),(void*)f_17127},
{C_text("f_17130:core_2escm"),(void*)f_17130},
{C_text("f_17145:core_2escm"),(void*)f_17145},
{C_text("f_17149:core_2escm"),(void*)f_17149},
{C_text("f_17151:core_2escm"),(void*)f_17151},
{C_text("f_17157:core_2escm"),(void*)f_17157},
{C_text("f_17165:core_2escm"),(void*)f_17165},
{C_text("f_17169:core_2escm"),(void*)f_17169},
{C_text("f_17181:core_2escm"),(void*)f_17181},
{C_text("f_17185:core_2escm"),(void*)f_17185},
{C_text("f_17194:core_2escm"),(void*)f_17194},
{C_text("f_17204:core_2escm"),(void*)f_17204},
{C_text("f_17207:core_2escm"),(void*)f_17207},
{C_text("f_17222:core_2escm"),(void*)f_17222},
{C_text("f_17226:core_2escm"),(void*)f_17226},
{C_text("f_17238:core_2escm"),(void*)f_17238},
{C_text("f_17250:core_2escm"),(void*)f_17250},
{C_text("f_17254:core_2escm"),(void*)f_17254},
{C_text("f_17263:core_2escm"),(void*)f_17263},
{C_text("f_17269:core_2escm"),(void*)f_17269},
{C_text("f_17275:core_2escm"),(void*)f_17275},
{C_text("f_17283:core_2escm"),(void*)f_17283},
{C_text("f_17285:core_2escm"),(void*)f_17285},
{C_text("f_17291:core_2escm"),(void*)f_17291},
{C_text("f_17305:core_2escm"),(void*)f_17305},
{C_text("f_17311:core_2escm"),(void*)f_17311},
{C_text("f_17325:core_2escm"),(void*)f_17325},
{C_text("f_17332:core_2escm"),(void*)f_17332},
{C_text("f_17339:core_2escm"),(void*)f_17339},
{C_text("f_17354:core_2escm"),(void*)f_17354},
{C_text("f_17369:core_2escm"),(void*)f_17369},
{C_text("f_17381:core_2escm"),(void*)f_17381},
{C_text("f_17387:core_2escm"),(void*)f_17387},
{C_text("f_17391:core_2escm"),(void*)f_17391},
{C_text("f_17408:core_2escm"),(void*)f_17408},
{C_text("f_17413:core_2escm"),(void*)f_17413},
{C_text("f_17420:core_2escm"),(void*)f_17420},
{C_text("f_17422:core_2escm"),(void*)f_17422},
{C_text("f_17429:core_2escm"),(void*)f_17429},
{C_text("f_17433:core_2escm"),(void*)f_17433},
{C_text("f_17436:core_2escm"),(void*)f_17436},
{C_text("f_17439:core_2escm"),(void*)f_17439},
{C_text("f_17451:core_2escm"),(void*)f_17451},
{C_text("f_17466:core_2escm"),(void*)f_17466},
{C_text("f_17497:core_2escm"),(void*)f_17497},
{C_text("f_17527:core_2escm"),(void*)f_17527},
{C_text("f_17530:core_2escm"),(void*)f_17530},
{C_text("f_17548:core_2escm"),(void*)f_17548},
{C_text("f_17551:core_2escm"),(void*)f_17551},
{C_text("f_17561:core_2escm"),(void*)f_17561},
{C_text("f_17568:core_2escm"),(void*)f_17568},
{C_text("f_17596:core_2escm"),(void*)f_17596},
{C_text("f_17606:core_2escm"),(void*)f_17606},
{C_text("f_17610:core_2escm"),(void*)f_17610},
{C_text("f_17614:core_2escm"),(void*)f_17614},
{C_text("f_17618:core_2escm"),(void*)f_17618},
{C_text("f_17622:core_2escm"),(void*)f_17622},
{C_text("f_17626:core_2escm"),(void*)f_17626},
{C_text("f_17638:core_2escm"),(void*)f_17638},
{C_text("f_17643:core_2escm"),(void*)f_17643},
{C_text("f_17661:core_2escm"),(void*)f_17661},
{C_text("f_17670:core_2escm"),(void*)f_17670},
{C_text("f_17673:core_2escm"),(void*)f_17673},
{C_text("f_17676:core_2escm"),(void*)f_17676},
{C_text("f_17701:core_2escm"),(void*)f_17701},
{C_text("f_17703:core_2escm"),(void*)f_17703},
{C_text("f_17713:core_2escm"),(void*)f_17713},
{C_text("f_17717:core_2escm"),(void*)f_17717},
{C_text("f_17726:core_2escm"),(void*)f_17726},
{C_text("f_17728:core_2escm"),(void*)f_17728},
{C_text("f_17738:core_2escm"),(void*)f_17738},
{C_text("f_17770:core_2escm"),(void*)f_17770},
{C_text("f_17777:core_2escm"),(void*)f_17777},
{C_text("f_17778:core_2escm"),(void*)f_17778},
{C_text("f_17782:core_2escm"),(void*)f_17782},
{C_text("f_17791:core_2escm"),(void*)f_17791},
{C_text("f_17794:core_2escm"),(void*)f_17794},
{C_text("f_17797:core_2escm"),(void*)f_17797},
{C_text("f_17800:core_2escm"),(void*)f_17800},
{C_text("f_17803:core_2escm"),(void*)f_17803},
{C_text("f_17806:core_2escm"),(void*)f_17806},
{C_text("f_17822:core_2escm"),(void*)f_17822},
{C_text("f_17830:core_2escm"),(void*)f_17830},
{C_text("f_17848:core_2escm"),(void*)f_17848},
{C_text("f_17854:core_2escm"),(void*)f_17854},
{C_text("f_17863:core_2escm"),(void*)f_17863},
{C_text("f_17873:core_2escm"),(void*)f_17873},
{C_text("f_17887:core_2escm"),(void*)f_17887},
{C_text("f_17908:core_2escm"),(void*)f_17908},
{C_text("f_17911:core_2escm"),(void*)f_17911},
{C_text("f_17914:core_2escm"),(void*)f_17914},
{C_text("f_17917:core_2escm"),(void*)f_17917},
{C_text("f_17920:core_2escm"),(void*)f_17920},
{C_text("f_17923:core_2escm"),(void*)f_17923},
{C_text("f_17959:core_2escm"),(void*)f_17959},
{C_text("f_17965:core_2escm"),(void*)f_17965},
{C_text("f_18005:core_2escm"),(void*)f_18005},
{C_text("f_18011:core_2escm"),(void*)f_18011},
{C_text("f_18100:core_2escm"),(void*)f_18100},
{C_text("f_18102:core_2escm"),(void*)f_18102},
{C_text("f_18114:core_2escm"),(void*)f_18114},
{C_text("f_18124:core_2escm"),(void*)f_18124},
{C_text("f_18137:core_2escm"),(void*)f_18137},
{C_text("f_18141:core_2escm"),(void*)f_18141},
{C_text("f_18150:core_2escm"),(void*)f_18150},
{C_text("f_18154:core_2escm"),(void*)f_18154},
{C_text("f_18163:core_2escm"),(void*)f_18163},
{C_text("f_18170:core_2escm"),(void*)f_18170},
{C_text("f_18176:core_2escm"),(void*)f_18176},
{C_text("f_18188:core_2escm"),(void*)f_18188},
{C_text("f_18212:core_2escm"),(void*)f_18212},
{C_text("f_18216:core_2escm"),(void*)f_18216},
{C_text("f_18235:core_2escm"),(void*)f_18235},
{C_text("f_18239:core_2escm"),(void*)f_18239},
{C_text("f_18242:core_2escm"),(void*)f_18242},
{C_text("f_18245:core_2escm"),(void*)f_18245},
{C_text("f_18249:core_2escm"),(void*)f_18249},
{C_text("f_18253:core_2escm"),(void*)f_18253},
{C_text("f_18256:core_2escm"),(void*)f_18256},
{C_text("f_18273:core_2escm"),(void*)f_18273},
{C_text("f_18276:core_2escm"),(void*)f_18276},
{C_text("f_18279:core_2escm"),(void*)f_18279},
{C_text("f_18318:core_2escm"),(void*)f_18318},
{C_text("f_18321:core_2escm"),(void*)f_18321},
{C_text("f_18327:core_2escm"),(void*)f_18327},
{C_text("f_18354:core_2escm"),(void*)f_18354},
{C_text("f_18361:core_2escm"),(void*)f_18361},
{C_text("f_18367:core_2escm"),(void*)f_18367},
{C_text("f_18377:core_2escm"),(void*)f_18377},
{C_text("f_18380:core_2escm"),(void*)f_18380},
{C_text("f_18387:core_2escm"),(void*)f_18387},
{C_text("f_18391:core_2escm"),(void*)f_18391},
{C_text("f_18395:core_2escm"),(void*)f_18395},
{C_text("f_18409:core_2escm"),(void*)f_18409},
{C_text("f_18413:core_2escm"),(void*)f_18413},
{C_text("f_18417:core_2escm"),(void*)f_18417},
{C_text("f_18420:core_2escm"),(void*)f_18420},
{C_text("f_18440:core_2escm"),(void*)f_18440},
{C_text("f_18444:core_2escm"),(void*)f_18444},
{C_text("f_18447:core_2escm"),(void*)f_18447},
{C_text("f_18467:core_2escm"),(void*)f_18467},
{C_text("f_18471:core_2escm"),(void*)f_18471},
{C_text("f_18473:core_2escm"),(void*)f_18473},
{C_text("f_18494:core_2escm"),(void*)f_18494},
{C_text("f_18498:core_2escm"),(void*)f_18498},
{C_text("f_18501:core_2escm"),(void*)f_18501},
{C_text("f_18505:core_2escm"),(void*)f_18505},
{C_text("f_18511:core_2escm"),(void*)f_18511},
{C_text("f_18517:core_2escm"),(void*)f_18517},
{C_text("f_18660:core_2escm"),(void*)f_18660},
{C_text("f_18663:core_2escm"),(void*)f_18663},
{C_text("f_18666:core_2escm"),(void*)f_18666},
{C_text("f_18669:core_2escm"),(void*)f_18669},
{C_text("f_18672:core_2escm"),(void*)f_18672},
{C_text("f_18675:core_2escm"),(void*)f_18675},
{C_text("f_18678:core_2escm"),(void*)f_18678},
{C_text("f_18681:core_2escm"),(void*)f_18681},
{C_text("f_18687:core_2escm"),(void*)f_18687},
{C_text("f_18690:core_2escm"),(void*)f_18690},
{C_text("f_18708:core_2escm"),(void*)f_18708},
{C_text("f_18711:core_2escm"),(void*)f_18711},
{C_text("f_18729:core_2escm"),(void*)f_18729},
{C_text("f_18742:core_2escm"),(void*)f_18742},
{C_text("f_18766:core_2escm"),(void*)f_18766},
{C_text("f_18770:core_2escm"),(void*)f_18770},
{C_text("f_18774:core_2escm"),(void*)f_18774},
{C_text("f_18795:core_2escm"),(void*)f_18795},
{C_text("f_18801:core_2escm"),(void*)f_18801},
{C_text("f_18812:core_2escm"),(void*)f_18812},
{C_text("f_18830:core_2escm"),(void*)f_18830},
{C_text("f_18842:core_2escm"),(void*)f_18842},
{C_text("f_18864:core_2escm"),(void*)f_18864},
{C_text("f_18868:core_2escm"),(void*)f_18868},
{C_text("f_18872:core_2escm"),(void*)f_18872},
{C_text("f_18882:core_2escm"),(void*)f_18882},
{C_text("f_18886:core_2escm"),(void*)f_18886},
{C_text("f_18890:core_2escm"),(void*)f_18890},
{C_text("f_18911:core_2escm"),(void*)f_18911},
{C_text("f_18935:core_2escm"),(void*)f_18935},
{C_text("f_18945:core_2escm"),(void*)f_18945},
{C_text("f_18949:core_2escm"),(void*)f_18949},
{C_text("f_18956:core_2escm"),(void*)f_18956},
{C_text("f_18960:core_2escm"),(void*)f_18960},
{C_text("f_18969:core_2escm"),(void*)f_18969},
{C_text("f_18984:core_2escm"),(void*)f_18984},
{C_text("f_18988:core_2escm"),(void*)f_18988},
{C_text("f_18994:core_2escm"),(void*)f_18994},
{C_text("f_19007:core_2escm"),(void*)f_19007},
{C_text("f_19021:core_2escm"),(void*)f_19021},
{C_text("f_19022:core_2escm"),(void*)f_19022},
{C_text("f_19029:core_2escm"),(void*)f_19029},
{C_text("f_19032:core_2escm"),(void*)f_19032},
{C_text("f_19043:core_2escm"),(void*)f_19043},
{C_text("f_19047:core_2escm"),(void*)f_19047},
{C_text("f_19055:core_2escm"),(void*)f_19055},
{C_text("f_19065:core_2escm"),(void*)f_19065},
{C_text("f_19086:core_2escm"),(void*)f_19086},
{C_text("f_19089:core_2escm"),(void*)f_19089},
{C_text("f_19098:core_2escm"),(void*)f_19098},
{C_text("f_19111:core_2escm"),(void*)f_19111},
{C_text("f_19118:core_2escm"),(void*)f_19118},
{C_text("f_19121:core_2escm"),(void*)f_19121},
{C_text("f_19127:core_2escm"),(void*)f_19127},
{C_text("f_19159:core_2escm"),(void*)f_19159},
{C_text("f_19167:core_2escm"),(void*)f_19167},
{C_text("f_19169:core_2escm"),(void*)f_19169},
{C_text("f_19175:core_2escm"),(void*)f_19175},
{C_text("f_19187:core_2escm"),(void*)f_19187},
{C_text("f_19193:core_2escm"),(void*)f_19193},
{C_text("f_19204:core_2escm"),(void*)f_19204},
{C_text("f_19210:core_2escm"),(void*)f_19210},
{C_text("f_19214:core_2escm"),(void*)f_19214},
{C_text("f_19223:core_2escm"),(void*)f_19223},
{C_text("f_19231:core_2escm"),(void*)f_19231},
{C_text("f_19248:core_2escm"),(void*)f_19248},
{C_text("f_19255:core_2escm"),(void*)f_19255},
{C_text("f_19258:core_2escm"),(void*)f_19258},
{C_text("f_19262:core_2escm"),(void*)f_19262},
{C_text("f_19269:core_2escm"),(void*)f_19269},
{C_text("f_19275:core_2escm"),(void*)f_19275},
{C_text("f_19281:core_2escm"),(void*)f_19281},
{C_text("f_19299:core_2escm"),(void*)f_19299},
{C_text("f_19306:core_2escm"),(void*)f_19306},
{C_text("f_19312:core_2escm"),(void*)f_19312},
{C_text("f_19315:core_2escm"),(void*)f_19315},
{C_text("f_19318:core_2escm"),(void*)f_19318},
{C_text("f_19321:core_2escm"),(void*)f_19321},
{C_text("f_19355:core_2escm"),(void*)f_19355},
{C_text("f_19359:core_2escm"),(void*)f_19359},
{C_text("f_19363:core_2escm"),(void*)f_19363},
{C_text("f_19367:core_2escm"),(void*)f_19367},
{C_text("f_19373:core_2escm"),(void*)f_19373},
{C_text("f_19377:core_2escm"),(void*)f_19377},
{C_text("f_19395:core_2escm"),(void*)f_19395},
{C_text("f_19399:core_2escm"),(void*)f_19399},
{C_text("f_19407:core_2escm"),(void*)f_19407},
{C_text("f_19417:core_2escm"),(void*)f_19417},
{C_text("f_19442:core_2escm"),(void*)f_19442},
{C_text("f_19469:core_2escm"),(void*)f_19469},
{C_text("f_19472:core_2escm"),(void*)f_19472},
{C_text("f_19478:core_2escm"),(void*)f_19478},
{C_text("f_19483:core_2escm"),(void*)f_19483},
{C_text("f_19488:core_2escm"),(void*)f_19488},
{C_text("f_19495:core_2escm"),(void*)f_19495},
{C_text("f_19497:core_2escm"),(void*)f_19497},
{C_text("f_19501:core_2escm"),(void*)f_19501},
{C_text("f_19504:core_2escm"),(void*)f_19504},
{C_text("f_19507:core_2escm"),(void*)f_19507},
{C_text("f_19537:core_2escm"),(void*)f_19537},
{C_text("f_19546:core_2escm"),(void*)f_19546},
{C_text("f_19549:core_2escm"),(void*)f_19549},
{C_text("f_19559:core_2escm"),(void*)f_19559},
{C_text("f_19563:core_2escm"),(void*)f_19563},
{C_text("f_19597:core_2escm"),(void*)f_19597},
{C_text("f_19631:core_2escm"),(void*)f_19631},
{C_text("f_19639:core_2escm"),(void*)f_19639},
{C_text("f_19649:core_2escm"),(void*)f_19649},
{C_text("f_19651:core_2escm"),(void*)f_19651},
{C_text("f_19676:core_2escm"),(void*)f_19676},
{C_text("f_19707:core_2escm"),(void*)f_19707},
{C_text("f_19713:core_2escm"),(void*)f_19713},
{C_text("f_19716:core_2escm"),(void*)f_19716},
{C_text("f_19725:core_2escm"),(void*)f_19725},
{C_text("f_19728:core_2escm"),(void*)f_19728},
{C_text("f_19731:core_2escm"),(void*)f_19731},
{C_text("f_19737:core_2escm"),(void*)f_19737},
{C_text("f_19740:core_2escm"),(void*)f_19740},
{C_text("f_19743:core_2escm"),(void*)f_19743},
{C_text("f_19756:core_2escm"),(void*)f_19756},
{C_text("f_19775:core_2escm"),(void*)f_19775},
{C_text("f_19793:core_2escm"),(void*)f_19793},
{C_text("f_19811:core_2escm"),(void*)f_19811},
{C_text("f_19821:core_2escm"),(void*)f_19821},
{C_text("f_19825:core_2escm"),(void*)f_19825},
{C_text("f_19833:core_2escm"),(void*)f_19833},
{C_text("f_19837:core_2escm"),(void*)f_19837},
{C_text("f_19841:core_2escm"),(void*)f_19841},
{C_text("f_19845:core_2escm"),(void*)f_19845},
{C_text("f_19873:core_2escm"),(void*)f_19873},
{C_text("f_19880:core_2escm"),(void*)f_19880},
{C_text("f_19885:core_2escm"),(void*)f_19885},
{C_text("f_19890:core_2escm"),(void*)f_19890},
{C_text("f_19894:core_2escm"),(void*)f_19894},
{C_text("f_19897:core_2escm"),(void*)f_19897},
{C_text("f_19900:core_2escm"),(void*)f_19900},
{C_text("f_19911:core_2escm"),(void*)f_19911},
{C_text("f_19917:core_2escm"),(void*)f_19917},
{C_text("f_19931:core_2escm"),(void*)f_19931},
{C_text("f_19941:core_2escm"),(void*)f_19941},
{C_text("f_19943:core_2escm"),(void*)f_19943},
{C_text("f_19968:core_2escm"),(void*)f_19968},
{C_text("f_20028:core_2escm"),(void*)f_20028},
{C_text("f_20032:core_2escm"),(void*)f_20032},
{C_text("f_20035:core_2escm"),(void*)f_20035},
{C_text("f_20038:core_2escm"),(void*)f_20038},
{C_text("f_20047:core_2escm"),(void*)f_20047},
{C_text("f_20059:core_2escm"),(void*)f_20059},
{C_text("f_20065:core_2escm"),(void*)f_20065},
{C_text("f_20081:core_2escm"),(void*)f_20081},
{C_text("f_20084:core_2escm"),(void*)f_20084},
{C_text("f_20087:core_2escm"),(void*)f_20087},
{C_text("f_20090:core_2escm"),(void*)f_20090},
{C_text("f_20103:core_2escm"),(void*)f_20103},
{C_text("f_20116:core_2escm"),(void*)f_20116},
{C_text("f_20126:core_2escm"),(void*)f_20126},
{C_text("f_20167:core_2escm"),(void*)f_20167},
{C_text("f_20174:core_2escm"),(void*)f_20174},
{C_text("f_20178:core_2escm"),(void*)f_20178},
{C_text("f_20190:core_2escm"),(void*)f_20190},
{C_text("f_20193:core_2escm"),(void*)f_20193},
{C_text("f_20211:core_2escm"),(void*)f_20211},
{C_text("f_20215:core_2escm"),(void*)f_20215},
{C_text("f_20227:core_2escm"),(void*)f_20227},
{C_text("f_20231:core_2escm"),(void*)f_20231},
{C_text("f_20243:core_2escm"),(void*)f_20243},
{C_text("f_20254:core_2escm"),(void*)f_20254},
{C_text("f_20271:core_2escm"),(void*)f_20271},
{C_text("f_20275:core_2escm"),(void*)f_20275},
{C_text("f_20287:core_2escm"),(void*)f_20287},
{C_text("f_20296:core_2escm"),(void*)f_20296},
{C_text("f_20299:core_2escm"),(void*)f_20299},
{C_text("f_20305:core_2escm"),(void*)f_20305},
{C_text("f_20311:core_2escm"),(void*)f_20311},
{C_text("f_20317:core_2escm"),(void*)f_20317},
{C_text("f_20320:core_2escm"),(void*)f_20320},
{C_text("f_20335:core_2escm"),(void*)f_20335},
{C_text("f_20340:core_2escm"),(void*)f_20340},
{C_text("f_20348:core_2escm"),(void*)f_20348},
{C_text("f_20354:core_2escm"),(void*)f_20354},
{C_text("f_20357:core_2escm"),(void*)f_20357},
{C_text("f_20368:core_2escm"),(void*)f_20368},
{C_text("f_20372:core_2escm"),(void*)f_20372},
{C_text("f_20376:core_2escm"),(void*)f_20376},
{C_text("f_20384:core_2escm"),(void*)f_20384},
{C_text("f_20391:core_2escm"),(void*)f_20391},
{C_text("f_20397:core_2escm"),(void*)f_20397},
{C_text("f_20422:core_2escm"),(void*)f_20422},
{C_text("f_20441:core_2escm"),(void*)f_20441},
{C_text("f_20444:core_2escm"),(void*)f_20444},
{C_text("f_20454:core_2escm"),(void*)f_20454},
{C_text("f_20471:core_2escm"),(void*)f_20471},
{C_text("f_20481:core_2escm"),(void*)f_20481},
{C_text("f_20483:core_2escm"),(void*)f_20483},
{C_text("f_20509:core_2escm"),(void*)f_20509},
{C_text("f_20519:core_2escm"),(void*)f_20519},
{C_text("f_20544:core_2escm"),(void*)f_20544},
{C_text("f_20571:core_2escm"),(void*)f_20571},
{C_text("f_20576:core_2escm"),(void*)f_20576},
{C_text("f_20592:core_2escm"),(void*)f_20592},
{C_text("f_20607:core_2escm"),(void*)f_20607},
{C_text("f_20650:core_2escm"),(void*)f_20650},
{C_text("f_20656:core_2escm"),(void*)f_20656},
{C_text("f_20684:core_2escm"),(void*)f_20684},
{C_text("f_20732:core_2escm"),(void*)f_20732},
{C_text("f_20757:core_2escm"),(void*)f_20757},
{C_text("f_20766:core_2escm"),(void*)f_20766},
{C_text("f_20785:core_2escm"),(void*)f_20785},
{C_text("f_20788:core_2escm"),(void*)f_20788},
{C_text("f_20791:core_2escm"),(void*)f_20791},
{C_text("f_20794:core_2escm"),(void*)f_20794},
{C_text("f_20798:core_2escm"),(void*)f_20798},
{C_text("f_20805:core_2escm"),(void*)f_20805},
{C_text("f_20820:core_2escm"),(void*)f_20820},
{C_text("f_20824:core_2escm"),(void*)f_20824},
{C_text("f_20836:core_2escm"),(void*)f_20836},
{C_text("f_20857:core_2escm"),(void*)f_20857},
{C_text("f_20861:core_2escm"),(void*)f_20861},
{C_text("f_20872:core_2escm"),(void*)f_20872},
{C_text("f_20887:core_2escm"),(void*)f_20887},
{C_text("f_20891:core_2escm"),(void*)f_20891},
{C_text("f_20906:core_2escm"),(void*)f_20906},
{C_text("f_20939:core_2escm"),(void*)f_20939},
{C_text("f_20960:core_2escm"),(void*)f_20960},
{C_text("f_20971:core_2escm"),(void*)f_20971},
{C_text("f_20975:core_2escm"),(void*)f_20975},
{C_text("f_21214:core_2escm"),(void*)f_21214},
{C_text("f_21219:core_2escm"),(void*)f_21219},
{C_text("f_21231:core_2escm"),(void*)f_21231},
{C_text("f_21256:core_2escm"),(void*)f_21256},
{C_text("f_21265:core_2escm"),(void*)f_21265},
{C_text("f_21272:core_2escm"),(void*)f_21272},
{C_text("f_21276:core_2escm"),(void*)f_21276},
{C_text("f_21292:core_2escm"),(void*)f_21292},
{C_text("f_21303:core_2escm"),(void*)f_21303},
{C_text("f_21306:core_2escm"),(void*)f_21306},
{C_text("f_21309:core_2escm"),(void*)f_21309},
{C_text("f_21312:core_2escm"),(void*)f_21312},
{C_text("f_21315:core_2escm"),(void*)f_21315},
{C_text("f_21318:core_2escm"),(void*)f_21318},
{C_text("f_21321:core_2escm"),(void*)f_21321},
{C_text("f_21346:core_2escm"),(void*)f_21346},
{C_text("f_21355:core_2escm"),(void*)f_21355},
{C_text("f_21373:core_2escm"),(void*)f_21373},
{C_text("f_21382:core_2escm"),(void*)f_21382},
{C_text("f_21391:core_2escm"),(void*)f_21391},
{C_text("f_21400:core_2escm"),(void*)f_21400},
{C_text("f_21409:core_2escm"),(void*)f_21409},
{C_text("f_21418:core_2escm"),(void*)f_21418},
{C_text("f_21436:core_2escm"),(void*)f_21436},
{C_text("f_21445:core_2escm"),(void*)f_21445},
{C_text("f_21454:core_2escm"),(void*)f_21454},
{C_text("f_21463:core_2escm"),(void*)f_21463},
{C_text("f_21472:core_2escm"),(void*)f_21472},
{C_text("f_21481:core_2escm"),(void*)f_21481},
{C_text("f_21490:core_2escm"),(void*)f_21490},
{C_text("f_21494:core_2escm"),(void*)f_21494},
{C_text("f_21496:core_2escm"),(void*)f_21496},
{C_text("f_21500:core_2escm"),(void*)f_21500},
{C_text("f_21504:core_2escm"),(void*)f_21504},
{C_text("f_21529:core_2escm"),(void*)f_21529},
{C_text("f_21540:core_2escm"),(void*)f_21540},
{C_text("f_21545:core_2escm"),(void*)f_21545},
{C_text("f_21549:core_2escm"),(void*)f_21549},
{C_text("f_21552:core_2escm"),(void*)f_21552},
{C_text("f_21555:core_2escm"),(void*)f_21555},
{C_text("f_21566:core_2escm"),(void*)f_21566},
{C_text("f_21577:core_2escm"),(void*)f_21577},
{C_text("f_21584:core_2escm"),(void*)f_21584},
{C_text("f_21593:core_2escm"),(void*)f_21593},
{C_text("f_21601:core_2escm"),(void*)f_21601},
{C_text("f_21605:core_2escm"),(void*)f_21605},
{C_text("f_21608:core_2escm"),(void*)f_21608},
{C_text("f_21611:core_2escm"),(void*)f_21611},
{C_text("f_21620:core_2escm"),(void*)f_21620},
{C_text("f_21648:core_2escm"),(void*)f_21648},
{C_text("f_21659:core_2escm"),(void*)f_21659},
{C_text("f_21679:core_2escm"),(void*)f_21679},
{C_text("f_21683:core_2escm"),(void*)f_21683},
{C_text("f_21704:core_2escm"),(void*)f_21704},
{C_text("f_21725:core_2escm"),(void*)f_21725},
{C_text("f_21742:core_2escm"),(void*)f_21742},
{C_text("f_21746:core_2escm"),(void*)f_21746},
{C_text("f_21767:core_2escm"),(void*)f_21767},
{C_text("f_21771:core_2escm"),(void*)f_21771},
{C_text("f_21775:core_2escm"),(void*)f_21775},
{C_text("f_21796:core_2escm"),(void*)f_21796},
{C_text("f_21821:core_2escm"),(void*)f_21821},
{C_text("f_21837:core_2escm"),(void*)f_21837},
{C_text("f_21847:core_2escm"),(void*)f_21847},
{C_text("f_21863:core_2escm"),(void*)f_21863},
{C_text("f_21867:core_2escm"),(void*)f_21867},
{C_text("f_21876:core_2escm"),(void*)f_21876},
{C_text("f_21917:core_2escm"),(void*)f_21917},
{C_text("f_21924:core_2escm"),(void*)f_21924},
{C_text("f_21927:core_2escm"),(void*)f_21927},
{C_text("f_21930:core_2escm"),(void*)f_21930},
{C_text("f_21933:core_2escm"),(void*)f_21933},
{C_text("f_21936:core_2escm"),(void*)f_21936},
{C_text("f_21944:core_2escm"),(void*)f_21944},
{C_text("f_21973:core_2escm"),(void*)f_21973},
{C_text("f_21980:core_2escm"),(void*)f_21980},
{C_text("f_21984:core_2escm"),(void*)f_21984},
{C_text("f_21999:core_2escm"),(void*)f_21999},
{C_text("f_22022:core_2escm"),(void*)f_22022},
{C_text("f_22043:core_2escm"),(void*)f_22043},
{C_text("f_22052:core_2escm"),(void*)f_22052},
{C_text("f_22058:core_2escm"),(void*)f_22058},
{C_text("f_22083:core_2escm"),(void*)f_22083},
{C_text("f_22115:core_2escm"),(void*)f_22115},
{C_text("f_22119:core_2escm"),(void*)f_22119},
{C_text("f_22127:core_2escm"),(void*)f_22127},
{C_text("f_22135:core_2escm"),(void*)f_22135},
{C_text("f_22139:core_2escm"),(void*)f_22139},
{C_text("f_22149:core_2escm"),(void*)f_22149},
{C_text("f_22176:core_2escm"),(void*)f_22176},
{C_text("f_22180:core_2escm"),(void*)f_22180},
{C_text("f_22199:core_2escm"),(void*)f_22199},
{C_text("f_22203:core_2escm"),(void*)f_22203},
{C_text("f_22219:core_2escm"),(void*)f_22219},
{C_text("f_22233:core_2escm"),(void*)f_22233},
{C_text("f_22239:core_2escm"),(void*)f_22239},
{C_text("f_22242:core_2escm"),(void*)f_22242},
{C_text("f_22245:core_2escm"),(void*)f_22245},
{C_text("f_22264:core_2escm"),(void*)f_22264},
{C_text("f_22272:core_2escm"),(void*)f_22272},
{C_text("f_22313:core_2escm"),(void*)f_22313},
{C_text("f_22317:core_2escm"),(void*)f_22317},
{C_text("f_22332:core_2escm"),(void*)f_22332},
{C_text("f_22336:core_2escm"),(void*)f_22336},
{C_text("f_22339:core_2escm"),(void*)f_22339},
{C_text("f_22342:core_2escm"),(void*)f_22342},
{C_text("f_22349:core_2escm"),(void*)f_22349},
{C_text("f_22369:core_2escm"),(void*)f_22369},
{C_text("f_22373:core_2escm"),(void*)f_22373},
{C_text("f_22380:core_2escm"),(void*)f_22380},
{C_text("f_22417:core_2escm"),(void*)f_22417},
{C_text("f_22424:core_2escm"),(void*)f_22424},
{C_text("f_22444:core_2escm"),(void*)f_22444},
{C_text("f_22465:core_2escm"),(void*)f_22465},
{C_text("f_22468:core_2escm"),(void*)f_22468},
{C_text("f_22475:core_2escm"),(void*)f_22475},
{C_text("f_22479:core_2escm"),(void*)f_22479},
{C_text("f_22485:core_2escm"),(void*)f_22485},
{C_text("f_22488:core_2escm"),(void*)f_22488},
{C_text("f_22491:core_2escm"),(void*)f_22491},
{C_text("f_22494:core_2escm"),(void*)f_22494},
{C_text("f_22497:core_2escm"),(void*)f_22497},
{C_text("f_22501:core_2escm"),(void*)f_22501},
{C_text("f_22512:core_2escm"),(void*)f_22512},
{C_text("f_22523:core_2escm"),(void*)f_22523},
{C_text("f_22529:core_2escm"),(void*)f_22529},
{C_text("f_22543:core_2escm"),(void*)f_22543},
{C_text("f_22553:core_2escm"),(void*)f_22553},
{C_text("f_22565:core_2escm"),(void*)f_22565},
{C_text("f_22568:core_2escm"),(void*)f_22568},
{C_text("f_22571:core_2escm"),(void*)f_22571},
{C_text("f_22590:core_2escm"),(void*)f_22590},
{C_text("f_22602:core_2escm"),(void*)f_22602},
{C_text("f_22631:core_2escm"),(void*)f_22631},
{C_text("f_22642:core_2escm"),(void*)f_22642},
{C_text("f_22652:core_2escm"),(void*)f_22652},
{C_text("f_22661:core_2escm"),(void*)f_22661},
{C_text("f_22672:core_2escm"),(void*)f_22672},
{C_text("f_22683:core_2escm"),(void*)f_22683},
{C_text("f_22686:core_2escm"),(void*)f_22686},
{C_text("f_22693:core_2escm"),(void*)f_22693},
{C_text("f_22703:core_2escm"),(void*)f_22703},
{C_text("f_22744:core_2escm"),(void*)f_22744},
{C_text("f_22811:core_2escm"),(void*)f_22811},
{C_text("f_22816:core_2escm"),(void*)f_22816},
{C_text("f_22828:core_2escm"),(void*)f_22828},
{C_text("f_22853:core_2escm"),(void*)f_22853},
{C_text("f_22862:core_2escm"),(void*)f_22862},
{C_text("f_22869:core_2escm"),(void*)f_22869},
{C_text("f_22894:core_2escm"),(void*)f_22894},
{C_text("f_22910:core_2escm"),(void*)f_22910},
{C_text("f_22919:core_2escm"),(void*)f_22919},
{C_text("f_22923:core_2escm"),(void*)f_22923},
{C_text("f_22927:core_2escm"),(void*)f_22927},
{C_text("f_22945:core_2escm"),(void*)f_22945},
{C_text("f_22947:core_2escm"),(void*)f_22947},
{C_text("f_22954:core_2escm"),(void*)f_22954},
{C_text("f_22961:core_2escm"),(void*)f_22961},
{C_text("f_22963:core_2escm"),(void*)f_22963},
{C_text("f_22980:core_2escm"),(void*)f_22980},
{C_text("f_23026:core_2escm"),(void*)f_23026},
{C_text("f_23029:core_2escm"),(void*)f_23029},
{C_text("f_23032:core_2escm"),(void*)f_23032},
{C_text("f_23035:core_2escm"),(void*)f_23035},
{C_text("f_23038:core_2escm"),(void*)f_23038},
{C_text("f_23045:core_2escm"),(void*)f_23045},
{C_text("f_23049:core_2escm"),(void*)f_23049},
{C_text("f_23053:core_2escm"),(void*)f_23053},
{C_text("f_6259:core_2escm"),(void*)f_6259},
{C_text("f_6262:core_2escm"),(void*)f_6262},
{C_text("f_6265:core_2escm"),(void*)f_6265},
{C_text("f_6268:core_2escm"),(void*)f_6268},
{C_text("f_6271:core_2escm"),(void*)f_6271},
{C_text("f_6274:core_2escm"),(void*)f_6274},
{C_text("f_6277:core_2escm"),(void*)f_6277},
{C_text("f_6280:core_2escm"),(void*)f_6280},
{C_text("f_6283:core_2escm"),(void*)f_6283},
{C_text("f_6286:core_2escm"),(void*)f_6286},
{C_text("f_6472:core_2escm"),(void*)f_6472},
{C_text("f_6486:core_2escm"),(void*)f_6486},
{C_text("f_6675:core_2escm"),(void*)f_6675},
{C_text("f_6681:core_2escm"),(void*)f_6681},
{C_text("f_6703:core_2escm"),(void*)f_6703},
{C_text("f_6715:core_2escm"),(void*)f_6715},
{C_text("f_6739:core_2escm"),(void*)f_6739},
{C_text("f_6745:core_2escm"),(void*)f_6745},
{C_text("f_6759:core_2escm"),(void*)f_6759},
{C_text("f_6767:core_2escm"),(void*)f_6767},
{C_text("f_6773:core_2escm"),(void*)f_6773},
{C_text("f_6791:core_2escm"),(void*)f_6791},
{C_text("f_6795:core_2escm"),(void*)f_6795},
{C_text("f_6801:core_2escm"),(void*)f_6801},
{C_text("f_6814:core_2escm"),(void*)f_6814},
{C_text("f_6828:core_2escm"),(void*)f_6828},
{C_text("f_6876:core_2escm"),(void*)f_6876},
{C_text("f_6890:core_2escm"),(void*)f_6890},
{C_text("f_6903:core_2escm"),(void*)f_6903},
{C_text("f_6915:core_2escm"),(void*)f_6915},
{C_text("f_6924:core_2escm"),(void*)f_6924},
{C_text("f_6932:core_2escm"),(void*)f_6932},
{C_text("f_6939:core_2escm"),(void*)f_6939},
{C_text("f_6953:core_2escm"),(void*)f_6953},
{C_text("f_6968:core_2escm"),(void*)f_6968},
{C_text("f_6976:core_2escm"),(void*)f_6976},
{C_text("f_6980:core_2escm"),(void*)f_6980},
{C_text("f_6984:core_2escm"),(void*)f_6984},
{C_text("f_7002:core_2escm"),(void*)f_7002},
{C_text("f_7024:core_2escm"),(void*)f_7024},
{C_text("f_7041:core_2escm"),(void*)f_7041},
{C_text("f_7081:core_2escm"),(void*)f_7081},
{C_text("f_7106:core_2escm"),(void*)f_7106},
{C_text("f_7119:core_2escm"),(void*)f_7119},
{C_text("f_7134:core_2escm"),(void*)f_7134},
{C_text("f_7140:core_2escm"),(void*)f_7140},
{C_text("f_7175:core_2escm"),(void*)f_7175},
{C_text("f_7181:core_2escm"),(void*)f_7181},
{C_text("f_7203:core_2escm"),(void*)f_7203},
{C_text("f_7214:core_2escm"),(void*)f_7214},
{C_text("f_7220:core_2escm"),(void*)f_7220},
{C_text("f_7226:core_2escm"),(void*)f_7226},
{C_text("f_7251:core_2escm"),(void*)f_7251},
{C_text("f_7279:core_2escm"),(void*)f_7279},
{C_text("f_7293:core_2escm"),(void*)f_7293},
{C_text("f_7299:core_2escm"),(void*)f_7299},
{C_text("f_7305:core_2escm"),(void*)f_7305},
{C_text("f_7311:core_2escm"),(void*)f_7311},
{C_text("f_7518:core_2escm"),(void*)f_7518},
{C_text("f_7524:core_2escm"),(void*)f_7524},
{C_text("f_7558:core_2escm"),(void*)f_7558},
{C_text("f_7654:core_2escm"),(void*)f_7654},
{C_text("f_7658:core_2escm"),(void*)f_7658},
{C_text("f_7661:core_2escm"),(void*)f_7661},
{C_text("f_7664:core_2escm"),(void*)f_7664},
{C_text("f_7667:core_2escm"),(void*)f_7667},
{C_text("f_7670:core_2escm"),(void*)f_7670},
{C_text("f_7680:core_2escm"),(void*)f_7680},
{C_text("f_7687:core_2escm"),(void*)f_7687},
{C_text("f_7694:core_2escm"),(void*)f_7694},
{C_text("f_7696:core_2escm"),(void*)f_7696},
{C_text("f_7700:core_2escm"),(void*)f_7700},
{C_text("f_7705:core_2escm"),(void*)f_7705},
{C_text("f_7707:core_2escm"),(void*)f_7707},
{C_text("f_7749:core_2escm"),(void*)f_7749},
{C_text("f_7781:core_2escm"),(void*)f_7781},
{C_text("f_7791:core_2escm"),(void*)f_7791},
{C_text("f_7804:core_2escm"),(void*)f_7804},
{C_text("f_7807:core_2escm"),(void*)f_7807},
{C_text("f_7820:core_2escm"),(void*)f_7820},
{C_text("f_7842:core_2escm"),(void*)f_7842},
{C_text("f_7846:core_2escm"),(void*)f_7846},
{C_text("f_7854:core_2escm"),(void*)f_7854},
{C_text("f_7860:core_2escm"),(void*)f_7860},
{C_text("f_7867:core_2escm"),(void*)f_7867},
{C_text("f_7874:core_2escm"),(void*)f_7874},
{C_text("f_7876:core_2escm"),(void*)f_7876},
{C_text("f_7880:core_2escm"),(void*)f_7880},
{C_text("f_7883:core_2escm"),(void*)f_7883},
{C_text("f_7887:core_2escm"),(void*)f_7887},
{C_text("f_7889:core_2escm"),(void*)f_7889},
{C_text("f_7891:core_2escm"),(void*)f_7891},
{C_text("f_7898:core_2escm"),(void*)f_7898},
{C_text("f_7901:core_2escm"),(void*)f_7901},
{C_text("f_7916:core_2escm"),(void*)f_7916},
{C_text("f_7920:core_2escm"),(void*)f_7920},
{C_text("f_7925:core_2escm"),(void*)f_7925},
{C_text("f_7930:core_2escm"),(void*)f_7930},
{C_text("f_7938:core_2escm"),(void*)f_7938},
{C_text("f_7940:core_2escm"),(void*)f_7940},
{C_text("f_7945:core_2escm"),(void*)f_7945},
{C_text("f_7962:core_2escm"),(void*)f_7962},
{C_text("f_7972:core_2escm"),(void*)f_7972},
{C_text("f_8011:core_2escm"),(void*)f_8011},
{C_text("f_8043:core_2escm"),(void*)f_8043},
{C_text("f_8054:core_2escm"),(void*)f_8054},
{C_text("f_8058:core_2escm"),(void*)f_8058},
{C_text("f_8061:core_2escm"),(void*)f_8061},
{C_text("f_8073:core_2escm"),(void*)f_8073},
{C_text("f_8077:core_2escm"),(void*)f_8077},
{C_text("f_8087:core_2escm"),(void*)f_8087},
{C_text("f_8091:core_2escm"),(void*)f_8091},
{C_text("f_8105:core_2escm"),(void*)f_8105},
{C_text("f_8112:core_2escm"),(void*)f_8112},
{C_text("f_8122:core_2escm"),(void*)f_8122},
{C_text("f_8126:core_2escm"),(void*)f_8126},
{C_text("f_8144:core_2escm"),(void*)f_8144},
{C_text("f_8151:core_2escm"),(void*)f_8151},
{C_text("f_8161:core_2escm"),(void*)f_8161},
{C_text("f_8165:core_2escm"),(void*)f_8165},
{C_text("f_8196:core_2escm"),(void*)f_8196},
{C_text("f_8207:core_2escm"),(void*)f_8207},
{C_text("f_8211:core_2escm"),(void*)f_8211},
{C_text("f_8214:core_2escm"),(void*)f_8214},
{C_text("f_8220:core_2escm"),(void*)f_8220},
{C_text("f_8223:core_2escm"),(void*)f_8223},
{C_text("f_8226:core_2escm"),(void*)f_8226},
{C_text("f_8250:core_2escm"),(void*)f_8250},
{C_text("f_8256:core_2escm"),(void*)f_8256},
{C_text("f_8259:core_2escm"),(void*)f_8259},
{C_text("f_8262:core_2escm"),(void*)f_8262},
{C_text("f_8265:core_2escm"),(void*)f_8265},
{C_text("f_8271:core_2escm"),(void*)f_8271},
{C_text("f_8274:core_2escm"),(void*)f_8274},
{C_text("f_8277:core_2escm"),(void*)f_8277},
{C_text("f_8292:core_2escm"),(void*)f_8292},
{C_text("f_8297:core_2escm"),(void*)f_8297},
{C_text("f_8301:core_2escm"),(void*)f_8301},
{C_text("f_8307:core_2escm"),(void*)f_8307},
{C_text("f_8312:core_2escm"),(void*)f_8312},
{C_text("f_8322:core_2escm"),(void*)f_8322},
{C_text("f_8337:core_2escm"),(void*)f_8337},
{C_text("f_8346:core_2escm"),(void*)f_8346},
{C_text("f_8356:core_2escm"),(void*)f_8356},
{C_text("f_8362:core_2escm"),(void*)f_8362},
{C_text("f_8369:core_2escm"),(void*)f_8369},
{C_text("f_8393:core_2escm"),(void*)f_8393},
{C_text("f_8408:core_2escm"),(void*)f_8408},
{C_text("f_8414:core_2escm"),(void*)f_8414},
{C_text("f_8417:core_2escm"),(void*)f_8417},
{C_text("f_8421:core_2escm"),(void*)f_8421},
{C_text("f_8424:core_2escm"),(void*)f_8424},
{C_text("f_8427:core_2escm"),(void*)f_8427},
{C_text("f_8439:core_2escm"),(void*)f_8439},
{C_text("f_8443:core_2escm"),(void*)f_8443},
{C_text("f_8471:core_2escm"),(void*)f_8471},
{C_text("f_8475:core_2escm"),(void*)f_8475},
{C_text("f_8479:core_2escm"),(void*)f_8479},
{C_text("f_8517:core_2escm"),(void*)f_8517},
{C_text("f_8550:core_2escm"),(void*)f_8550},
{C_text("f_8558:core_2escm"),(void*)f_8558},
{C_text("f_8575:core_2escm"),(void*)f_8575},
{C_text("f_8581:core_2escm"),(void*)f_8581},
{C_text("f_8584:core_2escm"),(void*)f_8584},
{C_text("f_8585:core_2escm"),(void*)f_8585},
{C_text("f_8599:core_2escm"),(void*)f_8599},
{C_text("f_8603:core_2escm"),(void*)f_8603},
{C_text("f_8617:core_2escm"),(void*)f_8617},
{C_text("f_8621:core_2escm"),(void*)f_8621},
{C_text("f_8625:core_2escm"),(void*)f_8625},
{C_text("f_8658:core_2escm"),(void*)f_8658},
{C_text("f_8663:core_2escm"),(void*)f_8663},
{C_text("f_8671:core_2escm"),(void*)f_8671},
{C_text("f_8675:core_2escm"),(void*)f_8675},
{C_text("f_8692:core_2escm"),(void*)f_8692},
{C_text("f_8694:core_2escm"),(void*)f_8694},
{C_text("f_8719:core_2escm"),(void*)f_8719},
{C_text("f_8742:core_2escm"),(void*)f_8742},
{C_text("f_8754:core_2escm"),(void*)f_8754},
{C_text("f_8761:core_2escm"),(void*)f_8761},
{C_text("f_8764:core_2escm"),(void*)f_8764},
{C_text("f_8773:core_2escm"),(void*)f_8773},
{C_text("f_8786:core_2escm"),(void*)f_8786},
{C_text("f_8806:core_2escm"),(void*)f_8806},
{C_text("f_8810:core_2escm"),(void*)f_8810},
{C_text("f_8814:core_2escm"),(void*)f_8814},
{C_text("f_8827:core_2escm"),(void*)f_8827},
{C_text("f_8831:core_2escm"),(void*)f_8831},
{C_text("f_8848:core_2escm"),(void*)f_8848},
{C_text("f_8869:core_2escm"),(void*)f_8869},
{C_text("f_8885:core_2escm"),(void*)f_8885},
{C_text("f_8896:core_2escm"),(void*)f_8896},
{C_text("f_8917:core_2escm"),(void*)f_8917},
{C_text("f_8926:core_2escm"),(void*)f_8926},
{C_text("f_8929:core_2escm"),(void*)f_8929},
{C_text("f_8932:core_2escm"),(void*)f_8932},
{C_text("f_8938:core_2escm"),(void*)f_8938},
{C_text("f_8946:core_2escm"),(void*)f_8946},
{C_text("f_8954:core_2escm"),(void*)f_8954},
{C_text("f_8969:core_2escm"),(void*)f_8969},
{C_text("f_8973:core_2escm"),(void*)f_8973},
{C_text("f_8975:core_2escm"),(void*)f_8975},
{C_text("f_8979:core_2escm"),(void*)f_8979},
{C_text("f_8983:core_2escm"),(void*)f_8983},
{C_text("f_8986:core_2escm"),(void*)f_8986},
{C_text("f_8992:core_2escm"),(void*)f_8992},
{C_text("f_9000:core_2escm"),(void*)f_9000},
{C_text("f_9004:core_2escm"),(void*)f_9004},
{C_text("f_9010:core_2escm"),(void*)f_9010},
{C_text("f_9014:core_2escm"),(void*)f_9014},
{C_text("f_9017:core_2escm"),(void*)f_9017},
{C_text("f_9020:core_2escm"),(void*)f_9020},
{C_text("f_9049:core_2escm"),(void*)f_9049},
{C_text("f_9070:core_2escm"),(void*)f_9070},
{C_text("f_9072:core_2escm"),(void*)f_9072},
{C_text("f_9097:core_2escm"),(void*)f_9097},
{C_text("f_9145:core_2escm"),(void*)f_9145},
{C_text("f_9149:core_2escm"),(void*)f_9149},
{C_text("f_9169:core_2escm"),(void*)f_9169},
{C_text("f_9183:core_2escm"),(void*)f_9183},
{C_text("f_9217:core_2escm"),(void*)f_9217},
{C_text("f_9261:core_2escm"),(void*)f_9261},
{C_text("f_9270:core_2escm"),(void*)f_9270},
{C_text("f_9299:core_2escm"),(void*)f_9299},
{C_text("f_9312:core_2escm"),(void*)f_9312},
{C_text("f_9330:core_2escm"),(void*)f_9330},
{C_text("f_9334:core_2escm"),(void*)f_9334},
{C_text("f_9346:core_2escm"),(void*)f_9346},
{C_text("f_9360:core_2escm"),(void*)f_9360},
{C_text("f_9408:core_2escm"),(void*)f_9408},
{C_text("f_9456:core_2escm"),(void*)f_9456},
{C_text("f_9490:core_2escm"),(void*)f_9490},
{C_text("f_9515:core_2escm"),(void*)f_9515},
{C_text("f_9535:core_2escm"),(void*)f_9535},
{C_text("f_9543:core_2escm"),(void*)f_9543},
{C_text("f_9551:core_2escm"),(void*)f_9551},
{C_text("f_9554:core_2escm"),(void*)f_9554},
{C_text("f_9559:core_2escm"),(void*)f_9559},
{C_text("f_9564:core_2escm"),(void*)f_9564},
{C_text("f_9574:core_2escm"),(void*)f_9574},
{C_text("f_9579:core_2escm"),(void*)f_9579},
{C_text("f_9604:core_2escm"),(void*)f_9604},
{C_text("f_9625:core_2escm"),(void*)f_9625},
{C_text("f_9630:core_2escm"),(void*)f_9630},
{C_text("f_9640:core_2escm"),(void*)f_9640},
{C_text("f_9643:core_2escm"),(void*)f_9643},
{C_text("f_9649:core_2escm"),(void*)f_9649},
{C_text("f_9652:core_2escm"),(void*)f_9652},
{C_text("f_9655:core_2escm"),(void*)f_9655},
{C_text("f_9661:core_2escm"),(void*)f_9661},
{C_text("f_9670:core_2escm"),(void*)f_9670},
{C_text("f_9676:core_2escm"),(void*)f_9676},
{C_text("f_9683:core_2escm"),(void*)f_9683},
{C_text("f_9714:core_2escm"),(void*)f_9714},
{C_text("f_9725:core_2escm"),(void*)f_9725},
{C_text("f_9727:core_2escm"),(void*)f_9727},
{C_text("f_9731:core_2escm"),(void*)f_9731},
{C_text("f_9735:core_2escm"),(void*)f_9735},
{C_text("f_9738:core_2escm"),(void*)f_9738},
{C_text("f_9744:core_2escm"),(void*)f_9744},
{C_text("f_9748:core_2escm"),(void*)f_9748},
{C_text("f_9753:core_2escm"),(void*)f_9753},
{C_text("f_9758:core_2escm"),(void*)f_9758},
{C_text("f_9766:core_2escm"),(void*)f_9766},
{C_text("f_9770:core_2escm"),(void*)f_9770},
{C_text("f_9783:core_2escm"),(void*)f_9783},
{C_text("f_9788:core_2escm"),(void*)f_9788},
{C_text("f_9792:core_2escm"),(void*)f_9792},
{C_text("f_9795:core_2escm"),(void*)f_9795},
{C_text("f_9800:core_2escm"),(void*)f_9800},
{C_text("f_9802:core_2escm"),(void*)f_9802},
{C_text("f_9827:core_2escm"),(void*)f_9827},
{C_text("f_9837:core_2escm"),(void*)f_9837},
{C_text("f_9842:core_2escm"),(void*)f_9842},
{C_text("f_9850:core_2escm"),(void*)f_9850},
{C_text("f_9852:core_2escm"),(void*)f_9852},
{C_text("f_9864:core_2escm"),(void*)f_9864},
{C_text("f_9869:core_2escm"),(void*)f_9869},
{C_text("f_9873:core_2escm"),(void*)f_9873},
{C_text("f_9877:core_2escm"),(void*)f_9877},
{C_text("f_9880:core_2escm"),(void*)f_9880},
{C_text("f_9886:core_2escm"),(void*)f_9886},
{C_text("f_9890:core_2escm"),(void*)f_9890},
{C_text("f_9900:core_2escm"),(void*)f_9900},
{C_text("f_9906:core_2escm"),(void*)f_9906},
{C_text("f_9910:core_2escm"),(void*)f_9910},
{C_text("f_9913:core_2escm"),(void*)f_9913},
{C_text("f_9931:core_2escm"),(void*)f_9931},
{C_text("f_9935:core_2escm"),(void*)f_9935},
{C_text("f_9939:core_2escm"),(void*)f_9939},
{C_text("f_9954:core_2escm"),(void*)f_9954},
{C_text("f_9958:core_2escm"),(void*)f_9958},
{C_text("f_9960:core_2escm"),(void*)f_9960},
{C_text("toplevel:core_2escm"),(void*)C_compiler_toplevel},
{C_text("va26382:core_2escm"),(void*)va26382},
{NULL,NULL}};
#endif

static C_PTABLE_ENTRY *create_ptable(void){
#ifdef C_ENABLE_PTABLES
return ptable;
#else
return NULL;
#endif
}

/*
o|hiding unexported module binding: chicken.compiler.core#d 
o|hiding unexported module binding: chicken.compiler.core#d 
o|hiding unexported module binding: chicken.compiler.core#partition 
o|hiding unexported module binding: chicken.compiler.core#span 
o|hiding unexported module binding: chicken.compiler.core#take 
o|hiding unexported module binding: chicken.compiler.core#drop 
o|hiding unexported module binding: chicken.compiler.core#split-at 
o|hiding unexported module binding: chicken.compiler.core#append-map 
o|hiding unexported module binding: chicken.compiler.core#every 
o|hiding unexported module binding: chicken.compiler.core#any 
o|hiding unexported module binding: chicken.compiler.core#cons* 
o|hiding unexported module binding: chicken.compiler.core#concatenate 
o|hiding unexported module binding: chicken.compiler.core#delete 
o|hiding unexported module binding: chicken.compiler.core#first 
o|hiding unexported module binding: chicken.compiler.core#second 
o|hiding unexported module binding: chicken.compiler.core#third 
o|hiding unexported module binding: chicken.compiler.core#fourth 
o|hiding unexported module binding: chicken.compiler.core#fifth 
o|hiding unexported module binding: chicken.compiler.core#delete-duplicates 
o|hiding unexported module binding: chicken.compiler.core#alist-cons 
o|hiding unexported module binding: chicken.compiler.core#filter 
o|hiding unexported module binding: chicken.compiler.core#filter-map 
o|hiding unexported module binding: chicken.compiler.core#remove 
o|hiding unexported module binding: chicken.compiler.core#unzip1 
o|hiding unexported module binding: chicken.compiler.core#last 
o|hiding unexported module binding: chicken.compiler.core#list-index 
o|hiding unexported module binding: chicken.compiler.core#lset-adjoin/eq? 
o|hiding unexported module binding: chicken.compiler.core#lset-difference/eq? 
o|hiding unexported module binding: chicken.compiler.core#lset-union/eq? 
o|hiding unexported module binding: chicken.compiler.core#lset-intersection/eq? 
o|hiding unexported module binding: chicken.compiler.core#list-tabulate 
o|hiding unexported module binding: chicken.compiler.core#lset<=/eq? 
o|hiding unexported module binding: chicken.compiler.core#lset=/eq? 
o|hiding unexported module binding: chicken.compiler.core#length+ 
o|hiding unexported module binding: chicken.compiler.core#find 
o|hiding unexported module binding: chicken.compiler.core#find-tail 
o|hiding unexported module binding: chicken.compiler.core#iota 
o|hiding unexported module binding: chicken.compiler.core#make-list 
o|hiding unexported module binding: chicken.compiler.core#posq 
o|hiding unexported module binding: chicken.compiler.core#posv 
o|hiding unexported module binding: chicken.compiler.core#safe-globals-flag 
o|hiding unexported module binding: chicken.compiler.core#original-program-size 
o|hiding unexported module binding: chicken.compiler.core#current-program-size 
o|hiding unexported module binding: chicken.compiler.core#current-analysis-database-size 
o|hiding unexported module binding: chicken.compiler.core#direct-call-ids 
o|hiding unexported module binding: chicken.compiler.core#foreign-variables 
o|hiding unexported module binding: chicken.compiler.core#external-to-pointer 
o|hiding unexported module binding: chicken.compiler.core#location-pointer-map 
o|hiding unexported module binding: chicken.compiler.core#pending-canonicalizations 
o|hiding unexported module binding: chicken.compiler.core#defconstant-bindings 
o|hiding unexported module binding: chicken.compiler.core#callback-names 
o|hiding unexported module binding: chicken.compiler.core#toplevel-scope 
o|hiding unexported module binding: chicken.compiler.core#toplevel-lambda-id 
o|hiding unexported module binding: chicken.compiler.core#unlikely-variables 
o|hiding unexported module binding: chicken.compiler.core#process-declaration 
o|hiding unexported module binding: chicken.compiler.core#foreign-stub 
o|hiding unexported module binding: chicken.compiler.core#make-foreign-stub 
o|hiding unexported module binding: chicken.compiler.core#foreign-stub? 
o|hiding unexported module binding: chicken.compiler.core#create-foreign-stub 
o|hiding unexported module binding: chicken.compiler.core#expand-foreign-lambda 
o|hiding unexported module binding: chicken.compiler.core#expand-foreign-lambda* 
o|hiding unexported module binding: chicken.compiler.core#expand-foreign-primitive 
o|hiding unexported module binding: chicken.compiler.core#update-line-number-database! 
o|hiding unexported module binding: chicken.compiler.core#lambda-literal 
o|hiding unexported module binding: chicken.compiler.core#make-lambda-literal 
o|hiding unexported module binding: chicken.compiler.core#lambda-literal? 
o|hiding unexported module binding: chicken.compiler.core#lambda-literal-arguments 
o|hiding unexported module binding: chicken.compiler.core#lambda-literal-directly-called 
S|applied compiler syntax:
S|  chicken.base#o		1
S|  chicken.format#sprintf		19
S|  scheme#for-each		30
S|  chicken.base#foldl		3
S|  scheme#map		42
S|  chicken.base#foldr		3
o|eliminated procedure checks: 505 
o|specializations:
o|  3 (scheme#positive? *)
o|  1 (chicken.base#sub1 *)
o|  1 (scheme#integer? *)
o|  17 (chicken.base#add1 *)
o|  2 (scheme#memq * list)
o|  3 (scheme#zero? *)
o|  2 (scheme#>= fixnum fixnum)
o|  1 (##sys#call-with-values (procedure () *) *)
o|  1 (chicken.base#current-error-port)
o|  1 (scheme#caddr (pair * (pair * pair)))
o|  4 (scheme#cadr (pair * pair))
o|  13 (scheme#cddr (pair * pair))
o|  2 (scheme#string-append string string)
o|  19 (##sys#check-output-port * * *)
o|  6 (scheme#= fixnum fixnum)
o|  3 (scheme#length list)
o|  3 (scheme#cdar (pair pair *))
o|  240 (scheme#eqv? * (or eof null fixnum char boolean symbol keyword))
o|  1 (scheme#eqv? * *)
o|  8 (##sys#check-list (or pair list) *)
o|  87 (scheme#cdr pair)
o|  32 (scheme#car pair)
o|  1 (##sys#debug-mode?)
(o e)|safe calls: 2244 
(o e)|assignments to immediate values: 41 
o|safe globals: (chicken.compiler.core#prepare-for-code-generation chicken.compiler.core#lambda-literal-direct chicken.compiler.core#lambda-literal-body chicken.compiler.core#lambda-literal-rest-argument-mode chicken.compiler.core#lambda-literal-customizable chicken.compiler.core#lambda-literal-looping chicken.compiler.core#lambda-literal-closure-size chicken.compiler.core#lambda-literal-directly-called chicken.compiler.core#lambda-literal-allocated chicken.compiler.core#lambda-literal-callee-signatures chicken.compiler.core#lambda-literal-float-temporaries chicken.compiler.core#lambda-literal-temporaries chicken.compiler.core#lambda-literal-rest-argument chicken.compiler.core#lambda-literal-argument-count chicken.compiler.core#lambda-literal-arguments chicken.compiler.core#lambda-literal-external chicken.compiler.core#lambda-literal-id chicken.compiler.core#lambda-literal? chicken.compiler.core#make-lambda-literal chicken.compiler.core#lambda-literal chicken.compiler.core#perform-closure-conversion chicken.compiler.core#analyze-expression chicken.compiler.core#perform-cps-conversion chicken.compiler.core#update-line-number-database! chicken.compiler.core#expand-foreign-primitive chicken.compiler.core#expand-foreign-lambda* chicken.compiler.core#expand-foreign-lambda chicken.compiler.core#create-foreign-stub chicken.compiler.core#foreign-stub-callback chicken.compiler.core#foreign-stub-cps chicken.compiler.core#foreign-stub-body chicken.compiler.core#foreign-stub-argument-names chicken.compiler.core#foreign-stub-argument-types chicken.compiler.core#foreign-stub-name chicken.compiler.core#foreign-stub-return-type chicken.compiler.core#foreign-stub-id chicken.compiler.core#foreign-stub? chicken.compiler.core#make-foreign-stub chicken.compiler.core#foreign-stub chicken.compiler.core#build-toplevel-procedure chicken.compiler.core#process-declaration chicken.compiler.core#canonicalize-expression chicken.compiler.core#compute-database-statistics chicken.compiler.core#initialize-compiler chicken.compiler.core#internal-bindings chicken.compiler.core#default-standard-bindings chicken.compiler.core#default-extended-bindings chicken.compiler.core#unlikely-variables chicken.compiler.core#used-libraries chicken.compiler.core#linked-libraries chicken.compiler.core#required-libraries chicken.compiler.core#toplevel-lambda-id chicken.compiler.core#toplevel-scope chicken.compiler.core#callback-names chicken.compiler.core#defconstant-bindings chicken.compiler.core#pending-canonicalizations chicken.compiler.core#location-pointer-map chicken.compiler.core#external-to-pointer chicken.compiler.core#external-variables chicken.compiler.core#foreign-lambda-stubs chicken.compiler.core#foreign-variables chicken.compiler.core#first-analysis chicken.compiler.core#direct-call-ids chicken.compiler.core#inline-substitutions-enabled chicken.compiler.core#constant-table chicken.compiler.core#inline-table chicken.compiler.core#immutable-constants chicken.compiler.core#line-number-database-2 chicken.compiler.core#current-analysis-database-size chicken.compiler.core#current-program-size chicken.compiler.core#original-program-size chicken.compiler.core#verbose-mode chicken.compiler.core#types-output-file chicken.compiler.core#emit-link-file chicken.compiler.core#static-extensions chicken.compiler.core#enable-specialization chicken.compiler.core#strict-variable-types chicken.compiler.core#bootstrap-mode chicken.compiler.core#compiler-syntax-enabled chicken.compiler.core#enable-inline-files chicken.compiler.core#inline-locally chicken.compiler.core#local-definitions chicken.compiler.core#standalone-executable chicken.compiler.core#compile-module-registration chicken.compiler.core#preserve-unchanged-import-libraries chicken.compiler.core#all-import-libraries chicken.compiler.core#import-libraries chicken.compiler.core#profiled-procedures chicken.compiler.core#undefine-shadowed-macros chicken.compiler.core#emit-closure-info chicken.compiler.core#unroll-limit chicken.compiler.core#inline-max-size chicken.compiler.core#external-protos-first chicken.compiler.core#disable-stack-overflow-checking chicken.compiler.core#explicit-use-flag chicken.compiler.core#safe-globals-flag chicken.compiler.core#no-global-procedure-checks chicken.compiler.core#no-procedure-checks chicken.compiler.core#no-argc-checks chicken.compiler.core#no-bound-checks chicken.compiler.core#emit-profile chicken.compiler.core#optimize-leaf-routines chicken.compiler.core#target-stack-size chicken.compiler.core#target-heap-size chicken.compiler.core#line-number-database-size chicken.compiler.core#block-compilation chicken.compiler.core#emit-debug-info chicken.compiler.core#emit-trace-info chicken.compiler.core#foreign-declarations chicken.compiler.core#used-units chicken.compiler.core#insert-timer-checks chicken.compiler.core#extended-bindings chicken.compiler.core#standard-bindings chicken.compiler.core#unit-name chicken.compiler.core#posv chicken.compiler.core#posq chicken.compiler.core#make-list chicken.compiler.core#iota chicken.compiler.core#find-tail chicken.compiler.core#find chicken.compiler.core#length+ chicken.compiler.core#lset=/eq? chicken.compiler.core#lset<=/eq? chicken.compiler.core#list-tabulate chicken.compiler.core#lset-intersection/eq? chicken.compiler.core#lset-union/eq? chicken.compiler.core#lset-difference/eq? chicken.compiler.core#lset-adjoin/eq? chicken.compiler.core#list-index chicken.compiler.core#last chicken.compiler.core#unzip1 chicken.compiler.core#remove chicken.compiler.core#filter-map chicken.compiler.core#filter chicken.compiler.core#alist-cons chicken.compiler.core#delete-duplicates chicken.compiler.core#fifth chicken.compiler.core#fourth chicken.compiler.core#third chicken.compiler.core#second chicken.compiler.core#first chicken.compiler.core#delete chicken.compiler.core#concatenate chicken.compiler.core#cons* chicken.compiler.core#any chicken.compiler.core#every chicken.compiler.core#append-map chicken.compiler.core#split-at chicken.compiler.core#drop chicken.compiler.core#take chicken.compiler.core#span chicken.compiler.core#partition chicken.compiler.core#d) 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#d 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#partition 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#span 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#drop 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#append-map 
o|inlining procedure: k6686 
o|inlining procedure: k6686 
o|inlining procedure: k6717 
o|inlining procedure: k6717 
o|merged explicitly consed rest parameter: xs350 
o|inlining procedure: k6747 
o|inlining procedure: k6747 
o|inlining procedure: k6775 
o|inlining procedure: k6775 
o|inlining procedure: k6803 
o|inlining procedure: k6803 
o|inlining procedure: k6934 
o|inlining procedure: k6934 
o|inlining procedure: k6926 
o|inlining procedure: k6926 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#remove 
o|inlining procedure: k7043 
o|contracted procedure: "(mini-srfi-1.scm:143) g453462" 
o|inlining procedure: k7043 
o|merged explicitly consed rest parameter: vals490 
o|inlining procedure: k7142 
o|inlining procedure: k7142 
o|merged explicitly consed rest parameter: lss501 
o|inlining procedure: k7183 
o|contracted procedure: "(mini-srfi-1.scm:164) g514515" 
o|inlining procedure: k7183 
o|merged explicitly consed rest parameter: lss523 
o|inlining procedure: k7228 
o|contracted procedure: "(mini-srfi-1.scm:171) g536537" 
o|inlining procedure: k7253 
o|contracted procedure: "(mini-srfi-1.scm:173) g552553" 
o|inlining procedure: k7269 
o|inlining procedure: k7269 
o|inlining procedure: k7253 
o|inlining procedure: k7228 
o|merged explicitly consed rest parameter: lss563 
o|inlining procedure: k7325 
o|inlining procedure: k7325 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#lset<=/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#lset=/eq? 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#length+ 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#find 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#find-tail 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#iota 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#make-list 
o|inlining procedure: k7526 
o|inlining procedure: k7526 
o|inlining procedure: k7716 
o|inlining procedure: k7716 
o|inlining procedure: k7736 
o|inlining procedure: k7736 
o|inlining procedure: k7752 
o|inlining procedure: k7752 
o|substituted constant variable: a7769 
o|substituted constant variable: a7771 
o|substituted constant variable: a7773 
o|inlining procedure: k7783 
o|inlining procedure: k7783 
o|inlining procedure: k7809 
o|inlining procedure: k7809 
o|inlining procedure: k7847 
o|inlining procedure: k7847 
o|inlining procedure: k7862 
o|inlining procedure: k7862 
o|inlining procedure: k7896 
o|contracted procedure: "(core.scm:547) chicken.compiler.core#update-line-number-database!" 
o|inlining procedure: k16428 
o|inlining procedure: k16428 
o|inlining procedure: k16450 
o|inlining procedure: k16450 
o|inlining procedure: k7896 
o|inlining procedure: k7964 
o|contracted procedure: "(core.scm:556) g869877" 
o|inlining procedure: k7964 
o|removed side-effect free assignment to unused variable: write-to-string811 
o|inlining procedure: k8013 
o|inlining procedure: k8013 
o|inlining procedure: k8025 
o|inlining procedure: k8025 
o|substituted constant variable: a8050 
o|inlining procedure: k8065 
o|inlining procedure: k8065 
o|inlining procedure: k8088 
o|inlining procedure: k8088 
o|inlining procedure: "(core.scm:586) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:584) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:596) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:594) chicken.compiler.core#third" 
o|inlining procedure: k8141 
o|inlining procedure: k8141 
o|substituted constant variable: a8216 
o|substituted constant variable: a8217 
o|inlining procedure: k8193 
o|inlining procedure: k8193 
o|substituted constant variable: a8252 
o|substituted constant variable: a8253 
o|inlining procedure: k8364 
o|inlining procedure: k8364 
o|inlining procedure: k8382 
o|inlining procedure: k8382 
o|inlining procedure: k8428 
o|inlining procedure: k8428 
o|inlining procedure: k8459 
o|inlining procedure: k8477 
o|inlining procedure: k8477 
o|inlining procedure: k8459 
o|inlining procedure: k8522 
o|inlining procedure: k8522 
o|inlining procedure: k8587 
o|inlining procedure: k8587 
o|inlining procedure: k8567 
o|inlining procedure: k8567 
o|inlining procedure: k8696 
o|inlining procedure: k8696 
o|inlining procedure: k8734 
o|inlining procedure: k8734 
o|inlining procedure: k8774 
o|consed rest parameter at call site: "(core.scm:722) chicken.compiler.core#lset-adjoin/eq?" 2 
o|inlining procedure: k8774 
o|inlining procedure: k8815 
o|inlining procedure: k8815 
o|inlining procedure: k8853 
o|inlining procedure: k8875 
o|inlining procedure: k8875 
o|consed rest parameter at call site: "(core.scm:750) chicken.compiler.core#lset-adjoin/eq?" 2 
o|inlining procedure: k8853 
o|inlining procedure: k9022 
o|inlining procedure: k9022 
o|inlining procedure: k9074 
o|inlining procedure: k9074 
o|inlining procedure: k9105 
o|inlining procedure: k9185 
o|contracted procedure: "(core.scm:778) g12901299" 
o|inlining procedure: k9185 
o|inlining procedure: k9219 
o|contracted procedure: "(core.scm:775) g12631272" 
o|inlining procedure: k9219 
o|inlining procedure: k9105 
o|inlining procedure: k9362 
o|contracted procedure: "(core.scm:799) g14161426" 
o|inlining procedure: k9362 
o|inlining procedure: k9410 
o|inlining procedure: k9410 
o|inlining procedure: k9458 
o|contracted procedure: "(core.scm:791) g13531362" 
o|inlining procedure: k9458 
o|inlining procedure: k9492 
o|inlining procedure: k9492 
o|inlining procedure: k9523 
o|inlining procedure: k9581 
o|inlining procedure: k9581 
o|inlining procedure: k9523 
o|inlining procedure: k9662 
o|inlining procedure: k9662 
o|inlining procedure: k9681 
o|inlining procedure: k9681 
o|inlining procedure: k9693 
o|inlining procedure: k9693 
o|inlining procedure: k9705 
o|inlining procedure: k9705 
o|inlining procedure: k9716 
o|inlining procedure: k9716 
o|inlining procedure: k9804 
o|inlining procedure: k9804 
o|inlining procedure: k9856 
o|inlining procedure: k9962 
o|contracted procedure: "(core.scm:867) g15941603" 
o|inlining procedure: k9962 
o|inlining procedure: k9856 
o|inlining procedure: k10105 
o|inlining procedure: k10105 
o|inlining procedure: k10132 
o|contracted procedure: "(core.scm:883) g16331642" 
o|inlining procedure: k10132 
o|inlining procedure: k10163 
o|inlining procedure: k10163 
o|inlining procedure: k10309 
o|inlining procedure: k10309 
o|inlining procedure: k10328 
o|inlining procedure: k10328 
o|inlining procedure: k10354 
o|inlining procedure: k10455 
o|contracted procedure: "(core.scm:978) g17541761" 
o|inlining procedure: k10455 
o|inlining procedure: k10514 
o|contracted procedure: "(core.scm:988) g17721779" 
o|inlining procedure: k10514 
o|inlining procedure: k10537 
o|inlining procedure: k10537 
o|inlining procedure: k10354 
o|inlining procedure: k10632 
o|inlining procedure: k10679 
o|contracted procedure: "(core.scm:1011) g18091818" 
o|inlining procedure: k10679 
o|inlining procedure: k10632 
o|substituted constant variable: saved189518961914 
o|inlining procedure: k10758 
o|inlining procedure: k10829 
o|inlining procedure: k10829 
o|inlining procedure: k10861 
o|inlining procedure: k10861 
o|inlining procedure: k10758 
o|inlining procedure: k10881 
o|inlining procedure: k10881 
o|inlining procedure: k10901 
o|inlining procedure: k10901 
o|contracted procedure: "(core.scm:1081) emit-import-lib814" 
o|inlining procedure: k8278 
o|inlining procedure: k8278 
o|inlining procedure: k8314 
o|inlining procedure: k8314 
o|substituted constant variable: a8357 
o|inlining procedure: k10914 
o|inlining procedure: k10914 
o|inlining procedure: k11011 
o|inlining procedure: k11058 
o|inlining procedure: k11058 
o|inlining procedure: k11070 
o|inlining procedure: k11070 
o|merged explicitly consed rest parameter: args19381951 
o|consed rest parameter at call site: tmp25797 1 
o|inlining procedure: k11011 
o|inlining procedure: k11158 
o|inlining procedure: k11166 
o|inlining procedure: k11166 
o|inlining procedure: k11193 
o|inlining procedure: k11193 
o|inlining procedure: k11158 
o|inlining procedure: k11223 
o|inlining procedure: k11223 
o|inlining procedure: k11265 
o|propagated global variable: tmp18401842 chicken.compiler.core#unit-name 
o|propagated global variable: tmp18401842 chicken.compiler.core#unit-name 
o|inlining procedure: k11265 
o|inlining procedure: k11275 
o|inlining procedure: k11361 
o|inlining procedure: k11361 
o|inlining procedure: k11275 
o|inlining procedure: k11398 
o|inlining procedure: k11398 
o|substituted constant variable: a11421 
o|substituted constant variable: a11422 
o|inlining procedure: k11414 
o|inlining procedure: k11414 
o|inlining procedure: k11435 
o|inlining procedure: "(core.scm:1145) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:1140) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:1156) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:1150) chicken.compiler.core#third" 
o|inlining procedure: k11511 
o|inlining procedure: k11511 
o|inlining procedure: k11587 
o|substituted constant variable: a11614 
o|substituted constant variable: a11615 
o|substituted constant variable: a11639 
o|substituted constant variable: a11640 
o|inlining procedure: k11632 
o|inlining procedure: k11632 
o|inlining procedure: k11587 
o|substituted constant variable: a11667 
o|substituted constant variable: a11668 
o|substituted constant variable: a11692 
o|substituted constant variable: a11693 
o|inlining procedure: k11685 
o|inlining procedure: k11685 
o|substituted constant variable: a11751 
o|substituted constant variable: a11752 
o|substituted constant variable: a11776 
o|substituted constant variable: a11777 
o|inlining procedure: k11769 
o|inlining procedure: k11769 
o|inlining procedure: k11435 
o|inlining procedure: k11835 
o|inlining procedure: k11835 
o|inlining procedure: k11866 
o|inlining procedure: k11866 
o|inlining procedure: k11932 
o|inlining procedure: k11932 
o|inlining procedure: "(core.scm:1199) chicken.compiler.core#second" 
o|inlining procedure: k11963 
o|inlining procedure: k11963 
o|inlining procedure: k12013 
o|inlining procedure: k12013 
o|inlining procedure: k12047 
o|inlining procedure: k12075 
o|inlining procedure: k12075 
o|inlining procedure: k12047 
o|inlining procedure: k12114 
o|inlining procedure: k12114 
o|inlining procedure: k12140 
o|inlining procedure: k12140 
o|contracted procedure: "(core.scm:1244) chicken.compiler.core#expand-foreign-primitive" 
o|inlining procedure: k16310 
o|inlining procedure: k16310 
o|inlining procedure: k16344 
o|contracted procedure: "(core.scm:1905) g35843593" 
o|inlining procedure: k16344 
o|inlining procedure: k16376 
o|inlining procedure: k16376 
o|inlining procedure: k16386 
o|inlining procedure: "(core.scm:1903) chicken.compiler.core#third" 
o|inlining procedure: k16386 
o|inlining procedure: "(core.scm:1903) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:1902) chicken.compiler.core#second" 
o|inlining procedure: k12166 
o|inlining procedure: "(core.scm:1250) chicken.compiler.core#fourth" 
o|inlining procedure: "(core.scm:1248) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:1247) chicken.compiler.core#second" 
o|inlining procedure: k12166 
o|inlining procedure: k12228 
o|inlining procedure: k12277 
o|inlining procedure: "(core.scm:1284) chicken.compiler.core#second" 
o|inlining procedure: k12277 
o|inlining procedure: "(core.scm:1281) chicken.compiler.core#first" 
o|inlining procedure: k12228 
o|substituted constant variable: a12314 
o|substituted constant variable: a12315 
o|inlining procedure: k12307 
o|inlining procedure: k12307 
o|inlining procedure: "(core.scm:1259) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:1258) chicken.compiler.core#second" 
o|inlining procedure: k12332 
o|substituted constant variable: a12405 
o|substituted constant variable: a12406 
o|inlining procedure: k12398 
o|inlining procedure: k12398 
o|inlining procedure: "(core.scm:1295) chicken.compiler.core#fourth" 
o|inlining procedure: "(core.scm:1294) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:1291) chicken.compiler.core#second" 
o|inlining procedure: k12332 
o|inlining procedure: k12501 
o|inlining procedure: k12501 
o|inlining procedure: "(core.scm:1334) chicken.compiler.core#fourth" 
o|inlining procedure: "(core.scm:1315) chicken.compiler.core#fourth" 
o|inlining procedure: "(core.scm:1312) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:1311) chicken.compiler.core#second" 
o|inlining procedure: k12576 
o|substituted constant variable: a12608 
o|substituted constant variable: a12609 
o|inlining procedure: k12601 
o|inlining procedure: k12601 
o|inlining procedure: "(core.scm:1339) chicken.compiler.core#second" 
o|inlining procedure: k12576 
o|inlining procedure: k12654 
o|inlining procedure: k12654 
o|substituted constant variable: a12710 
o|substituted constant variable: a12711 
o|inlining procedure: k12703 
o|inlining procedure: k12703 
o|substituted constant variable: a12743 
o|substituted constant variable: a12744 
o|inlining procedure: k12736 
o|inlining procedure: k12736 
o|inlining procedure: k12784 
o|inlining procedure: k12784 
o|inlining procedure: "(core.scm:1353) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:1351) chicken.compiler.core#second" 
o|inlining procedure: k12819 
o|contracted procedure: "(core.scm:1391) chicken.compiler.core#process-declaration" 
o|merged explicitly consed rest parameter: maxlen2683 
o|inlining procedure: k13705 
o|inlining procedure: k13705 
o|inlining procedure: k13721 
o|inlining procedure: k13721 
o|inlining procedure: k13737 
o|inlining procedure: k13737 
o|contracted procedure: "(core.scm:1555) chicken.compiler.core#filter-map" 
o|inlining procedure: k6981 
o|inlining procedure: k6981 
o|inlining procedure: k6970 
o|inlining procedure: k6970 
o|substituted constant variable: a13770 
o|substituted constant variable: a13771 
o|inlining procedure: k13796 
o|consed rest parameter at call site: "(core.scm:1567) chicken.compiler.core#lset-union/eq?" 2 
o|consed rest parameter at call site: "(core.scm:1566) chicken.compiler.core#lset-union/eq?" 2 
o|inlining procedure: k13796 
o|inlining procedure: k13827 
o|inlining procedure: k13827 
o|consed rest parameter at call site: "(core.scm:1569) check-decl2677" 3 
o|inlining procedure: k13849 
o|inlining procedure: k13849 
o|inlining procedure: k13880 
o|inlining procedure: k13880 
o|inlining procedure: k13899 
o|consed rest parameter at call site: "(core.scm:1590) chicken.compiler.core#lset-intersection/eq?" 2 
o|consed rest parameter at call site: "(core.scm:1589) chicken.compiler.core#lset-intersection/eq?" 2 
o|inlining procedure: k13899 
o|consed rest parameter at call site: "(core.scm:1592) check-decl2677" 3 
o|inlining procedure: k13945 
o|inlining procedure: k13945 
o|inlining procedure: k13962 
o|inlining procedure: k13962 
o|inlining procedure: k13976 
o|inlining procedure: k13976 
o|inlining procedure: k13990 
o|inlining procedure: k13990 
o|inlining procedure: k14004 
o|inlining procedure: k14025 
o|contracted procedure: "(core.scm:1603) g27402747" 
o|propagated global variable: g27552756 chicken.compiler.support#mark-variable 
o|inlining procedure: k14025 
o|inlining procedure: k14004 
o|inlining procedure: k14052 
o|inlining procedure: k14089 
o|contracted procedure: "(core.scm:1609) g27902797" 
o|propagated global variable: g28052806 chicken.compiler.support#mark-variable 
o|inlining procedure: k14089 
o|inlining procedure: k14112 
o|contracted procedure: "(core.scm:1606) g27652772" 
o|propagated global variable: g27802781 chicken.compiler.support#mark-variable 
o|inlining procedure: k14112 
o|inlining procedure: k14052 
o|inlining procedure: k14139 
o|inlining procedure: k14170 
o|contracted procedure: "(core.scm:1617) g28272849" 
o|propagated global variable: g28572858 chicken.compiler.support#mark-variable 
o|inlining procedure: k14170 
o|inlining procedure: k14193 
o|contracted procedure: "(core.scm:1616) g28172834" 
o|propagated global variable: g28422843 chicken.compiler.support#mark-variable 
o|inlining procedure: k14193 
o|inlining procedure: k14139 
o|inlining procedure: k14222 
o|inlining procedure: k14222 
o|inlining procedure: k14235 
o|inlining procedure: k14235 
o|inlining procedure: k14249 
o|inlining procedure: k14249 
o|inlining procedure: k14279 
o|contracted procedure: "(core.scm:1627) g28692876" 
o|propagated global variable: g28842885 chicken.compiler.support#mark-variable 
o|inlining procedure: k14279 
o|inlining procedure: k14301 
o|inlining procedure: k14324 
o|contracted procedure: "(core.scm:1629) g28942901" 
o|propagated global variable: g29092910 chicken.compiler.support#mark-variable 
o|inlining procedure: k14324 
o|inlining procedure: k14301 
o|inlining procedure: k14358 
o|consed rest parameter at call site: "(core.scm:1639) chicken.compiler.core#lset-difference/eq?" 2 
o|inlining procedure: k14358 
o|inlining procedure: k14392 
o|inlining procedure: k14392 
o|consed rest parameter at call site: "(core.scm:1645) chicken.compiler.core#lset-difference/eq?" 2 
o|inlining procedure: k14414 
o|inlining procedure: k14444 
o|contracted procedure: "(core.scm:1650) g29252932" 
o|propagated global variable: g29402941 chicken.compiler.support#mark-variable 
o|inlining procedure: k14444 
o|inlining procedure: k14414 
o|inlining procedure: k14477 
o|inlining procedure: k14477 
o|consed rest parameter at call site: "(core.scm:1660) chicken.compiler.core#lset-difference/eq?" 2 
o|consed rest parameter at call site: "(core.scm:1659) chicken.compiler.core#lset-difference/eq?" 2 
o|inlining procedure: k14503 
o|inlining procedure: k14533 
o|contracted procedure: "(core.scm:1664) g29572964" 
o|propagated global variable: g29722973 chicken.compiler.support#mark-variable 
o|inlining procedure: k14533 
o|inlining procedure: k14503 
o|substituted constant variable: a14577 
o|consed rest parameter at call site: "(core.scm:1668) check-decl2677" 3 
o|substituted constant variable: a14583 
o|substituted constant variable: a14585 
o|substituted constant variable: a14587 
o|substituted constant variable: a14589 
o|substituted constant variable: a14591 
o|inlining procedure: "(core.scm:1634) chicken.compiler.core#second" 
o|consed rest parameter at call site: "(core.scm:1633) check-decl2677" 3 
o|inlining procedure: k14596 
o|inlining procedure: k14596 
o|inlining procedure: k14615 
o|inlining procedure: k14615 
o|inlining procedure: k14631 
o|inlining procedure: k14631 
o|inlining procedure: k14653 
o|inlining procedure: k14672 
o|inlining procedure: k14672 
o|inlining procedure: k14653 
o|inlining procedure: k14701 
o|inlining procedure: k14731 
o|contracted procedure: "(core.scm:1689) g30323039" 
o|propagated global variable: g30473048 chicken.compiler.support#mark-variable 
o|inlining procedure: k14731 
o|inlining procedure: k14701 
o|inlining procedure: k14767 
o|inlining procedure: k14767 
o|consed rest parameter at call site: "(core.scm:1693) check-decl2677" 3 
o|inlining procedure: k14777 
o|consed rest parameter at call site: "(core.scm:1701) check-decl2677" 3 
o|inlining procedure: k14777 
o|inlining procedure: k14808 
o|inlining procedure: k14831 
o|contracted procedure: "(core.scm:1711) g30623069" 
o|propagated global variable: g30773078 chicken.compiler.support#mark-variable 
o|inlining procedure: k14831 
o|inlining procedure: k14808 
o|inlining procedure: k14856 
o|inlining procedure: k14951 
o|contracted procedure: "(core.scm:1720) g30923101" 
o|inlining procedure: k14872 
o|substituted constant variable: a14889 
o|inlining procedure: k14872 
o|inlining procedure: k14913 
o|inlining procedure: k14913 
o|substituted constant variable: a14936 
o|inlining procedure: k14951 
o|inlining procedure: k14856 
o|inlining procedure: k14990 
o|inlining procedure: k14990 
o|inlining procedure: k15003 
o|inlining procedure: k15003 
o|inlining procedure: k15012 
o|inlining procedure: k15012 
o|inlining procedure: k15029 
o|inlining procedure: k15061 
o|contracted procedure: "(core.scm:1741) g31333140" 
o|propagated global variable: g31483149 chicken.compiler.support#mark-variable 
o|inlining procedure: k15061 
o|inlining procedure: k15029 
o|inlining procedure: k15091 
o|inlining procedure: k15091 
o|inlining procedure: k15115 
o|contracted procedure: "(core.scm:1748) g31643171" 
o|propagated global variable: g31793180 chicken.compiler.support#mark-variable 
o|inlining procedure: k15115 
o|inlining procedure: k15139 
o|inlining procedure: k15170 
o|contracted procedure: "(core.scm:1755) g31893196" 
o|propagated global variable: g32043205 chicken.compiler.support#mark-variable 
o|inlining procedure: k15170 
o|inlining procedure: k15139 
o|inlining procedure: k15202 
o|inlining procedure: k15202 
o|inlining procedure: k15244 
o|inlining procedure: k15301 
o|inlining procedure: k15301 
o|inlining procedure: k15244 
o|inlining procedure: k15334 
o|substituted constant variable: a15355 
o|inlining procedure: k15334 
o|inlining procedure: k15366 
o|inlining procedure: k15366 
o|inlining procedure: k15394 
o|inlining procedure: k15429 
o|inlining procedure: k15429 
o|inlining procedure: k15394 
o|inlining procedure: k15460 
o|substituted constant variable: a15469 
o|inlining procedure: k15460 
o|inlining procedure: k15386 
o|inlining procedure: k15488 
o|inlining procedure: k15488 
o|inlining procedure: k15386 
o|inlining procedure: k15515 
o|inlining procedure: k15515 
o|substituted constant variable: a15526 
o|substituted constant variable: a15528 
o|substituted constant variable: a15530 
o|substituted constant variable: a15532 
o|substituted constant variable: a15534 
o|substituted constant variable: a15536 
o|substituted constant variable: a15538 
o|substituted constant variable: a15540 
o|substituted constant variable: a15542 
o|substituted constant variable: a15544 
o|substituted constant variable: a15546 
o|substituted constant variable: a15548 
o|substituted constant variable: a15550 
o|substituted constant variable: a15552 
o|substituted constant variable: a15554 
o|substituted constant variable: a15559 
o|substituted constant variable: a15561 
o|substituted constant variable: a15563 
o|substituted constant variable: a15565 
o|substituted constant variable: a15567 
o|substituted constant variable: a15569 
o|substituted constant variable: a15571 
o|substituted constant variable: a15573 
o|substituted constant variable: a15575 
o|substituted constant variable: a15577 
o|substituted constant variable: a15579 
o|substituted constant variable: a15581 
o|substituted constant variable: a15583 
o|substituted constant variable: a15585 
o|substituted constant variable: a15587 
o|substituted constant variable: a15589 
o|substituted constant variable: a15591 
o|substituted constant variable: a15593 
o|substituted constant variable: a15595 
o|substituted constant variable: a15597 
o|substituted constant variable: a15599 
o|substituted constant variable: a15601 
o|substituted constant variable: a15606 
o|substituted constant variable: a15608 
o|substituted constant variable: a15610 
o|substituted constant variable: a15612 
o|substituted constant variable: a15614 
o|substituted constant variable: a15616 
o|substituted constant variable: a15618 
o|substituted constant variable: a15620 
o|inlining procedure: k12861 
o|inlining procedure: k12861 
o|inlining procedure: k12819 
o|contracted procedure: "(core.scm:1396) chicken.compiler.core#split-at" 
o|inlining procedure: k6474 
o|inlining procedure: k6474 
o|inlining procedure: k12982 
o|inlining procedure: k12982 
o|inlining procedure: k13020 
o|inlining procedure: k13020 
o|inlining procedure: k13075 
o|inlining procedure: k13075 
o|inlining procedure: k13126 
o|inlining procedure: k13126 
o|substituted constant variable: a13177 
o|substituted constant variable: a13178 
o|inlining procedure: k13170 
o|inlining procedure: k13170 
o|inlining procedure: "(core.scm:1402) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:1401) chicken.compiler.core#fourth" 
o|inlining procedure: "(core.scm:1400) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:1398) chicken.compiler.core#first" 
o|inlining procedure: k13203 
o|inlining procedure: "(core.scm:1483) chicken.compiler.core#second" 
o|inlining procedure: k13221 
o|inlining procedure: k13221 
o|inlining procedure: k13262 
o|inlining procedure: k13262 
o|inlining procedure: k13203 
o|inlining procedure: k13332 
o|inlining procedure: k13332 
o|substituted constant variable: a13355 
o|substituted constant variable: a13357 
o|substituted constant variable: a13359 
o|substituted constant variable: a13361 
o|substituted constant variable: a13363 
o|substituted constant variable: a13365 
o|substituted constant variable: a13367 
o|substituted constant variable: a13369 
o|substituted constant variable: a13371 
o|substituted constant variable: a13373 
o|substituted constant variable: a13375 
o|substituted constant variable: a13377 
o|substituted constant variable: a13379 
o|substituted constant variable: a13381 
o|substituted constant variable: a13383 
o|substituted constant variable: a13388 
o|substituted constant variable: a13390 
o|substituted constant variable: a13395 
o|substituted constant variable: a13397 
o|substituted constant variable: a13399 
o|substituted constant variable: a13401 
o|substituted constant variable: a13403 
o|substituted constant variable: a13405 
o|substituted constant variable: a13407 
o|substituted constant variable: a13409 
o|substituted constant variable: a13411 
o|substituted constant variable: a13413 
o|substituted constant variable: a13415 
o|substituted constant variable: a13417 
o|substituted constant variable: a13419 
o|substituted constant variable: a13421 
o|substituted constant variable: a13423 
o|substituted constant variable: a13425 
o|substituted constant variable: a13427 
o|substituted constant variable: a13429 
o|substituted constant variable: a13431 
o|substituted constant variable: a13433 
o|substituted constant variable: a13435 
o|substituted constant variable: a13437 
o|substituted constant variable: a13439 
o|substituted constant variable: a13441 
o|substituted constant variable: a13443 
o|substituted constant variable: a13445 
o|substituted constant variable: a13447 
o|substituted constant variable: a13449 
o|inlining procedure: k13453 
o|inlining procedure: k13453 
o|substituted constant variable: a13460 
o|substituted constant variable: a13462 
o|substituted constant variable: a13464 
o|substituted constant variable: a13466 
o|substituted constant variable: a13468 
o|substituted constant variable: a13470 
o|substituted constant variable: a13472 
o|substituted constant variable: a13474 
o|substituted constant variable: a13479 
o|substituted constant variable: a13481 
o|substituted constant variable: a13483 
o|inlining procedure: k13513 
o|substituted constant variable: a13524 
o|substituted constant variable: a13525 
o|inlining procedure: k13513 
o|inlining procedure: k13541 
o|inlining procedure: k13541 
o|inlining procedure: k13628 
o|inlining procedure: k13628 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#foreign-stub 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#foreign-stub? 
o|inlining procedure: k15882 
o|inlining procedure: k15882 
o|inlining procedure: k15949 
o|contracted procedure: "(core.scm:1866) g34603470" 
o|inlining procedure: k15949 
o|contracted procedure: "(core.scm:1860) chicken.compiler.core#make-foreign-stub" 
o|inlining procedure: k16030 
o|inlining procedure: k16030 
o|contracted procedure: "(core.scm:1854) type->symbol3361" 
o|inlining procedure: k15736 
o|inlining procedure: k15736 
o|inlining procedure: k15751 
o|inlining procedure: k15751 
o|substituted constant variable: a15793 
o|substituted constant variable: a15795 
o|substituted constant variable: a15797 
o|inlining procedure: k15798 
o|inlining procedure: k15798 
o|inlining procedure: k16082 
o|inlining procedure: k16082 
o|inlining procedure: "(core.scm:1886) chicken.compiler.core#second" 
o|inlining procedure: k16141 
o|inlining procedure: k16141 
o|inlining procedure: "(core.scm:1880) chicken.compiler.core#third" 
o|inlining procedure: k16194 
o|inlining procedure: k16194 
o|inlining procedure: k16228 
o|contracted procedure: "(core.scm:1894) g35223531" 
o|inlining procedure: k16228 
o|inlining procedure: "(core.scm:1892) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:1891) chicken.compiler.core#second" 
o|inlining procedure: k16555 
o|inlining procedure: k16555 
o|inlining procedure: k16591 
o|inlining procedure: k16591 
o|contracted procedure: "(core.scm:1961) g37033704" 
o|inlining procedure: k16710 
o|inlining procedure: k16722 
o|inlining procedure: k16722 
o|inlining procedure: k16745 
o|inlining procedure: k16745 
o|inlining procedure: k16710 
o|inlining procedure: "(core.scm:1980) chicken.compiler.core#first" 
o|contracted procedure: "(core.scm:1980) g37143715" 
o|inlining procedure: k16812 
o|inlining procedure: "(core.scm:1983) chicken.compiler.core#first" 
o|inlining procedure: k16812 
o|inlining procedure: "(core.scm:1993) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:1991) chicken.compiler.core#first" 
o|contracted procedure: "(core.scm:1990) g37233724" 
o|inlining procedure: k16913 
o|inlining procedure: k16913 
o|inlining procedure: k16940 
o|contracted procedure: "(core.scm:2003) walk-call-unit3663" 
o|contracted procedure: "(core.scm:2031) g37723773" 
o|inlining procedure: "(core.scm:1999) chicken.compiler.core#first" 
o|inlining procedure: k16940 
o|inlining procedure: k17000 
o|contracted procedure: "(core.scm:2009) chicken.compiler.core#last" 
o|inlining procedure: k7083 
o|inlining procedure: k7083 
o|inlining procedure: k17000 
o|substituted constant variable: a17017 
o|substituted constant variable: a17022 
o|substituted constant variable: a17024 
o|substituted constant variable: a17026 
o|substituted constant variable: a17028 
o|inlining procedure: k17032 
o|inlining procedure: k17032 
o|inlining procedure: k17044 
o|inlining procedure: k17044 
o|inlining procedure: k17056 
o|inlining procedure: k17056 
o|substituted constant variable: a17063 
o|substituted constant variable: a17065 
o|substituted constant variable: a17067 
o|substituted constant variable: a17069 
o|substituted constant variable: a17071 
o|substituted constant variable: a17073 
o|substituted constant variable: a17075 
o|substituted constant variable: a17077 
o|substituted constant variable: a17079 
o|substituted constant variable: a17084 
o|substituted constant variable: a17086 
o|substituted constant variable: a17088 
o|substituted constant variable: a17090 
o|inlining procedure: k17094 
o|inlining procedure: k17094 
o|inlining procedure: k17106 
o|inlining procedure: k17106 
o|substituted constant variable: a17113 
o|substituted constant variable: a17115 
o|substituted constant variable: a17117 
o|substituted constant variable: a17119 
o|substituted constant variable: a17121 
o|consed rest parameter at call site: "(core.scm:2024) chicken.compiler.core#cons*" 2 
o|contracted procedure: "(core.scm:2017) g37643765" 
o|inlining procedure: k17293 
o|inlining procedure: k17293 
o|inlining procedure: k17334 
o|inlining procedure: k17334 
o|substituted constant variable: a17393 
o|inlining procedure: k17394 
o|inlining procedure: k17394 
o|substituted constant variable: a17401 
o|inlining procedure: k17443 
o|inlining procedure: k17443 
o|inlining procedure: k17467 
o|inlining procedure: k17467 
o|inlining procedure: k17485 
o|inlining procedure: k17485 
o|inlining procedure: "(core.scm:2091) chicken.compiler.core#first" 
o|inlining procedure: k17498 
o|inlining procedure: k17498 
o|inlining procedure: "(core.scm:2121) chicken.compiler.core#first" 
o|inlining procedure: k17569 
o|inlining procedure: k17578 
o|inlining procedure: "(core.scm:2115) chicken.compiler.core#third" 
o|inlining procedure: k17578 
o|inlining procedure: k17569 
o|inlining procedure: "(core.scm:2108) chicken.compiler.core#first" 
o|inlining procedure: k17627 
o|inlining procedure: k17645 
o|inlining procedure: k17645 
o|inlining procedure: k17627 
o|inlining procedure: k17730 
o|inlining procedure: k17730 
o|inlining procedure: "(core.scm:2139) chicken.compiler.core#first" 
o|inlining procedure: k17750 
o|inlining procedure: k17831 
o|inlining procedure: k17831 
o|inlining procedure: "(core.scm:2170) chicken.compiler.core#second" 
o|inlining procedure: k17865 
o|inlining procedure: k17865 
o|inlining procedure: "(core.scm:2154) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2152) chicken.compiler.core#third" 
o|inlining procedure: k17750 
o|inlining procedure: k17938 
o|inlining procedure: k17938 
o|inlining procedure: k17966 
o|inlining procedure: k17966 
o|substituted constant variable: a17985 
o|substituted constant variable: a17987 
o|inlining procedure: "(core.scm:2178) chicken.compiler.core#first" 
o|inlining procedure: k17991 
o|inlining procedure: k18015 
o|inlining procedure: k18015 
o|inlining procedure: "(core.scm:2199) chicken.compiler.core#first" 
o|inlining procedure: k17991 
o|substituted constant variable: a18034 
o|substituted constant variable: a18036 
o|substituted constant variable: a18041 
o|substituted constant variable: a18043 
o|substituted constant variable: a18048 
o|substituted constant variable: a18050 
o|substituted constant variable: a18052 
o|substituted constant variable: a18057 
o|substituted constant variable: a18059 
o|substituted constant variable: a18061 
o|substituted constant variable: a18066 
o|substituted constant variable: a18068 
o|substituted constant variable: a18073 
o|substituted constant variable: a18075 
o|inlining procedure: k18079 
o|inlining procedure: k18079 
o|substituted constant variable: a18092 
o|substituted constant variable: a18094 
o|substituted constant variable: a18096 
o|substituted constant variable: a18098 
o|inlining procedure: k18116 
o|inlining procedure: k18116 
o|inlining procedure: k18177 
o|inlining procedure: k18177 
o|inlining procedure: k18192 
o|inlining procedure: k18192 
o|inlining procedure: k18246 
o|inlining procedure: k18246 
o|inlining procedure: k18280 
o|inlining procedure: k18280 
o|inlining procedure: k18295 
o|inlining procedure: k18295 
o|inlining procedure: k18313 
o|inlining procedure: k18331 
o|inlining procedure: k18331 
o|inlining procedure: k18313 
o|propagated global variable: tmp40594061 chicken.compiler.core#local-definitions 
o|propagated global variable: tmp40594061 chicken.compiler.core#local-definitions 
o|inlining procedure: k18368 
o|inlining procedure: k18368 
o|inlining procedure: k18381 
o|inlining procedure: k18381 
o|inlining procedure: "(core.scm:2230) chicken.compiler.core#first" 
o|inlining procedure: k18421 
o|inlining procedure: k18421 
o|inlining procedure: "(core.scm:2224) chicken.compiler.core#first" 
o|inlining procedure: k18448 
o|inlining procedure: k18448 
o|inlining procedure: "(core.scm:2220) chicken.compiler.core#first" 
o|inlining procedure: k18506 
o|inlining procedure: k18506 
o|inlining procedure: k18522 
o|inlining procedure: k18522 
o|inlining procedure: k18541 
o|inlining procedure: k18541 
o|inlining procedure: k18556 
o|inlining procedure: k18556 
o|removed side-effect free assignment to unused variable: assigned-locally4087 
o|inlining procedure: k18575 
o|inlining procedure: k18575 
o|inlining procedure: k18589 
o|inlining procedure: k18589 
o|inlining procedure: k18604 
o|inlining procedure: k18604 
o|substituted constant variable: a18621 
o|substituted constant variable: a18623 
o|substituted constant variable: a18625 
o|substituted constant variable: a18627 
o|substituted constant variable: a18629 
o|substituted constant variable: a18631 
o|substituted constant variable: a18633 
o|substituted constant variable: a18635 
o|substituted constant variable: a18637 
o|substituted constant variable: a18639 
o|substituted constant variable: a18641 
o|substituted constant variable: a18643 
o|inlining procedure: k18682 
o|inlining procedure: k18703 
o|inlining procedure: k18724 
o|inlining procedure: "(core.scm:2504) chicken.compiler.core#first" 
o|inlining procedure: k18724 
o|inlining procedure: k18746 
o|inlining procedure: "(core.scm:2503) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2503) chicken.compiler.core#first" 
o|inlining procedure: k18746 
o|inlining procedure: "(core.scm:2500) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:2499) chicken.compiler.core#first" 
o|substituted constant variable: a18775 
o|inlining procedure: k18703 
o|inlining procedure: k18783 
o|inlining procedure: k18783 
o|inlining procedure: "(core.scm:2493) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2492) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:2491) chicken.compiler.core#second" 
o|inlining procedure: k18682 
o|inlining procedure: k18816 
o|inlining procedure: k18843 
o|inlining procedure: k18843 
o|inlining procedure: k18855 
o|inlining procedure: k18855 
o|inlining procedure: "(core.scm:2476) chicken.compiler.core#first" 
o|inlining procedure: k18816 
o|inlining procedure: k18903 
o|inlining procedure: k18915 
o|inlining procedure: k18915 
o|inlining procedure: k18918 
o|inlining procedure: k18936 
o|inlining procedure: k18936 
o|inlining procedure: "(core.scm:2454) chicken.compiler.core#first" 
o|inlining procedure: k18918 
o|inlining procedure: k18903 
o|inlining procedure: k18961 
o|inlining procedure: k18989 
o|inlining procedure: "(core.scm:2441) chicken.compiler.core#first" 
o|inlining procedure: k18989 
o|inlining procedure: "(core.scm:2443) chicken.compiler.core#first" 
o|inlining procedure: k19024 
o|inlining procedure: k19024 
o|inlining procedure: k19057 
o|inlining procedure: k19057 
o|inlining procedure: "(core.scm:2428) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:2426) chicken.compiler.core#second" 
o|inlining procedure: k18961 
o|inlining procedure: k19093 
o|inlining procedure: k19093 
o|inlining procedure: "(core.scm:2410) chicken.compiler.core#first" 
o|inlining procedure: k19122 
o|inlining procedure: k19122 
o|inlining procedure: k19152 
o|inlining procedure: k19152 
o|inlining procedure: "(core.scm:2371) chicken.compiler.core#second" 
o|inlining procedure: k19188 
o|inlining procedure: k19215 
o|inlining procedure: k19215 
o|inlining procedure: "(core.scm:2388) chicken.compiler.core#second" 
o|inlining procedure: k19188 
o|inlining procedure: k19264 
o|inlining procedure: k19264 
o|substituted constant variable: a19308 
o|substituted constant variable: a19309 
o|inlining procedure: k19322 
o|inlining procedure: k19331 
o|inlining procedure: k19343 
o|inlining procedure: k19343 
o|inlining procedure: k19331 
o|inlining procedure: k19322 
o|inlining procedure: k19379 
o|inlining procedure: "(core.scm:2344) chicken.compiler.core#first" 
o|inlining procedure: k19379 
o|inlining procedure: k19374 
o|inlining procedure: k19409 
o|inlining procedure: k19409 
o|inlining procedure: k19374 
o|inlining procedure: k19429 
o|inlining procedure: k19429 
o|inlining procedure: k19444 
o|inlining procedure: k19444 
o|consed rest parameter at call site: "(core.scm:2531) chicken.compiler.core#lset-adjoin/eq?" 2 
o|consed rest parameter at call site: "(core.scm:2536) chicken.compiler.core#lset-adjoin/eq?" 2 
o|inlining procedure: k19508 
o|inlining procedure: "(core.scm:2547) chicken.compiler.core#first" 
o|inlining procedure: k19508 
o|inlining procedure: k19538 
o|inlining procedure: "(core.scm:2560) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:2559) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2558) chicken.compiler.core#first" 
o|inlining procedure: k19538 
o|inlining procedure: k19598 
o|inlining procedure: k19598 
o|inlining procedure: "(core.scm:2564) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2563) chicken.compiler.core#first" 
o|inlining procedure: k19611 
o|inlining procedure: k19653 
o|inlining procedure: k19653 
o|inlining procedure: k19689 
o|inlining procedure: k19699 
o|inlining procedure: k19732 
o|inlining procedure: k19732 
o|inlining procedure: k19761 
o|inlining procedure: k19770 
o|inlining procedure: k19770 
o|inlining procedure: k19761 
o|inlining procedure: "(core.scm:2593) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2592) chicken.compiler.core#third" 
o|inlining procedure: k19797 
o|inlining procedure: k19812 
o|inlining procedure: k19812 
o|inlining procedure: k19797 
o|inlining procedure: "(core.scm:2580) chicken.compiler.core#first" 
o|inlining procedure: k19699 
o|inlining procedure: k19689 
o|inlining procedure: "(core.scm:2572) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:2571) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2570) chicken.compiler.core#first" 
o|inlining procedure: k19611 
o|consed rest parameter at call site: "(core.scm:2622) chicken.compiler.core#lset-difference/eq?" 2 
o|contracted procedure: "(core.scm:2619) chicken.compiler.core#delete-duplicates" 
o|inlining procedure: k6878 
o|inlining procedure: k6878 
o|inlining procedure: "(core.scm:2619) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2617) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2615) chicken.compiler.core#third" 
o|inlining procedure: k19945 
o|inlining procedure: k19945 
o|substituted constant variable: a19980 
o|substituted constant variable: a19982 
o|substituted constant variable: a19984 
o|substituted constant variable: a19986 
o|substituted constant variable: a19988 
o|inlining procedure: k19992 
o|inlining procedure: k19992 
o|inlining procedure: k20004 
o|inlining procedure: k20004 
o|substituted constant variable: a20011 
o|substituted constant variable: a20013 
o|substituted constant variable: a20015 
o|substituted constant variable: a20017 
o|substituted constant variable: a20019 
o|substituted constant variable: a20024 
o|substituted constant variable: a20026 
o|inlining procedure: k20039 
o|inlining procedure: k20039 
o|inlining procedure: k20060 
o|inlining procedure: k20060 
o|inlining procedure: "(core.scm:2638) chicken.compiler.core#first" 
o|inlining procedure: k20073 
o|inlining procedure: k20104 
o|inlining procedure: k20104 
o|inlining procedure: "(core.scm:2646) chicken.compiler.core#first" 
o|inlining procedure: k20073 
o|inlining procedure: k20179 
o|inlining procedure: "(core.scm:2684) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:2680) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2674) chicken.compiler.core#first" 
o|inlining procedure: k20179 
o|inlining procedure: k20355 
o|inlining procedure: k20355 
o|inlining procedure: k20399 
o|inlining procedure: k20399 
o|inlining procedure: k20442 
o|inlining procedure: k20485 
o|inlining procedure: k20485 
o|inlining procedure: k20521 
o|contracted procedure: "(core.scm:2730) g47234732" 
o|inlining procedure: k20521 
o|inlining procedure: k20442 
o|inlining procedure: "(core.scm:2725) chicken.compiler.core#fourth" 
o|inlining procedure: k20581 
o|inlining procedure: k20581 
o|inlining procedure: k20597 
o|inlining procedure: k20597 
o|inlining procedure: k20609 
o|inlining procedure: k20609 
o|inlining procedure: "(core.scm:2714) chicken.compiler.core#second" 
o|inlining procedure: k20645 
o|inlining procedure: k20645 
o|inlining procedure: k20651 
o|inlining procedure: k20651 
o|inlining procedure: k20671 
o|inlining procedure: k20671 
o|inlining procedure: "(core.scm:2700) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:2697) chicken.compiler.core#first" 
o|inlining procedure: k20686 
o|inlining procedure: k20686 
o|inlining procedure: k20734 
o|inlining procedure: k20734 
o|inlining procedure: "(core.scm:2690) chicken.compiler.core#third" 
o|inlining procedure: k20771 
o|inlining procedure: k20800 
o|inlining procedure: k20800 
o|inlining procedure: k20867 
o|inlining procedure: k20867 
o|inlining procedure: k20910 
o|inlining procedure: k20910 
o|inlining procedure: "(core.scm:2757) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:2756) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2754) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2753) chicken.compiler.core#first" 
o|inlining procedure: k20771 
o|inlining procedure: k20962 
o|inlining procedure: k20962 
o|inlining procedure: k20987 
o|inlining procedure: k20987 
o|inlining procedure: k20990 
o|inlining procedure: k20990 
o|substituted constant variable: a21000 
o|substituted constant variable: a21002 
o|substituted constant variable: a21004 
o|substituted constant variable: a21009 
o|substituted constant variable: a21011 
o|substituted constant variable: a21013 
o|inlining procedure: k21017 
o|inlining procedure: k21017 
o|inlining procedure: k21029 
o|inlining procedure: k21029 
o|inlining procedure: k21041 
o|inlining procedure: k21041 
o|inlining procedure: k21053 
o|inlining procedure: k21053 
o|inlining procedure: k21065 
o|inlining procedure: k21065 
o|inlining procedure: k21077 
o|inlining procedure: k21077 
o|inlining procedure: k21089 
o|inlining procedure: k21089 
o|inlining procedure: k21101 
o|inlining procedure: k21101 
o|substituted constant variable: a21114 
o|substituted constant variable: a21116 
o|substituted constant variable: a21118 
o|substituted constant variable: a21120 
o|substituted constant variable: a21122 
o|substituted constant variable: a21124 
o|substituted constant variable: a21126 
o|substituted constant variable: a21128 
o|substituted constant variable: a21130 
o|substituted constant variable: a21132 
o|substituted constant variable: a21134 
o|substituted constant variable: a21136 
o|substituted constant variable: a21138 
o|substituted constant variable: a21140 
o|substituted constant variable: a21142 
o|substituted constant variable: a21144 
o|substituted constant variable: a21146 
o|substituted constant variable: a21148 
o|inlining procedure: k21152 
o|inlining procedure: k21152 
o|substituted constant variable: a21165 
o|substituted constant variable: a21167 
o|substituted constant variable: a21169 
o|substituted constant variable: a21171 
o|substituted constant variable: a21173 
o|inlining procedure: k21177 
o|inlining procedure: k21177 
o|inlining procedure: k21189 
o|inlining procedure: k21189 
o|substituted constant variable: a21202 
o|substituted constant variable: a21204 
o|substituted constant variable: a21206 
o|substituted constant variable: a21208 
o|substituted constant variable: a21210 
o|substituted constant variable: a21212 
o|inlining procedure: k21233 
o|inlining procedure: k21233 
o|inlining procedure: k21273 
o|inlining procedure: k21273 
o|inlining procedure: "(core.scm:2799) chicken.compiler.core#first" 
o|inlining procedure: k21319 
o|inlining procedure: k21319 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#lambda-literal 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#lambda-literal? 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#lambda-literal-arguments 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#lambda-literal-directly-called 
o|inlining procedure: k21501 
o|inlining procedure: k21501 
o|inlining procedure: k21564 
o|inlining procedure: k21564 
o|propagated global variable: tmp49854987 chicken.compiler.core#no-bound-checks 
o|inlining procedure: k21585 
o|propagated global variable: tmp49854987 chicken.compiler.core#no-bound-checks 
o|inlining procedure: k21585 
o|propagated global variable: tmp49884990 chicken.compiler.support#unsafe 
o|propagated global variable: tmp49884990 chicken.compiler.support#unsafe 
o|inlining procedure: k21594 
o|inlining procedure: k21594 
o|inlining procedure: k21612 
o|inlining procedure: k21612 
o|inlining procedure: "(core.scm:2896) chicken.compiler.core#first" 
o|inlining procedure: k21634 
o|inlining procedure: "(core.scm:2900) chicken.compiler.core#fourth" 
o|inlining procedure: "(core.scm:2899) chicken.compiler.core#second" 
o|inlining procedure: k21634 
o|inlining procedure: "(core.scm:2913) chicken.compiler.core#second" 
o|inlining procedure: k21709 
o|inlining procedure: k21709 
o|inlining procedure: "(core.scm:2921) chicken.compiler.core#second" 
o|inlining procedure: k21751 
o|inlining procedure: "(core.scm:2925) chicken.compiler.core#first" 
o|inlining procedure: k21751 
o|inlining procedure: "(core.scm:2929) chicken.compiler.core#first" 
o|inlining procedure: k21801 
o|inlining procedure: "(core.scm:2934) chicken.compiler.core#first" 
o|inlining procedure: k21801 
o|inlining procedure: k21842 
o|inlining procedure: k21842 
o|inlining procedure: "(core.scm:2941) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2937) chicken.compiler.core#first" 
o|inlining procedure: k21868 
o|inlining procedure: "(core.scm:2950) chicken.compiler.core#first" 
o|inlining procedure: k21868 
o|inlining procedure: "(core.scm:3020) chicken.compiler.core#first" 
o|consed rest parameter at call site: "(core.scm:3019) chicken.compiler.core#lset-adjoin/eq?" 2 
o|contracted procedure: "(core.scm:2994) chicken.compiler.core#make-lambda-literal" 
o|inlining procedure: k21991 
o|inlining procedure: k21991 
o|inlining procedure: "(core.scm:2996) chicken.compiler.core#second" 
o|inlining procedure: k22024 
o|inlining procedure: k22024 
o|inlining procedure: k22041 
o|inlining procedure: k22041 
o|inlining procedure: k22053 
o|inlining procedure: k22053 
o|inlining procedure: k22074 
o|inlining procedure: k22074 
o|inlining procedure: "(core.scm:2969) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:2967) chicken.compiler.core#third" 
o|inlining procedure: k22084 
o|inlining procedure: "(core.scm:3030) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:3024) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:3023) chicken.compiler.core#first" 
o|inlining procedure: k22084 
o|inlining procedure: "(core.scm:3041) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:3036) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:3035) chicken.compiler.core#first" 
o|inlining procedure: k22185 
o|inlining procedure: k22270 
o|inlining procedure: k22270 
o|inlining procedure: k22284 
o|inlining procedure: k22284 
o|inlining procedure: "(core.scm:3056) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:3055) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:3045) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:3044) chicken.compiler.core#first" 
o|inlining procedure: k22185 
o|inlining procedure: "(core.scm:3073) chicken.compiler.core#third" 
o|substituted constant variable: a22397 
o|consed rest parameter at call site: "(core.scm:3072) chicken.compiler.core#lset-adjoin/eq?" 2 
o|inlining procedure: "(core.scm:3071) chicken.compiler.core#second" 
o|inlining procedure: k22409 
o|inlining procedure: "(core.scm:3086) chicken.compiler.core#first" 
o|inlining procedure: k22409 
o|inlining procedure: k22439 
o|inlining procedure: k22439 
o|inlining procedure: k22454 
o|substituted constant variable: a22481 
o|substituted constant variable: a22482 
o|inlining procedure: k22454 
o|inlining procedure: k22524 
o|inlining procedure: k22524 
o|inlining procedure: "(core.scm:3090) chicken.compiler.core#first" 
o|inlining procedure: k22554 
o|inlining procedure: "(core.scm:3113) chicken.compiler.core#third" 
o|inlining procedure: "(core.scm:3110) chicken.compiler.core#second" 
o|inlining procedure: "(core.scm:3107) chicken.compiler.core#first" 
o|inlining procedure: k22554 
o|inlining procedure: k22655 
o|inlining procedure: k22655 
o|consed rest parameter at call site: "(core.scm:3134) chicken.compiler.core#cons*" 2 
o|inlining procedure: "(core.scm:3126) chicken.compiler.core#first" 
o|inlining procedure: "(core.scm:3119) chicken.compiler.core#first" 
o|inlining procedure: k22720 
o|inlining procedure: k22720 
o|substituted constant variable: a22746 
o|substituted constant variable: a22748 
o|substituted constant variable: a22753 
o|substituted constant variable: a22755 
o|substituted constant variable: a22757 
o|substituted constant variable: a22759 
o|substituted constant variable: a22761 
o|substituted constant variable: a22763 
o|substituted constant variable: a22765 
o|substituted constant variable: a22767 
o|substituted constant variable: a22772 
o|substituted constant variable: a22774 
o|substituted constant variable: a22776 
o|substituted constant variable: a22778 
o|substituted constant variable: a22780 
o|substituted constant variable: a22782 
o|substituted constant variable: a22784 
o|substituted constant variable: a22786 
o|substituted constant variable: a22788 
o|substituted constant variable: a22790 
o|substituted constant variable: a22792 
o|substituted constant variable: a22794 
o|inlining procedure: k22798 
o|inlining procedure: k22798 
o|substituted constant variable: a22805 
o|substituted constant variable: a22807 
o|substituted constant variable: a22809 
o|inlining procedure: k22830 
o|inlining procedure: k22830 
o|inlining procedure: k22864 
o|inlining procedure: k22864 
o|inlining procedure: k22891 
o|inlining procedure: k22891 
o|contracted procedure: "(core.scm:3159) chicken.compiler.core#posv" 
o|inlining procedure: k7560 
o|inlining procedure: k7560 
o|inlining procedure: k22924 
o|inlining procedure: k22924 
o|inlining procedure: k22949 
o|inlining procedure: k22949 
o|contracted procedure: "(core.scm:3170) chicken.compiler.core#list-index" 
o|inlining procedure: k7108 
o|inlining procedure: k7108 
o|inlining procedure: k22965 
o|inlining procedure: k22965 
o|inlining procedure: k22978 
o|inlining procedure: k22978 
o|inlining procedure: k22996 
o|inlining procedure: k22996 
o|inlining procedure: k23011 
o|inlining procedure: k23011 
o|simplifications: ((if . 1)) 
o|replaced variables: 5157 
o|removed binding forms: 992 
o|substituted constant variable: r677623080 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#first 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#second 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#third 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#fourth 
o|substituted constant variable: r692723087 
o|contracted procedure: "(mini-srfi-1.scm:175) chicken.compiler.core#any" 
o|substituted constant variable: r671823076 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#list-tabulate 
o|substituted constant variable: r752723102 
o|substituted constant variable: r781023112 
o|inlining procedure: k7896 
o|substituted constant variable: r802623131 
o|removed call to pure procedure with unused result: "(core.scm:576) chicken.base#void" 
o|substituted constant variable: r847823167 
o|substituted constant variable: r847823167 
o|substituted constant variable: r856823176 
o|substituted constant variable: r887623187 
o|substituted constant variable: r887623187 
o|substituted constant variable: r887623189 
o|substituted constant variable: r887623189 
o|substituted constant variable: r971723226 
o|substituted constant variable: r971723226 
o|substituted constant variable: r1031023242 
o|substituted constant variable: r1031023242 
o|substituted constant variable: r1032923246 
o|substituted constant variable: r1032923246 
o|substituted constant variable: r1086223264 
o|substituted constant variable: r1086223264 
o|substituted constant variable: r1086223266 
o|substituted constant variable: r1086223266 
o|substituted constant variable: r1075923268 
o|substituted constant variable: r1075923268 
o|substituted constant variable: r1088223271 
o|substituted constant variable: r1105923281 
o|substituted constant variable: r1107123283 
o|substituted constant variable: r1126623297 
o|substituted constant variable: r1141523308 
o|substituted constant variable: r1141523308 
o|substituted constant variable: r1163323336 
o|substituted constant variable: r1163323336 
o|substituted constant variable: r1168623341 
o|substituted constant variable: r1168623341 
o|substituted constant variable: r1177023345 
o|substituted constant variable: r1177023345 
o|substituted constant variable: r1216723398 
o|substituted constant variable: r1227823423 
o|substituted constant variable: r1227823423 
o|substituted constant variable: r1222923430 
o|substituted constant variable: r1230823433 
o|substituted constant variable: r1230823433 
o|substituted constant variable: r1233323445 
o|substituted constant variable: r1239923448 
o|substituted constant variable: r1239923448 
o|substituted constant variable: r1257723490 
o|substituted constant variable: r1260223493 
o|substituted constant variable: r1260223493 
o|substituted constant variable: r1265523501 
o|substituted constant variable: r1270423505 
o|substituted constant variable: r1270423505 
o|substituted constant variable: r1273723509 
o|substituted constant variable: r1273723509 
o|substituted constant variable: r1372223526 
o|substituted constant variable: r1372223526 
o|substituted constant variable: r1373823530 
o|substituted constant variable: r697123535 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|substituted constant variable: r1491423638 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|substituted constant variable: r1501323646 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|substituted constant variable: r1530223663 
o|inlining procedure: k15334 
o|substituted constant variable: r1533523667 
o|substituted constant variable: r1533523667 
o|folded constant expression: (scheme#not (quote #f)) 
o|substituted constant variable: r1546123676 
o|inlining procedure: k13796 
o|inlining procedure: k13796 
o|substituted constant variable: n240 
o|substituted constant variable: r1307623694 
o|substituted constant variable: r1317123700 
o|substituted constant variable: r1317123700 
o|substituted constant variable: r1333323735 
o|substituted constant variable: r1333323735 
o|inlining procedure: k13317 
o|contracted procedure: "(core.scm:1854) g34193428" 
o|substituted constant variable: r1573723753 
o|substituted constant variable: r1579923758 
o|inlining procedure: k15804 
o|substituted constant variable: r1655623788 
o|substituted constant variable: r1744423849 
o|substituted constant variable: r1757923874 
o|substituted constant variable: r1757023875 
o|substituted constant variable: r1783223895 
o|substituted constant variable: r1801623926 
o|substituted constant variable: r1838223954 
o|substituted constant variable: r1842223961 
o|substituted constant variable: r1844923968 
o|substituted constant variable: r1874724010 
o|substituted constant variable: r1878424023 
o|substituted constant variable: r1890424063 
o|substituted constant variable: r1902524077 
o|substituted constant variable: r1902524078 
o|substituted constant variable: r1934424122 
o|substituted constant variable: r1933224123 
o|substituted constant variable: r1932324124 
o|substituted constant variable: r1937524135 
o|substituted constant variable: r1943024137 
o|inlining procedure: k19699 
o|substituted constant variable: r1973324183 
o|substituted constant variable: r1977124187 
o|substituted constant variable: r1976224188 
o|inlining procedure: k19699 
o|substituted constant variable: r1979824202 
o|substituted constant variable: r1970024208 
o|substituted constant variable: r1970024208 
o|substituted constant variable: r1969024210 
o|substituted constant variable: r1969024210 
o|substituted constant variable: r2064624318 
o|substituted constant variable: r2064624318 
o|substituted constant variable: r2064624320 
o|substituted constant variable: r2064624320 
o|substituted constant variable: r2065224323 
o|substituted constant variable: r2067224325 
o|substituted constant variable: r2096324375 
o|substituted constant variable: r2096324375 
o|substituted constant variable: r2098824377 
o|substituted constant variable: r2098824377 
o|substituted constant variable: r2098824379 
o|substituted constant variable: r2098824379 
o|propagated global variable: r2158624424 chicken.compiler.core#no-bound-checks 
o|substituted constant variable: r2184324478 
o|substituted constant variable: r2184324478 
o|substituted constant variable: r2199224504 
o|substituted constant variable: r2199224505 
o|substituted constant variable: r2205424519 
o|substituted constant variable: r756124660 
o|substituted constant variable: r2295024665 
o|substituted constant variable: r710924666 
o|inlining procedure: k22978 
o|inlining procedure: k22978 
o|substituted constant variable: r2301224676 
o|replaced variables: 746 
o|removed binding forms: 3972 
o|removed conditional forms: 1 
o|inlining procedure: k7284 
o|inlining procedure: k16466 
o|inlining procedure: "(core.scm:1925) chicken.compiler.core#alist-cons" 
o|inlining procedure: k8016 
o|inlining procedure: k8016 
o|inlining procedure: k8016 
o|contracted procedure: k8062 
o|inlining procedure: "(core.scm:713) chicken.compiler.core#alist-cons" 
o|inlining procedure: k8887 
o|inlining procedure: k8887 
o|inlining procedure: k10191 
o|inlining procedure: k10294 
o|inlining procedure: "(core.scm:934) chicken.compiler.core#alist-cons" 
o|inlining procedure: "(core.scm:1307) chicken.compiler.core#alist-cons" 
o|inlining procedure: "(core.scm:1302) chicken.compiler.core#fifth" 
o|inlining procedure: "(core.scm:1334) chicken.compiler.core#fifth" 
o|inlining procedure: "(core.scm:1320) chicken.compiler.core#alist-cons" 
o|inlining procedure: k13711 
o|inlining procedure: k14997 
o|inlining procedure: k14997 
o|substituted constant variable: r1533524931 
o|folded constant expression: (scheme#not (quote #f)) 
o|inlining procedure: k13117 
o|inlining procedure: k13117 
o|inlining procedure: "(core.scm:1507) chicken.compiler.core#alist-cons" 
o|inlining procedure: "(core.scm:1507) chicken.compiler.core#alist-cons" 
o|substituted constant variable: r157992375824965 
o|inlining procedure: k18003 
o|inlining procedure: "(core.scm:2282) chicken.compiler.core#alist-cons" 
o|inlining procedure: k18894 
o|inlining procedure: k18894 
o|inlining procedure: k18894 
o|inlining procedure: k18894 
o|inlining procedure: k19099 
o|inlining procedure: k19221 
o|inlining procedure: k19240 
o|inlining procedure: k19291 
o|inlining procedure: k19291 
o|inlining procedure: k19741 
o|inlining procedure: k20124 
o|inlining procedure: k20907 
o|inlining procedure: k22062 
o|inlining procedure: k22062 
o|inlining procedure: k22281 
o|replaced variables: 8 
o|removed binding forms: 831 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#fifth 
o|removed side-effect free assignment to unused variable: chicken.compiler.core#alist-cons 
o|substituted constant variable: r801725104 
o|substituted constant variable: r801725106 
o|substituted constant variable: r801725108 
o|substituted constant variable: r1019225147 
o|substituted constant variable: r1029525148 
o|inlining procedure: k12591 
o|substituted constant variable: z39325286 
o|contracted procedure: k18894 
o|substituted constant variable: r1889525336 
o|substituted constant variable: r1889525337 
o|substituted constant variable: r1924125344 
o|substituted constant variable: r1929225346 
o|substituted constant variable: r1970025016 
o|substituted constant variable: r1970025018 
o|substituted constant variable: r2090825371 
o|substituted constant variable: r2090825371 
o|substituted constant variable: r2206325376 
o|substituted constant variable: r2228225377 
o|substituted constant variable: r2228225377 
o|substituted constant variable: r2297925060 
o|substituted constant variable: r2297925062 
o|replaced variables: 44 
o|removed binding forms: 29 
o|removed conditional forms: 10 
o|replaced variables: 1 
o|removed binding forms: 64 
o|removed binding forms: 1 
o|simplifications: ((if . 105) (##core#call . 1860) (let . 100)) 
o|  call simplifications:
o|    chicken.base#void
o|    scheme#boolean?
o|    scheme#char?
o|    scheme#eof-object?
o|    chicken.base#fixnum?	2
o|    chicken.fixnum#fxmax	2
o|    chicken.fixnum#fx*	2
o|    scheme#=	4
o|    scheme#set-cdr!
o|    scheme#-	7
o|    ##sys#make-structure	2
o|    ##sys#check-structure	22
o|    ##sys#block-ref	22
o|    scheme#member	3
o|    chicken.fixnum#fx<=
o|    chicken.fixnum#fx-	6
o|    scheme#call-with-current-continuation
o|    chicken.fixnum#fx>=	2
o|    scheme#number?	3
o|    scheme#<
o|    scheme#>	3
o|    scheme#cddddr	3
o|    scheme#vector	2
o|    scheme#string?	5
o|    scheme#equal?	2
o|    scheme#set-car!	3
o|    scheme#list-ref	2
o|    scheme#cddr	20
o|    scheme#assoc
o|    scheme#list	102
o|    ##sys#cons	37
o|    scheme#cdddr	11
o|    scheme#cadddr	12
o|    scheme#cdr	31
o|    scheme#caddr	40
o|    ##sys#list	114
o|    scheme#list?	10
o|    scheme#cadr	103
o|    scheme#assq	18
o|    ##sys#call-with-values	9
o|    ##sys#apply	4
o|    scheme#caar	2
o|    scheme#symbol?	18
o|    scheme#length	15
o|    scheme#+	23
o|    scheme#values	3
o|    chicken.fixnum#fx+	9
o|    scheme#eq?	310
o|    scheme#memq	22
o|    ##sys#setslot	40
o|    ##sys#check-list	74
o|    scheme#pair?	115
o|    ##sys#slot	202
o|    scheme#cons	162
o|    scheme#null?	41
o|    scheme#car	142
o|    scheme#not	65
o|contracted procedure: k6683 
o|contracted procedure: k6692 
o|contracted procedure: k6705 
o|contracted procedure: k6750 
o|contracted procedure: k6761 
o|contracted procedure: k6778 
o|contracted procedure: k6785 
o|contracted procedure: k6806 
o|contracted procedure: k6832 
o|contracted procedure: k6917 
o|contracted procedure: k6929 
o|contracted procedure: k6947 
o|contracted procedure: k6955 
o|contracted procedure: k7026 
o|contracted procedure: k7034 
o|contracted procedure: k7046 
o|contracted procedure: k7068 
o|contracted procedure: k7064 
o|contracted procedure: k7049 
o|contracted procedure: k7052 
o|contracted procedure: k7060 
o|contracted procedure: k7145 
o|contracted procedure: k7171 
o|contracted procedure: k7151 
o|contracted procedure: k7165 
o|contracted procedure: k7186 
o|contracted procedure: k7193 
o|contracted procedure: k7216 
o|contracted procedure: k7209 
o|contracted procedure: k7231 
o|contracted procedure: k7238 
o|contracted procedure: k7295 
o|contracted procedure: k7244 
o|contracted procedure: k7256 
o|contracted procedure: k7263 
o|contracted procedure: k7288 
o|contracted procedure: k7284 
o|contracted procedure: k6720 
o|contracted procedure: k6735 
o|contracted procedure: k7529 
o|contracted procedure: k7548 
o|contracted procedure: k7535 
o|contracted procedure: k7542 
o|contracted procedure: k7710 
o|contracted procedure: k7713 
o|contracted procedure: k7719 
o|contracted procedure: k7723 
o|contracted procedure: k7729 
o|contracted procedure: k7733 
o|contracted procedure: k7739 
o|contracted procedure: k7743 
o|contracted procedure: k7755 
o|contracted procedure: k7763 
o|contracted procedure: k7759 
o|contracted procedure: k7774 
o|contracted procedure: k7786 
o|contracted procedure: k7796 
o|contracted procedure: k7800 
o|contracted procedure: k7812 
o|contracted procedure: k7838 
o|contracted procedure: k7828 
o|contracted procedure: k7912 
o|contracted procedure: k7893 
o|contracted procedure: k7902 
o|contracted procedure: k16431 
o|contracted procedure: k16492 
o|contracted procedure: k16447 
o|contracted procedure: k16488 
o|contracted procedure: k16456 
o|contracted procedure: k16463 
o|contracted procedure: k16474 
o|contracted procedure: k691125101 
o|contracted procedure: k16481 
o|contracted procedure: k7952 
o|contracted procedure: k7955 
o|contracted procedure: k7992 
o|contracted procedure: k7967 
o|contracted procedure: k7977 
o|contracted procedure: k7981 
o|contracted procedure: k7985 
o|contracted procedure: k7989 
o|contracted procedure: k8022 
o|contracted procedure: k8028 
o|contracted procedure: k8047 
o|contracted procedure: k8034 
o|contracted procedure: k8016 
o|contracted procedure: k8238 
o|contracted procedure: k8068 
o|contracted procedure: k8099 
o|contracted procedure: k8107 
o|contracted procedure: k8132 
o|contracted procedure: k8128 
o|contracted procedure: k8113 
o|contracted procedure: k8138 
o|contracted procedure: k8146 
o|contracted procedure: k8167 
o|contracted procedure: k8171 
o|contracted procedure: k8152 
o|contracted procedure: k8234 
o|contracted procedure: k8180 
o|contracted procedure: k8187 
o|contracted procedure: k8190 
o|contracted procedure: k8198 
o|contracted procedure: k8241 
o|contracted procedure: k8376 
o|contracted procedure: k13605 
o|contracted procedure: k8385 
o|contracted procedure: k13601 
o|contracted procedure: k8403 
o|contracted procedure: k8409 
o|contracted procedure: k13485 
o|contracted procedure: k8431 
o|contracted procedure: k8453 
o|contracted procedure: k8449 
o|contracted procedure: k8462 
o|contracted procedure: k8491 
o|contracted procedure: k8480 
o|contracted procedure: k8487 
o|contracted procedure: k8495 
o|contracted procedure: k8499 
o|contracted procedure: k8505 
o|contracted procedure: k8508 
o|contracted procedure: k8519 
o|contracted procedure: k8525 
o|contracted procedure: k8535 
o|contracted procedure: k8541 
o|contracted procedure: k8552 
o|contracted procedure: k8560 
o|contracted procedure: k8564 
o|contracted procedure: k8570 
o|contracted procedure: k8576 
o|contracted procedure: k8590 
o|contracted procedure: k8611 
o|contracted procedure: k8627 
o|contracted procedure: k8631 
o|contracted procedure: k8637 
o|contracted procedure: k8648 
o|contracted procedure: k8660 
o|contracted procedure: k8677 
o|contracted procedure: k8681 
o|contracted procedure: k8684 
o|contracted procedure: k8687 
o|contracted procedure: k8652 
o|contracted procedure: k8644 
o|contracted procedure: k8699 
o|contracted procedure: k8702 
o|contracted procedure: k8705 
o|contracted procedure: k8713 
o|contracted procedure: k8721 
o|contracted procedure: k8728 
o|contracted procedure: k8737 
o|contracted procedure: k8743 
o|contracted procedure: k691125118 
o|contracted procedure: k8756 
o|propagated global variable: z39325115 chicken.compiler.core#immutable-constants 
o|contracted procedure: k8768 
o|contracted procedure: k8777 
o|contracted procedure: k8780 
o|contracted procedure: k8793 
o|contracted procedure: k8800 
o|contracted procedure: k8818 
o|contracted procedure: k8833 
o|contracted procedure: k8837 
o|contracted procedure: k8843 
o|contracted procedure: k8850 
o|contracted procedure: k8856 
o|contracted procedure: k8859 
o|contracted procedure: k8897 
o|contracted procedure: k8862 
o|contracted procedure: k8878 
o|contracted procedure: k8881 
o|contracted procedure: k8909 
o|contracted procedure: k8912 
o|contracted procedure: k8918 
o|contracted procedure: k8921 
o|contracted procedure: k8933 
o|contracted procedure: k8943 
o|contracted procedure: k8956 
o|contracted procedure: k8961 
o|contracted procedure: k8964 
o|contracted procedure: k9006 
o|contracted procedure: k9061 
o|contracted procedure: k9025 
o|contracted procedure: k9028 
o|contracted procedure: k9031 
o|contracted procedure: k9039 
o|contracted procedure: k9043 
o|contracted procedure: k9051 
o|contracted procedure: k9055 
o|contracted procedure: k9077 
o|contracted procedure: k9080 
o|contracted procedure: k9083 
o|contracted procedure: k9091 
o|contracted procedure: k9099 
o|contracted procedure: k9108 
o|contracted procedure: k9111 
o|contracted procedure: k9128 
o|contracted procedure: k9140 
o|contracted procedure: k9124 
o|contracted procedure: k9120 
o|contracted procedure: k9151 
o|contracted procedure: k9179 
o|contracted procedure: k9175 
o|contracted procedure: k9171 
o|contracted procedure: k9188 
o|contracted procedure: k9210 
o|contracted procedure: k9160 
o|contracted procedure: k9164 
o|contracted procedure: k9206 
o|contracted procedure: k9191 
o|contracted procedure: k9194 
o|contracted procedure: k9202 
o|contracted procedure: k9222 
o|contracted procedure: k9244 
o|contracted procedure: k9137 
o|contracted procedure: k9240 
o|contracted procedure: k9225 
o|contracted procedure: k9228 
o|contracted procedure: k9236 
o|contracted procedure: k9253 
o|contracted procedure: k9256 
o|contracted procedure: k9262 
o|contracted procedure: k9265 
o|contracted procedure: k9271 
o|contracted procedure: k9282 
o|contracted procedure: k9294 
o|contracted procedure: k9309 
o|contracted procedure: k9322 
o|contracted procedure: k9318 
o|contracted procedure: k9325 
o|contracted procedure: k9305 
o|contracted procedure: k9301 
o|contracted procedure: k9278 
o|contracted procedure: k9336 
o|contracted procedure: k9356 
o|contracted procedure: k9352 
o|contracted procedure: k9348 
o|contracted procedure: k9401 
o|contracted procedure: k9365 
o|contracted procedure: k9391 
o|contracted procedure: k9395 
o|contracted procedure: k9387 
o|contracted procedure: k9368 
o|contracted procedure: k9371 
o|contracted procedure: k9379 
o|contracted procedure: k9383 
o|contracted procedure: k9449 
o|contracted procedure: k9413 
o|contracted procedure: k9416 
o|contracted procedure: k9419 
o|contracted procedure: k9427 
o|contracted procedure: k9431 
o|contracted procedure: k9439 
o|contracted procedure: k9443 
o|contracted procedure: k9461 
o|contracted procedure: k9483 
o|contracted procedure: k9291 
o|contracted procedure: k9479 
o|contracted procedure: k9464 
o|contracted procedure: k9467 
o|contracted procedure: k9475 
o|contracted procedure: k9495 
o|contracted procedure: k9498 
o|contracted procedure: k9501 
o|contracted procedure: k9509 
o|contracted procedure: k9517 
o|contracted procedure: k9526 
o|contracted procedure: k9529 
o|contracted procedure: k9532 
o|contracted procedure: k9546 
o|contracted procedure: k9570 
o|contracted procedure: k9584 
o|contracted procedure: k9587 
o|contracted procedure: k9590 
o|contracted procedure: k9598 
o|contracted procedure: k9606 
o|contracted procedure: k9615 
o|contracted procedure: k9618 
o|contracted procedure: k9632 
o|contracted procedure: k9635 
o|contracted procedure: k9644 
o|contracted procedure: k9656 
o|contracted procedure: k9665 
o|contracted procedure: k9684 
o|contracted procedure: k9690 
o|contracted procedure: k9699 
o|contracted procedure: k9716 
o|contracted procedure: k9779 
o|contracted procedure: k9775 
o|contracted procedure: k9807 
o|contracted procedure: k9810 
o|contracted procedure: k9813 
o|contracted procedure: k9821 
o|contracted procedure: k9829 
o|contracted procedure: k9859 
o|contracted procedure: k9891 
o|contracted procedure: k9902 
o|contracted procedure: k9916 
o|contracted procedure: k9946 
o|contracted procedure: k9949 
o|contracted procedure: k9965 
o|contracted procedure: k9987 
o|contracted procedure: k9925 
o|contracted procedure: k9983 
o|contracted procedure: k9968 
o|contracted procedure: k9971 
o|contracted procedure: k9979 
o|contracted procedure: k9943 
o|contracted procedure: k9996 
o|contracted procedure: k9999 
o|contracted procedure: k10025 
o|contracted procedure: k10028 
o|contracted procedure: k10040 
o|contracted procedure: k10049 
o|contracted procedure: k10052 
o|contracted procedure: k10089 
o|contracted procedure: k10108 
o|contracted procedure: k10118 
o|contracted procedure: k10122 
o|contracted procedure: k10135 
o|contracted procedure: k10157 
o|contracted procedure: k10008 
o|contracted procedure: k10153 
o|contracted procedure: k10138 
o|contracted procedure: k10141 
o|contracted procedure: k10149 
o|contracted procedure: k10022 
o|contracted procedure: k10166 
o|contracted procedure: k10198 
o|contracted procedure: k10202 
o|contracted procedure: k10191 
o|contracted procedure: k10217 
o|contracted procedure: k10224 
o|contracted procedure: k10249 
o|contracted procedure: k10240 
o|contracted procedure: k10266 
o|contracted procedure: k10262 
o|contracted procedure: k10255 
o|contracted procedure: k10272 
o|contracted procedure: k10275 
o|contracted procedure: k10278 
o|contracted procedure: k10301 
o|contracted procedure: k10305 
o|contracted procedure: k10324 
o|contracted procedure: k10316 
o|contracted procedure: k10320 
o|contracted procedure: k10309 
o|inlining procedure: k10294 
o|contracted procedure: k10328 
o|contracted procedure: k691125155 
o|contracted procedure: k10347 
o|contracted procedure: k10357 
o|contracted procedure: k10360 
o|contracted procedure: k10382 
o|contracted procedure: k10403 
o|contracted procedure: k10409 
o|contracted procedure: k10412 
o|contracted procedure: k10415 
o|contracted procedure: k10424 
o|contracted procedure: k10446 
o|contracted procedure: k10458 
o|contracted procedure: k10468 
o|contracted procedure: k10472 
o|contracted procedure: k10439 
o|contracted procedure: k10443 
o|contracted procedure: k10486 
o|contracted procedure: k10505 
o|contracted procedure: k10517 
o|contracted procedure: k10527 
o|contracted procedure: k10531 
o|contracted procedure: k10498 
o|contracted procedure: k10502 
o|contracted procedure: k10540 
o|contracted procedure: k10543 
o|contracted procedure: k10546 
o|contracted procedure: k10554 
o|contracted procedure: k10562 
o|contracted procedure: k10571 
o|contracted procedure: k10589 
o|contracted procedure: k10593 
o|contracted procedure: k10624 
o|contracted procedure: k10606 
o|contracted procedure: k10617 
o|inlining procedure: k10603 
o|contracted procedure: k10635 
o|contracted procedure: k10642 
o|contracted procedure: k10653 
o|contracted procedure: k10656 
o|contracted procedure: k10673 
o|contracted procedure: k10669 
o|contracted procedure: k10682 
o|contracted procedure: k10685 
o|contracted procedure: k10688 
o|contracted procedure: k10696 
o|contracted procedure: k10704 
o|contracted procedure: k10713 
o|contracted procedure: k10719 
o|contracted procedure: k10722 
o|contracted procedure: k10891 
o|contracted procedure: k10754 
o|contracted procedure: k10761 
o|contracted procedure: k10809 
o|contracted procedure: k10820 
o|contracted procedure: k10832 
o|contracted procedure: k10835 
o|contracted procedure: k10838 
o|contracted procedure: k10846 
o|contracted procedure: k10854 
o|contracted procedure: k10884 
o|contracted procedure: k10904 
o|contracted procedure: k8341 
o|contracted procedure: k8281 
o|contracted procedure: k8302 
o|contracted procedure: k8317 
o|contracted procedure: k8327 
o|contracted procedure: k8331 
o|contracted procedure: k10917 
o|contracted procedure: k10930 
o|contracted procedure: k11005 
o|contracted procedure: k11014 
o|contracted procedure: k11061 
o|contracted procedure: k11104 
o|contracted procedure: k11108 
o|contracted procedure: k11259 
o|contracted procedure: k11155 
o|contracted procedure: k11161 
o|contracted procedure: k11169 
o|contracted procedure: k11181 
o|contracted procedure: k11190 
o|contracted procedure: k11208 
o|contracted procedure: k11199 
o|contracted procedure: k11214 
o|contracted procedure: k11226 
o|contracted procedure: k11229 
o|contracted procedure: k11232 
o|contracted procedure: k11240 
o|contracted procedure: k11248 
o|contracted procedure: k11262 
o|contracted procedure: k11272 
o|contracted procedure: k11278 
o|contracted procedure: k11281 
o|contracted procedure: k11286 
o|contracted procedure: k11289 
o|contracted procedure: k11301 
o|contracted procedure: k11364 
o|contracted procedure: k11367 
o|contracted procedure: k11370 
o|contracted procedure: k11378 
o|contracted procedure: k11386 
o|contracted procedure: k11395 
o|contracted procedure: k11401 
o|contracted procedure: k11438 
o|contracted procedure: k11441 
o|contracted procedure: k11450 
o|contracted procedure: k11456 
o|contracted procedure: k11464 
o|contracted procedure: k11498 
o|contracted procedure: k11478 
o|contracted procedure: k11494 
o|contracted procedure: k11486 
o|contracted procedure: k11482 
o|contracted procedure: k11474 
o|contracted procedure: k11508 
o|contracted procedure: k11516 
o|contracted procedure: k11550 
o|contracted procedure: k11530 
o|contracted procedure: k11538 
o|contracted procedure: k11542 
o|contracted procedure: k11534 
o|contracted procedure: k11526 
o|contracted procedure: k11571 
o|contracted procedure: k11574 
o|contracted procedure: k11656 
o|contracted procedure: k11731 
o|contracted procedure: k11719 
o|contracted procedure: k11727 
o|contracted procedure: k11723 
o|contracted procedure: k11715 
o|contracted procedure: k11740 
o|contracted procedure: k11793 
o|contracted procedure: k11804 
o|contracted procedure: k11812 
o|contracted procedure: k11826 
o|contracted procedure: k11808 
o|contracted procedure: k11800 
o|contracted procedure: k11838 
o|contracted procedure: k11841 
o|contracted procedure: k11844 
o|contracted procedure: k11852 
o|contracted procedure: k11860 
o|contracted procedure: k11869 
o|contracted procedure: k11876 
o|contracted procedure: k11888 
o|contracted procedure: k11892 
o|contracted procedure: k11898 
o|contracted procedure: k11909 
o|contracted procedure: k11912 
o|contracted procedure: k11915 
o|contracted procedure: k11905 
o|contracted procedure: k11926 
o|contracted procedure: k11935 
o|contracted procedure: k11938 
o|contracted procedure: k11941 
o|contracted procedure: k11949 
o|contracted procedure: k11957 
o|contracted procedure: k11966 
o|contracted procedure: k11973 
o|contracted procedure: k11981 
o|contracted procedure: k11987 
o|contracted procedure: k11994 
o|contracted procedure: k12006 
o|contracted procedure: k12010 
o|contracted procedure: k12016 
o|contracted procedure: k12019 
o|contracted procedure: k12022 
o|contracted procedure: k12034 
o|contracted procedure: k12037 
o|contracted procedure: k12044 
o|contracted procedure: k12050 
o|contracted procedure: k12056 
o|contracted procedure: k12071 
o|contracted procedure: k12078 
o|contracted procedure: k12104 
o|contracted procedure: k12117 
o|contracted procedure: k12130 
o|contracted procedure: k12143 
o|contracted procedure: k12156 
o|contracted procedure: k16269 
o|contracted procedure: k16278 
o|contracted procedure: k16286 
o|contracted procedure: k16292 
o|contracted procedure: k16298 
o|contracted procedure: k16313 
o|contracted procedure: k16335 
o|contracted procedure: k16331 
o|contracted procedure: k16316 
o|contracted procedure: k16319 
o|contracted procedure: k16327 
o|contracted procedure: k16347 
o|contracted procedure: k16369 
o|contracted procedure: k16365 
o|contracted procedure: k16350 
o|contracted procedure: k16353 
o|contracted procedure: k16361 
o|contracted procedure: k16376 
o|contracted procedure: k16386 
o|contracted procedure: k16413 
o|contracted procedure: k16398 
o|contracted procedure: k16409 
o|contracted procedure: k16405 
o|contracted procedure: k12169 
o|contracted procedure: k12186 
o|contracted procedure: k12182 
o|contracted procedure: k12199 
o|contracted procedure: k12189 
o|contracted procedure: k12203 
o|contracted procedure: k12207 
o|contracted procedure: k12213 
o|contracted procedure: k12216 
o|contracted procedure: k12222 
o|contracted procedure: k12231 
o|contracted procedure: k12289 
o|contracted procedure: k12269 
o|contracted procedure: k12265 
o|contracted procedure: k12280 
o|contracted procedure: k12277 
o|contracted procedure: k12329 
o|contracted procedure: k12335 
o|contracted procedure: k12338 
o|contracted procedure: k12347 
o|contracted procedure: k12350 
o|contracted procedure: k12383 
o|contracted procedure: k12363 
o|contracted procedure: k12375 
o|contracted procedure: k12367 
o|contracted procedure: k691125194 
o|contracted procedure: k12371 
o|propagated global variable: z39325191 chicken.compiler.core#external-to-pointer 
o|contracted procedure: k686625199 
o|contracted procedure: k12391 
o|contracted procedure: k12422 
o|contracted procedure: k12425 
o|contracted procedure: k12569 
o|contracted procedure: k12562 
o|contracted procedure: k12437 
o|contracted procedure: k12559 
o|contracted procedure: k12444 
o|contracted procedure: k691125211 
o|contracted procedure: k12447 
o|contracted procedure: k12537 
o|contracted procedure: k12529 
o|contracted procedure: k12533 
o|contracted procedure: k12525 
o|contracted procedure: k12521 
o|contracted procedure: k12481 
o|contracted procedure: k12485 
o|contracted procedure: k12477 
o|contracted procedure: k12497 
o|contracted procedure: k686625204 
o|contracted procedure: k12514 
o|contracted procedure: k12573 
o|contracted procedure: k12579 
o|contracted procedure: k12582 
o|contracted procedure: k12585 
o|contracted procedure: k12629 
o|contracted procedure: k12632 
o|contracted procedure: k12638 
o|contracted procedure: k12729 
o|contracted procedure: k12725 
o|contracted procedure: k12651 
o|contracted procedure: k12664 
o|contracted procedure: k12696 
o|contracted procedure: k12692 
o|contracted procedure: k12794 
o|contracted procedure: k12804 
o|contracted procedure: k12797 
o|contracted procedure: k12822 
o|contracted procedure: k12833 
o|contracted procedure: k13731 
o|contracted procedure: k13702 
o|contracted procedure: k13708 
o|contracted procedure: k13724 
o|inlining procedure: k13711 
o|contracted procedure: k13721 
o|inlining procedure: k13711 
o|contracted procedure: k6961 
o|contracted procedure: k6973 
o|contracted procedure: k6996 
o|contracted procedure: k7004 
o|removed unused formal parameters: (return2712) 
o|contracted procedure: k13799 
o|contracted procedure: k13818 
o|contracted procedure: k13842 
o|contracted procedure: k13846 
o|contracted procedure: k13852 
o|contracted procedure: k13858 
o|contracted procedure: k13877 
o|contracted procedure: k13883 
o|contracted procedure: k13902 
o|contracted procedure: k13908 
o|contracted procedure: k13931 
o|contracted procedure: k13942 
o|contracted procedure: k13948 
o|contracted procedure: k13951 
o|contracted procedure: k13958 
o|contracted procedure: k13965 
o|contracted procedure: k13972 
o|contracted procedure: k13979 
o|contracted procedure: k13986 
o|contracted procedure: k13993 
o|contracted procedure: k14000 
o|contracted procedure: k14007 
o|contracted procedure: k14016 
o|contracted procedure: k14028 
o|contracted procedure: k14038 
o|contracted procedure: k14042 
o|contracted procedure: k14048 
o|contracted procedure: k14055 
o|contracted procedure: k14066 
o|contracted procedure: k14080 
o|contracted procedure: k14092 
o|contracted procedure: k14102 
o|contracted procedure: k14106 
o|contracted procedure: k14115 
o|contracted procedure: k14125 
o|contracted procedure: k14129 
o|contracted procedure: k14135 
o|contracted procedure: k14142 
o|contracted procedure: k14153 
o|contracted procedure: k14173 
o|contracted procedure: k14183 
o|contracted procedure: k14187 
o|contracted procedure: k14196 
o|contracted procedure: k14206 
o|contracted procedure: k14210 
o|contracted procedure: k14218 
o|contracted procedure: k14238 
o|contracted procedure: k14245 
o|contracted procedure: k14252 
o|contracted procedure: k14259 
o|contracted procedure: k14270 
o|contracted procedure: k14282 
o|contracted procedure: k14292 
o|contracted procedure: k14296 
o|contracted procedure: k14304 
o|contracted procedure: k14315 
o|contracted procedure: k14327 
o|contracted procedure: k14337 
o|contracted procedure: k14341 
o|contracted procedure: k14349 
o|contracted procedure: k14361 
o|contracted procedure: k14383 
o|contracted procedure: k14367 
o|contracted procedure: k14389 
o|contracted procedure: k14411 
o|contracted procedure: k14395 
o|contracted procedure: k14417 
o|contracted procedure: k14468 
o|contracted procedure: k14423 
o|contracted procedure: k14435 
o|contracted procedure: k14447 
o|contracted procedure: k14457 
o|contracted procedure: k14461 
o|contracted procedure: k14474 
o|contracted procedure: k14500 
o|contracted procedure: k14480 
o|contracted procedure: k14506 
o|contracted procedure: k14557 
o|contracted procedure: k14513 
o|contracted procedure: k14524 
o|contracted procedure: k14536 
o|contracted procedure: k14546 
o|contracted procedure: k14550 
o|contracted procedure: k14569 
o|contracted procedure: k14579 
o|contracted procedure: k14593 
o|contracted procedure: k14599 
o|contracted procedure: k14606 
o|contracted procedure: k14609 
o|contracted procedure: k14618 
o|contracted procedure: k14622 
o|contracted procedure: k14634 
o|contracted procedure: k14644 
o|contracted procedure: k14648 
o|contracted procedure: k14656 
o|contracted procedure: k14663 
o|contracted procedure: k14675 
o|contracted procedure: k14685 
o|contracted procedure: k14689 
o|contracted procedure: k14697 
o|contracted procedure: k14704 
o|contracted procedure: k14710 
o|contracted procedure: k14722 
o|contracted procedure: k14734 
o|contracted procedure: k14744 
o|contracted procedure: k14748 
o|contracted procedure: k14758 
o|contracted procedure: k14764 
o|contracted procedure: k14770 
o|contracted procedure: k14780 
o|contracted procedure: k14786 
o|contracted procedure: k14792 
o|contracted procedure: k14802 
o|contracted procedure: k14822 
o|contracted procedure: k14834 
o|contracted procedure: k14844 
o|contracted procedure: k14848 
o|contracted procedure: k14859 
o|contracted procedure: k14867 
o|contracted procedure: k14942 
o|contracted procedure: k14954 
o|contracted procedure: k14957 
o|contracted procedure: k14960 
o|contracted procedure: k14968 
o|contracted procedure: k14976 
o|contracted procedure: k14875 
o|contracted procedure: k14900 
o|contracted procedure: k14904 
o|contracted procedure: k14910 
o|contracted procedure: k14916 
o|contracted procedure: k14933 
o|contracted procedure: k14922 
o|contracted procedure: k14929 
o|contracted procedure: k14987 
o|contracted procedure: k14994 
o|contracted procedure: k15024 
o|contracted procedure: k15009 
o|contracted procedure: k15015 
o|contracted procedure: k15032 
o|contracted procedure: k15039 
o|contracted procedure: k15052 
o|contracted procedure: k15064 
o|contracted procedure: k15074 
o|contracted procedure: k15078 
o|contracted procedure: k15088 
o|contracted procedure: k15094 
o|contracted procedure: k15106 
o|contracted procedure: k15118 
o|contracted procedure: k15128 
o|contracted procedure: k15132 
o|contracted procedure: k15142 
o|contracted procedure: k15150 
o|contracted procedure: k15161 
o|contracted procedure: k15173 
o|contracted procedure: k15183 
o|contracted procedure: k15187 
o|contracted procedure: k15197 
o|contracted procedure: k15279 
o|contracted procedure: k15265 
o|contracted procedure: k15295 
o|contracted procedure: k15298 
o|contracted procedure: k15304 
o|contracted procedure: k15311 
o|contracted procedure: k15326 
o|contracted procedure: k15330 
o|contracted procedure: k15337 
o|contracted procedure: k15343 
o|contracted procedure: k15350 
o|contracted procedure: k15334 
o|contracted procedure: k15357 
o|contracted procedure: k15369 
o|contracted procedure: k15379 
o|contracted procedure: k15383 
o|contracted procedure: k15389 
o|contracted procedure: k15432 
o|contracted procedure: k15447 
o|contracted procedure: k15451 
o|contracted procedure: k15457 
o|contracted procedure: k15475 
o|contracted procedure: k15463 
o|contracted procedure: k15471 
o|contracted procedure: k15479 
o|contracted procedure: k15491 
o|contracted procedure: k15501 
o|contracted procedure: k15505 
o|contracted procedure: k15511 
o|contracted procedure: k15518 
o|contracted procedure: k15623 
o|removed unused parameter to known procedure: return2712 a13787 
o|contracted procedure: k12852 
o|contracted procedure: k12829 
o|contracted procedure: k12864 
o|contracted procedure: k12867 
o|contracted procedure: k12870 
o|contracted procedure: k12878 
o|contracted procedure: k12886 
o|contracted procedure: k12895 
o|contracted procedure: k12908 
o|contracted procedure: k6477 
o|contracted procedure: k6491 
o|contracted procedure: k6501 
o|contracted procedure: k6495 
o|contracted procedure: k12914 
o|contracted procedure: k13200 
o|contracted procedure: k12917 
o|contracted procedure: k13196 
o|contracted procedure: k12923 
o|contracted procedure: k13192 
o|contracted procedure: k12926 
o|contracted procedure: k12929 
o|contracted procedure: k12950 
o|contracted procedure: k12962 
o|contracted procedure: k12958 
o|contracted procedure: k12978 
o|contracted procedure: k12974 
o|contracted procedure: k12985 
o|contracted procedure: k13008 
o|contracted procedure: k13004 
o|contracted procedure: k12996 
o|contracted procedure: k13000 
o|contracted procedure: k12992 
o|contracted procedure: k13014 
o|contracted procedure: k13023 
o|contracted procedure: k13066 
o|contracted procedure: k13062 
o|contracted procedure: k13058 
o|contracted procedure: k13054 
o|contracted procedure: k13034 
o|contracted procedure: k13050 
o|contracted procedure: k13046 
o|contracted procedure: k13038 
o|contracted procedure: k13030 
o|contracted procedure: k13078 
o|contracted procedure: k13081 
o|contracted procedure: k13084 
o|contracted procedure: k13091 
o|contracted procedure: k13149 
o|contracted procedure: k13114 
o|contracted procedure: k13145 
o|contracted procedure: k13123 
o|contracted procedure: k13137 
o|contracted procedure: k13141 
o|contracted procedure: k13133 
o|contracted procedure: k13117 
o|contracted procedure: k13160 
o|contracted procedure: k13156 
o|contracted procedure: k13206 
o|contracted procedure: k13209 
o|contracted procedure: k13215 
o|contracted procedure: k13218 
o|contracted procedure: k13234 
o|contracted procedure: k13238 
o|contracted procedure: k13230 
o|contracted procedure: k13244 
o|contracted procedure: k13256 
o|contracted procedure: k13265 
o|contracted procedure: k13272 
o|contracted procedure: k13287 
o|contracted procedure: k13283 
o|contracted procedure: k13302 
o|contracted procedure: k13298 
o|contracted procedure: k13305 
o|contracted procedure: k13311 
o|contracted procedure: k13324 
o|contracted procedure: k13332 
o|contracted procedure: k691125282 
o|contracted procedure: k691125289 
o|contracted procedure: k13450 
o|contracted procedure: k13510 
o|contracted procedure: k13597 
o|contracted procedure: k13544 
o|contracted procedure: k13589 
o|contracted procedure: k13579 
o|contracted procedure: k13583 
o|contracted procedure: k13575 
o|contracted procedure: k13611 
o|contracted procedure: k13619 
o|contracted procedure: k13631 
o|contracted procedure: k13634 
o|contracted procedure: k13637 
o|contracted procedure: k13645 
o|contracted procedure: k13653 
o|contracted procedure: k13666 
o|contracted procedure: k13679 
o|contracted procedure: k13687 
o|contracted procedure: k15636 
o|contracted procedure: k15655 
o|contracted procedure: k15664 
o|contracted procedure: k15673 
o|contracted procedure: k15682 
o|contracted procedure: k15691 
o|contracted procedure: k15700 
o|contracted procedure: k15709 
o|contracted procedure: k15718 
o|contracted procedure: k16008 
o|contracted procedure: k15832 
o|contracted procedure: k15835 
o|contracted procedure: k15841 
o|contracted procedure: k15853 
o|contracted procedure: k15856 
o|contracted procedure: k15866 
o|contracted procedure: k15874 
o|contracted procedure: k15878 
o|contracted procedure: k15894 
o|contracted procedure: k15943 
o|contracted procedure: k15935 
o|contracted procedure: k15939 
o|contracted procedure: k15931 
o|contracted procedure: k15927 
o|contracted procedure: k15907 
o|contracted procedure: k15923 
o|contracted procedure: k15988 
o|contracted procedure: k15952 
o|contracted procedure: k15955 
o|contracted procedure: k15958 
o|contracted procedure: k15966 
o|contracted procedure: k15970 
o|contracted procedure: k15978 
o|contracted procedure: k15982 
o|contracted procedure: k15998 
o|contracted procedure: k16018 
o|contracted procedure: k16021 
o|contracted procedure: k16033 
o|contracted procedure: k16036 
o|contracted procedure: k16039 
o|contracted procedure: k16047 
o|contracted procedure: k16055 
o|contracted procedure: k16061 
o|contracted procedure: k16073 
o|contracted procedure: k16085 
o|contracted procedure: k16088 
o|contracted procedure: k16091 
o|contracted procedure: k16099 
o|contracted procedure: k16107 
o|contracted procedure: k15739 
o|contracted procedure: k15745 
o|contracted procedure: k15748 
o|contracted procedure: k15754 
o|contracted procedure: k15773 
o|contracted procedure: k15776 
o|contracted procedure: k15801 
o|contracted procedure: k15804 
o|contracted procedure: k16116 
o|contracted procedure: k16122 
o|contracted procedure: k16125 
o|contracted procedure: k16131 
o|contracted procedure: k16144 
o|contracted procedure: k16153 
o|contracted procedure: k16156 
o|contracted procedure: k16162 
o|contracted procedure: k16170 
o|contracted procedure: k16176 
o|contracted procedure: k16182 
o|contracted procedure: k16197 
o|contracted procedure: k16219 
o|contracted procedure: k16215 
o|contracted procedure: k16200 
o|contracted procedure: k16203 
o|contracted procedure: k16211 
o|contracted procedure: k16231 
o|contracted procedure: k16253 
o|contracted procedure: k16249 
o|contracted procedure: k16234 
o|contracted procedure: k16237 
o|contracted procedure: k16245 
o|contracted procedure: k16260 
o|contracted procedure: k16549 
o|contracted procedure: k16515 
o|contracted procedure: k16519 
o|contracted procedure: k16527 
o|contracted procedure: k16537 
o|contracted procedure: k16541 
o|contracted procedure: k16558 
o|contracted procedure: k16565 
o|contracted procedure: k16594 
o|contracted procedure: k16606 
o|contracted procedure: k16621 
o|contracted procedure: k16625 
o|contracted procedure: k16636 
o|contracted procedure: k16640 
o|contracted procedure: k16652 
o|contracted procedure: k16662 
o|contracted procedure: k16674 
o|contracted procedure: k16678 
o|contracted procedure: k16707 
o|contracted procedure: k16682 
o|contracted procedure: k16686 
o|contracted procedure: k16713 
o|contracted procedure: k16725 
o|contracted procedure: k16732 
o|contracted procedure: k16739 
o|contracted procedure: k16757 
o|contracted procedure: k16764 
o|contracted procedure: k16768 
o|contracted procedure: k16778 
o|contracted procedure: k16784 
o|contracted procedure: k16790 
o|contracted procedure: k16793 
o|contracted procedure: k16809 
o|contracted procedure: k16815 
o|contracted procedure: k16875 
o|contracted procedure: k16868 
o|contracted procedure: k16821 
o|contracted procedure: k16865 
o|contracted procedure: k16824 
o|contracted procedure: k16831 
o|contracted procedure: k16841 
o|contracted procedure: k16845 
o|contracted procedure: k16861 
o|contracted procedure: k16881 
o|contracted procedure: k16892 
o|contracted procedure: k16902 
o|contracted procedure: k16916 
o|contracted procedure: k16928 
o|contracted procedure: k16935 
o|contracted procedure: k16943 
o|contracted procedure: k16946 
o|contracted procedure: k16952 
o|contracted procedure: k16962 
o|contracted procedure: k17212 
o|contracted procedure: k17216 
o|contracted procedure: k17228 
o|contracted procedure: k17232 
o|contracted procedure: k17265 
o|contracted procedure: k17240 
o|contracted procedure: k17244 
o|contracted procedure: k16987 
o|contracted procedure: k16990 
o|contracted procedure: k16997 
o|contracted procedure: k17003 
o|contracted procedure: k7096 
o|contracted procedure: k7086 
o|contracted procedure: k17029 
o|contracted procedure: k17035 
o|contracted procedure: k17041 
o|contracted procedure: k17047 
o|contracted procedure: k17053 
o|contracted procedure: k17091 
o|contracted procedure: k17097 
o|contracted procedure: k17103 
o|contracted procedure: k17135 
o|contracted procedure: k17139 
o|contracted procedure: k17196 
o|contracted procedure: k17171 
o|contracted procedure: k17175 
o|contracted procedure: k17296 
o|contracted procedure: k17318 
o|contracted procedure: k17344 
o|contracted procedure: k17348 
o|contracted procedure: k17359 
o|contracted procedure: k17363 
o|contracted procedure: k17371 
o|contracted procedure: k17375 
o|contracted procedure: k17383 
o|contracted procedure: k19465 
o|contracted procedure: k17415 
o|contracted procedure: k17425 
o|contracted procedure: k17446 
o|contracted procedure: k17455 
o|contracted procedure: k17458 
o|contracted procedure: k17461 
o|contracted procedure: k17470 
o|contracted procedure: k17479 
o|contracted procedure: k17488 
o|contracted procedure: k17501 
o|contracted procedure: k17504 
o|contracted procedure: k17516 
o|contracted procedure: k17522 
o|contracted procedure: k17537 
o|contracted procedure: k17540 
o|contracted procedure: k17543 
o|contracted procedure: k17556 
o|contracted procedure: k17575 
o|contracted procedure: k17581 
o|contracted procedure: k17587 
o|contracted procedure: k17598 
o|contracted procedure: k17630 
o|contracted procedure: k17633 
o|contracted procedure: k17648 
o|contracted procedure: k17655 
o|contracted procedure: k17662 
o|contracted procedure: k17665 
o|contracted procedure: k17687 
o|contracted procedure: k17697 
o|contracted procedure: k17708 
o|contracted procedure: k17720 
o|contracted procedure: k17733 
o|contracted procedure: k17743 
o|contracted procedure: k17747 
o|contracted procedure: k17753 
o|contracted procedure: k17756 
o|contracted procedure: k17766 
o|contracted procedure: k17772 
o|contracted procedure: k17786 
o|contracted procedure: k17812 
o|contracted procedure: k17816 
o|contracted procedure: k17824 
o|contracted procedure: k17835 
o|contracted procedure: k17842 
o|contracted procedure: k17831 
o|contracted procedure: k17859 
o|contracted procedure: k17868 
o|contracted procedure: k17878 
o|contracted procedure: k17882 
o|contracted procedure: k17894 
o|contracted procedure: k17897 
o|contracted procedure: k17900 
o|contracted procedure: k17903 
o|contracted procedure: k17932 
o|contracted procedure: k17941 
o|contracted procedure: k17950 
o|contracted procedure: k17960 
o|contracted procedure: k17969 
o|contracted procedure: k17978 
o|contracted procedure: k17994 
o|contracted procedure: k17997 
o|contracted procedure: k18000 
o|contracted procedure: k18021 
o|contracted procedure: k18076 
o|contracted procedure: k18082 
o|contracted procedure: k18107 
o|contracted procedure: k18119 
o|contracted procedure: k18129 
o|contracted procedure: k18133 
o|contracted procedure: k18146 
o|contracted procedure: k18159 
o|contracted procedure: k18165 
o|contracted procedure: k18171 
o|contracted procedure: k18180 
o|contracted procedure: k18195 
o|contracted procedure: k18198 
o|contracted procedure: k18201 
o|contracted procedure: k18220 
o|contracted procedure: k18223 
o|contracted procedure: k18226 
o|contracted procedure: k18268 
o|contracted procedure: k18283 
o|contracted procedure: k18289 
o|contracted procedure: k18298 
o|contracted procedure: k18301 
o|contracted procedure: k18334 
o|contracted procedure: k18337 
o|contracted procedure: k18396 
o|contracted procedure: k18403 
o|contracted procedure: k18428 
o|contracted procedure: k18434 
o|contracted procedure: k18455 
o|contracted procedure: k18461 
o|contracted procedure: k18489 
o|contracted procedure: k691125333 
o|contracted procedure: k18485 
o|contracted procedure: k18514 
o|contracted procedure: k18519 
o|contracted procedure: k18525 
o|contracted procedure: k18532 
o|contracted procedure: k18538 
o|contracted procedure: k18544 
o|contracted procedure: k18551 
o|contracted procedure: k18559 
o|contracted procedure: k18565 
o|contracted procedure: k18571 
o|contracted procedure: k18578 
o|contracted procedure: k18585 
o|contracted procedure: k18592 
o|contracted procedure: k18599 
o|contracted procedure: k18607 
o|contracted procedure: k18615 
o|contracted procedure: k18644 
o|contracted procedure: k19438 
o|contracted procedure: k18651 
o|contracted procedure: k19435 
o|contracted procedure: k18655 
o|contracted procedure: k18803 
o|contracted procedure: k18694 
o|contracted procedure: k18697 
o|contracted procedure: k18700 
o|contracted procedure: k18777 
o|contracted procedure: k18715 
o|contracted procedure: k18718 
o|contracted procedure: k18721 
o|contracted procedure: k18730 
o|contracted procedure: k18743 
o|contracted procedure: k18749 
o|contracted procedure: k18756 
o|contracted procedure: k18760 
o|contracted procedure: k18780 
o|contracted procedure: k18786 
o|contracted procedure: k18813 
o|contracted procedure: k18819 
o|contracted procedure: k18822 
o|contracted procedure: k18837 
o|contracted procedure: k18849 
o|contracted procedure: k18852 
o|contracted procedure: k18873 
o|contracted procedure: k18900 
o|contracted procedure: k18906 
o|contracted procedure: k18921 
o|contracted procedure: k18924 
o|contracted procedure: k18927 
o|contracted procedure: k19077 
o|contracted procedure: k18964 
o|contracted procedure: k18973 
o|contracted procedure: k18980 
o|contracted procedure: k18999 
o|contracted procedure: k19012 
o|contracted procedure: k19034 
o|contracted procedure: k19048 
o|contracted procedure: k19060 
o|contracted procedure: k19070 
o|contracted procedure: k19074 
o|contracted procedure: k19090 
o|contracted procedure: k19099 
o|contracted procedure: k19140 
o|contracted procedure: k19131 
o|contracted procedure: k19146 
o|contracted procedure: k19181 
o|contracted procedure: k19149 
o|contracted procedure: k19197 
o|contracted procedure: k19237 
o|contracted procedure: k19218 
o|contracted procedure: k19240 
o|contracted procedure: k19328 
o|contracted procedure: k19334 
o|contracted procedure: k19340 
o|contracted procedure: k19346 
o|contracted procedure: k19368 
o|contracted procedure: k19382 
o|contracted procedure: k19389 
o|contracted procedure: k19400 
o|contracted procedure: k19412 
o|contracted procedure: k19422 
o|contracted procedure: k19426 
o|contracted procedure: k19447 
o|contracted procedure: k19457 
o|contracted procedure: k19461 
o|contracted procedure: k19511 
o|contracted procedure: k19514 
o|contracted procedure: k19517 
o|contracted procedure: k19523 
o|contracted procedure: k19532 
o|contracted procedure: k19541 
o|contracted procedure: k19550 
o|contracted procedure: k19565 
o|contracted procedure: k19569 
o|contracted procedure: k19573 
o|contracted procedure: k19583 
o|contracted procedure: k19576 
o|contracted procedure: k19589 
o|contracted procedure: k19592 
o|contracted procedure: k19601 
o|contracted procedure: k19608 
o|contracted procedure: k19614 
o|contracted procedure: k19617 
o|contracted procedure: k19620 
o|contracted procedure: k19853 
o|contracted procedure: k19846 
o|contracted procedure: k19623 
o|contracted procedure: k19626 
o|contracted procedure: k19636 
o|contracted procedure: k19644 
o|contracted procedure: k19656 
o|contracted procedure: k19659 
o|contracted procedure: k19662 
o|contracted procedure: k19670 
o|contracted procedure: k19678 
o|inlining procedure: k19685 
o|contracted procedure: k19692 
o|contracted procedure: k19702 
o|contracted procedure: k19717 
o|contracted procedure: k19720 
o|inlining procedure: k19699 
o|inlining procedure: k19699 
o|contracted procedure: k19758 
o|contracted procedure: k19780 
o|contracted procedure: k19784 
o|contracted procedure: k19767 
o|contracted procedure: k19794 
o|contracted procedure: k19827 
o|contracted procedure: k19800 
o|contracted procedure: k19806 
o|inlining procedure: k19685 
o|contracted procedure: k19859 
o|contracted procedure: k19862 
o|contracted procedure: k19869 
o|contracted procedure: k19875 
o|contracted procedure: k19905 
o|contracted procedure: k6881 
o|contracted procedure: k6884 
o|contracted procedure: k6894 
o|contracted procedure: k19913 
o|contracted procedure: k19928 
o|contracted procedure: k19936 
o|contracted procedure: k19948 
o|contracted procedure: k19951 
o|contracted procedure: k19954 
o|contracted procedure: k19962 
o|contracted procedure: k19970 
o|contracted procedure: k19989 
o|contracted procedure: k19995 
o|contracted procedure: k20001 
o|contracted procedure: k20042 
o|contracted procedure: k20051 
o|contracted procedure: k20054 
o|contracted procedure: k20070 
o|contracted procedure: k20076 
o|contracted procedure: k20094 
o|contracted procedure: k20118 
o|contracted procedure: k20107 
o|contracted procedure: k20121 
o|contracted procedure: k20138 
o|contracted procedure: k20146 
o|contracted procedure: k20142 
o|contracted procedure: k20134 
o|contracted procedure: k20149 
o|contracted procedure: k20156 
o|contracted procedure: k20162 
o|contracted procedure: k20182 
o|contracted procedure: k20185 
o|contracted procedure: k20201 
o|contracted procedure: k20205 
o|contracted procedure: k20217 
o|contracted procedure: k20221 
o|contracted procedure: k20233 
o|contracted procedure: k20237 
o|contracted procedure: k20245 
o|contracted procedure: k20258 
o|contracted procedure: k20261 
o|contracted procedure: k20264 
o|contracted procedure: k20276 
o|contracted procedure: k20279 
o|contracted procedure: k20282 
o|contracted procedure: k20288 
o|contracted procedure: k20291 
o|contracted procedure: k20300 
o|contracted procedure: k20306 
o|contracted procedure: k20312 
o|contracted procedure: k20641 
o|contracted procedure: k20325 
o|contracted procedure: k20337 
o|contracted procedure: k20349 
o|inlining procedure: k20329 
o|contracted procedure: k20362 
o|contracted procedure: k20385 
o|contracted procedure: k20378 
o|contracted procedure: k20393 
o|inlining procedure: k20329 
o|contracted procedure: k20402 
o|contracted procedure: k20405 
o|contracted procedure: k20408 
o|contracted procedure: k20416 
o|contracted procedure: k20424 
o|contracted procedure: k20557 
o|contracted procedure: k20561 
o|contracted procedure: k20565 
o|contracted procedure: k20431 
o|inlining procedure: k20435 
o|contracted procedure: k20445 
o|contracted procedure: k20456 
o|contracted procedure: k20476 
o|contracted procedure: k20488 
o|contracted procedure: k20515 
o|contracted procedure: k20495 
o|contracted procedure: k20503 
o|contracted procedure: k20499 
o|contracted procedure: k20524 
o|contracted procedure: k20527 
o|contracted procedure: k20530 
o|contracted procedure: k20538 
o|contracted procedure: k20546 
o|contracted procedure: k20465 
o|contracted procedure: k20473 
o|inlining procedure: k20435 
o|contracted procedure: k20553 
o|contracted procedure: k20573 
o|contracted procedure: k20578 
o|contracted procedure: k20587 
o|contracted procedure: k20594 
o|contracted procedure: k20597 
o|contracted procedure: k20612 
o|contracted procedure: k20615 
o|contracted procedure: k20618 
o|contracted procedure: k20626 
o|contracted procedure: k20634 
o|contracted procedure: k20668 
o|contracted procedure: k20664 
o|contracted procedure: k20674 
o|contracted procedure: k20725 
o|contracted procedure: k20689 
o|contracted procedure: k20715 
o|contracted procedure: k20719 
o|contracted procedure: k20711 
o|contracted procedure: k20692 
o|contracted procedure: k20695 
o|contracted procedure: k20703 
o|contracted procedure: k20707 
o|contracted procedure: k20737 
o|contracted procedure: k20740 
o|contracted procedure: k20743 
o|contracted procedure: k20751 
o|contracted procedure: k20759 
o|contracted procedure: k20774 
o|contracted procedure: k20777 
o|contracted procedure: k20780 
o|contracted procedure: k20810 
o|contracted procedure: k20814 
o|contracted procedure: k20826 
o|contracted procedure: k20830 
o|contracted procedure: k20843 
o|contracted procedure: k20847 
o|contracted procedure: k20851 
o|contracted procedure: k20877 
o|contracted procedure: k20881 
o|contracted procedure: k20896 
o|contracted procedure: k20900 
o|contracted procedure: k20923 
o|contracted procedure: k20916 
o|contracted procedure: k20907 
o|contracted procedure: k20926 
o|contracted procedure: k20933 
o|contracted procedure: k20943 
o|contracted procedure: k20950 
o|contracted procedure: k20962 
o|inlining procedure: k20954 
o|inlining procedure: k20954 
o|contracted procedure: k20983 
o|contracted procedure: k20979 
o|contracted procedure: k20993 
o|contracted procedure: k21014 
o|contracted procedure: k21020 
o|contracted procedure: k21026 
o|contracted procedure: k21032 
o|contracted procedure: k21038 
o|contracted procedure: k21044 
o|contracted procedure: k21050 
o|contracted procedure: k21056 
o|contracted procedure: k21062 
o|contracted procedure: k21068 
o|contracted procedure: k21074 
o|contracted procedure: k21080 
o|contracted procedure: k21086 
o|contracted procedure: k21092 
o|contracted procedure: k21098 
o|contracted procedure: k21104 
o|contracted procedure: k21149 
o|contracted procedure: k21155 
o|contracted procedure: k21174 
o|contracted procedure: k21180 
o|contracted procedure: k21186 
o|contracted procedure: k21192 
o|contracted procedure: k21216 
o|contracted procedure: k21224 
o|contracted procedure: k21236 
o|contracted procedure: k21239 
o|contracted procedure: k21242 
o|contracted procedure: k21250 
o|contracted procedure: k21258 
o|contracted procedure: k21267 
o|contracted procedure: k21294 
o|contracted procedure: k21282 
o|contracted procedure: k21286 
o|contracted procedure: k21326 
o|contracted procedure: k21348 
o|contracted procedure: k21357 
o|contracted procedure: k21375 
o|contracted procedure: k21384 
o|contracted procedure: k21393 
o|contracted procedure: k21402 
o|contracted procedure: k21411 
o|contracted procedure: k21420 
o|contracted procedure: k21438 
o|contracted procedure: k21447 
o|contracted procedure: k21456 
o|contracted procedure: k21465 
o|contracted procedure: k21474 
o|contracted procedure: k21483 
o|contracted procedure: k21518 
o|contracted procedure: k21514 
o|contracted procedure: k21510 
o|contracted procedure: k21534 
o|contracted procedure: k21560 
o|contracted procedure: k21615 
o|contracted procedure: k21624 
o|contracted procedure: k21631 
o|contracted procedure: k21637 
o|contracted procedure: k21640 
o|contracted procedure: k21643 
o|contracted procedure: k21650 
o|contracted procedure: k21660 
o|contracted procedure: k21663 
o|contracted procedure: k21667 
o|contracted procedure: k21671 
o|contracted procedure: k21685 
o|contracted procedure: k21691 
o|contracted procedure: k21706 
o|contracted procedure: k21695 
o|contracted procedure: k21712 
o|contracted procedure: k21716 
o|contracted procedure: k21729 
o|contracted procedure: k21733 
o|contracted procedure: k21748 
o|contracted procedure: k21754 
o|contracted procedure: k21758 
o|contracted procedure: k21777 
o|contracted procedure: k21783 
o|contracted procedure: k21798 
o|contracted procedure: k21804 
o|contracted procedure: k21808 
o|contracted procedure: k21815 
o|contracted procedure: k21823 
o|contracted procedure: k21829 
o|contracted procedure: k21832 
o|contracted procedure: k21850 
o|contracted procedure: k21857 
o|contracted procedure: k21871 
o|contracted procedure: k21878 
o|contracted procedure: k21885 
o|contracted procedure: k21889 
o|contracted procedure: k21895 
o|contracted procedure: k21898 
o|contracted procedure: k21901 
o|contracted procedure: k21913 
o|contracted procedure: k21919 
o|contracted procedure: k21953 
o|contracted procedure: k21949 
o|contracted procedure: k21961 
o|contracted procedure: k21967 
o|contracted procedure: k21974 
o|contracted procedure: k21957 
o|contracted procedure: k21988 
o|contracted procedure: k21994 
o|contracted procedure: k22003 
o|contracted procedure: k22009 
o|contracted procedure: k22016 
o|contracted procedure: k22027 
o|contracted procedure: k22034 
o|inlining procedure: k22024 
o|inlining procedure: k22024 
o|contracted procedure: k22044 
o|contracted procedure: k22068 
o|contracted procedure: k22071 
o|contracted procedure: k22062 
o|contracted procedure: k22087 
o|contracted procedure: k22090 
o|contracted procedure: k22093 
o|contracted procedure: k22140 
o|contracted procedure: k22096 
o|contracted procedure: k22105 
o|contracted procedure: k22109 
o|contracted procedure: k22121 
o|contracted procedure: k22129 
o|contracted procedure: k22153 
o|contracted procedure: k22156 
o|contracted procedure: k22159 
o|contracted procedure: k22163 
o|contracted procedure: k22170 
o|contracted procedure: k22182 
o|contracted procedure: k22188 
o|contracted procedure: k22191 
o|contracted procedure: k22194 
o|contracted procedure: k22225 
o|contracted procedure: k22221 
o|contracted procedure: k22209 
o|contracted procedure: k22213 
o|contracted procedure: k22234 
o|contracted procedure: k22250 
o|contracted procedure: k22254 
o|contracted procedure: k22258 
o|contracted procedure: k22266 
o|contracted procedure: k22297 
o|contracted procedure: k22290 
o|contracted procedure: k22281 
o|contracted procedure: k22300 
o|contracted procedure: k22307 
o|contracted procedure: k22321 
o|contracted procedure: k22406 
o|contracted procedure: k22324 
o|contracted procedure: k22402 
o|contracted procedure: k22327 
o|contracted procedure: k22350 
o|contracted procedure: k22353 
o|contracted procedure: k22357 
o|contracted procedure: k22361 
o|contracted procedure: k22375 
o|contracted procedure: k22394 
o|contracted procedure: k22383 
o|contracted procedure: k22390 
o|contracted procedure: k22412 
o|contracted procedure: k22425 
o|contracted procedure: k22433 
o|contracted procedure: k22436 
o|contracted procedure: k22451 
o|contracted procedure: k22457 
o|contracted procedure: k22517 
o|contracted procedure: k22537 
o|contracted procedure: k22544 
o|contracted procedure: k22557 
o|contracted procedure: k22560 
o|contracted procedure: k22573 
o|contracted procedure: k22577 
o|contracted procedure: k22584 
o|contracted procedure: k22592 
o|contracted procedure: k22596 
o|contracted procedure: k22604 
o|contracted procedure: k22608 
o|contracted procedure: k22612 
o|contracted procedure: k22616 
o|contracted procedure: k22620 
o|contracted procedure: k22626 
o|contracted procedure: k22636 
o|contracted procedure: k22644 
o|contracted procedure: k22648 
o|contracted procedure: k22663 
o|contracted procedure: k22674 
o|contracted procedure: k22678 
o|contracted procedure: k22697 
o|contracted procedure: k22705 
o|contracted procedure: k22709 
o|contracted procedure: k22713 
o|contracted procedure: k22717 
o|contracted procedure: k22723 
o|contracted procedure: k22726 
o|contracted procedure: k22730 
o|contracted procedure: k22795 
o|contracted procedure: k22813 
o|contracted procedure: k22821 
o|contracted procedure: k22833 
o|contracted procedure: k22836 
o|contracted procedure: k22839 
o|contracted procedure: k22847 
o|contracted procedure: k22855 
o|contracted procedure: k22876 
o|contracted procedure: k22880 
o|contracted procedure: k22900 
o|contracted procedure: k7563 
o|contracted procedure: k7570 
o|contracted procedure: k7578 
o|contracted procedure: k22913 
o|contracted procedure: k22933 
o|contracted procedure: k7111 
o|contracted procedure: k7124 
o|contracted procedure: k7130 
o|contracted procedure: k23021 
o|contracted procedure: k22968 
o|propagated global variable: r23022 ##sys#undefined-value 
o|contracted procedure: k22981 
o|inlining procedure: k22978 
o|contracted procedure: k22990 
o|inlining procedure: k22978 
o|contracted procedure: k22999 
o|contracted procedure: k23008 
o|contracted procedure: k23014 
o|contracted procedure: k23071 
o|contracted procedure: k23067 
o|simplifications: ((##core#call . 2) (if . 9) (let . 466)) 
o|  call simplifications:
o|    scheme#+	2
o|removed binding forms: 1660 
o|contracted procedure: k10294 
o|contracted procedure: k12501 
o|contracted procedure: k13832 
o|contracted procedure: k15247 
o|inlining procedure: k12982 
o|inlining procedure: k12982 
o|inlining procedure: k12982 
o|inlining procedure: k13328 
o|inlining procedure: k13328 
o|inlining procedure: k15882 
o|inlining procedure: k16119 
o|inlining procedure: "(core.scm:2094) grow3809" 
o|inlining procedure: "(core.scm:2101) grow3809" 
o|inlining procedure: k17549 
o|inlining procedure: "(core.scm:2105) grow3809" 
o|inlining procedure: "(core.scm:2188) grow3809" 
o|inlining procedure: "(core.scm:2085) grow3809" 
o|inlining procedure: "(core.scm:2505) quick-put!3816" 
o|inlining procedure: "(core.scm:2484) quick-put!3816" 
o|inlining procedure: "(core.scm:2461) quick-put!3816" 
o|inlining procedure: "(core.scm:2461) quick-put!3816" 
o|inlining procedure: "(core.scm:2413) quick-put!3816" 
o|inlining procedure: "(core.scm:2413) quick-put!3816" 
o|inlining procedure: "(core.scm:2377) quick-put!3816" 
o|inlining procedure: "(core.scm:2378) quick-put!3816" 
o|inlining procedure: "(core.scm:2392) quick-put!3816" 
o|inlining procedure: "(core.scm:2392) quick-put!3816" 
o|inlining procedure: "(core.scm:2387) quick-put!3816" 
o|inlining procedure: "(core.scm:2405) quick-put!3816" 
o|inlining procedure: "(core.scm:2404) quick-put!3816" 
o|inlining procedure: "(core.scm:2362) quick-put!3816" 
o|inlining procedure: k19689 
o|inlining procedure: k19689 
o|inlining procedure: k19689 
o|inlining procedure: k19689 
o|inlining procedure: k19689 
o|contracted procedure: k21787 
o|simplifications: ((let . 4)) 
o|replaced variables: 14 
o|removed binding forms: 12 
o|substituted constant variable: n381726789 
o|substituted constant variable: n381726793 
o|substituted constant variable: n381726799 
o|substituted constant variable: n381726809 
o|substituted constant variable: n381726813 
o|substituted constant variable: prop407526820 
o|substituted constant variable: prop407526827 
o|substituted constant variable: prop407526834 
o|substituted constant variable: val407626835 
o|substituted constant variable: prop407526841 
o|substituted constant variable: val407626842 
o|contracted procedure: "(core.scm:2461) quick-put!3816" 
o|substituted constant variable: prop407526850 
o|substituted constant variable: val407626851 
o|substituted constant variable: prop407526857 
o|substituted constant variable: val407626858 
o|substituted constant variable: prop407526864 
o|substituted constant variable: val407626865 
o|substituted constant variable: prop407526871 
o|substituted constant variable: val407626872 
o|substituted constant variable: prop407526878 
o|substituted constant variable: val407626879 
o|substituted constant variable: prop407526885 
o|substituted constant variable: val407626886 
o|substituted constant variable: prop407526892 
o|substituted constant variable: val407626893 
o|substituted constant variable: prop407526899 
o|substituted constant variable: prop407526906 
o|substituted constant variable: val407626907 
o|substituted constant variable: prop407526913 
o|substituted constant variable: val407626914 
o|replaced variables: 51 
o|removed binding forms: 7 
o|inlining procedure: k17507 
o|inlining procedure: k18733 
o|inlining procedure: k18831 
o|substituted constant variable: prop4075 
o|substituted constant variable: val4076 
o|replaced variables: 3 
o|removed binding forms: 68 
o|contracted procedure: k17440 
o|contracted procedure: k17473 
o|contracted procedure: k17519 
o|contracted procedure: k17935 
o|contracted procedure: k19282 
o|removed binding forms: 13 
o|removed binding forms: 4 
o|direct leaf routine/allocation: a7278 0 
o|direct leaf routine/allocation: loop633 0 
o|direct leaf routine/allocation: g13801390 18 
o|direct leaf routine/allocation: g16601673 0 
o|direct leaf routine/allocation: loop1872 0 
o|direct leaf routine/allocation: g434435 3 
o|direct leaf routine/allocation: loop474 0 
o|direct leaf routine/allocation: grow3809 29 
o|direct leaf routine/allocation: g40964108 0 
o|direct leaf routine/allocation: g46784687 0 
o|direct leaf routine/allocation: g52355236 0 
o|direct leaf routine/allocation: loop644 0 
o|direct leaf routine/allocation: new-literal4963 32 
o|direct leaf routine/allocation: g52515252 0 
o|contracted procedure: "(mini-srfi-1.scm:88) k6723" 
o|converted assignments to bindings: (loop633) 
o|contracted procedure: "(core.scm:795) k9435" 
o|contracted procedure: "(core.scm:893) k10111" 
o|converted assignments to bindings: (loop1872) 
o|contracted procedure: k17010 
o|converted assignments to bindings: (loop474) 
o|contracted procedure: "(core.scm:2137) k17690" 
o|contracted procedure: "(core.scm:2150) k17759" 
o|contracted procedure: "(core.scm:2311) k19450" 
o|contracted procedure: "(core.scm:2718) k20630" 
o|contracted procedure: k22888 
o|converted assignments to bindings: (loop644) 
o|simplifications: ((let . 4)) 
o|removed binding forms: 9 
o|direct leaf routine/allocation: loop338 0 
o|direct leaf routine/allocation: for-each-loop16591676 0 
o|direct leaf routine/allocation: for-each-loop40954119 0 
o|contracted procedure: k7272 
o|converted assignments to bindings: (loop338) 
o|contracted procedure: k10055 
o|converted assignments to bindings: (for-each-loop16591676) 
o|contracted procedure: k18647 
o|converted assignments to bindings: (for-each-loop40954119) 
o|simplifications: ((let . 3)) 
o|removed binding forms: 3 
o|customizable procedures: (a22946 loop480 g52055214 map-loop51995217 k21618 loop5178 k22442 k22463 immediate-literal4965 k22415 k22330 k22378 k22337 k22340 k22240 k22243 g51075108 k21978 k21845 walk4960 va26382 k21646 mapwalk4961 walk-var4958 k21550 k21553 blockvar-literal4964 walk-global4959 literal4962 g49764977 g48354836 g48064815 map-loop48004818 k20045 k20079 k20165 k20789 g47924793 map-loop46154632 map-loop45944639 k20315 map-loop46724696 map-loop47174735 loop4711 g47514760 map-loop47454763 maptransform4327 transform4326 ref-var4328 k19535 g44584467 map-loop44524470 loop384 k19711 test4322 k19729 k19735 register-direct-call!4324 register-customizable!4323 g44104419 map-loop44044422 chicken.compiler.core#concatenate gather4325 k19371 g41354142 for-each-loop41344145 k19297 k18664 g41964197 k19212 k19087 k18670 g42234230 for-each-loop42224240 k19027 k18992 k18676 k18828 k18685 k18706 k18727 k18186 k18243 k18271 k18316 g40424043 mark-rest-null?3813 mark-rest-cdr3812 g39863993 for-each-loop39853996 k17449 g39173925 for-each-loop39163929 k17798 k17801 g38863893 for-each-loop38853896 assign3814 loop3871 k17559 walk3810 walkeach3811 ref3815 loop3781 walk-arguments3665 chicken.compiler.core#cons* k16597 k16919 walk-call3662 walk-inline-call3664 cps-lambda3659 node-for-var?3660 loop3706 walk3661 map-loop35163534 map-loop35443561 loop3363 map-loop34133433 map-loop33873404 k15838 map-loop34543475 g26332642 map-loop26272645 k8771 g25842585 g25772578 loop2549 macro-alias807 loop241 g24902499 map-loop24842503 a13787 g32603267 for-each-loop32593291 k15397 g32163223 for-each-loop32153252 k15205 k15288 for-each-loop31883206 for-each-loop31633181 for-each-loop31323150 k15000 k14893 map-loop30863111 for-each-loop30613079 for-each-loop30313049 for-each-loop30133023 for-each-loop29953005 for-each-loop29562974 for-each-loop29242942 chicken.compiler.core#lset-difference/eq? for-each-loop28932911 for-each-loop28682886 for-each-loop28162844 for-each-loop28262859 for-each-loop27642782 for-each-loop27892807 for-each-loop27392757 chicken.compiler.core#lset-intersection/eq? globalize-all2679 check-decl2677 chicken.compiler.core#lset-union/eq? foldr419422 g424425 globalize2678 a12841 note-local2680 k12493 k12359 k12273 k16266 map-loop35783596 map-loop36063623 chicken.compiler.core#create-foreign-stub chicken.compiler.core#expand-foreign-lambda* chicken.compiler.core#expand-foreign-lambda fold2300 map-loop22612278 mapwalk816 g22342243 map-loop22282246 unquotify812 k11584 g21472148 g21382139 map-loop20382055 k10725 g18551864 map-loop18491879 k11175 loop1934 tmp15796 tmp25797 doloop19641965 for-each-loop983993 chicken.compiler.core#delete k10743 k10764 g20062015 map-loop20002018 map-loop18031822 g17261735 map-loop17201741 for-each-loop17711782 for-each-loop17531764 k10375 k10284 k10175 map-loop16271645 map-loop15881606 map-loop14961513 k9764 chicken.compiler.core#posq k9668 g14541463 map-loop14481466 map-loop13171334 map-loop13471365 map-loop13741398 map-loop14101434 map-loop12571275 map-loop12841305 chicken.compiler.core#unzip1 map-loop11691186 set-real-names!810 g12091219 map-loop12031224 canonicalize-body/ln809 chicken.compiler.core#lset-adjoin/eq? g11011110 map-loop10951113 g10521053 resolve-variable813 g951952 g941942 g931932 g927928 g923924 walk815 for-each-loop868882 handle-expansion-result808 mapupdate3633 walk3634 loop3636 lookup806 k7818 find-id805 g777784 for-each-loop776795 chicken.compiler.core#every foldl545549 foldl529533 chicken.compiler.core#filter foldl507511 loop491 map-loop447465 foldr401404 g406407 loop364 loop357 loop351 loop325) 
o|calls to known targets: 870 
o|identified direct recursive calls: f_6745 1 
o|identified direct recursive calls: f_6773 1 
o|identified direct recursive calls: f_6924 1 
o|identified direct recursive calls: f_7041 1 
o|identified direct recursive calls: f_7140 2 
o|identified direct recursive calls: f_6715 1 
o|identified direct recursive calls: f_7251 2 
o|identified direct recursive calls: f_7524 1 
o|identified direct recursive calls: f_9183 1 
o|identified direct recursive calls: f_9217 1 
o|identified direct recursive calls: f_9360 1 
o|identified direct recursive calls: f_9408 1 
o|identified direct recursive calls: f_9456 1 
o|identified direct recursive calls: f_10103 1 
o|identified direct recursive calls: f_11188 1 
o|identified direct recursive calls: f_16308 1 
o|identified direct recursive calls: f_16342 1 
o|identified direct recursive calls: f_6968 1 
o|identified direct recursive calls: f_6472 1 
o|identified direct recursive calls: f_15734 3 
o|identified direct recursive calls: f_16192 1 
o|identified direct recursive calls: f_16226 1 
o|identified direct recursive calls: f_7081 1 
o|identified direct recursive calls: f_19442 1 
o|identified direct recursive calls: f_20483 1 
o|identified direct recursive calls: f_20607 1 
o|identified direct recursive calls: f_20684 1 
o|identified direct recursive calls: f_7558 1 
o|fast box initializations: 140 
o|fast global references: 90 
o|fast global assignments: 52 
o|dropping unused closure argument: f_11188 
o|dropping unused closure argument: f_13700 
o|dropping unused closure argument: f_13760 
o|dropping unused closure argument: f_15725 
o|dropping unused closure argument: f_16114 
o|dropping unused closure argument: f_16151 
o|dropping unused closure argument: f_16553 
o|dropping unused closure argument: f_17422 
o|dropping unused closure argument: f_21265 
o|dropping unused closure argument: f_22963 
o|dropping unused closure argument: f_6675 
o|dropping unused closure argument: f_6739 
o|dropping unused closure argument: f_6767 
o|dropping unused closure argument: f_6795 
o|dropping unused closure argument: f_6915 
o|dropping unused closure argument: f_7024 
o|dropping unused closure argument: f_7081 
o|dropping unused closure argument: f_7134 
o|dropping unused closure argument: f_7175 
o|dropping unused closure argument: f_7220 
o|dropping unused closure argument: f_7299 
o|dropping unused closure argument: f_7518 
o|dropping unused closure argument: f_7889 
o|dropping unused closure argument: f_7945 
*/
/* end of file */
